//
//  AuthV4Test.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by paikjongman on 2017. 6. 8..
//  Copyright © 2017년 GCP. All rights reserved.
//

#include "AuthV4Test.hpp"
#include "ui/CocosGUI.h"
#include "UICombobox.hpp"
#include "UIInputList.hpp"
#include "Postbox.hpp"
#include <stdlib.h>
#include "GameManager.hpp"
#include "ResultUtils.hpp"
#include "ExternalAuth.hpp"

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;

const std::vector<const char*> providerList ={
    "GUEST"
    ,"HIVE"
    ,"FACEBOOK"
    ,"GOOGLE"
    ,"QQ"
    ,"WEIBO"
    ,"VK"
    ,"WECHAT"
    ,"APPLE"
    ,"SIGNIN_APPLE"
    ,"LINE"
    ,"TWITTER"
    ,"WEVERSE"
    ,"NAVER"
    ,"GOOGLE_PLAY_GAMES"
    ,"HUAWEI"
    ,"AUTO"
};

const std::vector<const char*> externalAuthProviderList = {
    "CUSTOM_TWITTER"
    ,"CUSTOM_NAVER"
};
std::string externalAuthProvider = "";
std::string externalAuthUserId = "";

int ProviderTypeToIndex(ProviderType provider){
    for(int i=0;i<providerList.size();++i)
    {
        if(ProviderInfo::providerTypeFromString(providerList[i]) == provider)
            return i;
    }
    return -1;
}

std::string PlayerInfoToString(const PlayerInfo & playerInfo){
    return StringUtils::format("PlayerToken: %s\n \
                                              PlayerId: %lld",
                                              playerInfo.playerToken.c_str(),
                                              playerInfo.playerId);
}

std::vector<std::string> getPlayerProviderStringList(const PlayerInfo& player){
    std::vector<std::string> v;
    for(auto it = player.providerInfoData.begin();it != player.providerInfoData.end();++it){
        v.push_back(ProviderInfo::stringProviderType(it->first));
    }
    return v;
}

std::string getProviderInfoString(const hive::ProviderInfo& info){
    return ResultUtils::getProviderInfoString(info);
}

void MessageBox(const hive::ProviderInfo& info){
    std::string message = getProviderInfoString(info);
    MessageBox(message.c_str(),"Call Success");
}
std::string getPlayerInfoString(const hive::PlayerInfo& info) {
    return ResultUtils::getPlayerInfoString(info);
}
std::string getProfileInfoString(const hive::ProfileInfo& info) {
    return ResultUtils::getProfileInfoString(info);
}

void MessageBox(const hive::PlayerInfo& info) {
    
    MessageBox(getPlayerInfoString(info).c_str(),"Player Info");
}

std::string getMaintenanceInfolistString(const std::vector<AuthV4MaintenanceInfo>& maintenanceInfolist) {
    std::string message ="";
    for(auto it = maintenanceInfolist.begin(); it != maintenanceInfolist.end(); ++it){
        const AuthV4MaintenanceInfo& info = (*it);
        message += "["+info.title+"]\n";
        message += info.message + "\n";
        message += "button : "+info.button+"\n";
        switch(info.action){
            case AuthV4MaintenanceActionType::OPEN_URL:
                message += "action : OPEN URL";
                break;
            case AuthV4MaintenanceActionType::EXIT:
                message += "action : EXIT";
                break;
            case AuthV4MaintenanceActionType::DONE:
                message += "action : DONE";
                break;
        }
        message += "url : "+info.url+"\n";
        message += "startDate : " +info.startDate+"\n";
        message += "endDate : " +info.endDate+"\n";
        message += "remainingTime : "+cocos2d::StringUtils::format("%d",info.remainingTime)+"sec\n\n";
    }
    return message;
}

void MessageBox(const std::vector<AuthV4MaintenanceInfo>& maintenanceInfolist){
    MessageBox(getMaintenanceInfolistString(maintenanceInfolist).c_str(),"Call Success");
}

bool AuthV4Test::init()
{
    if(!BaseView::init())
        return false;
    setUseShowLogButton(true);
    Logger::setLoggerListener([=](std::string tag, std::string level, std::string logMessage){
        addLogMessage(logMessage);
    });

    addButton("AuthV4.requestPermissionViewData",BUTTON_REQUESTPERMISSIONVIEWDATA);
//#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    addEditBox(nullptr, "android.permission.READ_PHONE_STATE\nandroid.permission.READ_PHONE_NUMBERS\nandroid.permission.WRITE_EXTERNAL_STORAGE\nandroid.permission.READ_CONTACTS\nandroid.permission.VIBRATE", nullptr, { screenSize.width, 300}, EditBox::InputMode::ANY, 20, EDITBOX_PERMISSION_LIST);
    addButton("PlatformHelper.requestUserPermissions", BUTTON_REQUEST_USER_PERMISSIONS);
//#endif
    addButton("AuthV4.setup", BUTTON_SETUP);
    addCombobox("ProviderType : ", {screenSize.width, DEFAULT_CONTENT_HEIGHT}, providerList, 0, COMBOBOX_LOGINTYPE);

    addCombobox("External auth provider : ", std::move(externalAuthProviderList), 0, COMBOBOX_EXTERNAL_AUTH_PROVIDER);
    addEditBox("", externalAuthProviderList[0], "", EDITBOX_EXTERNAL_AUTH_PROVIDER);
    addEditBox("External auth user id : ", "1234", "", EDITBOX_EXTERNAL_AUTH_USER_ID);

    BeginHorizontal();
    addButton("AuthV4.showSignin", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_SHOWSIGNIN);
    addButton("isAutoSignIn", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_ISAUTOSIGNIN);
    EndHorizontal();

    BeginHorizontal();
    addButton("AuthV4.signin", {screenSize.width/3, DEFAULT_CONTENT_HEIGHT}, BUTTON_SIGNIN);
    addButton("AuthV4.signout", {screenSize.width/3, DEFAULT_CONTENT_HEIGHT}, BUTTON_SIGNOUT);
    addButton("AuthV4.playerDelete", {screenSize.width/3, DEFAULT_CONTENT_HEIGHT}, BUTTON_PLAYERDELETE);
    EndHorizontal();
    addButton("External auth & AuthV4.signInWithAuthKey", BUTTON_EXTERNAL_AUTH_AND_SIGNIN_WITH_AUTH_KEY);

    BeginHorizontal();
    addButton("AuthV4.connect", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_CONNECT);
    addButton("AuthV4.disconnect", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_DISCONNECT);
    EndHorizontal();
    addButton("External auth & AuthV4.connectInWithAuthKey", BUTTON_EXTERNAL_AUTH_AND_CONNECT_WITH_AUTH_KEY);
    addButton("External auth & AuthV4.disconnectWithName", BUTTON_EXTERNAL_AUTH_AND_DISCONNECT_WITH_NAME);

    BeginHorizontal();
    addButton("AuthV4.getProviderFriendsList", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_GETPROVIDERFRIENDLIST);
    addButton("AuthV4.getPlayerInfo", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_GETPLAYERINFO);
    EndHorizontal();

    addButton("AuthV4.getProfile", BUTTON_GETPROFILE);
    addButton("AuthV4.resolveConflict", BUTTON_RESOLVE_CONFLICT);

    BeginHorizontal();
    addEditBox("playerId : ", "", "", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, 20, EDITBOX_SHOW_PROFILE);
    addButton("AuthV4.showProfile", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_SHOWPROFILE);
    EndHorizontal();

    addButton("AuthV4.showInquiry", {screenSize.width, DEFAULT_CONTENT_HEIGHT}, BUTTON_SHOWINQUIRY);
    addButton("AuthV4.showMyInquiry", {screenSize.width, DEFAULT_CONTENT_HEIGHT}, BUTTON_SHOWMYINQUIRY);


    BeginHorizontal();
    addButton("AuthV4.showChatbotInquiry", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_SHOWCHATBOTINQUIRY);
    addButton("AuthV4.showTerms", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_SHOWTERMS);
    EndHorizontal();

    addButton("AuthV4.showAdultConfirm", BUTTON_SHOWADULTCONFIRM);
    addButton("AuthV4.checkProvider", BUTTON_CHECK_PROVIDER);

    addTrueFalseToggleButton("isShow : ", 1, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_IS_SHOW_MAINTENANCE);
    BeginHorizontal();
    addButton("AuthV4.checkMaintenance", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_CHECK_MAINTENANCE);
    addButton("AuthV4.checkBlacklist", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_CHECK_BLACKLIST);
    EndHorizontal();

    addButton("AuthV4.showGamecenterLoginCancelDialog", BUTTON_SHOW_GAME_CENTER_LOGINCANCELDIALOG);
    addButton("AuthV4.setProviderChangedListener", BUTTON_SETPROVIDERCHANGEHANDLER);

    BeginHorizontal();
    addTrueFalseToggleButton("isReady : ", 1, {screenSize.width/3, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_IS_READY_ENGAGEMENT);
    addButton("AuthV4.setEngagementReady", {screenSize.width*2/3, DEFAULT_CONTENT_HEIGHT}, BUTTON_ISREADYENGAGEMENT);
    EndHorizontal();
    addButton("AuthV4.getAgeGateU13", BUTTON_GET_AGEGATEU13);
    addButton("AuthV4.reset", BUTTON_RESET);

    addButton("isActivateLogging",BUTTON_IS_ACTIVATE_LOGGING);
    addButton("AuthV4.selectConflict", BUTTON_SELECT_CONFLICT);
    addButton("AuthV4.showConflictSelection", BUTTON_SHOW_CONFLICTSELECTION);
    
    addButton("AuthV4.showDeviceManagement", BUTTON_SHOW_DEVICEMANAGEMENT);
    
    setUseBackButton(true, "- AuthV4 -");
    
    return true;
}
void AuthV4Test::onButtonEvent(int buttonTag)
{
    /*
     BUTTON_SETUP,
     BUTTON_SIGNIN,
     BUTTON_SIGNOUT,
     BUTTON_PLAYERDELETE,
     BUTTON_CONNECT,
     BUTTON_DISCONNECT,
     BUTTON_SELECT_SIGNIN,
     BUTTON_CHECK_PROVIDER,
     BUTTON_CHECK_MAINTENANCE,
     BUTTON_CHECK_BLACKLIST,
     BUTTON_SETPROVIDERCHANGEHANDLER,
     BUTTON_SHOWSIGNIN,
     BUTTON_SHOWCONFLICTSELECTION,
     BUTTON_SHOWTERMS,
     BUTTON_SHOWADULTCONFIRM,
     BUTTON_GETPROFILE,
     BUTTON_SHOWPROFILE,
     BUTTON_RESET,
     BUTTON_ISAUTOSIGNIN,
     BUTTON_GETPLAYERINFO
     */
    switch(buttonTag) {
        case BUTTON_REQUESTPERMISSIONVIEWDATA:
            AuthV4::requestPermissionViewData([=](ResultAPI const & result,PermissionViewData const & data) {
                Logger::log("result code:%d message:%s\n",result.code,result.message.c_str());

                if (result.isSuccess()) {
                    if (result.code == hive::ResultAPI::AuthV4SkipPermissionView) {
                        Logger::log("Can skip permission notice");
                        MessageBox("Can skip permission notice", "Notice");
                    }
                    else {
                        Logger::log("=== 커스텀 약관고지 로그 ===\n");
                        std::string permissionNotice = "";
                        permissionNotice+=StringUtils::format("제목 : %s\n",data.contents.c_str());

                        if (data.permissions.size() > 0) {
                            permissionNotice+=StringUtils::format("\n\n=== From permissions ===");
                        }
                        for (auto unit : data.permissions) {
                            permissionNotice+=StringUtils::format("\n권한명 : %s\n 타이틀 %s\n 내용 %s\n 권한 카테고리 : %d\n",unit.nativePermissionName.c_str(),unit.title.c_str(),unit.contents.c_str(),unit.permissionCategory);
                        }
                        if (data.commons.size() > 0) {
                            permissionNotice+=StringUtils::format("\n\n=== From commons ===");
                        }
                        for (auto unit : data.commons) {
                            permissionNotice+=StringUtils::format("\n권한명 : %s\n 타이틀 %s\n 내용 %s \n 권한 카테고리 : %d\n", unit.nativePermissionName.c_str(), unit.title.c_str(), unit.contents.c_str(), unit.permissionCategory);
                        }
                        Logger::log((data.contents + "\n").c_str());
                        Logger::log(permissionNotice.c_str());
                        MessageBox(permissionNotice.c_str(), data.contents.c_str());
                    }
                }
                else {
                    //AuthV4InProgress에러는 대기, 그 외 에러는 무시하고 AuthV4.setup을 호출할경우 HIVE SDK의 기본 권한고지 팝업이 사용됩니다.
                }
            });
            break;
        case BUTTON_SETUP:{
            AuthV4::setup([=](ResultAPI const & result,bool isAutoSignIn,std::string did,std::vector<ProviderType> const & providerTypeList){
                LogHead("setup",result);
                if(ErrorCheckResultAPI(result)) {
                    if(isAutoSignIn == false) {
                        //일단 0번째걸로 설정.
                        if (providerTypeList.size() > 0) {
                            const int index = ProviderTypeToIndex(providerTypeList[0]);
                            changeComboboxList(providerList,index,COMBOBOX_LOGINTYPE);
                        }
                    } else {
                        const int index = ProviderTypeToIndex(ProviderType::AUTO);
                        changeComboboxList(providerList,index,COMBOBOX_LOGINTYPE);
                    }
                    Logger::log("isAutoSignIn : %s\ndid : %s\n",isAutoSignIn?"true":"false",did.c_str());
                }
                else if (result.needExit()) {
                    Logger::log("result.needExit() = true");
                    // result.needExit() true일 경우 적절한 앱 종료 이벤트를 처리. exit(0)
                }
            });
            
            Promotion::setEngagementHandler(GameManager::engagemenetResult);
        }
            break;
            
        case BUTTON_SIGNIN:{
            //TODO provider type얻기
            ProviderType type = ProviderInfo::providerTypeFromString(getComboboxValue(COMBOBOX_LOGINTYPE));
            AuthV4::signIn(type, [=](ResultAPI const & result,PlayerInfo const & playerInfo){
                LogHead("signIn",result);
                if(ErrorCheckResultAPI(result)) {
                    GameManager::getInstance()->SetCurrentPlayerInfo(playerInfo);
                    MessageBox(PlayerInfoToString(playerInfo).c_str(),"Call Success");
                    Postbox::setLoginType(Postbox::v4);
                    Logger::log("playerInfo : %s\n",getPlayerInfoString(playerInfo).c_str());

                    Postbox::login([=](const Postbox::Login *login, Postbox::Error error){
                        if( login != nullptr )
                        {
                            int gold = login->gold;
                            int gem = login->gem;
                            std::string nickname = login->nickname;
                            Logger::log("nick %s, gold %d, gem %d", nickname.c_str(), gold, gem);
                        }
                        else
                        {
                            Logger::log("Postbox login failed.");
                            Logger::log("result code = %s, msg = %s", error.result_code.c_str(), error.result_msg.c_str());
                        }
                    });
                }
                else if (result.needExit()) {
                    Logger::log("result.needExit() = true");
                    // result.needExit() true일 경우 적절한 앱 종료 이벤트를 처리. exit(0)
                }
            });
        }
            break;

        case BUTTON_EXTERNAL_AUTH_AND_SIGNIN_WITH_AUTH_KEY:{

            std::string idpId = getEditboxValue(EDITBOX_EXTERNAL_AUTH_PROVIDER);
            std::string idpUserId = getEditboxValue(EDITBOX_EXTERNAL_AUTH_USER_ID);

            ExternalAuth::getInstance()->requestIdp(hive::Configuration::getZone(),
                                                           hive::Configuration::getAppId(),
                                                           ExternalAuth::AuthType::SIGNINIDP,
                                                           idpId,
                                                           idpUserId, [=](const ExternalAuth::ExternalAuthResponse* response) {

                Logger::log("[ExternalAuth] %s", response->toString().c_str());

                if (!response->isSuccess()) {
                    Logger::log("[ExternalAuth] response failed.");
                    return;
                }

                if (response->authKey == "") {
                    Logger::log("[ExternalAuth] authKey is NULL or empty.");
                    return;
                }

                AuthV4::signInWithAuthKey(response->authKey, [=](ResultAPI const & result, PlayerInfo const & playerInfo){
                    LogHead("signInWithAuthKey",result);
                    if(ErrorCheckResultAPI(result)) {
                        GameManager::getInstance()->SetCurrentPlayerInfo(playerInfo);
                        MessageBox(PlayerInfoToString(playerInfo).c_str(),"Call Success");
                        Postbox::setLoginType(Postbox::v4);
                        Logger::log("playerInfo : %s\n",getPlayerInfoString(playerInfo).c_str());

                        Postbox::login([=](const Postbox::Login *login, Postbox::Error error){
                            if( login != nullptr )
                            {
                                int gold = login->gold;
                                int gem = login->gem;
                                std::string nickname = login->nickname;
                                Logger::log("nick %s, gold %d, gem %d", nickname.c_str(), gold, gem);
                            }
                            else
                            {
                                Logger::log("Postbox login failed.");
                                Logger::log("result code = %s, msg = %s", error.result_code.c_str(), error.result_msg.c_str());
                            }
                        });
                    }
                    else if (result.needExit()) {
                        Logger::log("result.needExit() = true");
                        // result.needExit() true일 경우 적절한 앱 종료 이벤트를 처리. exit(0)
                    }
                });
            });
        }
            break;
            
        case BUTTON_SIGNOUT:{
            AuthV4::signOut([=](ResultAPI const & result){
                showResult(result);
                LogHead("signOut",result);
            });
        }
            break;

        case BUTTON_PLAYERDELETE:{
            AuthV4::playerDelete([=](ResultAPI const & result){
                showResult(result);
                LogHead("playerDelete",result);
            });
        }
            break;
            
        case BUTTON_CONNECT:{
            //provider type얻기
            showProviderSelectView([=](ProviderType type){
                AuthV4::connect(type,[=](ResultAPI const & connectResult,PlayerInfo const & conflictPlayer){
                    LogHead("connect",connectResult);
                    if(connectResult.getErrorCode() == ResultAPI::CONFLICT_PLAYER ){
                        Logger::log("conflictPlayer Info : %s\n",getPlayerInfoString(conflictPlayer).c_str());
                        GameManager::getInstance()->SetConflictPlayerInfo(conflictPlayer);
                    }
                });
            });
        }
            break;
            
        case BUTTON_DISCONNECT:{
            showProviderSelectView(GameManager::getInstance()->GetCurrentPlayerInfo(), [=](ProviderType type) {
                AuthV4::disconnect(type,[=](ResultAPI const & result){
                    LogHead("disconnect",result);
                    showResult(result);
                    GameManager::getInstance()->SetCurrentPlayerInfo(AuthV4::getPlayerInfo());
                });
            });
        }
            break;

        case BUTTON_EXTERNAL_AUTH_AND_CONNECT_WITH_AUTH_KEY:{
            std::string idpId = getEditboxValue(EDITBOX_EXTERNAL_AUTH_PROVIDER);
            std::string idpUserId = getEditboxValue(EDITBOX_EXTERNAL_AUTH_USER_ID);

            ExternalAuth::getInstance()->requestIdp(hive::Configuration::getZone(),
                                                           hive::Configuration::getAppId(),
                                                           ExternalAuth::AuthType::CONNECT,
                                                           idpId,
                                                           idpUserId, [=](const ExternalAuth::ExternalAuthResponse* response) {

                Logger::log("[ExternalAuth] %s", response->toString().c_str());

                if (!response->isSuccess()) {
                    Logger::log("[ExternalAuth] response failed.");
                    return;
                }

                if (response->authKey == "") {
                    Logger::log("[ExternalAuth] authKey is NULL or empty.");
                    return;
                }

                AuthV4::connectWithAuthKey(response->authKey, [=](ResultAPI const & connectResult,PlayerInfo const & conflictPlayer){
                    LogHead("connectWithAuthKey",connectResult);
                    if(connectResult.getErrorCode() == ResultAPI::CONFLICT_PLAYER ){
                        Logger::log("conflictPlayer Info : %s\n",getPlayerInfoString(conflictPlayer).c_str());
                        GameManager::getInstance()->SetConflictPlayerInfo(conflictPlayer);
                    }
                });
            });
        }
            break;

        case BUTTON_EXTERNAL_AUTH_AND_DISCONNECT_WITH_NAME:{
            std::string idpId = getEditboxValue(EDITBOX_EXTERNAL_AUTH_PROVIDER);
            AuthV4::disconnectWithName(idpId,[=](ResultAPI const & result){
                LogHead("disconnectWithName",result);
                showResult(result);
                GameManager::getInstance()->SetCurrentPlayerInfo(AuthV4::getPlayerInfo());
            });

        }
            break;

        case BUTTON_RESOLVE_CONFLICT:
            AuthV4::resolveConflict([=](ResultAPI const & result){
                showResult(result);
            });
            break;

        case BUTTON_SELECT_CONFLICT: {
            PlayerInfo playerInfo = GameManager::getInstance()->GetCurrentPlayerInfo();
            PlayerInfo conflictPlayer = GameManager::getInstance()->GetConflictPlayerInfo();
            PlayerID ids[] = {
                playerInfo.playerId
                ,conflictPlayer.playerId
            };
            std::vector<std::string> playerids = {
                cocos2d::StringUtils::format("current user : %lld",ids[0])
                ,cocos2d::StringUtils::format("conflict user : %lld",ids[1])
                ,"Cancle Connect"
            };
            pushView(Combobox::create("Select User", {screenSize.width, DEFAULT_CONTENT_HEIGHT}, std::move(playerids),false,[=](const char *selectItem,int index){
                if(index < 2){ //취소 상황입니다.
                    AuthV4::selectConflict(ids[index], [=](ResultAPI const & result,PlayerInfo const & playerInfo){
                        LogHead("selectConflict",result);
                        if(ErrorCheckResultAPI(result)){
                            GameManager::getInstance()->SetCurrentPlayerInfo(playerInfo);
                            MessageBox(playerInfo);
                            Logger::log("playerInfo : %s\n",getPlayerInfoString(playerInfo).c_str());
                        }
                        else if (result.needExit()) {
                            Logger::log("result.needExit() = true");
                            // result.needExit() true일 경우 적절한 앱 종료 이벤트를 처리. exit(0)
                        }
                    });
                }
            }));
        }
            
            break;
        case BUTTON_SHOW_CONFLICTSELECTION: {
            PlayerInfo playerInfo = GameManager::getInstance()->GetCurrentPlayerInfo();
            PlayerInfo conflictPlayer = GameManager::getInstance()->GetConflictPlayerInfo();

            IConflictViewInfo* info = nullptr;
            ConflictViewInfo* _info = new ConflictViewInfo(playerInfo.playerId,conflictPlayer.playerId);
                _info->SetValue("ID", playerInfo.playerId, conflictPlayer.playerId);
                _info->SetValue("Name", playerInfo.playerName.c_str(), conflictPlayer.playerName.c_str());
                _info->SetValue("Token", playerInfo.playerToken.c_str(), conflictPlayer.playerToken.c_str());
                info = _info;

            AuthV4::showConflictSelection(*info, [=](ResultAPI const & result, PlayerInfo const & playerInfo){
                if(ErrorCheckResultAPI(result)){
                    GameManager::getInstance()->SetCurrentPlayerInfo(playerInfo);
                    MessageBox(playerInfo);
                    Logger::log("playerInfo : %s\n",getPlayerInfoString(playerInfo).c_str());
                }
                else if (result.needExit()) {
                    Logger::log("result.needExit() = true");
                    // result.needExit() true일 경우 적절한 앱 종료 이벤트를 처리. exit(0)
                }
            });
            delete info;

        }
            break;
        case BUTTON_ISREADYENGAGEMENT:
        {
            bool isready = this->getSelectedToggleButtonIndex(TOGGLE_BUTTON_IS_READY_ENGAGEMENT);
            Promotion::setEngagementReady(isready);
            auto isreadystring = isready ? "set ready : on" : "set ready : off";
            MessageBox(isreadystring,"Engagement");
            
            LogHead("setEngagementReady");
            Logger::log("Engagement = %s\n",isready?"true":"false");
        }
            break;
            
        case BUTTON_CHECK_PROVIDER:{
            showProviderSelectView([=](ProviderType type) {
                AuthV4::checkProvider(type, [=](ResultAPI const & result,ProviderInfo const & providerInfo){
                    LogHead("checkProvider",result);
                    if(ErrorCheckResultAPI(result)){
                        MessageBox(providerInfo);
                    }
                    Logger::log("info = %s\n",getProviderInfoString(providerInfo).c_str());
                });
            });
        }
            break;
            
        case BUTTON_CHECK_MAINTENANCE:{
            bool maintenanceUI = getSelectedToggleButtonIndex(TOGGLE_BUTTON_IS_SHOW_MAINTENANCE);
            AuthV4::checkMaintenance(maintenanceUI, [=](ResultAPI const & result,std::vector<AuthV4MaintenanceInfo> const & maintenanceInfolist){
                LogHead("checkMaintenance",result);
                if(ErrorCheckResultAPI(result)){
                    if(!maintenanceUI){
                        //기본 메인터넌스 체크 UI 사용안할경우 정보를 출력한다.
                        MessageBox(maintenanceInfolist);
                    }
                    Logger::log("info = %s\n",getMaintenanceInfolistString(maintenanceInfolist).c_str());
                }
                else if (result.needExit()) {
                    Logger::log("result.needExit() = true");
                    // result.needExit() true일 경우 적절한 앱 종료 이벤트를 처리. exit(0)
                }
            });
        }
            break;
            
        case BUTTON_CHECK_BLACKLIST:{
            //TODO isShow 얻기
            bool blacklistUI = getSelectedToggleButtonIndex(TOGGLE_BUTTON_IS_SHOW_MAINTENANCE);
            AuthV4::checkBlacklist(blacklistUI, [=](ResultAPI const & result,std::vector<AuthV4MaintenanceInfo> const & maintenanceInfolist){
                LogHead("checkBlacklist",result);
                if(ErrorCheckResultAPI(result)){
                    //기본 블랙리스트 체크 UI 사용안할경우 정보를 출력한다.
                    if(!blacklistUI){
                        MessageBox(maintenanceInfolist);
                    }
                    Logger::log("info = %s\n",getMaintenanceInfolistString(maintenanceInfolist).c_str());
                }
                else if (result.needExit()) {
                    Logger::log("result.needExit() = true");
                    // result.needExit() true일 경우 적절한 앱 종료 이벤트를 처리. exit(0)
                }
            });
        }
            break;
            
        case BUTTON_SETPROVIDERCHANGEHANDLER:{
            AuthV4::setProviderChangedListener([=](ResultAPI const & result,ProviderInfo const & providerInfo){
                LogHead("setProviderChangedListener",result);
                if(ErrorCheckResultAPI(result)){
                    MessageBox(providerInfo);
                    auto infostr = getProviderInfoString(providerInfo);
                    Logger::log("providerInfo : %s",infostr.c_str());
                }
            });
        }
            break;
            
        case BUTTON_SHOWSIGNIN:{
            AuthV4::showSignIn([=](ResultAPI const & result,PlayerInfo const & playerInfo){
                LogHead("showSignIn",result);
                if(ErrorCheckResultAPI(result)){
                    Postbox::setLoginType(Postbox::v4);
                    GameManager::getInstance()->SetCurrentPlayerInfo(playerInfo);
                    MessageBox(playerInfo);
                    auto infostr = getPlayerInfoString(playerInfo);
                    Logger::log("playerinfo : %s\n",infostr.c_str());
                    
                }
                else if (result.needExit()) {
                    Logger::log("result.needExit() = true");
                    // result.needExit() true일 경우 적절한 앱 종료 이벤트를 처리. exit(0)
                }
            });
        }
            break;

        case BUTTON_SHOWINQUIRY:{
            AuthV4::showInquiry([=](ResultAPI const & result){
                showResult(result);
                LogHead("showInquiry",result);
            });
            break;
        }
        case BUTTON_SHOWCHATBOTINQUIRY:{
            // 챗봇 페이지 바로가기 호출시 전달받기로한 약속된 JSON 형식의 String 데이터
            std::string additionalInfo = "{\"evt_code\":1000,\"p1\":\"random\"}";
            AuthV4::showChatbotInquiry(additionalInfo, [=](ResultAPI const & result){
                showResult(result);
                LogHead("showChatbotInquiry",result);
            });
            break;
        }
        case BUTTON_SHOWMYINQUIRY: {
            AuthV4::showMyInquiry([=](ResultAPI const & result){
                showResult(result);
                LogHead("showMyInquiry",result);
            });
            break;
        }
        case BUTTON_SHOWTERMS:{
            AuthV4::showTerms([=](ResultAPI const & result){
                showResult(result);
                LogHead("showTerms",result);
            });
        }
            break;
            
        case BUTTON_SHOWADULTCONFIRM:{
            AuthV4::showAdultConfirm([=](ResultAPI const & result){
                showResult(result);
                LogHead("showAdultConfirm",result);
            });
        }
            break;
            
        case BUTTON_GETPROFILE:{
            pushView(InputList::create("Input Player Lists", [=](InputList* lists,std::vector<std::string> inputLists){
                //입력받은 플레이어 아이디를 스트링에서 롱롱으로

                if (inputLists.empty())
                {
                    MessageBox("Input list is empty","User Infos");
                    Logger::log("Input list is empty");
                    return;
                }

                std::vector<PlayerID> playerIdList;
                for(auto it = inputLists.begin();it != inputLists.end();++it){
                    char* pEnd;
                    if((*it) != "" && (*it).size() > 0)
                        playerIdList.push_back(strtoll((*it).c_str(),&pEnd,10));
                }
                AuthV4::getProfile(playerIdList, [=](ResultAPI const & result,std::vector<ProfileInfo> const & playerInfoList){
                    //모두 출력
                    LogHead("getProfile",result);
                    if(ErrorCheckResultAPI(result)){
                        std::string users;
                        for(auto it = playerInfoList.begin();it != playerInfoList.end();++it){
                            users += getProfileInfoString(*it)+"\n";
                        }
                        MessageBox(users.c_str(),"User Infos");
                        Logger::log("users : %s\n",users.c_str());
                    }
                });
            }));
        }
            break;
            
        case BUTTON_SHOWPROFILE:{
            char* pEnd;
            std::string value = getEditboxValue(EDITBOX_SHOW_PROFILE);
            PlayerID playerId = strtoll(value.c_str(),&pEnd,10);
            if(value.size() == 0 ){
                
                playerId = GameManager::getInstance()->GetCurrentPlayerInfo().playerId;
            }
            AuthV4::showProfile(playerId, [=](ResultAPI const & result){
                showResult(result);
                LogHead("showProfile",result);
            });
        }
            break;
            
        case BUTTON_RESET:{
            AuthV4::reset();
            LogHead("reset");
        }
            break;
            
        case BUTTON_ISAUTOSIGNIN:{
            bool isAutoSignIn = AuthV4::isAutoSignIn();
            MessageBox(isAutoSignIn == true ? "isAutoSignIn : true" : "isAutoSignIn : false", "isAutoSignIn");
            LogHead("isAutoSignIn");
            Logger::log("isAutoSignIn : %s\n",isAutoSignIn ? "true":"false");
            
        }
            break;
        case BUTTON_GETPLAYERINFO:{
            PlayerInfo info = AuthV4::getPlayerInfo();
            GameManager::getInstance()->SetCurrentPlayerInfo(info);
            MessageBox(info);
            LogHead("getPlayerInfo");
            Logger::log("PlayerInfo : %s\n",getPlayerInfoString(info).c_str());
            
        }
            break;
        case BUTTON_GETPROVIDERFRIENDLIST:{
            showProviderSelectView(GameManager::getInstance()->GetCurrentPlayerInfo(), [=](ProviderType selectProviderType){
                AuthV4::getProviderFriendsList(selectProviderType, [=](ResultAPI const & result,ProviderType providerType,std::map<std::string,PlayerID> providerUserIdList){
                    LogHead("getProviderFriendsList",result);
                    if(ErrorCheckResultAPI(result)){
                        Logger::log("ProviderType : %s\n",ProviderInfo::stringProviderType(providerType).c_str());
                        Logger::log("providerUserIdList = \n");
                        for(auto it : providerUserIdList){
                            Logger::log("ProviderID : %s , PlayerID : %lld\n",it.first.c_str(),it.second);
                        }
                    }
                });
            });
        }
            break;
        case BUTTON_SHOW_GAME_CENTER_LOGINCANCELDIALOG:
            AuthV4::showGameCenterLoginCancelDialog([=](bool isDismiss){
                Logger::log("showGameCenterLoginCancelDialog isDismiss : %s",isDismiss ? "true" : "false");
            });
            break;
        case BUTTON_GET_AGEGATEU13: {
//            bool ageGateU13 = AuthV4::getAgeGateU13();
            
            bool ageGateU13 = AuthV4::getAgeGateU13();
            
            MessageBox(ageGateU13 == true ? "ageGateU13 : true" : "ageGateU13 : false", "ageGateU13");
            LogHead("ageGateU13");
            Logger::log("ageGateU13 : %s\n",ageGateU13 ? "true":"false");
        }
            break;
        case BUTTON_IS_ACTIVATE_LOGGING: {
            bool isActivateLogging = Logger::isActivateLogging();
            MessageBox(isActivateLogging == true ? "isActivateLogging : true" : "isActivateLogging : false", "isActivateLogging");
        }
            break;
        case BUTTON_REQUEST_USER_PERMISSIONS: {

            std::vector<std::string> requests;

            std::string editBoxStr = getEditboxValue(EDITBOX_PERMISSION_LIST);
            std::istringstream ss(editBoxStr);
            std::string unitStr;

            while (std::getline(ss, unitStr, '\n')) {
                requests.push_back(unitStr);
            }

            PlatformHelper::requestUserPermissions(requests, [=](ResultAPI const & result, std::vector<std::string> const & granted, std::vector<std::string> const & denied) {
                Logger::log("=== PlatformHelper::requestUserPermissions ===\n");
                Logger::log("ResultAPI : %s\n", result.toString().c_str());

                if (result.code == hive::ResultAPI::PlatformHelperOSNotSupported) {
                    Logger::log("Only support Android\n");
                }
                else if (result.code == hive::ResultAPI::PlatformHelperOSVersionNotSupported) {
                    Logger::log("Android OS version not supported.\n");
                }

                if (!granted.empty()) {
                    Logger::log("동의한 권한 목록 : \n");
                    for (std::string name : granted) {
                        Logger::log("%s\n",name.c_str());
                    }
                }

                if (!denied.empty()) {
                    Logger::log("\n거절한 권한 목록: \n");
                    for (std::string name : denied) {
                        Logger::log("%s\n",name.c_str());
                    }
                }
            });
        }
            break;
        case BUTTON_SHOW_DEVICEMANAGEMENT: {
            AuthV4::showDeviceManagement([=](ResultAPI const & result){
                showResult(result);
                LogHead("showDeviceManagement",result);
            });
        }
            break;
        case COMBOBOX_EXTERNAL_AUTH_PROVIDER: {
            int selectedIndex = getSelectedComboboxIndex(COMBOBOX_EXTERNAL_AUTH_PROVIDER);
            externalAuthProvider = externalAuthProviderList[selectedIndex];
            setEditboxValue(EDITBOX_EXTERNAL_AUTH_PROVIDER, externalAuthProvider);
        }
            break;
    }
}

std::string AuthV4Test::getResultString(hive::ResultAPI const & api){
    return ResultUtils::getResultAPIString(api);
}
void AuthV4Test::LogHead(const char* funcName){
    ResultUtils::logHead("AuthV4", funcName);
}
void AuthV4Test::LogHead(const char* funcName,hive::ResultAPI const & api){
    ResultUtils::logHead("AuthV4", funcName, api);
}
void AuthV4Test::showResult(hive::ResultAPI const & api) {
    scheduleOnce([=](float dt){
        ResultUtils::showResult(api);
    }, 0.5f, "showResult");
    
}

bool AuthV4Test::ErrorCheckResultAPI(hive::ResultAPI const & api) {
    return ResultUtils::errorCheckResultAPI(api);
}
void AuthV4Test::showProviderSelectView(ProviderSelectCallback callback) {
    showProviderSelectView(toStringVector(providerList),callback);
}

void AuthV4Test::showProviderSelectView(const PlayerInfo& info,ProviderSelectCallback callback) {
    std::vector<std::string> v = getPlayerProviderStringList(GameManager::getInstance()->GetCurrentPlayerInfo());
    showProviderSelectView(v,callback);
}

void AuthV4Test::showProviderSelectView(std::vector<std::string> list,ProviderSelectCallback callback) {
    pushView(Combobox::create("Select Provider", {screenSize.width, DEFAULT_CONTENT_HEIGHT}, std::move(list), true, [=](const char *selectItem,int index){
        ProviderType type = ProviderInfo::providerTypeFromString(selectItem);
        callback(type);
    }));
}


