//
//  BaseView.cpp
//  hivesdk
//
//  Created by paikjongman on 2017. 2. 24..
//
//

#include "BaseView.hpp"
#include "cocos2d.h"
#include "ui/CocosGUI.h"
#include "UICombobox.hpp"
#include "UIMultiselect.hpp"
#include <sstream>
#include <time.h>

using namespace cocos2d;
using namespace cocos2d::ui;

std::string BaseView::logString = "";
int BaseView::normalButtonHeight = DEFAULT_CONTENT_HEIGHT;

BaseView::~BaseView()
{
    //혹시 제거되지 않은 노드가 있으면 제거해준다.
    for(auto& w:temporaryListItem) {
        w.second.widget->release();
    }
    
    
    for(auto& v:temporaryBaseView) {
        v.second->release();
    }
}

bool BaseView::init()
{
    if(!Node::init())
        return false;
    
    prevView = nullptr;
    
    auto safe = Director::getInstance()->getSafeAreaRect();
    
    auto layout = ListView::create();
    addChild(layout);
    _myListView = layout;
    layout->setPosition(safe.origin);//Vec2(0,0));
    layout->setContentSize(safe.size);//Director::getInstance()->getVisibleSize());
    layout->setTag(DEFAULT_LAYOUT_TAG);
    layout->setLayoutType(Layout::Type::VERTICAL);
    layout->setItemsMargin(10.0f);
    
    currentLayout.push(layout);

    screenSize = Director::getInstance()->getWinSize();
    
    scheduleUpdate();
    return true;
}


void BaseView::update(float dt)
{
    auto safe = Director::getInstance()->getSafeAreaRect();
    this->setContentSize(safe.size);
    _myListView->setContentSize(safe.size);
    
}
void BaseView::transitionView(BaseView *view)
{
    Node* p = getParent();
    assert(p != nullptr);
    this->setVisible(false);
    p->addChild(view);
}
void BaseView::pushView(BaseView* view)
{
    view->prevView = this;
    transitionView(view);
}
void BaseView::popView()
{
    assert(prevView != nullptr);
    prevView->setVisible(true);
    Node *p = getParent();
    assert(p != nullptr);
    p->removeChild(this);
}

std::string BaseView::getCurrentDateTime() {
    // 현재 시간을 밀리세컨드 단위까지 표현 가능
    // 년, 월, 일 ,시, 분, 초
    time_t time = std::time(nullptr);
    struct tm* timeInfo = localtime(&time);

    // 밀리세컨드
    struct timeval now;
    if (gettimeofday(&now, nullptr) != 0)
    {
        return "";
    }

    std::stringstream date;
    date << timeInfo->tm_year+1900 << "-" << timeInfo->tm_mon+1 << "-" << timeInfo->tm_mday << " ";
    date << timeInfo->tm_hour << ":" << timeInfo->tm_min << ":" << timeInfo->tm_sec << "." << now.tv_usec / 1000;

    return date.str();
}

void BaseView::setUseBackButton(bool isUse) {
    setUseBackButton(isUse, "Back");
}

void BaseView::setUseBackButton(bool isUse, const char* name)
{
    if(isUse)
    {
        auto button = makeButton(name, {screenSize.width, DEFAULT_CONTENT_HEIGHT});
        button->addTouchEventListener([=](Ref* r,Widget::TouchEventType t){
            if(t == Widget::TouchEventType::ENDED) {
                popView();
            }
        });
        
        auto layout = static_cast<ListView*>(getLayout());
        layout->insertCustomItem(button, 0);
    }
}

void BaseView::setUseShowLogButton(bool isUse)
{
    if(isUse)
    {
        BeginHorizontal();
        auto clearButton = makeButton("ClearLog", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT});
        LinearLayoutParameter* param = (LinearLayoutParameter*)clearButton->getLayoutParameter();
        param->setGravity(cocos2d::ui::LinearLayoutParameter::LinearGravity::LEFT);
        clearButton->addTouchEventListener([=](Ref* r,Widget::TouchEventType t){
            if(t == Widget::TouchEventType::ENDED) {
                logString = "";
            }
        });
        
        addWidget(clearButton, Node::INVALID_TAG);
      
        auto showButton = makeButton("ShowLog", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT});
        param = (LinearLayoutParameter*)showButton->getLayoutParameter();
        param->setGravity(cocos2d::ui::LinearLayoutParameter::LinearGravity::RIGHT);
        showButton->addTouchEventListener([=](Ref* r,Widget::TouchEventType t){
            if(t == Widget::TouchEventType::ENDED) {
                MessageBox(logString.c_str(), "Log");
            }
        });
        
        addWidget(showButton, Node::INVALID_TAG);
        
        EndHorizontal();
    }
}

void BaseView::addLogMessage(std::string msg)
{
    logString += msg + '\n';
}

void BaseView::addCallbackLogMessage(std::string msg, std::string title)
{
    logString += title + '\n';
    logString += "{\n";
    logString += msg;
    logString += "}\n\n";
}

void BaseView::addBaseView(BaseView* baseView,int tag)
{
    assert(getBaseView(tag) == nullptr);
    temporaryBaseView.insert(std::make_pair(tag,baseView));
    baseView->retain();
}

BaseView* BaseView::getBaseView(int tag)
{
    if(temporaryBaseView.find(tag) == temporaryBaseView.end())
        return nullptr;
    return temporaryBaseView[tag];
}

Button* BaseView::makeButtonImage(const char* buttonPath)
{
    auto button = Button::create(buttonPath);
    button->setLayoutParameter(getDefaultLayout());
    return button;
}


Button* BaseView::makeButton(const char* buttonName)
{
    return makeButton(buttonName, {screenSize.width, DEFAULT_CONTENT_HEIGHT});
}

Button* BaseView::makeButton(const char *buttonName, const cocos2d::Size size)
{
    auto button = makeButtonImage("button.png");
    button->setScale9Enabled(true);
    button->setTitleText(buttonName);
    button->setContentSize(Size(size.width, size.height));
    button->setTitleFontSize(DEFAULT_FONT_SIZE);
    return button;
}


void BaseView::changeMultiSelectList(const std::vector<const char*>& list,int tag)
{
    auto it = temporaryBaseView.find(tag);
    assert(it != temporaryBaseView.end());
    
    auto multiSelect = dynamic_cast<MultiSelect*>(it->second);
    assert(multiSelect != nullptr);

    auto button = getUIWidget<Button>(tag);
    auto view =  makeMultiSelect(multiSelect->getTitle().c_str(),list,button,tag);
    it->second = view;
    view->retain();
    multiSelect->release();
}
void BaseView::cleanMultiSelect(int tag)
{
    auto multiSelect = dynamic_cast<MultiSelect*>(getBaseView(tag));
    multiSelect->clearSelect();
}
void BaseView::setSelectionMultiSelectIndexes(const std::vector<int>& indexes,int tag)
{
    auto multiSelect = static_cast<MultiSelect*>(getBaseView(tag));
    for(auto& i:indexes){
        multiSelect->selectFromIndex(i);
    }
}

std::vector<int> BaseView::getMultiSelectIndexes(int tag)
{
    auto multiSelect = dynamic_cast<MultiSelect*>(getBaseView(tag));
    assert(multiSelect != nullptr);
    return multiSelect->getSelectIndexes();
}

MultiSelect* BaseView::makeMultiSelect(const char* name,const std::vector<const char*>& list,cocos2d::ui::Button* targetButton,int tag)
{
    std::vector<std::string> v = toStringVector(list);
    
    auto multiSelectCallback = [=](MultiSelect* select, const std::vector<int>& selectedIndexes) {
        std::string buttonCharactor = "";
        for(const auto& i:selectedIndexes) {
            buttonCharactor += StringUtils::format("[%s] ",v[i].c_str());
        }
        targetButton->setTitleText(buttonCharactor.size() > 0 ? buttonCharactor : "...");
    };
    
    auto view = MultiSelect::create(name,std::move(v),multiSelectCallback);
    
    targetButton->addTouchEventListener([=](Ref* r,Widget::TouchEventType t){
        if(t == Widget::TouchEventType::ENDED) {
            pushView(getBaseView(tag));
        }
    });
    
    return view;
}

void BaseView::addMultiSelectButton(const char* name,const std::vector<const char*>& list,int tag)
{
    addLabel(name);
    auto multiSelect = static_cast<MultiSelect*>(getBaseView(tag));
    auto button = makeButton("...");
    
    if(multiSelect == nullptr)
    {
        multiSelect = makeMultiSelect(name,list,button,tag);
        addBaseView(multiSelect,tag);
    }
    
    addWidget(button, tag);
}

void BaseView::addCombobox(const char *name,const std::vector<const char*>& list,int defaultSelection,int tag)
{
    addCombobox(name, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, list, defaultSelection, tag);
}

void BaseView::addCombobox(const char *name,const std::vector<const char*>& list,int defaultSelection,int tag, int additionalHeight)
{
    addCombobox(name, {screenSize.width, (float)(DEFAULT_CONTENT_HEIGHT + additionalHeight)}, list, defaultSelection, tag);
}

void BaseView::addCombobox(const char *name, const cocos2d::Size size, const std::vector<const char*>& list,int defaultSelection,int tag)
{
    BeginHorizontal();
    auto label = addLabel(name);
    //auto button = makeButton(list[defaultSelection], size);

    std::vector<const char*> listCopy = list;

    if (listCopy.size() <= 0) {
        listCopy.push_back("empty");
    }

    auto button = makeButton(listCopy[defaultSelection], {size.width - label->getContentSize().width, size.height});
    button->setName(name);
    combobox_selected[tag] =defaultSelection;

    std::vector<std::string> v = toStringVector(listCopy);

    button->addTouchEventListener([=](Ref* r,Widget::TouchEventType t){
        if(t == Widget::TouchEventType::ENDED) {

            pushView(Combobox::create(name, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, move(v), true,  [=](const char* selectItem, int index){
                CCLOG("combobox select %s %d",selectItem,index);
                button->setTitleText(selectItem);
                combobox_selected[tag] = index;
                this->onButtonEvent(tag);
            }));
        }
    });
    addWidget(button,tag);
    EndHorizontal();
}

void BaseView::addButton(const char* buttonName,int tag)
{
    addButton(buttonName, {screenSize.width, (float)BaseView::normalButtonHeight}, tag);
}

void BaseView::addButton(const char *buttonName, const cocos2d::Size size, int tag)
{
    auto button = makeButton(buttonName, size);
    button->addTouchEventListener([=](Ref* r,Widget::TouchEventType t){
        if(t == Widget::TouchEventType::ENDED) {
            this->onButtonEvent(tag);
        }
    });
    addWidget(button,tag);
}

void BaseView::addButton(const char* buttonName, ButtonMode mode, int tag)
{
    if (mode == ButtonMode::NORMAL) {
        // 일반 버튼
        addButton(buttonName, tag);
        return;
    }

    BeginHorizontal();
    // 일반 버튼 + parameter setting 버튼
    auto buttonNormal = makeButton(buttonName, {screenSize.width - DEFAULT_PARAM_BUTTON_WIDTH, (float)BaseView::normalButtonHeight});
    buttonNormal->addTouchEventListener([=](Ref* r,Widget::TouchEventType t){
        if(t == Widget::TouchEventType::ENDED) {
            this->onButtonEvent(tag);
        }
    });
    addWidget(buttonNormal, tag);

    int buttonParamTag = DEFAULT_PARAM_BUTTON_TAG + tag;
    auto buttonParam = makeButtonImage("SettingIcon.png");
    buttonParam->addTouchEventListener([=](Ref* r,Widget::TouchEventType t){
        if(t == Widget::TouchEventType::ENDED) {
            this->onButtonEvent(buttonParamTag);
        }
    });
    addWidget(buttonParam, buttonParamTag);

    EndHorizontal();
}


void BaseView::addButton(const char* buttonName, ButtonMode mode, int tag, int additionalHeight)
{
    if (mode == ButtonMode::NORMAL) {
        // 일반 버튼
        addButton(buttonName, tag);
        return;
    }

    int buttonHeight = normalButtonHeight + additionalHeight;

    BeginHorizontal();
    // 일반 버튼 + parameter setting 버튼
    auto buttonNormal = makeButton(buttonName, {screenSize.width - DEFAULT_PARAM_BUTTON_WIDTH, (float)buttonHeight});
    buttonNormal->addTouchEventListener([=](Ref* r,Widget::TouchEventType t){
        if(t == Widget::TouchEventType::ENDED) {
            this->onButtonEvent(tag);
        }
    });
    addWidget(buttonNormal, tag);

    int buttonParamTag = DEFAULT_PARAM_BUTTON_TAG + tag;
    auto buttonParam = makeButtonImage("SettingIcon.png");
    buttonParam->addTouchEventListener([=](Ref* r,Widget::TouchEventType t){
        if(t == Widget::TouchEventType::ENDED) {
            this->onButtonEvent(buttonParamTag);
        }
    });
    addWidget(buttonParam, buttonParamTag);

    EndHorizontal();
}


void BaseView::addButtonImage(const char *img,int tag) {
    auto button = makeButtonImage(img);
    button->addTouchEventListener([=](Ref* r,Widget::TouchEventType t){
        if(t == Widget::TouchEventType::ENDED) {
            this->onButtonEvent(tag);
        }
    });
    addWidget(button,tag);
}

void BaseView::addButtonImage(const char *img, const cocos2d::Size size, int tag)
{
    auto button = makeButtonImage(img);
    button->setContentSize(size);
    button->addTouchEventListener([=](Ref* r,Widget::TouchEventType t){
        if(t == Widget::TouchEventType::ENDED) {
            this->onButtonEvent(tag);
        }
    });
    addWidget(button,tag);
}


Button* BaseView::addTrueFalseToggleButton(const char* name, int defaultSelection, cocos2d::Size size, int tag)
{
    return addToggleButton(name, {"False", "True"}, defaultSelection, size, tag);
}

Button* BaseView::addOnOffToggleButton(const char* name, int defaultSelection, cocos2d::Size size, int tag)
{
    return addToggleButton(name, {"OFF", "ON"}, defaultSelection, size, tag);
}

Button* BaseView::addToggleButton(const char* name, const std::vector<const char*>& buttons, int defaultSelection, cocos2d::Size size, int tag)
{
    toggle_selected[tag] = defaultSelection;
    const char* selectedName = buttons[defaultSelection];

    BeginHorizontal();

    auto label = addLabel(name);
    auto button = makeButton(selectedName, {size.width - label->getContentSize().width, DEFAULT_CONTENT_HEIGHT});
    button->addTouchEventListener([=](Ref* r, Widget::TouchEventType t){
        if(t == Widget::TouchEventType::ENDED) {
            this->onButtonEvent(tag);
            toggle_selected[tag] =  ++toggle_selected[tag] % buttons.size();
            button->setTitleText(buttons[toggle_selected[tag]]);
        }
    });
    addWidget(button, tag);

    EndHorizontal();

    return button;
}

void BaseView::BeginHorizontal()
{
    auto buttonContainer = Layout::create();
    buttonContainer->setLayoutType(Layout::Type::HORIZONTAL);
    buttonContainer->setLayoutParameter(getDefaultLayout());
    currentLayout.push(buttonContainer);
}

void BaseView::EndHorizontal()
{
    auto layout = currentLayout.top();
    currentLayout.pop();
    
    //set layout size
    auto it = layout->getChildren().begin();
    auto accruedSize = Size(0,0);
    for(;it != layout->getChildren().end();++it)
    {
        auto size = (*it)->getContentSize();
        accruedSize = Size(accruedSize.width+size.width,MAX(accruedSize.height, size.height));
    }
    layout->setContentSize(accruedSize);
    addLayout(layout);
}

void BaseView::addEditBox(const char* placeholder,int tag)
{
    addEditBox(nullptr,placeholder, tag);
}

void BaseView::addEditBox(const char *name, const char *placeholder, int tag)
{
    addEditBox(name,nullptr,placeholder, tag);
}

void BaseView::addEditBox(const char* name,const char* text,const char* placeholder,int tag)
{
    addEditBox(name, text, placeholder, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, DEFAULT_FONT_SIZE,  tag);
}
void BaseView::addEditBox_Numeric(const char* name,const char* text,const char* placeholder,int tag)
{
    addEditBox(name, text, placeholder, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::NUMERIC, DEFAULT_FONT_SIZE,  tag);
}
void BaseView::addEditBox(const char* name, const char* text, const char* placeholder, cocos2d::Size size, cocos2d::ui::EditBox::InputMode inputMode, int fontSize, int tag)
{
    auto nameLabel = addLabel("");
    if(name != nullptr && strlen(name) != 0)
    {
        BeginHorizontal();
        nameLabel = addLabel(name);
    }

    auto image = Scale9Sprite::create("editbox.png");
    auto editbox = EditBox::create({size.width - nameLabel->getContentSize().width, size.height}, image);
    editbox->setFontColor(Color3B::BLACK);
    editbox->setFontSize(fontSize);
    editbox->setPlaceholderFontSize(fontSize);
    editbox->setPlaceHolder(placeholder);
    editbox->setLayoutParameter(getDefaultLayout());
    editbox->setInputMode(inputMode);

    if(text != nullptr && strlen(text) != 0)
    {
        editbox->setText(text);
    }

    addWidget(editbox,tag);

    if(name != nullptr && strlen(name) != 0)
    {
        EndHorizontal();
    }
}
cocos2d::ui::Text* BaseView::addLabel(const char *text, cocos2d::ui::LinearLayoutParameter::LinearGravity gravity, int fontSize, int tag)
{
    auto label = Text::create();
    label->setString(text);
    label->setFontSize(fontSize);
    LinearLayoutParameter* param = (LinearLayoutParameter*)getDefaultLayout();
    param->setGravity(gravity);
    label->setLayoutParameter(param);
    addWidget(label,tag);

    return label;
}
cocos2d::ui::Text* BaseView::addLabel(const char *text, LinearLayoutParameter::LinearGravity gravity, int tag)
{
    return addLabel(text, LinearLayoutParameter::LinearGravity::LEFT, DEFAULT_FONT_SIZE, tag);
}

cocos2d::ui::Text* BaseView::addLabel(const char* text,int tag)
{
    return addLabel(text, LinearLayoutParameter::LinearGravity::LEFT, tag);
}
cocos2d::ui::Text* BaseView::addLabel(const char *text)
{
    return addLabel(text,Node::INVALID_TAG);
}
void BaseView::addDottedLine()
{
    addLabel("──────────────────────────", LinearLayoutParameter::LinearGravity::CENTER_HORIZONTAL, Node::INVALID_TAG);
}
void BaseView::setButtonHeight(int height)
{
    normalButtonHeight = height;
}
cocos2d::ui::Layout* BaseView::getLayout()
{
    return currentLayout.top();//getChildByTag<Layout*>(DEFAULT_LAYOUT_TAG);
}

LayoutParameter* BaseView::getDefaultLayout() const
{
    auto param = LinearLayoutParameter::create();
    param->setGravity(LinearLayoutParameter::LinearGravity::LEFT);
    return param;
}
void BaseView::addLayout(cocos2d::ui::Widget* layout)
{
    addWidget(layout, Node::INVALID_TAG);
}
void BaseView::addWidget(cocos2d::ui::Widget* widget,int tag)
{
    addWidget(widget, widget->getLocalZOrder(),tag);
}

void BaseView::addWidget(cocos2d::ui::Widget* widget,int localZOrder,int tag)
{
    getLayout()->addChild(widget,localZOrder);
    assert(uis.find(tag) == uis.end());
    if(tag != Node::INVALID_TAG)
        uis[tag] = widget;
}

void BaseView::changeLabelText(std::string text,int tag)
{
    auto label = getUIWidget<Text>(tag);
    label->setString(text);
}

void BaseView::changeComboboxList(const std::vector<const char*>& list, int defaultSelection, int tag)
{
    auto button = getUIWidget<Button>(tag);
    button->setTitleText(list[defaultSelection]);
    combobox_selected[tag] = defaultSelection;
    
    std::vector<std::string> v = toStringVector(list);
    
    button->addTouchEventListener([=](Ref* r,Widget::TouchEventType t){
        if(t == Widget::TouchEventType::ENDED) {
            pushView(Combobox::create(button->getName().c_str(), {screenSize.width, DEFAULT_CONTENT_HEIGHT}, move(v), true, [=](const char *selectItem,int index){
                CCLOG("combobox select %s %d",selectItem,index);
                button->setTitleText(selectItem);
                combobox_selected[tag] = index;
                
            }));
        }
    });
}

void BaseView::changeComboboxList(const std::vector<const char*>& list, int defaultSelection, int tag, int additionalHeight)
{
    auto button = getUIWidget<Button>(tag);
    button->setTitleText(list[defaultSelection]);
    combobox_selected[tag] = defaultSelection;

    std::vector<std::string> v = toStringVector(list);

    int buttonHeight = DEFAULT_CONTENT_HEIGHT + additionalHeight;

    button->addTouchEventListener([=](Ref* r,Widget::TouchEventType t){
        if(t == Widget::TouchEventType::ENDED) {
            pushView(Combobox::create(button->getName().c_str(), {screenSize.width, (float)buttonHeight}, move(v), true, [=](const char *selectItem,int index){
                CCLOG("combobox select %s %d",selectItem,index);
                button->setTitleText(selectItem);
                combobox_selected[tag] = index;

            }));
        }
    });
}

std::string BaseView::getEditboxValue(int tag)
{
    auto editbox = getUIWidget<EditBox>(tag);
    return editbox->getText();
}

std::string BaseView::setEditboxValue(int tag, std::string value)
{
    auto editbox = getUIWidget<EditBox>(tag);
    editbox->setText(value.c_str());
    return editbox->getText();
}

std::string BaseView::getComboboxValue(int tag)
{
    auto button = getUIWidget<Button>(tag);
    return button->getTitleText();
}

int BaseView::getSelectedComboboxIndex(int tag)
{
    return combobox_selected[tag];
}

int BaseView::getSelectedToggleButtonIndex(int tag)
{
    return toggle_selected[tag];
}

void BaseView::setTrueFalseToggleButtonIndex(int index, int tag) {
    auto button = getUIWidget<Button>(tag);
    const std::vector<const char*>& tfv = {"False", "True"};
    button->setTitleText(tfv[index]);
    toggle_selected[tag] = index;
}

void BaseView::setOnOffToggleButtonIndex(int index, int tag) {
    auto button = getUIWidget<Button>(tag);
    const std::vector<const char*>& ofv = {"OFF", "ON"};
    button->setTitleText(ofv[index]);
    toggle_selected[tag] = index;
}

Widget* BaseView::getUIWidget(int tag)
{
    if(uis.find(tag) == uis.end())
    {
        return nullptr;
    }
    return uis[tag];
}
template<typename T>
T* BaseView::getUIWidget(int tag)
{
    auto widget = dynamic_cast<T*>(getUIWidget(tag));
    assert(widget != nullptr);
    return widget;
}
template cocos2d::ui::Text* BaseView::getUIWidget(int tag); //instantiate Text* type template

void BaseView::setUIEnabled(bool enable,int tag)
{
    auto widget = getUIWidget(tag);
    widget->setEnabled(enable);
}
void BaseView::setUIVisible(bool visible,int tag)
{
    auto widget = getUIWidget(tag);
    auto layout = getChildByTag<ListView*>(DEFAULT_LAYOUT_TAG);
    
    if(visible)
    {
        auto it = temporaryListItem.find(tag);
        assert(it != temporaryListItem.end());
        
        layout->insertCustomItem(it->second.widget, it->second.index);
        it->second.widget->release();
        
        temporaryListItem.erase(it);
    }
    else
    {
        auto mostTop = static_cast<Widget*>(findMostTopNode(widget));
        
        assert(temporaryListItem.find(tag) == temporaryListItem.end());
        
        size_t targetItemIndex = layout->getIndex(mostTop);
        assert(targetItemIndex != -1);
        
        mostTop->retain();
        temporaryListNode tempNode;
        tempNode.index = targetItemIndex;
        tempNode.widget = mostTop;
        temporaryListItem.insert(std::make_pair(tag, tempNode));
        layout->removeItem(targetItemIndex);
    }
}

cocos2d::Node* BaseView::findMostTopNode(cocos2d::Node* node)
{
    assert(node != nullptr);
    auto layout = getChildByTag<ListView*>(DEFAULT_LAYOUT_TAG);
    
    auto n = dynamic_cast<Widget*>(node);
    if(n != nullptr && layout->getIndex(n) != -1)
        return node;
    
    return findMostTopNode(node->getParent());
}

std::vector<std::string> BaseView::toStringVector(const std::vector<const char*>& v)
{
    std::vector<std::string> ret;
    for(auto item:v){
        std::string s;
        s.assign(item,strlen(item));
        ret.push_back(s);
    }
    return std::move(ret);
}

void BaseView::showSubTitle(const char *subTitle)
{
    BeginHorizontal();
    addLabel(subTitle, LinearLayoutParameter::LinearGravity::LEFT, DEFAULT_FONT_SIZE + 10, Node::INVALID_TAG);
    EndHorizontal();
}
