//
//  BaseView.hpp
//  hivesdk
//
//  Created by paikjongman on 2017. 2. 24..
//
//

#ifndef BaseView_hpp
#define BaseView_hpp

#include "cocos2d.h"
#include "ui/CocosGUI.h"
#include <HIVE_SDK_Plugin/HIVE_CPP.h>
#include <stack>

#define DEFAULT_LAYOUT_TAG 1
#define DEFAULT_FONT_SIZE 20            
#define DEFAULT_CONTENT_HEIGHT 50       // 일반 버튼 높이
#define ADDITIONAL_POSTBOX_ITEM_COMBOBOX_HEIGHT 30  // 우편함 아이템 상세 높이 추가
#define DEFAULT_PARAM_BUTTON_WIDTH 60   // 세팅 버튼 너비
#define DEFAULT_PARAM_BUTTON_HEIGHT 50  // 세팅 버튼 높이
#define DEFAULT_PARAM_BUTTON_TAG 10000  // 세팅 버튼 Default tag


class MultiSelect;

class BaseView : public cocos2d::Node
{
public:
    virtual bool init()override;
    
    /**
     뷰를 이동시킨다.
     현재 뷰는 제거된다.

     @param view 이동시킬 뷰
     */
    void transitionView(BaseView *view);
    
    /**
     뷰를 쌓는다.
     쌓는 뷰는 보이는 상태가 되며,
     현재 뷰는 안보이는 상태가 된다.

     @param view 쌓을 뷰
     */
    void pushView(BaseView* view);
    
    /**
     뷰를 내린다.
     현재 뷰는 사라진다.
     이전 뷰가 보이는 상태로 변경된다.

     */
    void popView();

    /*
     현재 시간(Date)을 밀리세컨드 단위까지 표시
     */
    static std::string getCurrentDateTime();
    
    virtual ~BaseView();
    
    virtual void update(float dt)override;

    // 다른 뷰에서 로그를 표시
    static void addLogMessage(std::string msg);
    static void addCallbackLogMessage(std::string msg, std::string title);
    
protected:

    enum ButtonMode: int {
        NORMAL = 0,
        NEED_PARAM = 1,
    };

    virtual void onButtonEvent(int buttonTag){};
    virtual void onRadioButtonSelectChange(int radioButtonTag,int index){};
    
    void addButton(const char *buttonName,int tag);
    void addButton(const char *buttonName, ButtonMode mode, int tag);
    void addButton(const char *buttonName, const cocos2d::Size size, int tag);
    void addButton(const char* buttonName, ButtonMode mode, int tag, int additionalHeight);
    void addButtonImage(const char *img,int tag);
    void addButtonImage(const char *img, const cocos2d::Size size, int tag);
    void addEditBox(const char* placeholder,int tag);
    void addEditBox(const char* name,const char* placeholder,int tag);
    void addEditBox(const char* name,const char* text,const char* placeholder,int tag);
    void addEditBox_Numeric(const char* name,const char* text,const char* placeholder,int tag);
    void addEditBox(const char* name, const char* text, const char* placeholder, cocos2d::Size size, cocos2d::ui::EditBox::InputMode inputMode, int fontSize, int tag);
    cocos2d::ui::Text* addLabel(const char *text, cocos2d::ui::LinearLayoutParameter::LinearGravity gravity, int fontSize, int tag);
    cocos2d::ui::Text* addLabel(const char *text, cocos2d::ui::LinearLayoutParameter::LinearGravity gravity, int tag);
    cocos2d::ui::Text* addLabel(const char *text,int tag);
    cocos2d::ui::Text* addLabel(const char *text);
    cocos2d::ui::Button* addTrueFalseToggleButton(const char* name, int defaultSelection, cocos2d::Size size, int tag);
    cocos2d::ui::Button* addOnOffToggleButton(const char* name, int defaultSelection, cocos2d::Size size, int tag);
    cocos2d::ui::Button* addToggleButton(const char* name, const std::vector<const char*>& buttons, int defaultSelection, cocos2d::Size, int tag);
    void addCombobox(const char *name,const std::vector<const char*>& list,int defaultSelection,int tag);
    void addCombobox(const char *name,const std::vector<const char*>& list,int defaultSelection,int tag, int additionalHeight);
    void addCombobox(const char *name, const cocos2d::Size size, const std::vector<const char*>& list,int defaultSelection,int tag);
    void addMultiSelectButton(const char* name,const std::vector<const char*>& list,int tag);
    void addDottedLine();
    void setButtonHeight(int height);

    void setUseBackButton(bool isUse);
    void setUseBackButton(bool isUse, const char* name);
    void setUseShowLogButton(bool isUse);

    
    void changeLabelText(std::string text,int tag);
    void changeComboboxList(const std::vector<const char*>& list, int defaultSelction, int tag);
    void changeComboboxList(const std::vector<const char*>& list, int defaultSelection, int tag, int additionalHeight);
    
    //multi select property 함수.
    void changeMultiSelectList(const std::vector<const char*>& list,int tag);
    void cleanMultiSelect(int tag);
    void setSelectionMultiSelectIndexes(const std::vector<int>& indexes,int tag);
    std::vector<int> getMultiSelectIndexes(int tag);
    //
    
    void setTrueFalseToggleButtonIndex(int index, int tag);
    void setOnOffToggleButtonIndex(int index, int tag);
    
    void setUIEnabled(bool enable,int tag);
    void setUIVisible(bool visible,int tag);
    
    std::string getEditboxValue(int tag);
    std::string setEditboxValue(int tag, std::string value);
    std::string getComboboxValue(int tag);
    int getSelectedComboboxIndex(int tag);
    int getSelectedToggleButtonIndex(int tag);
    
    cocos2d::ui::Widget* getUIWidget(int tag);
    template<typename T>
    T* getUIWidget(int tag);
    
    void BeginHorizontal();
    void EndHorizontal();
    
    void StartActivityIndicator();
    void EndActivityIndicator();
    
    void showSubTitle(const char *subTitle);
    
    // 스크린 사이즈
    cocos2d::Size screenSize;
    int defaultButtonHeight;

    BaseView* getBaseView(int tag);
    
protected:
    cocos2d::ui::Layout *getLayout();
    
    //multiSelectView를 만든다.
    std::vector<std::string> toStringVector(const std::vector<const char*>& v);
private:
    cocos2d::ui::LayoutParameter* getDefaultLayout() const;
    void addLayout(cocos2d::ui::Widget* layout);
    void addWidget(cocos2d::ui::Widget* widget,int tag);
    void addWidget(cocos2d::ui::Widget* widget,int localZOrder,int tag);
    BaseView *prevView;
    std::stack<cocos2d::ui::Layout*> currentLayout;
    std::map<int,cocos2d::ui::Widget*> uis;
    //TODO: 이렇게 할바에 combobox용 ui클래스 하나 만드는게 편할듯
    std::map<int,int> combobox_selected;

    //toggleButton에서 선택된 정보 key=tag, value = 선택된 값
    std::map<int,int> toggle_selected;
    
    struct temporaryListNode {
        ssize_t index;
        cocos2d::ui::Widget* widget;
    };
    //setUIVisible을통해 리스트에서 제외된 아이템을 담아놓습니다.
    std::map<int,temporaryListNode> temporaryListItem;
    //temporaryBaseView 임시 베이스 뷰 입니다.
    std::map<int,BaseView*> temporaryBaseView;
    
    void addBaseView(BaseView* baseView,int tag);
    
    friend BaseView;
    
    static std::string logString;
    static int normalButtonHeight;
    
    cocos2d::ui::Button* makeButton(const char* buttonName);
    cocos2d::ui::Button* makeButton(const char* buttonName, const cocos2d::Size size);
    cocos2d::ui::Button* makeButtonImage(const char* imgPath);
    MultiSelect* makeMultiSelect(const char* name,const std::vector<const char*>& list,cocos2d::ui::Button* targetButton,int tag);
    
    //ListView의 가장 최상단에 붙어있는 노드를 구한다.
    cocos2d::Node* findMostTopNode(cocos2d::Node* node);
    
    cocos2d::ui::ListView* _myListView;
};

#endif /* BaseView_hpp */
