//
//  ExternalAuth.hpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by sjyong on 2022/02/07.
//

#ifndef ExternalAuth_hpp
#define ExternalAuth_hpp

#include <stdio.h>
#include <HIVE_SDK_Plugin/HIVE_CPP.h>
#include "network/HttpClient.h"
#include "picojson.h"
#include <map>
#include <sstream>

#endif /* ExternalAuth_hpp */

using namespace std;
using namespace hive;

class ExternalAuth {
public:

    static ExternalAuth* getInstance()
    {
        if (instance == NULL) instance = new ExternalAuth();
        return instance;
    }

    struct ExternalAuthResponse {
        /*
         200 : 성공
         502 : 필수 파라메타 없음
         1005 : 미지원 AUTHTYPE
         4200 : 등록된 IDP 정보가 없음
         1000516 : AuthV4 API 연결 실패
         */
        long resultCode = 0;
        string resultMsg = "";
        picojson::value resultData;
        string authKey = "";

        bool isSuccess() const {
            if (resultCode == 200) {
                return true;
            }
            return false;
        }

        string toString() const {
            stringstream ss;
            ss << "resultCode = " << resultCode << '\n';
            ss << "resultMsg = " << resultMsg << '\n';
            ss << "resultData = " << resultData.serialize() << '\n';
            ss << "authKey = " << authKey << '\n';
            return ss.str();
        }
    };

    enum AuthType {
        SIGNINIDP = 0,
        CONNECT = 1,
    };

    void requestIdp(ZoneType zoneType, string appId, AuthType authType, string idpId, string idpUserId, function<void(const ExternalAuthResponse* response)> callback);

private:

    static ExternalAuth* instance;

    string authTypeToString(AuthType type);

    map<ZoneType, std::string> signInURL = {
        {ZoneType::TEST, "http://test-misample.com2us.net/account/custom_auth"},
        {ZoneType::SANDBOX, "http://sandbox-misample.com2us.net/account/custom_auth"},
        {ZoneType::REAL, "http://misample.com2us.net/account/custom_auth"},
    };

    void requestAPI(const char *url,const char *data,const cocos2d::network::ccHttpRequestCallback callback);
};

