//
//  Postbox.cpp
//  HIVE_SDK_v4_Cocos_Sample
//
//  Created by paikjongman on 2017. 3. 3..
//  Copyright © 2017년 ryuvsken. All rights reserved.
//

#include "Postbox.hpp"
#include "ParameterDataManager.hpp"
#include "cocos2d.h"


using namespace cocos2d;
using namespace cocos2d::network;
using namespace hive;

Postbox::LoginType Postbox::currentLoginType = Postbox::v1;
std::string Postbox::cache_session_token = "";

static const char *PostboxReal	= "http://misample.com2us.net/";
static const char *PostboxTest	= "http://test-misample.com2us.net/";
static const char *PostboxSandbox	= "http://sandbox-misample.com2us.net/";
static const char *PostboxDev    = "http://dev-misample.com2us.net/";

static const char *ApiLogin = "account/login";
static const char *ApiUserInfo = "account/user";
static const char *ApiPosts = "postbox/posts";
static const char *ApiReceive = "postbox/receive";
static const char *ApiVerify = "hive/iap/verify";

std::string ItemTypeToString(Postbox::ItemType itemType) {
    switch (itemType) {
        case Postbox::ItemType::consumable:
            return "consumable";
            break;
        default:
            return "subscription";
            break;
    }
}

std::string Postbox::getVidColumn(){
    std::string data;

    switch (currentLoginType) {
        case v0:
        {
            /* 개별모듈에는 VID 개념이 없고 UID만 존재하기 때문에 VID 필드에 UID 값으로 세팅 후 로그인 시도 해도 됨
             * 실제 우편함 서버에서 로그인시 인증서버 유저 테이블을 검색해서 유효한 값인지 체크하지 않음
             */
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();
            // 정식유저가 아니라면 게스트 유저로 판단 후 로그인
            if (uid.length() <= 0) {
                uid = ParameterDataManager::getInstance()->peppermintParams->getGuestUid();
            }
            data += "\"vid\":\"" + uid +"\",";
            break;
        }
        case v1:
        {
            auto account =  hive::Auth::getAccount();
            data += "\"vid\":\"" + account.vid +"\",";
            break;
        }
        case v4:
        {
            auto playerInfo = hive::AuthV4::getPlayerInfo();
            data += "\"vid\":\"" + cocos2d::StringUtils::format("%lld",playerInfo.playerId) +"\",";
            break;
        }
    }
    return data;
}

std::string Postbox::getWorldColumn(ColomnState state)
{
    std::string data;

    switch (currentLoginType) {
        case v0:
        {
            data += "\"world\":\"" + ParameterDataManager::getInstance()->activeUserParams->getServerId() + "\"";
            switch (state) {
                case CONTINUE:
                    data += ",";
                    break;
                case END:
                    break;
            }
            break;
        }
        case v1:
        case v4:
        {
            data += "\"world\":\"" + hive::Configuration::getServerId() + "\"";
            switch (state) {
                case CONTINUE:
                    data += ",";
                    break;
                case END:
                    break;
            }
            break;
        }
    }
    return data;
}

void Postbox::login(std::function<void(const Login *data,Error error)> callback)
{
    std::string data = "{";
    data += getVidColumn();
    data += getWorldColumn(CONTINUE);

    switch (currentLoginType) {
        case v0:
        {
            auto dataManager = ParameterDataManager::getInstance();

            data += "\"appid\":\"" + dataManager->peppermintParams->getAppId() + "\",";
            data += "\"language\":\"" + dataManager->activeUserParams->getGameLanguage() + "\",";
            data += "\"country\":\"" + dataManager->activeUserParams->getGameLanguage() + "\",";
            data += "\"appver\":\"" + hive::Configuration::getHiveSDKVersion() + "\",";

            data += "\"did\":\"" + dataManager->commonParams->getDid() +"\",";
            data += "\"vid_type\":\"v0\",";
            data += "\"lc_sessionkey\":\"" + dataManager->commonParams->getSessionKey() + "\"";
            break;
        }
        case v1:
        {
            data += "\"appid\":\"" + hive::Configuration::getAppId()+"\",";

            picojson::value v;
            picojson::parse(v,hive::Configuration::getConfiguration());

            data += "\"language\":\"" + v.get("gameLanguage").get<std::string>()+"\",";
            data += "\"country\":\""+v.get("gameLanguage").get<std::string>()+"\",";
            data += "\"appver\":\""+v.get("hiveSDKVersion").get<std::string>()+"\",";

            auto account =  hive::Auth::getAccount();
            data += "\"did\":\"" + account.did +"\",";
            data += "\"vid_type\":\"v1\",";
            data += "\"lc_sessionkey\":\"" + account.accessToken +"\"";
            if(account.uid.size() > 0) {
                data += ",\"uid\":\""+account.uid+"\"";
            }
            break;
        }
        case v4:
        {
            data += "\"appid\":\"" + hive::Configuration::getAppId()+"\",";

            picojson::value v;
            picojson::parse(v,hive::Configuration::getConfiguration());

            data += "\"language\":\"" + v.get("gameLanguage").get<std::string>()+"\",";
            data += "\"country\":\""+v.get("gameLanguage").get<std::string>()+"\",";
            data += "\"appver\":\""+v.get("hiveSDKVersion").get<std::string>()+"\",";

            auto playerInfo = hive::AuthV4::getPlayerInfo();
            data += "\"did\":\"" + playerInfo.did +"\",";
            data += "\"vid_type\":\"v4\",";
            data += "\"lc_sessionkey\":\"" + playerInfo.playerToken +"\"";
            break;
        }
        default:
            break;
    }
    
    
    data += "}";
    hive::Logger::log("request post box json : %s\n",data.c_str());
    request(getUrl(ApiLogin).c_str(), data.c_str(), [=](HttpClient* client, HttpResponse* response){
        Error err;
        picojson::value v;
        if(!parseAndErrorCheck(response,v,err))
        {
            callback(nullptr,err);
        }
        else
        {
            //로그인 정보 얻는다.
            auto ret = new Login();
            picojson::value info = v.get("result_data");
            ret->nickname = info.get("nickname").get<std::string>();
            ret->gold = static_cast<int>(info.get("gold").get<double>());
            ret->gem = static_cast<int>(info.get("gem").get<double>());
            ret->session_token = v.get("session_token").get<std::string>();
            cache_session_token = ret->session_token;
            callback(ret,err);
            delete ret;
        }
    });
}

void Postbox::userInfo(std::function<void(const UserInfo *data,Error error)> callback)
{
    std::string data = "{";
    data += getVidColumn();
    data += getWorldColumn(END);
    data += "}";
    
    request(getUrl(ApiUserInfo).c_str(), data.c_str(), [=](HttpClient* client, HttpResponse* response){
        Error err;
        picojson::value v;
        if(!parseAndErrorCheck(response,v,err))
        {
            callback(nullptr,err);
        }
        else
        {
            //유저 정보 얻는다.
            auto ret = new UserInfo();
            picojson::value info = v.get("result_data");
            ret->nickname = info.get("nickname").get<std::string>();
            ret->gold = static_cast<int>(info.get("gold").get<double>());
            ret->gem = static_cast<int>(info.get("gem").get<double>());
            callback(ret,err);
            delete ret;
        }
    });
}

void Postbox::posts(std::function<void(const Posts *data,Error error)> callback)
{
    std::string data = "{";
    data += "\"session_token\":\"" + cache_session_token +"\",";
    data += getVidColumn();
    data += getWorldColumn(END);

    data += "}";
    
    hive::Logger::log("posts : %s",data.c_str());
    
    request(getUrl(ApiPosts).c_str(), data.c_str(), [=](HttpClient* client, HttpResponse* response){
        Error err;
        picojson::value v;
        if(!parseAndErrorCheck(response,v,err))
        {
            callback(nullptr,err);
        }
        else
        {
            //포스트 데이터 얻는다.
            auto ret = new Posts();
            picojson::value result_data = v.get("result_data");
            ret->post_list = getPostData(result_data);
            ret->subscription_list = getSubscriptionData(result_data);
            ret->session_token = v.get("session_token").get<std::string>();
            callback(ret,err);
            delete ret;
        }

    });
}

void Postbox::postReceive(std::vector<const char*> postids,std::function<void(const PostsReceive *data,Error error)> callback)
{
    std::string data = "{";
    data += getWorldColumn(CONTINUE);
    data += getVidColumn();
    data += "\"session_token\":\"" + cache_session_token +"\",";
    data += "\"post_id_list\":[";
    //받을 아이템 json데이터에 array 형식으로 추가
    for(auto it = postids.cbegin();it != postids.cend();++it)
    {
        data += StringUtils::format("\"%s\"",*it);
        if((it+1) != postids.cend())
            data += ",";
    }
    data += "]";
    data += "}";
    
    request(getUrl(ApiReceive).c_str(), data.c_str(), [=](HttpClient* client, HttpResponse* response){
        Error err;
        picojson::value v;
        if(!parseAndErrorCheck(response,v,err))
        {
            callback(nullptr,err);
        }
        else
        {
            //데이터 파싱해서 리시브데이터 얻는다.
            auto ret = new PostsReceive();
            picojson::value info = v.get("result_data");
            if (!info.get("nickname").is<picojson::null>()) {
                ret->nickname = info.get("nickname").get<std::string>();
            }
            ret->gold = static_cast<int>(info.get("gold").get<double>());
            ret->gem = static_cast<int>(info.get("gem").get<double>());
            ret->post_list = getPostData(info);
            ret->session_token = v.get("session_token").get<std::string>();
            callback(ret,err);
            delete ret;
        }
    });
}

void Postbox::verifyReceipt(std::string bypassinfo,ItemType itemType,std::function<void(const VerifyReceipt *data, Error error)> callback) {
    std::string data = "{";
    data += getWorldColumn(CONTINUE);
    data += getVidColumn();
    data += "\"session_token\":\"" + cache_session_token +"\",";
    data += "\"item_type\":\"" + ItemTypeToString(itemType)  +"\",";
    data += "\"purchase_bypass_info\":\"" + bypassinfo +"\"";
    data += "}";
    
    CCLOG("%s",data.c_str());
    request(getUrl(ApiVerify).c_str(), data.c_str(), [=](HttpClient* client, HttpResponse* response){
        Error err;
        picojson::value v;
        if(!parseAndErrorCheck(response,v,err))
        {
            callback(nullptr,err);
        }
        else
        {
            auto ret = new VerifyReceipt();
            picojson::value info = v.get("result_data");
            ret->product_id = info.get("product_id").get<std::string>();
            callback(ret,err);
            delete ret;
        }
    });
}

void Postbox::request(const char *url,const char *data,const ccHttpRequestCallback callback)
{
    auto network = HttpClient::getInstance();
    auto *req = new HttpRequest();
    req->setUrl(url);
    req->setRequestType(HttpRequest::Type::POST);
    req->setHeaders({"Content-Type:application/json",
                    StringUtils::format("Content-Length:%zu",strlen(data))});
    req->setRequestData(data, strlen(data));

    req->setResponseCallback(callback);
    
    network->send(req);
    req->release();
}

std::string Postbox::getUrl(const char *api)
{
    switch (currentLoginType) {
        case v0:
        {
            switch (ParameterDataManager::getInstance()->activeUserParams->getServerState()) {
                case SANDBOX_SERVER:
                    return PostboxSandbox+std::string(api);
                case STAGING_SERVER:
                    return PostboxTest+std::string(api);
                case LIVE_SERVER:
                    return PostboxReal+std::string(api);
            }
        }
        case v1:
        case v4:
        {
            switch(hive::Configuration::getZone())
            {
                case ZoneType::SANDBOX:
                    return PostboxSandbox+std::string(api);
                case ZoneType::TEST:
                    return PostboxTest+std::string(api);
                case ZoneType::REAL:
                    return PostboxReal+std::string(api);
                case ZoneType::DEV:
                    return PostboxDev+std::string(api);
            }
        }
    }

    return "";
}

bool Postbox::parseAndErrorCheck(cocos2d::network::HttpResponse *response,picojson::value &v,Error &err)
{
    if(response->getResponseCode() != 200)
    {
        err.result_code = StringUtils::format("%ld",response->getResponseCode());
        err.result_msg = StringUtils::format("network error (%ld)",response->getResponseCode());
        return false;
    }
    else
    {
        auto retdata = std::string(response->getResponseData()->begin(),response->getResponseData()->end());
        std::string e = picojson::parse(v,retdata);
        
        printf("%s\n",retdata.c_str());
        if(!e.empty())
        {
            err.result_code = "0";
            err.result_msg = StringUtils::format("json parse error \n(%s)",e.c_str());
            return false;
        }
        
        err.result_code = v.get("result_code").get<std::string>();
        if(!v.get("result_msg").is<picojson::null>())
            err.result_msg = v.get("result_msg").get<std::string>();
        else
            err.result_msg = "";
        if(err.result_code != "200")
            return false;
    }
    
    return true;
}
Postbox::PostList Postbox::getPostData(picojson::value v)
{
    PostList posts;
    auto post_list = v.get("post_list").get<picojson::array>();
    for(auto it = post_list.cbegin();it != post_list.cend();++it)
    {
        Post p;
        p.post_id = (*it).get("post_id").get<std::string>();
        p.asset_code = (*it).get("asset_code").get<std::string>();
        p.asset_amount = static_cast<int>((*it).get("asset_amount").get<double>());
        if((*it).get("character_key").is<picojson::null>() == false)
            p.character_key = (*it).get("character_key").get<std::string>();
        else
            p.character_key = "";
        p.user_msg = (*it).get("user_msg").get<std::string>();
        p.date_added = (*it).get("date_added").get<std::string>();
        posts.push_back(p);
    }
    return posts;
}

Postbox::SubscriptionList Postbox::getSubscriptionData(picojson::value v)
{
    SubscriptionList subscriptions;
    auto subscription_list = v.get("post_subscription_list").get<picojson::array>();
    for(auto it = subscription_list.cbegin();it != subscription_list.cend();++it)
    {
        Subscription s;
        s.product_id = (*it).get("product_id").get<std::string>();
        s.market_id = (*it).get("market_id").get<std::string>();
        s.start_date = (*it).get("start_date").get<std::string>();
        s.start_date_ms = (*it).get("start_date_ms").get<std::string>();
        s.expire_date = (*it).get("expire_date").get<std::string>();
        s.expire_date_ms = (*it).get("expire_date_ms").get<std::string>();
        if((*it).get("character_key").is<picojson::null>() == false)
            s.character_key = (*it).get("character_key").get<std::string>();
        else
            s.character_key = "";
        subscriptions.push_back(s);
    }
    return subscriptions;
}

void Postbox::setLoginType(Postbox::LoginType type){
    currentLoginType = type;
}
