//
//  PromotionBannerView.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by sjyong on 16/09/2019.
//

#include "PromotionBannerView.hpp"
#include "cocos2d.h"
#include <string>

#define BUTTON_WIDTH 500
#define BUTTON_HEIGHT 200

#define BUTTON_SPRITE_WIDTH 200
#define BUTTON_SPRITE_HEIGHT 200

bool PromotionBannerView::init() {
    if (!BaseView::init())
        return false;

    return true;
}

bool PromotionBannerView::initialize(vector<PromotionBannerInfo> const bannerInfoList) {
    if (!PromotionBannerView::init())
        return false;

    // 클래스 변수 bannerInfos 초기화 및 복사
    bannerInfos.resize(bannerInfoList.size());
    std::copy(bannerInfoList.begin(), bannerInfoList.end(), bannerInfos.begin());

    for(int i=0; i<bannerInfos.size(); i++) {

        PromotionBannerView::loadImage(bannerInfos[i].imageUrl, i);
        addButton("", cocos2d::Size(BUTTON_WIDTH, BUTTON_HEIGHT), i);

        ui::Button* button = (ui::Button*)BaseView::getUIWidget(i);

        stringstream ss;
        ss << "pid: " << bannerInfos[i].pid << '\n';
        ss << "displayStartDate: "  << bannerInfos[i].displayStartDate << '\n';
        ss << "displayEndDate: "  << bannerInfos[i].displayEndDate << '\n';

        auto label = Label::createWithSystemFont(ss.str().c_str(), "Ariel", 20);
        label->setPosition(BUTTON_SPRITE_WIDTH + 10, (BUTTON_SPRITE_HEIGHT/2));
        label->setAnchorPoint(Point(0.0f,0.5f));
        button->addChild(label);
    }

    return true;
}

PromotionBannerView* PromotionBannerView::create(std::vector<PromotionBannerInfo> const bannerInfoList) {
    PromotionBannerView* ret = new(std::nothrow) PromotionBannerView();
    if (ret && ret-> initialize(bannerInfoList)) {
        ret->autorelease();
        return ret;
    } else {
        delete ret;
        ret = nullptr;
        return nullptr;
    }
}

void PromotionBannerView::onButtonEvent(int buttonTag) {
    stringstream contentsKey;
    contentsKey << buttonTag;
    PromotionBannerInfo bannerInfo = bannerInfos[buttonTag];
    std::string pidToString = std::to_string(bannerInfo.pid);
    std::string interworkData = bannerInfo.interworkData;

    if (!interworkData.empty() && interworkData.compare("null") != 0) {
        // 특정 게임의 Scene으로 이동
        Logger::log("intworkData exist. will not show CustomWebView.");
        return;
    }

    Promotion::showCustomContents(PromotionCustomType::DIRECT, pidToString, [=](ResultAPI result, PromotionEventType eventType){
        Logger::log("===Promotion::showCustumContents() called===");
        Logger::log("contentsKey = %d\nresult = %s\n", buttonTag, result.toString().c_str());
    });
}

void PromotionBannerView::loadImage(std::string imageUrl, int index)
{
    HttpRequest* request = new (std::nothrow) HttpRequest();
    request->setUrl(imageUrl);
    request->setRequestType(HttpRequest::Type::GET);
    request->setResponseCallback(CC_CALLBACK_2(PromotionBannerView::onRequestImageFromURLcompleted, this));
    stringstream tag_ss;
    tag_ss << index;
    request->setTag(tag_ss.str());
    HttpClient::getInstance()->send(request);
    request->release();
}

void PromotionBannerView::onRequestImageFromURLcompleted(HttpClient* sender, HttpResponse* response)
{
    if (!response)
    {
        Logger::log("onRequestImageFromURLcompleted - No response");
        return;
    }

    //Logger::log("onRequestImageFromURLcompleted - Response code: %lu", response->getResponseCode());

    if (!response->isSucceed())
    {
        Logger::log("onRequestImageFromURLcompleted - Response failed");
        Logger::log("onRequestImageFromURLcompleted - Error buffer: %s", response->getErrorBuffer());
        return;
    }
    //Logger::log("onRequestImageFromURLcompleted - Response code: %s", response->getResponseDataString());

    vector<char> *buffer = response->getResponseData();
    const char* file_char = buffer->data();
    Image* image = new Image();
    image->initWithImageData(reinterpret_cast<const unsigned char*>(&(buffer->front())), buffer->size());

    string requestTag = response->getHttpRequest()->getTag();
    int requestTagInt = atoi(requestTag.c_str());

    ui::Button* button = (ui::Button*)BaseView::getUIWidget(requestTagInt);

    //try-catch for gif image.
    try {
        Texture2D* texture = new Texture2D();
        texture->initWithImage(image);

        Sprite* sprite = Sprite::createWithTexture(texture);
        sprite->setAnchorPoint(Vec2(0, 0));
        sprite->setPosition(Vec2(0,0));
        auto size = sprite->getBoundingBox().size;
        sprite->setScaleX(BUTTON_SPRITE_WIDTH / size.width);
        sprite->setScaleY(BUTTON_SPRITE_HEIGHT / size.height);
        sprite->visit();

        button->addChild(sprite);
        
        if (requestTagInt == bannerInfos.size() - 1) {
            setUseBackButton(true, "PromotionBannerView");
        }

    } catch(exception e) {
        stringstream ss;
        ss << button->getTitleText() << "\nimage format error. check the log.";
        button->setTitleText(ss.str());

        string defaultImage = "Icon-152.png";
        button->loadTextures(defaultImage, defaultImage);

        Logger::log("Image format invalid. pid = %s, imageURL = %s", requestTag.c_str(), response->getHttpRequest()->getUrl());
    }
}
