/**
 * ProviderTest.cpp
 * HIVE_SDK_Cocos_Sample
 * 
 * Created by disker on 2019. 10. 15.
 * Copyright © 2019년 GCP. All rights reserved.
*/

#include <string>
#include <iostream>
#include "ProviderTest.hpp"

using namespace hive;
using namespace cocos2d;
using namespace cocos2d::ui;

bool ProviderTest::init()
{
    if (!BaseView::init())
        return false;

    setUseBackButton(true, "- ProviderTest -");
    setUseShowLogButton(true);

    Logger::setLoggerListener([=](std::string tag, std::string level, std::string logMessage) {
        addLogMessage(logMessage);
    });

#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    BeginHorizontal();
    addEditBox("achievementId : ", "CgkIzev2g9MJEAIQAQ", "", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, DEFAULT_FONT_SIZE, EDITBOX_ACHIEVEMENT_ID_REVEAL);
    addButton("ProviderGoogle.achievementsReveal", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_ACHIEVEMENTS_REVEAL);
    EndHorizontal();

    BeginHorizontal();
    addEditBox("achievementId : ", "CgkIzev2g9MJEAIQAw", "", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, DEFAULT_FONT_SIZE, EDITBOX_ACHIEVEMENT_ID_UNLOCK);
    addButton("ProviderGoogle.achievementsUnlock", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_ACHIEVEMENTS_UNLOCK);
    EndHorizontal();

    BeginHorizontal();
    addEditBox("achievementId : ", "CgkIzev2g9MJEAIQAg", "", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, DEFAULT_FONT_SIZE, EDITBOX_ACHIEVEMENT_ID_INCREMENT);
    addEditBox("value : ", "10", "", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, DEFAULT_FONT_SIZE, EDITBOX_VALUE);
    EndHorizontal();

    addButton("ProviderGoogle.achievementsIncrement", BUTTON_ACHIEVEMENTS_INCREMENT);
    addButton("ProviderGoogle.showAchievements", BUTTON_SHOW_ACHIEVEMENTS);

    BeginHorizontal();
    addEditBox("leaderboardId : ", "CgkIzev2g9MJEAIQBw", "", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, DEFAULT_FONT_SIZE, EDITBOX_LEADERBOARD_ID);
    addEditBox("score : ", "10", "", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, DEFAULT_FONT_SIZE, EDITBOX_SCORE);
    EndHorizontal();

    addButton("ProviderGoogle.leaderboardsSubmitScore", BUTTON_LEADERBOARDS_SUBMIT_SCORE);
    addButton("ProviderGoogle.showLeaderboards", BUTTON_SHOW_LEADERBOARDS);
    addButton("Provider.getGooglePlayerId", BUTTON_GET_GOOGLE_PLAYER_ID);

#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    addEditBox("score : ", "11", "", EDITBOX_SCORE);
    addEditBox("leaderboardIdentifier : ", "grp.com.com2us.hivesdk.rank01", "", EDITBOX_LEADERBOARD_IDENTIFIER);

    addButton("ProviderApple.reportScore", BUTTON_REPORT_SCORE);

    addButton("ProviderApple.showLeaderboard", BUTTON_SHOW_LEADERBOARD);
    addButton("ProviderApple.loadAchievements", BUTTON_LOAD_ACHIEVEMENTS);

    BeginHorizontal();
    addEditBox("percent : ", "70", "", {screenSize.width/3, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, DEFAULT_FONT_SIZE, EDITBOX_PERCENT);
    addTrueFalseToggleButton("ShowsCompletionBanner : ", 1, {screenSize.width*2/3, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_SHOWS_COMPLETION_BANNER);
    EndHorizontal();

    addEditBox("achievementIdentifier : ", "grp.com.com2us.hivesdk.10hit", "", EDITBOX_ACHIEVEMENT_IDENTIFIER);

    addButton("ProviderApple.reportAchievement", BUTTON_REPORT_ACHIEVEMENT);
    addButton("ProviderApple.showAchievements", BUTTON_SHOW_ACHIEVEMENTS);
    addButton("ProviderApple.resetAchievements", BUTTON_RESET_ACHIEVEMENTS);

#endif



    return true;
}

void ProviderTest::onButtonEvent(int buttonTag)
{
    switch (buttonTag)
    {
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
        case BUTTON_ACHIEVEMENTS_REVEAL:
        {
            std::string achievementId = getEditboxValue(EDITBOX_ACHIEVEMENT_ID_REVEAL);

            ProviderGoogle::achievementsReveal(achievementId, [=](ResultAPI const & result){
                Logger::log("=== ProviderGoogle::achiementsReveal Called ===");
                Logger::log("result code:%d message:%s\n", result.code, result.message.c_str());
            });
            break;
        }

        case BUTTON_ACHIEVEMENTS_UNLOCK:
        {
            std::string achievementId = getEditboxValue(EDITBOX_ACHIEVEMENT_ID_UNLOCK);

            ProviderGoogle::achievementsUnlock(achievementId, [=](ResultAPI const & result){
                Logger::log("=== ProviderGoogle::achievementsUnlock Called ===");
                Logger::log("result code:%d message:%s\n", result.code, result.message.c_str());
            });
            break;
        }

        case BUTTON_ACHIEVEMENTS_INCREMENT:
        {
            std::string achievementId = getEditboxValue(EDITBOX_ACHIEVEMENT_ID_INCREMENT);
            int value = stoi(getEditboxValue(EDITBOX_VALUE));

            ProviderGoogle::achievementsIncrement(achievementId, value, [=](ResultAPI const & result){
                Logger::log("=== ProviderGoogle::achiementsIncrement Called ===");
                Logger::log("result code:%d message:%s\n", result.code, result.message.c_str());
            });
            break;
        }

        case BUTTON_SHOW_ACHIEVEMENTS:
        {
            ProviderGoogle::showAchievements([=](ResultAPI const & result){
                Logger::log("=== ProviderGoogle::showAchievements Called ===");
                Logger::log("result code:%d message:%s\n", result.code, result.message.c_str());
            });
            break;
        }

        case BUTTON_LEADERBOARDS_SUBMIT_SCORE:
        {
            std::string leaderboardId = getEditboxValue(EDITBOX_LEADERBOARD_ID);
            long long score = stoll(getEditboxValue(EDITBOX_SCORE));

            ProviderGoogle::leaderboardsSubmitScore(leaderboardId, score, [=](ResultAPI const & result){
                Logger::log("=== ProviderGoogle::showAchievements Called ===");
                Logger::log("result code:%d message:%s\n", result.code, result.message.c_str());
            });
            break;
        }

        case BUTTON_SHOW_LEADERBOARDS:
        {
            ProviderGoogle::showLeaderboard([=](ResultAPI const & result){
                Logger::log("=== ProviderGoogle::showLeaderboards ===");
                Logger::log("result code:%d message:%s\n", result.code, result.message.c_str());
            });
            break;
        }

        case BUTTON_GET_GOOGLE_PLAYER_ID:
        {
            ProviderGoogle::getGooglePlayerId([=](ResultAPI const &result, std::string const &googlePlayerId, std::string const &authCode) {
                Logger::log("=== ProviderGoogle::getGooglePlayerId Called ===");

                std::stringstream log_ss;
                log_ss << "result code:" << result.code << " message:" << result.message << '\n';
                log_ss << "googlePlayerId:" << googlePlayerId << '\n';
                log_ss << "authCode:" << authCode << '\n';
                Logger::log(log_ss.str().c_str());
            });
            break;
        }
#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
        case BUTTON_REPORT_SCORE:
        {
            std::string score = getEditboxValue(EDITBOX_SCORE);
            std::string leaderboardIdentifier = getEditboxValue(EDITBOX_LEADERBOARD_IDENTIFIER);

            ProviderApple::reportScore(score, leaderboardIdentifier, [=](ResultAPI const & result){
                Logger::log("=== ProviderApple::repoertScore Called ===");
                Logger::log("result code:%d message:%s\n", result.code, result.message.c_str());
            });
            break;
        }
        case BUTTON_SHOW_LEADERBOARD:
        {
            ProviderApple::showLeaderboard([=](ResultAPI const & result){
                Logger::log("=== ProviderApple::showLeaderboard Called ===");
                Logger::log("result code:%d message:%s\n", result.code, result.message.c_str());
            });
            break;
        }
        case BUTTON_LOAD_ACHIEVEMENTS:
        {
            ProviderApple::loadAchievements([=](ResultAPI const & result, std::vector<ProviderAppleAchievement> const & achievements){
                Logger::log("=== ProviderApple::loadAchievements Called ===");

                std::stringstream log_ss;
                log_ss << "result code:" << result.code << "message:" << result.message << '\n';

                for (auto achievement : achievements) {
                    log_ss << "[" << achievement.identifier << ", " << achievement.percent << achievement.completed << "is Completed." << "]\n";
                }
                Logger::log(log_ss.str().c_str());
            });
            break;
        }
        case BUTTON_REPORT_ACHIEVEMENT:
        {
            std::string percent = getEditboxValue(EDITBOX_PERCENT);
            bool showsCompletionBanner = getSelectedToggleButtonIndex(TOGGLE_BUTTON_SHOWS_COMPLETION_BANNER);
            std::string achievementIdentifier = getEditboxValue(EDITBOX_ACHIEVEMENT_IDENTIFIER);

            ProviderApple::reportAchievement(percent, showsCompletionBanner, achievementIdentifier, [=](ResultAPI const & result){
                Logger::log("=== ProviderApple::reportAchievment Called ===");
                Logger::log("result code:%d message:%s\n", result.code, result.message.c_str());
            });
            break;
        }
        case BUTTON_SHOW_ACHIEVEMENTS:
        {
            ProviderApple::showAchievements([=](ResultAPI const & result){
                Logger::log("=== ProviderApple::showAchievements Called ===");
                Logger::log("result code:%d message:%s\n", result.code, result.message.c_str());
            });
            break;
        }
        case BUTTON_RESET_ACHIEVEMENTS:
        {
            ProviderApple::resetAchievements([=](ResultAPI const & result){
                Logger::log("=== ProviderApple::resetAchievements Called ===");
                Logger::log("result code:%d message:%s\n", result.code, result.message.c_str());
            });
            break;
        }
#endif
        default:
            break;
    }
}
