//
//  PushTest.cpp
//  hivesdk
//
//  Created by paikjongman on 2017. 2. 28..
//
//

#include <string>

#include "PushTest.hpp"

using namespace hive;
using namespace cocos2d;
using namespace cocos2d::ui;

const std::vector<const char*> pushBucketTypeLists = {"Overwrite","Inboxing","Bigtext"};
// 로컬 푸시 스트레스 테스트. 등록 가능한 최대 개수
const int stressLocalPushCount = 64;
// 일정 시간 간격으로 스트레스 로컬 푸시를 수신 받도록 설정(5초)
const int stressLocalPushTimeInterval = 5;
// 일정 시간 간격으로 현재 시간을 담은 레이블을 갱신 (0.01초)
const int updateLocalPushLabelTimeInterval = 0.01;
// 대용량 푸시 noticeID를 담은 배열
std::vector<int> stressLocalPushNoticeIDs;


static std::vector<const char*> color_list = {
    "NONE",
    "BLACK r0,g0,b0",
    "RED r255,g0,b0",
    "GREEN r0,g255,b0",
    "BLUE r0,g0,b255",
    "WHITE r255,g255,b255",
};

//color list의 값 인덱스와 동기화 local push의 값으로 넘긴다.
static std::vector<const char*> colors = {
    "" //NONE
    ,"{\"r\":0,\"g\":0,\"b\":0}" //BLACK
    ,"{\"r\":255,\"g\":0,\"b\":0}" //RED
    ,"{\"r\":0,\"g\":255,\"b\":0}" //GREEN
    ,"{\"r\":0,\"g\":0,\"b\":255}" //BLUE
    ,"{\"r\":255,\"g\":255,\"b\":255}" //WHITE
};

bool PushTest::init()
{
    if(!BaseView::init())
        return false;
    setUseShowLogButton(true);

    for(int i=1; i<=stressLocalPushCount; i++) {
        stressLocalPushNoticeIDs.push_back(i);
    }

    Logger::setLoggerListener([=](std::string tag, std::string level, std::string logMessage){
        addLogMessage(logMessage);
    });
    addLabel("- Request Push Token -", LinearGravity::CENTER_HORIZONTAL, Node::INVALID_TAG);
    addButton("Push.requestPushPermission", BUTTON_REQUEST_PUSH_PERMISSION);

    addLabel("- Remote Push -", LinearGravity::CENTER_HORIZONTAL, Node::INVALID_TAG);
    addButton("Push.getRemotePush()", BUTTON_GET_REMOTE_PUSH);

    BeginHorizontal();
    addOnOffToggleButton("Notice : ", 1, {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_NOTICE);
    addOnOffToggleButton("Night : ", 1, {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_NIGHT);
    EndHorizontal();

    addButton("Push.setRemotePush()", BUTTON_SET_REMOTE_PUSH);

    addLabel("- Local Push -", LinearGravity::CENTER_HORIZONTAL, Node::INVALID_TAG);
    addEditBox("Title : ", "Local push title", EDITBOX_TITLE);
    addEditBox("Message : ", "Local push message", EDITBOX_MESSAGE);
    addEditBox("BigMessage : ", "Local push big-message", EDITBOX_BIG_MESSAGE);

    addCombobox("BucketType : ", std::move(pushBucketTypeLists), 0,  COMBOBOX_BUCKET_TYPE);
    BeginHorizontal();
    addEditBox("BucketSize : ", "1", "BucketSize", {screenSize.width/3, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, 20, EDITBOX_BUCKET_SIZE);
    addEditBox("Push NO : ", "0", "Number Only", {screenSize.width/3, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, 20, EDITBOX_PUSH_NO);
    addEditBox("After : ", "5", "Time ms", {screenSize.width/3, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, 20, EDITBOX_AFTER);
    EndHorizontal();
    
    addEditBox("Group Id : ", "", "Local push group id", EDITBOX_GROUP_ID);

    addCombobox("Select Color : ", color_list, 0, COMBOBOX_COLOR);
    addButton("Push.registerLocalPush()", BUTTON_REGISTER_LOCAL_PUSH);
    addButton("Push.unregisterLocalPush()", BUTTON_UNREGISTER_LOCAL_PUSH);
    addButton("Push.unregisterLocalPushes()", BUTTON_UNREGISTER_LOCAL_PUSHES);

    // 로컬푸시 스트레스 테스트 영역
    addLabel("- Local Push stress test -", LinearGravity::CENTER_HORIZONTAL, LABEL_STRESS_LOCAL_PUSH);
    addButton("Push.registerLocalPush()", BUTTON_STRESS_REGISTER_LOCAL_PUSH);
    addButton("Push.unregisterLocalPush(noticeID)", BUTTON_STRESS_UNREGISTER_LOCAL_PUSH);
    addButton("Push.unregisterLocalPushes(noticeIDs)", BUTTON_STRESS_UNREGISTER_LOCAL_PUSHES);
    addButton("Push.unregisterAllLocalPushes()", BUTTON_UNREGISTER_ALL_LOCAL_PUSHES);
    // 0.01초 마다 호출되는 스케줄러
    schedule(schedule_selector(PushTest::updateLabelLocalPushStressTest), updateLocalPushLabelTimeInterval);

    
    // #12076: Foreground Push UI
    addLabel("- Push Setting when activated -", LinearGravity::CENTER_HORIZONTAL, Node::INVALID_TAG);
    addButton("Push.getForegroundPush()", BUTTON_GET_FOREGROUND_PUSH);
    addOnOffToggleButton("Local : ", 1, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_LOCAL);
    addOnOffToggleButton("Remote : ", 1, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_REMOTE);
    addButton("Push.setForegroundPush()", BUTTON_SET_FOREGROUND_PUSH);

    
    Push::getRemotePush(std::bind(&PushTest::onRemotePush, this, std::placeholders::_1, std::placeholders::_2));
    Push::getForegroundPush(std::bind(&PushTest::onPushSetting, this, std::placeholders::_1, std::placeholders::_2));
    // #12076 END
    setUseBackButton(true, "- Push Notification - ");
    
    return true;
}

/*
 BUTTON_GET_REMOTE_PUSH,
 RADIO_NOTICE,
 RADIO_NIGHT,
 BUTTON_SET_REMOTE_PUSH,
 EDITBOX_TITLE,
 EDITBOX_MESSAGE,
 EDITBOX_PUSH_NO,
 EDITBOX_AFTER,
 BUTTON_REGISTER_LOCAL_PUSH,
 BUTTON_UNREGISTER_LOCAL_PUSH,
 BUTTON_GET_FOREGROUND_PUSH,
 RADIO_LOCAL,
 RADIO_REMOTE,
 BUTTON_SET_FOREGROUND_PUSH
 */

void PushTest::onButtonEvent(int buttonTag)
{
    switch (buttonTag) {
        case BUTTON_GET_REMOTE_PUSH:
            Logger::log("\n\n=== 리모트 푸시 정보 조회 ===\n");
            Logger::log("Push::getRemotePush(onRemotePush) Called\n");
            
            Push::getRemotePush(std::bind(&PushTest::onRemotePush, this, std::placeholders::_1, std::placeholders::_2));
            break;
        case BUTTON_SET_REMOTE_PUSH:
        {
            RemotePush remotePush;
            remotePush.isAgreeNotice = getSelectedToggleButtonIndex(TOGGLE_BUTTON_NOTICE);
            remotePush.isAgreeNight = getSelectedToggleButtonIndex(TOGGLE_BUTTON_NIGHT);
            
            Logger::log("\n\n=== 리모트 푸시 정보 설정 ===\n");
            Logger::log("Push::setRemotePush(remotePush = %s, onRemotePush) Called\n", remotePush.toString().c_str());
            
            Push::setRemotePush(remotePush,
                                std::bind(&PushTest::onRemotePush, this, std::placeholders::_1, std::placeholders::_2));
            break;
        }
        case BUTTON_REGISTER_LOCAL_PUSH:
        {
            std::string title = getEditboxValue(EDITBOX_TITLE);
            std::string message = getEditboxValue(EDITBOX_MESSAGE);
            std::string bigMessage = getEditboxValue(EDITBOX_BIG_MESSAGE);
            int bucketType = getSelectedComboboxIndex(COMBOBOX_BUCKET_TYPE);
            std::string stdBucketSize = getEditboxValue(EDITBOX_BUCKET_SIZE);
            int bucketSize = std::atoi(stdBucketSize.c_str());
            std::string strNoticeId = getEditboxValue(EDITBOX_PUSH_NO);
            int noticeId = std::atoi(strNoticeId.c_str());//std::stoi(strNoticeId);
            std::string strAfter = getEditboxValue(EDITBOX_AFTER);
            long after = std::atol(strAfter.c_str());//std::stol(strAfter);
            std::string groupId = getEditboxValue(EDITBOX_GROUP_ID);
            
            LocalPush localPush;
            localPush.noticeId = noticeId;
            localPush.title = title.size() > 0 ? title : "Local Push Title";
            localPush.msg = message.size() > 0 ? message : "Local Push Message";
            localPush.bigmsg = bigMessage.size() > 0 ? bigMessage : "Local Push Big Message";
            localPush.buckettype = bucketType;
            localPush.bucketsize = bucketSize > 0 ? bucketSize : 1;
            localPush.after = std::max<long>( after, 10);
            localPush.groupId = groupId;
            
            int select_color = getSelectedComboboxIndex(COMBOBOX_COLOR);
            if(select_color > 0 && select_color < colors.size()) {
                localPush.icon_color = colors[select_color];
            }
            
            Logger::log("\n\n=== 로컬 푸시 등록 ===\n");
            Logger::log("Push::registerLocalPush(localPush = %s, onLocalPush) Called\n", localPush.toString().c_str());
            
            Push::registerLocalPush(localPush, std::bind(&PushTest::onLocalPush, this, std::placeholders::_1, std::placeholders::_2));
            break;
        }
        case BUTTON_UNREGISTER_LOCAL_PUSH:
        {
            std::string strNoticeId = getEditboxValue(EDITBOX_PUSH_NO);
            int noticeId = std::atoi(strNoticeId.c_str());//std::stoi(strNoticeId);
            
            Logger::log("\n\n=== 로컬 푸시 해제 ===\n");
            Logger::log("Push::unregisterLocalPush(noticeId = %d) Called\n", noticeId);
            
            Push::unregisterLocalPush(noticeId);
            break;
        }
        case BUTTON_UNREGISTER_LOCAL_PUSHES:
        {
            std::string strNoticeId = getEditboxValue(EDITBOX_PUSH_NO);
            int noticeId = std::atoi(strNoticeId.c_str());//std::stoi(strNoticeId);

            std::vector<int> noticeIDs;
            noticeIDs.push_back(noticeId);

            Logger::log("\n\n=== 로컬 푸시 배열 해제 ===\n");
            Logger::log("Push::unregisterLocalPushes([noticeId = %d]) Called\n", noticeId);

            Push::unregisterLocalPushes(noticeIDs);
            break;
        }
        case BUTTON_STRESS_REGISTER_LOCAL_PUSH:
        {
            Logger::log("\n\n=== 대량 로컬 푸시 등록 ===\n");

            for (int i=1; i<=stressLocalPushCount; i++) {
                LocalPush localPush;
                localPush.noticeId = i;
                localPush.title = StringUtils::format("stress_title %d", i);
                localPush.msg = StringUtils::format("stress_message %d", i);
                localPush.after = 5*i;

                Push::registerLocalPush(localPush, std::bind(&PushTest::onLocalPush, this, std::placeholders::_1, std::placeholders::_2));
            }
            break;
        }
        case BUTTON_STRESS_UNREGISTER_LOCAL_PUSH:
        {
            Logger::log("\n\n=== 대량 로컬 푸시 해제 ===\n");

            for (int i=1; i<=stressLocalPushCount; i++) {
                Push::unregisterLocalPush(i);
            }
            break;
        }
        case BUTTON_STRESS_UNREGISTER_LOCAL_PUSHES:
        {
            Logger::log("\n\n=== 대량 로컬 푸시 배열 해제 ===\n");
            Push::unregisterLocalPushes(stressLocalPushNoticeIDs);
            break;
        }
        case BUTTON_UNREGISTER_ALL_LOCAL_PUSHES:
        {
            Logger::log("\n\n=== 모든 로컬 푸시 해제 ===\n");
            Logger::log("Push::unregisterAllLocalPushes() Called\n");

            Push::unregisterAllLocalPushes();
            break;
        }


            
        // #12076: Foreground Push Event Handling
        case BUTTON_GET_FOREGROUND_PUSH:
        {
            Logger::log("\n\n=== 앱 활성화 시 푸시 수신 여부 조회 ===\n");
            Logger::log("Push::getForegroundPush(onPushSetting) Called\n");
            
            Push::getForegroundPush(std::bind(&PushTest::onPushSetting, this, std::placeholders::_1, std::placeholders::_2));
            break;
        }
        
        case BUTTON_SET_FOREGROUND_PUSH:
        {
            Logger::log("\n\n=== 앱 활성화 시 푸시 수신 여부 설정 ===\n");
            
            bool useLocal = getSelectedToggleButtonIndex(TOGGLE_BUTTON_LOCAL);
            bool useRemote = getSelectedToggleButtonIndex(TOGGLE_BUTTON_REMOTE);
            
            PushSetting pushSetting(useRemote, useLocal);
            
            Push::setForegroundPush(pushSetting, std::bind(&PushTest::onPushSetting, this, std::placeholders::_1, std::placeholders::_2));
            Logger::log("Push::setForegroundPush(pushSetting = %s, onPushSetting) Called\n", pushSetting.toString().c_str());

            break;
        }
        case BUTTON_REQUEST_PUSH_PERMISSION:
        {
            Logger::log("\n\n=== Request Push Permission===\n");
            Push::requestPushPermission();
            break;
        }
        // #12076 END
        default:
            break;
    }
}

// Local Push 등록 결과 통지 리스너 - Push::registerLocalPush()
void PushTest::onLocalPush(ResultAPI result, LocalPush localPush)
{
    Logger::log("PushTestView::onLocalPush() Callback\nresult = %s\n", result.toString().c_str());
    
    Logger::log("localPush = %s\n", localPush.toString().c_str());
}


// Remote Push 정보 결과 통지 리스너 - Push::getRemotePush() / Push::setRemotePush()
void PushTest::onRemotePush(ResultAPI result, RemotePush remotePush)
{
    Logger::log("PushTestView::onRemotePush() Callback\nresult = %s\n", result.toString().c_str());
    
    Logger::log("remotePush = %s\n", remotePush.toString().c_str());
    
    setOnOffToggleButtonIndex(remotePush.isAgreeNotice ? 1 : 0, TOGGLE_BUTTON_NOTICE);
    setOnOffToggleButtonIndex(remotePush.isAgreeNight ? 1 : 0, TOGGLE_BUTTON_NIGHT);
}

void PushTest::onPushSetting(ResultAPI result, PushSetting pushSetting)
{
    Logger::log("PushTestView::onPushSetting() Callback\nresult = %s\n", result.toString().c_str());
    Logger::log("pushSetting = %s\n", pushSetting.toString().c_str());
    
    setOnOffToggleButtonIndex(pushSetting.useForegroundLocalPush ? 1 : 0, TOGGLE_BUTTON_LOCAL);
    setOnOffToggleButtonIndex(pushSetting.useForegroundRemotePush ? 1 : 0, TOGGLE_BUTTON_REMOTE);
}

void PushTest::updateLabelLocalPushStressTest(float dt) {
    std::string currentTime = getCurrentDateTime();
    std::string stressLocalPushLabel = StringUtils::format("- Local Push stress test (%s) -", currentTime.c_str());
    changeLabelText(stressLocalPushLabel, LABEL_STRESS_LOCAL_PUSH);
}
