//
//  LoggerUtils.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by paikjongman on 2017. 8. 31..
//  Copyright © 2017년 GCP. All rights reserved.
//

#include "cocos2d.h"
#include "ResultUtils.hpp"

USING_NS_HIVE;
USING_NS_CC;

std::string ResultUtils::getResultAPIString(hive::ResultAPI const & result) {
    
    std::string errorToStr = "";
    switch (result.errorCode) {
        case hive::ResultAPI::ErrorCode::SUCCESS:
            errorToStr = "SUCCESS";
            break;
        case hive::ResultAPI::ErrorCode::RESTORE_NOT_OWNED:
            errorToStr = "RESTORE_NOT_OWNED";
            break;
        case hive::ResultAPI::ErrorCode::ITEM_DELIVERY_DELAYED:
            errorToStr = "ITEM_DELIVERY_DELAYED";
            break;
        case hive::ResultAPI::ErrorCode::ITEM_PENDING:
            errorToStr = "ITEM_PENDING";
            break;
        case hive::ResultAPI::ErrorCode::AUTHORIZED:
            errorToStr = "AUTHORIZED";
            break;
        case hive::ResultAPI::ErrorCode::INVALID_PARAM:
            errorToStr = "INVALID_PARAM";
            break;
        case hive::ResultAPI::ErrorCode::NOT_SUPPORTED:
            errorToStr = "NOT_SUPPORTED";
            break;
        case hive::ResultAPI::ErrorCode::IN_PROGRESS:
            errorToStr = "IN_PROGRESS";
            break;
        case hive::ResultAPI::ErrorCode::TIMEOUT:
            errorToStr = "TIMEOUT";
            break;
        case hive::ResultAPI::ErrorCode::NETWORK:
            errorToStr = "NETWORK";
            break;
        case hive::ResultAPI::ErrorCode::CANCELED:
            errorToStr = "CANCELED";
            break;
        case hive::ResultAPI::ErrorCode::NEED_INITIALIZE:
            errorToStr = "NEED_INITIALIZE";
            break;
        case hive::ResultAPI::ErrorCode::RESPONSE_FAIL:
            errorToStr = "RESPONSE_FAIL";
            break;
        case hive::ResultAPI::ErrorCode::INVALID_SESSION:
            errorToStr = "INVALID_SESSION";
            break;
        case hive::ResultAPI::ErrorCode::NEED_RESTORE:
            errorToStr = "NEED_RESTORE";
            break;
        case hive::ResultAPI::ErrorCode::CONFLICT_PLAYER:
            errorToStr = "CONFLICT_PLAYER";
            break;
        case hive::ResultAPI::ErrorCode::BLACKLIST:
            errorToStr = "BLACKLIST";
            break;
        case hive::ResultAPI::ErrorCode::DEVELOPER_ERROR:
            errorToStr = "DEVELOPER_ERROR";
            break;
        case hive::ResultAPI::ErrorCode::DUPLICATED_PROMOTION_CODE:
            errorToStr = "DUPLICATED_PROMOTION_CODE";
            break;
        case hive::ResultAPI::ErrorCode::UNKNOWN:
            errorToStr = "UNKNOWN";
            break;
    }
    return cocos2d::StringUtils::format("%s(%d:%s)",result.errorMessage.c_str(),result.errorCode,errorToStr.c_str());
}

void ResultUtils::showResult(hive::ResultAPI const & result) {
    if(result.isSuccess()){
        std::string message = getResultAPIString(result);
        MessageBox(message.c_str(), "API Success");
    }else{
        errorCheckResultAPI(result);
    }
}

bool ResultUtils::errorCheckResultAPI(hive::ResultAPI const & result) {
    if(!result.isSuccess()) {
        std::string message = getResultAPIString(result);
        if (result.needExit()) {
            message += "\nresult.needExit() = true\n";
        }
        MessageBox(message.c_str(), "API Call Fail");
        return false;
    }
    return true;
}
void ResultUtils::logHead(const char* className,const char* funcName){
    Logger::log("=== %s::%s ===\n",className,funcName);
}
void ResultUtils::logHead(const char* className,const char* funcName,hive::ResultAPI const & result){
    logHead(className,funcName);
    Logger::log("result = %s\n",getResultAPIString(result).c_str());
}

void ResultUtils::logProviderInfo(const char* className,const char* funcName,hive::ProviderInfo const & providerInfo) {
    Logger::log("providerInfo = %s\n",getProviderInfoString(providerInfo).c_str());
}
void ResultUtils::logPlayerInfo(const char* className,const char* funcName,hive::PlayerInfo const & playerInfo) {
    Logger::log("playerInfo = %s\n",getPlayerInfoString(playerInfo).c_str());
}
void ResultUtils::logProfileInfo(const char* className,const char* funcName,hive::ProfileInfo const & profileInfo) {
    Logger::log("profileInfo = %s\n",getProfileInfoString(profileInfo).c_str());
}

std::string ResultUtils::getProviderInfoString(const hive::ProviderInfo& info){
    return cocos2d::StringUtils::format("[%s]\nProvider User Id : %s",ProviderInfo::stringProviderType(info.providerType).c_str(),info.providerUserId.c_str());
}

std::string ResultUtils::getPlayerInfoString(const hive::PlayerInfo& info) {
    std::string message = "PlayerToken : "+info.playerToken + "\ndid : "+info.did+"\n";
    message += getProfileInfoString(info);

    message += "ProviderInfoData : {\n";
    int index =0;
    for(auto it = info.providerInfoData.begin();it != info.providerInfoData.end();++it) {
        message += StringUtils::format("[Provider %d]\n",index);
        ProviderType type = (*it).first;
        message += "key (providerType) : "+ ProviderInfo::stringProviderType(type) + "\n";
        ProviderInfo providerInfo = (*it).second;
        message += "ProviderInfo.Provider Type : " + ProviderInfo::stringProviderType(providerInfo.providerType)+"\n";
        message += "ProviderInfo.Provider Name : " + providerInfo.providerName+"\n";
        message += "ProviderInfo.Provider User ID : " + providerInfo.providerUserId+"\n";
        message += "ProviderInfo.Provider Email : " + providerInfo.providerEmail+"\n";
        message += ",\n";
        index++;
    }
    message += "}\n";

    message += "CustomProviderInfoData : {\n";
    int cIndex = 0;
    for (auto it = info.customProviderInfoData.begin(); it != info.customProviderInfoData.end(); ++it) {
        message += StringUtils::format("[Provider %d]\n", cIndex);
        message += "key (providerName) : "+ (*it).first + "\n";
        ProviderInfo providerInfo = (*it).second;
        message += "ProviderInfo.Provider Type : " + ProviderInfo::stringProviderType(providerInfo.providerType)+"\n";
        message += "ProviderInfo.Provider Name : " + providerInfo.providerName+"\n";
        message += "ProviderInfo.Provider User ID : " + providerInfo.providerUserId+"\n";
        message += "ProviderInfo.Provider Email : " + providerInfo.providerEmail+"\n";
        message += ",\n";
        cIndex++;
    }
    message += "}\n";

    return message;
}
std::string ResultUtils::getProfileInfoString(const hive::ProfileInfo& info) {
    std::string message = cocos2d::StringUtils::format("PlayerID : %lld\n",info.playerId);
    message += "Player Name : " + info.playerName + "\n";
    message += "Player Image Url : " + info.playerImageUrl + "\n";
    return message;
}

std::string ResultUtils::getAppInvitationCampaignString(const hive::AppInvitationCampaign& info) {
    std::string message = cocos2d::StringUtils::format("title : %s\nimageUrl : %s\ndescription : %s\norder : %d\ncount : %d\nlimit : %d",info.title.c_str(),
                                                       info.imageUrl.c_str(),
                                                       info.description.c_str(),
                                                       info.order,
                                                       info.count,
                                                       info.limit);
    return message;
}
std::string ResultUtils::getAppInvitationCampaignStageString(const hive::AppInvitationCampaignStage& info) {
    std::string message = getAppInvitationCampaignString(info) + "\ngoalCount : %d\ngoalTotal : %d";
    message = cocos2d::StringUtils::format(message.c_str(),info.goalCount,info.goalTotal);
    return message;
}

