//
//  SocialFacebookTest.cpp
//  hivesdk
//
//  Created by paikjongman on 2017. 2. 28..
//
//

#include "SocialFacebookTest.hpp"

using namespace hive;

std::vector<const char*> fbuidList;

std::vector<std::string> fbuidListStr;

std::vector<ProfileFacebook> fprofileLists;



bool SocialFacebookTest::init()
{
    if(!BaseView::init())
        return false;
    setUseShowLogButton(true);
    Logger::setLoggerListener([=](std::string tag, std::string level, std::string logMessage){
        addLogMessage(logMessage);
    });
    
    addButton("SocialFacebook.getMyProfile", BUTTON_GET_MY_PROFILE);
    addButton("SocialFacebook.getFriends", BUTTON_GET_FRIENDS);
    
    addMultiSelectButton("facebook appscoped id",fbuidList, MUILTY_FRIENDS_FBAPPID);
    addEditBox("Message : ","Message",EDITBOX_FRIENDS_MESSAGE);
    addButton("SocialFacebook.sendMessage", BUTTON_SEND_MESSAGE);
    
    addEditBox("Invitation Message : ","Invitation Message", EDITBOX_INVITATION_MESSAGE);
    addButton("SocialFacebook.showInvitationDialog", BUTTON_SHOW_INVITATION_DIALOG);
    
    addEditBox("Post Message : ","공유하기","Message", EDITBOX_POST_MESSAGE);
    addEditBox("Post ImageURL : ","http://naver.com/","ImageURL", EDITBOX_POST_IMAGE_URL);
    addEditBox("Post ContentURL : ","http://naver.com/","ContentURL", EDITBOX_POST_CONTENT_URL);
    
    addButton("SocialFacebook.postFacebook", BUTTON_POST_FACEBOOK);
    
    setUseBackButton(true, "- Social Facebook -");
    
    return true;
}
/*
 BUTTON_GET_MY_PROFILE = 0,
 BUTTON_GET_FRIENDS,
 MUILTY_FRIENDS_FBAPPID,
 EDITBOX_FRIENDS_MESSAGE,
 BUTTON_SEND_MESSAGE,
 EDITBOX_INVITATION_MESSAGE,
 BUTTON_SHOW_INVITATION_DIALOG,
 EDITBOX_POST_MESSAGE,
 BUTTON_POST_FACEBOOK
*/

void SocialFacebookTest::onButtonEvent(int buttonTag)
{
    switch (buttonTag) {
        case BUTTON_GET_MY_PROFILE:
            Logger::log("\n\n=== Facebook 프로필 정보 조회 ===\n");
            Logger::log("SocialFacebook::getMyProfile(onProfileFacebook) Called\n");
            SocialFacebook::getMyProfile(std::bind(&SocialFacebookTest::onProfileFacebook, this, std::placeholders::_1, std::placeholders::_2));
            break;
        case BUTTON_GET_FRIENDS:
            Logger::log("\n\n=== Facebook 친구 목록 조회 ===\n");
            Logger::log("SocialFacebook::getFriends(onFriendsFacebook) Called\n");
            SocialFacebook::getFriends(std::bind(&SocialFacebookTest::onFriendsFacebook, this, std::placeholders::_1, std::placeholders::_2));
            break;
        case BUTTON_SEND_MESSAGE:
        {
            FacebookMessage messageContents;
            
            std::vector<std::string> recipients;
            std::vector<int> vidIndex = getMultiSelectIndexes(MUILTY_FRIENDS_FBAPPID);
            std::string message = getEditboxValue(EDITBOX_FRIENDS_MESSAGE);

            for (int idx : vidIndex )
            {
                recipients.push_back(fbuidListStr[idx]);
            }

            messageContents.recipients = recipients;
            messageContents.dialogTitle = "Dialog Title";
            messageContents.message = message.length() > 0 ? message : "message";
            messageContents.data = "hidden data";
            
            Logger::log("\n\n=== Facebook 인증 사용자의 친구에게 메시지를 전송 ===\n");
            Logger::log("SocialFacebook::sendMessageFacebook(messageContents, onSendMessageFacebook) Called\n");
            
            SocialFacebook::sendMessageFacebook(messageContents,
                                                std::bind(&SocialFacebookTest::onSendMessageFacebook, this, std::placeholders::_1));
            break;
        }
        case BUTTON_SHOW_INVITATION_DIALOG:
        {
            FacebookMessage messageContents;
            
            // FB 초대는 recipients 사용하지 않음.
//            std::vector<std::string> recipients;
//            recipients.push_back("fid1");
//            messageContents.recipients = recipients;
            
            std::string message = getEditboxValue(EDITBOX_FRIENDS_MESSAGE);
            
            messageContents.dialogTitle = "Dialog Title";
            messageContents.message = message.length() > 0 ? message : "message";
            messageContents.data = "hidden data";
            
            Logger::log("\n\n=== Facebook 인증 사용자의 친구에게 초대 요청을 위한 대화상자 호출. ===\n");
            Logger::log("SocialFacebook::showInvitationDialog(messageContents, onShowInvitationDialogFacebook) Called\n");
            
            SocialFacebook::showInvitationDialog(messageContents,
                                                 std::bind(&SocialFacebookTest::onShowInvitationDialogFacebook, this, std::placeholders::_1));
            break;
        }
        case BUTTON_POST_FACEBOOK:
        {
            std::string contentURL = getEditboxValue(EDITBOX_POST_CONTENT_URL);
			
            Logger::log("\n\n=== Facebook 게시글 등록. ===\n");
            Logger::log("SocialFacebook::postFacebookWithContentURL(contentURL, onPostFacebook) Called\n");
			
            SocialFacebook::postFacebookWithContentURL(contentURL,
                                         std::bind(&SocialFacebookTest::onPostFacebook, this, std::placeholders::_1));
            break;
        }
        default:
            break;
    }
}


// Facebook 프로필 정보 결과 통지 리스너 - SocialFacebook::getMyProfile()
void SocialFacebookTest::onProfileFacebook(ResultAPI result, std::vector<ProfileFacebook> profileList)
{
    Logger::log("SocialFacebookTestView::onProfileFacebook() Callback\nresult = %s\n", result.toString().c_str());
    
    Logger::log("SocialFacebook::ProfileFacebook List =\n");
    
    for (int i = 0; i < (int)profileList.size(); i++) {
        
        ProfileFacebook profile = profileList[i];
        
        Logger::log("%s\n", profile.toString().c_str());
    }
}


// Facebook 친구 정보 결과 통지 리스너 - SocialFacebook::getFriends()
void SocialFacebookTest::onFriendsFacebook(ResultAPI result, std::vector<ProfileFacebook> profileList)
{
    Logger::log("SocialFacebookTestView::onFriendsFacebook() Callback\nresult = %s\n", result.toString().c_str());
    
    Logger::log("SocialFacebook::ProfileFacebook List =\n");
    
    if(profileList.size() > 0)
    {
        fbuidList.clear();
        fbuidListStr.clear();
        fprofileLists.clear();
        fprofileLists.assign(profileList.begin(),profileList.end());
        
        for (int i = 0; i < (int)fprofileLists.size(); i++) {
            
            ProfileFacebook profile = fprofileLists[i];
            
            Logger::log("%s\n", profile.uid.c_str());
            
            fbuidListStr.push_back(fprofileLists[i].uid);
        }
        
        for (int i=0;i<fprofileLists.size();++i) {
            fbuidList.push_back(fbuidListStr[i].c_str());
        }
        
        cocos2d::Director::getInstance()->getScheduler()->performFunctionInCocosThread([=](){
            changeMultiSelectList(fbuidList, MUILTY_FRIENDS_FBAPPID);
        });
    }
}


// Facebook 인증 사용자의 친구 메시지 전송 결과 통지 리스너 - SocialFacebook::sendMessageFacebook()
void SocialFacebookTest::onSendMessageFacebook(ResultAPI result)
{
    Logger::log("SocialFacebookTestView::onSendMessageFacebook() Callback\nresult = %s\n", result.toString().c_str());
}


// Facebook 인증 사용자의 친구 요청 대화상자 호출 결과 통지 리스너 - SocialFacebook::showInvitationDialog()
void SocialFacebookTest::onShowInvitationDialogFacebook(ResultAPI result)
{
    Logger::log("SocialFacebookTestView::onShowInvitationDialogFacebook() Callback\nresult = %s\n", result.toString().c_str());
}


// Facebook 인증 사용자의 게시글 등록 요청 - SocialFacebook::postFacebook()
void SocialFacebookTest::onPostFacebook(ResultAPI result)
{
    Logger::log("SocialFacebookTestView::onPostFacebookCB() Callback\nresult = %s\n", result.toString().c_str());
}
