//
//  HivePromotionScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by sjyong on 2021/10/15.
//

#include "C2SModuleAPIList.hpp"
#include "HivePromotionScene.hpp"
#include "HivePromotionParamScene.hpp"
#include "ParameterDataManager.hpp"
#include "ui/CocosGUI.h"

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;

#include <map>
#include <sstream>

using namespace::std;

bool HivePromotionScene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- HivePromotion -");

    addButton("PromotionInitialize", C2SModuleAPIList::PROMOTION_INITIALIZE);
    addButton("PromotionShow", ButtonMode::NEED_PARAM, C2SModuleAPIList::PROMOTION_SHOW);
    addButton("PromotionShowCustom", ButtonMode::NEED_PARAM, C2SModuleAPIList::PROMOTION_SHOW_CUSTOM);
    addButton("GetOfferwallState", C2SModuleAPIList::PROMOTION_GET_OFFERWALL_STATE);
    addButton("ShowOfferWall", ButtonMode::NEED_PARAM, C2SModuleAPIList::PROMOTION_SHOW_OFFERWALL);
    addButton("ShowReview", ButtonMode::NEED_PARAM, C2SModuleAPIList::PROMOTION_SHOW_REVIEW);
    addButton("ShowNativeReview", ButtonMode::NEED_PARAM, C2SModuleAPIList::PROMOTION_SHOW_NATIVE_REVIEW);
    addButton("ShowExit", ButtonMode::NEED_PARAM, C2SModuleAPIList::PROMOTION_SHOW_EXIT);
    addButton("GetViewInfo", ButtonMode::NEED_PARAM, C2SModuleAPIList::PROMOTION_GET_VIEW_INFO);
    addButton("GetBadgeInfo", ButtonMode::NEED_PARAM, C2SModuleAPIList::PROMOTION_GET_BADGE_INFO);
    addButton("PromotionGetBannerInfo", ButtonMode::NEED_PARAM, C2SModuleAPIList::PROMOTION_GET_BANNER_INFO);
    addButton("PromotionGetAppInvitationData", ButtonMode::NEED_PARAM, C2SModuleAPIList::PROMOTION_GET_APP_INVITATION_DATA);
    addButton("PromotionShowUAShare", ButtonMode::NEED_PARAM, C2SModuleAPIList::PROMOTION_SHOW_UA_SHARE);
    addButton("SetAdditionalInfo", ButtonMode::NEED_PARAM, C2SModuleAPIList::PROMOTION_SET_ADDITIONAL_INFO);
    addButton("UpdatePromotionData", C2SModuleAPIList::PROMOTION_UPDATE_PROMOTION_DATA);
    addButton("HivePromotionGetVersion", C2SModuleAPIList::PROMOTION_HIVE_PROMOTION_GET_VERSION);

    scheduleUpdate();

    return true;
}

void HivePromotionScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void HivePromotionScene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        HivePromotionParamScene* paramScene = HivePromotionParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    C2SModuleAPIList::HivePromotionAPI api = static_cast<C2SModuleAPIList::HivePromotionAPI>(buttonTag);
    switch (api) {
        case C2SModuleAPIList::PROMOTION_INITIALIZE:
        {
            CS_PromotionInitialize();

            addLogMessage("CS_PromotionInitialize called.");
            break;
        }
        case C2SModuleAPIList::PROMOTION_SHOW:
        {
            CSPromotionViewType viewType = ParameterDataManager::getInstance()->hivePromotionParams->getViewType();
            int isForced = ParameterDataManager::getInstance()->hivePromotionParams->getIsForced();
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();

            CS_PromotionShow(viewType, isForced, uid.c_str(), [](CSPromotionAPICallResult result, CSPromotionResult resultType) {
                addCSPromotionViewHandlerCallbackLog(result, resultType);
            });
            break;
        }
        case C2SModuleAPIList::PROMOTION_SHOW_CUSTOM:
        {
            CSPromotionCustomType customType = ParameterDataManager::getInstance()->hivePromotionParams->getCustomType();
            string contents = ParameterDataManager::getInstance()->hivePromotionParams->getContents();
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();

            CS_PromotionShowCustom(customType, contents.c_str(), uid.c_str(), [](CSPromotionAPICallResult result, CSPromotionResult resultType){
                addCSPromotionViewHandlerCallbackLog(result, resultType);
            });

            break;
        }
        case C2SModuleAPIList::PROMOTION_GET_OFFERWALL_STATE:
        {
            CSOfferwallState state = CS_GetOfferwallState();
            string stateString = CSOfferwallStateToString(state);

            stringstream log;
            log << "CS_PromotionInitialize called. " << "result : " << stateString << '\n';
            addLogMessage(log.str().c_str());
            break;
        }
        case C2SModuleAPIList::PROMOTION_SHOW_OFFERWALL:
        {
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();

            CS_ShowOfferwall(uid.c_str(), [](CSPromotionAPICallResult result, CSPromotionResult resultType){
                addCSPromotionViewHandlerCallbackLog(result, resultType);
            });
            break;
        }
        case C2SModuleAPIList::PROMOTION_SHOW_REVIEW:
        {
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();

            CS_ShowReview(uid.c_str(), [](CSPromotionAPICallResult result, CSPromotionResult resultType){
                addCSPromotionViewHandlerCallbackLog(result, resultType);
            });
            break;
        }
        case C2SModuleAPIList::PROMOTION_SHOW_NATIVE_REVIEW:
        {
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();

            CS_ShowNativeReview(uid.c_str());

            stringstream log;
            log << "CS_ShowNativeReview called. ";
            addLogMessage(log.str().c_str());
            break;
        }
        case C2SModuleAPIList::PROMOTION_SHOW_EXIT:
        {
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();

            CS_ShowExit(uid.c_str(), [](CSPromotionAPICallResult result, CSPromotionResult resultType){
                addCSPromotionViewHandlerCallbackLog(result, resultType);
            });
            break;
        }
        case C2SModuleAPIList::PROMOTION_GET_VIEW_INFO:
        {
            CSPromotionCustomType customType = ParameterDataManager::getInstance()->hivePromotionParams->getCustomType();
            string contents = ParameterDataManager::getInstance()->hivePromotionParams->getContents();
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();

            CS_GetViewInfo(customType, contents.c_str(), uid.c_str(), [](CSPromotionAPICallResult result, const char* viewInfo){
                stringstream log;
                string resultString = CSPromotionAPICallResultToString(result);

                log << "result : " << resultString << '\n';
                log << "viewInfo : " << viewInfo << '\n';

                addCallbackLogMessage(log.str().c_str(), "CSPromotionInfoHandler : ");
            });
            break;
        }
        case C2SModuleAPIList::PROMOTION_GET_BADGE_INFO:
        {
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();

            CS_GetBadgeInfo(uid.c_str(), [](CSPromotionAPICallResult result, const char* badgeInfos){
                stringstream log;
                string resultString = CSPromotionAPICallResultToString(result);

                log << "result : " << resultString << '\n';
                log << "badgeInfos : " << badgeInfos << '\n';

                addCallbackLogMessage(log.str().c_str(), "CSPromotionBadgeHandler : ");
            });
            break;
        }
        case C2SModuleAPIList::PROMOTION_GET_BANNER_INFO:
        {
            string campaignType = ParameterDataManager::getInstance()->hivePromotionParams->getCampaignType();
            string bannerType = ParameterDataManager::getInstance()->hivePromotionParams->getBannerType();
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();

            // BannerInfo 요청 시 request 데이터 로그에 추가 (GCPHQ-4020)
            stringstream requestLog;
            requestLog << "CS_PromotionGetBannerInfo request : \ncampaignType = " << campaignType << ", bannerType = " << bannerType << ", uid : " << uid << '\n';
            addLogMessage(requestLog.str());

            CS_PromotionGetBannerInfo(campaignType.c_str(), bannerType.c_str(), uid.c_str(), [](CSPromotionAPICallResult result, const char* bannerInfos){
                stringstream log;
                string resultString = CSPromotionAPICallResultToString(result);

                log << "result : " << resultString << '\n';
                log << "bannerInfos : " << bannerInfos << '\n';

                addCallbackLogMessage(log.str().c_str(), "CSPromotionBannerHandler : ");
            });
            break;
        }
        case C2SModuleAPIList::PROMOTION_GET_APP_INVITATION_DATA:
        {
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();
            string additionalInfo = "{\"uatoken\":\"asdfsafsdfs\"}";
            ParameterDataManager::getInstance()->hivePromotionParams->setAdditionalInfo(additionalInfo);

            CS_PromotionGetAppInvitationData(uid.c_str(), additionalInfo.c_str(), [](CSPromotionAPICallResult result, const char* appInvitationData){
                stringstream log;
                string resultString = CSPromotionAPICallResultToString(result);

                log << "result : " << resultString << '\n';

                if (appInvitationData != NULL) {
                    picojson::value invitationDataValue;
                    // string to JSON
                    picojson::parse(invitationDataValue, appInvitationData);

                    string inviteMessage = invitationDataValue.get("inviteMessage").to_str();
                    string inviteCommonLink = invitationDataValue.get("inviteCommonLink").to_str();
                    string inviteHiveLink = invitationDataValue.get("inviteHivemsgLink").to_str();
                    string inviteFacebookLink = invitationDataValue.get("inviteFacebookLink").to_str();

                    ParameterDataManager::getInstance()->hivePromotionParams->setInviteMessage(inviteMessage);
                    ParameterDataManager::getInstance()->hivePromotionParams->addLinkToInviteLinkList(inviteCommonLink);
                    ParameterDataManager::getInstance()->hivePromotionParams->addLinkToInviteLinkList(inviteHiveLink);
                    ParameterDataManager::getInstance()->hivePromotionParams->addLinkToInviteLinkList(inviteFacebookLink);

                    log << "appInvitationData : " << appInvitationData << '\n';
                } else {
                    log << "appInvitationData : " << "NULL" << '\n';
                }

                addCallbackLogMessage(log.str().c_str(), "CSPromotionAppInvitationDataHandler : ");
            });
            break;
        }
        case C2SModuleAPIList::PROMOTION_SHOW_UA_SHARE:
        {

            string inviteMessage = ParameterDataManager::getInstance()->hivePromotionParams->getInviteMessage();
            string inviteLink = ParameterDataManager::getInstance()->hivePromotionParams->getInviteLink();

            CS_PromotionShowUAShare(inviteMessage.c_str(), inviteLink.c_str(), [](CSPromotionAPICallResult result){
                stringstream log;
                string resultString = CSPromotionAPICallResultToString(result);

                log << "result : " << resultString << '\n';

                addCallbackLogMessage(log.str().c_str(), "CSPromotionShareHandler : ");
            });
            break;
        }
        case C2SModuleAPIList::PROMOTION_SET_ADDITIONAL_INFO:
        {
            string additionalInfo = ParameterDataManager::getInstance()->hivePromotionParams->getAdditionalInfo();

            CS_SetAdditionalInfo(additionalInfo.c_str());

            addLogMessage("CS_SetAdditionalInfo called.");
            break;
        }
        case C2SModuleAPIList::PROMOTION_UPDATE_PROMOTION_DATA:
        {
            CS_UpdatePromotionData();

            addLogMessage("CS_UpdatePromotionData called.");
            break;
        }
        case C2SModuleAPIList::PROMOTION_HIVE_PROMOTION_GET_VERSION:
        {
            const char* version = CS_HivePromotionGetVersion();

            stringstream log;
            log << "CS_HivePromotionGetVersion called. result : " << version;

            addLogMessage(log.str().c_str());
            break;
        }
        default:
            break;
    }
}

void HivePromotionScene::addCSPromotionViewHandlerCallbackLog(CSPromotionAPICallResult result, CSPromotionResult resultType)
{
    stringstream log;

    string resultString = CSPromotionAPICallResultToString(result);
    string resultTypeString = CSPromotionResultToString(resultType);

    log << "result : " << resultString << '\n';
    log << "resultType : " << resultTypeString << '\n';

    BaseView::addCallbackLogMessage(log.str().c_str(), "CSPromotionViewHandler : ");
}

string HivePromotionScene::CSPromotionAPICallResultToString(CSPromotionAPICallResult result)
{
    switch (result) {
        case CS_PROMOTION_SUCCESS:
            return "CS_PROMOTION_SUCCESS";
        case CS_PROMOTION_INVALID_PARAM:
            return "CS_PROMOTION_INVALID_PARAM";
        case CS_PROMOTION_NOT_SUPPORTED:
            return "CS_PROMOTION_NOT_SUPPORTED";
        case CS_PROMOTION_IN_PROGRESS:
            return "CS_PROMOTION_IN_PROGRESS";
        case CS_PROMOTION_TIMEOUT:
            return "CS_PROMOTION_TIMEOUT";
        case CS_PROMOTION_NETWORK:
            return "CS_PROMOTION_NETWORK";
        case CS_PROMOTION_CANCELED:
            return "CS_PROMOTION_CANCELED";
        case CS_PROMOTION_NEED_INITIALIZE:
            return "CS_PROMOTION_NEED_INITIALIZE";
        case CS_PROMOTION_RESPONSE_FAIL:
            return "CS_PROMOTION_RESPONSE_FAIL";
        case CS_PROMOTION_INVALID_SESSION:
            return "CS_PROMOTION_INVALID_SESSION";
        case CS_PROMOTION_NEED_RESTORE:
            return "CS_PROMOTION_NEED_RESTORE";
        case CS_PROMOTION_RESTORE_NOT_OWNED:
            return "CS_PROMOTION_RESTORE_NOT_OWNED";
        default:
            return "CS_PROMOTION_UNKNOWN";
    }
}

string HivePromotionScene::CSPromotionResultToString(CSPromotionResult resultType)
{
    switch (resultType) {
        case CS_PROMOTION_RESULT_NEWS_OPEN:
            return "CS_PROMOTION_RESULT_NEWS_OPEN";
        case CS_PROMOTION_RESULT_NEWS_CLOSE:
            return "CS_PROMOTION_RESULT_NEWS_CLOSE";

        case CS_PROMOTION_RESULT_NOTICE_OPEN:
            return "CS_PROMOTION_RESULT_NOTICE_OPEN";
        case CS_PROMOTION_RESULT_NOTICE_CLOSE:
            return "CS_PROMOTION_RESULT_NOTICE_CLOSE";

        case CS_PROMOTION_RESULT_BANNER_OPEN:
            return "CS_PROMOTION_RESULT_BANNER_OPEN";
        case CS_PROMOTION_RESULT_BANNER_CLOSE:
            return "CS_PROMOTION_RESULT_BANNER_CLOSE";

        case CS_PROMOTION_RESULT_CUSTOM_VIEW_OPEN:
            return "CS_PROMOTION_RESULT_CUSTOM_VIEW_OPEN";
        case CS_PROMOTION_RESULT_CUSTOM_VIEW_CLOSE:
            return "CS_PROMOTION_RESULT_CUSTOM_VIEW_CLOSE";

        case CS_PROMOTION_RESULT_CUSTOM_BOARD_OPEN:
            return "CS_PROMOTION_RESULT_CUSTOM_BOARD_OPEN";
        case CS_PROMOTION_RESULT_CUSTOM_BOARD_CLOSE:
            return "CS_PROMOTION_RESULT_CUSTOM_BOARD_CLOSE";

        case CS_PROMOTION_RESULT_SPOT_OPEN:
            return "CS_PROMOTION_RESULT_SPOT_OPEN";
        case CS_PROMOTION_RESULT_SPOT_CLOSE:
            return "CS_PROMOTION_RESULT_SPOT_CLOSE";

        case CS_PROMOTION_RESULT_DIRECT_OPEN:
            return "CS_PROMOTION_RESULT_DIRECT_OPEN";
        case CS_PROMOTION_RESULT_DIRECT_CLOSE:
            return "CS_PROMOTION_RESULT_DIRECT_CLOSE";

        case CS_PROMOTION_RESULT_OFFERWALL_OPEN:
            return "CS_PROMOTION_RESULT_OFFERWALL_OPEN";
        case CS_PROMOTION_RESULT_OFFERWALL_CLOSE:
            return "CS_PROMOTION_RESULT_OFFERWALL_CLOSE";

        case CS_PROMOTION_RESULT_REVIEW_OPEN:
            return "CS_PROMOTION_RESULT_REVIEW_OPEN";
        case CS_PROMOTION_RESULT_REVIEW_CLOSE:
            return "CS_PROMOTION_RESULT_REVIEW_CLOSE";

        case CS_PROMOTION_RESULT_MORE_GAMES_OPEN:
            return "CS_PROMOTION_RESULT_MORE_GAMES_OPEN";
        case CS_PROMOTION_RESULT_MORE_GAMES_CLOSE:
            return "CS_PROMOTION_RESULT_MORE_GAMES_CLOSE";
        case CS_PROMOTION_RESULT_MORE_GAMES_EXIT:
            return "CS_PROMOTION_RESULT_MORE_GAMES_EXIT";

        case CS_PROMOTION_RESULT_START_PLAYBACK:
            return  "CS_PROMOTION_RESULT_START_PLAYBACK";
        case CS_PROMOTION_RESULT_FINISH_PLAYBACK:
            return "CS_PROMOTION_RESULT_FINISH_PLAYBACK";

        case CS_PROMOTION_RESULT_NEWS_GOBACK:
            return "CS_PROMOTION_RESULT_NEWS_GOBACK";
        case CS_PROMOTION_RESULT_NOTICE_GOBACK:
            return "CS_PROMOTION_RESULT_NOTICE_GOBACK";
        case CS_PROMOTION_RESULT_CUSTOM_VIEW_GOBACK:
            return "CS_PROMOTION_RESULT_CUSTOM_VIEW_GOBACK";
        case CS_PROMOTION_RESULT_CUSTOM_BOARD_GOBACK:
            return "CS_PROMOTION_RESULT_CUSTOM_BOARD_GOBACK";
        case CS_PROMOTION_RESULT_SPOT_GOBACK:
            return "CS_PROMOTION_RESULT_SPOT_GOBACK";
        case CS_PROMOTION_RESULT_DIRECT_GOBACK:
            return "CS_PROMOTION_RESULT_DIRECT_GOBACK";
        case CS_PROMOTION_RESULT_OFFERWALL_GOBACK:
            return "CS_PROMOTION_RESULT_OFFERWALL_GOBACK";
        default:
            return "UNKNOWN";
    }
}

string HivePromotionScene::CSOfferwallStateToString(CSOfferwallState state)
{
    switch (state) {
        case CS_OFFERWALL_STATE_ENABLED:
            return "CS_OFFERWALL_STATE_ENABLED";
        case CS_OFFERWALL_STATE_DISABLED:
            return "CS_OFFERWALL_STATE_DISABLED";
        default:
            return "CS_OFFERWALL_STATE_UNKNOWN";
    }
}
