//
//  InAppV4Scene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by sjyong on 2021/09/27.
//

#include "C2SModuleAPIList.hpp"
#include "InAppV4Scene.hpp"
#include "InAppV4ParamScene.hpp"
#include "ParameterDataManager.hpp"
#include "ui/CocosGUI.h"
#include "PostboxView.hpp"

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;

#include <map>
#include <sstream>
#include <string>

// 영수증 및 bypassInfo 로그 노출 길이 최대 값
#define INFO_MAX_LENGTH 1000

using namespace::std;

bool InAppV4Scene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- InAppV4 -");


    addButton("MarketConnect", C2SModuleAPIList::INAPPV4_MARKET_CONNECT);
    addButton("GetMarketProductInfo", ButtonMode::NEED_PARAM, C2SModuleAPIList::INAPPV4_GET_MARKET_PRODUCT_INFO);
    addButton("GetProductInfo", C2SModuleAPIList::INAPPV4_GET_PRODUCT_INFO);
    addButton("GetSubscriptionProductInfo", C2SModuleAPIList::INAPPV4_GET_SUBSCRIPTION_PRODUCT_INFO);
    addButton("Purchase", ButtonMode::NEED_PARAM, C2SModuleAPIList::INAPPV4_PURCHSE);
    addButton("PurchaseSubscription", ButtonMode::NEED_PARAM, C2SModuleAPIList::INAPPV4_PURCHASE_SUBSCRIPTION);
    addButton("Restore", C2SModuleAPIList::INAPPV4_RESTORE);
    addButton("RestoreSubscription", C2SModuleAPIList::INAPPV4_RESTORE_SUBSCRIPTION);
    addButton("TransactionFinish", ButtonMode::NEED_PARAM, C2SModuleAPIList::INAPPV4_TRANSACTION_FINISH);
    addButton("CheckPromotionPurchase", C2SModuleAPIList::INAPPV4_CHECK_PROMOTE_PURCHASE);
//    addButton("SetUid", ButtonMode::NEED_PARAM, C2SModuleAPIList::INAPPV4_SET_UID);   // removed, C2S.12.0
    addButton("GetSelectedMarket", C2SModuleAPIList::INAPPV4_GET_SELECTED_MARKET);
    addButton("GetAccountUuid", C2SModuleAPIList::INAPPV4_GET_ACCOUNT_UUID);
    addButton("ShowMarketSelection", C2SModuleAPIList::INAPPV4_SHOW_MARKET_SELECTION);
    addButton("GetBalanceInfo", C2SModuleAPIList::INAPPV4_GET_BALANCE_INFO);
    addButton("ShowCharge", C2SModuleAPIList::INAPPV4_SHOW_CHARGE);
    addButton("SetLifeCycleCB", C2SModuleAPIList::INAPPV4_SET_LIFE_CYCLE_CB);
    addButton("ShowInAppOffers", C2SModuleAPIList::INAPPV4_SHOW_INAPP_MESSAGES);
    addButton("GetVersion", C2SModuleAPIList::INAPPV4_GET_VERSION);
    addButton("Postbox", C2SModuleAPIList::POSTBOX_OPEN);

    scheduleUpdate();

    return true;
}

void InAppV4Scene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void InAppV4Scene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        InAppV4ParamScene* paramScene = InAppV4ParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    C2SModuleAPIList::InAppV4API api = static_cast<C2SModuleAPIList::InAppV4API>(buttonTag);
    switch (api) {
        case C2SModuleAPIList::INAPPV4_MARKET_CONNECT:
        {
            CS_IapV4MarketConnect([](CS_IapV4Result iapResult, int numOfMarketList, int* marketList){

                stringstream log;
                log << IapV4ResultToString(iapResult);
                log << "{\n";
                for (int i=0; i<numOfMarketList; i++) {
                    log << marketList[i] << "(" << getMarketNameWithIndex(marketList[i]) << ")" << '\n';

                }
                log << "}\n";

                addCallbackLogMessage(log.str(), "MarketListCB : ");
            });

            addLogMessage("CS_IapV4MarketConnect called.");
            break;
        }
        case C2SModuleAPIList::INAPPV4_GET_MARKET_PRODUCT_INFO:
        {
            string marketPid = ParameterDataManager::getInstance()->inAppV4Params->getMarketProductInfoPidList();

            /**
             * (1) split marketPid value
             *
             * ex) marketPid : "item001,item002,item003"
             * >> vector v : item001 / item002 / item003 (index 3)
             *
             * ex) marketPid : "item001"
             * >> vector v : item001 (index 1)
             */
            vector<const char*> marketPidVector ;
            size_t pos = 0;
            string token;
            string delimiter = ",";
            bool existDelimiter = false;
            while ((pos = marketPid.find(delimiter)) != string::npos) {
                token = marketPid.substr(0, pos);
                marketPidVector.push_back(token.c_str());     // 값 value
                marketPid.erase(0, pos + delimiter.length());   // 전체 값에서 push한 값 삭제
                existDelimiter = true;          // delimiter 존재 확인
            }
            if(existDelimiter) {
                // delimiter 있는 경우 마지막 남은 값 push
                marketPidVector.push_back(marketPid.c_str());
            } else {
                // delimiter 없는 경우
                marketPidVector.push_back(marketPid.c_str());
            }

            /**
             * (2) convert vector to const char**
             */
            const char** marketPidArray = (const char**) malloc(sizeof(const char*)*(marketPidVector.size() + 1) );
            for(size_t i = 0; i < marketPidVector.size(); ++i) {
                marketPidArray[i] = strdup(marketPidVector[i]);
            }
            marketPidArray[marketPidVector.size()] = nullptr;

            /**
             * (3) C2S Module API call (CS_IapV4GetMarketProductInfo)
             */
            CS_IapV4GetMarketProductInfo(marketPidArray, [](CS_IapV4Result iapResult, int numOfProduct, CS_IapV4ProductInfo* productInfo) {

                stringstream log;
                log << IapV4ResultToString(iapResult);
                log << "numOfProduct : " << numOfProduct << '\n';
                log << "{\n";
                for (int i=0; i<numOfProduct; i++) {
                    log << IapV4ProductInfoToString(productInfo[i]) << '\n';
                    // Data set
                    CS_IapV4ProductInfo info = productInfo[i];
                    ParameterDataManager::getInstance()->inAppV4Params->addMarketPidToList(info.marketPid);
                }
                log << "}\n";

                addCallbackLogMessage(log.str(), "ProductInfoCB : ");
            });

            /**
             * (4) const char** (marketPidArray) memory reset
             */
            free(marketPidArray);

            break;
        }
        case C2SModuleAPIList::INAPPV4_GET_PRODUCT_INFO:
        {
            CS_IapV4GetProductInfo([](CS_IapV4Result iapResult, int numOfProduct, CS_IapV4ProductInfo* productInfo){

                stringstream log;
                log << IapV4ResultToString(iapResult);
                log << "numOfProduct : " << numOfProduct << '\n';
                log << "{\n";
                for (int i=0; i<numOfProduct; i++) {
                    log << IapV4ProductInfoToString(productInfo[i]) << '\n';
                    // Data set
                    CS_IapV4ProductInfo info = productInfo[i];
                    ParameterDataManager::getInstance()->inAppV4Params->addMarketPidToList(info.marketPid);
                }
                log << "}\n";

                addCallbackLogMessage(log.str(), "ProductInfoCB : ");
            });

            addLogMessage("CS_IapV4GetProductInfo called.");
            break;
        }
        case C2SModuleAPIList::INAPPV4_GET_SUBSCRIPTION_PRODUCT_INFO:
        {
            CS_IapV4GetSubscriptionProductInfo([](CS_IapV4Result iapResult, int numOfProduct, CS_IapV4ProductInfo* productInfo){

                stringstream log;
                log << IapV4ResultToString(iapResult);
                log << "numOfProduct : " << numOfProduct << '\n';
                log << "{\n";
                for (int i=0; i<numOfProduct; i++) {
                    log << IapV4ProductInfoToString(productInfo[i]) << '\n';

                    // Data set
                    CS_IapV4ProductInfo info = productInfo[i];
                    ParameterDataManager::getInstance()->inAppV4Params->addMarketPidToList(info.marketPid);
                }
                log << "}\n";

                addCallbackLogMessage(log.str(), "ProductInfoCB : ");
            });

            addLogMessage("CS_IapV4GetSubscriptionProductInfo called.");
            break;
        }
        case C2SModuleAPIList::INAPPV4_PURCHSE:
        {
            string marketPid = ParameterDataManager::getInstance()->inAppV4Params->getMarketPid();
            string additionalInfo = ParameterDataManager::getInstance()->inAppV4Params->getAdditionalInfo();

            CS_IapV4Purchase(marketPid.c_str(), additionalInfo.c_str(), [](CS_IapV4Result iapResult, CS_IapV4ReceiptInfo receiptInfo){

                stringstream log;
                log << IapV4ResultToString(iapResult);

                if (iapResult.errorCode != 0) {
                    addCallbackLogMessage(log.str(), "ReceiptInfoCB : ");
                    return;
                }
                log << "receiptInfo : " << IapV4ReceiptInfoToString(receiptInfo) << '\n';

                verifyReceipt(receiptInfo);
                addCallbackLogMessage(log.str(), "ReceiptInfoCB : ");
            });

            addLogMessage("CS_IapV4Purchase called.");
            break;
        }
        case C2SModuleAPIList::INAPPV4_PURCHASE_SUBSCRIPTION:
        {
            string marketPid = ParameterDataManager::getInstance()->inAppV4Params->getMarketPid();
            string oldMarketPid = ParameterDataManager::getInstance()->inAppV4Params->getOldMarketPid();
            string additionalInfo = ParameterDataManager::getInstance()->inAppV4Params->getAdditionalInfo();

            CS_IapV4PurchaseSubscription(marketPid.c_str(), oldMarketPid.c_str(), additionalInfo.c_str(),
                                         [](CS_IapV4Result iapResult, CS_IapV4ReceiptInfo receiptInfo){

                stringstream log;
                log << IapV4ResultToString(iapResult);

                if (iapResult.errorCode != 0) {
                    addCallbackLogMessage(log.str(), "ReceiptInfoCB : ");
                    return;
                }
                log << "receiptInfo : " << IapV4ReceiptInfoToString(receiptInfo) << '\n';

                verifyReceipt(receiptInfo);
                addCallbackLogMessage(log.str(), "ReceiptInfoCB : ");
            });

            addLogMessage("CS_IapV4PurchaseSubscription called.");
            break;
        }
        case C2SModuleAPIList::INAPPV4_RESTORE:
        {
            CS_IapV4Restore([](CS_IapV4Result iapResult, int numOfReceipt, CS_IapV4ReceiptInfo* receiptInfo){

                stringstream log;
                log << IapV4ResultToString(iapResult);

                if (iapResult.errorCode != 0) {
                    addCallbackLogMessage(log.str(), "ReceiptInfoCB : ");
                    return;
                }
                log << "numOfRecipt : " << numOfReceipt << '\n';
                log << "receiptInfo : " << '\n';
                log << "{\n";

                for (int i=0; i<numOfReceipt; i++) {
                    log << IapV4ReceiptInfoToString(receiptInfo[i]) << ", ";
                    verifyReceipt(receiptInfo[i]);
                }
                log << "\n}\n";

                addCallbackLogMessage(log.str(), "ReceiptInfoCB : ");
            });

            addLogMessage("CS_IapV4Restore called.");
            break;
        }
        case C2SModuleAPIList::INAPPV4_RESTORE_SUBSCRIPTION:
        {
            CS_IapV4RestoreSubscription([](CS_IapV4Result iapResult, int numOfReceipt, CS_IapV4ReceiptInfo* receiptInfo){
                stringstream log;
                log << IapV4ResultToString(iapResult);

                if (iapResult.errorCode != 0) {
                    addCallbackLogMessage(log.str(), "ReceiptInfoCB : ");
                    return;
                }

                log << "numOfRecipt : " << numOfReceipt << '\n';
                log << "receiptInfo : " << '\n';
                log << "{\n";

                for (int i=0; i<numOfReceipt; i++) {
                    log << IapV4ReceiptInfoToString(receiptInfo[i]) << ", ";
                    verifyReceipt(receiptInfo[i]);
                }
                log << "\n}\n";

                addCallbackLogMessage(log.str(), "ReceiptInfoCB : ");
            });

            addLogMessage("CS_IapV4RestoreSubscription called.");
            break;
        }
        case C2SModuleAPIList::INAPPV4_TRANSACTION_FINISH:
        {
            string marketPid = ParameterDataManager::getInstance()->inAppV4Params->getMarketPid();

            CS_IapV4TransactionFinish(marketPid.c_str(), [](CS_IapV4Result iapV4Result, const char* marketPid){

                stringstream log;
                log << IapV4ResultToString(iapV4Result);

                if (iapV4Result.errorCode != 0) {
                    addCallbackLogMessage(log.str(), "TransactionCB : ");
                    return;
                }
                log << "marketPid : ";

                if (marketPid != NULL){
                    log << marketPid << '\n';
                }
                else {
                    log << "NULL" << '\n';
                }

                addCallbackLogMessage(log.str(), "TransactionCB : ");
            });

            stringstream log;
            log << "CS_IapV4TransactionFinish called. marketPid : " << marketPid <<'\n';
            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::INAPPV4_CHECK_PROMOTE_PURCHASE:
        {
            CS_IapV4CheckPromotePurchase([](CS_IapV4Result iapResult, const char* marketPid){
                stringstream log;
                log << IapV4ResultToString(iapResult);

                if (marketPid != NULL) {
                    log << "marketPid : " << marketPid << '\n';
                }

                addCallbackLogMessage(log.str(), "CheckPromotePurcahseCB : ");
            });

            addLogMessage("CS_IapV4CheckPromotePurchase called.");
            break;
        }
        case C2SModuleAPIList::INAPPV4_SET_UID:
        {
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();

//            CS_IapV4SetUid(uid.c_str());      // deprecated

            stringstream log;
            log << "CS_IapV4SetUid called. uid : " << uid <<'\n';
            addLogMessage(log.str());
            addLogMessage("( CS_IapV4SetUid API는 KS C2S에서 더이상 작동하지 않습니다.(deprecated) )");
            break;
        }
        case C2SModuleAPIList::INAPPV4_GET_SELECTED_MARKET:
        {
            int market = CS_IapV4GetSelectedMarket();

            stringstream log;
            log << "CS_IapV4GetSelectedMarket called. market : " << market <<'\n';
            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::INAPPV4_GET_ACCOUNT_UUID:
        {
            const char* accountUuid = CS_IapV4GetAccountUuid();

            stringstream log;
            log << "CS_IapV4GetAccountUuid called. accountUuid : " << accountUuid << '\n';
            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::INAPPV4_SHOW_MARKET_SELECTION:
        {
            CS_IapV4ShowMarketSelection([](CS_IapV4Result iapResult, int numOfMarketList, int* marketList){

                stringstream log;
                log << IapV4ResultToString(iapResult);
                log << "numOfMarketList : " << numOfMarketList << '\n';
                log << "{\n";
                for (int i=0; i<numOfMarketList; i++) {
                    log << marketList[i] << '\n';
                }
                log << "}\n";

                addCallbackLogMessage(log.str(), "MarketListCB : ");
            });

            addLogMessage("CS_IapV4ShowMarketSelection called.");
            break;
        }
        case C2SModuleAPIList::INAPPV4_GET_BALANCE_INFO:
        {
            CS_IapV4GetBalanceInfo([](CS_IapV4Result iapResult, int balance){

                stringstream log;
                log << IapV4ResultToString(iapResult);
                log << "balance : " << balance << '\n';

                addCallbackLogMessage(log.str(), "BalanceCB : ");
            });

            addLogMessage("CS_IapV4GetBalanceInfo called.");
            break;
        }
        case C2SModuleAPIList::INAPPV4_SHOW_CHARGE:
        {
            CS_IapV4ShowCharge([](CS_IapV4Result iapResult, int balance){

                stringstream log;
                log << IapV4ResultToString(iapResult);
                log << "balance : " << balance << '\n';

                addCallbackLogMessage(log.str(), "BalanceCB : ");
            });

            addLogMessage("CS_IapV4ShowCharge called.");
            break;
        }
        case C2SModuleAPIList::INAPPV4_SET_LIFE_CYCLE_CB:
        {
            CS_IapV4SetLifecycleCB([](int type, const char* data){

                stringstream log;
                log << "type : " << type << '\n';
                log << "data : " << data << '\n';

                addCallbackLogMessage(log.str(), "IapV4LifecycleCB : ");
            });

            addLogMessage("CS_IapV4SetLifecycleCB called.");
            break;
        }
        case C2SModuleAPIList::INAPPV4_GET_VERSION:
        {
            const char* version = CS_IapV4GetVersion();

            stringstream log;
            log << "CS_IapV4GetVersion called. version : " << version << '\n';
            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::INAPPV4_SHOW_INAPP_MESSAGES:
        {
            CS_IapV4ShowInAppMessages(3, [](int responseCode){

                stringstream log;
                log << "responseCode : " << responseCode << '\n';

                addCallbackLogMessage(log.str(), "InAppOffersCB : ");
            });

            addLogMessage("CS_IapV4ShowInAppOffers called.");
            break;
        }
        default:
            break;
    }

    // MARK: PostBox
    C2SModuleAPIList::PostBox postbox = static_cast<C2SModuleAPIList::PostBox>(buttonTag);
    switch (postbox) {
        case C2SModuleAPIList::POSTBOX_OPEN:
        {
            pushView(PostBoxView::create());
        }
        default:
            break;
    }
}

string InAppV4Scene::IapV4ProductInfoToString(CS_IapV4ProductInfo productInfo)
{
    stringstream description;
    if (productInfo.productType)
        description << "productType : " << productInfo.productType << '\n';
    description << "marketPid : " << productInfo.marketPid << '\n';
    description << "currency : " << productInfo.currency << '\n';
    description << "price : " << productInfo.price << '\n';
    description << "displayPrice : " << productInfo.displayPrice << '\n';
    description << "title : " << productInfo.title << '\n';
    description << "description : " << productInfo.description << '\n';
    description << "originalJson : " << productInfo.originalJson << '\n';
    description << "displayOriginalPrice : " << productInfo.displayOriginalPrice << '\n';
    description << "originalPrice : " << productInfo.originalPrice << '\n';
    description << "iconUrl : " << productInfo.iconUrl << '\n';
    description << "coinsReward : " << productInfo.coinsReward << '\n';

    return description.str();
}

string InAppV4Scene::IapV4ReceiptInfoToString(CS_IapV4ReceiptInfo receiptInfo)
{
    stringstream description;
    description << '\n' << "- type : " << to_string(receiptInfo.type) << '\n';
    
    if (receiptInfo.bypassInfo)
        description << "- bypassInfo : " << ((string)receiptInfo.bypassInfo).substr(0, INFO_MAX_LENGTH) << '\n';
    if (receiptInfo.hiveiapReceipt)
        description << "- hiveiapReceipt : " << ((string)receiptInfo.hiveiapReceipt).substr(0, INFO_MAX_LENGTH) << '\n';
    if (receiptInfo.productType)
        description << "- productType : " << receiptInfo.productType << '\n';
    if (receiptInfo.marketPid)
        description << "- marketPid : " << receiptInfo.marketPid << '\n';
    if (receiptInfo.currency)
        description << "- currency : " << receiptInfo.currency << '\n';

    description << "- price : " << to_string(receiptInfo.price) << '\n';

    if (receiptInfo.displayPrice)
        description << "- displayPrice : " << receiptInfo.displayPrice << '\n';
    
    if (receiptInfo.iapPayload)
        description << "- iapPayload : " << receiptInfo.iapPayload << '\n';
    if (receiptInfo.quantity)
        description << "- quantity : " << to_string(receiptInfo.quantity) << '\n';

    return description.str();
}

string InAppV4Scene::IapV4ResultToString(CS_IapV4Result result)
{
    stringstream description;
    description << "result : " << result.result << '\n';
    description << "errorCode : " << result.errorCode << '\n';
    description << "errorMessage : " << result.errorMessage << '\n';

    return description.str();
}

void InAppV4Scene::verifyReceipt(CS_IapV4ReceiptInfo receiptInfo)
{
    Postbox::ItemType itemType = Postbox::consumable;

    if (strcmp(receiptInfo.productType,"subscription") == 0) {
        itemType = Postbox::subscription;
    }
    // 영수증 검증
    Postbox::verifyReceipt(receiptInfo.bypassInfo, itemType, [](const Postbox::VerifyReceipt* verifyReceipt, Postbox::Error error){
        stringstream verifyLog;
        if (verifyReceipt != NULL) {
            verifyLog << "product_id : " << verifyReceipt->product_id << '\n';
        }
        else {
            verifyLog << "product_id : " << "NULL" << '\n';
        }
        verifyLog << "result_code : " << error.result_code << '\n';
        verifyLog << "result_msg : " << error.result_msg << '\n';

        addCallbackLogMessage(verifyLog.str(), "Postbox::verifyReceiptCB : ");
    });
}

string InAppV4Scene::getMarketNameWithIndex(int index)
{
    switch(index) {
        case 1: {
            return "Apple_AppStore";
        }
        case 2 : {
            return "Google_PlayStore";
        }
        case 3 : {
            return "Hive_Lebi";
        }
        case 4 : {
            return "OneStore";
        }
        case 5 : {
            return "Amazon_AppStore";
        }
        case 6 : {
            return "Samsung_GalaxyStore";
        }
        case 7 : {
            return "Huawei_AppGallery";
        }
        default : {
            return "not selected";
        }
    }
}
