//
//  MercuryParamScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by sjyong on 2021/10/26.
//

#include "MercuryParamScene.hpp"
#include "ParameterDataManager.hpp"
#include "ui/CocosGUI.h"

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;

#include <map>

using namespace std;

bool MercuryParamScene::init()
{
    if (!BaseView::init())
        return false;

    addButton("OK", BUTTON_OK);

    scheduleUpdate();

    // showType
    showTypeStringList = {
        "MERCURY_SHOW_EVENT_LEGACY",
        "MERCURY_SHOW_EVENT",
        "MERCURY_SHOW_NOTICE_TOP",
        "MERCURY_SHOW_NOTICE_BOTTOM",
        "MERCURY_SHOW_FORCED_NOTICE_TOP",
        "MERCURY_SHOW_FORCED_NOTICE_BOTTOM",
        "MERCURY_SHOW_NOTICE_ONLY",
        "MERCURY_SHOW_FORCED_NOTICE_ONLY",
    };

    for (int i=0; i<showTypeStringList.size(); i++) {
        showTypeList.push_back(showTypeStringList[i].c_str());
    }

    // badgeTarget
    badgeTargetStringList = {
        "MERCURY_BADGE_FOR_EVENT",
        "MERCURY_BADGE_FOR_NOTICE",
        "MERCURY_BADGE_FOR_NOTICE_ONLY",
        "MERCURY_BADGE_FOR_NONE",
    };

    for (int i=0; i<badgeTargetStringList.size(); i++) {
        badgeTargetList.push_back(badgeTargetStringList[i].c_str());
    }

    // campaignType
    campaignTypeStringList = {
        "all",
        "event",
        "notice",
        "cross"
    };

    for (int i=0; i<campaignTypeStringList.size(); i++) {
        campaignTypeList.push_back(campaignTypeStringList[i].c_str());
    }

    // bannerType
    bannerTypeStringList = {
        "great",
        "small",
        "rolling"
    };

    for (int i=0; i<bannerTypeStringList.size(); i++) {
        bannerTypeList.push_back(bannerTypeStringList[i].c_str());
    }

    return true;
}

void MercuryParamScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void MercuryParamScene::onButtonEvent(int buttonTag)
{
    if (buttonTag != BUTTON_OK) {
        return;
    }


    setCompletionHandler();
    setCompletionHandler = NULL;
    this->popView();
}

void MercuryParamScene::showParameters(int tag)
{
    api = static_cast<C2SModuleAPIList::MercuryAPI>(tag);
    switch (api) {
        case C2SModuleAPIList::MERCURY_SHOW_EX:
        {
            showShowExParam([=](){
                string uid = getEditboxValue(EDITBOX_UID);
                string showTypeString = getComboboxValue(COMBOBOX_SHOW_TYPE);
                MERCURY_SHOW_TYPE showType = stringToShowType(showTypeString);
                string additionalInfo = getEditboxValue(EDITBOX_ADDITIONAL_INFO);

                ParameterDataManager::getInstance()->commonParams->setUid(uid);
                ParameterDataManager::getInstance()->mercuryParams->setShowType(showType);
                ParameterDataManager::getInstance()->mercuryParams->setAdditionalInfo(additionalInfo);
            });
            break;
        }
        case C2SModuleAPIList::MERCURY_SHOW_EX_CUSTOM_VIEW_ID:
        {
            showShowExCustomViewIdParam([=](){
                string uid = getEditboxValue(EDITBOX_UID);
                string viewId = getEditboxValue(EDITBOX_VIEW_ID);
                string additionalInfo = getEditboxValue(EDITBOX_ADDITIONAL_INFO);

                ParameterDataManager::getInstance()->commonParams->setUid(uid);
                ParameterDataManager::getInstance()->mercuryParams->setViewID(std::stoi(viewId));
                ParameterDataManager::getInstance()->mercuryParams->setAdditionalInfo(additionalInfo);
            });
            break;
        }
        case C2SModuleAPIList::MERCURY_GET_CUSTOM_VIEW_INFO:
        {
            showGetCustomViewInfoParam([=](){
                string uid = getEditboxValue(EDITBOX_UID);
                string showTypeString = getComboboxValue(COMBOBOX_VIEW_ID);
                MERCURY_SHOW_TYPE showType = stringToShowType(showTypeString);

                ParameterDataManager::getInstance()->commonParams->setUid(uid);
                ParameterDataManager::getInstance()->mercuryParams->setShowType(showType);
            });
            break;
        }
        case C2SModuleAPIList::MERCURY_GET_CUSTOM_VIEW_INFO_CUSTOM_VIEW_ID:
        {
            showGetCustomViewInfoCustomViewIdParam([=](){
                string uid = getEditboxValue(EDITBOX_UID);
                string viewId = getEditboxValue(EDITBOX_VIEW_ID);

                ParameterDataManager::getInstance()->commonParams->setUid(uid);
                ParameterDataManager::getInstance()->mercuryParams->setViewID(std::stoi(viewId));
            });
            break;
        }
        case C2SModuleAPIList::MERCURY_SHOW_REVIEW_POPUP:
        {
            showShowPopupParam(POPUP_REVIEW, [=](){
                string uid = getEditboxValue(EDITBOX_UID);
                string vid = getEditboxValue(EDITBOX_VID);
                string did = getEditboxValue(EDITBOX_DID);

                ParameterDataManager::getInstance()->commonParams->setUid(uid);
                ParameterDataManager::getInstance()->commonParams->setVid(vid);
                ParameterDataManager::getInstance()->commonParams->setDid(did);
            });
            break;
        }
        case C2SModuleAPIList::MERCURY_SHOW_NATIVE_REVIEW_POPUP:
        {
            showShowPopupParam(POPUP_NATIVE_REVIEW, [=](){
                string uid = getEditboxValue(EDITBOX_UID);
                string vid = getEditboxValue(EDITBOX_VID);
                string did = getEditboxValue(EDITBOX_DID);

                ParameterDataManager::getInstance()->commonParams->setUid(uid);
                ParameterDataManager::getInstance()->commonParams->setVid(vid);
                ParameterDataManager::getInstance()->commonParams->setDid(did);
            });
            break;
        }
        case C2SModuleAPIList::MERCURY_SHOW_MORE_GAMES_POPUP:
        {
            showShowPopupParam(POPUP_MORE_GAMES, [=](){
                string uid = getEditboxValue(EDITBOX_UID);
                string vid = getEditboxValue(EDITBOX_VID);
                string did = getEditboxValue(EDITBOX_DID);

                ParameterDataManager::getInstance()->commonParams->setUid(uid);
                ParameterDataManager::getInstance()->commonParams->setVid(vid);
                ParameterDataManager::getInstance()->commonParams->setDid(did);
            });
            break;
        }
        case C2SModuleAPIList::MERCURY_SET_UID:
        {
            showSetUidParam([=](){
                string uid = getEditboxValue(EDITBOX_UID);
                ParameterDataManager::getInstance()->commonParams->setUid(uid);
            });
            break;
        }
        case C2SModuleAPIList::MERCURY_SET_IS_USING_STAGING:
        {
            showSetIsUsingStagingParam([=](){
                int isUsingStaging = getSelectedToggleButtonIndex(TOGGLE_BUTTON_IS_USING_STAGING);
                ParameterDataManager::getInstance()->mercuryParams->setIsUsingStaging(isUsingStaging);
            });
            break;
        }
        case C2SModuleAPIList::MERCURY_SET_LOG:
        {
            showSetLogParam([=](){
                int isLog = getSelectedToggleButtonIndex(TOGGLE_BUTTON_IS_LOG);
                ParameterDataManager::getInstance()->mercuryParams->setIsLog(isLog);
            });
            break;
        }
        case C2SModuleAPIList::MERCURY_GET_BADGE:
        {
            showGetBadgeParam([=](){
                int badgeTargetIndex = getSelectedComboboxIndex(COMBOBOX_BADGE_TARGET);
                string badgeTargetString = badgeTargetStringList[badgeTargetIndex];
                MERCURY_BADGE_TARGET badgeTarget = stringToBadgeTarget(badgeTargetString);

                ParameterDataManager::getInstance()->mercuryParams->setBadgeTarget(static_cast<int>(badgeTarget));
            });
            break;
        }
        case C2SModuleAPIList::MERCURY_GET_BADGE_TYPE:
        {
            showGetBadgeTypeParam([=](){
                int badgeTargetIndex = getSelectedComboboxIndex(COMBOBOX_BADGE_TARGET);
                string badgeTargetString = badgeTargetStringList[badgeTargetIndex];
                MERCURY_BADGE_TARGET badgeTarget = stringToBadgeTarget(badgeTargetString);

                ParameterDataManager::getInstance()->mercuryParams->setBadgeTarget(static_cast<int>(badgeTarget));
            });
            break;
        }
        case C2SModuleAPIList::MERCURY_REQUEST_UA_INFO:
        {
            showRequestUAInfoParam([=](){
                string uid = getEditboxValue(EDITBOX_UID);
                string additionalInfo = getEditboxValue(EDITBOX_ADDITIONAL_INFO);

                ParameterDataManager::getInstance()->commonParams->setUid(uid);
                ParameterDataManager::getInstance()->mercuryParams->setAdditionalInfo(additionalInfo);
            });
            break;
        }
        case C2SModuleAPIList::MERCURY_REQUEST_UA_LINK_SHARE:
        {
            showRequestUALinkShareParam([=](){
                string urlLink = getEditboxValue(EDITBOX_URL_LINK);
                string shareMessage = getEditboxValue(EDITBOX_SHARE_MESSAGE);

                ParameterDataManager::getInstance()->mercuryParams->setUrlLink(urlLink);
                ParameterDataManager::getInstance()->mercuryParams->setShareMessage(shareMessage);
            });
            break;
        }
        case C2SModuleAPIList::MERCURY_GET_BANNER_INFO:
        {
            showGetBannerInfoParam([=](){
                string campaignType = getComboboxValue(COMBOBOX_CAMPAIGN_TYPE);
                string bannerType = getComboboxValue(COMBOBOX_BANNER_TYPE);

                ParameterDataManager::getInstance()->mercuryParams->setCampaignType(campaignType);
                ParameterDataManager::getInstance()->mercuryParams->setBannerType(bannerType);
            });
            break;
        }
        default:
            break;
    }
}

void MercuryParamScene::showShowExParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- ShowEx -");
    setCompletionHandler = completionHandler;

    string uid = ParameterDataManager::getInstance()->commonParams->getUid();

    MERCURY_SHOW_TYPE showType = ParameterDataManager::getInstance()->mercuryParams->getShowType();
    long index = find(showTypeStringList.begin(), showTypeStringList.end(), showTypeToString(showType)) - showTypeStringList.begin();

    string additionalInfo = ParameterDataManager::getInstance()->mercuryParams->getAdditionalInfo();

    addEditBox("uid : ", uid.c_str(), "", EDITBOX_UID);
    addCombobox("mercuryShowType : ", showTypeList, index, COMBOBOX_SHOW_TYPE);
    addEditBox("additionalInfo : ", additionalInfo.c_str(), "" , EDITBOX_ADDITIONAL_INFO);
}

void MercuryParamScene::showShowExCustomViewIdParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- ShowEx(custom viewId) -");
    setCompletionHandler = completionHandler;

    string uid = ParameterDataManager::getInstance()->commonParams->getUid();
    int viewId = ParameterDataManager::getInstance()->mercuryParams->getViewID();
    string additionalInfo = ParameterDataManager::getInstance()->mercuryParams->getAdditionalInfo();

    addEditBox("uid : ", uid.c_str(), "", EDITBOX_UID);
    addEditBox("viewId : ", std::to_string(viewId).c_str(), "", EDITBOX_VIEW_ID);
    addEditBox("additionalInfo : ", additionalInfo.c_str(), "" , EDITBOX_ADDITIONAL_INFO);
}

void MercuryParamScene::showGetCustomViewInfoParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- GetCustomViewInfo -");
    setCompletionHandler = completionHandler;

    string uid = ParameterDataManager::getInstance()->commonParams->getUid();
    MERCURY_SHOW_TYPE showType = ParameterDataManager::getInstance()->mercuryParams->getShowType();
    long index = find(showTypeStringList.begin(), showTypeStringList.end(), showTypeToString(showType)) - showTypeStringList.begin();

    addEditBox("uid : ", uid.c_str(), "", EDITBOX_UID);
    addCombobox("viewID : ", showTypeList, index, COMBOBOX_VIEW_ID);
}

void MercuryParamScene::showGetCustomViewInfoCustomViewIdParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- GetCustomViewInfo(custom viewId) -");
    setCompletionHandler = completionHandler;

    string uid = ParameterDataManager::getInstance()->commonParams->getUid();
    int viewId = ParameterDataManager::getInstance()->mercuryParams->getViewID();

    addEditBox("uid : ", uid.c_str(), "", EDITBOX_UID);
    addEditBox("viewId : ", std::to_string(viewId).c_str(), "", EDITBOX_VIEW_ID);
}

void MercuryParamScene::showShowPopupParam(PopupType type, function<void()> completionHandler)
{
    switch (type) {
        case POPUP_REVIEW:
            setUseBackButton(true, "- ShowReviewPopup -");
            break;
        case POPUP_NATIVE_REVIEW:
            setUseBackButton(true, "- ShowNativeReviewPopup -");
            break;
        case POPUP_MORE_GAMES:
            setUseBackButton(true, "- ShowMoregamesPopup -");
            break;
    }

    setCompletionHandler = completionHandler;

    string uid = ParameterDataManager::getInstance()->commonParams->getUid();
    string vid = ParameterDataManager::getInstance()->commonParams->getVid();
    string did = ParameterDataManager::getInstance()->commonParams->getDid();

    addEditBox("uid : ", uid.c_str(), "", EDITBOX_UID);
    addEditBox("vid : ", uid.c_str(), "", EDITBOX_VID);
    addEditBox("did : ", uid.c_str(), "", EDITBOX_DID);
}

void MercuryParamScene::showSetUidParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SetUid -");
    setCompletionHandler = completionHandler;

    string uid = ParameterDataManager::getInstance()->commonParams->getUid();

    addEditBox("uid : ", uid.c_str(), "", EDITBOX_UID);
}

void MercuryParamScene::showSetIsUsingStagingParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SetIsUsingStaging -");
    setCompletionHandler = completionHandler;

    int isUsingStaging = ParameterDataManager::getInstance()->mercuryParams->getIsUsingStaging();

    addTrueFalseToggleButton("isUsingStaging : ", isUsingStaging, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_IS_USING_STAGING);
}

void MercuryParamScene::showSetLogParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SetLog -");
    setCompletionHandler = completionHandler;

    int isLog = ParameterDataManager::getInstance()->mercuryParams->getIsLog();

    addTrueFalseToggleButton("isLog : ", isLog, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_IS_LOG);
}

void MercuryParamScene::showGetBadgeParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- GetBadge -");
    setCompletionHandler = completionHandler;

    int badgeTarget = ParameterDataManager::getInstance()->mercuryParams->getBadgeTarget();
    MERCURY_BADGE_TARGET badgeTargetEnum = static_cast<MERCURY_BADGE_TARGET>(badgeTarget);
    long index = find(badgeTargetList.begin(), badgeTargetList.end(), badgeTargetToString(badgeTargetEnum)) - badgeTargetList.begin();

    addCombobox("badge Target : ", badgeTargetList, index, COMBOBOX_BADGE_TARGET);

}

void MercuryParamScene::showGetBadgeTypeParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- GetBadgeType -");
    setCompletionHandler = completionHandler;

    int badgeTarget = ParameterDataManager::getInstance()->mercuryParams->getBadgeTarget();
    MERCURY_BADGE_TARGET badgeTargetEnum = static_cast<MERCURY_BADGE_TARGET>(badgeTarget);
    long index = find(badgeTargetList.begin(), badgeTargetList.end(), badgeTargetToString(badgeTargetEnum)) - badgeTargetList.begin();

    addCombobox("badge Target : ", badgeTargetList, index, COMBOBOX_BADGE_TARGET);
}

void MercuryParamScene::showRequestUAInfoParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- RequestUAInfo -");
    setCompletionHandler = completionHandler;

    string uid = ParameterDataManager::getInstance()->commonParams->getUid();
    string additionalInfo = ParameterDataManager::getInstance()->mercuryParams->getAdditionalInfo();

    addEditBox("uid : ", uid.c_str(), "", EDITBOX_UID);
    addEditBox("additionalInfo : ", additionalInfo.c_str(), "", EDITBOX_ADDITIONAL_INFO);
}

void MercuryParamScene::showRequestUALinkShareParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- RequestUALinkShare -");
    setCompletionHandler = completionHandler;

    string urlLink = ParameterDataManager::getInstance()->mercuryParams->getUrlLink();
    string shareMessage = ParameterDataManager::getInstance()->mercuryParams->getShareMessage();

    addEditBox("urlLink : ", urlLink.c_str(), "", EDITBOX_URL_LINK);
    addEditBox("shareMessage : ", shareMessage.c_str(), "", EDITBOX_SHARE_MESSAGE);
}

void MercuryParamScene::showGetBannerInfoParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- GetBannerInfo -");
    setCompletionHandler = completionHandler;

    string campaignType = ParameterDataManager::getInstance()->mercuryParams->getCampaignType();
    long cIndex = find(campaignTypeList.begin(), campaignTypeList.end(), campaignType) - campaignTypeList.begin();

    string bannerType = ParameterDataManager::getInstance()->mercuryParams->getBannerType();
    long bIndex = find(bannerTypeList.begin(), bannerTypeList.end(), bannerType) - bannerTypeList.begin();

    addCombobox("campaignType : ", campaignTypeList, cIndex, COMBOBOX_CAMPAIGN_TYPE);
    addCombobox("bannerType : ", bannerTypeList, bIndex, COMBOBOX_BANNER_TYPE);
}

string MercuryParamScene::showTypeToString(MERCURY_SHOW_TYPE value)
{
    switch (value) {
        case MERCURY_SHOW_EVENT_LEGACY:
            return "MERCURY_SHOW_EVENT_LEGACY";
        case MERCURY_SHOW_EVENT:
            return "MERCURY_SHOW_EVENT";
        case MERCURY_SHOW_NOTICE_TOP:
            return "MERCURY_SHOW_NOTICE_TOP";
        case MERCURY_SHOW_NOTICE_BOTTOM:
            return "MERCURY_SHOW_NOTICE_BOTTOM";
        case MERCURY_SHOW_FORCED_NOTICE_TOP:
            return "MERCURY_SHOW_FORCED_NOTICE_TOP";
        case MERCURY_SHOW_FORCED_NOTICE_BOTTOM:
            return "MERCURY_SHOW_FORCED_NOTICE_BOTTOM";
        case MERCURY_SHOW_NOTICE_ONLY:
            return "MERCURY_SHOW_NOTICE_ONLY";
        case MERCURY_SHOW_FORCED_NOTICE_ONLY:
            return "MERCURY_SHOW_FORCED_NOTICE_ONLY";
        default:
            return "UNKNOWN";
    }
}

MERCURY_SHOW_TYPE MercuryParamScene::stringToShowType(string value)
{
    if (value == "MERCURY_SHOW_EVENT_LEGACY") {
        return MERCURY_SHOW_EVENT_LEGACY;
    }
    else if (value == "MERCURY_SHOW_EVENT") {
        return MERCURY_SHOW_EVENT;
    }
    else if (value == "MERCURY_SHOW_NOTICE_TOP") {
        return MERCURY_SHOW_NOTICE_TOP;
    }
    else if (value == "MERCURY_SHOW_NOTICE_BOTTOM"){
        return MERCURY_SHOW_NOTICE_BOTTOM;
    }
    else if (value == "MERCURY_SHOW_FORCED_NOTICE_TOP"){
        return MERCURY_SHOW_FORCED_NOTICE_TOP;
    }
    else if (value == "MERCURY_SHOW_FORCED_NOTICE_BOTTOM"){
        return MERCURY_SHOW_FORCED_NOTICE_BOTTOM;
    }
    else if (value == "MERCURY_SHOW_NOTICE_ONLY"){
        return MERCURY_SHOW_NOTICE_ONLY;
    }
    else {
        return MERCURY_SHOW_FORCED_NOTICE_ONLY;
    }
}

string MercuryParamScene::badgeTargetToString(MERCURY_BADGE_TARGET value)
{
    switch (value) {
        case MERCURY_BADGE_FOR_EVENT:
            return "MERCURY_BADGE_FOR_EVENT";
        case MERCURY_BADGE_FOR_NOTICE:
            return "MERCURY_BADGE_FOR_NOTICE";
        case MERCURY_BADGE_FOR_NOTICE_ONLY:
            return "MERCURY_BADGE_FOR_NOTICE_ONLY";
        case MERCURY_BADGE_FOR_NONE:
            return "MERCURY_BADGE_FOR_NONE";
        case MERCURY_BADGE_FOR_CUSTOM_BOARD_BASE:
            return "MERCURY_BADGE_FOR_CUSTOM_BOARD_BASE";
        case MERCURY_BADGE_FOR_CUSTOM_VIEW_BASE:
            return "MERCURY_BADGE_FOR_CUSTOM_VIEW_BASE";
        case MERCURY_BADGE_FOR_CUSTOM_VIEW_MAX:
            return "MERCURY_BADGE_FOR_CUSTOM_VIEW_MAX";
        default:
            return "UNKNOWN";
    }
}


MERCURY_BADGE_TARGET MercuryParamScene::stringToBadgeTarget(string value)
{
    if (value == "MERCURY_BADGE_FOR_EVENT") {
        return MERCURY_BADGE_FOR_EVENT;
    }
    else if (value == "MERCURY_BADGE_FOR_NOTICE") {
        return MERCURY_BADGE_FOR_NOTICE;
    }
    else if (value == "MERCURY_BADGE_FOR_NOTICE_ONLY") {
        return MERCURY_BADGE_FOR_NOTICE_ONLY;
    }
    else if (value == "MERCURY_BADGE_FOR_NONE") {
        return MERCURY_BADGE_FOR_NONE;
    }
    else if (value == "MERCURY_BADGE_FOR_CUSTOM_BOARD_BASE") {
        return MERCURY_BADGE_FOR_CUSTOM_BOARD_BASE;
    }
    else if (value == "MERCURY_BADGE_FOR_CUSTOM_VIEW_BASE") {
        return MERCURY_BADGE_FOR_CUSTOM_VIEW_BASE;
    }
    else {
        return MERCURY_BADGE_FOR_CUSTOM_VIEW_MAX;
    }
}
