//
//  OfferwallScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by sjyong on 2021/09/27.
//


#include "C2SModuleAPIList.hpp"
#include "OfferwallScene.hpp"
#include "OfferwallParamScene.hpp"
#include "ParameterDataManager.hpp"
#include "ui/CocosGUI.h"

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;

#include <map>
#include <sstream>

using namespace::std;

bool OfferwallScene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- Offerwall -");


//    addButton("Initialize", ButtonMode::NEED_PARAM, C2SModuleAPIList::OFFERWALL_INITIALIZE);  // removed, C2S.19.6
    addButton("InitializeEx", ButtonMode::NEED_PARAM, C2SModuleAPIList::OFFERWALL_INITIALIZE_EX);
//    addButton("SetLog", ButtonMode::NEED_PARAM, C2SModuleAPIList::OFFERWALL_SET_LOG);     // removed, C2S.20.0
    addButton("ShowEx", ButtonMode::NEED_PARAM, C2SModuleAPIList::OFFERWALL_SHOW_EX);
//    addButton("RewardFinish", C2SModuleAPIList::OFFERWALL_REWARD_FINISH);     // removed C2S.19.6
    addButton("GetState", C2SModuleAPIList::OFFERWALL_GET_STATE);
    addButton("GetVersion", C2SModuleAPIList::OFFERWALL_GET_VERSION);

    scheduleUpdate();

    return true;
}

void OfferwallScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void OfferwallScene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        OfferwallParamScene* paramScene = OfferwallParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    C2SModuleAPIList::OfferwallAPI api = static_cast<C2SModuleAPIList::OfferwallAPI>(buttonTag);
    switch (api) {
        case C2SModuleAPIList::OFFERWALL_INITIALIZE:
        {
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
            string uid = ParameterDataManager::getInstance()->offerwallParams->getUid();
            int isUsingStaging = ParameterDataManager::getInstance()->offerwallParams->getIsUsingStaging();

            CS_OfferwallInitialize(uid.c_str(), isUsingStaging, [](OFFERWALL_RESULT result){

                stringstream log;
                log << "result : " << OfferwallResultToString(result) <<'\n';

                addCallbackLogMessage(log.str(), "OfferwallCB : ");
            });

            addLogMessage("CS_OfferwallInitialize called.");

#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
            addLogMessage("CS_OfferwallInitialize (Android only)");
#endif
            break;
        }
        case C2SModuleAPIList::OFFERWALL_INITIALIZE_EX:
        {

#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
            string uid = ParameterDataManager::getInstance()->offerwallParams->getUid();
            int isUsingStaging = ParameterDataManager::getInstance()->offerwallParams->getIsUsingStaging();

            CS_OfferwallInitializeEx(uid.c_str(), isUsingStaging, [](OFFERWALL_RESULT result){

                stringstream log;
                log << "result : " << OfferwallResultToString(result) <<'\n';

                addCallbackLogMessage(log.str(), "OfferwallCB : ");

            }, [](int error, const char* errorMessage, int result, const char* eventID, const char* assetCode, int assetAmount){

                stringstream log;
                log << "errorMessage : " << errorMessage << '\n';
                log << "result : " << result << '\n';
                log << "eventID : " << eventID << '\n';
                log << "assetCode : " << assetCode << '\n';
                log << "assetAmount : " << assetAmount << '\n';

                addCallbackLogMessage(log.str(), "OfferwallRewardCB : ");
            });

            addLogMessage("CS_OfferwallInitializeEx called.");

#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
            addLogMessage("CS_OfferwallInitializeEx (Android only)");
#endif
            break;
        }
        case C2SModuleAPIList::OFFERWALL_SET_LOG:
        {
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
            int isLog = ParameterDataManager::getInstance()->offerwallParams->getIsLog();

//            CS_OfferwallSetLog(isLog);

            addLogMessage("CS_OfferwallSetLog called.");
            addLogMessage("CS_OfferwallSetLog deprecated.");

#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
            addLogMessage("CS_OfferwallSetLog (Android only)");
#endif
            break;
        }
        case C2SModuleAPIList::OFFERWALL_SHOW_EX:
        {
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
            string additionalInfo = ParameterDataManager::getInstance()->offerwallParams->getAdditionalInfo();

            CS_OfferwallShowEx(additionalInfo.c_str());

            addLogMessage("CS_OfferwallShowEx called.");

#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
            addLogMessage("CS_OfferwallShowEx (Android only)");
#endif
            break;
        }
        case C2SModuleAPIList::OFFERWALL_REWARD_FINISH:
        {
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
//            CS_OfferwallRewardFinish();

            addLogMessage("CS_OfferwallRewardFinish called.");
            addLogMessage("( CS_OfferwallRewardFinish API는 KS C2S에서 더이상 작동하지 않습니다.(deprecated) )");

#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
            addLogMessage("CS_OfferwallRewardFinish (Android only)");
#endif
            break;
        }
        case C2SModuleAPIList::OFFERWALL_GET_STATE:
        {
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
            int state = CS_OfferwallGetState();

            stringstream log;
            log << "CS_OfferwallGetState called. state : " << state << '\n';
            addLogMessage(log.str());
            
#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
            addLogMessage("CS_OfferwallGetState (Android only)");
#endif

            break;
        }
        case C2SModuleAPIList::OFFERWALL_GET_VERSION:
        {
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
            string version = CS_OfferwallGetVersion();

            stringstream log;
            log << "CS_OfferwallGetState called. version : " << version << '\n';

            addLogMessage(log.str());

#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
            addLogMessage("CS_OfferwallGetState (Android only)");
#endif
        }
        default:
            break;
    }
}

string OfferwallScene::OfferwallResultToString(OFFERWALL_RESULT result)
{
    switch (result) {
        case OFFERWALL_OPEN:
            return "OFFERWALL_OPEN";
        case OFFERWALL_CLOSE:
            return "OFFERWALL_CLOSE";
        case OFFERWALL_NETWORK_DISCONNECT:
            return "OFFERWALL_NETWORK_DISCONNECT";
        case OFFERWALL_ACTIVATION:
            return "OFFERWALL_ACTIVATION";
        case OFFERWALL_DEACTIVATION:
            return "OFFERWALL_DEACTIVATION";
        default:
            return "UNKNOWN";
    }
}


