#ifdef __ANDROID__
#include <string.h>
#include <jni.h>
#include "C2SHIVEPromotion.h"
#include <android/log.h>

static JavaVM* g_vm;
static jobject g_obj;
static jclass C2SModuleHivePromotionClass;

CSPromotionViewHandler g_promoViewHandler = 0;
CSPromotionInfoHandler g_promoInfoHandler = 0;
CSPromotionBadgeHandler g_promoBadgeHandler = 0;
CSPromotionBannerHandler g_promotionBannerHandler = 0;
CSPromotionAppInvitationDataHandler g_promotionAppInvitationDataHandler = 0;
CSPromotionShareHandler g_promotionShareHandler = 0;


#define TAG "HivePromotion"
#define LOGV(...) __android_log_print(ANDROID_LOG_VERBOSE, "HivePromotion", __VA_ARGS__)
#define LOGD(...) __android_log_print(ANDROID_LOG_DEBUG  , "HivePromotion", __VA_ARGS__)
#define LOGI(...) __android_log_print(ANDROID_LOG_INFO   , "HivePromotion", __VA_ARGS__)
#define LOGW(...) __android_log_print(ANDROID_LOG_WARN   , "HivePromotion", __VA_ARGS__)
#define LOGE(...) __android_log_print(ANDROID_LOG_ERROR  , "HivePromotion", __VA_ARGS__)

/**
 * jstring type to Char
 */
char* jstringToCharForHivePromotion(JNIEnv* env, jstring str)
{
	char* cDesc = strdup( (*env)->GetStringUTFChars(env, str, 0) );
	return cDesc;
}

JNIEXPORT void JNICALL
Java_com_com2us_module_hivepromotion_PromotionJNI_nativeHivePromotionViewCallback(JNIEnv* env, jobject obj, jint errorCode, jint viewResult)
{
	if(g_promoViewHandler)
	{
		g_promoViewHandler(errorCode, viewResult);
	}
}

JNIEXPORT void JNICALL
Java_com_com2us_module_hivepromotion_PromotionJNI_nativeHivePromotionInfoCallback(JNIEnv* env, jobject obj, jint errorCode, jstring jsonData)
{
	if(g_promoInfoHandler)
	{
		char *jsonDataString = jstringToCharForHivePromotion(env, jsonData);
		g_promoInfoHandler(errorCode, jsonDataString);
		(*env)->ReleaseStringUTFChars(env, jsonData, jsonDataString);
	}
}

JNIEXPORT void JNICALL
Java_com_com2us_module_hivepromotion_PromotionJNI_nativeHivePromotionBadgeCallback(JNIEnv* env, jobject obj, jint errorCode, jstring jsonData)
{
	if(g_promoBadgeHandler)
	{
		char *jsonDataString = jstringToCharForHivePromotion(env, jsonData);
		g_promoBadgeHandler(errorCode, jsonDataString);
		(*env)->ReleaseStringUTFChars(env, jsonData, jsonDataString);
	}
}

JNIEXPORT void JNICALL
Java_com_com2us_module_hivepromotion_PromotionJNI_nativeHivePromotionBannerInfoCallback(JNIEnv* env, jobject obj, jint errorCode, jstring jsonData)
{
	if(g_promotionBannerHandler)
	{
		char *jsonDataString = jstringToCharForHivePromotion(env, jsonData);
		g_promotionBannerHandler(errorCode, jsonDataString);
		(*env)->ReleaseStringUTFChars(env, jsonData, jsonDataString);
	}
}

JNIEXPORT void JNICALL
Java_com_com2us_module_hivepromotion_PromotionJNI_nativeHivePromotionUAAppInvitationDataCallback(JNIEnv* env, jobject obj, jint errorCode, jstring jsonData)
{
	if(g_promotionAppInvitationDataHandler)
	{
		char *jsonDataString = jstringToCharForHivePromotion(env, jsonData);
		g_promotionAppInvitationDataHandler(errorCode, jsonDataString);
		(*env)->ReleaseStringUTFChars(env, jsonData, jsonDataString);
	}
}

JNIEXPORT void JNICALL
Java_com_com2us_module_hivepromotion_PromotionJNI_nativeHivePromotionUAShareCallback(JNIEnv* env, jobject obj, jint errorCode)
{
	if(g_promotionShareHandler)
	{
		g_promotionShareHandler(errorCode);
	}
}


/**
 * initialize HivePromotion
 */
JNIEXPORT void JNICALL
Java_com_com2us_module_hivepromotion_PromotionJNI_nativeHivePromotionInitialize(JNIEnv* env, jobject thiz, jobject obj)
{
    (*env)->GetJavaVM(env, &g_vm);
    g_obj = (*env)->NewGlobalRef(env, obj);
    C2SModuleHivePromotionClass = (*env)->NewGlobalRef(env, (*env)->GetObjectClass(env, obj));
}


/** ** ** **
 * API
 */


void CS_PromotionInitialize()
{
    JNIEnv* env;
    jmethodID method;

    if(JNI_OK == (*g_vm)->GetEnv(g_vm, (void **)&env, JNI_VERSION_1_6))
    {
        method = (*env)->GetStaticMethodID(env, C2SModuleHivePromotionClass, "nativePromotionInit", "()V");
        if(0 != method && C2SModuleHivePromotionClass != 0)
        {
            (*env)->CallStaticVoidMethod(env, C2SModuleHivePromotionClass, method);
        }
    }
}

void CS_PromotionShow(CSPromotionViewType type, int isForced, const char* uid, CSPromotionViewHandler handler)
{
	JNIEnv* env = 0;
	jmethodID method;

	g_promoViewHandler = handler;

	if(JNI_OK == (*g_vm)->GetEnv(g_vm, (void **)&env, JNI_VERSION_1_6))
	{
		method = (*env)->GetStaticMethodID(env, C2SModuleHivePromotionClass, "nativeShowPromotion", "(IILjava/lang/String;)V");
		if(0 != method)
		{
			jint jtype = (jint)type;
			jstring juid = (*env)->NewStringUTF(env, uid);

			(*env)->CallStaticVoidMethod(env, C2SModuleHivePromotionClass, method, jtype, isForced, juid);
			// memory free
			(*env)->DeleteLocalRef(env, juid);
		}
	}
}

void CS_PromotionShowCustom(CSPromotionCustomType type, const char* contents, const char* uid, CSPromotionViewHandler handler)
{
	JNIEnv* env = 0;
	jmethodID method;

    g_promoViewHandler = handler;

	if(JNI_OK == (*g_vm)->GetEnv(g_vm, (void **)&env, JNI_VERSION_1_6))
	{
		method = (*env)->GetStaticMethodID(env, C2SModuleHivePromotionClass, "nativeShowCustomContents", "(ILjava/lang/String;Ljava/lang/String;)V");
		if(0 != method)
		{
			jint jtype = (jint)type;
			jstring jcontents = (*env)->NewStringUTF(env, contents);
			jstring juid = (*env)->NewStringUTF(env, uid);

			(*env)->CallStaticVoidMethod(env, C2SModuleHivePromotionClass, method, jtype, jcontents, juid);

			// memory free
			(*env)->DeleteLocalRef(env, jcontents);
			(*env)->DeleteLocalRef(env, juid);
		}
	}
}

void CS_PromotionShowCustomOnGameWindow(CSPromotionCustomType type, const char* contents, const char* uid, CSPromotionViewHandler handler)
{
	JNIEnv* env = 0;
	jmethodID method;

	g_promoViewHandler = handler;

	if(JNI_OK == (*g_vm)->GetEnv(g_vm, (void **)&env, JNI_VERSION_1_6))
	{
		method = (*env)->GetStaticMethodID(env, C2SModuleHivePromotionClass, "nativeShowCustomContentsOnGameWindow", "(ILjava/lang/String;Ljava/lang/String;)V");
		if(0 != method)
		{
			jint jtype = (jint)type;
			jstring jcontents = (*env)->NewStringUTF(env, contents);
			jstring juid = (*env)->NewStringUTF(env, uid);

			(*env)->CallStaticVoidMethod(env, C2SModuleHivePromotionClass, method, jtype, jcontents, juid);

			// memory free
			(*env)->DeleteLocalRef(env, jcontents);
			(*env)->DeleteLocalRef(env, juid);
		}
	}
}

CSOfferwallState CS_GetOfferwallState()
{
    JNIEnv* env;
    jmethodID method;

    if(JNI_OK == (*g_vm)->GetEnv(g_vm, (void **)&env, JNI_VERSION_1_6))
    {
        method = (*env)->GetStaticMethodID(env, C2SModuleHivePromotionClass, "nativeGetOfferwallState", "()I");
        if(0 != method)
        {
            int retVal = (*env)->CallStaticIntMethod(env, C2SModuleHivePromotionClass, method);

            return retVal;
        }
    }
    return 1;
}


void CS_ShowOfferwall(const char* uid, CSPromotionViewHandler handler)
{
	JNIEnv* env;
	jmethodID method;

    g_promoViewHandler = handler;

	if(JNI_OK == (*g_vm)->GetEnv(g_vm, (void **)&env, JNI_VERSION_1_6))
	{
		method = (*env)->GetStaticMethodID(env, C2SModuleHivePromotionClass, "nativeShowOfferwall", "(Ljava/lang/String;)V");

		if(0 != method)
		{
			jstring juid = (*env)->NewStringUTF(env, uid);

			(*env)->CallStaticVoidMethod(env, C2SModuleHivePromotionClass, method, juid);

			// memory free
			(*env)->DeleteLocalRef(env, juid);
		}
	}
}

void CS_ShowReview(const char* uid, CSPromotionViewHandler handler)
{
	JNIEnv* env;
	jmethodID method;

    g_promoViewHandler = handler;

	if(JNI_OK == (*g_vm)->GetEnv(g_vm, (void **)&env, JNI_VERSION_1_6))
	{
		method = (*env)->GetStaticMethodID(env, C2SModuleHivePromotionClass, "nativeShowReview", "(Ljava/lang/String;)V");

		if(0 != method)
		{
			jstring juid = (*env)->NewStringUTF(env, uid);

			(*env)->CallStaticVoidMethod(env, C2SModuleHivePromotionClass, method, juid);

			// memory free
			(*env)->DeleteLocalRef(env, juid);
		}
	}
}

void CS_ShowNativeReview(const char* uid)
{
    JNIEnv* env;
    jmethodID method;

    if(JNI_OK == (*g_vm)->GetEnv(g_vm, (void **)&env, JNI_VERSION_1_6))
    {
        method = (*env)->GetStaticMethodID(env, C2SModuleHivePromotionClass, "nativeShowNativeReview", "(Ljava/lang/String;)V");

        if(0 != method)
        {
            jstring juid = (*env)->NewStringUTF(env, uid);

            (*env)->CallStaticVoidMethod(env, C2SModuleHivePromotionClass, method, juid);

            // memory free
            (*env)->DeleteLocalRef(env, juid);
        }
    }
}

void CS_ShowExit(const char* uid, CSPromotionViewHandler handler)
{
	JNIEnv* env;
	jmethodID method;

    g_promoViewHandler = handler;

	if(JNI_OK == (*g_vm)->GetEnv(g_vm, (void **)&env, JNI_VERSION_1_6))
	{
		method = (*env)->GetStaticMethodID(env, C2SModuleHivePromotionClass, "nativeShowExit", "(Ljava/lang/String;)V");

		if(0 != method)
		{
			jstring juid = (*env)->NewStringUTF(env, uid);

			(*env)->CallStaticVoidMethod(env, C2SModuleHivePromotionClass, method, juid);

			// memory free
			(*env)->DeleteLocalRef(env, juid);
		}
	}
}



/**
 *
 */
void CS_GetViewInfo(CSPromotionCustomType type, const char* contents, const char* uid, CSPromotionInfoHandler handler)
{
	JNIEnv* env;
	jmethodID method;

    g_promoInfoHandler = handler;

	if(JNI_OK == (*g_vm)->GetEnv(g_vm, (void **)&env, JNI_VERSION_1_6))
	{
		method = (*env)->GetStaticMethodID(env, C2SModuleHivePromotionClass, "nativeGetViewInfo", "(ILjava/lang/String;Ljava/lang/String;)V");

		if(0 != method)
		{
			jint jtype = (jint)type;
			jstring jcontents = (*env)->NewStringUTF(env, contents);
			jstring juid = (*env)->NewStringUTF(env, uid);

			(*env)->CallStaticVoidMethod(env, C2SModuleHivePromotionClass, method, jtype, jcontents, juid);

			// memory free
			(*env)->DeleteLocalRef(env, jcontents);
			(*env)->DeleteLocalRef(env, juid);
		}
	}
}

void CS_GetBadgeInfo(const char* uid, CSPromotionBadgeHandler handler)
{
	JNIEnv* env;
	jmethodID method;

	g_promoBadgeHandler = handler;

	if(JNI_OK == (*g_vm)->GetEnv(g_vm, (void **)&env, JNI_VERSION_1_6))
	{
		method = (*env)->GetStaticMethodID(env, C2SModuleHivePromotionClass, "nativeGetBadgeInfo", "(Ljava/lang/String;)V");
		if(0 != method)
		{
			jstring juid = (*env)->NewStringUTF(env, uid);

			(*env)->CallStaticVoidMethod(env, C2SModuleHivePromotionClass, method, juid);

			// memory free
			(*env)->DeleteLocalRef(env, juid);
		}
	}
}

void CS_PromotionGetBannerInfo(const char* campaignType, const char* bannerType, const char* uid, CSPromotionBannerHandler handler)
{
    JNIEnv* env;
    jmethodID method;

    g_promotionBannerHandler = handler;

    if(JNI_OK == (*g_vm)->GetEnv(g_vm, (void **)&env, JNI_VERSION_1_6))
    {
        method = (*env)->GetStaticMethodID(env, C2SModuleHivePromotionClass, "nativeGetBannerInfo", "(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V");
        if(0 != method)
        {
            jstring jcampaignType = (*env)->NewStringUTF(env, campaignType);
            jstring jbannerType = (*env)->NewStringUTF(env, bannerType);
            jstring juid = (*env)->NewStringUTF(env, uid);

            (*env)->CallStaticVoidMethod(env, C2SModuleHivePromotionClass, method, jcampaignType, jbannerType, juid);

            // memory free
            (*env)->DeleteLocalRef(env, jcampaignType);
            (*env)->DeleteLocalRef(env, jbannerType);
            (*env)->DeleteLocalRef(env, juid);
        }
    }
}

void CS_PromotionGetAppInvitationData(const char* uid, const char* additionalInfo, CSPromotionAppInvitationDataHandler handler)
{
    JNIEnv* env;
    jmethodID method;

    g_promotionAppInvitationDataHandler = handler;

    if(JNI_OK == (*g_vm)->GetEnv(g_vm, (void **)&env, JNI_VERSION_1_6))
    {
        method = (*env)->GetStaticMethodID(env, C2SModuleHivePromotionClass, "nativeGetAppInvitationData", "(Ljava/lang/String;Ljava/lang/String;)V");
        if(0 != method)
        {
            jstring juid = (*env)->NewStringUTF(env, uid);
            jstring jadditionalInfo = (*env)->NewStringUTF(env, additionalInfo);

            (*env)->CallStaticVoidMethod(env, C2SModuleHivePromotionClass, method, juid, jadditionalInfo);

            // memory free
            (*env)->DeleteLocalRef(env, juid);
            (*env)->DeleteLocalRef(env, jadditionalInfo);
        }
    }
}

void CS_PromotionShowUAShare(const char* inviteMessage, const char* inviteLink, CSPromotionShareHandler handler)
{
    JNIEnv* env;
    jmethodID method;

    g_promotionShareHandler = handler;

    if(JNI_OK == (*g_vm)->GetEnv(g_vm, (void **)&env, JNI_VERSION_1_6))
    {
        method = (*env)->GetStaticMethodID(env, C2SModuleHivePromotionClass, "nativeShowUAShare", "(Ljava/lang/String;Ljava/lang/String;)V");
        if(0 != method)
        {
            jstring jinviteMessage = (*env)->NewStringUTF(env, inviteMessage);
            jstring jinviteLink = (*env)->NewStringUTF(env, inviteLink);

            (*env)->CallStaticVoidMethod(env, C2SModuleHivePromotionClass, method, jinviteMessage, jinviteLink);

            // memory free
            (*env)->DeleteLocalRef(env, jinviteMessage);
            (*env)->DeleteLocalRef(env, jinviteLink);
        }
    }
}

void CS_SetAdditionalInfo(const char* additionalInfo)
{
	JNIEnv* env;
	jmethodID method;

	if(JNI_OK == (*g_vm)->GetEnv(g_vm, (void **)&env, JNI_VERSION_1_6))
	{
		method = (*env)->GetStaticMethodID(env, C2SModuleHivePromotionClass, "nativeSetAdditionalInfo", "(Ljava/lang/String;)V");
		if(0 != method)
		{
			jstring jadditionalInfo = (*env)->NewStringUTF(env, additionalInfo);

			(*env)->CallStaticVoidMethod(env, C2SModuleHivePromotionClass, method, jadditionalInfo);

			// memory free
			(*env)->DeleteLocalRef(env, jadditionalInfo);
		}
	}
}

void CS_UpdatePromotionData()
{
    JNIEnv* env = 0;
    jmethodID method;

    if(JNI_OK == (*g_vm)->GetEnv(g_vm, (void **)&env, JNI_VERSION_1_6))
    {
        method = (*env)->GetStaticMethodID(env, C2SModuleHivePromotionClass, "nativeUpdatePromotionData", "()V");
        if(0 != method)
        {
            (*env)->CallStaticVoidMethod(env, C2SModuleHivePromotionClass, method);
        }
    }
}

const char* CS_HivePromotionGetVersion()
{
	JNIEnv* env = 0;
	jmethodID method;

	jstring jversion = 0;
	char* ret = "0";

	if(JNI_OK == (*g_vm)->GetEnv(g_vm, (void **)&env, JNI_VERSION_1_6))
	{
		method = (*env)->GetStaticMethodID(env, C2SModuleHivePromotionClass, "nativeHivePromotionGetVersion", "()Ljava/lang/String;");
		if(0 != method)
		{
			jversion = (jstring)(*env)->CallStaticObjectMethod(env, C2SModuleHivePromotionClass, method);
		}
	}

	if(jversion != 0) {
		ret = strdup( (*env)->GetStringUTFChars(env, jversion, 0) );
		(*env)->DeleteLocalRef(env, jversion);
		/* (ret == '\0') == (ret == NULL) == (ret == 0) */
		if (ret != NULL)
		{
			if (ret[0] == '\0')
			{
				free(ret);
				ret = 0;    /* or ret = NULL; */
			}
		}
	}

	return ret;
}
#endif
