/**
 * C2SHIVEPromotion.h
 * C2SModule HIVEPromotion
 *
 * Created by joungdaun on 2017. 5. 16.
 * Copyright © 2020~ GCP. All rights reserved.
 *
 * Last Update : C2S.21.0.2
 */

#ifndef __C2SPROMOTION_H__
#define __C2SPROMOTION_H__

#ifdef __cplusplus
extern "C" {
#endif

    /**
     * 사용자에게 노출할 프로모션 뷰의 타입이다.<br/>
     *  showPromotion API의 인자로 사용한다.
     */
    typedef enum _CSPromotionViewType{
        CS_PROMOTION_BANNERLEGACY   = -1,       // 전면배너 (웹뷰만 사용하는 구 버전)
        CS_PROMOTION_BANNER         = 0,        // 전면 배너
        CS_PROMOTION_NEWS,                      // 새소식 페이지
        CS_PROMOTION_NOTICE,                    // HIVE 공지사항
    } CSPromotionViewType;
    

    /**
     * 사용자에게 노출할 커스텀 컨텐츠의 타입이다.<br/>
     *  showCustomContents API의 인자로 사용한다.
     */
    typedef enum _CSPromotionCustomType{
        CS_PROMOTION_VIEW           = 0,            // 커스텀 뷰
        CS_PROMOTION_BOARD,                         // 커스텀 보드
        CS_PROMOTION_SPOT,                          // 스팟 배너
        CS_PROMOTION_DIRECT                         // 다이렉트 뷰
    } CSPromotionCustomType;
    
    /**
     * 프로모션 뷰의 Open/Close 결과에 대한 타입이다.
     */
    typedef enum _CSPromotionResult{
        CS_PROMOTION_RESULT_NEWS_OPEN = 0,        		// 프로모션 뷰(NEWS)가 열렸을 때
        CS_PROMOTION_RESULT_NEWS_CLOSE,        			// 프로모션 뷰(NEWS)가 닫혔을 때
        
        CS_PROMOTION_RESULT_NOTICE_OPEN,        		// 프로모션 뷰(NOTICE)가 열렸을 때
        CS_PROMOTION_RESULT_NOTICE_CLOSE,        		// 프로모션 뷰(NOTICE)가 닫혔을 때
        
        CS_PROMOTION_RESULT_BANNER_OPEN,        		// 프로모션 뷰(BANNER)가 열렸을 때
        CS_PROMOTION_RESULT_BANNER_CLOSE,        		// 프로모션 뷰(BANNER)가 닫혔을 때
        
        CS_PROMOTION_RESULT_CUSTOM_VIEW_OPEN,        	// 프로모션 뷰(CUSTOM_VIEW)가 열렸을 때
        CS_PROMOTION_RESULT_CUSTOM_VIEW_CLOSE,        	// 프로모션 뷰(CUSTOM_VIEW)가 닫혔을 때
        
        CS_PROMOTION_RESULT_CUSTOM_BOARD_OPEN,        	// 프로모션 뷰(CUSTOM_BOARD)가 열렸을 때
        CS_PROMOTION_RESULT_CUSTOM_BOARD_CLOSE,        	// 프로모션 뷰(CUSTOM_BOARD)가 닫혔을 때
        
        CS_PROMOTION_RESULT_SPOT_OPEN,        			// 프로모션 뷰(SPOT)가 열렸을 때
        CS_PROMOTION_RESULT_SPOT_CLOSE,        			// 프로모션 뷰(SPOT)가 닫혔을 때
        
        CS_PROMOTION_RESULT_DIRECT_OPEN,        		// 프로모션 뷰(DIRECT)가 열렸을 때
        CS_PROMOTION_RESULT_DIRECT_CLOSE,        		// 프로모션 뷰(DIRECT)가 닫혔을 때
        
        CS_PROMOTION_RESULT_OFFERWALL_OPEN,             // 오퍼월 뷰가 열렸을 때
        CS_PROMOTION_RESULT_OFFERWALL_CLOSE,            // 오퍼월 뷰가 닫혔을 때
        
        CS_PROMOTION_RESULT_REVIEW_OPEN,        		// REVIEW 팝업이 열렸을 때
        CS_PROMOTION_RESULT_REVIEW_CLOSE,        		// REVIEW 팝업이 닫혔을 때
        
        CS_PROMOTION_RESULT_MORE_GAMES_OPEN,        	// MORE_GAMES 팝업이 열렸을 때 (AD only)
        CS_PROMOTION_RESULT_MORE_GAMES_CLOSE,        	// MORE_GAMES 팝업이 닫혔을 때 (AD only)
        CS_PROMOTION_RESULT_MORE_GAMES_EXIT,        	// MORE_GAMES 팝업 버튼중 종료 버튼을 눌렀을 때 (AD only)
        
        CS_PROMOTION_RESULT_START_PLAYBACK,             // 프로모션 동영상이 시작될때
        CS_PROMOTION_RESULT_FINISH_PLAYBACK,            // 프로모션 동영상이 종료될때
        
		CS_PROMOTION_RESULT_NEWS_GOBACK,				// 프로모션 뷰(NEWS)가 GoBack스킴 전달 받았을 때
		CS_PROMOTION_RESULT_NOTICE_GOBACK,				// 프로모션 뷰(NOTICE)가 GoBack스킴 전달 받았을 때
		CS_PROMOTION_RESULT_CUSTOM_VIEW_GOBACK,			// 프로모션 뷰(CUSTOM_VIEW)가 GoBack스킴 전달 받았을 때
		CS_PROMOTION_RESULT_CUSTOM_BOARD_GOBACK,		// 프로모션 보드(CUSTOM_BOARD)가 GoBack스킴 전달 받았을 때
		CS_PROMOTION_RESULT_SPOT_GOBACK,				// 프로모션 뷰(SPOT)가 GoBack 스킴 전달 받았을 때
		CS_PROMOTION_RESULT_DIRECT_GOBACK,				// 프로모션 뷰(DIRECT)가 GoBack 스킴 전달 받았을 때
		CS_PROMOTION_RESULT_OFFERWALL_GOBACK,			// 오퍼월 뷰가 GoBack 스킴 전달 받았을 때
    } CSPromotionResult;
    
    /**
     * 게임내 오퍼월을 호출할 수 있는 버튼의 노출 여부 상태
     */
    typedef enum _CSOfferwallState {
        CS_OFFERWALL_STATE_ENABLED      = 0,
        CS_OFFERWALL_STATE_DISABLED,
        CS_OFFERWALL_STATE_UNKNOWN,
    } CSOfferwallState;
    
    /**
     * 에러 코드
     */
    typedef enum _CSPromotionAPICallResult{
        CS_PROMOTION_SUCCESS            = 0,
        CS_PROMOTION_INVALID_PARAM      = -1,
        CS_PROMOTION_NOT_SUPPORTED      = -2,
        CS_PROMOTION_IN_PROGRESS        = -3,
        CS_PROMOTION_TIMEOUT            = -4,
        CS_PROMOTION_NETWORK            = -5,
        CS_PROMOTION_CANCELED           = -6,
        CS_PROMOTION_NEED_INITIALIZE    = -7,
        CS_PROMOTION_RESPONSE_FAIL		= -8,
        CS_PROMOTION_INVALID_SESSION	= -9,
        CS_PROMOTION_NEED_RESTORE		= -10,
        CS_PROMOTION_RESTORE_NOT_OWNED	= 10,
        CS_PROMOTION_UNKNOWN			= -99
        
    } CSPromotionAPICallResult;

    /****
     * 콜백
     ****/

    /**
     * @brief 프로모션 뷰 Show계열 API 결과 통지
     */
    typedef void (*CSPromotionViewHandler)(CSPromotionAPICallResult result, CSPromotionResult resultType);
    
    /**
     * @brief 프로모션 뷰의 정보 조회 결과 통지
     */
    typedef void (*CSPromotionInfoHandler)(CSPromotionAPICallResult result, const char* viewInfos);
    
    /**
     * @brief 프로모션 배지 정보 조회 결과 통지
     */
    typedef void (*CSPromotionBadgeHandler)(CSPromotionAPICallResult result, const char* badgeInfos);

    /**
     * @brief 프로모션 배너 정보 조회 결과 통지
     */
    typedef void (*CSPromotionBannerHandler)(CSPromotionAPICallResult result, const char* bannerInfos);

    /**
     * @brief 프로모션 집객 App invitation 정보 조회 결과 통지
     */
    typedef void (*CSPromotionAppInvitationDataHandler)(CSPromotionAPICallResult result, const char* appInvitationData);

    /**
     * @brief 프로모션 집객 공유하기 결과 통지
     */
    typedef void (*CSPromotionShareHandler)(CSPromotionAPICallResult result);


    /****
     * API
     ****/

    /**
     * HIVE 프로모션 초기화.
     */
    void CS_PromotionInitialize(void);
    
    /**
     * 대배너, 새소식, 공지사항 웹뷰를 띄운다.
     */
    void CS_PromotionShow(CSPromotionViewType type, int isForced, const char* uid, CSPromotionViewHandler handler);
    
    /**
     * 메뉴 탭을 입력받아 새소식 페이지를 노출시킨다.
     */
    void CS_PromotionShowNews(const char* menu, const char* giftPidList, const char* uid, CSPromotionViewHandler handler);  // 추후 공개 예정 API

    /**
     * 커스텀 컨텐츠를 가진 웹뷰를 띄운다.
     */
    void CS_PromotionShowCustom(CSPromotionCustomType type, const char* contents, const char* uid, CSPromotionViewHandler handler);

    /**
     * 커스텀 컨텐츠를 가진 웹뷰를 Game Window에서 띄운다. (Direct 배너를 Android에서 dialog에 띄우기 위한 API)
     */
    void CS_PromotionShowCustomOnGameWindow(CSPromotionCustomType type, const char* contents, const char* uid, CSPromotionViewHandler handler);
    
    /**
     * 커스텀 컨텐츠를 가진 웹뷰를 띄운다. with UE Parameter
     * HivePromotion 4.2.3 update
     */
    void CS_PromotionShowCustomWithParams(CSPromotionCustomType type, const char* contents, const char* uid, const char* params, CSPromotionViewHandler handler);
    
    /**
     * @brief 게임내에서 무료 충전소를 노출하기 위한 버튼 UI 는 보여지거나 숨길 수 있도록 구성해야 한다.
     * 이 메서드는 게임내에서 오퍼월(무료 충전소) 을 호출 할 수 있는 버튼 노출 가능 여부를 반환한다.
     */
    CSOfferwallState CS_GetOfferwallState(void);
    
    /**
     * 무료 충전소 화면 노출
     */
    void CS_ShowOfferwall(const char* uid, CSPromotionViewHandler handler);
    
    /**
     * 무료 충전소 화면 노출 with UE Parameter
     * HivePromotion 4.2.3 update
     */
    void CS_ShowOfferwallWithParams(const char* uid, const char* params, CSPromotionViewHandler handler);
    
    /**
     * 리뷰 화면 노출
     */
    void CS_ShowReview(const char* uid, CSPromotionViewHandler handler);        // 4.2.0 deprecated
    
    /**
     * 네이티브 리뷰 화면 노출
     * Added in HivePromotion 4.2.0
     */
    void CS_ShowNativeReview(const char* uid);
    
    /**
     * 종료 팝업 노출
     */
    void CS_ShowExit(const char* uid, CSPromotionViewHandler handler);
    
    /**
     * 커스텀 컨텐츠에 대한 정보를 가져온다.
     * HivePromotion에서 제공하는 웹뷰를 게임에서 원하는 UI로 띄울 수 있다.
     */
    void CS_GetViewInfo(CSPromotionCustomType type, const char* contents, const char* uid, CSPromotionInfoHandler handler);
    
    /**
     * 프로모션 뷰의 뱃지 데이터를 가져온다.
     * 유저에게 새로운 정보가 있다는 것을 알려줄 수 있다.
     */
    void CS_GetBadgeInfo(const char* uid, CSPromotionBadgeHandler handler);

    /**
     * 프로모션 뷰의 배너 데이터를 가져온다.
     * @param campaignType "event", "notice", "all", 새로 정의 한 String
     * @param bannerType "great", "small", "rolling", 새로 정의 한 String
     */
    void CS_PromotionGetBannerInfo(const char* campaignType, const char* bannerType, const char* uid, CSPromotionBannerHandler handler);
    
    /**
     * 유저 초대 데이터를 가져온다.
     */
    void CS_PromotionGetAppInvitationData(const char* uid, const char* additionalInfo, CSPromotionAppInvitationDataHandler handler);

    /**
     * 유저 초대링크 공유 페이지를 띄운다.
     */
    void CS_PromotionShowUAShare(const char* inviteMessage, const char* inviteLink, CSPromotionShareHandler handler);

    /**
     * GMS등을 통해 별도의 가공 없이 게임 서버 API 또는 게임 서버의 중계용 API등 그에 준하는 API에 전달되는 값이다.
     */
    void CS_SetAdditionalInfo(const char* additionalInfo);

    /**
     * Promotion 새소식 페이지 등의 노출에 필요한 정보를 갱신한다.
     * Added in HivePromotion 4.4.0
     */
    void CS_UpdatePromotionData(void);

    /**
     * Hive Promotion 모듈 버전을 반환한다.
     */
    const char* CS_HivePromotionGetVersion(void);
    
    
#ifdef __cplusplus
};
#endif


#endif
