#include <stdbool.h>

//
//  C2SHub2.h
//  Peppermint
//
//  Created by Jahyun Oh on 7/24/12.            // Android
//  Created by Archaeo Lucid on 2014. 12. 10.   // iOS
//  Copyright (c) 2012 Com2uS. All rights reserved.
//
//  Last Update : C2S.21.0.2

#ifndef __C2S_HUB_2_H__
#define __C2S_HUB_2_H__

#ifdef __cplusplus
extern "C" {
#endif

typedef enum HiveSocialServerState
{
	HiveSocialServerLive	= 0,
	HiveSocialServerSandbox,
	HiveSocialServerTest,
	HiveSocialServerStaging
}HiveSocialServerState;
    
/**
 @abstract
 Hub(Hive) server로 부터의 응답 callback.
 @param jsonString server로부터 응답받은 json값.
 @param param 함수 호출시 입력한 callback parameter
 */
typedef void (*HubCallbackWithJSON) (const char *jsonString, void *param);


/**
 @abstract
 Hub(Hive) module을 초기화한다.
 @param appId Hub(Hive)등록시 받은 application id.
 @param serverState SERVER_STATE_TYPE(release, staging, sandbox)
 @return Hub error code.
 */
int CS_hubInitializeEx(
	const char *appId,
	HiveSocialServerState serverState
);

/**
 @abstract
 Hub(Hive)로 사용자 인증을 요청한다.
 @param closeCallback server로부터의 응답 callback
 @param callbackParam closeCallback으로 전달된 parameter.
 @return Hub error code.
 @discussion
 server로부터 정상적으로 인증을 시도하게 되면 0(HUB_E_SUCCESS)을 반환한다. 이후 server로부터의 응답을 callback에서 받을 수 있다.
 */
int CS_hubAuth(
	HubCallbackWithJSON closeCallback,
	void *callbackParam
);

/**
 @abstract
 Hub(Hive)로 사용자 인증을 요청한다. (param 추가)
 @param param 추가 옵션 param.
 @param closeCallback server로부터의 응답 callback
 @param callbackParam closeCallback으로 전달된 parameter.
 @return Hub error code.
 @discussion
 server로부터 정상적으로 인증을 시도하게 되면 0(HUB_E_SUCCESS)을 반환한다. 이후 server로부터의 응답을 callback에서 받을 수 있다.
 */
int CS_hubAuthWithParam(
    const char *param,
    HubCallbackWithJSON closeCallback,
    void *callbackParam
);

/**
 @abstract
 인증되어 있는 사용자를 logout을 요청한다.
 @param closeCallback server로부터의 응답 callback.
 @param callbackParam closeCallback으로 전달된 parameter.
 @return Hub error code.
 @discussion
 server로부터 정상적으로 logout을 시도하게 되면 0(HUB_E_SUCCESS)을 반환한다. 이후 server로부터의 응답을 callback에서 받을 수 있다.
 */
int CS_hubLogout(
	HubCallbackWithJSON closeCallback,
	void *callbackParam
);

/**
 @abstract
 인증되어 있는 사용자계정의 탈퇴를 요청한다.
 @param closeCallback server로부터의 응답 callback.
 @param callbackParam closeCallback으로 전달된 parameter.
 @return Hub error code.
 @discussion
 server로부터 정상적으로 delete을 시도하게 되면 0(HUB_E_SUCCESS)을 반환한다. 이후 server로부터의 응답을 callback에서 받을 수 있다.
 */
int CS_hubUserDelete(
    HubCallbackWithJSON closeCallback,
    void *callbackParam
);

/**
 @abstract
 특정 경로(ex. home, feed, ...)의 dialog를 표시한다.
 @param relativePath 접근하고자 하는 hive page 경로.
 @param closeCallback server로부터의 응답 callback.
 @param callbackParam closeCallback으로 전달될 parameter.
 @return Hub error code.
 */
int CS_hubShowDialog(
	const char *relativePath,
	HubCallbackWithJSON closeCallback,
	void *callbackParam
);

/**
 @abstract
 server로 비동기 요청을 한다.
 @param requestName 비동기 요청을 위한 요청 이름.
 @param params 비동기 요청시 필요한 parameter. The default is nil.
 @param responseCallback server로부터의 응답 callback.
 @param callbackParam responseCallback으로 전달될 parameter.
 @return Hub error code.
 */
int CS_hubAsyncRequest(
	const char *requestName,
	const char *params,
	HubCallbackWithJSON responseCallback,
	void *callbackParam
);
	
/**
 @abstract
 게스트 계정을 정식전환으로 요청시 사용된다.
 게스트 상태에서 함수를 호출하면, 정식 사용자 인증을 시도한다.
 인증에 성공하게 되면 해당 정식 사용자의 uid정보를 가져온다.
 @param closeCallback server로부터의 응답 callback.
 @param callbackParam closeCallback으로 전달될 parameter.
 @return Hub error code.
 */
int CS_hubGuestAcquireUid(
	HubCallbackWithJSON closeCallback,
	void *callbackParam
);

/**
 @abstract
 게스트 계정을 정식 사용자로 전환을 시도한다.
 @param guestUid 현재 로그인 되어 있는 게스트 계정의 uid.
 @param candidateUid 전환하고자 하는 정식 사용자의 uid.
 @param closeCallback server로부터의 응답 callback.
 @param callbackParam closeCallback으로 전달될 parameter.
 @return Hub error code.
 @discussion
 이 함수를 호출하기 이전에 CS_hubGuestAcquireUid()을 호출하여 전환하고자 하는 정식 사용자의 uid정보를 server로부터 얻어와야 한다.
 */
int CS_hubGuestBind(
	const char *guestUid,
	const char *candidateUid,
	HubCallbackWithJSON closeCallback,
	void *callbackParam
);


// Social Request Service and Type
#define CSHUB_SOCIAL_SERVICE_FACEBOOK	"facebook"
#define CSHUB_SOCIAL_SERVICE_GOOGLEPLUS	"googleplus"

#define CSHUB_SOCIAL_USER_PROFILE		"social/request_user_profile"
#define CSHUB_SOCIAL_FRIENDS			"social/request_friends"
#define CSHUB_SOCIAL_SHARE				"social/share_app_activity"
#define CSHUB_SOCIAL_APP_MESSAGE		"social/send_app_message"
#define CSHUB_SOCIAL_APP_INVITATION		"social/send_app_invitation"
#define CSHUB_SOCIAL_INVITATION_LIST	"social/request_invitation_list"
#define CSHUB_SOCIAL_QUERY				"social/query"
#define CSHUB_SOCIAL_BUSINESS_MODEL		"social/business_model"
#define CSHUB_SOCIAL_INVITE_DIALOG		"social/invite_dialog"
#define CSHUB_SOCIAL_RECEIVED_INVITE	"social/received_invite"
#define CSHUB_SOCIAL_USER_PROFILE_IMAGE "social/user_profile_image"
#define CSHUB_SOCIAL_IS_AUTHORIZED		"social/is_authorized"
#define CSHUB_SOCIAL_LOGOUT				"social/logout"
#define CSHUB_SOCIAL_CONNECT			"social/connect"

/**
 @abstract
 Hub에서 지원되는 외부 소셜 서비스 기능을 사용한다.
 @param socialService 사용하고자 하는 외부 소셜 서비스의 이름. ex) facebook, googleplus, ...
 @param requestName 사용하고자 하는 외부 소셜 서비스의 기능 이름. ex)social/share_app_activity, ...
 @param params 서비스 사용시 필요한 parameter. JSON은 string format. The default is NULL.
 @param responseCallback server로부터의 응답 callback.
 @param callbackParam responseCallback으로 전달될 parameter.
 @return Hub error code.
 @discussion
 요청시 hub server로의 요청이 아닌 외부 소셜 서비스의 서버로 요청한다.
 서비스에 따라 지원되지 않는 기능이 존재한다.
 */
int CS_hubSocialRequest(
	const char *socialService,
	const char *requestName,
	const char *params,
	HubCallbackWithJSON responseCallback,
	void *callbackParam
);
	
/**
 @abstract
 Hub(Hive)로 사용자 인증을 요청한다.
 @param engageQuery engagement로 입력받은 Query.
 @param closeCallback server로부터의 응답 callback
 @param callbackParam closeCallback으로 전달된 parameter.
 @return Hub error code.
 @discussion
 server로부터 정상적으로 인증을 시도하게 되면 0(HUB_E_SUCCESS)을 반환한다. 이후 server로부터의 응답을 callback에서 받을 수 있다.
 */
int CS_hubAuthWithQuery(
    const char *engageQuery,
    HubCallbackWithJSON closeCallback,
    void *callbackParam
);

/**
 @abstract
 특정 경로(ex. home, feed, ...)의 dialog를 표시한다.
 @param relativePath 접근하고자 하는 hive page 경로.
 @param engageQuery engagement로 입력받은 Query.
 @param closeCallback server로부터의 응답 callback.
 @param callbackParam closeCallback으로 전달될 parameter.
 @return Hub error code.
 */
int CS_hubShowDialogWithQuery(
    const char *relativePath,
    const char *engageQuery,
    HubCallbackWithJSON closeCallback,
    void *callbackParam
);

/**
 @abstract
 parameter를 추가하여 챗봇으로 전달한다.
 @param chatbotParam 전달하고자 하는 data
 @return Hub error code.
 */
int CS_hubShowDialogForChatbot (
    const char *chatbotParam,
    HubCallbackWithJSON closeCallback,
    void *callbackParam
);

/**
 @abstract
 Hub(Hive)모듈 로그인 Dialog를 투명하게 노출한다.
 @return Hub error code.
 */
int CS_hubSetAuthDialogClear(void);


/**
 * (Android Only)
 * 인증 서버에 요청하여 PGS PlayerId와 매칭된 UID 정보가 있는 경우
 * 세션을 받아와 셋팅한다. (별도 콜백이 존재하지 않음)
 * (Google Play Games on PC 지원 기능)
 * @return
 */
int CS_hubSetPGSAutoLogin();


/**
 * (Android Only)
 * 인증 서버에 요청하여 PGS PlayerId와 매칭된 UID 정보가 있는 경우
 * 세션을 받아와 셋팅한다.
 * 콜백으로 세션 셋팅 여부를 확인할 수 있다.
 * (Google Play Games on PC 지원 기능)
 * @param closeCallback
 * @param callbackParam
 * @return
 */
int CS_hubSetPGSAutoLoginWithCB(HubCallbackWithJSON closeCallback, void *callbackParam);


/**
 * (Android Only)
 * PGS PlayerId로 UID를 발급받는 로그인 API를 호출한다.
 * (Google Play Games on PC 적용 과정에서 개발되었으나 현재 미사용 API, GPGonPC 정식 런칭 시 잔존 여부 확인 예정)
 * @param closeCallback
 * @param callbackParam
 * @return
 */
int CS_hubAuthByPGS(HubCallbackWithJSON	closeCallback, void *callbackParam);


/**
 * (Android Only)
 * PGS 기능을 사용할 수 있는 여부를 반환한다.
 * (Google Play Games on PC 지원 기능)
 * @return
 */
int CS_hubIsPGSAvailable();



// +++++ +++++ +++++ +++++ +++++ 
// Deprecated (removed)
// +++++ +++++ +++++ +++++ +++++ 

// (All)
// int CS_hubSetOption (const char *key, const char *value) __deprecated_msg("Not Support");
// int CS_hubUninitialize(void) __deprecated_msg("Not Support");
// int CS_hubInitialize(const char *appId, const char *gameIndex, bool isTestServer) __deprecated_msg("Implement CS_hubInitializeEx() instead");

// (Android)
// int CS_hubPGSLoginProc(HubCallbackWithJSON closeCallback, void *callbackParam);
// int CS_hubGetIsPGS();

// (iOS)
// void CS_hubApplicationDidBecomeActive(void) __deprecated_msg("Replace C2SAppDelegate");
// bool CS_hubApplicationOpenURL(NSURL* url, NSString* sourceApplication, id annotation) __deprecated_msg("Replace C2SAppDelegate");



#ifdef __cplusplus
}
#endif

#endif	// __C2S_HUB_2_H__
