#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

@class RecaptchaError;

/**
 * For better readability purposes, use this type for all timeout durations.
 */
typedef int64_t RecaptchaMinutesDuration;

/**
 * Class that handles the verification phase of reCAPTCHA 2FA. An instance of this class will
 * be returned by @c RecaptchaClient#ChallengeAccount, when your user inputs the 2FA pin you
 * can call @c verifyPin to finish the 2FA workflow.
 */
@interface RecaptchaVerificationHandler : NSObject

/**
 * The email address (possibly obfuscated).
 */
@property(nonatomic, readonly, copy) NSString *identifier;

/**
 * Challenge expiration time in minutes.
 */
@property(nonatomic, readonly) RecaptchaMinutesDuration timeout;

/**
 * Initializes a handler bound to a verification token.
 * @param verificationToken token returned from the challengeAccount call.
 */
- (instancetype)initWithVerificationToken:(NSString *)verificationToken
                                  siteKey:(NSString *)siteKey
                               identifier:(NSString *)identifier
                                  timeout:(RecaptchaMinutesDuration)timeout;

/**
 * Verifies that the user's input pin is valid.
 * @param pin The pin that the user typed in.
 * @param onVerification Callback that will be invoked with the resulting reCAPTCHA
 * token or error.
 */
- (void)verifyPin:(NSString *)pin
    onVerification:(void (^)(NSString *_Nullable, RecaptchaError *_Nullable))onVerification;

@end

NS_ASSUME_NONNULL_END
