#include "ApiAction.h"
#include "Parameter.h"
#include "DealCallback.h"
#include "LogController.h"
#include "DataManager.h"
// UI Scene (view)
#include "LoginListScene.h"
#include "Postbox.hpp"

#ifdef __ANDROID__
#include "JNISample.h"
#endif

#include "../external/json/rapidjson.h"
#include "../external/json/document.h"
#include "../external/json/writer.h"
#include "../external/json/stringbuffer.h"

#ifdef __ANDROID__
#include "cocos2d-x/Module_Cocos2d-x.h"
#include "C2SModule.h"
#else
#include "Module_Cocos2d-x.h"
#include "C2SModule.h"
#endif

using namespace rapidjson;
using namespace C2SModule_Cocos2dx;

USING_NS_CC;

extern "C" const char* getHiveCountryCode();

std::string ApiAction::selectLoginInfo = "";		// HIVE 1.10.1
int ApiAction::initPostBox = 0;                // HIVE 1.18.4

Scene* ApiAction::createScene() {
	auto scene = Scene::create();
	auto layer = ApiAction::create();
	scene->addChild(layer);
	return scene;
}

bool ApiAction::init()
{
	if (!Layer::init()) {
		return false;
	}

	log("### ApiAction : init() ###");
	return true;
}

void ApiAction::initialize() {
    log("====>>>> ApiAction::Initialize");

    ::CommonResultDelegate resultDelegate = DealCallback::ResultDelegate;
    ::C2SModuleSocial_SetCommonResultDelegate(resultDelegate);

    // 기존 테섭(스테이징 서버)의 사용은 1.7.0 버전에서 사라지고 sandbox 서버가 신규 추가되었음.
    Document arg;
    arg.SetObject();
    Document::AllocatorType &allocator = arg.GetAllocator();

    arg.AddMember("showLog", rapidjson::Value(Parameter::paramInitialize.showLog), allocator);
    arg.AddMember("useDevelop", rapidjson::Value(Parameter::paramInitialize.useDevelop),
                  allocator);    // for staging
    arg.AddMember("useStaging", rapidjson::Value(Parameter::paramInitialize.useStaging),
                  allocator);    // for staging
    arg.AddMember("useSandbox", rapidjson::Value(Parameter::paramInitialize.useSandbox), allocator);

    /**
     * appId 설정
     */
    // AppInfo에서 사용하던 appId는 더이상 사용하지 않으며 1.7.0 부터는 appId를 initialize시에
    // 직접 설정할 수 있으며 미입력시 AndroidManifest.xml의 패키지 네임을 appId로 사용하게 됨.
    // HIVE SDK 1.13.0 테스트부터 misample --> hivesdk 로 앱아이디 변경.
    if (Parameter::paramInitialize.HIVEIAP == true && Parameter::paramInitialize.useOneStore ==
                                                      true) {        // HIVE IAP로 OneStore 테스트를 할 경우 앱아이디 설정.
        Parameter::paramInitialize.appId = "com.com2us.misample.normal.freefull.onestore.kr.android.common";        // OneStore 테스트 앱아이디는 misample로 설정.
        arg.AddMember("appId", StringRef(Parameter::paramInitialize.appId.c_str()), allocator);
    } else if (Parameter::paramInitialize.HIVEIAP == false &&
               Parameter::paramInitialize.useOneStore == true) {
//		Parameter::paramInitialize.appId = "com.com2us.misample.normal.freefull.google.global.android.common";		// 원스토어 IAP 4이하 테스트 시 등록된 앱아이디
        Parameter::paramInitialize.appId = "com.com2us.hivesdk.normal.freefull.onestore.kr.android.common";            // 원스토어 IAP 5이상 테스트 시 등록된 앱아이디
        arg.AddMember("appId", StringRef(Parameter::paramInitialize.appId.c_str()), allocator);
    } else {
        if (Parameter::paramInitialize.appId == "") {        // 설정 X일 경우. (default)
            // androidmanifest.xml 의 com.com2us.misample.normal.freefull.google.global.android.common 로 수행
#ifdef __ANDROID__
            log("==== initialize appId : android appid");
            arg.AddMember("appId",
                          "com.com2us.hivesdk.normal.freefull.google.global.android.common",
                          allocator);
#else
            log("==== initialize appId : iphone appid");
			arg.AddMember("appId",
                          "com.com2us.hivesdk.normal.freefull.apple.global.ios.universal",
                          allocator);
#endif

        } else {
            arg.AddMember("appId", StringRef(Parameter::paramInitialize.appId.c_str()), allocator);
        }
    }

    // TODO : test code 작동중... (앱아이디 강제 입력할 경우 사용)
    /**
     * 앱아이디 강제 설정 시
     */
//	arg["appId"] = "com.com2us.ninepb3d.normal.freefull.google.global.android.common";				    // TODO : test code, 앱 아이디 강제 적용시
    // 테스트 앱아이디 목록
//	arg.AddMember("appId", "com.gamevil.kritikamobile.android.google.global.normal", allocator);		    // 크리티카
//	arg.AddMember("appId", "com.com2us.hivesdk.normal.freefull.onestore.kr.android.common", allocator);		// 원스토어 (IAP v5 테스트)
//    arg.AddMember("appId", "com.gamevil.valkyrie.android.google.normal", allocator);

    log("==== initialize appId : %s", Parameter::paramInitialize.appId.c_str());

    /**
     * permissionWriteExternalStorage 설정
     */
    if (Parameter::paramInitialize.permissionWriteExternalStorage == false) {
        // permissionWriteExternalStorage : false (default 값이 false)
    } else {
        arg.AddMember("permissionWriteExternalStorage", true, allocator);
    }

    /**
     * serverId 설정
     */
    if (Parameter::paramInitialize.serverId == "") {
        // 특정 serverId를 전달하지 않음
    } else {
        arg.AddMember("serverId", StringRef(Parameter::paramInitialize.serverId.c_str()),
                      allocator);
    }

    /**
     * hiveOrientation 설정 (Hive 1.19.6)
     */
     if (Parameter::paramInitialize.hiveOrientation == "") {
     	// 특정 orientation 설정 값을 전달하지 않음.
     } else {
		 // orientation 값 확인
		 if(Parameter::paramInitialize.hiveOrientation == "a" || Parameter::paramInitialize.hiveOrientation == "all")
			 arg.AddMember("hiveOrientation", "all", allocator);
		 else if(Parameter::paramInitialize.hiveOrientation == "p" || Parameter::paramInitialize.hiveOrientation == "portrait")
			 arg.AddMember("hiveOrientation", "portrait", allocator);
		 else if(Parameter::paramInitialize.hiveOrientation == "l" || Parameter::paramInitialize.hiveOrientation == "landscape")
			 arg.AddMember("hiveOrientation", "landscape", allocator);
		 else {

		 }
     }

	if (Parameter::paramInitialize.useHercules == true) {    // for useHercules
		arg.AddMember("useHercules", true, allocator);
	} else {
		arg.AddMember("useHercules", false, allocator);
	}

	/**
	 * channel 설정 (Hive 1.20.3)
	 */
    arg.AddMember("channel", StringRef(Parameter::paramInitialize.channel.c_str()), allocator);

    /**
     * HIVE Promotion 설정 (HIVE 1.12.0)
     */
    if (Parameter::paramInitialize.hivePromotion == true) {
        rapidjson::Value argHivePromotion(rapidjson::kObjectType);
        argHivePromotion.AddMember("showLog", true, allocator);
        arg.AddMember("HivePromotion", argHivePromotion, allocator);
    } else {
		/**
    	* ===== argPromotion 설정 =====
    	*/
		rapidjson::Value argPromotion(rapidjson::kObjectType);
		argPromotion.AddMember("showLog", true, allocator);
		if (Parameter::paramInitialize.useImageBanner == true)
			argPromotion.AddMember("useImageBanner", true,
								   allocator);      // HIVE 1.18.1, 이미지 배너 초기화 (v1.19.1에서 오픈)
		else {
			argPromotion.AddMember("useImageBanner", false,
								   allocator);      // HIVE 1.18.1, 이미지 배너 초기화	(v1.19.1에서 오픈)
		}
		if (Parameter::paramInitialize.useDevelop == true) {    // for staging
			argPromotion.AddMember("useStaging", true, allocator);
			argPromotion.AddMember("useDevelop", true, allocator);
		}

		arg.AddMember("Promotion", argPromotion, allocator);
    }

    /**
     * Game Language 설정 (HIVE 1.12.0)
     */
    arg.AddMember("gameLanguage", StringRef(Parameter::paramInitialize.gameLanguage.c_str()),
                  allocator);

    // ===== argSns 설정 =====
//	Json::Value argSns;
    rapidjson::Value argSns(rapidjson::kObjectType);
    if (Parameter::paramInitialize.useDevelop == true) {            // for staging
        argSns.AddMember("useStaging", true, allocator);
        argSns.AddMember("useDevelop", true, allocator);
    }
    // facebook permission 설정 (1.19.8)
    // 샘플에서 입력받는 데이터 예 (textfield : user_friends 혹은 textfield : user_friends,user_gender)
    // -- set facebook permissions (start) --
    rapidjson::Value facebookPermissions(rapidjson::kArrayType);

    std::string s = Parameter::paramInitialize.facebookPermissions;
    std::string delimiter = ",";

    size_t pos = 0;
    std::string token;
    bool existDelimiter = false;
    while ((pos = s.find(delimiter)) != std::string::npos) {
        log("# facebook permissions splitting...");
        token = s.substr(0, pos);
        log("# facebook permissions data : %s", token.c_str());
        facebookPermissions.PushBack(StringRef(token.c_str()), allocator);
        s.erase(0, pos + delimiter.length());
        existDelimiter = true;
    }
    if(existDelimiter)	// split된 마지막 인자를 pushback
        facebookPermissions.PushBack(StringRef(s.c_str()), allocator);

    if(existDelimiter) {
        log("# facebook permissions (multi) #");
        argSns.AddMember("facebookPermissions", facebookPermissions, allocator);
    }
    else {
        log("# vid (single) #");
        facebookPermissions.PushBack(StringRef(s.c_str()), allocator);
        argSns.AddMember("facebookPermissions", facebookPermissions, allocator);
    }
    // -- set facebook permissions (end) --

    /**
     * ===== argInapp 설정 ===== (misample의 OneStore 아이템 기준)
     */
    rapidjson::Value argInapp(rapidjson::kObjectType);
    argInapp.AddMember("showLog", true, allocator);
    argInapp.AddMember("useGoogleAccountId", true, allocator);      // added HIVE v1.16.4
//    argInapp.AddMember("autoVerify", true, allocator);				// TODO : 인앱 autoVerify 사용 테스트
    //#if defined(__LEBI_TEST__)
    if (Parameter::paramInitialize.useOneStore == true) {            // [1] one store로 초기화
        if (Parameter::paramInitialize.HIVEIAP == true) {            // [1-1] HIVE IAP로 초기화
            // markerTarget 정보 필요 없음.
            //argInapp.AddMember("marketTarget", "tstore", allocator);
            // inappId 정보 필요 없음.
            //argInapp.AddMember("inappId", "OA00689529", allocator);
            // pidList는 없어도 상관 없음.
            // (add pidList)
        } else {                                                        // [1-2] 구인앱으로 초기화
            argInapp.AddMember("marketTarget", "tstore", allocator);

//			Json::Value inappList;
            rapidjson::Value inappList(rapidjson::kArrayType);

//            argInapp.AddMember("inappId", "OA00267036", allocator); // 마켓 등록 아이디 (misample 사용 시 마켓 아이디)
//            inappList.PushBack("0910033926", allocator);
//            inappList.PushBack("0900395110", allocator);

            argInapp.AddMember("inappId", "OA00734237",
                               allocator); // 마켓 등록 아이디 (hivesdk 사용 시 마켓 아이디)
            inappList.PushBack("item_001", allocator);
            inappList.PushBack("item0001", allocator);

            argInapp.AddMember("pidList", inappList, allocator);
        }

    } else if (Parameter::paramInitialize.useGalaxyStore ==
               true) {        // 갤럭시 스토어, added HIVE v1.17.2
        log("### social_init, saumsung galaxystore init");
        argInapp.AddMember("marketTarget", "galaxystore", allocator);

        rapidjson::Value inappList(rapidjson::kArrayType);

        inappList.PushBack("item1", allocator);
        inappList.PushBack("item2", allocator);

        argInapp.AddMember("pidList", inappList, allocator);
        if (Parameter::paramInitialize.useDevelop == true) {    // for staging
            argInapp.AddMember("useStaging", true, allocator);
            argInapp.AddMember("useDevelop", true, allocator);
        }
    } else {                                                            // [2] 구글 마켓으로 초기화
        argInapp.AddMember("marketTarget", "googleplay_lebi",
                           allocator);       // 현재 사용하고 있는 c2smodule (구글 구매 및 러비 구매 가능 버전)
//        argInapp.AddMember("marketTarget", "lebi", allocator);
//        argInapp.AddMember("marketTarget", "googleplay", allocator);
//		#endif

        rapidjson::Value inappList(rapidjson::kArrayType);
        inappList.PushBack("item01", allocator);        // hivesdk : item01, misample : m1
        inappList.PushBack("item02", allocator);        // hivesdk : item02, misample : u1
//        inappList.PushBack("com.com2us.mlbgm.normal.freefull.google.global.android.common.cash01", allocator);      // TODO : test code, 임의의 pid 강제 적용시

        argInapp.AddMember("pidList", inappList, allocator);
        if (Parameter::paramInitialize.useDevelop == true) {    // for staging
            argInapp.AddMember("useStaging", true, allocator);
            argInapp.AddMember("useDevelop", true, allocator);
        }

    }

    /**
     * // TODO : test code
     * White Domain 우회 옵션
     */
	if(Parameter::paramInitialize.HIVEIAP == true) {
		arg.AddMember("useWhiteDomain", true, allocator);
	};

    /**
     * ===== argPush 설정 =====
     */
    rapidjson::Value argPush(rapidjson::kObjectType);
    argPush.AddMember("showLog", true, allocator);
//	argPush.AddMember("useThirdPartyPush", true, allocator);  // Amazon 푸쉬 및 JPush 사용 여부(사용여부에 따라 각각의 라이브러리를 추가해주어야 합니다.)
    if (Parameter::paramInitialize.useDevelop == true) {    // for staging
        argPush.AddMember("useStaging", true, allocator);
        argPush.AddMember("useDevelop", true, allocator);
    }



    /**
     * ===== showNotice 설정 =====			// HIVE 1.10.0
     */
    if (Parameter::paramInitialize.showNotice == false) {
        // showNotice (false) --> 약관만 보고 공지 없이 초기화, showNotice (true or null) --> 약관 + 공지 보기(기존 플로우)
        arg.AddMember("showNotice", false, allocator);
    } else
        arg.AddMember("showNotice", true, allocator);

    /**
     * ===== showPermissionView 설정 =====			// HIVE 1.17.2
     * HIVE SDK의 권한 고지를 노출할지 여부 판단
     */
    if (Parameter::paramInitialize.showPermissionView == false) {
        arg.AddMember("showHivePermissionView", false, allocator);
    }


    /**
     * ===== argHiveIAP 설정 =====
     */
//    rapidjson::Value argHiveIAP(rapidjson::kObjectType);
//    argHiveIAP.AddMember("showLog", true, allocator);
//    if (Parameter::paramInitialize.useDevelop == true) {    // for staging
//        argHiveIAP.AddMember("useStaging", true, allocator);
//        argHiveIAP.AddMember("useDevelop", true, allocator);
//    }

    /**
     * ===== argHiveIAP 설정 =====
     */
    rapidjson::Value argInappV4(rapidjson::kObjectType);
    argInappV4.AddMember("showLog", true, allocator);
    if (Parameter::paramInitialize.useDevelop == true) {    // for staging
        argInappV4.AddMember("useStaging", true, allocator);
        argInappV4.AddMember("useDevelop", true, allocator);
    }

    /**
     * ===== 각 모듈 초기화 정보 전달 =====
     */
    arg.AddMember("Sns", argSns, allocator);
    arg.AddMember("Push", argPush, allocator);


    // init 설정에서 HIVE IAP, Inapp V4 값 확인.
//    if (Parameter::paramInitialize.HIVEIAP == true) {
//		arg.AddMember("HiveIAP", argHiveIAP, allocator);
//    }
//	if (Parameter::paramInitialize.useInappV4 == true)
		arg.AddMember("InappV4", argInappV4, allocator);
//	else {
//		arg.AddMember("Inapp", argInapp, allocator);
//	}

	// ===== 네트워크 초기 시간 설정 =====			// HIVE 1.13.0
	if(Parameter::paramInitialize.timeoutSeconds == "") {
		arg.AddMember("timeoutSeconds", 5, allocator);
	}
	else {
//		arg.AddMember("timeoutSeconds", StringRef(Parameter::paramInitialize.timeoutSeconds.c_str()), allocator);
        arg.AddMember("timeoutSeconds", atoi(Parameter::paramInitialize.timeoutSeconds.c_str()), allocator);
	}


	/**
	 * === tracking module ===
	 */
//	Json::Value argTrackingModule;
	rapidjson::Value argTrackingModule(rapidjson::kObjectType);

	//event keyMap test
//	Json::Value argPartyTrackEventKey;
//	argPartyTrackEventKey["Purchase"] = "Purchase";
//	argPartyTrackEventKey["TutorialComplete"] = "TutorialComplete";
	rapidjson::Value argPartyTrackEventKey(rapidjson::kObjectType);
	argPartyTrackEventKey.AddMember("Purchase", "Purchase", allocator);
	argPartyTrackEventKey.AddMember("TutorialComplete", "TutorialComplete", allocator);

	// 170417 : Misample의 Adjust, PartyTrack 트래킹 데이터 오염 이슈가 있어 키값 변경.

//	Json::Value argAdjustEventKey;
//	argAdjustEventKey["Purchase"] = "8uwijl";					// Adjust(Purchase) : 기존(cny1lg), 변경(8uwijl)
//	argAdjustEventKey["TutorialComplete"] = "wwzrv2";			// Adjust(TutorialComplete) : 기존(7ndx92), 변경(wwzrv2)
//	argAdjustEventKey["Update"] = "aby3jx";						// Adjust(Update) : 기존(stzv9n), 변경(aby3jx)
	rapidjson::Value argAdjustEventKey(rapidjson::kObjectType);
#ifdef __ANDROID__
	argAdjustEventKey.AddMember("Purchase", "8uwijl", allocator);
	argAdjustEventKey.AddMember("TutorialComplete", "wwzrv2", allocator);
	argAdjustEventKey.AddMember("Update", "aby3jx", allocator);
#else
    argAdjustEventKey.AddMember("Purchase", "py318i", allocator);
	argAdjustEventKey.AddMember("TutorialComplete", "gvfh2l", allocator);
	argAdjustEventKey.AddMember("Update", "sosx8y", allocator);
#endif

//	Json::Value argTuneEventKey;
//	argTuneEventKey["Purchase"] = "Purchase";
//	argTuneEventKey["TutorialComplete"] = "Tutorial Complete";
	rapidjson::Value argTuneEventKey(rapidjson::kObjectType);
	argTuneEventKey.AddMember("Purchase", "Purchase", allocator);
	argTuneEventKey.AddMember("TutorialComplete", "Tutorial Complete", allocator);

	//============ PartyTrack Setting =================//
//	Json::Value argPartyTrackSetting;
	// default
//	argPartyTrackSetting["appID"] = 9353;											// PartyTrack(appID) : 기존(5725), 변경(9353)
//	argPartyTrackSetting["appKey"] = "c0cfe655be1a77829c7ffed34c4ccf1a";			// PartyTrack(appKey) : 기존(), 변경(c0cfe655be1a77829c7ffed34c4ccf1a)
//	argPartyTrackSetting["optimizeFacebookAd"] = false; 							// 페이스북 광고 최적화 설정. 필요할 경우 true 셋팅
//	argPartyTrackSetting["Matching"] = argPartyTrackEventKey;

	rapidjson::Value argPartyTrackSetting(rapidjson::kObjectType);
	argPartyTrackSetting.AddMember("appID", 9353, allocator);
	argPartyTrackSetting.AddMember("appKey", "c0cfe655be1a77829c7ffed34c4ccf1a", allocator);
	argPartyTrackSetting.AddMember("optimizeFacebookAd", false, allocator);
	argPartyTrackSetting.AddMember("Matching", argPartyTrackEventKey, allocator);

//	argTrackingModule["Partytrack"] = argPartyTrackSetting;
	argTrackingModule.AddMember("Partytrack", argPartyTrackSetting, allocator);

	//============ Tune Setting =================//
//	Json::Value argTuneSetting;
//	argTuneSetting["advertiserID"] = "169452";
//	argTuneSetting["conversionKey"] = "24305c3a354951afe96d1800ad9299bf";
////	argTuneSetting["isFacebookEventLogging"] = false;					// TODO : isFacebookEventLogging default = true, 추후 변수로 true or false를 받을 수 있게 추가
//	argTuneSetting["Matching"] = argTuneEventKey;
//
//	argTrackingModule["MAT"] = argTuneSetting;

	//============ Adjust Setting =================//
//	Json::Value argAdjustSetting;
//	argAdjustSetting["appToken"] = "f02nseiur2m8";				// Adjust(appToken) : 기존(hx9gfc2j6nxs), 변경(f02nseiur2m8)
	rapidjson::Value argAdjustSetting(rapidjson::kObjectType);
#ifdef __ANDROID__
	argAdjustSetting.AddMember("appToken", "f02nseiur2m8", allocator);
#else
    argAdjustSetting.AddMember("appToken", "8szd9kddtt6o", allocator);
#endif

	//============ Adjust App Secret Setting =================//
//	Json::Value argAdjustAppSecretSetting;
//	argAdjustAppSecretSetting["secretId"] = "1";
//	argAdjustAppSecretSetting["info1"] = "1263786494";
//	argAdjustAppSecretSetting["info2"] = "303424327";
//	argAdjustAppSecretSetting["info3"] = "1829017077";
//	argAdjustAppSecretSetting["info4"] = "1052886214";
//	argAdjustSetting["AppSecretInfo"] = argAdjustAppSecretSetting;

	rapidjson::Value argAdjustAppSecretSetting(rapidjson::kObjectType);
	argAdjustAppSecretSetting.AddMember("secretId", "1", allocator);
#ifdef __ANDROID__
	argAdjustAppSecretSetting.AddMember("info1", "1263786494", allocator);
	argAdjustAppSecretSetting.AddMember("info2", "303424327", allocator);
	argAdjustAppSecretSetting.AddMember("info3", "1829017077", allocator);
	argAdjustAppSecretSetting.AddMember("info4", "1052886214", allocator);
#else
    argAdjustAppSecretSetting.AddMember("info1", "2071917595", allocator);
	argAdjustAppSecretSetting.AddMember("info2", "254356864", allocator);
	argAdjustAppSecretSetting.AddMember("info3", "427896519", allocator);
	argAdjustAppSecretSetting.AddMember("info4", "820983601", allocator);
#endif

	argAdjustSetting.AddMember("AppSecretInfo", argAdjustAppSecretSetting, allocator);
	argAdjustSetting.AddMember("Matching", argAdjustEventKey, allocator);

//	argAdjustSetting["Matching"] = argAdjustEventKey;

//	argTrackingModule["Adjust"] = argAdjustSetting;
	argTrackingModule.AddMember("Adjust", argAdjustSetting, allocator);
    
    //============ singular Setting =================//
//    Json::Value argSingularEventKey;
//    argSingularEventKey["Update"] = "Update";
//    argSingularEventKey["Purchase"] = "Purchase";
//    argSingularEventKey["TutorialComplete"] = "Tutorial_complete";
	rapidjson::Value argSingularEventKey(rapidjson::kObjectType);
	argSingularEventKey.AddMember("Update", "Update", allocator);
	argSingularEventKey.AddMember("Purchase", "purchase", allocator);
	argSingularEventKey.AddMember("TutorialComplete", "Tutorial Complete", allocator);
	argSingularEventKey.AddMember("Install", "install", allocator);

    
//    Json::Value argSingularSetting;
    // default
//    argSingularSetting["apiKey"] = "com2ususa_dev_7705679f";
//    argSingularSetting["secretKey"] = "9b7ff73a0fc13c6c9397a9881dba2673";
//    argSingularSetting["Matching"] = argSingularEventKey;
	rapidjson::Value argSingularSetting(rapidjson::kObjectType);
	argSingularSetting.AddMember("apiKey", "com2ususa_dev_7705679f", allocator);
	argSingularSetting.AddMember("secretKey", "9b7ff73a0fc13c6c9397a9881dba2673", allocator);
	argSingularSetting.AddMember("Matching", argSingularEventKey, allocator);


//    argTrackingModule["Singular"] = argSingularSetting;
	argTrackingModule.AddMember("Singular", argSingularSetting, allocator);
    
    //============ AppsFlyer Setting =================//
    rapidjson::Value argAppsFlyerEventKey(rapidjson::kObjectType);
    argAppsFlyerEventKey.AddMember("Update", "Update", allocator);
    argAppsFlyerEventKey.AddMember("Install", "Install", allocator);
    argAppsFlyerEventKey.AddMember("TutorialComplete", "Tutorial Complete", allocator);

    rapidjson::Value argAppsFlyerSetting(rapidjson::kObjectType);
#ifdef __ANDROID__
    argAppsFlyerSetting.AddMember("key", "G6yEQjHw4CrEN8sEzF9onU", allocator);
#else
    argAppsFlyerSetting.AddMember("key", "G6yEQjHw4CrEN8sEzF9onU", allocator);
    argAppsFlyerSetting.AddMember("itunesConnectAppId", "1202135972", allocator);
#endif
    argAppsFlyerSetting.AddMember("Matching", argAppsFlyerEventKey, allocator);

    argTrackingModule.AddMember("AppsFlyer", argAppsFlyerSetting, allocator);
    argTrackingModule.AddMember("debugMode", true, allocator);

	arg.AddMember("Tracking", argTrackingModule, allocator);

//	argTrackingModule["debugMode"] = true;

//	arg["Tracking"] = argTrackingModule;

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);

	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("### strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (Initialize)
	LogController::addLog("[Request : C2SModuleSocial_Initialize] >>>>>\n" + strJSON);

	::C2SModuleSocial_Initialize(strJSON.c_str(), &error);
}

void ApiAction::login()
{
	log("====>>>> ApiAction::login()");
	if(Parameter::paramUIState.isHiveLogin == true) {
		log("### Type : HIVE login");
		C2SModule_Cocos2dx::C2SModuleError error;

		// 로그 정보 추가 (Login)
		LogController::addLog("[Request : C2SModuleSns_Login] >>>>>\nNULL");

		if (::C2SModuleSns_Login((char*) NULL, &error) == false) {
			log("====>>>> Hub Login error (%s)", error.get()->GetMessage());
			// 로그 정보 추가 (Login : error)
			std::stringstream errorCode;
			errorCode << error.get()->GetCode();
			LogController::addLog("[error : C2SModuleSns_Login] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
			// QA 로그 정보 추가
			LogController::setApiResultState(true, error.get()->GetCode());
		}

	} else if(Parameter::paramUIState.isHiveLogin == false) {
		log("### Type : Guest login");

		// added 1.18.0, 계정 충돌 팝업에서 back to game 후 그냥 게스트 로그인 시도 시 기능 메뉴 화면으로 넘어 갈 수 있도록 처리
        DataManager::setState(TAG_SCENENUM_LOGIN_LIST, "isSelection", false);               // TODO : 1.18.0
        DataManager::setParamValue_bool(TAG_APIACTION_INITIALIZE, "loginState", true);     // TODO : 1.18.0

		Document argGuestLogin;
		argGuestLogin.SetObject();
		Document::AllocatorType& allocator = argGuestLogin.GetAllocator();

		argGuestLogin.AddMember("type", "guest", allocator);

		StringBuffer buffer;
		Writer<StringBuffer> writer(buffer);
		argGuestLogin.Accept(writer);

		std::string strJSON = buffer.GetString();

//		Json::Value argGuestLogin;
//		argGuestLogin["type"] = "guest";

//		Json::StyledWriter writer;
//		std::string strJSON = writer.write(argGuestLogin);

		C2SModule_Cocos2dx::C2SModuleError error;
		log("### strJSON : %s", strJSON.c_str());

		// 로그 정보 추가 (Login)
		LogController::addLog("[Request : C2SModuleSns_Login] >>>>>\n" + strJSON);

		if (::C2SModuleSns_Login(strJSON.c_str(), &error) == false) {
			log("====>>>> Guest Login error (%s)", error.get()->GetMessage());
			// 로그 정보 추가 (Login : error)
			std::stringstream errorCode;
			errorCode << error.get()->GetCode();
			LogController::addLog("[error : C2SModuleSns_Login] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
			// QA 로그 정보 추가
			LogController::setApiResultState(true, error.get()->GetCode());
		}
	}
}

void ApiAction::login_select()		// HIVE 1.10.1
{
	log("====>>>> ApiAction::logins_select()");
	log("### Type : Select login");

	std::string strJSON = DataManager::getParamValue(TAG_APIACTION_LOGIN, "selectLoginInfo");

	C2SModule_Cocos2dx::C2SModuleError error;
	log("### strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (Login)
	LogController::addLog("[Request : C2SModuleSns_Login(selectLogin)] >>>>>\n" + strJSON);

	if(strJSON == "") {
		log("### no select info");
		return;
	}

	// 셀렉트 로그인 상황 -> 선택창을 띄우기 전에 우선 멈춤.
	if (::C2SModuleSns_Login(strJSON.c_str(), &error) == false) {
		log("====>>>> Select Login error (%s)", error.get()->GetMessage());
		// 로그 정보 추가 (Login : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleSns_Login(selectLogin)] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

void ApiAction::login_customSelect()	// HIVE 1.10.1
{
	log("====>>>> ApiAction::login_customSelect()");
	log("### Type : Select(Custom) login");

	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	arg.AddMember("type", "customSelect", allocator);
	arg.AddMember("selectVid", StringRef(Parameter::paramLogin.selectedVid.c_str()), allocator);

//	Json::Value arg;
//	arg["type"] = "customSelect";
//	arg["selectVid"] = Parameter::paramLogin.selectedVid;

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("### strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (Login)
	LogController::addLog("[Request : C2SModuleSns_Login(customselectLogin)] >>>>>\n" + strJSON);

	if(::C2SModuleSns_Login(strJSON.c_str(), &error) == false) {
		log("====>>>> Custom Select Login error (%s)", error.get()->GetMessage());
		// 로그 정보 추가 (Login : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleSns_Login(customselectLogin)] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

void ApiAction::logout() {
	if(Parameter::paramLogin.type == "guest") {
		std::string message = "- guest 유저는 로그아웃 금지!\n- 로그인 화면으로 이동!\n- 게스트로 계속 진행할 경우 [게스트 접속] 수행!\n- 계정 전환을 원할 경우 [하이브 접속] 수행!";
		MessageBox(message.c_str(), "guest Logout");
		return;
	} else {
		C2SModule_Cocos2dx::C2SModuleError error;

		// 로그 정보 추가 (Logout)
		LogController::addLog("[Request : C2SModuleSns_Logout] >>>>>\nNULL");

		if (::C2SModuleSns_Logout((char*) NULL, &error) == false) {
			log("====>>>> Logout error (%s)", error.get()->GetMessage());
			// 로그 정보 추가 (Logout : error)
			std::stringstream errorCode;
			errorCode << error.get()->GetCode();
			LogController::addLog("[error : C2SModuleSns_Logout] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
			// QA 로그 정보 추가
			LogController::setApiResultState(true, error.get()->GetCode());
		}
//		else
//			DataManager::getApiAction(TAG_APIACTION_INITIALIZE);
	}
}

// Hive 1.19.5.2, 1.20.0+
void ApiAction::logoutUserExit() {

	std::string message = "회원 탈퇴 후 로그아웃 호출입니다!\nguest 및 Hive 정식 유저 탈퇴 후 호출되어야합니다!";
	showDialogMsg(message.c_str());

	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	arg.AddMember("type", "userExit", allocator);

	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("### strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (Logout)
	LogController::addLog("[Request : C2SModuleSns_Logout(userExit)] >>>>>\n" + strJSON);

	if(::C2SModuleSns_Logout(strJSON.c_str(), &error) == false) {
		log("====>>>> Logout (userExit) error (%s)", error.get()->GetMessage());
		// 로그 정보 추가 (Login : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleSns_Logout(userExit)] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

void ApiAction::getLoginType() {
	// Show getLoginType
	std::string getLoginType(::GetLoginType());
	log("====>>>> getLoginType (%s)", getLoginType.c_str()); 
	// 로그 정보 추가 (getLoginType)  
	LogController::addLog("[Request : GetLoginType] >>>>>\n");
	if (::GetLoginType() != (char *) NULL) {
		LogController::addLog("[Result : GetLoginType] >>>>>\ngetLoginType () : " + getLoginType);
        DataManager::setParamValue_string(TAG_APIACTION_LOGIN, "getLoginType", getLoginType);
	}
	else
		LogController::addLog("[Result : GetLoginType] >>>>>\ngetLoginType () : NULL");
}

void ApiAction::pushRegister()
{
	log("====>>>> ApiAction::pushRegister()");
	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	arg.AddMember("pushId", Parameter::paramPushRegister.pushId, allocator);
	arg.AddMember("title", StringRef(Parameter::paramPushRegister.title.c_str()), allocator);
	arg.AddMember("message", StringRef(Parameter::paramPushRegister.message.c_str()), allocator);
	arg.AddMember("ticker", "알림 도착", allocator);
	arg.AddMember("type", StringRef(Parameter::paramPushRegister.type.c_str()), allocator);
//	arg.AddMember("icon", "", allocator);
//	arg.AddMember("sound", "", allocator);
//	arg.AddMember("active", "app", allocator);
	arg.AddMember("after", Parameter::paramPushRegister.after, allocator);
    if(Parameter::paramPushRegister.groupId != "")
        arg.AddMember("groupId", StringRef(Parameter::paramPushRegister.groupId.c_str()), allocator);


//	Json::Value arg;
//
//	arg["pushId"] = Parameter::paramPushRegister.pushId;
//	arg["title"] = Parameter::paramPushRegister.title;
//	arg["message"] = Parameter::paramPushRegister.message;
//	arg["ticker"] = "알림 도착";									// (optional)
//	arg["type"] = Parameter::paramPushRegister.type;
//	// arg["icon"] = "";										// (optional)
//	// arg["sound"] = "";										// (optional)
////	arg["active"] = "app";										// (optional)
//	arg["after"] = Parameter::paramPushRegister.after;

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("### strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (pushRegister)
	LogController::addLog("[Request : C2SModulePush_Register] >>>>>\n" + strJSON);

	if (::C2SModulePush_Register(strJSON.c_str(), &error) == false) {
		log("====>>>> Push register error (%s)", error.get()->GetMessage());
		// 로그 정보 추가 (pushRegister : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModulePush_Register] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

void ApiAction::pushUnregister()
{
	log("====>>>> ApiAction::pushUnregister()");
	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	arg.AddMember("pushId", Parameter::paramPushUnregister.pushId, allocator);

//	Json::Value arg;
//
//	arg["pushId"] = Parameter::paramPushUnregister.pushId;

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("### strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (pushUnregister)
	LogController::addLog("[Request : C2SModulePush_Unregister] >>>>>\n" + strJSON);

	if (::C2SModulePush_Unregister(strJSON.c_str(), &error) == false) {
		log("====>>>> Push unregister error (%s)", error.get()->GetMessage());
		// 로그 정보 추가 (pushUnregister : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModulePush_Unregister] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

void ApiAction::pushDescription()
{
	log("====>>>> ApiAction::pushDescription()");
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

//    arg.AddMember("usePush", Parameter::paramPushDescription.usePush, allocator);
//    arg.AddMember("sound", Parameter::paramPushDescription.sound, allocator);
//    arg.AddMember("vibrate", Parameter::paramPushDescription.vibrate, allocator);
//    arg.AddMember("operationGCM", Parameter::paramPushDescription.operationGCM, allocator);
//    arg.AddMember("operationLocal", Parameter::paramPushDescription.operationLocal, allocator);
//    arg.AddMember("useNotice", Parameter::paramPushDescription.useNotice, allocator);
//    arg.AddMember("useNight", Parameter::paramPushDescription.useNight, allocator);
	if(Parameter::paramPushDescription.usePushInt == 1)
		arg.AddMember("usePush", true, allocator);
	else if(Parameter::paramPushDescription.usePushInt == 2)
		arg.AddMember("usePush", false, allocator);
	else
		arg.AddMember("usePush", "", allocator);

	if(Parameter::paramPushDescription.soundInt == 1)
		arg.AddMember("sound", true, allocator);
	else if(Parameter::paramPushDescription.soundInt == 2)
		arg.AddMember("sound", false, allocator);
	else
		arg.AddMember("sound", "", allocator);

	if(Parameter::paramPushDescription.vibInt == 1)
		arg.AddMember("vibrate", true, allocator);
	else if(Parameter::paramPushDescription.vibInt == 2)
		arg.AddMember("vibrate", false, allocator);
	else
		arg.AddMember("vibrate", "", allocator);

#ifdef __ANDROID__
	if(Parameter::paramPushDescription.operGcmInt == 1)
		arg.AddMember("operationGCM", true, allocator);
	else if(Parameter::paramPushDescription.operGcmInt == 2)
		arg.AddMember("operationGCM", false, allocator);
	else
		arg.AddMember("operationGCM", "", allocator);
#else
	if(Parameter::paramPushDescription.operGcmInt == 1)
		arg.AddMember("operationRemote", true, allocator);
	else if(Parameter::paramPushDescription.operGcmInt == 2)
		arg.AddMember("operationRemote", false, allocator);
	else
		arg.AddMember("operationRemote", "", allocator);
#endif
	//
	if(Parameter::paramPushDescription.operLocInt == 1)
		arg.AddMember("operationLocal", true, allocator);
	else if(Parameter::paramPushDescription.operLocInt == 2)
		arg.AddMember("operationLocal", false, allocator);
	else
		arg.AddMember("operationLocal", "", allocator);

	if(Parameter::paramPushDescription.noticeInt == 1)
		arg.AddMember("useNotice", true, allocator);
	else if(Parameter::paramPushDescription.noticeInt == 2)
		arg.AddMember("useNotice", false, allocator);
	else {
		// NULL 인경우 arg에서 제외
	}

	if(Parameter::paramPushDescription.nightInt == 1)
		arg.AddMember("useNight", true, allocator);
	else if(Parameter::paramPushDescription.nightInt == 2)
		arg.AddMember("useNight", false, allocator);
	else {
		// NULL 인 경우 arg에서 제외
	}
//	Json::Value arg;
//
//	arg["usePush"] = Parameter::paramPushDescription.usePush;
//	arg["sound"] = Parameter::paramPushDescription.sound;
//	arg["vibrate"] = Parameter::paramPushDescription.vibrate;
//	arg["operationGCM"] = Parameter::paramPushDescription.operationGCM;
//	arg["operationLocal"] = Parameter::paramPushDescription.operationLocal;
//	arg["useNotice"] = Parameter::paramPushDescription.useNotice;					// 1.8.0+
//	arg["useNight"] = Parameter::paramPushDescription.useNight;						// 1.8.0+
//
//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("### strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (pushDescription)
	LogController::addLog("[Request : C2SModulePush_Description] >>>>>\n" + strJSON);

	if( ::C2SModulePush_Description(strJSON.c_str(), &error) == false )
	{
 		log("====>>>> Push description error (%s)", error.get()->GetMessage());
 		// 로그 정보 추가 (pushDescription : error)
 		std::stringstream errorCode;
 		errorCode << error.get()->GetCode();
 		LogController::addLog("[error : C2SModulePush_Description] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
 		// QA 로그 정보 추가
 		LogController::setApiResultState(true, error.get()->GetCode());
 	}
}

void ApiAction::pushRegisterList()
{
	int after = 5;
	for(int pushCnt = 1000001; pushCnt <= 1000050; pushCnt++) {
		log("====>>>> ApiAction::pushRegisterList()");
		Document arg;
		arg.SetObject();
		Document::AllocatorType& allocator = arg.GetAllocator();

		std::string pushCntMessage = std::to_string(pushCnt);

		arg.AddMember("pushId", pushCnt, allocator);
		arg.AddMember("title", StringRef(pushCntMessage.c_str()), allocator);
		arg.AddMember("message", StringRef(Parameter::paramPushRegister.message.c_str()), allocator);
		arg.AddMember("ticker", "알림 도착", allocator);
		arg.AddMember("type", StringRef(Parameter::paramPushRegister.type.c_str()), allocator);
		arg.AddMember("after", after, allocator);
		after += 5;

		StringBuffer buffer;
		Writer<StringBuffer> writer(buffer);
		arg.Accept(writer);

		std::string strJSON = buffer.GetString();

		C2SModule_Cocos2dx::C2SModuleError error;
		log("### strJSON : %s", strJSON.c_str());

		// 로그 정보 추가 (pushRegister)
		LogController::addLog("[Request : C2SModulePush_Register] >>>>>\n" + strJSON);

		if (::C2SModulePush_Register(strJSON.c_str(), &error) == false) {
			log("====>>>> Push register error (%s)", error.get()->GetMessage());
			// 로그 정보 추가 (pushRegister : error)
			std::stringstream errorCode;
			errorCode << error.get()->GetCode();
			LogController::addLog("[error : C2SModulePush_Register] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
			// QA 로그 정보 추가
			LogController::setApiResultState(true, error.get()->GetCode());
		}
	}
}

void ApiAction::pushUnregisterList()
{
	for(int pushCnt = 1000001; pushCnt <= 1000070; pushCnt++) {
		log("====>>>> ApiAction::pushUnregisterList()");
		Document arg;
		arg.SetObject();
		Document::AllocatorType &allocator = arg.GetAllocator();

		arg.AddMember("pushId", pushCnt, allocator);

		StringBuffer buffer;
		Writer<StringBuffer> writer(buffer);
		arg.Accept(writer);

		std::string strJSON = buffer.GetString();

		C2SModule_Cocos2dx::C2SModuleError error;
		log("### strJSON : %s", strJSON.c_str());

		// 로그 정보 추가 (pushUnregister)
		LogController::addLog("[Request : C2SModulePush_Unregister] >>>>>\n" + strJSON);

		if (::C2SModulePush_Unregister(strJSON.c_str(), &error) == false) {
			log("====>>>> Push unregister error (%s)", error.get()->GetMessage());
			// 로그 정보 추가 (pushUnregister : error)
			std::stringstream errorCode;
			errorCode << error.get()->GetCode();
			LogController::addLog(
					"[error : C2SModulePush_Unregister] >>>>>\nerror (" + errorCode.str() + ")(" +
					error.get()->GetMessage() + ")");
			// QA 로그 정보 추가
			LogController::setApiResultState(true, error.get()->GetCode());
		}
	}
}

void ApiAction::pushRequestPermission()
{
    log("====>>>> ApiAction::pushRequestPermission()");
    std::string strJSON = "";
    C2SModule_Cocos2dx::C2SModuleError error;

    // 로그 정보 추가 (pushRequestPermission)
    LogController::addLog("[Request : C2SModulePush_RequestPermission] >>>>>\n" + strJSON);
	
    if(::C2SModulePush_RequestPermission(strJSON.c_str(), &error) == false) {
        log("====>>>> Push requestPermission error (%s)", error.get()->GetMessage());
        // 로그 정보 추가 (pushUnregister : error)
        std::stringstream errorCode;
        errorCode << error.get()->GetCode();
        LogController::addLog(
                "[error : C2SModulePush_RequestPermission] >>>>>\nerror (" + errorCode.str() + ")(" +
                error.get()->GetMessage() + ")");
        // QA 로그 정보 추가
        LogController::setApiResultState(true, error.get()->GetCode());
    }
}

void ApiAction::snsUser()
{
	log("====>>>> ApiAction::snsUser()");

	if(Parameter::paramSnsUser.callState == "Hive") {		// [1] HIVE USER 정보 가져오기 (별도의 arg 값 없음.)
		if(Parameter::paramSnsUser.vid == "") {					// [1-1] 자신의 HIVE 계정 정보 가져오기
			log("### Sns user, vid is empty");
			C2SModule_Cocos2dx::C2SModuleError error;

			// 로그 정보 추가 (snsUser)
			LogController::addLog("[Request : C2SModuleSns_User] >>>>>\nNULL");

			if (::C2SModuleSns_User((char*) NULL, &error) == false) {
				log("====>>>> HIVE(no vids). User Show error (%s)", error.get()->GetMessage());
				log("====>>>> HIVE(no vids). User Show error code (%d)", error.get()->GetCode());
				// 로그 정보 추가 (snsUser : error)
				std::stringstream errorCode;
				errorCode << error.get()->GetCode();
				LogController::addLog("[error : C2SModuleSns_User] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
				// QA 로그 정보 추가
				LogController::setApiResultState(true, error.get()->GetCode());
			}
		} else {                                                // [1-2] 특정 vids의 HIVE 계정 정보 가져오기 (vid 값 설정.)
            log("### Sns user, vid is not empty");
            Document arg;
            arg.SetObject();
            Document::AllocatorType& allocator = arg.GetAllocator();

            rapidjson::Value argVidList(rapidjson::kArrayType); // 주의! kArrayType 사용

            std::string s = Parameter::paramSnsUser.vid;
            std::string delimiter = ",";

            size_t pos = 0;
            std::string token;
            bool existDelimiter = false;
            while ((pos = s.find(delimiter)) != std::string::npos) {
                log("# vid splitting...");
                token = s.substr(0, pos);
                log("# vid data : %s", token.c_str());
                argVidList.PushBack(StringRef(token.c_str()), allocator);
                s.erase(0, pos + delimiter.length());
                existDelimiter = true;
            }
            if(existDelimiter)
                argVidList.PushBack(StringRef(s.c_str()), allocator);

            if(existDelimiter) {
                log("# vid (multi) #");
                arg.AddMember("vid", argVidList, allocator);
            }
            else {
                log("# vid (single) #");
                arg.AddMember("vid", StringRef(Parameter::paramSnsUser.vid.c_str()), allocator);
            }
//			Json::Value arg;
//			Json::Value vidList;
//			vidList.append("106265878");
//			vidList.append("107649702");
//			arg["vid"] = Parameter::paramSnsUser.vid;
//			arg["vid"] = vidList;		// TODO : test code, vid 2개 이상 리스트로 전송 (/users API 작동 확인)

//			Json::StyledWriter writer;
//			std::string strJSON = writer.write(arg);
            StringBuffer buffer;
            Writer<StringBuffer> writer(buffer);
            arg.Accept(writer);

            std::string strJSON = buffer.GetString();
            log("### vid : %s", strJSON.c_str());

			C2SModule_Cocos2dx::C2SModuleError error;

			// 로그 정보 추가 (snsUser)
			LogController::addLog("[Request : C2SModuleSns_User] >>>>>\n" + strJSON);

			if( ::C2SModuleSns_User( strJSON.c_str(), &error ) == false ) {
				log( "====>>>> HIVE(exist vids). User Show error (%s)", error.get()->GetMessage() );
				log( "====>>>> HIVE(exist vids). User Show error code (%d)", error.get()->GetCode() );
				// 로그 정보 추가 (snsUser : error)
				std::stringstream errorCode;
				errorCode << error.get()->GetCode();
				LogController::addLog("[error : C2SModuleSns_User] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
				// QA 로그 정보 추가
				LogController::setApiResultState(true, error.get()->GetCode());
			}
		}
	}
	else if(Parameter::paramSnsUser.callState == "facebook") {		// [2] Facebook 정보 가져오기
		if(Parameter::paramSnsUser.facebookId == "") {					// [2-1] 자기자신의 Facebook 정보 요청
            Document arg;
            arg.SetObject();
            Document::AllocatorType& allocator = arg.GetAllocator();

            arg.AddMember("type", "facebook", allocator);
//		    Json::Value arg;
//			arg["type"] = "facebook";

//			Json::StyledWriter writer;
//			std::string strJSON = writer.write(arg);
            StringBuffer buffer;
            Writer<StringBuffer> writer(buffer);
            arg.Accept(writer);

            std::string strJSON = buffer.GetString();

			C2SModule_Cocos2dx::C2SModuleError error;

			// 로그 정보 추가 (snsUser)
			LogController::addLog("[Request : C2SModuleSns_User] >>>>>\n" + strJSON);

			if( ::C2SModuleSns_User( strJSON.c_str(), &error ) == false ) {
				log( "====>>>> Facebook(no facebookId). User Show error (%s)", error.get()->GetMessage() );
				log( "====>>>> Facebook(no facebookId). User Show error code (%d)", error.get()->GetCode() );
				// 로그 정보 추가 (snsUser : error)
				std::stringstream errorCode;
				errorCode << error.get()->GetCode();
				LogController::addLog("[error : C2SModuleSns_User] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
				// QA 로그 정보 추가
				LogController::setApiResultState(true, error.get()->GetCode());
			}
		} else {														// [2-2] 특정 facebookId의 정보 가져오기
            Document arg;
            arg.SetObject();
            Document::AllocatorType& allocator = arg.GetAllocator();

            arg.AddMember("type", "facebook", allocator);
            arg.AddMember("facebookId", StringRef(Parameter::paramSnsUser.facebookId.c_str()), allocator);

//		    Json::Value arg;
//			arg["type"] = "facebook";
//			arg["facebookId"] = Parameter::paramSnsUser.facebookId;

//			Json::StyledWriter writer;
//			std::string strJSON = writer.write(arg);
            StringBuffer buffer;
            Writer<StringBuffer> writer(buffer);
            arg.Accept(writer);

            std::string strJSON = buffer.GetString();

			C2SModule_Cocos2dx::C2SModuleError error;

			// 로그 정보 추가 (snsUser)
			LogController::addLog("[Request : C2SModuleSns_User] >>>>>\n" + strJSON);

			if( ::C2SModuleSns_User( strJSON.c_str(), &error ) == false ) {
				log( "====>>>> Facebook(exist facebookId). User Show error (%s)", error.get()->GetMessage() );
				log( "====>>>> Facebook(exist facebookId). User Show error code (%d)", error.get()->GetCode() );
				// 로그 정보 추가 (snsUser : error)
				std::stringstream errorCode;
				errorCode << error.get()->GetCode();
				LogController::addLog("[error : C2SModuleSns_User] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
				// QA 로그 정보 추가
				LogController::setApiResultState(true, error.get()->GetCode());
			}
		}
	}
	else if(Parameter::paramSnsUser.callState == "message") {		// [3] 본인의 받은 새 메세지 갯수 요청
        Document arg;
        arg.SetObject();
        Document::AllocatorType& allocator = arg.GetAllocator();

        arg.AddMember("type", "message", allocator);
//
//	    Json::Value arg;
//		arg["type"] = "message";

//		Json::StyledWriter writer;
//		std::string strJSON = writer.write(arg);
        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();

		C2SModule_Cocos2dx::C2SModuleError error;

		// 로그 정보 추가 (snsUser)
		LogController::addLog("[Request : C2SModuleSns_User] >>>>>\n" + strJSON);

		if( ::C2SModuleSns_User( strJSON.c_str(), &error ) == false ) {
			log("====>>>> Message. User Show error (%s)", error.get()->GetMessage() );
			log("====>>>> Message. User Show error code (%d)", error.get()->GetCode() );
			// 로그 정보 추가 (snsUser : error)
			std::stringstream errorCode;
			errorCode << error.get()->GetCode();
			LogController::addLog("[error : C2SModuleSns_User] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
			// QA 로그 정보 추가
			LogController::setApiResultState(true, error.get()->GetCode());
		}
	}
}

void ApiAction::snsUserDelete()
{
	log("====>>>> ApiAction::snsUserDelete()");

	C2SModule_Cocos2dx::C2SModuleError error;

	// 로그 정보 추가 (snsUser)
	LogController::addLog("[Request : C2SModuleSns_UserDelete] >>>>>\nNULL");

	if (::C2SModuleSns_UserDelete((char*) NULL, &error) == false) {
		log("====>>>> HIVE(no vids). UserDelete Show error (%s)", error.get()->GetMessage());
		log("====>>>> HIVE(no vids). UserDelete Show error code (%d)", error.get()->GetCode());
		// 로그 정보 추가 (snsUserDelete : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleSns_UserDelete] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

void ApiAction::snsPosting()
{
	log("====>>>> ApiAction::snsPosting()");
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

//	Json::Value arg;

	// [0] HIVE ==> 포스팅 기능은 삭제되었습니다.

	if(Parameter::paramSnsPosting.service == "facebook") {				// [1] facebook
		log("### service : facebook");
        arg.AddMember("service", StringRef(Parameter::paramSnsPosting.service.c_str()), allocator);
//		arg["service"] = Parameter::paramSnsPosting.service;

		// HIVE SDK 1.12.1부터 Facebook 포스팅시 message, title, imageUrl을 arg로 사용하지 않음.
		/*
		arg["message"] = Parameter::paramSnsPosting.message;
		arg["title"] = Parameter::paramSnsPosting.title;

		// base64 encoded data string 처리
		std::string str;
		ssize_t pSize = 0;
		unsigned char*filecont =  FileUtils::sharedFileUtils()->getFileData("HelloWorld.png", "r", &pSize);
		str = base64_encode(filecont, pSize);
		log("### image(encoding) success : %s", str.c_str());
		arg["image"] = str;
		 */

//		arg["appLinkURL"] = Parameter::paramSnsPosting.appLinkURL;
        arg.AddMember("appLinkURL", StringRef(Parameter::paramSnsPosting.appLinkURL.c_str()), allocator);
	} else if(Parameter::paramSnsPosting.service == "googleplus") {		// [2] google plus
		log("### service : googleplus");
//		arg["service"] = Parameter::paramSnsPosting.service;
//		arg["message"] = Parameter::paramSnsPosting.message;

        arg.AddMember("service", StringRef(Parameter::paramSnsPosting.service.c_str()), allocator);
        arg.AddMember("message", StringRef(Parameter::paramSnsPosting.message.c_str()), allocator);

		// TODO : test code
		// base64 encoded data string 처리
//		std::string str;
//		ssize_t pSize = 0;
//		unsigned char*filecont =  FileUtils::sharedFileUtils()->getFileData("HelloWorld.png", "r", &pSize);
//		str = base64_encode(filecont, pSize);
//		log("### image(encoding) success : %s", str.c_str());
//		arg["image"] = str;
	}

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("### strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (snsPosting)
	LogController::addLog("[Request : C2SModuleSns_Post] >>>>>\n" + strJSON);

	if (::C2SModuleSns_Post(strJSON.c_str(), &error) == false) {
		log("====>>>> Sns posting error (%s)", error.get()->GetMessage());
		log("====>>>> Sns posting error code (%d)", error.get()->GetCode());
		// 로그 정보 추가 (snsUser : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleSns_Post] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

// Sns_Posting : 이미지 첨부를 위한 base64_encoding 변수
static const std::string base64_chars =
"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
"abcdefghijklmnopqrstuvwxyz"
"0123456789+/";

// Sns_Posting : 이미지 첨부를 위한 base64_encoding 함수
std::string ApiAction::base64_encode(unsigned char const* bytes_to_encode, unsigned int in_len) {
    std::string ret;
    int i = 0;
    int j = 0;
    unsigned char char_array_3[3];
    unsigned char char_array_4[4];

    while (in_len--) {
        char_array_3[i++] = *(bytes_to_encode++);
        if (i == 3) {
            char_array_4[0] = (char_array_3[0] & 0xfc) >> 2;
            char_array_4[1] = ((char_array_3[0] & 0x03) << 4) + ((char_array_3[1] & 0xf0) >> 4);
            char_array_4[2] = ((char_array_3[1] & 0x0f) << 2) + ((char_array_3[2] & 0xc0) >> 6);
            char_array_4[3] = char_array_3[2] & 0x3f;

            for(i = 0; (i <4) ; i++)
                ret += base64_chars[char_array_4[i]];
            i = 0;
        }
    }

    if (i)
    {
        for(j = i; j < 3; j++)
            char_array_3[j] = '\0';

        char_array_4[0] = (char_array_3[0] & 0xfc) >> 2;
        char_array_4[1] = ((char_array_3[0] & 0x03) << 4) + ((char_array_3[1] & 0xf0) >> 4);
        char_array_4[2] = ((char_array_3[1] & 0x0f) << 2) + ((char_array_3[2] & 0xc0) >> 6);
        char_array_4[3] = char_array_3[2] & 0x3f;

        for (j = 0; (j < i + 1); j++)
            ret += base64_chars[char_array_4[j]];

        while((i++ < 3))
            ret += '=';
    }
    return ret;
}

void ApiAction::snsSetGuestUserInfo()		// HIVE 1.10.1 : Hoolai(훌라이)에서 게스트 정보를 정식 로그인처럼 사용하기 위함.
{
	log(" ====>>>> ApiAction::snsSetGuestUserInfo()");
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

    arg.AddMember("guestVid",  StringRef(Parameter::paramLogin.hoolaiVid.c_str()), allocator);
    arg.AddMember("guestSessionToken",  StringRef(Parameter::paramLogin.hoolaiSessionKey.c_str()), allocator);

//	Json::Value arg;
//
//	arg["guestVid"] = Parameter::paramLogin.hoolaiVid;
//	arg["guestSessionToken"] = Parameter::paramLogin.hoolaiSessionKey;

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("### strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (snsSetGuestUserInfo)
	LogController::addLog("[Request : C2SModuleSns_SetGuestUserInfo] >>>>>\n" + strJSON);

	if( ::C2SModuleSns_SetGuestUserInfo(strJSON.c_str(), &error) == false) {
		log("====>>>> Sns SetGuestUserInfo error (%s)", error.get()->GetMessage());
		// 로그 정보 추가 (snsSetGuestUserInfo : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleSns_SetGuestUserInfo] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());

		MessageBox("Sns_SetGuestUserInfo : 에러!", "Error");
	} else {
		MessageBox("Sns_SetGuestUserInfo : 호출 완료!", "Success");
	}
}

void ApiAction::promotionShow()
{
	log("====>>>> ApiAction::promotionShow()");
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

//	Json::Value arg;

//	arg["type"] = Parameter::paramPromotionShow.type;
    arg.AddMember("type", StringRef(Parameter::paramPromotionShow.type.c_str()), allocator);

	if(Parameter::paramPromotionShow.type == "notice") {					// 공지
//		arg["forced"] = Parameter::paramPromotionShow.forced;
        arg.AddMember("forced",  Parameter::paramPromotionShow.forced, allocator);
	} else if(Parameter::paramPromotionShow.type == "event") {				// 이벤트
//		arg["forced"] = Parameter::paramPromotionShow.forced;
        arg.AddMember("forced",  Parameter::paramPromotionShow.forced, allocator);
	} else if(Parameter::paramPromotionShow.type == "customWebview") {		// 커스텀 웹뷰
//		arg["viewId"] = Parameter::paramPromotionShow.viewId;
        arg.AddMember("viewId",  Parameter::paramPromotionShow.viewId, allocator);
	} else if(Parameter::paramPromotionShow.type == "fullBanner") {
		arg.AddMember("fullbannerLegacy",  Parameter::paramPromotionShow.useLegacyBanner, allocator);
	}

//    rapidjson::Value argAddInfo(rapidjson::kObjectType);
//    argAddInfo.AddMember("server", "0", allocator);
//    argAddInfo.AddMember("character", "0", allocator);
//    arg.AddMember("additionalInfo", argAddInfo, allocator);

    arg.AddMember("additionalInfo", StringRef(Parameter::paramHivePromotion.additionalInfo.c_str()), allocator);

//	Json::FastWriter writer;
//	Json::Value argAddInfo;
//	argAddInfo["server"] = "0";
//	argAddInfo["character"] = "0";
//	arg["additionalInfo"] = writer.write(argAddInfo);
//	arg["additionalInfo"] = "{\n\t\"server\": \"SEA-1\",\n\t\"character\": \"0\"\n}";		 // TODO : test code, additionalInfo 데이터 테스트

//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (promotionShow)
	LogController::addLog("[Request : C2SModulePromotion_Show] >>>>>\n" + strJSON);

	if( ::C2SModulePromotion_Show( strJSON.c_str(), &error ) == false ) {
		log("====>>>> Mercury error (%s)", error.get()->GetMessage() );
		log("====>>>> Promotion Show Error");
		// 로그 정보 추가 (promotionShow : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModulePromotion_Show] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

void ApiAction::promotionInfo()
{
	if(Parameter::paramPromotionInfo.callState == "PromotionInfo") {			// [1] 프로모션 정보(커스텀 웹뷰를 제외하고)
        Document arg;
        arg.SetObject();
        Document::AllocatorType& allocator = arg.GetAllocator();

        arg.AddMember("type", StringRef(Parameter::paramPromotionInfo.type.c_str()), allocator);
        arg.AddMember("viewId", StringRef(Parameter::paramPromotionInfo.viewId_string.c_str()), allocator);
//	    Json::Value arg;
//		Json::Value argAddInfo;

//		arg["type"] = Parameter::paramPromotionInfo.type;
//		arg["viewId"] = Parameter::paramPromotionInfo.viewId_string;

//		Json::FastWriter writer;
//		std::string strJSON = writer.write(arg);
        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();

		C2SModule_Cocos2dx::C2SModuleError error;
		log("====>>>> strJSON : %s", strJSON.c_str());

		// 로그 정보 추가 (promotionInfo)
		LogController::addLog("[Request : C2SModulePromotion_Info] >>>>>\n" + strJSON);

		if( ::C2SModulePromotion_Info( strJSON.c_str(), &error ) == false ) {
			log( "====>>>> Mercury error (%s)", error.get()->GetMessage() );
			log("##### Promotion Info Error #####");
			// 로그 정보 추가 (promotionInfo : error)
			std::stringstream errorCode;
			errorCode << error.get()->GetCode();
			LogController::addLog("[error : C2SModulePromotion_Info] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
			// QA 로그 정보 추가
			LogController::setApiResultState(true, error.get()->GetCode());
		}
	}
	else if(Parameter::paramPromotionInfo.callState == "customWebview") {		// [2] 프로모션 정보(커스텀 웹뷰)
        Document arg;
        arg.SetObject();
        Document::AllocatorType& allocator = arg.GetAllocator();

        arg.AddMember("type", StringRef(Parameter::paramPromotionInfo.type.c_str()), allocator);
        arg.AddMember("viewId", Parameter::paramPromotionInfo.viewId_int, allocator);

//	    Json::Value arg;
//		Json::Value argAddInfo;

//		arg["type"] = Parameter::paramPromotionInfo.type;
//		arg["viewId"] = Parameter::paramPromotionInfo.viewId_int;

//		Json::FastWriter writer;
//		std::string strJSON = writer.write(arg);
        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();

		C2SModule_Cocos2dx::C2SModuleError error;
		log("====>>>> strJSON : %s", strJSON.c_str());

		// 로그 정보 추가 (promotionInfo)
		LogController::addLog("[Request : C2SModulePromotion_Info] >>>>>\n" + strJSON);

		if( ::C2SModulePromotion_Info( strJSON.c_str(), &error ) == false ) {
			log("====>>>> Mercury error (%s)", error.get()->GetMessage() );
			log("##### Promotion Info Error #####");
			// 로그 정보 추가 (promotionInfo : error)
			std::stringstream errorCode;
			errorCode << error.get()->GetCode();
			LogController::addLog("[error : C2SModulePromotion_Info] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
			// QA 로그 정보 추가
			LogController::setApiResultState(true, error.get()->GetCode());
		}
	}else if(Parameter::paramPromotionInfo.callState == "GetOfferwallState") {	// [3] GetOfferwallState
		// 로그 정보 추가 (getofferwallState)
		LogController::addLog("[Request : GetOfferwallState] >>>>>\nnot request");

		// Log 확인.
		log("##### GetOfferwallState() 진입 #####");

		if(::GetOfferwallState() == -14) {
			log("===>>> GetOfferwallState() = -14");
		} else if(::GetOfferwallState() == -15) {
			log("===>>> GetOfferwallState() = -15");
		}
		// 로그 정보 추가 (getofferwallState : result)
		std::stringstream getOfferwallStateValue;
		getOfferwallStateValue << ::GetOfferwallState();
		LogController::addLog("[Result : GetOfferwallState] >>>>>\n GetOfferwallState (" + getOfferwallStateValue.str() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(false, 0);
	}
}

void ApiAction::snsFriends()
{
	log("====>>>> ApiAction::snsFriends()");
//	Json::Value arg;
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

	if(Parameter::paramSnsFriends.service == "") {					// [1] service -> HIVE
//		arg["type"] = Parameter::paramSnsFriends.type;
        arg.AddMember("type", StringRef(Parameter::paramSnsFriends.type.c_str()), allocator);
		if(Parameter::paramSnsFriends.type == "getInvitations") {
			// TODO : limit 값 설정할 수 있게 변경 필요.
//			arg["limit"] = 0;
		}
	} else if(Parameter::paramSnsFriends.service == "facebook") {	// [2] service -> facebook
        arg.AddMember("type", StringRef(Parameter::paramSnsFriends.type.c_str()), allocator);
        arg.AddMember("service", StringRef(Parameter::paramSnsFriends.service.c_str()), allocator);

//	    arg["type"] = Parameter::paramSnsFriends.type;
//		arg["service"] = Parameter::paramSnsFriends.service;
		if(Parameter::paramSnsFriends.callState != "friendsList") {
			if(Parameter::paramSnsFriends.limit != 0) {
                arg.AddMember("limit", Parameter::paramSnsFriends.limit, allocator);
//				arg["limit"] = Parameter::paramSnsFriends.limit;
			} else {
				// 0일 경우 가장 최근에 초대한 1명의 정보만 가져옵니다..
			}
		}
	} else if(Parameter::paramSnsFriends.service == "googleplus") { // [3] service -> google plus
        arg.AddMember("type", StringRef(Parameter::paramSnsFriends.type.c_str()), allocator);
        arg.AddMember("service", StringRef(Parameter::paramSnsFriends.service.c_str()), allocator);

//	    arg["type"] = Parameter::paramSnsFriends.type;
//		arg["service"] = Parameter::paramSnsFriends.service;
	}

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (snsFriends)
	LogController::addLog("[Request : C2SModuleSns_Friends] >>>>>\n" + strJSON);

	if( ::C2SModuleSns_Friends(strJSON.c_str(), &error ) == false )
	{
		log( "====>>>> Friends list error" );
		// 로그 정보 추가 (snsFriends : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleSns_Friends] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

void ApiAction::snsMessage()
{
	log("====>>>> ApiAction::snsMessage()");

	if(Parameter::paramPromotionAcquisition.callState == 5 || Parameter::paramPromotionAcquisition.callState == 6) {		// HIVE 1.9.0, 집객 기능에서 '하이브 초대 Link 공유' 호출시.
        Document arg;
        arg.SetObject();
        Document::AllocatorType& allocator = arg.GetAllocator();
//	    Json::Value arg;

		if(Parameter::paramPromotionAcquisition.callState == 6) {
            arg.AddMember("message", StringRef(Parameter::paramPromotionAcquisition.message.c_str()), allocator);
//            arg["message"] = "프로모션 집객 링크 + 메시지 테스트중...";
        }
		else
        {
//            arg["message"] = "";
            arg.AddMember("message", StringRef(Parameter::paramSnsMessage.message.c_str()), allocator);
        }

//		arg["invite"] = Parameter::paramPromotionAcquisition.invite;
        arg.AddMember("invite", Parameter::paramPromotionAcquisition.invite, allocator);
		if(Parameter::paramPromotionAcquisition.vidoruid == true) {
//			arg["vid"] = Parameter::paramPromotionAcquisition.vid;
            arg.AddMember("vid", StringRef(Parameter::paramPromotionAcquisition.vid.c_str()), allocator);
		} else if(Parameter::paramPromotionAcquisition.vidoruid == false) {
//			arg["uid"] = Parameter::paramPromotionAcquisition.vid;
            arg.AddMember("uid", StringRef(Parameter::paramPromotionAcquisition.vid.c_str()), allocator);
		}

        arg.AddMember("linkShare", true, allocator);
        arg.AddMember("linkShareURL", StringRef(Parameter::paramPromotionAcquisition.inviteHivemsgLink.c_str()), allocator);
        arg.AddMember("imageURL", StringRef(Parameter::paramPromotionAcquisition.imageURL.c_str()), allocator);
        arg.AddMember("thumbnailURL", StringRef(Parameter::paramPromotionAcquisition.thumbnailURL.c_str()), allocator);
        arg.AddMember("usePush", Parameter::paramPromotionAcquisition.usePush, allocator);

//		arg["linkShare"] = true;/*
//		arg["linkShareURL"] = Parameter::paramPromotionAcquisition.inviteHivemsgLink;
//
//		arg["imageURL"] = Parameter::paramPromotionAcquisition.imageURL;
//		arg["thumbnailURL"] = Parameter::paramPromotionAcquisition.thumbnailURL;
//		arg["usePush"] = Parameter::paramPromotionAcquisition.usePush;*/

		// reset _ acquisition callState
		Parameter::paramPromotionAcquisition.callState = 0;

//		Json::StyledWriter writer;
//		std::string strJSON = writer.write(arg);
        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();

		C2SModule_Cocos2dx::C2SModuleError error;
		log("====>>>> strJSON : %s", strJSON.c_str());

		// 로그 정보 추가 (snsMessage)
		LogController::addLog("[Request : C2SModuleSns_Message] >>>>>\n" + strJSON);

		if( ::C2SModuleSns_Message(strJSON.c_str(), &error ) == false )
		{
			log( "====>>>> Sns_Message(for Promotion Acquisition HIVE INVITE LINK) error" );
			// 로그 정보 추가 (snsMessage : error)
			std::stringstream errorCode;
			errorCode << error.get()->GetCode();
			LogController::addLog("[error : C2SModuleSns_Message] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
			// QA 로그 정보 추가
			LogController::setApiResultState(true, error.get()->GetCode());
		}
	}else if(Parameter::paramSnsMessage.callState == "Hive") {				// [1] HIVE 메시지 전송
        Document arg;
        arg.SetObject();
        Document::AllocatorType& allocator = arg.GetAllocator();

//	    Json::Value arg;
		if(Parameter::paramSnsMessage.viduid == true) {
//			arg["vid"] = Parameter::paramSnsMessage.vid;
            arg.AddMember("vid", StringRef(Parameter::paramSnsMessage.vid.c_str()), allocator);
		}else if(Parameter::paramSnsMessage.viduid == false) {
//			arg["uid"] = Parameter::paramSnsMessage.vid;
            arg.AddMember("uid", StringRef(Parameter::paramSnsMessage.vid.c_str()), allocator);
		}

		// TODO : test code (리스트 타입으로 넣기)
//		Json::Value vidList;
//		vidList.append("61978454");				// mystic0707 : Live
//		vidList.append("40076632");				// mystic0707 : Sandbox
//		arg["vid"] = vidList;
//		Json::Value uidList;
//		uidList.append("122820361");
//		arg["uid"] = uidList;

        arg.AddMember("message", StringRef(Parameter::paramSnsMessage.message.c_str()), allocator);
        arg.AddMember("imageURL", StringRef(Parameter::paramSnsMessage.imageURL.c_str()), allocator);
        arg.AddMember("thumbnailURL", StringRef(Parameter::paramSnsMessage.thumbnailURL.c_str()), allocator);
        arg.AddMember("usePush", Parameter::paramSnsMessage.usePush, allocator);
        arg.AddMember("invite", Parameter::paramSnsMessage.invite, allocator);

//		arg["message"] = Parameter::paramSnsMessage.message;
//		arg["imageURL"] = Parameter::paramSnsMessage.imageURL;
//		arg["thumbnailURL"] = Parameter::paramSnsMessage.thumbnailURL;
//		arg["usePush"] = Parameter::paramSnsMessage.usePush;
//		arg["invite"] = Parameter::paramSnsMessage.invite;

//		Json::StyledWriter writer;
//		std::string strJSON = writer.write(arg);
        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();

		C2SModule_Cocos2dx::C2SModuleError error;
		log("====>>>> strJSON : %s", strJSON.c_str());

		// 로그 정보 추가 (snsMessage)
		LogController::addLog("[Request : C2SModuleSns_Message] >>>>>\n" + strJSON);

		if( ::C2SModuleSns_Message(strJSON.c_str(), &error ) == false )
		{
			log( "====>>>> Sns_Message(Hive) error" );
			// 로그 정보 추가 (snsMessage : error)
			std::stringstream errorCode;
			errorCode << error.get()->GetCode();
			LogController::addLog("[error : C2SModuleSns_Message] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
			// QA 로그 정보 추가
			LogController::setApiResultState(true, error.get()->GetCode());
		}
	}else if(Parameter::paramSnsMessage.callState == "facebook") {			// [2] Facebook 메시지 전송
        Document arg;
        arg.SetObject();
        Document::AllocatorType& allocator = arg.GetAllocator();

//	    Json::Value arg;

        arg.AddMember("service", StringRef(Parameter::paramSnsMessage.service.c_str()), allocator);
        arg.AddMember("type", StringRef(Parameter::paramSnsMessage.type.c_str()), allocator);
        arg.AddMember("message", StringRef(Parameter::paramSnsMessage.message.c_str()), allocator);

//		arg["service"] = Parameter::paramSnsMessage.service;
//		arg["type"] = Parameter::paramSnsMessage.type;
//		arg["message"] = Parameter::paramSnsMessage.message;

//		Json::StyledWriter writer;
//		std::string strJSON = writer.write(arg);
        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();

		C2SModule_Cocos2dx::C2SModuleError error;
		log("====>>>> strJSON : %s", strJSON.c_str());

		// 로그 정보 추가 (snsMessage)
		LogController::addLog("[Request : C2SModuleSns_Message] >>>>>\n" + strJSON);

		if( ::C2SModuleSns_Message(strJSON.c_str(), &error ) == false )
		{
			log( "====>>>> Sns_Message(facebook) error" );
			// 로그 정보 추가 (snsMessage : error)
			std::stringstream errorCode;
			errorCode << error.get()->GetCode();
			LogController::addLog("[error : C2SModuleSns_Message] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
			// QA 로그 정보 추가
			LogController::setApiResultState(true, error.get()->GetCode());
		}
	}
}

void ApiAction::snsSession()
{
//	Json::Value arg;
//	Json::FastWriter writer;

    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

	C2SModule_Cocos2dx::C2SModuleError error;

	if(Parameter::paramSnsSession.service == "facebook") {
	    arg.AddMember("service", "facebook", allocator);
//		arg["service"] = "facebook"; 			// facebook 접속 유무

//		std::string strJSON = writer.write(arg);
        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();
		log("====>>>> strJSON : %s", strJSON.c_str());

		// 로그 정보 추가 (snsSession)
		LogController::addLog("[Request : C2SModuleSns_Session] >>>>>\n" + strJSON);

		if( ::C2SModuleSns_Session( strJSON.c_str(), &error ) == false ) {
			log( "====>>>> Sns Session error(facebook) (%s)", error.get()->GetMessage() );
			// 로그 정보 추가 (snsSession : error)
			std::stringstream errorCode;
			errorCode << error.get()->GetCode();
			LogController::addLog("[error : C2SModuleSns_Session] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
			// QA 로그 정보 추가
			LogController::setApiResultState(true, error.get()->GetCode());
		}
	} else {
		// 로그 정보 추가 (snsSession)				// HIVE 세션.
		LogController::addLog("[Request : C2SModuleSns_Session] >>>>>\nNULL");

		if( ::C2SModuleSns_Session( (char*)NULL, &error ) == false ) {
			log( "====>>>> Sns Session error(Hive) (%s)", error.get()->GetMessage() );
			// 로그 정보 추가 (snsSession : error)
			std::stringstream errorCode;
			errorCode << error.get()->GetCode();
			LogController::addLog("[error : C2SModuleSns_Session] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
			// QA 로그 정보 추가
			LogController::setApiResultState(true, error.get()->GetCode());
		}
	}
}

void ApiAction::snsShow()
{
//	Json::Value arg;

    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

//	arg["type"] = Parameter::paramSnsShow.type;
    arg.AddMember("type", StringRef(Parameter::paramSnsShow.type.c_str()), allocator);
	if(Parameter::paramSnsShow.type == "message") {
//		arg["uid"] = Parameter::paramSnsShow.uid;
        arg.AddMember("uid", StringRef(Parameter::paramSnsShow.uid.c_str()), allocator);
	}
	else if(Parameter::paramSnsShow.type == "chatbot") {			// HIVE 1.18.1
		arg.AddMember("additionalInfo", StringRef(Parameter::paramSnsShow.additionalInfo.c_str()), allocator);
	}

//	Json::FastWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (snsShow)
	LogController::addLog("[Request : C2SModuleSns_Show] >>>>>\n" + strJSON);

	if( ::C2SModuleSns_Show( strJSON.c_str(), &error ) == false ) {
		log( "====>>>> Sns Show error (%s)", error.get()->GetMessage() );
		// 로그 정보 추가 (snsShow : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleSns_Show] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

void ApiAction::socialShow()
{
	log("====>>>> socialShow");

    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();
//	Json::Value arg;

//	arg["type"] = Parameter::paramSocialShow.type;
    arg.AddMember("type", StringRef(Parameter::paramSocialShow.type.c_str()), allocator);
	if(Parameter::paramSocialShow.type == "adultAuth") {
//		arg["gameLanguage"] = Parameter::paramInitialize.gameLanguage;
        arg.AddMember("gameLanguage", StringRef(Parameter::paramInitialize.gameLanguage.c_str()), allocator);
	} else if(Parameter::paramSocialShow.type == "requestPermission") {		// HIVE 1.11.0
//		arg["isChecked"] = Parameter::paramSocialShow.isChecked;
		::CommonResultDelegate resultDelegate = DealCallback::ResultDelegate;
		::C2SModuleSocial_SetCommonResultDelegate(resultDelegate);
	} else if(Parameter::paramSocialShow.type == "requestPermissionList") {	// HIVE 1.17.2
		// Android 권한 추가 요청
		rapidjson::Value permissionList(rapidjson::kArrayType);

		// TODO : 정상적인 권한 요청
//		permissionList.PushBack("android.permission.READ_CONTACTS", allocator);
		permissionList.PushBack("android.permission.READ_PHONE_STATE", allocator);
		permissionList.PushBack("android.permission.WRITE_EXTERNAL_STORAGE", allocator);
        // TODO : 비정상적인 권한 요청
//        permissionList.PushBack("def", allocator);
//		permissionList.PushBack("abc", allocator);
//		permissionList.PushBack("android.permission.WRITE_EXTERNAL_STORAGE", allocator);

		arg.AddMember("requests", permissionList, allocator);
	}

	// HIVE IAP
	/*
	if(Parameter::paramSocialShow.type == "adultAuth") {
		arg["gameLanguage"] = Parameter::paramSocialShow.gameLanguage;
	}
	*/

//	Json::FastWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (socialShow)
	LogController::addLog("[Request : C2SModuleSocial_Show] >>>>>\n" + strJSON);

	if( ::C2SModuleSocial_Show( strJSON.c_str(), &error ) == false ) {
		log( "====>>>> Social Show error (%s)", error.get()->GetMessage() );
		// 로그 정보 추가 (socialShow : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleSocial_Show] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

void ApiAction::socialInfo()		// HIVE 1.10.0
{
	::CommonResultDelegate resultDelegate = DealCallback::ResultDelegate;
	::C2SModuleSocial_SetCommonResultDelegate(resultDelegate);

    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();


//    Json::Value arg;

//	arg["type"] = "notice";										// 추후에 다른 type이 입력된 경우 변수 지정.
//	arg["serverId"] = Parameter::paramInitialize.serverId;

//    arg.AddMember("type", "notice", allocator);
    if(Parameter::paramSocialInfo.type == "permission") {       // added HIVE 1.17.0
        arg.AddMember("type", StringRef(Parameter::paramSocialInfo.type.c_str()), allocator);
    }
    else if(Parameter::paramSocialInfo.type == "notice") {
        arg.AddMember("type", StringRef(Parameter::paramSocialInfo.type.c_str()), allocator);
    }

//	Json::FastWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (socialShow)
	LogController::addLog("[Request : C2SModuleSocial_Info] >>>>>\n" + strJSON);

	if( ::C2SModuleSocial_Info( strJSON.c_str(), &error ) == false ) {
		log( "====>>>> Social Info error (%s)", error.get()->GetMessage() );
		// 로그 정보 추가 (socialShow : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleSocial_Info] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

void ApiAction::inappPurchase()
{
	log("====>>>> inappPurchase");
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

//	Json::Value arg;

    arg.AddMember("pid", StringRef(Parameter::paramInappPurchase.pid.c_str()), allocator);
    arg.AddMember("quantity", Parameter::paramInappPurchase.quantity, allocator);

//	arg["pid"] = Parameter::paramInappPurchase.pid;
//	arg["quantity"] = Parameter::paramInappPurchase.quantity;
	if(Parameter::paramInappPurchase.additionalInfo != "") {
        arg.AddMember("additionalInfo", StringRef(Parameter::paramInappPurchase.additionalInfo.c_str()), allocator);
//	    arg["additionalInfo"] = Parameter::paramInappPurchase.additionalInfo;
	}

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (inappPurchase)
	LogController::addLog("[Request : C2SModuleInapp_Purchase] >>>>>\n" + strJSON);

//	if( ::C2SModuleInapp_Purchase( strJSON.c_str(), &error ) == false )
//	{
//		log( "====>>>> Inapp Purchase error (%s)", error.get()->GetMessage() );
//		// 로그 정보 추가 (inappPurchase : error)
//		std::stringstream errorCode;
//		errorCode << error.get()->GetCode();
//		LogController::addLog("[error : C2SModuleInapp_Purchase] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
//		// QA 로그 정보 추가
//		LogController::setApiResultState(true, error.get()->GetCode());
//	}
}

void ApiAction::inappFinish()
{
	log("====>>>> inappFinish");
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

//	Json::Value arg;

//	arg["isPurchaseSuccess"] = Parameter::paramInappFinish.isPurchaseSuccess;
    arg.AddMember("isPurchaseSuccess", Parameter::paramInappFinish.isPurchaseSuccess, allocator);

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (inappFinish)
	LogController::addLog("[Request : C2SModuleInapp_Finish] >>>>>\n" + strJSON);

//	if( ::C2SModuleInapp_Finish( strJSON.c_str(), &error ) == false )
//	{
//		log( "====>>>> Inapp Finish error (%s)", error.get()->GetMessage() );
//		// 로그 정보 추가 (inappFinish : error)
//		std::stringstream errorCode;
//		errorCode << error.get()->GetCode();
//		LogController::addLog("[error : C2SModuleInapp_Purchase] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
//		// QA 로그 정보 추가
//		LogController::setApiResultState(true, error.get()->GetCode());
//	}
}

void ApiAction::inappRestore()
{
	log("====>>>> inappRestore");

	C2SModule_Cocos2dx::C2SModuleError error;

	// 로그 정보 추가 (inappRestore)
	LogController::addLog("[Request : C2SModuleInapp_Restore] >>>>>\nNULL");

//	if( ::C2SModuleInapp_Restore( (char*)NULL, &error ) == false ) {
//		log( "====>>>> Inapp Restore error (%s)", error.get()->GetMessage() );
//		// 로그 정보 추가 (inappRestore : error)
//		std::stringstream errorCode;
//		errorCode << error.get()->GetCode();
//		LogController::addLog("[error : C2SModuleInapp_Restore] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
//		// QA 로그 정보 추가
//		LogController::setApiResultState(true, error.get()->GetCode());
//	}
}

void ApiAction::inappRequestBalance()
{
	log("====>>>> inappRequestBalance");
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

//	Json::Value arg;

//	arg["uid"] = Parameter::paramInappRequestBalance.uid;
    arg.AddMember("uid", StringRef(Parameter::paramInappRequestBalance.uid.c_str()), allocator);

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (inappRequestBalance)
	LogController::addLog("[Request : C2SModuleInapp_RequestBalance] >>>>>\n" + strJSON);

#ifdef __ANDROID__
//	if( ::C2SModuleInapp_RequestBalance( strJSON.c_str(), &error ) == false )
//	{
//		log( "====>>>> Inapp RequestBalance error (%s)", error.get()->GetMessage() );
//		// 로그 정보 추가 (inappRestore : error)
//		std::stringstream errorCode;
//		errorCode << error.get()->GetCode();
//		LogController::addLog("[error : C2SModuleInapp_RequestBalance] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
//		// QA 로그 정보 추가
//		LogController::setApiResultState(true, error.get()->GetCode());
//	}
#endif
}

void ApiAction::inappPrepare()
{
	log("====>>>> inappPrepare");
    Document argInapp;
    argInapp.SetObject();
    Document::AllocatorType& allocator = argInapp.GetAllocator();

    rapidjson::Value inappList(rapidjson::kArrayType);
    inappList.PushBack(StringRef(Parameter::paramInappPrepare.pidList.c_str()), allocator);

//	Json::Value argInapp;
//
//	Json::Value inappList;

	// example ---
	//inappList.append("050301010001"); 	// Lebi misample에 등록된 pid.
	//inappList.append("u1");
	//argInapp ["pidList"] = inappList;

//	inappList.append(Parameter::paramInappPrepare.pidList);		// 리스트로 삽입.
//	argInapp ["pidList"] = inappList;
//	argInapp ["showLog"] = true;

	argInapp.AddMember("pidList", inappList, allocator);
	argInapp.AddMember("showLog", true, allocator);

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(argInapp);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    argInapp.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (inappPrepare)
	LogController::addLog("[Request : C2SModuleInapp_Prepare] >>>>>\n" + strJSON);

//	if( ::C2SModuleInapp_Prepare( strJSON.c_str(), &error ) == false ) {
//		log( "====>>>> Inapp Prepare error (%s)", error.get()->GetMessage() );
//		// 로그 정보 추가 (inappPrepare : error)
//		std::stringstream errorCode;
//		errorCode << error.get()->GetCode();
//		LogController::addLog("[error : C2SModuleInapp_Prepare] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
//		// QA 로그 정보 추가
//		LogController::setApiResultState(true, error.get()->GetCode());
//	}
}

void ApiAction::trackerSendEvent()
{
	log("====>>>> trackerSendEvent");

    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

    arg.AddMember("event", StringRef(Parameter::paramTrackerSendEvent.event.c_str()), allocator);

//	Json::Value arg;
//	arg["event"] = Parameter::paramTrackerSendEvent.event; 	// initialize때 등록한 이벤트 문자열

	// ===== UNITY 에서만 Revenue 기능 사용. =====
	/*
	Json::Value argRevenueInfo;
	argRevenueInfo["title"] = Parameter::paramTrackerSendEvent.title;
	argRevenueInfo["description"] = Parameter::paramTrackerSendEvent.description;
	argRevenueInfo["itemCount"] = Parameter::paramTrackerSendEvent.itemCount;
	argRevenueInfo["currency"] = Parameter::paramTrackerSendEvent.currency;
	argRevenueInfo["price"] = Parameter::paramTrackerSendEvent.price;

	if(Parameter::paramTrackerSendEvent.title == "" &&
			Parameter::paramTrackerSendEvent.description == "" &&
			Parameter::paramTrackerSendEvent.currency == "" &&
			Parameter::paramTrackerSendEvent.price == "")
	{
		// not use argRevenueInfo
	} else {
		arg["revenue"] = argRevenueInfo;
	}
	*/

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (trackerSendEvent)
	LogController::addLog("[Request : C2SModuleTracker_SendEvent] >>>>>\n" + strJSON);
	// QA 로그 정보 추가
	LogController::setApiResultState(false, 0);

	if( ::C2SModuleTracker_SendEvent( strJSON.c_str(), &error ) == false ) {
		log( "====>>>> Tracking SendEvent error (%s)", error.get()->GetMessage() );
		// 로그 정보 추가 (trackerSendEvent : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleTracker_SendEvent] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

void ApiAction::trackerSetEnable()
{
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

//    arg.AddMember("Partytrack", Parameter::paramTrackerSetEnable.Partytrack, allocator);
    arg.AddMember("Adjust", Parameter::paramTrackerSetEnable.Adjust, allocator);
    arg.AddMember("Singular", Parameter::paramTrackerSetEnable.Singular, allocator);
    arg.AddMember("AppsFlyer", Parameter::paramTrackerSetEnable.AppsFlyer, allocator);

//    Json::Value arg;
//	arg["Partytrack"] = Parameter::paramTrackerSetEnable.Partytrack;	// true : 사용, false : 미사용
//	arg["Adjust"] = Parameter::paramTrackerSetEnable.Adjust;			// true : 사용, false : 미사용
////	arg["MAT"] = Parameter::paramTrackerSetEnable.MAT;					// true : 사용, false : 미사용		// deprecated : 1.14.1
//	arg["Sigular"] = Parameter::paramTrackerSetEnable.Singular;			// true : 사용, false : 미사용

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (trackerSetEnable)
	LogController::addLog("[Request : C2SModuleTracker_SendEvent] >>>>>\n" + strJSON);
	// QA 로그 정보 추가
	LogController::setApiResultState(false, 0);

	if( ::C2SModuleTracker_SetEnable( strJSON.c_str(), &error ) == false ) {
		log( "====>>>> Tracking SetEnable error (%s)", error.get()->GetMessage() );
		// 로그 정보 추가 (trackerSetEnable : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleTracker_SendEvent] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

// HIVE 1.9.0
void ApiAction::userAcquisition()
{
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

//	Json::Value arg;
	if(Parameter::paramPromotionAcquisition.callState == 1) {				// (1) acquisition info
//		arg["gameWorld"] = Parameter::paramPromotionAcquisition.gameWorld;			// "kr";				// HIVE 1.12.0부터 gameWorld 및 gameLanguage 값을 키로 사용하지 않음.
		Parameter::paramPromotionAcquisition.isGetInfo = true;						// get info : OK
	} else if(Parameter::paramPromotionAcquisition.callState == 2
			&& Parameter::paramPromotionAcquisition.isGetInfo == true) {	// (2) qrcode save
//		arg["type"] = "qrcodeSave";
//		arg["data"] = Parameter::paramPromotionAcquisition.qrcode;
		arg.AddMember("type", "qrcodeSave", allocator);
		arg.AddMember("data", StringRef(Parameter::paramPromotionAcquisition.qrcode.c_str()), allocator);
	} else if(Parameter::paramPromotionAcquisition.callState == 3
			&& Parameter::paramPromotionAcquisition.isGetInfo == true) {	// (3) qrcode share
//		arg["type"] = "qrcodeShare";
//		arg["data"] = Parameter::paramPromotionAcquisition.qrcode;
        arg.AddMember("type", "qrcodeShare", allocator);
        arg.AddMember("data", StringRef(Parameter::paramPromotionAcquisition.qrcode.c_str()), allocator);
	} else if(Parameter::paramPromotionAcquisition.callState == 4
			&& Parameter::paramPromotionAcquisition.isGetInfo == true) {	// (4) invite link
//		arg["type"] = "linkShare";
//		arg["url"] = Parameter::paramPromotionAcquisition.inviteCommonLink;
        arg.AddMember("type", "linkShare", allocator);
        arg.AddMember("url", StringRef(Parameter::paramPromotionAcquisition.inviteCommonLink.c_str()), allocator);
	} else if(Parameter::paramPromotionAcquisition.callState == 5) {		// (5) hive invite link
//		ApiAction::snsMessage();
//        return;

        arg.AddMember("type", "linkShare", allocator);
        arg.AddMember("url", StringRef(Parameter::paramPromotionAcquisition.inviteCommonLink.c_str()), allocator);
        arg.AddMember("message", StringRef(Parameter::paramPromotionAcquisition.message.c_str()), allocator);
	} else if(Parameter::paramPromotionAcquisition.callState == 6) {
		ApiAction::snsMessage();
		return;
	} else if(Parameter::paramPromotionAcquisition.callState == 7) {
		arg.AddMember("type", "senderInfo", allocator);
	}

	// additionalInfo 설정
//	if(Parameter::paramInitialize.hivePromotion == false) {
////		arg["additionalInfo"] = Parameter::paramPromotionAcquisition.addtionalInfo;
//        arg.AddMember("additionalInfo", StringRef(Parameter::paramPromotionAcquisition.addtionalInfo.c_str()), allocator);
//	}
//	else{
////		arg["additionalInfo"] = Parameter::paramHivePromotion.additionalInfo;
//        arg.AddMember("additionalInfo", StringRef(Parameter::paramHivePromotion.additionalInfo.c_str()), allocator);
//	}

	if(Parameter::paramPromotionAcquisition.callState != 7) {
		arg.AddMember("additionalInfo", StringRef(Parameter::paramPromotionAcquisition.addtionalInfo.c_str()), allocator);
	}

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (trackerSetEnable)
	LogController::addLog("[Request : C2SModulePromotion_Acquisition] >>>>>\n" + strJSON);
	// QA 로그 정보 추가
	LogController::setApiResultState(false, 0);

	if( ::C2SModulePromotion_Acquisition( strJSON.c_str(), &error ) == false ) {
		log( "====>>>> Acquisition error (%s)", error.get()->GetMessage() );

		// 로그 정보 추가 (trackerSetEnable : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModulePromotion_Acquisition] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

// HIVE 1.10.0
void ApiAction::engageConsumeCoupon()
{
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

//    Json::Value arg;

    arg.AddMember("couponId", StringRef(Parameter::paramEngageComsumeCoupon.couponId.c_str()), allocator);
    arg.AddMember("language", StringRef(Parameter::paramEngageComsumeCoupon.language.c_str()), allocator);
    arg.AddMember("additionalInfo", StringRef(Parameter::paramEngageComsumeCoupon.additionalInfo.c_str()), allocator);

//	arg["couponId"] = Parameter::paramEngageComsumeCoupon.couponId;
//	arg["language"] = Parameter::paramEngageComsumeCoupon.language;
//	arg["additionalInfo"] = Parameter::paramEngageComsumeCoupon.additionalInfo;

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (C2SModuleEngagement_ConsumeCoupon)
	LogController::addLog("[Request : C2SModuleEngagement_ConsumeCoupon] >>>>>\n" + strJSON);
	// QA 로그 정보 추가
	LogController::setApiResultState(false, 0);

	if( ::C2SModuleEngagement_ConsumeCoupon(strJSON.c_str(), &error) == false) {
		log( "====>>>> C2SModuleEngagement_ConsumeCoupon error (%s)", error.get()->GetMessage() );

		// 로그 정보 추가 (C2SModuleEngagement_ConsumeCoupon : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleEngagement_ConsumeCoupon] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

// GetEvent Api 삭제 : HIVE 1.11.0
// HIVE 1.10.0
//void ApiAction::engageGetEventAct()
//{
//	Json::Value arg;
//
//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
//
//	C2SModule_Cocos2dx::C2SModuleError error;
//	log("====>>>> strJSON : %s", strJSON.c_str());
//
//	// 로그 정보 추가 (C2SModuleEngagement_GetEvents)
//	LogController::addLog("[Request : C2SModuleEngagement_GetEvents] >>>>>\n" + strJSON);
//	// QA 로그 정보 추가
//	LogController::setApiResultState(false, 0);
//
//	if( ::C2SModuleEngagement_GetEvents(strJSON.c_str(), &error) == false) {
//		log( "====>>>> C2SModuleEngagement_GetEvents error (%s)", error.get()->GetMessage() );
//
//		// 로그 정보 추가 (C2SModuleEngagement_GetEvents : error)
//		std::stringstream errorCode;
//		errorCode << error.get()->GetCode();
//		LogController::addLog("[error : C2SModuleEngagement_GetEvents] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
//		// QA 로그 정보 추가
//		LogController::setApiResultState(true, error.get()->GetCode());
//	}
//}

// HIVE 1.11.0
void ApiAction::engageSetReady()
{
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

//    Json::Value arg;

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (C2SModuleEngagement_SetReady)
	LogController::addLog("[Request : C2SModuleEngagement_SetReady] >>>>>\n" + strJSON);
	// QA 로그 정보 추가
	LogController::setApiResultState(false, 0);

	if( ::C2SModuleEngagement_SetReady(strJSON.c_str(), &error) == false) {
//    if( ::C2SModuleEngagement_SetReady((char*)NULL, &error) == false) {
		log( "====>>>> C2SModuleEngagement_SetReady error (%s)", error.get()->GetMessage() );

		// 로그 정보 추가 (C2SModuleEngagement_SetReady : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleEngagement_SetReady] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

// HIVE 1.10.0
// HIVE IAP
void ApiAction::HiveIAP_Info()
{
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

//    Json::Value arg;
//	arg["type"] = Parameter::paramHiveIAPInfo.type;
    arg.AddMember("type", StringRef(Parameter::paramHiveIAPInfo.type.c_str()), allocator);

	if(Parameter::paramHiveIAPInfo.type != "balance") {
//		arg["locationCode"] = Parameter::paramHiveIAPInfo.locationCode;
//		arg["worldName"] = Parameter::paramInitialize.serverId;
//		arg["gameLanguage"] = Parameter::paramInitialize.gameLanguage;

        arg.AddMember("locationCode", StringRef(Parameter::paramHiveIAPInfo.locationCode.c_str()), allocator);
        arg.AddMember("worldName", StringRef(Parameter::paramInitialize.serverId.c_str()), allocator);
        arg.AddMember("gameLanguage", StringRef(Parameter::paramInitialize.gameLanguage.c_str()), allocator);
	}

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// ===== 로그 정보 없이 하는 것으로 약속 =====
	// 로그 정보 추가 (trackerSetEnable)
	LogController::addLog("[Request : C2SModuleHiveIAP_Info] >>>>>\n" + strJSON);
	// QA 로그 정보 추가
	LogController::setApiResultState(false, 0);

//	if( ::C2SModuleHiveIAP_Info( strJSON.c_str(), &error ) == false ) {
//		log( "====>>>> C2SModuleHiveIAP_Info error (%s)", error.get()->GetMessage() );
//
//		// 로그 정보 추가 (trackerSetEnable : error)
//		std::stringstream errorCode;
//		errorCode << error.get()->GetCode();
//		LogController::addLog("[error : C2SModuleHiveIAP_Info] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
//		// QA 로그 정보 추가
//		LogController::setApiResultState(true, error.get()->GetCode());
//	}
}

// HIVE 1.10.0
// HIVE IAP
void ApiAction::HiveIAP_Show()
{
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

//    Json::Value arg;

//	arg["type"] = Parameter::paramHiveIAPShow.type;
//	arg["gameLanguage"] = Parameter::paramInitialize.gameLanguage;

    arg.AddMember("type", StringRef(Parameter::paramHiveIAPShow.type.c_str()), allocator);
    arg.AddMember("gameLanguage", StringRef(Parameter::paramInitialize.gameLanguage.c_str()), allocator);

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (trackerSetEnable)
	LogController::addLog("[Request : C2SModuleHiveIAP_Show] >>>>>\n" + strJSON);
	// QA 로그 정보 추가
	LogController::setApiResultState(false, 0);

//	if( ::C2SModuleHiveIAP_Show( strJSON.c_str(), &error ) == false ) {
//		log( "====>>>> C2SModuleHiveIAP_Show error (%s)", error.get()->GetMessage() );
//
//		// 로그 정보 추가 (trackerSetEnable : error)
//		std::stringstream errorCode;
//		errorCode << error.get()->GetCode();
//		LogController::addLog("[error : C2SModuleHiveIAP_Show] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
//		// QA 로그 정보 추가
//		LogController::setApiResultState(true, error.get()->GetCode());
//	}
}

// HIVE 1.10.0
// HIVE IAP
void ApiAction::HiveIAP_Purchase()
{

	ApiAction::HiveIAP_GameServerLoginConnection();

    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

//	Json::Value arg;

//	arg["pid"] = Parameter::paramHiveIAPPurchase.pid;
//	arg["additionalInfo"] = Parameter::paramHiveIAPPurchase.additionalInfo;			// HIVE SDK 1.12.1

    arg.AddMember("pid", StringRef(Parameter::paramHiveIAPPurchase.pid.c_str()), allocator);
    arg.AddMember("additionalInfo", StringRef(Parameter::paramHiveIAPPurchase.additionalInfo.c_str()), allocator);

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (trackerSetEnable)
	LogController::addLog("[Request : HiveIAP_Purchase] >>>>>\n" + strJSON);
	// QA 로그 정보 추가 (임시 삭제)
	LogController::setApiResultState(false, 0);

//	if( ::C2SModuleHiveIAP_Purchase( strJSON.c_str(), &error ) == false ) {
//		log( "====>>>> HiveIAP_Purchase error (%s)", error.get()->GetMessage() );
//
//		// 로그 정보 추가 (trackerSetEnable : error)
//		std::stringstream errorCode;
//		errorCode << error.get()->GetCode();
//		LogController::addLog("[error : HiveIAP_Purchase] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
//		// QA 로그 정보 추가 (임시 삭제)
//		LogController::setApiResultState(true, error.get()->GetCode());
//	}
}

// HIVE 1.10.0
// HIVE IAP
void ApiAction::HiveIAP_Restore()
{
//	ApiAction::HiveIAP_GameServerLoginConnection();

    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

//	Json::Value arg;

//	arg["additionalInfo"] = Parameter::paramHiveIAPPurchase.additionalInfo;			// HIVE SDK 1.12.1
    arg.AddMember("additionalInfo", StringRef(Parameter::paramHiveIAPPurchase.additionalInfo.c_str()), allocator);

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (snsUser)
	LogController::addLog("[Request : C2SModuleHiveIAP_Restore] >>>>>\nNULL");
	// QA 로그 정보 추가 (임시 삭제)
	LogController::setApiResultState(false, 0);

//	if (::C2SModuleHiveIAP_Restore((char*) NULL, &error) == false) {
//		log("====>>>> C2SModuleHiveIAP_Restore. User Show error (%s)", error.get()->GetMessage());
//		log("====>>>> C2SModuleHiveIAP_Restore. User Show error code (%d)", error.get()->GetCode());
//		// 로그 정보 추가 (snsUser : error)
//		std::stringstream errorCode;
//		errorCode << error.get()->GetCode();
//		LogController::addLog("[error : C2SModuleHiveIAP_Restore] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
//		// QA 로그 정보 추가
//		LogController::setApiResultState(true, error.get()->GetCode());
//	}
}

// HIVE 1.10.0
void ApiAction::HiveIAP_PostBox()
{
	// 우편함 내용 호출.
	log("### HIVE IAP POSTBOX start");

    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

    arg.AddMember("type", "login", allocator);
    arg.AddMember("vid", StringRef(DataManager::getParamValue(TAG_APIACTION_LOGIN, "vid").c_str()), allocator);
    arg.AddMember("vidSession", StringRef(DataManager::getParamValue(TAG_APIACTION_LOGIN, "sessionToken").c_str()), allocator);
    arg.AddMember("uid", StringRef(DataManager::getParamValue(TAG_APIACTION_LOGIN, "uid").c_str()), allocator);
    arg.AddMember("uidSession", StringRef(DataManager::getParamValue(TAG_APIACTION_LOGIN, "sessionToken").c_str()), allocator);
    arg.AddMember("country", "KR", allocator);
    arg.AddMember("gameLanguage", "ko", allocator);
    arg.AddMember("worldName", "KR", allocator);
    arg.AddMember("useSandbox", DataManager::getParamValue_bool(TAG_APIACTION_INITIALIZE, "useSandbox"), allocator);
    arg.AddMember("isPostBoxShow", true, allocator);

//	Json::Value arg;
//	arg["type"] = "login";
//	arg["vid"] = DataManager::getParamValue(TAG_APIACTION_LOGIN, "vid");
//	arg["vidSession"] = DataManager::getParamValue(TAG_APIACTION_LOGIN, "sessionToken");
//	arg["uid"] = DataManager::getParamValue(TAG_APIACTION_LOGIN, "uid");
//	arg["uidSession"] = DataManager::getParamValue(TAG_APIACTION_LOGIN, "sessionToken");
//	arg["country"] = "KR";
//	arg["gameLanguage"] = "ko";
//	arg["worldName"] = "KR";
//	arg["useSandbox"] = DataManager::getParamValue_bool(TAG_APIACTION_INITIALIZE, "useSandbox");
//	arg["isPostBoxShow"] = true;

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();
	log("====>>>> strJSON : %s", strJSON.c_str());

#ifdef __ANDROID__
	if(Parameter::paramInitialize.usePostboxV4forAndroid == false) {
		PostBoxRequest(strJSON.c_str());
	}
#endif
	log("### HIVE IAP POSTBOX end");
}

void ApiAction::HiveIAP_GameServerLoginConnection()
{
	// 게임서버에 계정에 대한 정보 우선 전달
	log("### HiveIAP_GameServerLoginConnection");

    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

    arg.AddMember("type", "login", allocator);
    arg.AddMember("vid", StringRef(DataManager::getParamValue(TAG_APIACTION_LOGIN, "vid").c_str()), allocator);
    arg.AddMember("vidSession", StringRef(DataManager::getParamValue(TAG_APIACTION_LOGIN, "sessionToken").c_str()), allocator);
    arg.AddMember("uid", StringRef(DataManager::getParamValue(TAG_APIACTION_LOGIN, "uid").c_str()), allocator);
    arg.AddMember("uidSession", StringRef(DataManager::getParamValue(TAG_APIACTION_LOGIN, "sessionToken").c_str()), allocator);
    arg.AddMember("country", "KR", allocator);
    arg.AddMember("gameLanguage", "ko", allocator);
    arg.AddMember("worldName", "KR", allocator);
    arg.AddMember("useSandbox", DataManager::getParamValue_bool(TAG_APIACTION_INITIALIZE, "useSandbox"), allocator);
    arg.AddMember("isPostBoxShow", false, allocator);

//	Json::Value arg;
//	arg["type"] = "login";
//	arg["vid"] = DataManager::getParamValue(TAG_APIACTION_LOGIN, "vid");
//	arg["vidSession"] = DataManager::getParamValue(TAG_APIACTION_LOGIN, "sessionToken");
//	arg["uid"] = DataManager::getParamValue(TAG_APIACTION_LOGIN, "uid");
//	arg["uidSession"] = DataManager::getParamValue(TAG_APIACTION_LOGIN, "sessionToken");
//	arg["country"] = "KR";
//	arg["gameLanguage"] = "ko";
//	arg["worldName"] = "KR";
//	arg["useSandbox"] = DataManager::getParamValue_bool(TAG_APIACTION_INITIALIZE, "useSandbox");
//	arg["isPostBoxShow"] = false;

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();
	log("====>>>> strJSON : %s", strJSON.c_str());

#ifdef __ANDROID__
	if(Parameter::paramInitialize.usePostboxV4forAndroid == false) {
		PostBoxRequest(strJSON.c_str());
	}
#endif
	log("### HIVE IAP POSTBOX end");// failed to authenticate the productid
}

// HIVE 1.12.0
void ApiAction::HivePromotion_Show()
{
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

//	Json::Value arg;

//	arg["type"] = Parameter::paramHivePromotion.type;
    arg.AddMember("type", StringRef(Parameter::paramHivePromotion.type.c_str()), allocator);
	if(Parameter::paramHivePromotion.type == "news"
			|| Parameter::paramHivePromotion.type == "banner"
			|| Parameter::paramHivePromotion.type == "notice")
	{
//		arg["forced"] = Parameter::paramHivePromotion.forced;
        arg.AddMember("forced", Parameter::paramHivePromotion.forced, allocator);
	}
	if(Parameter::paramHivePromotion.type == "customView") {
//		arg["contentKey"] = Parameter::paramHivePromotion.contentKey_view;
        arg.AddMember("contentKey", StringRef(Parameter::paramHivePromotion.contentKey_view.c_str()), allocator);
	}else if(Parameter::paramHivePromotion.type == "customBoard") {
//		arg["contentKey"] = Parameter::paramHivePromotion.contentKey_board;
        arg.AddMember("contentKey", StringRef(Parameter::paramHivePromotion.contentKey_board.c_str()), allocator);
	}else if(Parameter::paramHivePromotion.type == "spot") {
//		arg["contentKey"] = Parameter::paramHivePromotion.contentKey_spot;
        arg.AddMember("contentKey", StringRef(Parameter::paramHivePromotion.contentKey_spot.c_str()), allocator);
	}else if(Parameter::paramHivePromotion.type == "direct") {
//		arg["contentKey"] = Parameter::paramHivePromotion.contentKey_direct;
        arg.AddMember("contentKey", StringRef(Parameter::paramHivePromotion.contentKey_direct.c_str()), allocator);
	}

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (C2SModuleHivePromotion_Show)
	LogController::addLog("[Request : C2SModuleHivePromotion_Show] >>>>>\n" + strJSON);
	// QA 로그 정보 추가
	LogController::setApiResultState(false, 0);

	if( ::C2SModuleHivePromotion_Show(strJSON.c_str(), &error) == false) {
		log( "====>>>> C2SModuleHivePromotion_Show error (%s)", error.get()->GetMessage() );

		// 로그 정보 추가 (C2SModuleHivePromotion_Show : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleHivePromotion_Show] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

// HIVE 1.12.0
void ApiAction::HivePromotion_Info()
{
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

//	Json::Value arg;

//	arg["type"] = Parameter::paramHivePromotion.type;
    arg.AddMember("type", StringRef(Parameter::paramHivePromotion.type.c_str()), allocator);
	if(Parameter::paramHivePromotion.type == "customView") {
//		arg["contentKey"] = Parameter::paramHivePromotion.contentKey_view;
        arg.AddMember("contentKey", StringRef(Parameter::paramHivePromotion.contentKey_view.c_str()), allocator);
	}else if(Parameter::paramHivePromotion.type == "customBoard") {
//		arg["contentKey"] = Parameter::paramHivePromotion.contentKey_board;
        arg.AddMember("contentKey", StringRef(Parameter::paramHivePromotion.contentKey_board.c_str()), allocator);
	}else if(Parameter::paramHivePromotion.type == "spot") {
//		arg["contentKey"] = Parameter::paramHivePromotion.contentKey_spot;
        arg.AddMember("contentKey", StringRef(Parameter::paramHivePromotion.contentKey_spot.c_str()), allocator);
	}else if(Parameter::paramHivePromotion.type == "direct") {
//		arg["contentKey"] = Parameter::paramHivePromotion.contentKey_direct;
        arg.AddMember("contentKey", StringRef(Parameter::paramHivePromotion.contentKey_direct.c_str()), allocator);
	}

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (C2SModuleHivePromotion_Info)
	LogController::addLog("[Request : C2SModuleHivePromotion_Info] >>>>>\n" + strJSON);
	// QA 로그 정보 추가
	LogController::setApiResultState(false, 0);

	if( ::C2SModuleHivePromotion_Info(strJSON.c_str(), &error) == false) {
		log( "====>>>> C2SModuleHivePromotion_Info error (%s)", error.get()->GetMessage() );

		// 로그 정보 추가 (C2SModuleHivePromotion_Info : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleHivePromotion_Info] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

// HIVE 1.12.0
void ApiAction::HivePromotion_SetAdditionalInfo()
{
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

    rapidjson::Value argAddInfo(rapidjson::kObjectType);

//	Json::Value arg;
//
//	Json::Value argAddInfo;
//	Json::StyledWriter addWriter;
//	std::string strAddInfoJson;

	if(Parameter::paramHivePromotion.additionalInfo == "")
	{
//		argAddInfo["character"] = "0";
//		argAddInfo["server"] = "0";
//		strAddInfoJson = addWriter.write(argAddInfo);
//		arg["additionalInfo"] = strAddInfoJson;

        argAddInfo.AddMember("character", "0", allocator);
        argAddInfo.AddMember("server", "0", allocator);
        arg.AddMember("additionalInfo", argAddInfo, allocator);
	}
	else
	{
//		arg["additionalInfo"] = Parameter::paramHivePromotion.additionalInfo;
        arg.AddMember("additionalInfo", StringRef(Parameter::paramHivePromotion.additionalInfo.c_str()), allocator);
	}

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (C2SModuleHivePromotion_SetAdditionalInfo)
	LogController::addLog("[Request : C2SModuleHivePromotion_SetAdditionalInfo] >>>>>\n" + strJSON);
	// QA 로그 정보 추가
	LogController::setApiResultState(false, 0);

	if( ::C2SModuleHivePromotion_SetAdditionalInfo(strJSON.c_str(), &error) == false) {
		log( "====>>>> C2SModuleHivePromotion_SetAdditionalInfo error (%s)", error.get()->GetMessage() );

		// 로그 정보 추가 (C2SModuleHivePromotion_SetAdditionalInfo : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleHivePromotion_SetAdditionalInfo] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

// HIVE 1.12.0
void ApiAction::HivePromotion_GetOfferwallState()
{
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

//	Json::Value arg;

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (C2SModuleHivePromotion_GetOfferwallState)
	LogController::addLog("[Request : C2SModuleHivePromotion_GetOfferwallState] >>>>>\n" + strJSON);
	// QA 로그 정보 추가
	LogController::setApiResultState(false, 0);

	if( ::C2SModuleHivePromotion_GetOfferwallState(strJSON.c_str(), &error) == false) {
		log( "====>>>> C2SModuleHivePromotion_GetOfferwallState error (%s)", error.get()->GetMessage() );

		// 로그 정보 추가 (C2SModuleHivePromotion_GetOfferwallState : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleHivePromotion_GetOfferwallState] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

void ApiAction::HivePromotion_UpdatePromotionData()
{
	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

//	Json::Value arg;

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (C2SModuleHivePromotion_UpdatePromotionData)
	LogController::addLog("[Request : C2SModuleHivePromotion_UpdatePromotionData] >>>>>\n" + strJSON);
	// QA 로그 정보 추가
	LogController::setApiResultState(false, 0);

	if( ::C2SModuleHivePromotion_UpdatePromotionData(strJSON.c_str(), &error) == false) {
		log( "====>>>> C2SModuleHivePromotion_UpdatePromotionData error (%s)", error.get()->GetMessage() );

		// 로그 정보 추가 (C2SModuleHivePromotion_UpdatePromotionData : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleHivePromotion_UpdatePromotionData] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

// HIVE 1.12.0
void ApiAction::SetGameInfo_SetGameInfo()
{
    ::CommonResultDelegate resultDelegate = DealCallback::ResultDelegate;
    ::C2SModuleSocial_SetCommonResultDelegate(resultDelegate);

	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	arg.AddMember("serverId", StringRef(Parameter::paramInitialize.serverId.c_str()), allocator);
	arg.AddMember("gameLanguage", StringRef(Parameter::paramInitialize.gameLanguage.c_str()), allocator);
//	arg.AddMember("vid", StringRef(Parameter::paramInitialize.setGameInfo_vid.c_str()), allocator);
	arg.AddMember("emergencyMode", Parameter::paramInitialize.setGameInfo_emergencyMode, allocator);	// Hive KS 1.19.4
	arg.AddMember("channel", StringRef(Parameter::paramInitialize.channel.c_str()), allocator); // Hive 1.20.3

//    std::string dMsg = "[SetGameInfo] serverId(";
//    std::string dResultValue(Parameter::paramInitialize.serverId.c_str());
//    std::string dMsg2 = ", gameLanguage(";
//    std::string dResultValue2(Parameter::paramInitialize.gameLanguage.c_str());
//    dMsg.append(dResultValue);
//    dMsg.append(dMsg2);
//    dMsg.append(dResultValue2);
//	showToastMsg(dMsg.c_str());

//	Json::Value arg;
//
//	arg["serverId"] = Parameter::paramInitialize.serverId;
//	arg["gameLanguage"] = Parameter::paramInitialize.gameLanguage;
//	arg["vid"] = Parameter::paramInitialize.setGameInfo_vid;	// HIVE 1.16.1, 임의의 VID 설정 기능 추가
																	// HIVE SDK에 이미 VID가 설정된적이 없는 경우 setGameInfo API 호출에 포함된 VID가 설정된다.
																	// HIVE 로그인을 할 수 없는 특수한 상황에 사용하기 위한 요소이다.

//	Json::StyledWriter writer;
//	std::string strJSON = writer.write(arg);
    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (C2SModuleSocial_SetGameInfo)
	LogController::addLog("[Request : C2SModuleSocial_SetGameInfo] >>>>>\n" + strJSON);
	// QA 로그 정보 추가
	LogController::setApiResultState(false, 0);

	if( ::C2SModuleSocial_SetGameInfo(strJSON.c_str(), &error) == false) {
		log( "====>>>> C2SModuleSocial_SetGameInfo error (%s)", error.get()->GetMessage() );

		// 로그 정보 추가 (C2SModuleSocial_SetGameInfo : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleSocial_SetGameInfo] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());

	}
}

void ApiAction::SetGameInfo_GetHiveCountryCode()
{
    std::string countryCode(::GetHiveCountry());
    log("====>>>> GetHiveCountry : %s", countryCode.c_str());
    #ifdef __ANDROID__
        MessageBox(countryCode.c_str(), "");
    #else
        LogController::addLog("====>>>> GetHiveCountry : " + countryCode);
    #endif
}

void ApiAction::SetGameInfo_GetChannel()
{
	std::string channel(::GetChannel());
	log("====>>>> GetChannel : %s", channel.c_str());
#ifdef __ANDROID__
	MessageBox(channel.c_str(), "");
#else
	LogController::addLog("====>>>> GetChannel : " + channel);
#endif
}

void ApiAction::SetGameInfo_SetChannel()
{
    std::string channel = Parameter::paramInitialize.channel;
	::SetChannel(channel.c_str());
	log("====>>>> SetChannel : %s", channel.c_str());
#ifdef __ANDROID__
	MessageBox(channel.c_str(), "");
#else
	LogController::addLog("====>>>> SetChannel : " + channel);
#endif
}

void ApiAction::SetGameInfo_UpdateServerId()
{
	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	arg.AddMember("serverId", StringRef(Parameter::paramInitialize.serverId.c_str()), allocator);

	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (C2SModuleSocial_UpdateServerId)
	LogController::addLog("[Request : C2SModuleSocial_UpdateServerId] >>>>>\n" + strJSON);
	// QA 로그 정보 추가
	LogController::setApiResultState(false, 0);

	if( ::C2SModuleSocial_UpdateServerId(strJSON.c_str(), &error) == false) {
		log( "====>>>> C2SModuleSocial_UpdateServerId error (%s)", error.get()->GetMessage() );

		// 로그 정보 추가 (C2SModuleSocial_UpdateServerId : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleSocial_UpdateServerId] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

void ApiAction::SetGameInfo_UpdateGameLanguage()
{
	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	arg.AddMember("gameLanguage", StringRef(Parameter::paramInitialize.gameLanguage.c_str()), allocator);

	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();

	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가 (C2SModuleSocial_UpdateGameLanguage)
	LogController::addLog("[Request : C2SModuleSocial_UpdateGameLanguage] >>>>>\n" + strJSON);
	// QA 로그 정보 추가
	LogController::setApiResultState(false, 0);

	if( ::C2SModuleSocial_UpdateGameLanguage(strJSON.c_str(), &error) == false) {
		log( "====>>>> C2SModuleSocial_UpdateGameLanguage error (%s)", error.get()->GetMessage() );

		// 로그 정보 추가 (C2SModuleSocial_UpdateGameLanguage : error)
		std::stringstream errorCode;
		errorCode << error.get()->GetCode();
		LogController::addLog("[error : C2SModuleSocial_UpdateGameLanguage] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
		// QA 로그 정보 추가
		LogController::setApiResultState(true, error.get()->GetCode());
	}
}

void ApiAction::SetGameInfo_SendData()
{
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

    arg.AddMember("type", "funnelsLogs", allocator);
    arg.AddMember("funnelsId", StringRef(Parameter::paramSendData.funnelsId.c_str()), allocator);
    arg.AddMember("optionTag", StringRef(Parameter::paramSendData.optionTag.c_str()), allocator);


    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

    C2SModule_Cocos2dx::C2SModuleError error;
    log("====>>>> strJSON : %s", strJSON.c_str());

    // 로그 정보 추가 (C2SModuleSocial_UpdateGameLanguage)
    LogController::addLog("[Request : C2SModuleSocial_SendData] >>>>>\n" + strJSON);
    // QA 로그 정보 추가
    LogController::setApiResultState(false, 0);

    if( ::C2SModuleSocial_SendData(strJSON.c_str(), &error) == false) {
        log( "====>>>> C2SModuleSocial_SendData error (%s)", error.get()->GetMessage() );

        // 로그 정보 추가 (C2SModuleSocial_UpdateGameLanguage : error)
        std::stringstream errorCode;
        errorCode << error.get()->GetCode();
        LogController::addLog("[error : C2SModuleSocial_SendData] >>>>>\nerror (" + errorCode.str() + ")(" + error.get()->GetMessage() + ")");
        // QA 로그 정보 추가
        LogController::setApiResultState(true, error.get()->GetCode());
    }
}

// HIVE 1.18.3
void ApiAction::promotionBannerInfo() 
{
	Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

    // type 값 확인
    if(Parameter::paramPromotionBannerInfo.campaignType == "a" || Parameter::paramPromotionBannerInfo.campaignType == "all")
        arg.AddMember("campaignType", "all", allocator);
    else if(Parameter::paramPromotionBannerInfo.campaignType == "e" || Parameter::paramPromotionBannerInfo.campaignType == "event")
        arg.AddMember("campaignType", "event", allocator);
    else if(Parameter::paramPromotionBannerInfo.campaignType == "n" || Parameter::paramPromotionBannerInfo.campaignType == "notice")
        arg.AddMember("campaignType", "notice", allocator);
    else
        arg.AddMember("campaignType", StringRef(Parameter::paramPromotionBannerInfo.campaignType.c_str()), allocator);

    if(Parameter::paramPromotionBannerInfo.bannerType == "g" || Parameter::paramPromotionBannerInfo.bannerType == "great")
        arg.AddMember("bannerType", "great", allocator);
    else if(Parameter::paramPromotionBannerInfo.bannerType == "s" || Parameter::paramPromotionBannerInfo.bannerType == "small")
        arg.AddMember("bannerType", "small", allocator);
    else if(Parameter::paramPromotionBannerInfo.bannerType == "r" || Parameter::paramPromotionBannerInfo.bannerType == "rolling")
        arg.AddMember("bannerType", "rolling", allocator);
    else
        arg.AddMember("bannerType", StringRef(Parameter::paramPromotionBannerInfo.bannerType.c_str()), allocator);

    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	callApiAndMakeLog("C2SModulePromotion_GetBannerInfo", strJSON);
}

// HIVE 1.18.3
void ApiAction::HivePromotion_BannerInfo()
{
	Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

	// type 값 확인
	if(Parameter::paramHivePromotionBannerInfo.campaignType == "a" || Parameter::paramHivePromotionBannerInfo.campaignType == "all")
		arg.AddMember("campaignType", "all", allocator);
	else if(Parameter::paramHivePromotionBannerInfo.campaignType == "e" || Parameter::paramHivePromotionBannerInfo.campaignType == "event")
		arg.AddMember("campaignType", "event", allocator);
	else if(Parameter::paramHivePromotionBannerInfo.campaignType == "n" || Parameter::paramHivePromotionBannerInfo.campaignType == "notice")
		arg.AddMember("campaignType", "notice", allocator);
	else
		arg.AddMember("campaignType", StringRef(Parameter::paramHivePromotionBannerInfo.campaignType.c_str()), allocator);

	if(Parameter::paramHivePromotionBannerInfo.bannerType == "g" || Parameter::paramHivePromotionBannerInfo.bannerType == "great")
		arg.AddMember("bannerType", "great", allocator);
	else if(Parameter::paramHivePromotionBannerInfo.bannerType == "s" || Parameter::paramHivePromotionBannerInfo.bannerType == "small")
		arg.AddMember("bannerType", "small", allocator);
	else if(Parameter::paramHivePromotionBannerInfo.bannerType == "r" || Parameter::paramHivePromotionBannerInfo.bannerType == "rolling")
		arg.AddMember("bannerType", "rolling", allocator);
	else
		arg.AddMember("bannerType", StringRef(Parameter::paramHivePromotionBannerInfo.bannerType.c_str()), allocator);

    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

	callApiAndMakeLog("C2SModuleHivePromotion_GetBannerInfo", strJSON);
}

// HIVE 1.18.4
void ApiAction::InappV4_ItemInfo() {
	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	// type 값 확인
	if(Parameter::paramInAppV4.type == "c" || Parameter::paramInAppV4.type == "consumable")
		arg.AddMember("type", "consumable", allocator);
	else if(Parameter::paramInAppV4.type == "s" || Parameter::paramInAppV4.type == "subscription")
		arg.AddMember("type", "subscription", allocator);
	else if(Parameter::paramInAppV4.type == "m" || Parameter::paramInAppV4.type == "market") {
        arg.AddMember("type", "market", allocator);

		rapidjson::Value inappList(rapidjson::kArrayType);
#ifdef __ANDROID__
		// 안드로이드는 getMarketProductInfo가 Not supported API라서 사실 의미 없음
		inappList.PushBack("com.com2us.hivesdk.normal.freefull.apple.global.ios.universal.cs01", allocator);
		inappList.PushBack("com.com2us.hivesdk.normal.freefull.apple.global.ios.universal.cs02", allocator);
		inappList.PushBack("com.com2us.hivesdk.normal.freefull.apple.global.ios.universal.cs03", allocator);
#else
		inappList.PushBack("com.com2us.hivesdk.normal.freefull.apple.global.ios.universal.cs01", allocator);
		inappList.PushBack("com.com2us.hivesdk.normal.freefull.apple.global.ios.universal.cs02", allocator);
		inappList.PushBack("com.com2us.hivesdk.normal.freefull.apple.global.ios.universal.cs03", allocator);
#endif
		arg.AddMember("marketPidList", inappList, allocator);
    }
	else
		arg.AddMember("type", "", allocator);

	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();

	callApiAndMakeLog("C2SModuleInappV4_ItemInfo", strJSON);
}

// HIVE 1.18.4
void ApiAction::InappV4_Purchase() {
	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	// pid
#ifdef __ANDROID__
	arg.AddMember("pid", StringRef(Parameter::paramInAppV4.pid.c_str()), allocator);
#else
	std::string tempPid = "com.com2us.hivesdk.normal.freefull.apple.global.ios.universal." + Parameter::paramInAppV4.pid;
	arg.AddMember("pid", StringRef(tempPid.c_str()), allocator);
#endif
	// iapPayload
	arg.AddMember("iapPayload", StringRef(Parameter::paramInAppV4.iapPayload.c_str()), allocator);

	// type 값 확인
	if(Parameter::paramInAppV4.type == "c" || Parameter::paramInAppV4.type == "consumable")
		arg.AddMember("type", "consumable", allocator);
	else if(Parameter::paramInAppV4.type == "s" || Parameter::paramInAppV4.type == "subscription") {
		arg.AddMember("type", "subscription", allocator);

		// 구독 상품 구매 시 item upgrade 혹은 downgrade 처리
		// old market pid 구성
		if(Parameter::paramInAppV4.pid == "sub01" || Parameter::paramInAppV4.pid == "sub10"
			|| Parameter::paramInAppV4.pid == "sub11" || Parameter::paramInAppV4.pid == "sub30") {
			if(Parameter::paramInAppV4.google_sub_sub01 == true) {
				Parameter::paramInAppV4.oldMarketPid = "sub01";
				arg.AddMember("oldMarketPid", StringRef(Parameter::paramInAppV4.oldMarketPid.c_str()), allocator);
				MessageBox("이미 동일 그룹에 구독중인 상품(sub01)이 있습니다!\n구매 상태를 변경합니다!", "");
			}
			else if(Parameter::paramInAppV4.google_sub_sub10 == true) {
				Parameter::paramInAppV4.oldMarketPid = "sub10";
				arg.AddMember("oldMarketPid", StringRef(Parameter::paramInAppV4.oldMarketPid.c_str()), allocator);
                MessageBox("이미 동일 그룹에 구독중인 상품(sub10)이 있습니다!\n구매 상태를 변경합니다!", "");
			}
			else if(Parameter::paramInAppV4.google_sub_sub11 == true) {
				Parameter::paramInAppV4.oldMarketPid = "sub11";
				arg.AddMember("oldMarketPid", StringRef(Parameter::paramInAppV4.oldMarketPid.c_str()), allocator);
                MessageBox("이미 동일 그룹에 구독중인 상품(sub11)이 있습니다!\n구매 상태를 변경합니다!", "");
			}
			else if(Parameter::paramInAppV4.google_sub_sub30 == true) {
				Parameter::paramInAppV4.oldMarketPid = "sub30";
				arg.AddMember("oldMarketPid", StringRef(Parameter::paramInAppV4.oldMarketPid.c_str()), allocator);
                MessageBox("이미 동일 그룹에 구독중인 상품(sub30)이 있습니다!\n구매 상태를 변경합니다!", "");
			}
		}
		else if(Parameter::paramInAppV4.pid == "sub013" || Parameter::paramInAppV4.pid == "sub103"
				|| Parameter::paramInAppV4.pid == "sub113" || Parameter::paramInAppV4.pid == "sub303") {
			if(Parameter::paramInAppV4.google_sub_sub013 == true) {
				Parameter::paramInAppV4.oldMarketPid = "sub013";
				arg.AddMember("oldMarketPid", StringRef(Parameter::paramInAppV4.oldMarketPid.c_str()), allocator);
                MessageBox("이미 동일 그룹에 구독중인 상품(sub013)이 있습니다!\n구매 상태를 변경합니다!", "");
			}
			else if(Parameter::paramInAppV4.google_sub_sub103 == true) {
				Parameter::paramInAppV4.oldMarketPid = "sub103";
				arg.AddMember("oldMarketPid", StringRef(Parameter::paramInAppV4.oldMarketPid.c_str()), allocator);
                MessageBox("이미 동일 그룹에 구독중인 상품(sub103)이 있습니다!\n구매 상태를 변경합니다!", "");
			}
			else if(Parameter::paramInAppV4.google_sub_sub113 == true) {
				Parameter::paramInAppV4.oldMarketPid = "sub113";
				arg.AddMember("oldMarketPid", StringRef(Parameter::paramInAppV4.oldMarketPid.c_str()), allocator);
                MessageBox("이미 동일 그룹에 구독중인 상품(sub113)이 있습니다!\n구매 상태를 변경합니다!", "");
			}
			else if(Parameter::paramInAppV4.google_sub_sub303 == true) {
				Parameter::paramInAppV4.oldMarketPid = "sub303";
				arg.AddMember("oldMarketPid", StringRef(Parameter::paramInAppV4.oldMarketPid.c_str()), allocator);
                MessageBox("이미 동일 그룹에 구독중인 상품(sub303)이 있습니다!\n구매 상태를 변경합니다!", "");
			}
		}
	}
	else
		arg.AddMember("type", "", allocator);


	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();

	callApiAndMakeLog("C2SModuleInappV4_Purchase", strJSON);
}

// HIVE 1.18.4
void ApiAction::InappV4_Restore() {
	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	// type 값 확인
	if(Parameter::paramInAppV4.type == "c" || Parameter::paramInAppV4.type == "consumable")
		arg.AddMember("type", "consumable", allocator);
	else if(Parameter::paramInAppV4.type == "s" || Parameter::paramInAppV4.type == "subscription")
		arg.AddMember("type", "subscription", allocator);
	else
		arg.AddMember("type", "", allocator);
	// pid
#ifdef __ANDROID__
	arg.AddMember("pid", StringRef(Parameter::paramInAppV4.pid.c_str()), allocator);
#else
	std::string tempPid = "com.com2us.hivesdk.normal.freefull.apple.global.ios.universal." + Parameter::paramInAppV4.pid;
	arg.AddMember("pid", StringRef(tempPid.c_str()), allocator);
#endif

	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();

	callApiAndMakeLog("C2SModuleInappV4_Restore", strJSON);
}

// HIVE 1.18.4
void ApiAction::InappV4_TransactionFinish() {
	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	// pid
#ifdef __ANDROID__
	arg.AddMember("pid", StringRef(Parameter::paramInAppV4.pid.c_str()), allocator);
#else
	std::string tempPid = "com.com2us.hivesdk.normal.freefull.apple.global.ios.universal." + Parameter::paramInAppV4.pid;
	arg.AddMember("pid", StringRef(tempPid.c_str()), allocator);
#endif
	arg.AddMember("isPurchaseSuccess", true, allocator);

	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();

	callApiAndMakeLog("C2SModuleInappV4_TransactionFinish", strJSON);
}

// HIVE 1.18.4
void ApiAction::InappV4_GetSelectedMarket() {
	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	// 별도의 input arg 없음

	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();

	callApiAndMakeLog("C2SModuleInappV4_GetSelectedMarket", strJSON);
}

// HIVE 1.18.4
void ApiAction::InappV4_GetAccountUuid() {
	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	// 별도의 input arg 없음

	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();

	callApiAndMakeLog("C2SModuleInappV4_GetAccountUuid", strJSON);
}

// HIVE 1.18.4
void ApiAction::InappV4_MarketSelect() {
	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	// 별도의 input arg 없음

	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();

	callApiAndMakeLog("C2SModuleInappV4_SelectMarket", strJSON);
}

// HIVE 1.18.4
void ApiAction::InappV4_RequestBalance() {
	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	// 별도의 input arg 없음

	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();

	callApiAndMakeLog("C2SModuleInappV4_RequestBalance", strJSON);
}

// HIVE 1.18.4
void ApiAction::InappV4_ChargeBalance() {
	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	// 별도의 input arg 없음

	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();

	callApiAndMakeLog("C2SModuleInappV4_ChargeBalance", strJSON);
}

void ApiAction::InappV4_CheckPromotePurchase() {
    Document arg;
    arg.SetObject();
    Document::AllocatorType& allocator = arg.GetAllocator();

    // 별도의 input arg 없음

    StringBuffer buffer;
    Writer<StringBuffer> writer(buffer);
    arg.Accept(writer);

    std::string strJSON = buffer.GetString();

    callApiAndMakeLog("C2SModuleInappV4_CheckPromotePurchase", strJSON);
}

void ApiAction::InappV4_MarketConnect() {
	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	// 별도의 input arg 없음

	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();

	callApiAndMakeLog("C2SModuleInappV4_MarketConnect", strJSON);
}

void ApiAction::InappV4_ShowInAppMessages() {
	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	arg.AddMember("categoryId", Parameter::paramInAppV4.categoryId, allocator);

	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();

	callApiAndMakeLog("C2SModuleInappV4_ShowInAppMessages", strJSON);
}

// HIVE 1.18.4
void ApiAction::InappV4_PostBox_Login() {
	// 게임서버에 계정에 대한 정보 우선 전달
	log("### PostBoxIapV4 PostBox_GameServerLoginConnection");

	ApiAction::InappV4_PostBox_Init();

	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	arg.AddMember("type", "login", allocator);
	arg.AddMember("vid", StringRef(Parameter::paramLogin.vid.c_str()), allocator);   // TODO : 개별 모듈 샘플에선 VID 자리에 UID를 삽입
	arg.AddMember("uid", StringRef(Parameter::paramLogin.uid.c_str()), allocator);
	arg.AddMember("did", StringRef(Parameter::paramInitialize.did.c_str()), allocator);
	arg.AddMember("country", "kr", allocator);
	arg.AddMember("gameLanguage", "ko", allocator);
	arg.AddMember("world", StringRef(Parameter::paramInitialize.serverId.c_str()), allocator);
	arg.AddMember("isPostBoxShow", false, allocator);
	arg.AddMember("vidSession", StringRef(Parameter::paramLogin.sessionToken.c_str()), allocator);

	if(Parameter::paramInitialize.useStaging == true) {
		arg.AddMember("useStaging", true, allocator);
		log("### PostBoxIapV4 PostBox_GameServerLoginConnection, test server");
	}
	else {
		if(Parameter::paramInitialize.useSandbox == true) {
			arg.AddMember("useSandbox", true, allocator);
			log("### PostBoxIapV4 PostBox_GameServerLoginConnection, sandbox server");
		}
		else {
			// do not setting
			log("### PostBoxIapV4 PostBox_GameServerLoginConnection, live server");
		}
	}

	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();
	log("====>>>> strJSON : %s", strJSON.c_str());

#ifdef __ANDROID__
	if(Parameter::paramInitialize.usePostboxV4forAndroid == false) {
		PostBoxRequest(strJSON.c_str());
	} else {
		// KS Cocos 우편함 기능 개선
		Postbox::login([=](const Postbox::Login *login, Postbox::Error error){
			if( login != nullptr )
			{
//			int gold = login->gold;
//			int gem = login->gem;
//			std::string nickname = login->nickname;
//			Logger::log("nick %s, gold %d, gem %d", nickname.c_str(), gold, gem);

				MessageBox("Postbox Login[로그인] 성공", "Postbox Login[로그인] 성공");
				Parameter::paramInitialize.postboxV4Login = true;
			}
			else
			{
//			Logger::log("Postbox login failed.");
//			Logger::log("result code = %s, msg = %s", error.result_code.c_str(), error.result_msg.c_str());

				MessageBox("Postbox Login[로그인] 실패", "Postbox Login[로그인] 실패");
			}
		});
	}
#else
	// KS Cocos 우편함 기능 개선
	Postbox::login([=](const Postbox::Login *login, Postbox::Error error){
		if( login != nullptr )
		{
//			int gold = login->gold;
//			int gem = login->gem;
//			std::string nickname = login->nickname;
//			Logger::log("nick %s, gold %d, gem %d", nickname.c_str(), gold, gem);

			MessageBox("Postbox Login[로그인] 성공", "Postbox Login[로그인] 성공");
			Parameter::paramInitialize.postboxV4Login = true;
		}
		else
		{
//			Logger::log("Postbox login failed.");
//			Logger::log("result code = %s, msg = %s", error.result_code.c_str(), error.result_msg.c_str());

			MessageBox("Postbox Login[로그인] 실패", "Postbox Login[로그인] 실패");
		}
	});
#endif
	log("### PostBoxIapV4 PostBox_GameServerLoginConnection POSTBOX end");// failed to authenticate the productid
}

// HIVE 1.18.4
void ApiAction::InappV4_PostBox_Show() {
	// 포스트 박스 보이기
	log("### PostBoxIapV4 PostBox_ShowPostBox");

	ApiAction::InappV4_PostBox_Init();

	Document arg;
	arg.SetObject();
	Document::AllocatorType& allocator = arg.GetAllocator();

	arg.AddMember("type", "showPostBox", allocator);

	StringBuffer buffer;
	Writer<StringBuffer> writer(buffer);
	arg.Accept(writer);

	std::string strJSON = buffer.GetString();
	log("====>>>> strJSON : %s", strJSON.c_str());

#ifdef __ANDROID__
	if(Parameter::paramInitialize.usePostboxV4forAndroid == false) {
		PostBoxRequest(strJSON.c_str());
	}
#endif
	log("### PostBoxIapV4 PostBox_ShowPostBox POSTBOX end");// failed to authenticate the productid
}

void ApiAction::InappV4_PostBox_Init() {
    // post box init
    if (ApiAction::initPostBox == 0) {
        Document arg;
        arg.SetObject();
        Document::AllocatorType &allocator = arg.GetAllocator();

        if(Parameter::paramInitialize.useStaging == true) {
            arg.AddMember("useStaging", true, allocator);
            log("### PostBoxIapV4 InappV4_PostBox_Init, test server");
        }
        else {
            if(Parameter::paramInitialize.useSandbox == true) {
                arg.AddMember("useSandbox", true, allocator);
                log("### PostBoxIapV4 InappV4_PostBox_Init, sandbox server");
            }
            else {
                arg.AddMember("useLive", true, allocator);
                log("### PostBoxIapV4 InappV4_PostBox_Init, live server");
            }
        }

        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();
#ifdef __ANDROID__
		postBoxInit(strJSON.c_str());
#endif
        ApiAction::initPostBox = 1;
    }
}

/**
 * 이하
 * API 호출 시 input arg 정리와
 * API 호출 및 로그 발생을 담당하는 함수
 *
 * callApiAndMakeLog, callApiWithSelectionApi, processAfterApiCalling
 */

/**
 * API 호출에 필요한 input arg를 정리
 * @param apiName
 * @param strJSON
 */
void ApiAction::callApiAndMakeLog(std::string apiName, std::string strJSON) {
	C2SModule_Cocos2dx::C2SModuleError error;
	log("====>>>> strJSON : %s", strJSON.c_str());

	// 로그 정보 추가
	LogController::addLog("[Request : " + apiName + "] >>>>>\n" + strJSON);
	// QA 로그 정보 추가
	LogController::setApiResultState(false, 0);

	ApiAction::callApiWithSelectionApi(apiName, strJSON);
}

/**
 * API 이름에 따라 해당하는 SDK v1 API를 호출
 * @param apiName
 * @param intputArg
 */
void ApiAction::callApiWithSelectionApi(std::string apiName, std::string intputArg) {
	C2SModule_Cocos2dx::C2SModuleError error;

	// BannerInfo
	if(apiName == "C2SModulePromotion_GetBannerInfo") {
        if( ::C2SModulePromotion_GetBannerInfo(intputArg.c_str(), &error) == false)
            ApiAction::processAfterApiCalling(apiName, error.get()->GetCode(), error.get()->GetMessage());
	}
	else if(apiName == "C2SModuleHivePromotion_GetBannerInfo") {
		if( ::C2SModuleHivePromotion_GetBannerInfo(intputArg.c_str(), &error) == false)
			ApiAction::processAfterApiCalling(apiName, error.get()->GetCode(), error.get()->GetMessage());
	}
	// InappV4
	else if(apiName == "C2SModuleInappV4_ItemInfo") {
		if( ::C2SModuleInappV4_ItemInfo(intputArg.c_str(), &error) == false)
			ApiAction::processAfterApiCalling(apiName, error.get()->GetCode(), error.get()->GetMessage());
	}
	else if(apiName == "C2SModuleInappV4_Purchase") {
		if( ::C2SModuleInappV4_Purchase(intputArg.c_str(), &error) == false)
			ApiAction::processAfterApiCalling(apiName, error.get()->GetCode(), error.get()->GetMessage());
	}
	else if(apiName == "C2SModuleInappV4_Restore") {
		if( ::C2SModuleInappV4_Restore(intputArg.c_str(), &error) == false)
			ApiAction::processAfterApiCalling(apiName, error.get()->GetCode(), error.get()->GetMessage());
	}
	else if(apiName == "C2SModuleInappV4_TransactionFinish") {
		if( ::C2SModuleInappV4_TransactionFinish(intputArg.c_str(), &error) == false)
			ApiAction::processAfterApiCalling(apiName, error.get()->GetCode(), error.get()->GetMessage());
	}
	else if(apiName == "C2SModuleInappV4_GetSelectedMarket") {
		if( ::C2SModuleInappV4_GetSelectedMarket(intputArg.c_str(), &error) == false)
			ApiAction::processAfterApiCalling(apiName, error.get()->GetCode(), error.get()->GetMessage());
	}
	else if(apiName == "C2SModuleInappV4_GetAccountUuid") {
		if( ::C2SModuleInappV4_GetAccountUuid(intputArg.c_str(), &error) == false)
			ApiAction::processAfterApiCalling(apiName, error.get()->GetCode(), error.get()->GetMessage());
	}
	else if(apiName == "C2SModuleInappV4_MarketConnect") {
		if( ::C2SModuleInappV4_MarketConnect(intputArg.c_str(), &error) == false)
			ApiAction::processAfterApiCalling(apiName, error.get()->GetCode(), error.get()->GetMessage());
	}
#ifdef __ANDROID__
	else if(apiName == "C2SModuleInappV4_SelectMarket") {
		if( ::C2SModuleInappV4_SelectMarket(intputArg.c_str(), &error) == false)
			ApiAction::processAfterApiCalling(apiName, error.get()->GetCode(), error.get()->GetMessage());
	}
	else if(apiName == "C2SModuleInappV4_RequestBalance") {
		if( ::C2SModuleInappV4_RequestBalance(intputArg.c_str(), &error) == false)
			ApiAction::processAfterApiCalling(apiName, error.get()->GetCode(), error.get()->GetMessage());
	}
	else if(apiName == "C2SModuleInappV4_ChargeBalance") {
		if( ::C2SModuleInappV4_ChargeBalance(intputArg.c_str(), &error) == false)
			ApiAction::processAfterApiCalling(apiName, error.get()->GetCode(), error.get()->GetMessage());
	}else if(apiName == "C2SModuleInappV4_ShowInAppMessages") {
		if( ::C2SModuleInappV4_ShowInAppMessages(intputArg.c_str(), &error) == false)
			ApiAction::processAfterApiCalling(apiName, error.get()->GetCode(), error.get()->GetMessage());
	}
    
#else
    else if(apiName == "C2SModuleInappV4_CheckPromotePurchase") {
        if( ::C2SModuleInappV4_CheckPromotePurchase(intputArg.c_str(), &error) == false)
            ApiAction::processAfterApiCalling(apiName, error.get()->GetCode(), error.get()->GetMessage());
    }
#endif
    else
		log("callApiWithSelectionApi, not supported api");
}

/**
 * API 호출 이후 로그 처리
 * @param apiName
 * @param errorCode
 * @param errorMsg
 */
void ApiAction::processAfterApiCalling(std::string apiName, int errorCode, std::string errorMsg) {
	log( "====>>>> (%s) error (%s)", apiName.c_str(), errorMsg.c_str() );

	// 로그 정보 추가
	std::stringstream ssErrorCode;
	ssErrorCode << errorCode;
	LogController::addLog("[error : " + apiName + "] >>>>>\nerror (" + ssErrorCode.str() + ")(" + errorMsg + ")");
	// QA 로그 정보 추가
	LogController::setApiResultState(true, errorCode);
}

// end of source
