#include "DealCallback.h"
#include "ApiAction.h"
#include "Parameter.h"
#include "LogController.h"
#include "DataManager.h"
#include "Postbox.hpp"
// UI Scene(view)
#include "ModuleListScene.h"
#include "LoginListScene.h"

#ifdef __ANDROID__
#include "JNISample.h"
#endif

#include "../external/json/rapidjson.h"
#include "../external/json/document.h"
#include "../external/json/writer.h"
#include "../external/json/stringbuffer.h"

#ifdef __ANDROID__
#include "C2SModule.h"
#include "C2SModuleApi.h"
#else
#include "C2SModule.h"
#include "C2SModuleApi.h"
#include "Module_iOS.h"
#endif
USING_NS_CC;
using namespace rapidjson;


//rapidjson::Value myData(rapidjson::kArrayType);
//rapidjson::Value gFriends(rapidjson::kArrayType);
//Json::Value DealCallback::gFriends = Json::Value::null;

Scene* DealCallback::createScene() {
	auto scene = Scene::create();
	auto layer = DealCallback::create();
	scene->addChild(layer);
	return scene;
}

bool DealCallback::init()
{
	if (!Layer::init()) {
		return false;
	}

	log("### DealCallback : init() ###");

	return true;
}

void DealCallback::ResultDelegate(C2SModuleApi api, const char* json, C2SModuleErrorCode code, const char* message) {
	log("====>>>> ResultDelegate api (%d)", api);
	log("====>>>> ResultDelegate json (%s)", json);
	log("====>>>> ResultDelegate message (%s)", message);
	log("====>>>> error code : code %d", code);

	bool isAuthorized = false;
	std::string did;
	std::string vid;

	// ResultDelegate 결과 정보를 샘플앱 로그로 전달
    std::string c2sModuleApiResult = "[Result : " + DealCallback::getApiName(api) + "]\n";
    if(json != NULL) {
        c2sModuleApiResult.append("[json] >>>>>\n");
        c2sModuleApiResult.append(json);
        c2sModuleApiResult.append("\n");
    } else {
        c2sModuleApiResult.append("[json] >>>>>\n");
        c2sModuleApiResult.append("NULL\n");
    }
    if(message != NULL) {
        c2sModuleApiResult.append("[message] >>>>>\n");
        c2sModuleApiResult.append(message);
        c2sModuleApiResult.append("\n");
    } else {
        c2sModuleApiResult.append("[message] >>>>>\n");
        c2sModuleApiResult.append("NULL\n");
    }
    if(code != NULL) {
        c2sModuleApiResult.append("[C2SModuleErrorCode] >>>>>\n");
        c2sModuleApiResult.append(to_string(code));
        c2sModuleApiResult.append("\n");
    } else {
        c2sModuleApiResult.append("[C2SModuleErrorCode] >>>>>\n");
        c2sModuleApiResult.append("NULL\n");
    }
    LogController::addLog(c2sModuleApiResult);

	// QA 로그 정보 추가
	LogController::setApiResultState(false, 0);

	if (json == NULL) {
		json = "";
	}
    Document arg;
    arg.Parse(json);

	switch (api) {
	/** 소셜 초기화 */
	case C2SModuleApi_SocialInitialize: {
		log("====>>>> ResultDelegate : C2SModuleApi_SocialInitialize");

//		if (reader.parse(json, arg) == false) {
//			log("====>>>> parse failed : (%s)", json);
//			return;
//		}

//		isAuthorized = arg.get("isAuthorized", true).asBool();
//		did = arg.get("did", "").asString();

        if(json == NULL) {
            log("====>>>> (C2SModuleApi_SocialInitialize) (json is null)");
            return;
        }

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

        if(code != 0)
        {
            log("====>>>> C2SModuleApi_SocialInitialize error occured");
            return;
        }

		isAuthorized = arg["isAuthorized"].GetBool();
		did = arg["did"].GetString();

		log("====>>>> isAuthorized : (%d)", isAuthorized);
		// set value
		Parameter::paramInitialize.isAuthorized = isAuthorized;

		log("====>>>> did : (%s)", did.c_str());
		// set value
		Parameter::paramInitialize.did = did;

		if (arg.HasMember("autoLoginType")) {		// autoLoginType이 존재하는 경우
//			std::string authType = arg.get("autoLoginType", "").asString();
			std::string authType = arg["autoLoginType"].GetString();

			log("====>>>> authLoginType : (%s)", authType.c_str());
			// set value
			Parameter::paramInitialize.authLoginType = authType;

			if (!authType.empty() && authType.compare("platform") == 0) {			// (1) autoLoginType = platform
				log("====>>>> Platform Login");

				// 오토 로그인 진행 (하이브 유저) : 오토 로그인을 할 경우 해당 코드를 사용합니다.
//				C2SModule_Cocos2dx::C2SModuleError error;
//				if (::C2SModuleSns_Login((char*) NULL, &error) == false) {
//					log("====>>>> Platform Login error (%s)", error.get()->GetMessage());
//				}
			} else if (!authType.empty() && authType.compare("guest") == 0) {		// (2) autoLoginType = guest
				log("====>>>> Guest Login");
				C2SModule_Cocos2dx::C2SModuleError error;
//				Json::Value argGuestLogin;
//				argGuestLogin["type"] = "guest";

				// 오토 로그인 진행 (게스트 유저) : 오토 로그인을 할 경우 해당 코드를 사용합니다.
//				Json::StyledWriter writer;
//				std::string strJSON = writer.write(argGuestLogin);
//				if (::C2SModuleSns_Login(strJSON.c_str(), &error) == false) {
//					log("====>>>> Guest Login error (%s)", error.get()->GetMessage());
//				}
			} else if (!authType.empty() && authType.compare("authorized") == 0) {	// (3) autoLoginType = authorized
				// 세션키는 없으나 정식 로그인한적이 있는 유저
				// nothing
                log("====>>>> authType nothing 1");
			} else {																// (4) autoLoginType = ""
				// empty string 세션키, 자동 로그인 불가
				// nothing
                log("====>>>> authType nothing 2");
			}
		} else {								// autoLoginType이 없는 경우
			log("====>>>> not autoLogin ");

			if (!isAuthorized) {	// (1) isAuthrized = false --> 정식 유저로 로그인한 적이 없다.
				log("### autoLoginType : X, isAuthorized : false");
				log("====>>>> Recommend : Guest Login + Hive Login");
			} else {				// (2) isAuthrized = true --> 정식 유저로 로그인한 적이 있다.
				log("### autoLoginType : X, isAuthorized : true");
				log("====>>>> Recommend : Hive Login UI");
			}
		}

		//
		DataManager::getApiAction(TAG_APIACTION_GETLOGINSTATE);
	}
	break;

	/** sns 세션 정보 */
	case C2SModuleApi_SnsSession: {
		log("====>>>> ResultDelegate : C2SModuleApi_SnsSession");

//		if (reader.parse(json, arg) == false) {
//			log("====>>>> parse failed : (%s)", json);
//			return;
//		}

		if(arg.HasParseError())
		{
			log("====>>>> parse failed : (%s)", json);
			return;
		}


//		int temp_vid = arg.get("vid", 0).asInt();

		std::string temp_vid;
		if(arg.HasMember("vid"))
		{
			temp_vid = arg["vid"].GetString();
			log("### vid (HIVE) : %s", temp_vid.c_str());
		}

		log("====>>>> SnsSession : HIVE User");

//		log("### sessionToken (HIVE) : %s", arg.get("sessionToken", "").asString().c_str());
        if(arg.HasMember("sessionToken")) {
            log("### sessionToken (HIVE) : %s", arg["sessionToken"].GetString());
        }

		// facebook 로그인 여부일때 페이스북 로그인 여부
//		bool temp_isAuthorized = arg.get("isAuthorized", false).asBool();	// default : false

        if(arg.HasMember("isAuthorized"))
        {
            bool temp_isAuthorized = arg["isAuthorized"].GetBool();	// default : false
            if(temp_isAuthorized == true) {
                log("====>>>> SnsSession : Facebook User");
                log("### isAuthorized(facebook) : %d", temp_isAuthorized);
            }
        }
	}
	break;

	/** sns 로그인 */
	case C2SModuleApi_SnsLogin: {
		log("====>>>> ResultDelegate : C2SModuleApi_SnsLogin");

//		if (reader.parse(json, arg) == false) {
//			log("====>>>> parse failed : (%s)", json);
//			return;
//		}

		if(arg.HasParseError())
		{
			log("====>>>> parse failed : (%s)", json);
			return;
		}

		//
		if (code != 0) {
			if(code == -3)
				Parameter::paramLogin.callState = false;
			return;
		} else {
			Parameter::paramLogin.callState = true;
		}

		if (arg.HasMember("interworkArg")) {
			// 프로모션 타겟팅 시작시 로그인이 불린 경우 들어옴.
			// Interwork 상황의 로그인 처리. ( ex. DoInterworkLogin(arg); )

			std::string interworkMessage = "C2SModuleApi_SnsLogin : InterworkStart";
			MessageBox(interworkMessage.c_str(), "");	// 메시지 다이얼로그

			log("====>>>> ResultDelegate : C2SModuleApi_SnsLogin interwork login..");
		} else {
			// 일반적인 로그인 상황 ( ex. DoLogin(arg); )
			if (arg.HasMember("type")) {	// type이 존재하는 경우 (게스트 혹은 하이브 유저)
				Parameter::paramLogin.callState = true;
//				std::string rst_type = arg.get("type", "").asString();
				std::string rst_type = arg["type"].GetString();
				// set value
				Parameter::paramLogin.type = rst_type;

				if (!rst_type.empty() && rst_type.compare("select") == 0) {

                    // Select User
//					std::string currentVid = arg.get("currentVid", "").asString();
//					std::string usedVid = arg.get("usedVid", "").asString();

                    std::string currentVid = arg["currentVid"].GetString();
                    std::string usedVid = arg["usedVid"].GetString();

                    log("### SnsLogin -> type : platform(select)");
//					log("### (1) type (HIVE)(Select) : %s", arg.get("type", "").asString().c_str());
//					log("### (2) currentVid (HIVE)(Select) : %s", arg.get("currentVid", "").asString().c_str());
//					log("### (3) usedVid (HIVE)(Select): %s", arg.get("usedVid", "").asString().c_str());

                    log("### (1) type (HIVE)(Select) : %s", arg["type"].GetString());
                    log("### (2) currentVid (HIVE)(Select) : %s", arg["currentVid"].GetString());
                    log("### (3) usedVid (HIVE)(Select): %s", arg["usedVid"].GetString());

                    Document requestArg;
                    requestArg.SetObject();
                    Document::AllocatorType &requestArgAllocator = requestArg.GetAllocator();

                    rapidjson::Value argCurrent(rapidjson::kObjectType);
                    argCurrent.AddMember("vid", StringRef(currentVid.c_str()), requestArgAllocator);
                    Parameter::paramLogin.select_currentVid = currentVid;        // 커스텀 할 수 있게 값을 전달.

//					Json::Value argCurrent;
//					argCurrent["vid"] = currentVid;
//					Parameter::paramLogin.select_currentVid = currentVid;		// 커스텀 할 수 있게 값을 전달.

//					Json::Value currentData;
//					currentData["Name"] = "test1";
//					currentData["Level"] = "1";
//					currentData["Gold"] = "11";
//					currentData["Cash"] = "111";
//
//					argCurrent["data"] = currentData;

                    rapidjson::Value currentData(rapidjson::kObjectType);
                    currentData.AddMember("Name", "test1", requestArgAllocator);
                    currentData.AddMember("Level", "1", requestArgAllocator);
                    currentData.AddMember("Gold", "11", requestArgAllocator);
                    currentData.AddMember("Cash", "111", requestArgAllocator);

                    argCurrent.AddMember("data", currentData, requestArgAllocator);


//					Json::Value argUsed;
//					argUsed["vid"] = usedVid;
//					Parameter::paramLogin.select_usedVid = usedVid;				// 커스텀 할 수 있게 값을 전달.

                    rapidjson::Value argUsed(rapidjson::kObjectType);
                    argUsed.AddMember("vid", StringRef(usedVid.c_str()), requestArgAllocator);
                    Parameter::paramLogin.select_usedVid = usedVid;

//					Json::Value usedData;
//					usedData["Name"] = "test2";
//					usedData["Level"] = "2";
//					usedData["Gold"] = "22";
//					usedData["Cash"] = "222";
//
//					argUsed["data"] = usedData;

                    rapidjson::Value usedData(rapidjson::kObjectType);
                    usedData.AddMember("Name", "test2", requestArgAllocator);
                    usedData.AddMember("Level", "2", requestArgAllocator);
                    usedData.AddMember("Gold", "22", requestArgAllocator);
                    usedData.AddMember("Cash", "222", requestArgAllocator);

                    argUsed.AddMember("data", usedData, requestArgAllocator);

//					Json::Value arg;
//					arg["type"] = "select";
//					arg["currentVidData"] = argCurrent;
//					arg["usedVidData"] = argUsed;

                    requestArg.AddMember("type", "select", requestArgAllocator);
                    requestArg.AddMember("currentVidData", argCurrent, requestArgAllocator);
                    requestArg.AddMember("usedVidData", argUsed, requestArgAllocator);

//					Json::StyledWriter writer;
//					std::string strJSON = writer.write(arg);
                    StringBuffer buffer;
                    Writer<StringBuffer> writer(buffer);
                    requestArg.Accept(writer);

                    std::string strJSON = buffer.GetString();

					DataManager::setParamValue_string(TAG_APIACTION_LOGIN, "selectLoginInfo", strJSON);	// 셀렉트 로그인 관련 정보 전달.

                    // 셀렉트 로그인 상황 -> 셀렉트 선택창을 띄우지 않기 위해 C2SModuleSns_Login 요청을 주석 처리.

                    // HIVE 1.10.1 업데이트에서 주석처리 되었습니다. (자동 셀렉트 상황 방지)
                    // 하이브 로그인 시 상황에 따라 자동으로 셀렉트 설정 화면이 나올수 있게 하는 코드.
//					C2SModule_Cocos2dx::C2SModuleError error;
//					if (::C2SModuleSns_Login(strJSON.c_str(), &error) == false) {
//						log("====>>>> Guest Login error (%s)", error.get()->GetMessage());
//					}

                    LoginListScene::isSelectState = true;            // 셀렉트 로그인 가능하게 bool(flag) 값 변경.

                    // 셀렉트 상황이 된 것을 임의의 dialog로 알림
					MessageBox("- 하이브 로그인 성공!\n- 유저 정보 선택을 해주세요!\n(화면 애니메니션 효과가 완료되면 진행해주세요)", "");
                    // Scene 이동
                    // LoginListScene으로 이동시키는 이유 : Engagement 기능등을 통해 외부 화면에서 로그인되었을 경우 해당 LoginListScene에서 유저 정보 선택을 할 수 있게 하기 위해.
                    if (Parameter::paramSceneChecker.sceneState != 1) {
                        // 이미 LoginListScene에 도달해 있는 경우 다시 LoginListScene으로 교체할 필요 없음
                        log("### paramSceneChecker : need to go LoginListScene");
//                        auto scene = TransitionFadeTR::create(1.0, LoginListScene::createScene());
                        // 메인스레드에서 화면 전환이 이루어지도록 변경
                        Director::getInstance()->getScheduler()->performFunctionInCocosThread([=]()->void{
                            auto scene = LoginListScene::createScene();

                            Director::getInstance()->replaceScene(scene);
                        });
                    } else {
                        log("### paramSceneChecker : already arrived LoginListScene");
                    }
                    // c2ssinki TODO : select 상황에서 씬이동 체크
				} else {
//					vid = arg.get("vid", "").asString();
                    vid = arg["vid"].GetString();

					// show value
					log("### Api_SnsLogin -> type : guest or Hive(no_select)");
//					log("### (1) vid (guest or Hive)(no_Select) : %s", arg.get("vid", "").asString().c_str());
//					log("### (2) uid (guest or Hive)(no_Select) : %s", arg.get("uid", "").asString().c_str());
//					log("### (3) sessionToken (guest or Hive)(no_Select) : %s", arg.get("sessionToken", "").asString().c_str());

                    log("### (1) vid (guest or Hive)(no_Select) : %s", arg["vid"].GetString());
                    if(arg.HasMember("uid"))
                    {
                        log("### (2) uid (guest or Hive)(no_Select) : %s", arg["uid"].GetString());
                    }
                    log("### (3) sessionToken (guest or Hive)(no_Select) : %s", arg["sessionToken"].GetString());

					// set value
//					Parameter::paramLogin.vid = arg.get("vid", "").asString();
//					Parameter::paramLogin.uid = arg.get("uid", "").asString();
//					Parameter::paramLogin.sessionToken = arg.get("sessionToken", "").asString();

//                    DataManager::setParamValue_string(TAG_APIACTION_LOGIN, "vid", arg["vid"].GetString());
                    Parameter::paramLogin.vid = arg["vid"].GetString();

                    if(arg.HasMember("uid"))
                    {
//                        DataManager::setParamValue_string(TAG_APIACTION_LOGIN, "uid", arg["uid"].GetString());
                        Parameter::paramLogin.uid = arg["uid"].GetString();
                        Parameter::paramLogin.type = "platform";
                    }
                    Parameter::paramLogin.sessionToken = arg["sessionToken"].GetString();

					DataManager::setParamValue_bool(TAG_APIACTION_INITIALIZE, "loginState", true);	// 씬 전환.
                    // TODO : test code (jschoi)
//                    DataManager::setParamValue_bool(TAG_APIACTION_INITIALIZE, "callState", true);	// 씬 전환.
				}
			} else {    // type이 없는 경우! --> 정식 로그인을 요청해야 한다.
                // case 1 : 앱 실행 후 바로 하이브 로그인.
                // case 2 : 게스트 유저가 HIVE 정식 유저로 회원가입하여 로그인.
                // case 3 : select 상황에서 특정 계정을 선택하여 로그인.

                if (arg.HasMember("vid")) {
                    vid = arg["vid"].GetString();

                    // show value
                    log("##### Api_SnsLogin -> no type() #####");
                    log("##### (1) vid (no type)() : %s #####", vid.c_str());
                    Parameter::paramLogin.vid = vid;
                }
                if (arg.HasMember("uid")) {
                    log("##### (2) uid (no type)() : %s #####", arg["uid"].GetString());
                    Parameter::paramLogin.uid = arg["uid"].GetString();
                    Parameter::paramLogin.type = "platform";
                }
                if (arg.HasMember("sessionToken")) {
                    Parameter::paramLogin.sessionToken = arg["sessionToken"].GetString();
                }

                DataManager::setParamValue_bool(TAG_APIACTION_INITIALIZE, "loginState", true);    // 씬 전환.

                // for select situation : 셀렉트 상황이 끝났을 경우 특정 변수를 false로 전환.
                DataManager::setState(TAG_SCENENUM_LOGIN_LIST, "isSelection", false);
			}

			bool isGuestLogin = false;

//			Json::StyledWriter writer;
//			std::string strJSON2 = writer.write(arg);

            StringBuffer buffer;
            Writer<StringBuffer> writer(buffer);
            arg.Accept(writer);

            std::string strJSON2 = buffer.GetString();

			if (arg.HasMember("type")) {		// type이 존재하지만.
//				std::string type = arg.get("type", "").asString();
                std::string type = arg["type"].GetString();

				if (!type.empty() && type.compare("guest") == 0) {	// type이 게스트인 경우
					isGuestLogin = true;
					// show value
					log("### Api_SnsLogin -> type : guest");
//					log("### (1) sessionkey (GUEST) : %s", arg.get("sessionToken", "").asString().c_str());
//					log("### (2) vid (GUEST) : %s", arg.get("vid", "").asString().c_str());
//					log("### (3) type (GUEST) : %s", arg.get("type", "").asString().c_str());
                    log("### (1) sessionkey (GUEST) : %s", arg["sessionToken"].GetString());
                    log("### (2) vid (GUEST) : %s", arg["vid"].GetString());
                    log("### (3) type (GUEST) : %s", arg["type"].GetString());

					// set value
//					Parameter::paramLogin.vid = arg.get("vid", "").asString();
//					Parameter::paramLogin.uid = arg.get("uid", "").asString();
//					Parameter::paramLogin.sessionToken = arg.get("sessionToken", "").asString();

//                    DataManager::setParamValue_string(TAG_APIACTION_LOGIN, "vid", arg["vid"].GetString());
                    Parameter::paramLogin.vid = arg["vid"].GetString();
                    log("### paramLogin.vid : %s", Parameter::paramLogin.vid.c_str());
                    if(arg.HasMember("uid"))
                    {
//                        DataManager::setParamValue_string(TAG_APIACTION_LOGIN, "uid", arg["uid"].GetString());
                        Parameter::paramLogin.uid = arg["uid"].GetString();
                    }
                    else
                        DataManager::setParamValue_string(TAG_APIACTION_LOGIN, "uid", "");
                    Parameter::paramLogin.sessionToken = arg["sessionToken"].GetString();
				}
			}

			if (!isGuestLogin) {
				// 하이브 유저에 따른 UI 변경.
				// nothing
			} else {
				// 게스트 유저에 따른 UI 변경.
				// nothing
			}

			//전면 배너 호출 테스트용 코드
//			Json::Value argBanner;
//			argBanner["type"] = "fullBanner"; //전면 배너 호출시 type은 fullBanner
//
//			Json::StyledWriter writerBanner;
//			std::string strJSONBanner = writerBanner.write(argBanner);
//
//			C2SModule_Cocos2dx::C2SModuleError error;
//
//			if (::C2SModulePromotion_Show(strJSONBanner.c_str(), &error) == false) {
//				log("====>>>> error code : %d", error.get()->GetCode());
//				log("====>>>> Mercury error (%s)", error.get()->GetMessage());
//			}
		}
	}
	break;

	/** sns 로그아웃 */
	case C2SModuleApi_SnsLogout: {
		log("====>>>> ResultDelegate : C2SModuleApi_SnsLogout");
		//로그아웃된 UI로 변경.
	}
	break;

//    /** sns 탈퇴 */
//	C2SModuleApi_SnsUnregister:

    /** sns 포스팅 */
	case C2SModuleApi_SnsPost : {
		log("====>>>> ResultDelegate : C2SModuleApi_SnsPost");
	}
	break;

	/** sns 메시지 보내기 */
	case C2SModuleApi_SnsMessage: {
		log("====>>>> ResultDelegate : C2SModuleApi_SnsMessage");

//		if (reader.parse(json, arg) == false) {
//			log("====>>>> parse failed : (%s)", json);
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

        // 다이얼로그와 로그로 출력 (191007)
        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();

        log("====>>>> strJSON : %s", strJSON.c_str());

        // TODO : 게스트 상태에서 호출 시 크래시 이슈가 있어 임시 주석 처리 (191007)
//        if (arg.HasMember("service")) //Invite Dialog를 호출한 경우
//		{
////			std::string type = arg.get("service", "").asString();
//            std::string type = arg["service"].GetString();
//
//			if (!type.empty() && type.compare("facebook") == 0) {
//			//친구 초대를 성공한 경우 결과 인자로 "service", "facebookId"가 넘어오며,
//			//취소등의 이유로 실패한 경우 "service" 인자만 넘어옵니다.
////				log("====>>>> C2SModuleApi_SnsMessage service : %s", arg.get("service", "").asString().c_str());
//                log("====>>>> C2SModuleApi_SnsMessage service : %s", arg["service"].GetString());
//				if (arg.HasMember("facebookId")) {
//					log("====>>>> C2SModuleApi_SnsMessage facebookId : %s", arg["facebookId"][0].GetString()); 	//메세지 전송 실패할 경우 해당 값은 안내려옵니다.
////                    arg["facebookId"][0].asString().c_str()); 	//메세지 전송 실패할 경우 해당 값은 안내려옵니다.
//				}
//			}
//		}
	}
	break;

	/** sns 유저 정보 요청 */
	case C2SModuleApi_SnsUser: {
		log("====>>>> C2SModuleApi_SnsUser");

//		if (reader.parse(json, arg) == false) {
//			log("====>>>> parse failed : (%s)", json);
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

        // 다이얼로그와 로그로 출력 (190505)
        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();
//        showDialogMsg(strJSON.c_str());	// 메시지 다이얼로그
        log("====>>>> strJSON : %s", strJSON.c_str());

        // TODO : jschoi
        // 크래시 이슈가 있어 임시 주석 처리 (190505)
		if (arg.HasMember("service")) {
//			log("====>>>> C2SModuleApi_SnsUser service : %s",	arg.get("service", "").asString().c_str());
//			std::string type = arg.get("service", "").asString();

            log("====>>>> C2SModuleApi_SnsUser service : %s",	arg["service"].GetString());
            std::string type = arg["service"].GetString();

//			if (!type.empty() && type.compare("facebook") == 0) {
//				if (arg.HasMember("url")) // 나를 초대한 친구의 profile image url 정보를 호출한 경우
//				{
////					log("====>>>> C2SModuleApi_SnsUser FB profile image url : %s", arg.get("url", "").asString().c_str());
//                    log("====>>>> C2SModuleApi_SnsUser FB profile image url : %s", arg["url"].GetString());
//				}
//
//				if (arg.HasMember("uid")) // 나의 Facebook 계정 정보를 호출한 경우
//				{
////					log("====>>>> C2SModuleApi_SnsUser FB My accout uid : %s", arg.get("uid", "").asString().c_str());
////					log("====>>>> C2SModuleApi_SnsUser FB My accout username : %s", arg.get("username", "").asString().c_str());
////					log("====>>>> C2SModuleApi_SnsUser FB My accout second_email : %s", arg.get("second_email", "").asString().c_str());
////					log("====>>>> C2SModuleApi_SnsUser FB My accout e-mail : %s", arg.get("e-mail", "").asString().c_str());
////					log("====>>>> C2SModuleApi_SnsUser FB My accout gender : %s", arg.get("gender", "").asString().c_str());
////					log("====>>>> C2SModuleApi_SnsUser FB My accout screen_name : %s", arg.get("screen_name", "").asString().c_str());
////					log("====>>>> C2SModuleApi_SnsUser FB My accout language : %s", arg.get("language", "").asString().c_str());
////					log("====>>>> C2SModuleApi_SnsUser FB My accout country : %s", arg.get("country", "").asString().c_str());
//
//                    log("====>>>> C2SModuleApi_SnsUser FB My accout uid : %s", arg["uid"].GetString());
//                    log("====>>>> C2SModuleApi_SnsUser FB My accout username : %s", arg["username"].GetString());
//                    log("====>>>> C2SModuleApi_SnsUser FB My accout second_email : %s", arg["second_email"].GetString());
//                    log("====>>>> C2SModuleApi_SnsUser FB My accout e-mail : %s", arg["e-mail"].GetString());
//                    log("====>>>> C2SModuleApi_SnsUser FB My accout gender : %s", arg["gender"].GetString());
//                    log("====>>>> C2SModuleApi_SnsUser FB My accout screen_name : %s", arg["screen_name"].GetString());
//                    log("====>>>> C2SModuleApi_SnsUser FB My accout language : %s", arg["language"].GetString());
//                    log("====>>>> C2SModuleApi_SnsUser FB My accout country : %s", arg["country"].GetString());
//				}
//			}
		} else {
			// 나의 정보 가져가기
			log("====>>>> C2SModuleApi_SnsUser HIVE My User Info");

			if(arg.HasMember("list"))
            {
                log("### set myData : arg[list]");
                const rapidjson::Value& dataList = arg["list"];

//			if (myData != Json::Value::null) {
                if(!dataList.IsNull()) {
//				int count = myData.size();
                    int count = dataList.Size();
                    if (count == 0) {
                        // Display : no DATA!
//				MenuItem* item = MenuItemLabel::create(Label::createWithTTF("No Data.","fonts/NanumGothic.ttc", 40));
//				myDataMenu->addChild(item);
//				std::string vid = "no Data!";
                    } else {
                        for (int i = 0; i < count; i++) {
                            log("### get in Api_snsUser --> myData");

                            if(Parameter::paramSnsUser.vid == "")	// 나의 정보 호출하기 였을 경우.
                            {
                                // set value
//							std::string vid = myData[i].get("vid", "undefined").asString();
//							std::string uid = myData[i].get("uid", "undefined").asString();
//							Parameter::paramLogin.uid = uid;	// set value (Parameter)
//							std::string nickName = myData[i].get("nickName", "undefined").asString();
//							std::string profileImageURL = myData[i].get("profileImageURL", "undefined").asString();
//							std::string alias = myData[i].get("alias", "undefined").asString();
//							std::string country = myData[i].get("country", "undefined").asString();
//							std::string birthday = myData[i].get("birthday", "undefined").asString();
//							std::string gender = myData[i].get("gender", "undefined").asString();
//							std::string comment = myData[i].get("comment", "undefined").asString();
//							std::string email = myData[i].get("e-mail", "undefined").asString();
//							std::string facebookId = myData[i].get("facebookId", "undefined").asString();
//							int newMessageCount = myData[i].get("newMessageCount", 0).asInt();
//							int newFriendCount = myData[i].get("newFriendCount", 0).asInt();
//							int newRequestCount = myData[i].get("newRequestCount", 0).asInt();

                                if(arg["list"][i].HasMember("vid"))
                                {
                                    std::string vid = arg["list"][i]["vid"].GetString();
                                    log("### myData_vid : %s", vid.c_str());
                                    if(Parameter::paramInitialize.setGameInfo_emergencyMode == true) {
                                        Parameter::paramLogin.vid = vid;
                                    }
                                }

                                if(arg["list"][i].HasMember("uid"))
                                {
                                    std::string uid = arg["list"][i]["uid"].GetString();
                                    DataManager::setParamValue_string(TAG_APIACTION_LOGIN, "uid", uid);
                                    log("### myData_uid : %s", uid.c_str());
                                    if(Parameter::paramInitialize.setGameInfo_emergencyMode == true) {
                                        Parameter::paramLogin.uid = uid;
                                    }
                                }

                                if(arg["list"][i].HasMember("nickName"))
                                {
                                    std::string nickName = arg["list"][i]["nickName"].GetString();
                                    log("### myData_nickname : %s", nickName.c_str());
                                }

                                if(arg["list"][i].HasMember("profileImageURL"))
                                {
                                    std::string profileImageURL = arg["list"][i]["profileImageURL"].GetString();
                                    log("### myData_profileImageURL : %s", profileImageURL.c_str());
                                }

                                if(arg["list"][i].HasMember("alias"))
                                {
                                    std::string alias = arg["list"][i]["alias"].GetString();
                                    log("### myData_alias : %s", alias.c_str());
                                }

                                if(arg["list"][i].HasMember("country"))
                                {
                                    std::string country = arg["list"][i]["country"].GetString();
                                    log("### myData_country : %s",country.c_str());
                                }

                                if(arg["list"][i].HasMember("birthday"))
                                {
                                    std::string birthday = arg["list"][i]["birthday"].GetString();
                                    log("### myData_birthday : %s", birthday.c_str());
                                }

                                if(arg["list"][i].HasMember("gender"))
                                {
                                    std::string gender = arg["list"][i]["gender"].GetString();
                                    log("### myData_gender : %s", gender.c_str());
                                }

                                if(arg["list"][i].HasMember("comment"))
                                {
                                    std::string comment = arg["list"][i]["comment"].GetString();
                                    log("### myData_comment : %s", comment.c_str());
                                }

                                if(arg["list"][i].HasMember("e-mail"))
                                {
                                    std::string email = arg["list"][i]["e-mail"].GetString();
                                    log("### myData_e-mail : %s", email.c_str());
                                }

                                if(arg["list"][i].HasMember("facebookId"))
                                {
                                    std::string facebookId = arg["list"][i]["facebookId"].GetString();
                                    log("### myData_facebookId : %s", facebookId.c_str());
                                }

                                if(arg["list"][i].HasMember("newMessageCount"))
                                {
                                    int newMessageCount = arg["list"][i]["newMessageCount"].GetInt();
                                    log("### myData_newMessageCount : %d", newMessageCount);
                                }

                                if(arg["list"][i].HasMember("newFriendCount"))
                                {
                                    int newFriendCount = arg["list"][i]["newFriendCount"].GetInt();
                                    log("### myData_newFriendCount : %d", newFriendCount);
                                }

                                if(arg["list"][i].HasMember("newRequestCount"))
                                {
                                    int newRequestCount = arg["list"][i]["newRequestCount"].GetInt();
                                    log("### myData_newRequestCount : %d", newRequestCount);
                                }
                            }
                            else {	// 특정 vid의 정보 호출.
                                // TODO : 모두 파싱할 때 crash 가능성 있음(json contains 체크 필요)
                            }
                        }
                    }

//				myData.clear();
//				myData = Json::Value::null;
                }
            }
		}
	}
	break;

    case C2SModuleApi_SnsUserDelete: {
        if(code == 0) {
            // 계정 삭제 성공일 경우에만 로그인 화면으로 이동
            DataManager::setParamValue_bool(TAG_APIACTION_LOGIN, "callState", false);

            Director::getInstance()->getScheduler()->performFunctionInCocosThread([=]()->void{
                auto scene = TransitionFadeTR::create(1.0, LoginListScene::createScene());
                Director::getInstance()->replaceScene(scene);
            });
        }
    }
        break;

	/** sns 친구 정보 요청 */
	case C2SModuleApi_SnsFriends: {
		log("====>>>> C2SModuleApi_SnsFriends");
//		myData = Json::Value::null;

//		if (reader.parse(json, arg) == false) {
//			log("====>>>> parse failed : (%s)", json);
//			return;
//		}

        if(json != "" || json != NULL)
        {
            log("====>>>> parse failed : no arg");
            return;
        }
        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

		// getInvitations 콜백 처리
//		std::string type = arg.get("type", "").asString();
//		if(!type.empty() && type.compare("getInvitations") == 0)
//		{
//			log("====>>>> 11111");
//			int msgCount = arg["messages"].size();
//			log("====>>>> C2SModuleApi_SnsFriends message count : %d", msgCount);
//			if(msgCount == 0)
//			{
//				// 메세지 없음.
//			}
//			else
//			{
//				for(int i = 0; i < msgCount; i++)
//				{
//					std::string msg = arg["messages"][i]["message"].asString();
//					std::string senderId = arg["messages"][i]["sender_id"].asString();
//					std::string receiverId = arg["messages"][i]["receiver_id"].asString();
//					log("====>>>> C2SModuleApi_SnsFriends msg : %s", msg.c_str());
//					log("====>>>> C2SModuleApi_SnsFriends senderId : %s", senderId.c_str());
//					log("====>>>> C2SModuleApi_SnsFriends receiverId : %s", receiverId.c_str());
//				}
//			}
//		}

        if(json != "" || json != NULL) {
            if (arg.HasMember("service")) // Facebook에서 나를 초대한 친구의 목록을 호출한 경우
            {
        //			std::string type = arg.get("type", "").asString();
                std::string type = arg["type"].GetString();

                if (!type.empty() && type.compare("facebook") == 0) {
        //				std::string id = arg["receivedInvite"][0]["from"]["id"].asString();
                    std::string id = arg["receivedInvite"][0]["from"]["id"].GetString();
                    log("====>>>> C2SModuleApi_SnsFriends value %s", id.c_str());
        //				std::string receivedInviteFacebookId = arg["data"]["values"][0]["nameValuePairs"]["from"].asString();
        //				log("====>>>> C2SModuleApi_SnsFriends receivedInvite %s", receivedInviteFacebookId.c_str());
                }
            } else {    //HIVE, Facebook, Google+, Sinaweibo 의 친구 리스트를 호출한 경우
                log("### Sns_Friends : HIVE, facebook, googleplus");
        //			gFriends = arg["other"];
        //
        //			Json::StyledWriter writer;
        //			std::string strJSON = writer.write(arg);
        //
        //			int gFriendsCnt = gFriends.size();
        //			log("### gFriendsCnt size : %d", gFriendsCnt);
        //			std::string sns_vid = gFriends[0]["vid"].asString();
        //			log("### Sns_Friends value (sns_vid : %s)", sns_vid.c_str());
        //			std::string sns_uid = gFriends[0]["uid"].asString();
        //			log("### Sns_Friends value (sns_uid : %s)", sns_uid.c_str());
        //			std::string sns_nickName = gFriends[0]["nickName"].asString();
        //			log("### Sns_Friends value (sns_nickName : %s)", sns_nickName.c_str());
        //			std::string sns_profileImageURL = gFriends[0]["profileImageURL"].asString();
        //			log("### Sns_Friends value (sns_profileImageURL : %s)", sns_profileImageURL.c_str());
        //			std::string sns_vid2 = gFriends[1]["vid"].asString();
        //			log("### Sns_Friends value (sns_vid2 : %s)", sns_vid2.c_str());
        //			std::string sns_uid2 = gFriends[1]["uid"].asString();
        //			log("### Sns_Friends value (sns_uid2 : %s)", sns_uid2.c_str());
        //			std::string sns_nickName2 = gFriends[1]["nickName"].asString();
        //			log("### Sns_Friends value (sns_nickName2 : %s)", sns_nickName2.c_str());
        //			std::string sns_profileImageURL2 = gFriends[1]["profileImageURL"].asString();
        //			log("### Sns_Friends value (sns_profileImageURL2 : %s)", sns_profileImageURL2.c_str());
        //
        //			std::string sns_vid3 = arg["game"][0]["vid"].asString();
        //			log("### Sns_Friends value (sns_vid3 : %s)", sns_vid3.c_str());
        //			std::string sns_uid3 = arg["game"][0]["uid"].asString();
        //			log("### Sns_Friends value (sns_uid3 : %s)", sns_uid3.c_str());
        //			std::string sns_nickName3 = arg["game"][0]["nickName"].asString();
        //			log("### Sns_Friends value (sns_nickName3 : %s)", sns_nickName3.c_str());
        //			std::string sns_profileImageURL3 = arg["game"][0]["profileImageURL"].asString();
        //			log("### Sns_Friends value (sns_profileImageURL3 : %s)", sns_profileImageURL3.c_str());
        //			std::string sns_vid4 = arg["game"][1]["vid"].asString();
        //			log("### Sns_Friends value (sns_vid4 : %s)", sns_vid4.c_str());
        //			std::string sns_uid4 = arg["game"][1]["uid"].asString();
        //			log("### Sns_Friends value (sns_uid4 : %s)", sns_uid4.c_str());
        //			std::string sns_nickName4 = arg["game"][1]["nickName"].asString();
        //			log("### Sns_Friends value (sns_nickName4 : %s)", sns_nickName4.c_str());
        //			std::string sns_profileImageURL4 = arg["game"][1]["profileImageURL"].asString();
        //			log("### Sns_Friends value (sns_profileImageURL4 : %s)", sns_profileImageURL4.c_str());

        //            Json::StyledWriter writer;
        //            std::string strJSON = writer.write(arg);

                if (arg.HasMember("other")) {
                    int gFriendsCnt = arg["other"].Size();
                    log("### gFriendsCnt size : %d", gFriendsCnt);

                    for (int i = 0; i < gFriendsCnt; i++) {
                        std::string sns_vid = arg["other"][i]["vid"].GetString();
                        log("### Sns_Friends value (sns_vid : %s)", sns_vid.c_str());
                        std::string sns_uid = arg["other"][i]["uid"].GetString();
                        log("### Sns_Friends value (sns_uid : %s)", sns_uid.c_str());
                        std::string sns_nickName = arg["other"][i]["nickName"].GetString();
                        log("### Sns_Friends value (sns_nickName : %s)", sns_nickName.c_str());
                        std::string sns_profileImageURL = arg["other"][i]["profileImageURL"].GetString();
                        log("### Sns_Friends value (sns_profileImageURL : %s)",
                            sns_profileImageURL.c_str());
                    }
                }

                if (arg.HasMember("game")) {
                    int gFriendsCnt = arg["game"].Size();
                    log("### gFriendsCnt size : %d", gFriendsCnt);

                    for (int i = 0; i < gFriendsCnt; i++) {
                        std::string sns_vid3 = arg["game"][i]["vid"].GetString();
                        log("### Sns_Friends value (sns_vid3 : %s)", sns_vid3.c_str());
                        std::string sns_uid3 = arg["game"][i]["uid"].GetString();
                        log("### Sns_Friends value (sns_uid3 : %s)", sns_uid3.c_str());
                        std::string sns_nickName3 = arg["game"][i]["nickName"].GetString();
                        log("### Sns_Friends value (sns_nickName3 : %s)", sns_nickName3.c_str());
                        std::string sns_profileImageURL3 = arg["game"][i]["profileImageURL"].GetString();
                        log("### Sns_Friends value (sns_profileImageURL3 : %s)",
                            sns_profileImageURL3.c_str());
                    }
                }

                StringBuffer buffer;
                Writer<StringBuffer> writer(buffer);
                arg.Accept(writer);

                std::string strJSON = buffer.GetString();

                log("====>>>> strJSON : %s", strJSON.c_str());
            }
        }
	}
	break;

	case C2SModuleApi_InappPrepare: {
		log(" ====>>>> ResultDelegate : C2SModuleApi_InappPrepare");
	}
	break;

	/** inapp 구매 요청 */
	case C2SModuleApi_InappPurchase: {
		log("=====>>>> ResultDelegate : C2SModuleApi_InappPurchase");
		log("=====>>>> ResultDelegate : C2SModuleApi_InappPurchase error code : %d", code);
		if (code != C2SModuleErrorCode_Success)
			return;

		//Google Receipt Sample Code.
//		if (reader.parse(json, arg) == false) {
//			log("====>>>> C2SModuleApi_InappPurchase parse failed : (%s)", json);
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

        // 로그로 출력 (190511)
        if(!arg.IsNull()) {
            StringBuffer buffer;
            Writer<StringBuffer> writer(buffer);
            arg.Accept(writer);

            std::string strJSON = buffer.GetString();
//        showDialogMsg(strJSON.c_str());	// 메시지 다이얼로그
            log("====>>>> strJSON : %s", strJSON.c_str());
        }

        // TODO : 파싱 시 크래시 이슈로 임시 주석 처리 (190511)
//		if (arg.HasMember("receipt")) {
////			std::string s_strSignature = arg["receipt"]["GOOGLEPLAY_SIGNATURE"].asString();
//            std::string s_strSignature = arg["receipt"]["GOOGLEPLAY_SIGNATURE"].GetString();
//			log("====>>>> ResultDelegate : s_strSignature(%s)",
//					s_strSignature.c_str());
//
////			std::string s_strReceipt = arg["receipt"]["GOOGLEPLAY_RECEIPT"].asString();
//            std::string s_strReceipt = arg["receipt"]["GOOGLEPLAY_RECEIPT"].GetString();
//			log("====>>>> ResultDelegate : s_strReceipt(%s)",	s_strReceipt.c_str());
//
//			//inappFinish();
//		}
	}
	break;

	case C2SModuleApi_InappRestore: {
		log("=====>>>> ResultDelegate : C2SModuleApi_InappRestore");
		log("=====>>>> ResultDelegate : C2SModuleApi_InappRestore error code : %d", code);
		if (code != C2SModuleErrorCode_Success || json == NULL)
			return;

		//Google Receipt Sample Code.
//		if (reader.parse(json, arg) == false) {
//			log("====>>>> C2SModuleApi_InappRestore parse failed : (%s)", json);
//			return;
//		}

        // 로그로 출력 (190520)
        if(!arg.IsNull()) {
            StringBuffer buffer;
            Writer<StringBuffer> writer(buffer);
            arg.Accept(writer);

            std::string strJSON = buffer.GetString();
//        showDialogMsg(strJSON.c_str());	// 메시지 다이얼로그
            log("====>>>> strJSON : %s", strJSON.c_str());
        }

        // TODO : 파싱 시 크래시 이슈로 임시 주석 처리 (190520)
//        if(arg.HasParseError())
//        {
//            log("====>>>> parse failed : (%s)", json);
//            return;
//        }
//
//		if (arg.HasMember("receipt")) {
////			std::string s_strSignature = arg["receipt"]["GOOGLEPLAY_SIGNATURE"].asString();
//            std::string s_strSignature = arg["receipt"]["GOOGLEPLAY_SIGNATURE"].GetString();
//			log("====>>>> ResultDelegate : s_strSignature(%s)", s_strSignature.c_str());
//
////			std::string s_strReceipt = arg["receipt"]["GOOGLEPLAY_RECEIPT"].asString();
//            std::string s_strReceipt = arg["receipt"]["GOOGLEPLAY_RECEIPT"].GetString();
//			log("====>>>> ResultDelegate : s_strReceipt(%s)", s_strReceipt.c_str());
//
//			//inappFinish();
//		}
	}
	break;

	case C2SModuleApi_InappFinish: {
		log("=====>>>> ResultDelegate : C2SModuleApi_InappFinish");
		//addCash();
	}
	break;

	/** 러비 잔액 조회 */
	case C2SModuleApi_InappRequestBalance: {
		log("====>>>> ResultDelegate : C2SModuleApi_InappRequestBalance");
	}
	break;

	/** inapp 상품 정보 */
	case C2SModuleApi_InappDescription: {
		log("====>>>> ResultDelegate : C2SModuleApi_InappDescription");
	}
	break;

	// HIVE IAP
	/** HiveIAP Initialize */
	/*
	case C2SModuleApi_HiveIAPinitialize: {
		log("=====>>>> ResultDelegate : C2SModuleApi_HiveIAPinitialize");

		Json::Value arg;
		Json::Reader reader;

		if (reader.parse(json, arg) == false) {
			log("====>>>> C2SModuleApi_HiveIAPinitialize parse failed : (%s)", json);
			return;
		}

		int count = arg["marketIds"].size();

		for(int i = 0; i < count; i++) {
			int marketId = arg["marketIds"][i].asInt();
			if(marketId == 3) {
				// Lebi 사용 가능.
				log("====>>>> C2SModuleApi_HiveIAPinitialize : Lebi market available.");
			}
		}

		break;
	}
	*/

	// HIVE IAP
	/** HiveIAP_Info */
	/*
	case C2SModuleApi_HiveIAPInfo: {
		log("====>>>> ResultDelegate : C2SModuleApi_HiveIAPInfo");

		Json::Value arg;
		Json::Reader reader;

		if (reader.parse(json, arg) == false) {
			log("====>>>> C2SModuleApi_HiveIAPInfo parse failed : (%s)", json);
			return;
		}

		std::string type = arg.get("type", "").asString();
		if(!type.empty() && type.compare("shopInfo") == 0) {
			int count = arg["products"].size();
			for(int i = 0; i < count; i++) {
				std::string pid = arg["products"][i]["pid"].asString();
				// 나머지 데이터 파싱 가능.
			}
		}else if(!type.empty() && type.compare("badge") == 0) {
			bool isBadgeExist = arg["isBadgeExist"].asBool();
			std::string badgeImageUrl = arg["badgeImageUrl"].asString();
		}else if(!type.empty() && type.compare("balance") == 0) {
			if(arg.isMember("balance")) {
				int balance = arg["balance"].asInt();
			}
		}
	}
	break;
	*/

	// HIVE IAP
	/** HiveIAP_Show */
	/*
	case C2SModuleApi_HiveIAPShow: {
		log("====>>>> ResultDelegate : C2SModuleApi_HiveIAPShow");

		Json:Value arg;
		Json::Reader reader;

		if (reader.parse(json, arg) == false) {
			log("====>>>> C2SModuleApi_HiveIAPShow parse failed : (%s)", json);
			return;
		}

		std::string type = arg.get("type", "").asString();
		if(!type.empty() && type.compare("charge") == 0) {
			if(arg.isMember("balance")) {
				int balance = arg["balance"].asInt();
			}
		}else if(!type.empty() && type.compare("payment") == 0) {

		}
	}
	break;
	*/

	// HIVE IAP
	/** HiveIAP_Purchase */
	/*
	case C2SModuleApi_HiveIAPPurchase:
	{

	}
	break;
	*/

	// HIVE IAP
	/** HiveIAP_Restore */
	/*
	case C2SModuleApi_HiveIAPRestore:
	{

	}
	break;
	*/

	case C2SModuleApi_PushRegister: {
		log("====>>>> ResultDelegate : C2SModuleApi_PushRegister");
	}
	break;

	case C2SModuleApi_PushUnregister: {
		log("====>>>> ResultDelegate : C2SModuleApi_PushUnregister");
//		Json::Value arg;
//		Json::Reader reader;

//		if (reader.parse(json, arg) == false) {
//			log("====>>>> parse failed : (%s)", json);
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }
	}
	break;

	case C2SModuleApi_PushDescription: {
		log("====>>>> ResultDelegate : C2SModuleApi_PushDescription");
//		if (reader.parse(json, arg) == false) {
//			log("====>>>> C2SModuleApi_PushDescription parse failed : (%s)", json);
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

//		Json::FastWriter writer;
//		std::string strJSON = writer.write(arg);

        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();

		log("====>>>> strJSON : %s", strJSON.c_str());
	}
	break;

	case C2SModuleApi_PromotionShow: {
		log("====>>>> ResultDelegate : C2SModuleApi_PromotionShow");
//		if (reader.parse(json, arg) == false) {
//			log("====>>>> C2SModuleApi_PromotionShow parse failed : (%s)", json);
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

//		int offerwallState = arg.get("offerwallState", 0).asInt();

        if(arg.HasMember("offerwallState"))
        {
            int offerwallState = arg["offerwallState"].GetInt();

            log("====>>>> ResultDelegate : offerwallState(%d)", offerwallState);
            if (offerwallState == -14) {
                // Offerwall Activation
                log(" ====>>>> ResultDelegate PromotionShow Activation");
            } else {    // offerwallState == -15 or ?
                // Offerwall Deactivation
                log(" ====>>>> ResultDelegate PromotionShow Deactivation");
            }
        }

//		int finishApp = arg.get("finishApp", 0).asInt();

        if(arg.HasMember("finishApp"))
        {
            int finishApp = arg["finishApp"].GetInt();
            /*
            if (finishApp == -1) {
                Director::getInstance()->end();		// 종료하기 누르면 앱 종료.
            }
            */
        }

		// [Interwork]
		if (arg.HasMember("isInterworkStart")) {
//			bool isInterworkStart = arg.get("isInterworkStart", false).asBool();
            bool isInterworkStart = arg["isInterworkStart"].GetBool();

			if (isInterworkStart) {
				// 프로모션 타겟팅 시작으로 인해 프로모션 쇼가 종료되고 interwork의 시작을 알림.
				log("====>>>> ResultDelegate : C2SModuleApi_PromotionShow interworkStart isInterWork true.");
				std::string interworkMessage = "C2SModuleApi_PromotionShow : InterworkStart";
				MessageBox(interworkMessage.c_str(), "");
			}
		}
	}
	break;

	case C2SModuleApi_PromotionAcquisition:
	{
		log( "====>>>> ResultDelegate : C2SModuleApi_PromotionAcquisition");

//		if( reader.parse(json, arg) == false ) {
//			log( "====>>>> C2SModuleApi_PromotionAcquisition parse failed : (%s)", json );
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();

        log("====>>>> strJSON : %s", strJSON.c_str());

        // TODO : jschoi, 파싱 로직 개선 (190506)
		// Json Parsing (promotion acquisition)
		if(Parameter::paramPromotionAcquisition.callState == 1) {
			std::string totalInfo = "";

			// (1) get link data
//			Parameter::paramPromotionAcquisition.inviteCommonLink = arg.get("inviteCommonLink", "").asString();
//			Parameter::paramPromotionAcquisition.inviteFacebookLink = arg.get("inviteFacebookLink", "").asString();
//			Parameter::paramPromotionAcquisition.inviteHivemsgLink = arg.get("inviteHivemsgLink", "").asString();
//			Parameter::paramPromotionAcquisition.linkShareURL = arg.get("inviteHivemsgLink", "").asString();

            log("jschoi acquisition testlog 001");
            if(arg.HasMember("inviteCommonLink")) {
                Parameter::paramPromotionAcquisition.inviteCommonLink = arg["inviteCommonLink"].GetString();
                log("### Parsing Complete! Link(inviteCommonLink) : %s", Parameter::paramPromotionAcquisition.inviteCommonLink.c_str());
            }
            if(arg.HasMember("inviteFacebookLink")) {
                Parameter::paramPromotionAcquisition.inviteFacebookLink = arg["inviteFacebookLink"].GetString();
                log("### Parsing Complete! Link(inviteFacebookLink) : %s", Parameter::paramPromotionAcquisition.inviteFacebookLink.c_str());
            }
            if(arg.HasMember("inviteHivemsgLink")) {
                Parameter::paramPromotionAcquisition.inviteHivemsgLink = arg["inviteHivemsgLink"].GetString();
                log("### Parsing Complete! Link(inviteHivemsgLink) : %s", Parameter::paramPromotionAcquisition.inviteHivemsgLink.c_str());
            }
            if(arg.HasMember("inviteHivemsgLink")) {
                Parameter::paramPromotionAcquisition.linkShareURL = arg["inviteHivemsgLink"].GetString();
                log("### Parsing Complete! Link(inviteHivemsgLink) : %s", Parameter::paramPromotionAcquisition.linkShareURL.c_str());
            }

            log("jschoi acquisition testlog 002");

			// (2) get qrcode data
            log("jschoi acquisition testlog 003");
//			Parameter::paramPromotionAcquisition.qrcode = arg.get("qrcode", "").asString();
            if(arg.HasMember("qrcode")) {
                Parameter::paramPromotionAcquisition.qrcode = arg["qrcode"].GetString();
                log("### Parsing Complete! QRCODE : %s", Parameter::paramPromotionAcquisition.qrcode.c_str());
            }

			// (3) get stageCampaigns data
//			Json::Value json_stageCampaigns = arg["stageCampaigns"];
//			Json::StyledWriter writer;
//			std::string str_stageCampaigns = writer.write(json_stageCampaigns);

            log("jschoi acquisition testlog 004");
            rapidjson::Value argStageCampaigns(rapidjson::kObjectType);
            if(arg.HasMember("stageCampaigns")) {
                log("jschoi acquisition testlog 0041");
//                std::string str_stageCampaigns = arg["stageCampaigns"].GetString();
//                Parameter::paramPromotionAcquisition.stageCampaigns = str_stageCampaigns;
//                log("### Parsing Complete! StageCampaigns : %s", Parameter::paramPromotionAcquisition.stageCampaigns.c_str());

                log("jschoi acquisition testlog 0042");

                log("jschoi acquisition testlog 0043");
                argStageCampaigns = arg["stageCampaigns"];
                // TODO : parse argStageCampaigns
            }

			// (4) get eachCampaign data
            log("jschoi acquisition testlog 005");
//			Json::Value json_eachCampaigns = arg["eachCampaigns"];
//			std::string str_eachCampaigns = writer.write(json_eachCampaigns);

            rapidjson::Value argEachCampaigns(rapidjson::kObjectType);
            if(arg.HasMember("eachCampaigns")) {
                argEachCampaigns = arg["eachCampaigns"];
                // TODO : parse argEachCampaigns
            }
//          std::string str_eachCampaigns = arg["eachCampaigns"].GetString();
//			Parameter::paramPromotionAcquisition.eachCampaigns = str_eachCampaigns;
//			log("### Parsing Complete! EachCampaigns : %s", Parameter::paramPromotionAcquisition.eachCampaigns.c_str());

			// ===== ===== ===== ===== =====
            // TODO : jschoi, 세부 파싱 로직은 주석 처리 (주석 시작)
//			// create totalInfo
//			// (1) link data
//            log("jschoi acquisition testlog 006");
//			totalInfo += "[LINK 보기]\n";
//			totalInfo += "inviteCommonLink : \n";
//			totalInfo += (Parameter::paramPromotionAcquisition.inviteCommonLink + "\n");
//			totalInfo += "inviteFacebookLink : \n";
//			totalInfo += (Parameter::paramPromotionAcquisition.inviteFacebookLink + "\n");
//			totalInfo += "inviteHivemsgLink : \n";
//			totalInfo += (Parameter::paramPromotionAcquisition.inviteHivemsgLink + "\n\n");
//			// (2) stageCampaigns data
//            log("jschoi acquisition testlog 007");
//			totalInfo += "[StageCampaigns 보기]\n";
//			for(int i = 0; i < arg["stageCampaigns"].Size(); i++) {
//				std::stringstream ss;
//				ss << (i+1);
//				std::string num = ss.str();
//				totalInfo += ("(" + num + ")\n");
//
//                log("jschoi acquisition testlog 008");
//				std::stringstream sss;
////				sss << json_stageCampaigns[i].get("count","").asInt();
//                sss << arg["stageCampaigns"][i]["count"].GetInt();
//				num = sss.str();
//				totalInfo += ("count : " + num + "\n");
//
////				totalInfo += ("desc : " + json_stageCampaigns[i].get("desc", "").asString() + "\n");
////				totalInfo += ("imgURL : " + json_stageCampaigns[i].get("imgURL", "").asString() + "\n");
//
//                log("jschoi acquisition testlog 009");
//                std::string desc = arg["stageCampaigns"][i]["desc"].GetString();
//                std::string imgURL = arg["stageCampaigns"][i]["imgURL"].GetString();
//                totalInfo += ("desc : " + desc + "\n");
//                totalInfo += ("imgURL : " + imgURL + "\n");
//
////				Json::Value json_item = json_stageCampaigns[i]["item"];
////				Json::Value json_nameValuePairs = json_item["nameValuePairs"];
//
//				// get item info
////				totalInfo += ("item : gems : " + json_stageCampaigns[i]["item"].get("gem", "").asString() + "\n");
////				totalInfo += ("item : ruby : " + json_stageCampaigns[i]["item"].get("ruby", "").asString() + "\n");
////				Json::StyledWriter writer;
////				std::string itemInfo = writer.write(json_stageCampaigns[i]["item"]);
//                std::string itemInfo = arg["stageCampaigns"][i]["item"].GetString();
//				totalInfo += ("item : " + itemInfo + "\n");
//
//
//                log("jschoi acquisition testlog 010");
//				std::stringstream ssss;
////				ssss << json_stageCampaigns[i].get("limit","").asInt();
//                ssss << arg["stageCampaigns"][i]["limit"].GetInt();
//				num = ssss.str();
//				totalInfo += ("limit : " + num + "\n");
//
////				totalInfo += ("title : " + json_stageCampaigns[i].get("title", "").asString() + "\n");
//
//                log("jschoi acquisition testlog 011");
//                std::string title = arg["stageCampaigns"][i]["title"].GetString();
//				totalInfo += ("title : " + title + "\n");
//			}
//            log("jschoi acquisition testlog 012");
//			totalInfo += "\n";
//			// (3) eachCampaigns data
//			totalInfo += "[EachCampaigns 보기]\n";
//			for(int i = 0; i < arg["eachCampaigns"].Size(); i++) {
//				std::stringstream ss;
//				ss << (i+1);
//				std::string num = ss.str();
//				totalInfo += ("(" + num + ")\n");
//
//				std::stringstream sss;
////				sss << json_eachCampaigns[i].get("count","").asInt();
//                sss << arg["eachCampaigns"][i]["count"].GetInt();
//				num = sss.str();
//				totalInfo += ("count : " + num + "\n");
//
////				totalInfo += ("desc : " + json_eachCampaigns[i].get("desc", "").asString() + "\n");
////				totalInfo += ("imgURL : " + json_eachCampaigns[i].get("imgURL", "").asString() + "\n");
//
//                std::string desc = arg["eachCampaigns"][i]["desc"].GetString();
//                std::string imgURL = arg["eachCampaigns"][i]["imgURL"].GetString();
//                totalInfo += ("desc : " + desc + "\n");
//                totalInfo += ("imgURL : " + imgURL + "\n");
//
////				Json::Value json_item = json_eachCampaigns[i]["item"];
////				Json::Value json_nameValuePairs = json_item["nameValuePairs"];
//
//				// get item info
////				totalInfo += ("item : gems : " + json_eachCampaigns[i]["item"].get("gem", "").asString() + "\n");
////				totalInfo += ("item : ruby : " + json_eachCampaigns[i]["item"].get("ruby", "").asString() + "\n");
////				Json::StyledWriter writer;
////				std::string itemInfo = writer.write(json_eachCampaigns[i]["item"]);
//				std::string itemInfo = arg["eachCampaigns"][i]["item"].GetString();
//				totalInfo += ("item : " + itemInfo + "\n");
//
//
//				std::stringstream ssss;
////				ssss << json_eachCampaigns[i].get("limit","").asInt();
//				ssss << arg["eachCampaigns"][i]["limit"].GetInt();
//				num = ssss.str();
//				totalInfo += ("limit : " + num + "\n");
//
////				totalInfo += ("title : " + json_eachCampaigns[i].get("title", "").asString() + "\n");
//                std::string title = arg["eachCampaigns"][i]["title"].GetString();
//				totalInfo += ("title : " + title + "\n");
//			}
// TODO : jschoi, 주석 처리 (종료)

			// ===== ===== ===== ===== =====

			// sandbox 서버의 경우 아직 집객 정보 등록이 안되어있어서 반환할 수 있는 값이 없음. (date 160525)
			/*
			if(Parameter::paramInitialize.useSandbox == true) {
				std::string sandbox_message = "샌드박스 서버에 등록된 집객 정보가 없습니다.";
				showDialogMsg(sandbox_message.c_str());
			} else {
			*/

			// (1) show totalInfo
//			showDialogMsg(totalInfo.c_str());       // TODO : 주석 처리
			// (2) show QRcode
			DataManager::switchDisplayQRcode(true);
		}
	}
	break;

	case C2SModuleApi_PromotionInfo: {
		log("====>>>> ResultDelegate : C2SModuleApi_PromotionInfo");

//		if (reader.parse(json, arg) == false) {
//			log("====>>>> C2SModuleApi_PromotionInfo parse failed : (%s)", json);
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

		log("PromotionInfoTest (1)");
//		Json::FastWriter writer;
//		std::string strJSON = writer.write(arg);

        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();

		log("====>>>> strJSON : %s", strJSON.c_str());
	}
	break;

	//    /** 소셜 뷰 출력 */
	case C2SModuleApi_SocialShow: {
		log(" ====>>>>ResultDelegate SocialShow");

		// HIVE IAP
		/*
		Json::Value arg;
		Json::Reader reader;

		if (reader.parse(json, arg) == false) {
			log("====>>>> C2SModuleApi_SocialShow parse failed : (%s)", json);
			return;
		}

		std::string type = arg.get("type", "").asString();
		if(!type.empty() && type.compare("adultAuth") == 0) {
			if(arg.isMember("isAdult")) {
				bool isAdult = arg["isAdult"].asBool();
				if(isAdult) {
					// 성인 인증 성공.
				} else {
					// 성인 인증 실패.
				}
			}
			if(arg.isMembet("description")) {
				std::string description = arg["description"].asString();
				log("====>>>> description : %s", description.c_str());
			}
		}
		*/

//		if (reader.parse(json, arg) == false) {
//			log("====>>>> C2SModuleApi_PromotionInfo parse failed : (%s)", json);
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

//		Json::FastWriter writer;
//		std::string strJSON = writer.write(arg);

        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();
		log("====>>>> strJSON : %s", strJSON.c_str());
	}
	break;

	// HIVE 1.10.0
	case C2SModuleApi_SocialInfo : {
		log("====>>>> ResultDelegate : C2SModuleApi_SocialInfo");

		if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

		// 다이얼로그와 로그로 출력 (190505)
        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();
        MessageBox(strJSON.c_str(), "");	// 메시지 다이얼로그
        log("====>>>> strJSON : %s", strJSON.c_str());

		// TODO : jschoi
		// 크래시 이슈가 있어 임시 주석 처리 (190505)
////		if( reader.parse(json, arg) == false ) {
////			log( "====>>>> C2SModuleApi_SocialInfo parse failed : (%s)", json );
////			return;
////		}
//
//        if(arg.HasParseError())
//        {
//            log("====>>>> parse failed : (%s)", json);
//            return;
//        }
//
//		// Json Parsing (C2SModuleApi_SocialInfo), type = notice
////		bool notice_show = arg.get("show", false).asBool();
////		std::string info_type = arg.get("type", "").asString();
//
//        bool notice_show = arg["show"].GetBool();
//        std::string info_type = arg["type"].GetString();
//
////		Json::Value dataArg;
////		Json::Value dataArg = arg["data"];
//
//        const rapidjson::Value& dataArg = arg["data"];
//
//		if(dataArg.IsNull() || dataArg.Size() <= 0) {
//			log("### data is empty");
//
//			// show log
//			log("### show : %d\n", notice_show);
//			log("### type : %s\n", info_type.c_str());
//		}
//		else {
//			log("### data is not empty");
//
////			dataArg = arg["data"];
////			int data_errno = dataArg.get("errno", 0).asInt();
////			std::string data_error = dataArg.get("error", "").asString();
////			int data_notice_show = dataArg.get("notice_show", "").asInt();
////			std::string data_notice_title = dataArg.get("notice_title", "").asString();
////			std::string data_notice_message = dataArg.get("notice_button", "").asString();
////			int data_notice_action = dataArg.get("notice_action", "").asInt();
////			std::string data_notice_url = dataArg.get("notice_url", "").asString();
////			int data_remaining_time = dataArg.get("remaining_time", "").asInt();
//
//            int data_errno = arg["data"]["errno"].GetInt();
//            std::string data_error = arg["data"]["error"].GetString();
//            int data_notice_show = arg["data"]["notice_show"].GetInt();
//            std::string data_notice_title = arg["data"]["notice_title"].GetString();
//            std::string data_notice_message = arg["data"]["notice_button"].GetString();
//            int data_notice_action = arg["data"]["notice_action"].GetInt();
//            std::string data_notice_url = arg["data"]["notice_url"].GetString();
//            int data_remaining_time = arg["data"]["remaining_time"].GetInt();
//
//			// show log
//			log("### show : %d\n", notice_show);
//			log("### type : %s\n", info_type.c_str());
//			log("### [data] :\n");
//			log("### errno : %d\n", data_errno);
//			log("### error : %s\n", data_error.c_str());
//			log("### notice_show : %d\n", data_notice_show);
//			log("### notice_title : %s\n", data_notice_title.c_str());
//			log("### notice_message : %s\n", data_notice_message.c_str());
//			log("### notice_action : %d\n", data_notice_action);
//			log("### notice_url : %s\n", data_notice_url.c_str());
//			log("### remaining_time : %d\n", data_remaining_time);
//		}
	}
	break;

    case C2SModuleApi_SocialSetGameInfo: {
        log("====>>>> ResultDelegate : C2SModuleApi_SocialInfo");

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

        // 다이얼로그와 로그로 출력 (190505)
        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();
        MessageBox(strJSON.c_str(), "");	// 메시지 다이얼로그
        log("====>>>> strJSON : %s", strJSON.c_str());

        if (arg.HasMember("vid")) {
            std::string vid = arg["vid"].GetString();
            if(Parameter::paramInitialize.setGameInfo_emergencyMode == true) {
                log("### paramLogin.vid = %s", vid.c_str());
                Parameter::paramLogin.vid = vid;
            }
        }
        if (arg.HasMember("uid")) {
            std::string uid = arg["uid"].GetString();
            if(Parameter::paramInitialize.setGameInfo_emergencyMode == true) {
                log("### paramLogin.uid = %s", uid.c_str());
                Parameter::paramLogin.uid = uid;
            }
        }


    }
    break;

	case C2SModuleApi_TrackingSendEvent: {
		log("====>>>> ResultDelegate : C2SModuleApi_TrackingSendEvent");
		log("### trackingSendEvent api over");
	}
	break;

	case C2SModuleApi_TrackingSetEnable: {
		log("====>>>> ResultDelegate : C2SModuleApi_TrackingSetEnable");
	}
	break;

	// [Interwork]
	case C2SModuleApi_InterworkStart: {
		log("====>>>> ResultDelegate : C2SModuleApi_InterworkStart");

//		if (reader.parse(json, arg) == false) {
//			log("====>>>> C2SModuleApi_InterworkStart parse failed : (%s)", json);
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

		if (arg.HasMember("interworkArg")) {
			// 프로모션 타겟팅 시작시 들어옴.
			log(
				"====>>>> ResultDelegate : C2SModuleApi_InterworkStart interworkArg exist.");
//			std::string type = arg["interworkArg"]["type"].asString();
            std::string type = arg["interworkArg"]["type"].GetString();

			// 인터워크 메시지 출력
			std::stringstream interworkMessage;
			interworkMessage << "C2SModuleApi_InterworkStart : (" + type + ")";
			std::string interworkMessage_string = interworkMessage.str();
			MessageBox(interworkMessage_string.c_str(), "");

			if (!type.empty()) {
				log("====>>>> C2SModuleApi_InterworkStart type : %s",	type.c_str());
			}
		} else {
			//
		}
	}
	break;

	case C2SModuleApi_InterworkDelegate: {
		log("====>>>> ResultDelegate : C2SModuleApi_InterworkDelegate");

//		if (reader.parse(json, arg) == false) {
//			log("====>>>> C2SModuleApi_InterworkDelegate parse failed : (%s)", json);
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

		if (arg.HasMember("interworkArg")) {
			// 프로모션 타겟팅 종료 및 게임기능 관련  들어옴.
			log("====>>>> ResultDelegate : C2SModuleApi_InterworkDelegate interworkArg exist.");
//			std::string type = arg["interworkArg"]["type"].asString();
            std::string type = arg["interworkArg"]["type"].GetString();

			// 인터워크 메시지 출력
			std::stringstream interworkMessage;
			interworkMessage << "C2SModuleApi_InterworkDelegate : (" + type + ")";
			std::string interworkMessage_string = interworkMessage.str();
			MessageBox(interworkMessage_string.c_str(), "");

			if (!type.empty()) {
				// 타입에 맞는 처리를 게임내에서 처리함.
				log("====>>>> C2SModuleApi_InterworkDelegate type : %s", type.c_str());
			}
		} else {
			//
		}
	}
	break;

	case C2SModuleApi_HiveIAPInfo : {
		log("====>>>> ResultDelegate : C2SModuleApi_HiveIAPInfo");

//		if (reader.parse(json, arg) == false) {
//			log("====>>>> C2SModuleApi_HiveIAPInfo parse failed : (%s)", json);
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

		// parsing
		if(Parameter::paramHiveIAPInfo.type == "shopInfo") {
			log("[DealCallback::ResultDelegate] type = showInfo");

//			Json::Value products = arg["products"];
//			Json::StyledWriter writer;
//			std::string str_temp = writer.write(products);

            const rapidjson::Value& products = arg["products"];

            StringBuffer buffer;
            Writer<StringBuffer> writer(buffer);
            products.Accept(writer);

            std::string str_temp = buffer.GetString();
            log("[DealCallback::ResultDelegate] HIVEIAP INFO products : %s ", str_temp.c_str());
			log("[DealCallback::ResultDelegate] products size : %d ", arg["products"].Size());
			Parameter::paramHiveIAPInfoItemList.clear();
			for(int i = 0; i < arg["products"].Size(); i++) {
				HiveIAPInfoItem itemInfoList;
				std::stringstream ss;
//				log("[DealCallback::ResultDelegate] description1 : %s ", products[i].get("description","").asString().c_str());
//				ss.str(products[i].get("description","").asString());
                ss.str(arg["products"][i]["description"].GetString());
				itemInfoList.description = ss.str();
//				log("[DealCallback::ResultDelegate] description2 : %s ", itemInfoList.description.c_str());
//				ss.str(products[i].get("displayPrice","").asString());
                ss.str(arg["products"][i]["displayPrice"].GetString());
				itemInfoList.displayPrice = ss.str();
//				log("[DealCallback::ResultDelegate] displayPrice : %s ", itemInfoList.displayPrice.c_str());
//				ss.str(products[i].get("marketPid", "").asString());
                ss.str(arg["products"][i]["marketPid"].GetString());
				itemInfoList.marketPid = ss.str();
//				ss.str(products[i].get("pid","").asString());
                ss.str(arg["products"][i]["pid"].GetString());
				itemInfoList.pid = ss.str();
//				ss.str(products[i].get("productImageUrl","").asString());
                ss.str(arg["products"][i]["productImageUrl"].GetString());
				itemInfoList.productImageUrl = ss.str();
//				ss.str(products[i].get("title","").asString());
                ss.str(arg["products"][i]["title"].GetString());
				itemInfoList.title = ss.str();

				Parameter::paramHiveIAPInfoItemList.push_back(itemInfoList);
			}
//			for(int i = 0; i < Parameter::paramHiveIAPInfoItemList.size(); i++) {
//				HiveIAPInfoItem itemInfoList;
//				itemInfoList = Parameter::paramHiveIAPInfoItemList[i];
//				log("[DealCallback::ResultDelegate] data : %s ", itemInfoList.description.c_str());
//				log("[DealCallback::ResultDelegate] data : %s ", itemInfoList.displayPrice.c_str());
//				log("[DealCallback::ResultDelegate] data : %s ", itemInfoList.marketPid.c_str());
//				log("[DealCallback::ResultDelegate] data : %s ", itemInfoList.pid.c_str());
//				log("[DealCallback::ResultDelegate] data : %s ", itemInfoList.productImageUrl.c_str());
//				log("[DealCallback::ResultDelegate] data : %s ", itemInfoList.title.c_str());
//			}

			ModuleListScene::isDisplayHiveIAPInfoItem = true;
		}

	}
	break;

	case C2SModuleApi_HiveIAPPurchase : {
		log("====>>>> ResultDelegate : C2SModuleApi_HiveIAPPurchase");

		// 구매 성공된 아이템 처리.
		if(code == 0) {		// 에러 (0 : 정상) 이면.
			for(int i = 0; i < Parameter::paramHiveIAPInfoItemList.size(); i++) {
				HiveIAPInfoItem infoItem;
				infoItem = Parameter::paramHiveIAPInfoItemList[i];
				if( DataManager::getParamValue(TAG_APIACTION_HIVEIAP_PURCHASE, "pid") == infoItem.pid) {
					Parameter::paramHiveIAPInfoItemPurchaseList.push_back(infoItem);
					log("### purchase item get in! ###");
				}
			}
		}
		else if(code == -300) {
			// restore가 필요한 상황.
			std::string restoreMessage = "[HIVE IAP] PlayStore needs restore!";
			MessageBox(restoreMessage.c_str(), "");
		}

	}
	break;

	case C2SModuleApi_HiveIAPRestore :
	{
		log("====>>>> ResultDelegate : C2SModuleApi_HiveIAPRestore");

		// 올바른 구매 복구 상황.
		if(code == 0) {
			std::string restoreMessage = "[HIVE IAP] Restore success!";
			MessageBox(restoreMessage.c_str(), "");
		}
	}
	break;

	// HIVE 1.12.0
	case C2SModuleApi_HivePromotionShow :
	{
		log("====>>>> ResultDelegate : C2SModuleApi_HivePromotionShow");
//		if (reader.parse(json, arg) == false) {
//			log("====>>>> C2SModuleApi_HivePromotionShow parse failed : (%s)", json);
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

//		Json::FastWriter writer;
//		std::string strJSON = writer.write(arg);

        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();
		log("====>>>> strJSON : %s", strJSON.c_str());
	}
	break;

	case C2SModuleApi_HivePromotionInfo :
	{
		log("====>>>> ResultDelegate : C2SModuleApi_HivePromotionInfo");
//		if (reader.parse(json, arg) == false) {
//			log("====>>>> C2SModuleApi_HivePromotionInfo parse failed : (%s)", json);
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

		log("PromotionInfoTest (1)");
//		Json::FastWriter writer;
//		std::string strJSON = writer.write(arg);

        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();
		log("====>>>> strJSON : %s", strJSON.c_str());
//		log("PromotionInfoTest (2)");
//
//		Json::Value json_data = arg["data"];
//		log("PromotionInfoTest (3) : %s", writer.write(json_data).c_str());
//		Json::Value json_data_badgeType = json_data["badgeType"];
//		log("PromotionInfoTest (4) : %s", writer.write(json_data_badgeType).c_str());
//		log("PromotionInfoTest (5)");
	}
	break;

	case C2SModuleApi_HivePromotionGetOfferwallState :
	{
		log("====>>>> ResultDelegate : C2SModuleApi_HivePromotionGetOfferwallState");
//		if (reader.parse(json, arg) == false) {
//			log("====>>>> C2SModuleApi_HivePromotionGetOfferwallState parse failed : (%s)", json);
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

//		Json::FastWriter writer;
//		std::string strJSON = writer.write(arg);

        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();
		log("====>>>> strJSON : %s", strJSON.c_str());
	}
	break;

	case C2SModuleApi_InappCheckPromotePurchase :
	{
		log("====>>>> ResultDelegate : C2SModuleApi_InappCheckPromotePurchase");
//		if (reader.parse(json, arg) == false) {
//			log("====>>>> C2SModuleApi_InappCheckPromotePurchase parse failed : (%s)", json);
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

//		Json::FastWriter writer;
//		std::string strJSON = writer.write(arg);

        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();
		log("====>>>> strJSON : %s", strJSON.c_str());
	}
	break;

	case C2SModuleApi_HiveIAPCheckPromotePurchase :
	{
		log("====>>>> ResultDelegate : C2SModuleApi_HiveIAPCheckPromotePurchase");
//		if (reader.parse(json, arg) == false) {
//			log("====>>>> C2SModuleApi_HiveIAPCheckPromotePurchase parse failed : (%s)", json);
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

//		Json::FastWriter writer;
//		std::string strJSON = writer.write(arg);

        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();
		log("====>>>> strJSON : %s", strJSON.c_str());
	}
	break;

	case C2SModuleApi_PromotionCompanion :
	{
		log("====>>>> ResultDelegate : C2SModuleApi_PromotionCompanion");
//		if (reader.parse(json, arg) == false) {
//			log("====>>>> C2SModuleApi_PromotionCompanion parse failed : (%s)", json);
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

//		Json::FastWriter writer;
//		std::string strJSON = writer.write(arg);

        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();
		log("====>>>> strJSON : %s", strJSON.c_str());
	}
	break;

	case C2SModuleApi_PromotionPlayback :
	{
		log("====>>>> ResultDelegate : C2SModuleApi_PromotionPlayback");
//		if (reader.parse(json, arg) == false) {
//			log("====>>>> C2SModuleApi_PromotionPlayback parse failed : (%s)", json);
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

//		Json::FastWriter writer;
//		std::string strJSON = writer.write(arg);

        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();
		log("====>>>> strJSON : %s", strJSON.c_str());
	}
	break;

	case C2SModuleApi_HivePromotionPlayback :
	{
		log("====>>>> ResultDelegate : C2SModuleApi_HivePromotionPlayback");
//		if (reader.parse(json, arg) == false) {
//			log("====>>>> C2SModuleApi_HivePromotionPlayback parse failed : (%s)", json);
//			return;
//		}

        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

//		Json::FastWriter writer;
//		std::string strJSON = writer.write(arg);

        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        arg.Accept(writer);

        std::string strJSON = buffer.GetString();
		log("====>>>> strJSON : %s", strJSON.c_str());
	}
	break;

    case C2SModuleApi_InappV4ItemInfo : {
        log("====>>>> ResultDelegate : C2SModuleApi_InappV4ItemInfo");

        Show_InappV4ItemPidList(json);
    }
    break;

    case C2SModuleApi_InappV4Purchase :
    {
        log("====>>>> ResultDelegate : C2SModuleApi_InappV4Purchase");

//#ifdef __ANDROID__ // TODO: Handling for Android support only in v1.19.1 KS QA
        if(code == 0) {
            PostBox_VerifyConnection(json);
        }
        else {
            MessageBox("아이템 검증 수행 불가! 에러 로그를 확인해주세요!", "");
        }
//#endif //#ifdef __ANDROID__
    }
    break;

    case C2SModuleApi_InappV4Restore : {
        log("====>>>> ResultDelegate : C2SModuleApi_InappV4Restore");

//#ifdef __ANDROID__ // TODO: Handling for Android support only in v1.19.1 KS QA
        if(code == 0) {
            PostBox_VerifyConnection_ReceiptList(json);
        } else {
            MessageBox("아이템 검증 수행 불가! 에러 로그를 확인해주세요!", "");
        }

        if(json == NULL) {
            log("====>>>> (C2SModuleApi_InappV4Restore) (json is null)");
            return;
        }
        if(code != 0)
        {
            log("====>>>> C2SModuleApi_InappV4Restore error occured");
            return;
        }

        Document arg;
        arg.Parse(json);
        if(arg.HasParseError())
        {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

#ifdef __ANDROID__
        /**
         * 구글 구독 상품 그룹 테스트
         */
        log("Google Subscription Group Checking...");
        MessageBox("Google PlayStore 구독 상품 구독상태 갱신 완료!", "");
        std::string rType = "";
        std::string rPid = "";

        if (arg.HasMember("type")) {
            rType = arg["type"].GetString();

            if (rType == "subscription") {
                log("=== receiptList : subscription");

                if (arg.HasMember("receiptList")) {
                    const rapidjson::Value &receiptListRjson = arg["receiptList"];
                    log("=== receiptList checking...");

                    if (!receiptListRjson.IsNull()) {
                        int count = receiptListRjson.Size();
                        log("=== receiptList checking...001");

                        // 상태 초기화
                        Parameter::paramInAppV4.google_sub_sub01 = false;
                        Parameter::paramInAppV4.google_sub_sub10 = false;
                        Parameter::paramInAppV4.google_sub_sub11 = false;
                        Parameter::paramInAppV4.google_sub_sub30 = false;
                        Parameter::paramInAppV4.google_sub_sub013 = false;
                        Parameter::paramInAppV4.google_sub_sub103 = false;
                        Parameter::paramInAppV4.google_sub_sub113 = false;
                        Parameter::paramInAppV4.google_sub_sub303 = false;

                        for (int i = 0; i < count; i++) {
                            log("=== receiptList checking...002");
                            if (arg["receiptList"][i].HasMember("sku")) {
                                log("=== receiptList checking...003");
                                rPid = arg["receiptList"][i]["sku"].GetString();

                                log("=== receiptList checking...004");
                                // 아이템 체크
                                if (rPid == "sub01") {
                                    Parameter::paramInAppV4.google_sub_sub01 = true;
                                    log("=== already get sub01");
                                } else if (rPid == "sub10") {
                                    Parameter::paramInAppV4.google_sub_sub10 = true;
                                    log("=== already get sub10");
                                } else if (rPid == "sub11") {
                                    Parameter::paramInAppV4.google_sub_sub11 = true;
                                    log("=== already get sub11");
                                } else if (rPid == "sub30") {
                                    Parameter::paramInAppV4.google_sub_sub30 = true;
                                    log("=== already get sub30");
                                } else if (rPid == "sub013") {
                                    Parameter::paramInAppV4.google_sub_sub013 = true;
                                    log("=== already get sub013");
                                } else if (rPid == "sub103") {
                                    Parameter::paramInAppV4.google_sub_sub103 = true;
                                    log("=== already get sub103");
                                } else if (rPid == "sub113") {
                                    Parameter::paramInAppV4.google_sub_sub113 = true;
                                    log("=== already get sub113");
                                } else if (rPid == "sub303") {
                                    Parameter::paramInAppV4.google_sub_sub303 = true;
                                    log("=== already get sub303");
                                }
                                log("=== receiptList checking...005");
                            }
                        } // end of for
                    }
                }
            }
        }
        
#endif //#ifdef __ANDROID__
    }
    break;

	default:
		break;
	}
}

void DealCallback::Show_InappV4ItemPidList(const char* json) {
    //
    log("### Show_InappV4ItemPidList");

    Document arg;
    arg.Parse(json);

    if(arg.HasParseError()) {
        log("====>>>> parse failed : (%s)", json);
        return;
    }

    std::string dMsg = "[pid 목록]\n";
    std::string currentPid = "";

    if(arg.HasMember("productList")) {

        const rapidjson::Value& productListRjson = arg["productList"];

        if(!productListRjson.IsNull()) {
            int count = productListRjson.Size();

            for(int i = 0; i < count; i++) {

                if(arg["productList"][i].HasMember("marketPid")) {
                    currentPid = arg["productList"][i]["marketPid"].GetString();
                    if(i < count)
                        dMsg.append(currentPid+"\n");
                    else
                        dMsg.append(currentPid);
                }
            }
        }
    }
    else
        dMsg.append("pid가 없습니다.");

    MessageBox(dMsg.c_str(), "");
}

void DealCallback::PostBox_VerifyConnection(const char* json) {
    // IAP 서버에 영수증 검증 요청
    log("### PostBox_VerifyConnection");

    if(ApiAction::initPostBox == 1) {

        Document arg;
        arg.Parse(json);

        if (arg.HasParseError()) {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

        std::string hiveiapReceipt = "";
        std::string purchase_bypass_info = "";
        std::string marketPid = "";
        double price = 0;
        std::string currency = "";
        std::string item_type = "";
        bool isItemTypeConsumable = true;

        Document argVerify;
        argVerify.SetObject();
        Document::AllocatorType &verifyAllocator = argVerify.GetAllocator();

        argVerify.AddMember("type", "verify", verifyAllocator);
        argVerify.AddMember("world", "kr", verifyAllocator);
        argVerify.AddMember("vid", StringRef(Parameter::paramLogin.vid.c_str()), verifyAllocator);
        argVerify.AddMember("uid", StringRef(Parameter::paramLogin.uid.c_str()), verifyAllocator);
        argVerify.AddMember("vidSession", StringRef(Parameter::paramLogin.sessionToken.c_str()),verifyAllocator);

        if (arg.HasMember("receipt")) {
            if (arg["receipt"] == "") {
                log("### PostBoxIapV4 PostBox_VerifyConnection, invalid data(receipt)");
                return;
            }

            if (arg["receipt"].HasMember("hiveiapReceipt")) {
                hiveiapReceipt = arg["receipt"]["hiveiapReceipt"].GetString();
                argVerify.AddMember("hiveiap_receipt", StringRef(hiveiapReceipt.c_str()),
                                    verifyAllocator);
            } else {
                log("### PostBoxIapV4 PostBox_VerifyConnection, invalid data(hiveiapReceipt)");
                return;
            }
            if (arg["receipt"].HasMember("bypassInfo")) {
                purchase_bypass_info = arg["receipt"]["bypassInfo"].GetString();
                argVerify.AddMember("purchase_bypass_info", StringRef(purchase_bypass_info.c_str()),
                                    verifyAllocator);
            } else {
                log("### PostBoxIapV4 PostBox_VerifyConnection, invalid data(bypassInfo)");
                return;
            }

            if (arg["receipt"].HasMember("product")) {
                if (arg["receipt"]["product"].HasMember("marketPid")) {
                    marketPid = arg["receipt"]["product"]["marketPid"].GetString();
                    argVerify.AddMember("market_id", StringRef(marketPid.c_str()), verifyAllocator);
                } else {
                    log("### PostBoxIapV4 PostBox_VerifyConnection, invalid data(marketPid)");
                    return;
                }
                if (arg["receipt"]["product"].HasMember("price")) {
                    price = arg["receipt"]["product"]["price"].GetDouble();
                    argVerify.AddMember("price", price, verifyAllocator);
                } else {
                    log("### PostBoxIapV4 PostBox_VerifyConnection, invalid data(price)");
                    return;
                }
                if (arg["receipt"]["product"].HasMember("currency")) {
                    currency = arg["receipt"]["product"]["currency"].GetString();
                    argVerify.AddMember("currency", StringRef(currency.c_str()), verifyAllocator);
                } else {
                    log("### PostBoxIapV4 PostBox_VerifyConnection, invalid data(currency)");
                    return;
                }
                if (arg["receipt"]["product"].HasMember("productType")) {
                    item_type = arg["receipt"]["product"]["productType"].GetString();
                    argVerify.AddMember("item_type", StringRef(item_type.c_str()), verifyAllocator);
                    // TODO : KS Cocos 우편함 개선
                    if(item_type == "consumable") {
                        log("### PostBoxIapV4 PostBox_VerifyConnection isItemTypeConsumable = true");
                        isItemTypeConsumable = true;
                    } else {
                        log("### PostBoxIapV4 PostBox_VerifyConnection isItemTypeConsumable = false");
                        isItemTypeConsumable = false;
                    }
                } else {
                    log("### PostBoxIapV4 PostBox_VerifyConnection, invalid data(productType)");
                    return;
                }
            } else {
                return;
            }
        }


        if (Parameter::paramInitialize.useStaging == true) {
            argVerify.AddMember("useStaging", true, verifyAllocator);
            log("### PostBoxIapV4 PostBox_VerifyConnection, test server");
        } else {
            if (Parameter::paramInitialize.useSandbox == true) {
                argVerify.AddMember("useSandbox", true, verifyAllocator);
                log("### PostBoxIapV4 PostBox_VerifyConnection, sandbox server");
            } else {
                // do not setting
                log("### PostBoxIapV4 PostBox_VerifyConnection, live server");
            }
        }

        StringBuffer buffer;
        Writer<StringBuffer> writer(buffer);
        argVerify.Accept(writer);

        std::string strJSON = buffer.GetString();

        log("====>>>> strJSON : %s", strJSON.c_str());

#ifdef __ANDROID__
        if(Parameter::paramInitialize.usePostboxV4forAndroid == false) {
            PostBoxRequest(strJSON.c_str());
        } else {
            if(Parameter::paramInitialize.postboxV4Login == true) {
                // KS Cocos 우편함 개선
                Postbox::ItemType purchasedItemType;
                if(isItemTypeConsumable == true) {
                    purchasedItemType = Postbox::ItemType::consumable;
                } else {
                    purchasedItemType = Postbox::ItemType::subscription;
                }

                Postbox::verifyReceipt(purchase_bypass_info , purchasedItemType, [=](const Postbox::VerifyReceipt *data,Postbox::Error error)mutable{
                    if(data != nullptr) {
                        //영수증 검증 성공
                        MessageBox("[Postbox Verify 성공]", "[Postbox Verify 성공]");
                    } else {
                        MessageBox("[Postbox Verify 실패]", "[Postbox Verify 실패]");
                    }
                });
            } else {
                MessageBox("[need PostBox Login]", "");
            }
        }
#else
        if(Parameter::paramInitialize.postboxV4Login == true) {
                // KS Cocos 우편함 개선
                Postbox::ItemType purchasedItemType;
                if(isItemTypeConsumable == true) {
                    purchasedItemType = Postbox::ItemType::consumable;
                } else {
                    purchasedItemType = Postbox::ItemType::subscription;
                }

                Postbox::verifyReceipt(purchase_bypass_info , purchasedItemType, [=](const Postbox::VerifyReceipt *data,Postbox::Error error)mutable{
                    if(data != nullptr) {
                        //영수증 검증 성공
                        MessageBox("[Postbox Verify 성공]", "[Postbox Verify 성공]");
                    } else {
                        MessageBox("[Postbox Verify 실패]", "[Postbox Verify 실패]");
                    }
                });
            } else {
                MessageBox("[need PostBox Login]", "");
            }

#endif
        log("### PostBox_VerifyConnection POSTBOX end");// failed to authenticate the productid
    }
    else {
        MessageBox("[need PostBox Login]", "");
    }
}

void DealCallback::PostBox_VerifyConnection_ReceiptList(const char* json) {
    // IAP 서버에 영수증 검증 요청
    log("### PostBox_VerifyConnection_ReceiptList");

    if(ApiAction::initPostBox == 1) {

        Document arg;
        arg.Parse(json);

        if (arg.HasParseError()) {
            log("====>>>> parse failed : (%s)", json);
            return;
        }

        std::string hiveiapReceipt = "";
        std::string purchase_bypass_info = "";
        std::string marketPid = "";
        double price = 0;
        std::string currency = "";
        std::string item_type = "";
        bool isItemTypeConsumable = true;

        Document argVerify;
        argVerify.SetObject();
        Document::AllocatorType &verifyAllocator = argVerify.GetAllocator();

        if (arg.HasMember("receiptList")) {

            const rapidjson::Value &receiptListRjson = arg["receiptList"];

            if (!receiptListRjson.IsNull()) {
                int count = receiptListRjson.Size();

                for (int i = 0; i < count; i++) {

                    argVerify.AddMember("type", "verify", verifyAllocator);
                    argVerify.AddMember("world", "kr", verifyAllocator);
//                    argVerify.AddMember("vid", StringRef(Parameter::paramLogin.vid.c_str()), verifyAllocator);
//                    argVerify.AddMember("uid", StringRef(Parameter::paramLogin.uid.c_str()),verifyAllocator);
                    argVerify.AddMember("vid", StringRef(Parameter::paramLogin.vid.c_str()), verifyAllocator);
                    argVerify.AddMember("uid", StringRef(Parameter::paramLogin.uid.c_str()), verifyAllocator);
                    argVerify.AddMember("vidSession", StringRef(Parameter::paramLogin.sessionToken.c_str()), verifyAllocator);

                    if (arg["receiptList"][i].HasMember("hiveiapReceipt")) {
                        hiveiapReceipt = arg["receiptList"][i]["hiveiapReceipt"].GetString();
                        argVerify.AddMember("hiveiap_receipt", StringRef(hiveiapReceipt.c_str()),
                                            verifyAllocator);
                    }
                    if (arg["receiptList"][i].HasMember("bypassInfo")) {
                        purchase_bypass_info = arg["receiptList"][i]["bypassInfo"].GetString();
                        argVerify.AddMember("purchase_bypass_info",
                                            StringRef(purchase_bypass_info.c_str()),
                                            verifyAllocator);
                    }

                    if (arg["receiptList"][i].HasMember("product")) {
                        if (arg["receiptList"][i]["product"].HasMember("marketPid")) {
                            marketPid = arg["receiptList"][i]["product"]["marketPid"].GetString();
                            argVerify.AddMember("market_id", StringRef(marketPid.c_str()),
                                                verifyAllocator);
                        }
                        if (arg["receiptList"][i]["product"].HasMember("price")) {
                            price = arg["receiptList"][i]["product"]["price"].GetDouble();
                            argVerify.AddMember("price", price, verifyAllocator);
                        }
                        if (arg["receiptList"][i]["product"].HasMember("currency")) {
                            currency = arg["receiptList"][i]["product"]["currency"].GetString();
                            argVerify.AddMember("currency", StringRef(currency.c_str()),
                                                verifyAllocator);
                        }
                        if (arg["receiptList"][i]["product"].HasMember("productType")) {
                            item_type = arg["receiptList"][i]["product"]["productType"].GetString();
                            argVerify.AddMember("item_type", StringRef(item_type.c_str()),
                                                verifyAllocator);
                            // TODO : KS Cocos 우편함 개선
                            if(item_type == "consumable") {
                                log("### PostBoxIapV4 PostBox_VerifyConnection isItemTypeConsumable = true");
                                isItemTypeConsumable = true;
                            } else {
                                log("### PostBoxIapV4 PostBox_VerifyConnection isItemTypeConsumable = false");
                                isItemTypeConsumable = false;
                            }
                        }
                    }

                    if (Parameter::paramInitialize.useStaging == true) {
                        argVerify.AddMember("useStaging", true, verifyAllocator);
                        log("### PostBoxIapV4 PostBox_VerifyConnection_ReceiptList, test server");
                    } else {
                        if (Parameter::paramInitialize.useSandbox == true) {
                            argVerify.AddMember("useSandbox", true, verifyAllocator);
                            log("### PostBoxIapV4 PostBox_VerifyConnection_ReceiptList, sandbox server");
                        } else {
                            // do not setting
                            log("### PostBoxIapV4 PostBox_VerifyConnection_ReceiptList, live server");
                        }
                    }

                    StringBuffer buffer;
                    Writer<StringBuffer> writer(buffer);
                    argVerify.Accept(writer);

                    std::string strJSON = buffer.GetString();

                    log("====>>>> strJSON : %s", strJSON.c_str());

#ifdef __ANDROID__
                    if(Parameter::paramInitialize.usePostboxV4forAndroid == false) {
                        PostBoxRequest(strJSON.c_str());
                    } else {
                        if(Parameter::paramInitialize.postboxV4Login == true) {
                            // KS Cocos 우편함 개선
                            Postbox::ItemType purchasedItemType;
                            if(isItemTypeConsumable == true) {
                                purchasedItemType = Postbox::ItemType::consumable;
                            } else {
                                purchasedItemType = Postbox::ItemType::subscription;
                            }

                            Postbox::verifyReceipt(purchase_bypass_info , purchasedItemType, [=](const Postbox::VerifyReceipt *data,Postbox::Error error)mutable{
                                if(data != nullptr) {
                                    //영수증 검증 성공
                                    MessageBox("[Postbox Verify 성공]", "[Postbox Verify 성공]");
                                } else {
                                    MessageBox("[Postbox Verify 실패]", "[Postbox Verify 실패]");
                                }
                            });
                        } else {
                            MessageBox("[need PostBox Login]", "");
                        }
                    }
#else
                    if(Parameter::paramInitialize.postboxV4Login == true) {
                            // KS Cocos 우편함 개선
                            Postbox::ItemType purchasedItemType;
                            if(isItemTypeConsumable == true) {
                                purchasedItemType = Postbox::ItemType::consumable;
                            } else {
                                purchasedItemType = Postbox::ItemType::subscription;
                            }

                            Postbox::verifyReceipt(purchase_bypass_info , purchasedItemType, [=](const Postbox::VerifyReceipt *data,Postbox::Error error)mutable{
                                if(data != nullptr) {
                                    //영수증 검증 성공
                                    MessageBox("[Postbox Verify 성공]", "[Postbox Verify 성공]");
                                } else {
                                    MessageBox("[Postbox Verify 실패]", "[Postbox Verify 실패]");
                                }
                            });
                        } else {
                            MessageBox("[need PostBox Login]", "");
                        }
#endif
                    log("### PostBox_VerifyConnection_ReceiptList POSTBOX end");// failed to authenticate the productid
                }
            }
        }
    } else {
        MessageBox("[need PostBox Login]", "");
    }
}

std::string DealCallback::getApiName(C2SModuleApi api)
{
	switch(api) {
		case C2SModuleApi_SocialInitialize : {			// 0
			/** 소셜 초기화 */
			return "C2SModuleApi_SocialInitialize";
			break;
		}case C2SModuleApi_SocialShow : {
			/** 소셜 뷰 출력 */
			return "C2SModuleApi_SocialShow";
			break;
		}case C2SModuleApi_SnsSession : {
			/** sns 세션 정보 */
			return "C2SModuleApi_SnsSession";
			break;
		}case C2SModuleApi_SnsLogin : {
			/** sns 로그인 */
			return "C2SModuleApi_SnsLogin";
			break;
		}case C2SModuleApi_SnsLogout : {				// 4
			/** sns 로그아웃 */
			return "C2SModuleApi_SnsLogout";
			break;
		}case C2SModuleApi_SnsUnregister : {
			/** sns 탈퇴 */
			return "C2SModuleApi_SnsUnregister";
			break;
		}case C2SModuleApi_SnsUser : {
			/** sns 유저 정보 요청 */
			return "C2SModuleApi_SnsUser";
			break;
		}case C2SModuleApi_SnsFriends : {
			/** sns 친구 정보 요청 */
			return "C2SModuleApi_SnsFriends";
			break;
		}case C2SModuleApi_SnsMessage : {
			/** sns 메시지 보내기 */
			return "C2SModuleApi_SnsMessage";
			break;
		}case C2SModuleApi_SnsPost : {					// 9
			/** sns 포스팅 */
			return "C2SModuleApi_SnsPost";
			break;
		}case C2SModuleApi_SnsShow : {
			/** sns 뷰 출력 */
			return "C2SModuleApi_SnsShow";
			break;
		}case C2SModuleApi_InappPrepare : {
			/** inapp 마켓 타겟팅 */
			return "C2SModuleApi_InappPrepare";
			break;
		}case C2SModuleApi_InappRestore : {
			/** inapp 구매 복구 */
			return "C2SModuleApi_InappRestore";
			break;
		}case C2SModuleApi_InappPurchase : {
			/** inapp 구매 요청 */
			return "C2SModuleApi_InappPurchase";
			break;
		}case C2SModuleApi_InappFinish : {				// 14
			/** inapp 구매 완료 */
			return "C2SModuleApi_InappFinish";
			break;
		}case C2SModuleApi_InappDescription : {
			/** inapp 상품 정보 */
			return "C2SModuleApi_InappDescription";
			break;
		}case C2SModuleApi_PushRegister : {
			/** push 푸쉬 등록 */
			return "C2SModuleApi_PushRegister";
			break;
		}case C2SModuleApi_PushUnregister : {
			/** push 푸쉬 제거 */
			return "C2SModuleApi_PushUnregister";
			break;
		}case C2SModuleApi_PushDescription : {
			/** push 푸쉬 옵션 */
			return "C2SModuleApi_PushDescription";
			break;
		}case C2SModuleApi_PromotionShow : {
			/** promotion 뷰 출력 */
			return "C2SModuleApi_PromotionShow";
			break;
		}case C2SModuleApi_InappUpdated : {				// 20
			/** inapp purchase updated */
			return "C2SModuleApi_InappUpdated";
			break;
		}case C2SModuleApi_SnsPGSLoginProc : {
			/** PGS 로그인 */
			return "C2SModuleApi_SnsPGSLoginProc";
			break;
		}case C2SModuleApi_InappRequestBalance : {
			/** Lebi 잔액 */
			return "C2SModuleApi_InappRequestBalance";
			break;
		}case C2SModuleApi_TrackingSendEvent : {
			/** 트래킹 이벤트 보내기 */
			return "C2SModuleApi_TrackingSendEvent";
			break;
		}case C2SModuleApi_TrackingSetEnable : {		// 24
			/** 트래킹 이벤트 SetEanble **/
			return "C2SModuleApi_TrackingSetEnable";
			break;
		}case C2SModuleApi_PromotionInfo : {
			/** promotion 웹뷰 정보 출력 **/
			return "C2SModuleApi_PromotionInfo";
			break;
		}case C2SModuleApi_SnsConnect : {
			/** sns connect */
			return "C2SModuleApi_SnsConnect";
			break;
		}case C2SModuleApi_InterworkStart : {
			/** interwork 시작 */
			return "C2SModuleApi_InterworkStart";
			break;
		}case C2SModuleApi_InterworkDelegate : {
			/** interwork 끝 */
			return "C2SModuleApi_InterworkDelegate";
			break;
		}
		// HIVE 1.9.0
		case C2SModuleApi_PromotionAcquisition : {		// 29
			/** C2SModuleApi_PromotionAcquisition */
			return "C2SModuleApi_PromotionAcquisition";
			break;
		}
		// HIVE 1.10.0
		case C2SModuleApi_HiveIAPInitialize : {
			/** C2SModuleApi_HiveIAPInitialize */
			return "C2SModuleApi_HiveIAPInitialize";
			break;
		}case C2SModuleApi_HiveIAPInfo : {
			/** C2SModuleApi_HiveIAPInfo */
			return "C2SModuleApi_HiveIAPInfo";
			break;
		}case C2SModuleApi_HiveIAPShow : {
			/** C2SModuleApi_HiveIAPShow */
			return "C2SModuleApi_HiveIAPShow";
			break;
		}case C2SModuleApi_HiveIAPPurchase : {
			/** C2SModuleApi_HiveIAPPurchase */
			return "C2SModuleApi_HiveIAPPurchase";
			break;
		}case C2SModuleApi_HiveIAPRestore : {			// 34
			/** C2SModuleApi_HiveIAPRestore */
			return "C2SModuleApi_HiveIAPRestore";
			break;
		}case C2SModuleApi_EngagementProcessScheme : {
			/** C2SModuleApi_EngagementProcessScheme */
			return "C2SModuleApi_EngagementProcessScheme";
			break;
		}case C2SModuleApi_EngagementGetEvents : {
			/** C2SModuleApi_EngagementGetEvents */
			return "C2SModuleApi_EngagementGetEvents";
			break;
		}case C2SModuleApi_EngagementConsumeCoupon : {
			/** C2SModuleApi_EngagementConsumeCoupon */
			return "C2SModuleApi_EngagementConsumeCoupon";
			break;
		}case C2SModuleApi_EngagementStart : {
			/** C2SModuleApi_EngagementStart */
			return "C2SModuleApi_EngagementStart";
			break;
		}case C2SModuleApi_EngagementEnd : {			// 39
			/** C2SModuleApi_EngagementEnd */
			return "C2SModuleApi_EngagementEnd";
			break;
		}case C2SModuleApi_SocialInfo : {
			/** C2SModuleApi_SocialInfo */
			return "C2SModuleApi_SocialInfo";
			break;
		}
		case C2SModuleApi_EngagementGameScheme : {
			/** C2SModuleApi_EngagementGameScheme */
			return "C2SModuleApi_EngagementGameScheme";
			break;
		}
		// HIVE 1.12.0
		case C2SModuleApi_HivePromotionShow : {
			/** C2SModuleApi_HivePromotionShow */
			return "C2SModuleApi_HivePromotionShow";
			break;
		}
		case C2SModuleApi_HivePromotionInfo : {
			/** C2SModuleApi_HivePromotionInfo */
			return "C2SModuleApi_HivePromotionInfo";
			break;
		}
		case C2SModuleApi_HivePromotionGetOfferwallState : {		// 44
			/** C2SModuleApi_HivePromotionGetOfferwallState */
			return "C2SModuleApi_HivePromotionGetOfferwallState";
			break;
		}
		case C2SModuleApi_InappCheckPromotePurchase : {
			/** C2SModuleApi_InappCheckPromotePurchase */
			return "C2SModuleApi_InappCheckPromotePurchase";
			break;
		}
		case C2SModuleApi_HiveIAPCheckPromotePurchase : {
			/** C2SModuleApi_HiveIAPCheckPromotePurchase */
			return "C2SModuleApi_HiveIAPCheckPromotePurchase";
			break;
		}
		case C2SModuleApi_PromotionCompanion : {
			/** C2SModuleApi_PromotionCompanion */
			return "C2SModuleApi_PromotionCompanion";
			break;
		}
		case C2SModuleApi_PromotionPlayback : {
			/** Promotion Youtube Playback */
			return "C2SModuleApi_PromotionPlayback";
			break;
		}
		case C2SModuleApi_HivePromotionPlayback : {
			/** HivePromotion Youtube Playback */
			return "C2SModuleApi_HivePromotionPlayback";
			break;
		}
		case C2SModuleApi_PromotionGetBannerInfo : {
            /** Promotion getBannerInfo */
            return "C2SModuleApi_PromotionGetBannerInfo";
            break;
		}
        case C2SModuleApi_HivePromotionGetBannerInfo : {
            /** HivePromotion getBannerInfo */
            return "C2SModuleApi_HivePromotionGetBannerInfo";
            break;
        }
        // IAP V4
        case C2SModuleApi_InappV4Initialize : {
            /** InappV4 Initialize */
            return "C2SModuleApi_InappV4Initialize";
            break;
        }
        case C2SModuleApi_InappV4MarketConnect : {
            /** InappV4 MarketConnect */
            return "C2SModuleApi_InappV4MarketConnect";
            break;
        }
        case C2SModuleApi_InappV4ItemInfo : {
            /** InappV4 ItemInfo */
            return "C2SModuleApi_InappV4ItemInfo";
            break;
        }
        case C2SModuleApi_InappV4Purchase : {
            /** InappV4 Purchase */
            return "C2SModuleApi_InappV4Purchase";
            break;
        }
        case C2SModuleApi_InappV4Restore : {
            /** InappV4 Restore */
            return "C2SModuleApi_InappV4Restore";
            break;
        }
        case C2SModuleApi_InappV4TransactionFinish : {
            /** InappV4 TransactionFinish */
            return "C2SModuleApi_InappV4TransactionFinish";
            break;
        }
        case C2SModuleApi_InappV4SelectMarket : {
            /** InappV4 SelectMarket */
            return "C2SModuleApi_InappV4SelectMarket";
            break;
        }
        case C2SModuleApi_InappV4RequestBalance : {
            /** InappV4 RequestBalance */
            return "C2SModuleApi_InappV4RequestBalance";
            break;
        }
        case C2SModuleApi_InappV4ChargeBalance : {
            /** InappV4 ChargeBalance */
            return "C2SModuleApi_InappV4ChargeBalance";
            break;
        }
        case C2SModuleApi_InappV4CheckPromotePurchase : {
            /** InappV4 CheckPromotePurchase */
            return "C2SModuleApi_InappV4CheckPromotePurchase";
            break;
        }
        case C2SModuleApi_InappV4GetSelectedMarket : {
            /** InappV4 GetSelectedMarket */
            return "C2SModuleApi_InappV4GetSelectedMarket";
            break;
        }
        case C2SModuleApi_SocialSetGameInfo : {
            return "C2SModuleApi_SocialSetGameInfo";
            break;
        }
        case C2SModuleApi_SnsUserDelete : {
            return "C2SModuleApi_SnsUserDelete";
            break;
        }
        case C2SModuleApi_InappV4GetAccountUuid : {
            /** InappV4 GetAccountUuid */
            return "C2SModuleApi_InappV4GetAccountUuid";
            break;
        }
        case C2SModuleApi_InappV4ShowInAppMessages : {
            /** InappV4 ShowInAppMessages */
            return "C2SModuleApi_InappV4ShowInAppMessages";
            break;
        }
		default : {
			return "### DealCallback::getApiName error ###";
		}
	}
}
