#include "LogController.h"
#include "ModuleListScene.h"		// ModuleListScene에 로그 출력을 위함.
#include "Parameter.h"

#ifdef __ANDROID__
#include "JNISample.h"
#endif

#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
#include "platform/android/jni/JniHelper.h"
#include "jni.h"
#endif

USING_NS_CC;

std::vector<LogData> LogController::allLogDataList;
std::vector<LogData> LogController::localLogDataList;

QALogData LogController::qaLogData;

Scene* LogController::createScene() {
	auto scene = Scene::create();
	auto layer = LogController::create();
	scene->addChild(layer);
	return scene;
}

bool LogController::init()
{
	if (!Layer::init()) {
		return false;
	}

	log("### LogController : init() ###");
	return true;
}

// 현재 시간을 반환하는 함수
std::string LogController::getLogTime()
{
	struct tm *tm;
	time_t timep;

	timeval tv;
	gettimeofday(&tv, nullptr);
	timep = tv.tv_sec;

	tm = localtime(&timep);
	int year = tm->tm_year + 1900;
	int month = tm->tm_mon + 1;
	int day = tm->tm_mday;
	int hour = tm->tm_hour;
	int min = tm->tm_min;
	int second = tm->tm_sec;

	std::stringstream timeInfo;
	timeInfo << "[" << year << "/" << month << "/" << day << ", " << hour << ":" << min << ":" << second << "]";

	return timeInfo.str();
}

// 외부에서 로그를 추가시키기 위해 호출되는 함수
void LogController::addLog(std::string logData)
{
	std::string info = logData;
	LogController::addLocalLog(info);	// 로컬 로그에 추가
	LogController::addAllLog(info);		// 전체 로그에 추가
}

// 지역 로그를 추가하는 함수
void LogController::addLocalLog(std::string logData)
{
	LogData tempLogData;
	tempLogData.logTime = LogController::getLogTime();			// 로그 시간 가져오기
	tempLogData.logText = logData;								// 로그 정보 가져오기

	LogController::localLogDataList.push_back(tempLogData);
}

// 전체 로그를 추가하는 함수
void LogController::addAllLog(std::string logData)
{
	LogData tempLogData;
	tempLogData.logTime = LogController::getLogTime();			// 로그 시간 가져오기
	tempLogData.logText = logData;								// 로그 정보 가져오기

	LogController::allLogDataList.push_back(tempLogData);
}

// UI에 종합적인 로그 정보를 반환하기 위한 함수 (전체 로그)
std::string LogController::getFormattedAllLogString()
{
	std::string info = "";
	for(const LogData& logData : LogController::allLogDataList) {
		info += logData.logTime + "\n";
		info += logData.logText + "\n\n";
	}

	return info;
}

// UI에 종합적인 로그 정보를 반환하기 위한 함수 (지역 로그)
std::string LogController::getFormattedLocalLogString()
{
	std::string info = "";
	for(const LogData& logData : LogController::localLogDataList) {
		info += logData.logTime + "\n";
		info += logData.logText + "\n\n";
	}

	return info;
}

// 로그 다이얼로그를 띄우는 함수
void LogController::displayLog(std::string logString)
{
	MessageBox(logString.c_str(), "");
}

// 전체 로그를 삭제하는 함수
void LogController::clearAllLog()
{
	LogController::allLogDataList.clear();
}

// 지역 로그를 삭제하는 함수
void LogController::clearLocalLog()
{
	LogController::localLogDataList.clear();
}

void LogController::saveAllLog()
{
	std::string value = "";
	value = "mailto:jschoi@com2us.com";
	value += "?";
	value += "subject=cocos2dx_QA_SampleApp_Alllog";
	value += ("&body=" + LogController::getFormattedAllLogString());
	openURL(value.c_str());
}

void LogController::saveLocalLog()
{
	std::string value = "";
	value = "mailto:jschoi@com2us.com";
	value += "?";
	value += "subject=cocos2dx_QA_SampleApp_Locallog";
	value += ("&body=" + LogController::getFormattedLocalLogString());
	openURL(value.c_str());
}

LabelTTF* LogController::createQALog(std::string log)
{
	Size visibleSize = Director::getInstance()->getVisibleSize();
	LabelTTF* label = LabelTTF::create(log, "arial.ttf", 10, Size(visibleSize.width * 0.7, visibleSize.height * 0.1),
			TextHAlignment::CENTER, TextVAlignment::CENTER );
	label->setPosition(visibleSize.width * 0.3, visibleSize.height * 0.05);
	label->setColor(Color3B(0, 0, 0));
	label->setTag(TAG_LABEL_QALOG);

	return label;
}

std::string LogController::getQALog(int tag, bool isResult)
{
	// ModuleListScene의 define 값을 활용 --> Manager의 getCurrentParameter 활용
	std::string value = "";

	switch(tag) {
		case TAG_LABEL_QALOG_INITPARAMETER : {
			value = "초기화 정보 설정 저장!";
			break;
		}
		case TAG_BTN_MODULE_LIST_PUSH_REGISTER : {
			isResult?value = LogController::qaLogData.result_pushRegister:value = LogController::qaLogData.setting_pushRegister;
			break;
		}case TAG_BTN_MODULE_LIST_PUSH_UNREGISTER : {
			isResult?value = LogController::qaLogData.result_pushUnregister:value = LogController::qaLogData.setting_pushUnregister;
			break;
		}case TAG_BTN_MODULE_LIST_PUSH_DESCRIPTION : {
			isResult?value = LogController::qaLogData.result_pushDescription:value = LogController::qaLogData.setting_pushDescription;
			break;
		}case TAG_BTN_MODULE_LIST_SNS_USER_HIVE : {
			isResult?value = LogController::qaLogData.result_snsUser_HIVE:value = LogController::qaLogData.setting_snsUser_HIVE;
			break;
		}case TAG_BTN_MODULE_LIST_SNS_USER_FACEBOOK : {
			isResult?value = LogController::qaLogData.result_snsUser_facebook:value = LogController::qaLogData.setting_snsUser_facebook;
			break;
		}case TAG_BTN_MODULE_LIST_SNS_USER_MESSAGE : {
			isResult?value = LogController::qaLogData.result_snsUser_message:value = LogController::qaLogData.setting_snsUser_message;
			break;
		}case TAG_BTN_MODULE_LIST_SNS_USER_DELETE : {
			isResult?value = LogController::qaLogData.result_snsUser_delete:value = LogController::qaLogData.setting_snsUser_delete;
			break;
		}case TAG_BTN_MODULE_LIST_SNS_POST_HIVE : {
			// 보류된 기능 (하이브 포스팅)
			break;
		}case TAG_BTN_MODULE_LIST_SNS_POST_FACEBOOK : {
			isResult?value = LogController::qaLogData.result_snsPost_facebook:value = LogController::qaLogData.setting_snsPost_facebook;
			break;
		}case TAG_BTN_MODULE_LIST_SNS_POST_GOOGLEPLUS : {
			isResult?value = LogController::qaLogData.result_snsPost_googldplus:value = LogController::qaLogData.setting_snsPost_googldplus;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTION_NOTICE : {
			// no save param
			isResult?value = LogController::qaLogData.result_promotionShow_notice:value = LogController::qaLogData.setting_promotionShow_notice;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTION_FOCRED_NOTICE : {
			// no save param
			isResult?value = LogController::qaLogData.result_promotionShow_forcedNotice:value = LogController::qaLogData.setting_promotionShow_forcedNotice;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTION_EVENT : {
			// no save param
			isResult?value = LogController::qaLogData.result_promotionShow_event:value = LogController::qaLogData.setting_promotionShow_event;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTION_FORCED_EVENT : {
			// no save param
			isResult?value = LogController::qaLogData.result_promotionShow_forcedEvent:value = LogController::qaLogData.setting_promotionShow_forcedEvent;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTION_FULLBANNER : {
			// no save param
			isResult?value = LogController::qaLogData.result_promotionShow_fullBanner:value = LogController::qaLogData.setting_promotionShow_fullBanner;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTION_OFFERWALL : {
			// no save param
			isResult?value = LogController::qaLogData.result_promotionShow_offerwall:value = LogController::qaLogData.setting_promotionShow_offerwall;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTION_NOTICEONLY : {
			// no save param
			isResult?value = LogController::qaLogData.result_promotionShow_noticeOnly:value = LogController::qaLogData.setting_promotionShow_noticeOnly;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTION_CUSTOMWEBVIEW : {
			isResult?value = LogController::qaLogData.result_promotionShow_customWebview:value = LogController::qaLogData.setting_promotionShow_customWebview;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTION_REVIEW : {
			// no save param
			isResult?value = LogController::qaLogData.result_promotionShow_review:value = LogController::qaLogData.setting_promotionShow_review;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTION_MOREGAMES : {
			// no save param
			isResult?value = LogController::qaLogData.result_promotionShow_moreGames:value = LogController::qaLogData.setting_promotionShow_moreGames;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_NOTICE : {
			// no save param
			isResult?value = LogController::qaLogData.result_promotionInfo_badgeNotice:value = LogController::qaLogData.setting_promotionInfo_badgeNotice;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_EVENT : {
			// no save param
			isResult?value = LogController::qaLogData.result_promotionInfo_badgeEvent:value = LogController::qaLogData.setting_promotionInfo_badgeEvent;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_NOTICEONLY : {
			// no save param
			isResult?value = LogController::qaLogData.result_promotionInfo_badgeNoticeOnly:value = LogController::qaLogData.setting_promotionInfo_badgeNoticeOnly;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_CUSTOMWEBVIEW : {
			// no save param
			isResult?value = LogController::qaLogData.result_promotionInfo_badgeCustomWebview:value = LogController::qaLogData.setting_promotionInfo_badgeCustomWebview;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_FULLBANNER : {
			// no save param
			isResult?value = LogController::qaLogData.result_promotionInfo_viewFullBanner:value = LogController::qaLogData.setting_promotionInfo_viewFullBanner;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_NOTICE : {
			// no save param
			isResult?value = LogController::qaLogData.result_promotionInfo_viewNotice:value = LogController::qaLogData.setting_promotionInfo_viewNotice;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_EVENT : {
			// no save param
			isResult?value = LogController::qaLogData.result_promotionInfo_viewEvent:value = LogController::qaLogData.setting_promotionInfo_viewEvent;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_NOTICEONLY : {
			// no save param
			isResult?value = LogController::qaLogData.result_promotionInfo_viewNoticeOnly:value = LogController::qaLogData.setting_promotionInfo_viewNoticeOnly;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_CUSTOMWEBVIEW : {
			isResult?value = LogController::qaLogData.result_promotionInfo_viewCustomWebview:value = LogController::qaLogData.setting_promotionInfo_viewCustomWebview;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTIONINFO_GETOFFERWALLSTATE : {
			// no save param
			isResult?value = LogController::qaLogData.result_getOfferwallState:value = LogController::qaLogData.setting_getOfferwallState;
			break;
		}case TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_GAME : {
			// no save param
			isResult?value = LogController::qaLogData.result_snsFriends_HIVE:value = LogController::qaLogData.setting_snsFriends_HIVE;
			break;
		}case TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_OTHER : {
			// no save param
			isResult?value = LogController::qaLogData.result_snsFriends_other:value = LogController::qaLogData.setting_snsFriends_other;
			break;
		}case TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_ALL : {
			// no save param
			isResult?value = LogController::qaLogData.result_snsFriends_all:value = LogController::qaLogData.setting_snsFriends_all;
			break;
		}case TAG_BTN_MODULE_LIST_INVITEFRIENDS_FACEBOOK_RECEIVEDINVITE : {
			isResult?value = LogController::qaLogData.result_snsFriends_FB_receivedInvite:value = LogController::qaLogData.setting_snsFriends_FB_receivedInvite;
			break;
		}case TAG_BTN_MODULE_LIST_INVITEFRIENDS_FACEBOOK_MESSAGE : {
			isResult?value = LogController::qaLogData.result_snsMessage_FB_message:value = LogController::qaLogData.setting_snsMessage_FB_message;
			break;
		}case TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_MESSAGE : {
			isResult?value = LogController::qaLogData.result_snsMessage_HIVE_message:value = LogController::qaLogData.setting_snsMessage_HIVE_message;
			break;
		}case TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_SESSION : {
			// no save param
			isResult?value = LogController::qaLogData.result_snsSession_HIVE_session:value = LogController::qaLogData.setting_snsSession_HIVE_session;
			break;
		}case TAG_BTN_MODULE_LIST_INVITEFRIENDS_FACEBOOK_SESSION : {
			// no save param
			isResult?value = LogController::qaLogData.result_snsSession_FB_session:value = LogController::qaLogData.setting_snsSession_FB_session;
			break;
		}case TAG_BTN_MODULE_LIST_INAPP_LEBI_PREPARE : {
			isResult?value = LogController::qaLogData.result_inappPrepare:value = LogController::qaLogData.setting_inappPrepare;
			break;
		}case TAG_BTN_MODULE_LIST_INAPP_LEBI_REQUESTBALANCE : {
			isResult?value = LogController::qaLogData.result_inappRequestBalance:value = LogController::qaLogData.setting_inappRequestBalance;
			break;
		}case TAG_BTN_MODULE_LIST_INAPP_PLATFORM_PURCHASE : {
			isResult?value = LogController::qaLogData.result_inappPurchase:value = LogController::qaLogData.setting_inappPurchase;
			break;
		}case TAG_BTN_MODULE_LIST_INAPP_PLATFORM_RESTORE : {
			// no save param
			isResult?value = LogController::qaLogData.result_inappRestore:value = LogController::qaLogData.setting_inappRestore;
			break;
		}case TAG_BTN_MODULE_LIST_INAPP_PLATFORM_FINISH : {
			isResult?value = LogController::qaLogData.result_inappFinish:value = LogController::qaLogData.setting_inappFinish;
			break;
		}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_HIVEHOME : {
			// no save param
			isResult?value = LogController::qaLogData.result_snsShow_HIVEHome:value = LogController::qaLogData.setting_snsShow_HIVEHome;
			break;
		}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_HIVEUSER : {
			// no save param
			isResult?value = LogController::qaLogData.result_snsShow_profile:value = LogController::qaLogData.setting_snsShow_profile;
			break;
		}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_HIVECAFE : {
			// no save param
			isResult?value = LogController::qaLogData.result_snsShow_HIVECafe:value = LogController::qaLogData.setting_snsShow_HIVECafe;
			break;
		}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_EDITPROFILE : {
			// no save param
			isResult?value = LogController::qaLogData.result_snsShow_editProfile:value = LogController::qaLogData.setting_snsShow_editProfile;
			break;
		}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_FINDFRIENDS : {
			// no save param
			isResult?value = LogController::qaLogData.result_snsShow_findFriends:value = LogController::qaLogData.setting_snsShow_findFriends;
			break;
		}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_INQUIRY : {
			// no save param
			isResult?value = LogController::qaLogData.result_snsShow_inquiry:value = LogController::qaLogData.setting_snsShow_inquiry;
			break;
		}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_TERMS : {
			// no save param
			isResult?value = LogController::qaLogData.result_socialShow_terms:value = LogController::qaLogData.setting_socialShow_terms;
			break;
		}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_MESSAGE : {
			// no save param
			isResult?value = LogController::qaLogData.result_socialShow_message:value = LogController::qaLogData.setting_socialShow_message;
			break;
		}case TAG_BTN_MODULE_LIST_TRACKER_EVENT : {
			isResult?value = LogController::qaLogData.result_tracker_sendEvent:value = LogController::qaLogData.setting_tracker_sendEvent;
			break;
		}case TAG_BTN_MODULE_LIST_TRACKER_SETENABLE : {
			isResult?value = LogController::qaLogData.result_tracker_setEnable:value = LogController::qaLogData.setting_tracker_setEnable;
			break;
		}
		// HIVE 1.9.0 : Acquisition
		case TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_INFO : {
			isResult?value = LogController::qaLogData.result_promotionAcquisition_info:value = LogController::qaLogData.setting_promotionAcquisition_info;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_QRCODESAVE : {
			isResult?value = LogController::qaLogData.result_promotionAcquisition_qrcodeSave:value = LogController::qaLogData.setting_promotionAcquisition_qrcodeSave;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_QRCODESHARE : {
			isResult?value = LogController::qaLogData.result_promotionAcquisition_qrcodeShare:value = LogController::qaLogData.setting_promotionAcquisition_qrcodeShare;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_LINKSHARE : {
			isResult?value = LogController::qaLogData.result_promotionAcquisition_linkShare:value = LogController::qaLogData.setting_promotionAcquisition_linkShare;
			break;
		}case TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_HIVELINKSHARE : {
			isResult?value = LogController::qaLogData.result_promotionAcquisition_hiveLinkShare:value = LogController::qaLogData.setting_promotionAcquisition_hiveLinkShare;
			break;
		}
		// HIVE 1.10.0 : Social Show & Info
		case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_ADULTAUTH : {
			isResult?value = LogController::qaLogData.result_socialShow_adultAuth:value = LogController::qaLogData.setting_socialShow_adultAuth;
			break;
		}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_GETNOTICE : {
			isResult?value = LogController::qaLogData.result_socialShow_notice:value = LogController::qaLogData.setting_socialShow_notice;
			break;
		}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_GETNOTICEINFO : {
			isResult?value = LogController::qaLogData.result_socialInfo_notice:value = LogController::qaLogData.setting_socialInfo_notice;
			break;
		}
		// HIVE 1.10.0 : HIVE IAP
		case TAG_BTN_MODULE_LIST_HIVEIAP_INFO_ITEM : {
			isResult?value = LogController::qaLogData.result_HiveIAP_info_item:value = LogController::qaLogData.setting_HiveIAP_info_item;
			break;
		}case TAG_BTN_MODULE_LIST_HIVEIAP_INFO_BADGE : {
			isResult?value = LogController::qaLogData.result_HiveIAP_info_badge:value = LogController::qaLogData.setting_HiveIAP_info_badge;
			break;
		}case TAG_BTN_MODULE_LIST_HIVEIAP_INFO_LEBI : {
			isResult?value = LogController::qaLogData.result_HiveIAP_info_lebi:value = LogController::qaLogData.setting_HiveIAP_info_lebi;
			break;
		}case TAG_BTN_MODULE_LIST_HIVEIAP_SHOW_GOOGLELEBISELECT : {
			isResult?value = LogController::qaLogData.result_HiveIAP_show_selelct:value = LogController::qaLogData.setting_HiveIAP_show_selelct;
			break;
		}case TAG_BTN_MODULE_LIST_HIVEIAP_SHOW_LEBICHARGE : {
			isResult?value = LogController::qaLogData.result_HiveIAP_show_charge:value = LogController::qaLogData.setting_HiveIAP_show_charge;
			break;
		}case TAG_BTN_MODULE_LIST_HIVEIAP_PURCHASE : {
			isResult?value = LogController::qaLogData.result_HiveIAP_purchase:value = LogController::qaLogData.setting_HiveIAP_purchase;
			break;
		}case TAG_BTN_MODULE_LIST_HIVEIAP_RESTORE : {
			isResult?value = LogController::qaLogData.result_HiveIAP_restore:value = LogController::qaLogData.setting_HiveIAP_restore;
			break;
		}case TAG_BTN_MODULE_LIST_HIVEIAP_POSTBOX : {
			isResult?value = LogController::qaLogData.result_HiveIAP_postbox:value = LogController::qaLogData.setting_HiveIAP_postbox;
			break;
		}
		// HIVE 1.10.0 : Engagement
		case TAG_BTN_MODULE_LIST_ENGAGEMENT_GETEVENTS : {
			isResult?value = LogController::qaLogData.result_engagement_getEvents:value = LogController::qaLogData.setting_engagement_getEvents;
			break;
		}case TAG_BTN_MODULE_LIST_ENGAGEMENT_CONSUMECOUPON : {
			isResult?value = LogController::qaLogData.result_engagement_consumeCoupon:value = LogController::qaLogData.setting_engagement_consumeCoupon;
			break;
		}case TAG_BTN_MODULE_LIST_ENGAGEMENT_SETREADY : {
			isResult?value = LogController::qaLogData.result_engagement_setReady:value = LogController::qaLogData.setting_engagement_setReady;
			break;
		}default : {
			value = "## LogController::getQALog error ##";
			break;
		}
	}

	return value;
}

// ModuleListScene에서 메인 버튼 클릭에 따른 상태값 저장.
void LogController::setApiRequestState(int tag)
{
	LogController::qaLogData.result_apiRequestState = tag;
}

// ModuleListScene의 std::string 값으로 log 값을 변경시켜주는 함수.
void LogController::setApiResultState(bool isError, int errorNum)
{
	std::string value = "";

	if(isError == true) {	// error일 경우
		std::stringstream temp;
		temp << errorNum;
		value = LogController::getQALog(LogController::qaLogData.result_apiRequestState, true) + "호출 에러(" + temp.str() + ")";
	}
	else {	// 성공한 경우
		value = LogController::getQALog(LogController::qaLogData.result_apiRequestState, true) + "호출 성공!";
	}

	// QA 로그가 등록되지 않아 "" 인경우 그냥 "" 출력으로 정의 (update 1.9.0)
	if(LogController::getQALog(LogController::qaLogData.result_apiRequestState, true) == "")
		value = "";

	// ModuleListScene에 갱신.
	ModuleListScene::ui_qaLogMessage = value;
}

// 로그 이메일 전송하기.
extern "C"
{
    void openURL(const char *url) {
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
        JniMethodInfo t;
        if (JniHelper::getStaticMethodInfo(t, "org/cocos2dx/cpp/AppActivity","openURL" ,"(Ljava/lang/String;)V"))
        {
            //t.env->CallStaticVoidMethod(t.classID,t.methodID, delayTime);
            jstring StringArg1 = t.env->NewStringUTF(url);
            t.env->CallStaticVoidMethod(t.classID, t.methodID, StringArg1);
            t.env->DeleteLocalRef(StringArg1);
            t.env->DeleteLocalRef(t.classID);
        }
#endif
    }
}

// Temp Func
#ifdef __ANDROID__

#else
extern "C" void showDialogMsg(const char* message)
{
    cocos2d::MessageBox(message, "Result");
}

extern "C" void postBoxInit(const char* data)
{
    log("postBoxInit");
}

extern "C" void PostBoxRequest(const char* _name)
{
    log("PostBoxRequest 01");
}
#endif
