#include "LogScene.h"
#include "LogController.h"

#ifdef __ANDROID__
#include "JNISample.h"
#endif

USING_NS_CC;

float LogScene::ui_previousBtn_x = 0.1;		// 이전화면 돌아가기 버튼
float LogScene::ui_previousBtn_y = 0.85;
float LogScene::ui_showLogBtn_x = 0.15;		// 로그 보기 버튼
float LogScene::ui_showLogBtn_y = 0.05;
float LogScene::ui_saveLogBtn_x = 0.5;		// 로그 저장 버튼
float LogScene::ui_saveLogBtn_y = 0.05;
float LogScene::ui_clearLogBtn_x = 0.85;	// 로그 지우기 버튼
float LogScene::ui_claerLogBtn_y = 0.05;
float LogScene::ui_titleLabel_x = 0.5;		// 타이틀 라벨 위치
float LogScene::ui_titleLabel_y = 0.85;
// popup
int LogScene::popValue_logScene_logType = TAG_POP_LOGSCENE_LOGTYPE_LOCAL;
// ui간의 중복 클릭 방지
bool LogScene::ui_isOpenedWindow = false;

Scene* LogScene::createScene() {
	auto scene = Scene::create();
	auto layer = LogScene::create();
	scene->addChild(layer);
	return scene;
}

bool LogScene::init()
{
	if (!Layer::init()) {
		return false;
	}

	Size visibleSize = Director::getInstance()->getVisibleSize();
	Vec2 origin = Director::getInstance()->getVisibleOrigin();

	log("### LogScene : init() ###");

	// layer 생성
	auto layer = LayerColor::create(Color4B(255, 255, 255, 255));
	layer->setContentSize(Size(visibleSize.width, visibleSize.height));
	layer->setTag(TAG_LAYER_LOGSCENE);
	this->addChild(layer);

	// label 생성 ( 현재 모듈 리스트의 파라미터 리스트 이름 )
	log("### LogScene : create parameter list name label ###");
	layer->addChild(LogScene::createLabels(TAG_LABEL_LOGSCENE_TITLE));

	// button 생성 ( 돌아가기 버튼 )
	LogScene::update_otherButtons(TAG_BTN_PREVIOUS_SCENE);
	// button 생성 ( showLog 버튼 )
	LogScene::update_otherButtons(TAG_BTN_LOGSCENE_SHOWLOG);
	// button 생성 ( save 버튼 )
	LogScene::update_otherButtons(TAG_BTN_LOGSCENE_SAVE);
	// button 생성 ( clear 버튼 )
	LogScene::update_otherButtons(TAG_BTN_LOGSCENE_CLEAR);

	// label 생성 ( CurrentParameter에 따른 label들 표시 )
	LogScene::update_labels(TAG_LABEL_LOGSCENE_LOGTYPE);
	// Popup Button 생성
	LogScene::update_popupButtons(TAG_POP_LOGSCENE_LOGTYPE);

	return true;
}

LabelTTF* LogScene::createLabels(int tag)
{
	Size visibleSize = Director::getInstance()->getVisibleSize();
	LabelTTF* label = LabelTTF::create();
	std::string temp = "";

	switch(tag) {
		case TAG_LABEL_LOGSCENE_TITLE :{
			temp = "로그";
			break;
		}
		default : {
			log("### LogScene::createLabels(int tag) error ###");
			break;
		}
	}
	label = LabelTTF::create(temp, "arial.ttf", 30);
	label->setPosition(
			visibleSize.width * LogScene::ui_titleLabel_x,
			visibleSize.height * LogScene::ui_titleLabel_y);
	label->setColor(Color3B(0, 0, 0));
	return label;
}

void LogScene::update_otherButtons(int btnType)
{
	Size visibleSize = Director::getInstance()->getVisibleSize();
	Layer *layer = (Layer *)this->getChildByTag(TAG_LAYER_LOGSCENE);

	switch(btnType) {
		case TAG_BTN_PREVIOUS_SCENE : {
			// previous
			log("### LogScene : create other buttons(previous) ###");
			auto btn = Menu::create(
					LogScene::createButtons(TAG_BTN_PREVIOUS_SCENE),
					NULL);
			btn->setPosition(
					visibleSize.width * LogScene::ui_previousBtn_x,
					visibleSize.height * LogScene::ui_previousBtn_y);
			layer->addChild(btn);
			break;
		}
		case TAG_BTN_LOGSCENE_SHOWLOG : {
			// previous
			log("### LogScene : create other buttons(show log) ###");
			auto btn = Menu::create(
					LogScene::createButtons(TAG_BTN_LOGSCENE_SHOWLOG),
					NULL);
			btn->setPosition(
					visibleSize.width * LogScene::ui_showLogBtn_x,
					visibleSize.height * LogScene::ui_showLogBtn_y);
			layer->addChild(btn);
			break;
		}case TAG_BTN_LOGSCENE_SAVE : {
			// show_log
			log("### LogScene : create other buttons(save log) ###");
			auto btn_log = Menu::create(
					LogScene::createButtons(TAG_BTN_LOGSCENE_SAVE), NULL);
			btn_log->setPosition(
					visibleSize.width * LogScene::ui_saveLogBtn_x,
					visibleSize.height * LogScene::ui_saveLogBtn_y);
			layer->addChild(btn_log);
			break;
		}case TAG_BTN_LOGSCENE_CLEAR : {
			// close_log
			log("### LogScene : create other buttons(clear log) ###");
			auto btn = Menu::create(
					LogScene::createButtons(TAG_BTN_LOGSCENE_CLEAR),
					NULL);
			btn->setPosition(
					visibleSize.width * LogScene::ui_clearLogBtn_x,
					visibleSize.height * LogScene::ui_claerLogBtn_y);
			layer->addChild(btn);
			break;
		}
		default : {
			// error
			log("### LogScene : create other buttons(error) ###");
			break;
		}
	}
}

MenuItemImage* LogScene::createButtons(int tag)
{
	auto btn = MenuItemImage::create();

	switch(tag) {
		// previous
		case TAG_BTN_PREVIOUS_SCENE : {
			btn = MenuItemImage::create(
					"btn_previous_normal.png", "btn_previous_clicked.png",
					CC_CALLBACK_1(LogScene::buttonEvent, this));
			btn->setScale(0.3f);
			btn->setTag(tag);
			return btn;
			break;
		}case TAG_BTN_LOGSCENE_SHOWLOG : {
			btn = MenuItemImage::create(
					"btn_showLog_normal.png", "btn_showLog_clicked.png",
					CC_CALLBACK_1(LogScene::buttonEvent, this));
			btn->setScale(0.8f);
			break;
		}case TAG_BTN_LOGSCENE_SAVE : {
			btn = MenuItemImage::create(
					"btn_logSave_normal.png", "btn_logSave_clicked.png",
					CC_CALLBACK_1(LogScene::buttonEvent, this));
			btn->setScale(0.8f);
			break;
		}case TAG_BTN_LOGSCENE_CLEAR : {
			btn = MenuItemImage::create(
					"btn_logClear_normal.png", "btn_logClear_clicked.png",
					CC_CALLBACK_1(LogScene::buttonEvent, this));
			btn->setScale(0.8f);
			break;
		}
		default : {
			btn = MenuItemImage::create(
					"btn_error.png", "btn_error.png",
					CC_CALLBACK_1(LogScene::buttonEvent, this));
			break;
		}
	}
	btn->setTag(tag);
	return btn;
}

void LogScene::buttonEvent(Ref *sender)
{
	if(LogScene::ui_isOpenedWindow == true)	// ui 중복 클릭 방지
		return;

	auto btn = (MenuItem*) sender;

	switch(btn->getTag()) {
		case TAG_BTN_PREVIOUS_SCENE : {
			// 이전 scene 으로 이동
            Director::getInstance()->getScheduler()->performFunctionInCocosThread([=]()->void{
                Director::getInstance()->popScene();
            });
			break;
		}case TAG_BTN_LOGSCENE_SHOWLOG : {		// 로그 보여주기
			// show log (local or all)
			if(LogScene::popValue_logScene_logType == TAG_POP_LOGSCENE_LOGTYPE_LOCAL) {
				LogController::displayLog(LogController::getFormattedLocalLogString());
			} else if(LogScene::popValue_logScene_logType == TAG_POP_LOGSCENE_LOGTYPE_ALL) {
				LogController::displayLog(LogController::getFormattedAllLogString());
			}
			break;
		}case TAG_BTN_LOGSCENE_SAVE : {			// 로그 저장하기
			// save log
			if(LogScene::popValue_logScene_logType == TAG_POP_LOGSCENE_LOGTYPE_LOCAL) {
				LogController::saveLocalLog();
			} else if(LogScene::popValue_logScene_logType == TAG_POP_LOGSCENE_LOGTYPE_ALL) {
				LogController::saveAllLog();
			}
			break;
		}case TAG_BTN_LOGSCENE_CLEAR : {		// 로그 삭제하기
			// clear log
			if(LogScene::popValue_logScene_logType == TAG_POP_LOGSCENE_LOGTYPE_LOCAL) {
				LogController::clearLocalLog();
				std::string clearMessage = "LocalLog Clear!";	// 로그 클리어 메시지
				MessageBox(clearMessage.c_str(), "");
			} else if(LogScene::popValue_logScene_logType == TAG_POP_LOGSCENE_LOGTYPE_ALL) {
				LogController::clearAllLog();
				std::string clearMessage = "AllLog Clear!";		// 로그 클리어 메시지
				MessageBox(clearMessage.c_str(), "");
			}
			break;
		}
		default : {
			break;
		}
	}
}

void LogScene::update_labels(int tag)
{
	Size visibleSize = Director::getInstance()->getVisibleSize();
	auto labels = Menu::create();

	switch(tag) {
		case TAG_LABEL_LOGSCENE_LOGTYPE : {			// log type
			log("### LogScene::update_labels(int tag) : log type ###");
			labels = Menu::create(
					LogScene::createParamLabels(TAG_LABEL_LOGSCENE_LOGTYPE),
					NULL);
			labels->setPosition(
					visibleSize.width * 0.2,
					visibleSize.height * 0.7);
			Layer *layer = (Layer *)this->getChildByTag(TAG_LAYER_LOGSCENE);
			layer->addChild(labels);
			break;
		}
		default : {
			log("### LogScene::update_labels(int tag) error ###");
			break;
		}
	}
}

MenuItemLabel* LogScene::createParamLabels(int tag)
{
	auto label = Label::create();

	switch(tag) {
		case TAG_LABEL_LOGSCENE_LOGTYPE : {
			label = Label::createWithSystemFont("log type :", "arial.ttf", 15);			// log type
			break;
		}default : {
			log("### LogScene::createParamLabels(int tag) error ###");
			break;
		}
	}
	label->setColor(Color3B(0, 0, 0));

	auto item = MenuItemLabel::create(label, CC_CALLBACK_1(LogScene::labelEvent, this));
	item->setTag(tag);
	return item;
}

void LogScene::labelEvent(Ref *sender)
{
	auto label = (MenuItem*) sender;

	switch(label->getTag()) {
		default : {
			// no event
			break;
		}
	}
}

void LogScene::update_popupButtons(int tag)
{
	Size visibleSize = Director::getInstance()->getVisibleSize();
	auto popupBtn = Menu::create();

	switch(tag) {
		case TAG_POP_LOGSCENE_LOGTYPE : {			// log type
			log("### LogScene : create popupButtons(log type) ###");
			popupBtn = Menu::create(
					LogScene::createPopupButtons(LogScene::popValue_logScene_logType),
					NULL);
			popupBtn->setPosition(Point(visibleSize.width * 0.7,
					visibleSize.height * 0.7));
			popupBtn->setTag(TAG_POP_LOGSCENE);
			break;
		}
		default : {
			log("### LogScene::update_popupButtons(int tag) error ###");
			break;
		}
	}
	Layer *layer = (Layer *)this->getChildByTag(TAG_LAYER_LOGSCENE);
	layer->addChild(popupBtn);
}

MenuItemFont* LogScene::createPopupButtons(int tag) {
	MenuItemFont::setFontSize(15);
	switch (tag) {
		case TAG_POP_LOGSCENE_LOGTYPE_LOCAL: {
			auto button_ = MenuItemFont::create("[ local ]",
					CC_CALLBACK_1(LogScene::showPopupEvent, this));
			button_->setColor(ccc3(0, 0, 255));
			button_->setTag(TAG_POP_LOGSCENE_LOGTYPE);
			return button_;
		}case TAG_POP_LOGSCENE_LOGTYPE_ALL: {
			auto button_ = MenuItemFont::create("[ all ]",
					CC_CALLBACK_1(LogScene::showPopupEvent, this));
			button_->setColor(ccc3(0, 0, 255));
			button_->setTag(TAG_POP_LOGSCENE_LOGTYPE);
			return button_;
		}default: {
			auto button_ = MenuItemFont::create("[ error ]",
					CC_CALLBACK_1(LogScene::showPopupEvent, this));
			button_->setColor(ccc3(0, 0, 255));
			return button_;
		}
	}
}

void LogScene::showPopupEvent(Ref *sender)
{
	if(LogScene::ui_isOpenedWindow == true)	// popup 활성화 시 중복 클릭 방지
		return;

	auto mItem = (MenuItem*) sender;

	switch (mItem->getTag()) {
		case TAG_POP_LOGSCENE_LOGTYPE: {
			// (Beta) Display Pop-up
			Size visibleSize = Director::getInstance()->getVisibleSize();
			auto popup_layer = LayerColor::create(Color4B(0, 125, 125, 255));
			popup_layer->setContentSize(Size(visibleSize.width * 0.90, visibleSize.height * 0.2));
			popup_layer->setPosition(Point(visibleSize.width * 0.05, visibleSize.height * 0.05));
			popup_layer->setTag(-10000);
			this->addChild(popup_layer);

			MenuItemFont::setFontSize(15);
			auto button_1 = MenuItemFont::create("[ local ]",
					CC_CALLBACK_1(LogScene::closePopupEvent, this));
			button_1->setColor(ccc3(255, 255, 255));
			button_1->setTag(TAG_POP_LOGSCENE_LOGTYPE_LOCAL);
			auto button_2 = MenuItemFont::create("[ all ]",
					CC_CALLBACK_1(LogScene::closePopupEvent, this));
			button_2->setColor(ccc3(255, 255, 255));
			button_2->setTag(TAG_POP_LOGSCENE_LOGTYPE_ALL);

			auto Popup_1 = Menu::create(button_1, button_2, NULL);
			Popup_1->alignItemsVerticallyWithPadding(visibleSize.height * 0.015);
			Popup_1->setPosition(Point(visibleSize.width * 0.45, visibleSize.height * 0.1));
			popup_layer->addChild(Popup_1);
			LogScene::ui_isOpenedWindow = true;	// 팝업 생성 -> 중복 클릭 방지 On
			break;
		}
		default:
			break;
	}
}

void LogScene::closePopupEvent(Ref *sender)
{
	auto mItem = (MenuItem*) sender;
	Size visibleSize = Director::getInstance()->getVisibleSize();

	// remove
	Layer *layer = (Layer *) this->getChildByTag(TAG_LAYER_LOGSCENE);
	this->removeChildByTag(-10000, true);
	layer->removeChildByTag(TAG_POP_LOGSCENE, true);

	LogScene::ui_isOpenedWindow = false; // 팝업 해제 -> 중복 클릭 방지 Off
	// toss data
	switch (mItem->getTag()) {
		case TAG_POP_LOGSCENE_LOGTYPE_LOCAL : {
			LogScene::popValue_logScene_logType = TAG_POP_LOGSCENE_LOGTYPE_LOCAL;
			break;
		}case TAG_POP_LOGSCENE_LOGTYPE_ALL : {
			LogScene::popValue_logScene_logType = TAG_POP_LOGSCENE_LOGTYPE_ALL;
			break;
		}
		default :
			break;
	}
	// re-draw
	LogScene::update_popupButtons(TAG_POP_LOGSCENE_LOGTYPE);
}
