#include "LoginListScene.h"
#include "ModuleGroupListScene.h"
#include "InitListScene.h"
#include "LogScene.h"
#include "DataManager.h"
#include "LogController.h"
#include "Parameter.h"

#ifdef __ANDROID__
#include "JNISample.h"
#endif

#ifdef __ANDROID__
#include "cocos2d-x/Module_Cocos2d-x.h"
#include "C2SModule.h"
#else
#include "Module_Cocos2d-x.h"
#include "C2SModule.h"
#endif

USING_NS_CC;

float LoginListScene::ui_mainBtnScale = 0.5f;
float LoginListScene::ui_paddingSize = 0.02;
float LoginListScene::ui_mainBtns_x = 0.5;
float LoginListScene::ui_mainBtns_y = 0.5;
float LoginListScene::ui_appIdLabel_x = 0.5;
float LoginListScene::ui_appIdLabel_y = 0.85;
float LoginListScene::ui_showLogBtn_x = 0.85;
float LoginListScene::ui_showLogBtn_y = 0.05;

// added 1.10.1
bool LoginListScene::isSelectState = false;
bool LoginListScene::ui_isOpenedWindow = false;		// UI 중복 클릭 방지.

Scene* LoginListScene::createScene() {
	auto scene = Scene::create();
	auto layer = LoginListScene::create();
	scene->addChild(layer);
	return scene;
}

bool LoginListScene::init()
{
	if (!Layer::init()) {
		return false;
	}

	Size visibleSize = Director::getInstance()->getVisibleSize();
	Vec2 origin = Director::getInstance()->getVisibleOrigin();

	log("### LoginListScene : init() ###");

	// layer 생성
	auto layer = LayerColor::create(Color4B(255, 255, 255, 255));
	layer->setContentSize(Size(visibleSize.width, visibleSize.height));
	layer->setTag(TAG_LAYER_LOGIN_LIST);
	this->addChild(layer);

	// label 생성 ( 초기화시 사용한 AppID에 해당하는 App의 이름 )
	//log("### LoginListScene : create app id label ###");
	//layer->addChild(LoginListScene::createLabels(TAG_LABEL_LOGIN_LIST_APPID));

	// QA Log 생성( 서버 접속 완료 표시 )
	std::string serverValue = "";
	if(DataManager::getParamValue_bool(TAG_APIACTION_INITIALIZE, "useStaging") == true) {		// for staging
		serverValue = "Server(Staging) 접속 완료!";
	} else if(DataManager::getParamValue_bool(TAG_APIACTION_INITIALIZE, "useSandbox") == true) {
		serverValue = "Server(Sandbox) 접속 완료!";
	} else {
		serverValue = "Server(Live Server) 접속 완료!";
	}
	layer->addChild(LogController::createQALog(serverValue));	// draw

	// button 생성 ( 메인 버튼 )
	log("### LoginListScene : create main buttons ###");
	auto btns_main = Menu::create(
			LoginListScene::createButtons(TAG_BTN_LOGIN_LIST_HIVE),
			LoginListScene::createButtons(TAG_BTN_LOGIN_LIST_GUEST),
			LoginListScene::createButtons(TAG_BTN_LOGIN_LIST_GETLOGINTYPE),
			LoginListScene::createButtons(TAG_BTN_LOGIN_LIST_USERSELECT),
			LoginListScene::createButtons(TAG_BTN_LOGIN_LIST_USERSELECTCUSTOM),
			LoginListScene::createButtons(TAG_BTN_LOGIN_LIST_SETUSERINFO_VID),
			LoginListScene::createButtons(TAG_BTN_LOGIN_LIST_GOTO_INITLIST_SCENE),
			NULL);
	btns_main->alignItemsVerticallyWithPadding(visibleSize.height * LoginListScene::ui_paddingSize);
	btns_main->setPosition(
			visibleSize.width * LoginListScene::ui_mainBtns_x,
			visibleSize.height * LoginListScene::ui_mainBtns_y);
	layer->addChild(btns_main);

	// button 생성 ( 로그 활성 버튼 )
	log("### LoginListScene : create main buttons ###");
	auto btn_log = Menu::create(
			LoginListScene::createButtons(TAG_BTN_SHOW_LOG), NULL);
	log("### LoginListScene : jschoi test code 001");
	btn_log->setPosition(
			visibleSize.width * LoginListScene::ui_showLogBtn_x,
			visibleSize.height * LoginListScene::ui_showLogBtn_y);
	log("### LoginListScene : jschoi test code 002");
	layer->addChild(btn_log);

	Parameter::paramSceneChecker.sceneState = 1;

	log("### LoginListScene : jschoi test code 003");
	schedule(schedule_selector(LoginListScene::changeSceneFlag));

	log("### LoginListScene : jschoi test code 004");
    
	return true;
}

MenuItemImage* LoginListScene::createButtons(int tag)
{
	auto btn = MenuItemImage::create();

	switch(tag) {
		case TAG_BTN_LOGIN_LIST_HIVE : {
			btn = MenuItemImage::create(
					"btn_LoginListScene_Hive_normal.png", "btn_LoginListScene_Hive_clicked.png",
					CC_CALLBACK_1(LoginListScene::buttonEvent, this));
			btn->setScale(0.7f);
			//btn->setScale(LoginListScene::ui_mainBtnScale);
			break;
		}
		case TAG_BTN_LOGIN_LIST_GUEST : {
			btn = MenuItemImage::create(
					"btn_LoginListScene_Guest_normal.png", "btn_LoginListScene_Guest_clicked.png",
					CC_CALLBACK_1(LoginListScene::buttonEvent, this));
			btn->setScale(0.7f);
			//btn->setScale(LoginListScene::ui_mainBtnScale);
			break;
		}
		case TAG_BTN_LOGIN_LIST_GETLOGINTYPE : {
			btn = MenuItemImage::create(
					"btn_LoginListScene_LoginType_normal.png", "btn_LoginListScene_LoginType_clicked.png",
					CC_CALLBACK_1(LoginListScene::buttonEvent, this));
			btn->setScale(0.7f);
			//btn->setScale(LoginListScene::ui_mainBtnScale);
			break;
		}
		case TAG_BTN_LOGIN_LIST_USERSELECT : {
			btn = MenuItemImage::create(
					"btn_LoginListScene_Select_normal.png", "btn_LoginListScene_Select_clicked.png",
					CC_CALLBACK_1(LoginListScene::buttonEvent, this));
			btn->setScale(0.7f);
			//btn->setScale(LoginListScene::ui_mainBtnScale);
			break;
		}
		case TAG_BTN_LOGIN_LIST_USERSELECTCUSTOM : {
			btn = MenuItemImage::create(
					"btn_LoginListScene_CustomSelect_normal.png", "btn_LoginListScene_CustomSelect_clicked.png",
					CC_CALLBACK_1(LoginListScene::buttonEvent, this));
			//btn->setScale(LoginListScene::ui_mainBtnScale);
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_LOGIN_LIST_SETUSERINFO_VID : {		// HIVE 1.16.1
			btn = MenuItemImage::create(
					"btn_LoginListScene_SetVID_normal.png", "btn_LoginListScene_SetVID_clicked.png",
					CC_CALLBACK_1(LoginListScene::buttonEvent, this));
			//btn->setScale(LoginListScene::ui_mainBtnScale);
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_LOGIN_LIST_GOTO_INITLIST_SCENE : {		// HIVE 1.19.6
			btn = MenuItemImage::create(
					"btn_LoginListScene_GoInitScene_normal.png", "btn_LoginListScene_GoInitScene_clicked.png",
					CC_CALLBACK_1(LoginListScene::buttonEvent, this));
			//btn->setScale(LoginListScene::ui_mainBtnScale);
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_SHOW_LOG : {
			btn = MenuItemImage::create(
					"btn_displayLog_normal.png", "btn_displayLog_clicked.png",
					CC_CALLBACK_1(LoginListScene::buttonEvent, this));
			break;
		}
		default : {
			btn = MenuItemImage::create(
					"btn_error.png", "btn_error.png",
					CC_CALLBACK_1(LoginListScene::buttonEvent, this));
			break;
		}
	}
	btn->setTag(tag);
	return btn;
}

void LoginListScene::buttonEvent(Ref *sender)
{
	auto btn = (MenuItem*) sender;

	switch(btn->getTag()) {
		case TAG_BTN_LOGIN_LIST_HIVE : {
			if(LoginListScene::ui_isOpenedWindow == true)
				return;
			//DataManager::setParamValue_bool(TAG_APIACTION_LOGIN, "callState", true);
			// (1) DataManager 호출
			DataManager::setState(TAG_SCENENUM_LOGIN_LIST, "isHiveLogin", true);
			DataManager::getApiAction(TAG_APIACTION_LOGIN);
			// (2) Scene 이동
			if(LoginListScene::isSelectState == true) {
				// 셀렉트 상황일 씬을 경우 넘기지 않음.

			} else {
				// 셀렉트 상황이 아닐 경우 씬을 넘김.
//				DataManager::setParamValue_bool(TAG_APIACTION_INITIALIZE, "loginState", true);
			}

			break;
		}
		case TAG_BTN_LOGIN_LIST_GUEST : {
			if(LoginListScene::ui_isOpenedWindow == true)
							return;
            
            DataManager::getApiAction(TAG_APIACTION_GETLOGINSTATE);

			if(DataManager::getParamValue(TAG_APIACTION_LOGIN,"getLoginType") == "platform") {
				std::string message = "로그인 타입을 확인해주세요!\n(guest 로그인 불가!)";
				MessageBox(message.c_str(), "");
				return;
			}
			// (1) DataManager 호출
			DataManager::setState(TAG_SCENENUM_LOGIN_LIST, "isHiveLogin", false);
			DataManager::getApiAction(TAG_APIACTION_LOGIN);
			// (2) Scene 이동
			log("### buttonEvent : guestlogin loginState = true ###");
			DataManager::setParamValue_bool(TAG_APIACTION_INITIALIZE, "loginState", true);
			break;
		}
		case TAG_BTN_LOGIN_LIST_GETLOGINTYPE : {
			if(LoginListScene::ui_isOpenedWindow == true)
				return;

			DataManager::getApiAction(TAG_APIACTION_GETLOGINSTATE);
			// QA 로그 출력
			Layer *layer = (Layer *)this->getChildByTag(TAG_LAYER_LOGIN_LIST);
			layer->removeChildByTag(TAG_LABEL_QALOG, true);		// remove
			std::string value = "";
			value = DataManager::getParamValue(TAG_APIACTION_LOGIN, "getLoginType") + " 타입 로그인 가능!";
			layer->addChild(LogController::createQALog(value));	// re-draw
			break;
		}
		case TAG_BTN_LOGIN_LIST_USERSELECT : {
			if(LoginListScene::ui_isOpenedWindow == true)
				return;

			// 셀렉트 상황일 경우 진행.
			if(LoginListScene::isSelectState == true) {
				// 셀렉트 상황이 되었을 경우 특정 변수 true로 전환.(잠금)
				DataManager::setState(TAG_SCENENUM_LOGIN_LIST, "isSelection", true);

				DataManager::setState(TAG_SCENENUM_LOGIN_LIST, "isHiveLogin", true);
				DataManager::getApiAction(TAG_APIACTION_LOGIN_SELECT);
//				DataManager::setParamValue_bool(TAG_APIACTION_INITIALIZE, "loginState", true);
			} else {
				MessageBox("유저 정보 선택 상황이 아닙니다!", "");
			}
			LoginListScene::isSelectState = false;		// 셀렉트 로그인 bool(flag) 초기화.
			break;
		}
		case TAG_BTN_LOGIN_LIST_SETUSERINFO_VID : {
			// setGameInfo API 호출하여 VID 설정
//			DataManager::getApiAction(TAG_APIACTION_SETGAMEINFO_SETGAMEINFO);
			// 설명 팝업 노출
			MessageBox("로그인 없이 모듈 그룹화면으로 이동합니다.\n로그인 없이 SetGameInfo(게임정보설정)에서 emergencyMode=true를 설정해 호출하고,\n구매하기 기능을 테스트하기 위한 기능입니다.", "");
			// scene 이동
            Director::getInstance()->getScheduler()->performFunctionInCocosThread([=]()->void{
                auto scene = TransitionFadeTR::create(1.0, ModuleGroupListScene::createScene());
                Director::getInstance()->replaceScene(scene);
            });
			break;
		}
		case TAG_BTN_LOGIN_LIST_GOTO_INITLIST_SCENE : {
			// 설명 팝업 노출
			MessageBox("강제로 초기화 페이지로 이동합니다.", "");
			// scene 이동
            Director::getInstance()->getScheduler()->performFunctionInCocosThread([=]()->void{
                auto scene = TransitionFadeTR::create(1.0, InitListScene::createScene());
                Director::getInstance()->replaceScene(scene);
            });
			break;
		}
		case TAG_BTN_LOGIN_LIST_USERSELECTCUSTOM : {
			if(LoginListScene::ui_isOpenedWindow == true)
				return;

			// 셀렉트 상황일 경우 진행.
			if(LoginListScene::isSelectState == true) {
				// 팝업이 중복으로 열리는 것 방지. 버튼 중복 클릭 방지.
				if(LoginListScene::ui_isOpenedWindow == true)
					return;

				LoginListScene::ui_isOpenedWindow = true;

				// 팝업 생성.
				Size visibleSize = Director::getInstance()->getVisibleSize();		// 사용 x, 320 x 480 고정 중.
				auto popup_layer = LayerColor::create(Color4B(0, 125, 125, 255));
				popup_layer->setContentSize(Size(visibleSize.width * 0.90, visibleSize.height * 0.9));
				popup_layer->setPosition(Point(visibleSize.width * 0.05, visibleSize.height * 0.05));
				popup_layer->setTag(-100000);
				this->addChild(popup_layer);

				// 이전 유저정보 및 현재 유저정보 가져오기.
				std::string select1Info = "";
				select1Info = ("vid : " + DataManager::getParamValue(TAG_APIACTION_LOGIN, "currentVid"));
				select1Info += ("\nlev : 11\ngold : 1111\ncash : 11111");
				std::string select2Info = "";
				select2Info = ("vid : " + DataManager::getParamValue(TAG_APIACTION_LOGIN, "usedVid"));
				select2Info += ("\nlev : 22\ngold : 2222\ncash : 22222");

				LabelTTF* label_select1 = LabelTTF::create(select1Info, "arial.ttf", 15, Size(visibleSize.width * 0.9, 110),
						TextHAlignment::CENTER, TextVAlignment::CENTER);
				label_select1->setPosition(Point(visibleSize.width * 0.45, visibleSize.height * 0.7));
				label_select1->setColor(Color3B(0, 0, 0));
				popup_layer->addChild(label_select1);
				LabelTTF* label_select2 = LabelTTF::create(select2Info, "arial.ttf", 15, Size(visibleSize.width * 0.9, 110),
						TextHAlignment::CENTER, TextVAlignment::CENTER);
				label_select2->setPosition(Point(visibleSize.width * 0.45, visibleSize.height * 0.3));
				label_select2->setColor(Color3B(0, 0, 0));
				popup_layer->addChild(label_select2);

				// 유저 선택 버튼 생성.
				MenuItemFont::setFontSize(15);
				auto button_1 = MenuItemFont::create("[ 1 계정 선택 ]",
						CC_CALLBACK_1(LoginListScene::closeCustomSelectPopup, this));
				button_1->setColor(ccc3(255, 255, 255));
				button_1->setTag(TAG_BTN_POPUP_SELECT1);
				auto Popup_1 = Menu::create(button_1, NULL);
				Popup_1->setPosition(Point(visibleSize.width * 0.45, visibleSize.height * 0.5));
				popup_layer->addChild(Popup_1);
				auto button_2 = MenuItemFont::create("[ 2 계정 선택 ]",
						CC_CALLBACK_1(LoginListScene::closeCustomSelectPopup, this));
				button_2->setColor(ccc3(255, 255, 255));
				button_2->setTag(TAG_BTN_POPUP_SELECT2);
				auto Popup_2 = Menu::create(button_2, NULL);
				Popup_2->setPosition(Point(visibleSize.width * 0.45, visibleSize.height * 0.1));
				popup_layer->addChild(Popup_2);

			} else {
				MessageBox("유저 정보 선택 상황이 아닙니다!", "");
			}

			break;
		}
		case TAG_BTN_SHOW_LOG : {
			log("### LoginListScene : buttonEvent() showLog ###");
            Director::getInstance()->getScheduler()->performFunctionInCocosThread([=]()->void{
                Director::getInstance()->pushScene(LogScene::createScene());
            });
			break;
		}
		default : {
			log("### LoginListScene : buttonEvent() error ###");
			break;
		}
	}
}

void LoginListScene::closeCustomSelectPopup(Ref *sender)
{
	auto mItem = (MenuItem*) sender;
	// remove
	switch(mItem->getTag()) {
		case TAG_BTN_POPUP_SELECT1 : {
			this->removeChildByTag(-100000, true);

			// 선택된 유저 정보 셋팅. (선택된 vid 정보)
			DataManager::setParamValue_string(TAG_APIACTION_LOGIN, "selectedVid",
					DataManager::getParamValue(TAG_APIACTION_LOGIN, "currentVid"));
			DataManager::setParamValue_string(TAG_APIACTION_LOGIN, "vid",
					DataManager::getParamValue(TAG_APIACTION_LOGIN, "currentVid"));
			// login api 호출.
			DataManager::getApiAction(TAG_APIACTION_LOGIN_CUSTOMSELECT);

			LoginListScene::isSelectState = false;		// 셀렉트 로그인 bool(flag) 초기화.
			LoginListScene::ui_isOpenedWindow = false;	// 팝업이 닫힘 -> ui 중복 클릭 해제.
			DataManager::setParamValue_bool(TAG_APIACTION_INITIALIZE, "loginState", true);		// scene 이동 준비.
			break;
		}
		case TAG_BTN_POPUP_SELECT2 : {
			this->removeChildByTag(-100000, true);

			// 선택된 유저 정보 셋팅. (선택된 vid 정보)
			DataManager::setParamValue_string(TAG_APIACTION_LOGIN, "selectedVid",
					DataManager::getParamValue(TAG_APIACTION_LOGIN, "usedVid"));
			DataManager::setParamValue_string(TAG_APIACTION_LOGIN, "vid",
					DataManager::getParamValue(TAG_APIACTION_LOGIN, "usedVid"));
			// login api 호출.
			DataManager::getApiAction(TAG_APIACTION_LOGIN_CUSTOMSELECT);

			LoginListScene::isSelectState = false;		// 셀렉트 로그인 bool(flag) 초기화.
			LoginListScene::ui_isOpenedWindow = false;	// 팝업이 닫힘 -> ui 중복 클릭 해제.
			DataManager::setParamValue_bool(TAG_APIACTION_INITIALIZE, "loginState", true);		// scene 이동 준비.
			break;
		}
		default :
			break;
	}
}

LabelTTF* LoginListScene::createLabels(int tag)
{
	Size visibleSize = Director::getInstance()->getVisibleSize();
	LabelTTF* label = LabelTTF::create();

	switch(tag) {
		case TAG_LABEL_LOGIN_LIST_APPID : {
			std::string temp_appId = "";
			temp_appId = DataManager::getParamValue(TAG_APIACTION_INITIALIZE, "appId");
			if(temp_appId == "") {
				temp_appId = "com.com2us.misample.normal.freefull.google.global.android.common";
			}
			label = LabelTTF::create(temp_appId, "arial.ttf", 15, Size(visibleSize.width * 0.9, 110),
					TextHAlignment::CENTER, TextVAlignment::CENTER );
			label->setPosition(
					visibleSize.width * LoginListScene::ui_appIdLabel_x,
					visibleSize.height * LoginListScene::ui_appIdLabel_y);
			label->setColor(Color3B(0, 0, 0));
			//label->setScale(LoginListScene::ui_mainBtnScale);
			break;
		}
		default :
			break;
	}

	return label;
}

void LoginListScene::changeSceneFlag(float dt)
{
	if(DataManager::getParamValue_bool(TAG_APIACTION_INITIALIZE, "loginState") == true
			&& DataManager::getParamValue_bool(TAG_APIACTION_LOGIN, "callState") == true) {

		// 셀렉트 상황이있을 경우 특정 변수의 true 여부 체크(false 일 경우에만 씬 전환)
		if(DataManager::getParamValue_bool(TAG_SCENENUM_LOGIN_LIST, "isSelection") == false) {
			// scene 전환
            Director::getInstance()->getScheduler()->performFunctionInCocosThread([=]()->void{
                auto scene = TransitionFadeTR::create(1.0, ModuleGroupListScene::createScene());
                Director::getInstance()->replaceScene(scene);
            });
			// 로컬 로그 삭제
			//LogController::clearLocalLog();
			// flag값을 다시 false 로 변경
			DataManager::setParamValue_bool(TAG_APIACTION_INITIALIZE, "loginState", false);
		}
	}
}
