#include "ModuleGroupListScene.h"
#include "ModuleListScene.h"
#include "LoginListScene.h"
#include "LogScene.h"
#include "DataManager.h"
#include "LogController.h"
#include "Parameter.h"

#ifdef __ANDROID__
#include "JNISample.h"
#endif

#ifdef __ANDROID__
#include "cocos2d-x/Module_Cocos2d-x.h"
#include "C2SModule.h"
#else
#include "Module_Cocos2d-x.h"
#include "C2SModule.h"
#endif

USING_NS_CC;
USING_NS_CC_EXT;

float ModuleGroupListScene::ui_mainBtnScale = 0.5f;
float ModuleGroupListScene::ui_paddingSize = 0.02;
float ModuleGroupListScene::ui_mainBtns_x = 0.5;
float ModuleGroupListScene::ui_mainBtns_y = 0.5;
float ModuleGroupListScene::ui_infoBtn_x = 0.9;
float ModuleGroupListScene::ui_infoBtn_y = 0.95;
float ModuleGroupListScene::ui_appIdLabel_x = 0.5;
float ModuleGroupListScene::ui_appIdLabel_y = 0.85;
float ModuleGroupListScene::ui_loginServerTypeLabel_x = 0.3;
float ModuleGroupListScene::ui_loginServerTypeLabel_y = 0.95;
float ModuleGroupListScene::ui_scrollViewHeight = 0.7;
float ModuleGroupListScene::ui_scrollView_y = 0.1;
float ModuleGroupListScene::ui_scrollViewConSize = 1000.0;
float ModuleGroupListScene::ui_showLogBtn_x = 0.85;
float ModuleGroupListScene::ui_showLogBtn_y = 0.05;
float ModuleGroupListScene::ui_touchStart_y = 0.0;

Scene* ModuleGroupListScene::createScene() {
	auto scene = Scene::create();
	auto layer = ModuleGroupListScene::create();
	scene->addChild(layer);
	return scene;
}

bool ModuleGroupListScene::init()
{
	if (!Layer::init()) {
		return false;
	}

	Size visibleSize = Director::getInstance()->getVisibleSize();
	Vec2 origin = Director::getInstance()->getVisibleOrigin();

	log("### ModuleGroupListScene : init() ###");

	// layer 생성
	auto layer = LayerColor::create(Color4B(255, 255, 255, 255));
	layer->setContentSize(Size(visibleSize.width, visibleSize.height));
	layer->setTag(TAG_LAYER_MODULE_GROUP_LIST);
	this->addChild(layer);

	// label 생성 ( 로그인 및 서버 상태 )
	log("### ModuleGroupListScene : create loginType & serverType label ###");
	DataManager::getApiAction(TAG_APIACTION_GETLOGINSTATE);
	schedule(schedule_selector(ModuleGroupListScene::update_server_loginState));

	// label 생성 ( 초기화시 사용한 AppID에 해당하는 App의 이름 )
	//log("### ModuleGroupListScene : create app id label ###");
	//layer->addChild(ModuleGroupListScene::createLabels(TAG_LABEL_MODULE_GROUP_LIST_APPID));

    GetLoginType();

	// 스크롤뷰 생성 ( 메인 버튼 포함 )
	auto scrollView = ScrollView::create(
			Size(visibleSize.width, visibleSize.height * ModuleGroupListScene::ui_scrollViewHeight));
	scrollView->setDirection(ScrollView::Direction::VERTICAL);
	scrollView->setContentOffset(Vec2(0, ModuleListScene::ui_scrollViewConSize * -0.2));
	scrollView->setPosition(
			ccp(0, visibleSize.height * ModuleGroupListScene::ui_scrollView_y));
	scrollView->setColor(Color3B(255, 255, 255));
	scrollView->setContentSize(
			Size(visibleSize.width, ModuleGroupListScene::ui_scrollViewConSize));
	scrollView->setTag(TAG_SCLVIEW_MODULE_GROUP_LIST);
	scrollView->setBounceable(true);
	this->addChild(scrollView);

	// button 생성 ( 메인 버튼 ) : loginType에 따라 생성
	ModuleGroupListScene::update_mainButtons(
			DataManager::getParamValue(TAG_APIACTION_LOGIN, "getLoginType"));

	// button 생성 ( 인포 버튼 )
	log("### ModuleGroupListScene : create info button ###");
	auto btn_info = Menu::create(
			ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_INFO),
			NULL);
	btn_info->setPosition(
			visibleSize.width * ModuleGroupListScene::ui_infoBtn_x,
			visibleSize.height * ModuleGroupListScene::ui_infoBtn_y );
	layer->addChild(btn_info);

	// button 생성 ( 로그 활성 버튼 )
	log("### LoginListScene : create main buttons ###");
	auto btn_log = Menu::create(
			ModuleGroupListScene::createButtons(TAG_BTN_SHOW_LOG), NULL);
	btn_log->setPosition(
			visibleSize.width * ModuleGroupListScene::ui_showLogBtn_x,
			visibleSize.height * ModuleGroupListScene::ui_showLogBtn_y);
	layer->addChild(btn_log);

	// QA 로그 출력 ( 로그인 타입 여부 )
	// (1) 기존 로그가 있다면 삭제.
	//Layer *layer = (Layer *)this->getChildByTag(TAG_LAYER_MODULE_GROUP_LIST);
	layer->removeChildByTag(TAG_LABEL_QALOG, true);
	// (2) 새로 로그 생성.
	std::string temp_qaLog = "";
	if(DataManager::getParamValue_bool(TAG_SCENENUM_LOGIN_LIST, "isHiveLogin") == true) {
		temp_qaLog = "Success:HIVE Login";
	} else {
		temp_qaLog = "Success:Guest Login";
	}
	LabelTTF* label = LabelTTF::create(temp_qaLog, "arial.ttf", Parameter::paramSampleSetting.normalFontSize, Size(visibleSize.width * 0.7, visibleSize.height * 0.1),
			TextHAlignment::CENTER, TextVAlignment::CENTER );
	label->setPosition(visibleSize.width * 0.3, visibleSize.height * 0.05);
	label->setColor(Color3B(0, 0, 0));
	label->setTag(TAG_LABEL_QALOG);
	layer->addChild(label);

	Parameter::paramSceneChecker.sceneState = 2;

	return true;
}

void ModuleGroupListScene::update_mainButtons(std::string loginType)
{
	Size visibleSize = Director::getInstance()->getVisibleSize();
	ScrollView *scrollView = (ScrollView *)this->getChildByTag(TAG_SCLVIEW_MODULE_GROUP_LIST);
	//Layer *layer2 = (Layer *)this->getChildByTag(7777777);

	/*
	if(loginType == "guest") {
		// guest
		log("### ModuleGroupListScene : create main buttons(guest) ###");
		auto btns_main = ModuleGroupListScene::create(
				ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_PUSHMESSAGE),
				ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_USERINFO),
				ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_SNSPOSTING),
				ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_PROMOTION),
				ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_PROMOTIONINFO),
				ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_INVITEFRIENDS),
				ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_INAPP),
				ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_HIVEWEBVIEW),
				ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_TRACKER),
				NULL);
		btns_main->alignItemsVerticallyWithPadding(visibleSize.height * ModuleGroupListScene::ui_paddingSize);
		btns_main->setPosition(visibleSize.width * 0.5, ModuleGroupListScene::ui_scrollViewConSize * 0.5);
		//btns_main->setPosition(720 * 0.5, 1280 * 0.5);
		scrollView->addChild(btns_main);
		//layer2->addChild(btns_main); // test
	}
	else if(loginType == "platform") {
		// platform
		log("### ModuleGroupListScene : create main buttons(platform) ###");
		auto btns_main = ModuleGroupListScene::create(
				ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_PUSHMESSAGE),
				ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_USERINFO),
				ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_SNSPOSTING),
				ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_PROMOTION),
				ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_PROMOTIONINFO),
				ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_INVITEFRIENDS),
				ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_INAPP),
				ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_HIVEWEBVIEW),
				ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_TRACKER),
				NULL);
		btns_main->alignItemsVerticallyWithPadding(visibleSize.height * ModuleGroupListScene::ui_paddingSize);
		btns_main->setPosition(visibleSize.width * 0.5, ModuleGroupListScene::ui_scrollViewConSize * 0.5);
		//btns_main->setScale(ModuleGroupListScene::ui_mainBtnScale);
		scrollView->addChild(btns_main);
		//layer2->addChild(btns_main); // test
	}
	else {
		// error
		log("### ModuleGroupListScene : create main buttons(error) ###");
	}
	*/

	log("### ModuleGroupListScene : create main buttons(all type) ###");
	auto btns_main = ModuleMenu::create(
			ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_PUSHMESSAGE),
			ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_USERINFO),
			ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_SNSPOSTING),
			ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_PROMOTION),
			ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_PROMOTIONINFO),
			ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_PROMOTIONACQUISITION),
			ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_INVITEFRIENDS),
//			ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_INAPP),
			ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_HIVEWEBVIEW),
			ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_TRACKER),
//			ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_HIVEIAP),
			ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_ENGAGEMENT),
			ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_HIVEPROMOTION),
			ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_HIVEPROMOTIONINFO),
			ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_SETGAMEINFO),
			ModuleGroupListScene::createButtons(TAG_BTN_MODULE_GROUP_LIST_INAPPV4),
			NULL);
	btns_main->alignItemsVerticallyWithPadding(visibleSize.height * ModuleGroupListScene::ui_paddingSize);
	btns_main->setPosition(visibleSize.width * 0.5, ModuleGroupListScene::ui_scrollViewConSize * 0.5);
	//btns_main->setScale(ModuleGroupListScene::ui_mainBtnScale);
	scrollView->addChild(btns_main);
	//layer2->addChild(btns_main); // test
}

MenuItem* ModuleGroupListScene::createButtons(int tag)
{
	auto btn = MenuItemImage::create();

	switch(tag) {
		case TAG_BTN_MODULE_GROUP_LIST_PUSHMESSAGE : {
			btn = MenuItemImage::create(
					"btn_MGListScene_Push_normal.png", "btn_MGListScene_Push_clicked.png",
					CC_CALLBACK_1(ModuleGroupListScene::buttonEvent, this));
            btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_USERINFO : {
			btn = MenuItemImage::create(
					"btn_MGListScene_SnsUser_normal.png", "btn_MGListScene_SnsUser_clicked.png",
					CC_CALLBACK_1(ModuleGroupListScene::buttonEvent, this));
            btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_SNSPOSTING : {
			btn = MenuItemImage::create(
					"btn_MGListScene_SnsPost_normal.png", "btn_MGListScene_SnsPost_clicked.png",
					CC_CALLBACK_1(ModuleGroupListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_PROMOTION : {
			btn = MenuItemImage::create(
					"btn_MGListScene_Promotion_normal.png", "btn_MGListScene_Promotion_clicked.png",
					CC_CALLBACK_1(ModuleGroupListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_PROMOTIONINFO : {
			btn = MenuItemImage::create(
					"btn_MGListScene_PromoInfo_normal.png", "btn_MGListScene_PromoInfo_clicked.png",
					CC_CALLBACK_1(ModuleGroupListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_PROMOTIONACQUISITION : {	// HIVE 1.9.0
			btn = MenuItemImage::create(
					"btn_MGListScene_PromoAcquisition_normal.png", "btn_MGListScene_PromoAcquisition_clicked.png",
					CC_CALLBACK_1(ModuleGroupListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_INVITEFRIENDS : {
			btn = MenuItemImage::create(
					"btn_MGListScene_InviteFriends_normal.png", "btn_MGListScene_InviteFriends_clicked.png",
					CC_CALLBACK_1(ModuleGroupListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_INAPP : {
			btn = MenuItemImage::create(
					"btn_MGListScene_Inapp_normal.png", "btn_MGListScene_Inapp_clicked.png",
					CC_CALLBACK_1(ModuleGroupListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_HIVEWEBVIEW : {
			btn = MenuItemImage::create(
					"btn_MGListScene_WebView_normal.png", "btn_MGListScene_WebView_clicked.png",
					CC_CALLBACK_1(ModuleGroupListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_TRACKER : {
			btn = MenuItemImage::create(
					"btn_MGListScene_Tracker_normal.png", "btn_MGListScene_Tracker_clicked.png",
					CC_CALLBACK_1(ModuleGroupListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_HIVEIAP : {				// HIVE 1.10.0
			btn = MenuItemImage::create(
					"btn_MGListScene_HiveIAP_normal.png", "btn_MGListScene_HiveIAP_clicked.png",
					CC_CALLBACK_1(ModuleGroupListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_ENGAGEMENT : {			// HIVE 1.10.0
			btn = MenuItemImage::create(
					"btn_MGListScene_Engagement_normal.png", "btn_MGListScene_Engagement_clicked.png",
					CC_CALLBACK_1(ModuleGroupListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_HIVEPROMOTION : {		// HIVE 1.12.0
			btn = MenuItemImage::create(
					"btn_MGListScene_HivePromotion_normal.png", "btn_MGListScene_HivePromotion_clicked.png",
					CC_CALLBACK_1(ModuleGroupListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_HIVEPROMOTIONINFO : {	// HIVE 1.12.0
			btn = MenuItemImage::create(
					"btn_MGListScene_HivePromotionInfo_normal.png", "btn_MGListScene_HivePromotionInfo_clicked.png",
					CC_CALLBACK_1(ModuleGroupListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_SETGAMEINFO : {	// HIVE 1.12.0
			btn = MenuItemImage::create(
					"btn_MGListScene_SetGameInfo_normal.png", "btn_MGListScene_SetGameInfo_clicked.png",
					CC_CALLBACK_1(ModuleGroupListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_INAPPV4 : {	// HIVE 1.18.4
			btn = MenuItemImage::create(
					"btn_MGListScene_InAppV4_normal.png", "btn_MGListScene_InAppV4_clicked.png",
					CC_CALLBACK_1(ModuleGroupListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_INFO : {
			btn = MenuItemImage::create(
					"btn_infomation_normal.png", "btn_infomation_clicked.png",
					CC_CALLBACK_1(ModuleGroupListScene::buttonEvent, this));
			btn->setScale(0.3f);
			break;
		}
		case TAG_BTN_SHOW_LOG : {
			btn = MenuItemImage::create(
					"btn_displayLog_normal.png", "btn_displayLog_clicked.png",
					CC_CALLBACK_1(ModuleGroupListScene::buttonEvent, this));
			btn->setScale(0.8f);
			break;
		}
		default : {
			btn = MenuItemImage::create(
					"btn_error.png", "btn_error.png",
					CC_CALLBACK_1(ModuleGroupListScene::buttonEvent, this));
			break;
		}
	}
	btn->setTag(tag);
	return btn;
}

void ModuleGroupListScene::buttonEvent(Ref *sender)
{
	auto btn = (MenuItem*) sender;

	if(btn->getTag() == TAG_BTN_SHOW_LOG) {
		// show log scene
        Director::getInstance()->getScheduler()->performFunctionInCocosThread([=]()->void{
            Director::getInstance()->pushScene(LogScene::createScene());
        });

	} else {
		if(ModuleGroupListScene::allowClickAction() == true
				|| btn->getTag() == TAG_BTN_MODULE_GROUP_LIST_INFO) {	// 스크롤뷰 안에서만 클릭되었을 때
			DataManager::setCurrentModule(btn->getTag());
			DataManager::setCurrentParameter(btn->getTag());

			if(btn->getTag() != TAG_BTN_MODULE_GROUP_LIST_INFO)	// 다른 기능 이동 시 로컬 로그 삭제
				LogController::clearLocalLog();

			log("### ModuleGroupListScene to ModuleListScene ###");
            Director::getInstance()->getScheduler()->performFunctionInCocosThread([=]()->void{
                auto scene = TransitionFadeTR::create(1.0, ModuleListScene::createScene());
                Director::getInstance()->replaceScene(scene);
            });
		}
	}
}

LabelTTF* ModuleGroupListScene::createLabels(int tag)
{
	Size visibleSize = Director::getInstance()->getVisibleSize();
	LabelTTF* label = LabelTTF::create();

	switch(tag) {
		case TAG_LABEL_MODULE_GROUP_LIST_APPID : {
			std::string temp_appId = "";
			temp_appId = DataManager::getParamValue(TAG_APIACTION_INITIALIZE, "appId");
			if(temp_appId == "") {
				temp_appId = "com.com2us.misample.normal.freefull.google.global.android.common";
			}
			label = LabelTTF::create(temp_appId, "arial.ttf", Parameter::paramSampleSetting.normalFontSize, Size(visibleSize.width * 0.9, 110),
					TextHAlignment::CENTER, TextVAlignment::CENTER );
			label->setPosition(
					visibleSize.width * ModuleGroupListScene::ui_appIdLabel_x,
					visibleSize.height * ModuleGroupListScene::ui_appIdLabel_y);
			label->setColor(Color3B(0, 0, 0));
			break;
		}
		case TAG_LABEL_MODULE_GROUP_LIST_LOGINSERVERTYPE : {
			std::string temp_loginType = DataManager::getParamValue(TAG_APIACTION_LOGIN, "getLoginType");
			std::string temp_serverType = DataManager::getParamValue(TAG_APIACTION_INITIALIZE, "useSandbox");
			std::string temp = "Login Type : " + temp_loginType;

			// (1) Live Server 혹은 Sandbox로 설정
			std::string temp_serverType_select = "";
			if(temp_serverType == "0") {
				temp_serverType_select = "\nServer Type : Live Server";
			} else if (temp_serverType == "1") {
				temp_serverType_select = "\nServer Type : Sandbox";
			}
			// (2) Staging일 경우 설정 (staging이 true일 경우 우선 변경)			// for staging
			if(DataManager::getParamValue_bool(TAG_APIACTION_INITIALIZE, "useStaging") == true) {
				temp_serverType_select = "\nServer Type : Staging";
			}
			temp = temp + temp_serverType_select;	// label 종합하기

			label = LabelTTF::create(temp, "arial.ttf", Parameter::paramSampleSetting.normalFontSize);
			label->setPosition(
					visibleSize.width * ModuleGroupListScene::ui_loginServerTypeLabel_x,
					visibleSize.height * ModuleGroupListScene::ui_loginServerTypeLabel_y);
			label->setColor(Color3B(0, 0, 0));
			label->setTag(TAG_LABEL_MODULE_GROUP_LIST_LOGINSERVERTYPE);
			break;
		}
		default :
			break;
	}

	return label;
}

bool ModuleGroupListScene::allowClickAction()
{
	Size visibleSize = Director::getInstance()->getVisibleSize();

//	log("### allowClickAction : location y : (%f)", m_touchesStart.y);

	if(ModuleGroupListScene::ui_touchStart_y > (visibleSize.height * 0.1)
			&& ModuleGroupListScene::ui_touchStart_y < (visibleSize.height * 0.8))
		return true;	// 스크롤뷰 안에는 클릭했을 때
	else
		return false;	// 스크롤뷰 외부를 클릭했을 떼
}

void ModuleGroupListScene::update_server_loginState(float dt)
{
	Layer *layer = (Layer *)this->getChildByTag(TAG_LAYER_MODULE_GROUP_LIST);
	// remove
	layer->removeChildByTag(TAG_LABEL_MODULE_GROUP_LIST_LOGINSERVERTYPE, true);
	// re-draw
	layer->addChild(ModuleGroupListScene::createLabels(TAG_LABEL_MODULE_GROUP_LIST_LOGINSERVERTYPE));
}

// ********** Touch Event (between ScrollView and MenuItemImage) ********** ********** ********** **********
bool ModuleMenu::initWithArray(const Vector<MenuItem*>& arrayOfItems) {
	if (Layer::init()) {
		_enabled = true;
		Size sizeVisible = Director::getInstance()->getWinSize();

		this->ignoreAnchorPointForPosition(true);
		setAnchorPoint(Vec2(0.5f, 0.5f));
		this->setContentSize(sizeVisible);
		setPosition(Vec2(sizeVisible.width / 2, sizeVisible.height / 2));

		int count = 0;
		for (auto& item : arrayOfItems) {
			this->addChild(item, count);
			count++;
		}
		_selectedItem = nullptr;
		_state = Menu::State::WAITING;
		setCascadeColorEnabled(true);
		setCascadeOpacityEnabled(true);

		auto touchListener = EventListenerTouchOneByOne::create();
		touchListener->setSwallowTouches(false);
		touchListener->onTouchBegan = CC_CALLBACK_2(ModuleGroupListScene::onTouchBegan, this);
		touchListener->onTouchMoved = CC_CALLBACK_2(ModuleGroupListScene::onTouchMoved, this);
		touchListener->onTouchEnded = CC_CALLBACK_2(ModuleGroupListScene::onTouchEnded, this);
		touchListener->onTouchCancelled = CC_CALLBACK_2(ModuleGroupListScene::onTouchCancelled, this);
		_eventDispatcher->addEventListenerWithSceneGraphPriority(touchListener, this);

		return true;
	}
	return false;
}

ModuleMenu* ModuleMenu::createWithArray(const Vector<MenuItem*>& arrayOfItems) {
	auto ret = new ModuleMenu();
	if (ret && ret->initWithArray(arrayOfItems)) {
		ret->autorelease();
	} else {
		CC_SAFE_DELETE(ret);
	}
	return ret;
}

ModuleMenu* ModuleMenu::createWithItem(MenuItem* item) {
	return ModuleMenu::create(item, nullptr);
}

ModuleMenu* ModuleMenu::createWithItems(MenuItem *item, va_list args) {
	Vector<MenuItem*> items;
	if (item) {
		items.pushBack(item);
		MenuItem *i = va_arg(args, MenuItem*);
		while (i) {
			items.pushBack(i);
			i = va_arg(args, MenuItem*);
		}
	}
	return ModuleMenu::createWithArray(items);
}

ModuleMenu* ModuleMenu::create(MenuItem* item, ...) {
	va_list args;
	va_start(args, item);

    ModuleMenu *ret = ModuleMenu::createWithItems(item, args);
	va_end(args);

	return ret;
}

bool ModuleMenu::onTouchBegan(Touch* touch, Event* event) {
	m_touchesStart = touch->getStartLocation();
	ModuleGroupListScene::ui_touchStart_y = m_touchesStart.y;
	//log("### get location(start) x : (%f), y : (%f)", m_touchesStart.x, m_touchesStart.y);
	return Menu::onTouchBegan(touch, event);
}

void ModuleMenu::onTouchMoved(Touch* touch, Event* event) {
	Menu::onTouchMoved(touch, event);
}

void ModuleMenu::onTouchEnded(Touch* touch, Event *event) {
	// move은 가로 or 세로로 얼마만큼 Move되었나를 체크하여 그 이상이면 버튼 동작을 해지 합니다.
	// 값이 낮을수록 민감하게 반응하지만 너무 낮으면 버튼 터치가 힘들 수 있습니다.
	float move = 5.0f;

	m_touchesEnd = touch->getLocation();
	Vec2 difference = m_touchesEnd - m_touchesStart;
	//log("### get location(end) x : (%f), y : (%f)", m_touchesEnd.x, m_touchesEnd.y);

	if (difference.x > move || difference.y > move) {
		Menu::onTouchCancelled(touch, event);
	} else if (difference.x < -move || difference.y < -move) {
        Layer::onTouchCancelled(touch, event);
	} else {
		Menu::onTouchEnded(touch, event);
	}
}
// ********** ********** ********** ********** ********** ********** ********** ********** **********
