#include "ModuleListScene.h"
#include "ModuleGroupListScene.h"
#include "ParameterListScene.h"
#include "LoginListScene.h"
#include "LogScene.h"
#include "DataManager.h"
#include "LogController.h"
#include "Parameter.h"

#ifdef __ANDROID__
#include "JNISample.h"
#endif

#include <stdio.h>

#ifdef __ANDROID__
#include "cocos2d-x/Module_Cocos2d-x.h"
#include "C2SModule.h"
#else
#include "Module_Cocos2d-x.h"
#include "C2SModule.h"
#endif

USING_NS_CC;
USING_NS_CC_EXT;

float ModuleListScene::ui_paddingSize = 0.02;
float ModuleListScene::ui_mainBtns_x = 0.4;
	//float ModuleListScene::ui_mainBtns_y = 0.5;
float ModuleListScene::ui_settingBtns_x = 0.75;
	//float ModuleListScene::ui_settingBtns_y = 0.9;
float ModuleListScene::ui_previousBtn_x = 0.1;
float ModuleListScene::ui_previousBtn_y = 0.85;
float ModuleListScene::ui_moduleListNameLabel_x = 0.5;
float ModuleListScene::ui_moduleListNameLabel_y = 0.85;
float ModuleListScene::ui_loginServerTypeLabel_x = 0.3;
float ModuleListScene::ui_loginServerTypeLabel_y = 0.95;
float ModuleListScene::ui_scrollViewHeight = 0.7;
float ModuleListScene::ui_scrollView_y = 0.1;
float ModuleListScene::ui_scrollViewConSize = 1000.0;
float ModuleListScene::ui_showLogBtn_x = 0.85;
float ModuleListScene::ui_showLogBtn_y = 0.05;
float ModuleListScene::ui_logoutBtn_x = 0.5;
float ModuleListScene::ui_logoutBtn_y = 0.05;
float ModuleListScene::ui_showInfoBtn_x = 0.15;
float ModuleListScene::ui_showInfoBtn_y = 0.05;
float ModuleListScene::ui_touchStart_y = 0.0;
float ModuleListScene::ui_HiveIAPInfoItemImage_x = 0.45;		// HIVE 1.10.0
float ModuleListScene::ui_HiveIAPInfoItemImage_y = 0.15;		// HIVE 1.10.0
float ModuleListScene::ui_HiveIAPInfoItemImage_margin_y = 0.3;	// HIVE 1.10.0
float ModuleListScene::ui_paramLabel_y = 0.75;
float ModuleListScene::ui_paddingSize_y = 0.06;
float ModuleListScene::ui_paramLabel_x = 0.3;

std::string ModuleListScene::ui_qaLogMessage = "";
std::string ModuleListScene::ui_infoText = "";

bool ModuleListScene::isDisplayedQRcode = false;				// HIVE 1.9.0
bool ModuleListScene::isDisplayHiveIAPInfoItem = false;			// HIVE 1.10.0
bool ModuleListScene::isDisplayHiveIAPPurchaseItem = false; 	// HIVE 1.10.0
bool ModuleListScene::ui_isOpenedWindow = false;				// UI 중복 클릭 방지.

int ModuleListScene::imageCntHiveIAPInfoItem = 0;

// static variable ( textfield )
bool ModuleListScene::isFocusedTextField = false;
int ModuleListScene::focusedTextFieldState = 0;
CCRect ModuleListScene::tfArea_info_vid_sesseionKey;


using namespace cocos2d;

Scene* ModuleListScene::createScene() {
	auto scene = Scene::create();
	auto layer = ModuleListScene::create();
	scene->addChild(layer);

	return scene;
}

bool ModuleListScene::init()
{
	if (!Layer::init()) {
		return false;
	}

	Size visibleSize = Director::getInstance()->getVisibleSize();
	Vec2 origin = Director::getInstance()->getVisibleOrigin();

	log("### ModuleListScene : init() ###");

	// layer 생성
	auto layer = LayerColor::create(Color4B(255, 255, 255, 255));
	layer->setContentSize(Size(visibleSize.width, visibleSize.height));
	layer->setTag(TAG_LAYER_MODULE_LIST);
	this->addChild(layer);

	////// UI 그리기 시작 //////

	// label 생성 ( 로그인 및 서버 상태 )
	log("### ModuleListScene : create loginType & serverType label ###");
	schedule(schedule_selector(ModuleListScene::update_server_loginState));

	// label 생성 ( 현재 모듈 리스트의 이름 )
	log("### ModuleListScene : create module list name label ###");
	layer->addChild(ModuleListScene::createLabels(DataManager::getCurrentModule()));

	// button 생성 ( 돌아가기 버튼 )
	ModuleListScene::update_otherButtons(TAG_BTN_PREVIOUS_SCENE);
	// button 생성 ( 로그 버튼 )
	ModuleListScene::update_otherButtons(TAG_BTN_SHOW_LOG);
	// button 생성 ( 로그아웃 버튼 )
	ModuleListScene::update_otherButtons(TAG_BTN_MODULE_LIST_LOGOUT);
	// button 생성 ( 회원탈퇴 로그아웃 버튼 ), v1.19.5.2, v1.20.0+
	ModuleListScene::update_otherButtons(TAG_BTN_MODULE_LIST_LOGOUT_USEREXIT);
	// button 생성 ( 인포보기 버튼 )
	ModuleListScene::update_otherButtons(TAG_BTN_MODULE_LIST_SHOWINFO);

	// 스크롤뷰 생성 ( 메인 버튼을 포함하는 )
	auto scrollView = ScrollView::create(
			Size(visibleSize.width, visibleSize.height * ModuleListScene::ui_scrollViewHeight));
	scrollView->setDirection(ScrollView::Direction::VERTICAL);
	scrollView->setContentOffset(Vec2(0, ModuleListScene::ui_scrollViewConSize * -0.2));
	scrollView->setPosition(
			ccp(0, visibleSize.height * ModuleListScene::ui_scrollView_y));
	scrollView->setColor(Color3B(255, 255, 255));
	scrollView->setContentSize(
			Size(visibleSize.width, ModuleListScene::ui_scrollViewConSize));
	scrollView->setTag(TAG_SCLVIEW_MODULE_LIST);
	scrollView->setBounceable(true);
	this->addChild(scrollView);

	// button 생성 ( 메인 버튼 ) : loginType에 따라 생성
	ModuleListScene::update_mainButtons(DataManager::getCurrentModule());

	// button 생성 ( setting 버튼 ) : loginType에 따라 생성
	ModuleListScene::update_settingButtons(DataManager::getCurrentModule());

	// label 생성 (훌라이 관련 vid, sessionKey)
	ModuleListScene::update_labels(DataManager::getCurrentModule());

	// textfield 생성 (Info scene 안의 vid 및 sesseionKey 정보)
	ModuleListScene::update_textFields(DataManager::getCurrentModule());

	// QALOG 변경을 감지하여 계속 갱신.
	schedule(schedule_selector(ModuleListScene::update_QALOG));

	// info 정보 변경을 감지하여 계속 갱신.
	schedule(schedule_selector(ModuleListScene::update_info));
	// info 화면에서 info 다이얼로그 생성
	//ModuleListScene::displayInfo(DataManager::getCurrentModule());

	// (HIVE 1.9.0) QRCode 팝업 생성.
	schedule(schedule_selector(ModuleListScene::displayQRcodePopup));

	// HIVE 1.10.0 : HiveIAP Info Item 팝업 생성.
	schedule(schedule_selector(ModuleListScene::displayHiveIAPInfoItemPopup));

	// HIVE 1.10.0 : HiveIAP Purchase Item 팝업 생성.
	schedule(schedule_selector(ModuleListScene::displayHiveIAPPurchaseItemPopup));

	this->setKeypadEnabled(true);

	Parameter::paramSceneChecker.sceneState = 2;

	return true;
}

MenuItemImage* ModuleListScene::createButtons(int tag)
{
	auto btn = MenuItemImage::create();

	switch(tag) {
		// previous
		case TAG_BTN_PREVIOUS_SCENE : {
			ModuleListScene::ui_qaLogMessage = "";	// QALog 메시지 초기화.

			btn = MenuItemImage::create(
					"btn_previous_normal.png", "btn_previous_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.3f);
			btn->setTag(tag);
			return btn;
			break;
		}
		// push
		case TAG_BTN_MODULE_LIST_PUSH_REGISTER : {
			btn = MenuItemImage::create(
					"btn_MListScene_PushReg_normal.png", "btn_MListScene_PushReg_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PUSH_UNREGISTER : {
			btn = MenuItemImage::create(
					"btn_MListScene_PushUnreg_normal.png", "btn_MListScene_PushUnreg_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PUSH_DESCRIPTION : {
			btn = MenuItemImage::create(
					"btn_MListScene_PushDes_normal.png", "btn_MListScene_PushDes_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PUSH_REGISTER_LIST : {
			btn = MenuItemImage::create(
					"btn_MListScene_PushRegList_normal.png", "btn_MListScene_PushRegList_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PUSH_UNREGISTER_LIST : {
			btn = MenuItemImage::create(
					"btn_MListScene_PushUnregList_normal.png", "btn_MListScene_PushUnregList_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
        case TAG_BTN_MODULE_LIST_PUSH_REQUEST_PERMISSION : {
            btn = MenuItemImage::create(
                    "btn_MListScene_PushRequestPermission_normal.png", "btn_MListScene_PushRequestPermission_clicked.png",
                    CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
            btn->setScale(0.7f);
            break;
        }
		// sns_user
		case TAG_BTN_MODULE_LIST_SNS_USER_HIVE : {
			btn = MenuItemImage::create(
					"btn_MListScene_SnsUserHive_normal.png", "btn_MListScene_SnsUserHive_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_SNS_USER_FACEBOOK : {
			btn = MenuItemImage::create(
					"btn_MListScene_SnsUserFB_normal.png", "btn_MListScene_SnsUserFB_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_SNS_USER_MESSAGE : {
			btn = MenuItemImage::create(
					"btn_MListScene_SnsUserMessage_normal.png", "btn_MListScene_SnsUserMessage_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_SNS_USER_DELETE : {    			// Hive 1.20.1
			btn = MenuItemImage::create(
					"btn_MListScene_SnsUserDelete_normal.png", "btn_MListScene_SnsUserDelete_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		// sns_post
		case TAG_BTN_MODULE_LIST_SNS_POST_HIVE : {
			btn = MenuItemImage::create(
					"btn_MListScene_SnsPostHive_normal.png", "btn_MListScene_SnsPostHive_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_SNS_POST_FACEBOOK : {
			btn = MenuItemImage::create(
					"btn_MListScene_SnsPostFB_normal.png", "btn_MListScene_SnsPostFB_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_SNS_POST_GOOGLEPLUS : {
			btn = MenuItemImage::create(
					"btn_MListScene_SnsPostGP_normal.png", "btn_MListScene_SnsPostGP_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		// promotion
		case TAG_BTN_MODULE_LIST_PROMOTION_NOTICE : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoNotice_normal.png", "btn_MListScene_PromoNotice_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTION_FOCRED_NOTICE : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoForcedNotice_normal.png", "btn_MListScene_PromoForcedNotice_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTION_EVENT : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoEvent_normal.png", "btn_MListScene_PromoEvent_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTION_FORCED_EVENT : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoForcedEvent_normal.png", "btn_MListScene_PromoForcedEvent_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTION_FULLBANNER : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoFullBanner_normal.png", "btn_MListScene_PromoFullBanner_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTION_FULLBANNER_IMAGE_BANNER : {
            btn = MenuItemImage::create(
                    "btn_MListScene_PromoFullBanner_legacy_normal.png", "btn_MListScene_PromoFullBanner_legacy_clicked.png",
                    CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
            btn->setScale(0.7f);
            break;
        }
		case TAG_BTN_MODULE_LIST_PROMOTION_OFFERWALL : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoOfferwall_normal.png", "btn_MListScene_PromoOfferwall_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTION_NOTICEONLY : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoNoticeOnly_normal.png", "btn_MListScene_PromoNoticeOnly_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTION_CUSTOMWEBVIEW : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoCustom_normal.png", "btn_MListScene_PromoCustom_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTION_REVIEW : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoReview_normal.png", "btn_MListScene_PromoReview_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTION_MOREGAMES : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoMoreGames_normal.png", "btn_MListScene_PromoMoreGames_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTION_REVIEW_EX : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoReviewEx_normal.png", "btn_MListScene_PromoReviewEx_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		// promotion_info
		case TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_NOTICE : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoBadgeNotice_normal.png", "btn_MListScene_PromoBadgeNotice_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_EVENT : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoBadgeEvent_normal.png", "btn_MListScene_PromoBadgeEvent_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_NOTICEONLY : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoBadgeNoticeOnly_normal.png", "btn_MListScene_PromoBadgeNoticeOnly_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_CUSTOMWEBVIEW : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoBadgeCustom_normal.png", "btn_MListScene_PromoBadgeCustom_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_FULLBANNER : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoViewFullBanner_normal.png", "btn_MListScene_PromoViewFullBanner_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_NOTICE : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoViewNotice_normal.png", "btn_MListScene_PromoViewNotice_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_EVENT : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoViewEvent_normal.png", "btn_MListScene_PromoViewEvent_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_NOTICEONLY : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoViewNoticeOnly_normal.png", "btn_MListScene_PromoViewNoticeOnly_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_CUSTOMWEBVIEW : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoViewCustom_normal.png", "btn_MListScene_PromoViewCustom_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTIONINFO_GETOFFERWALLSTATE : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoOfferwallState_normal.png", "btn_MListScene_PromoOfferwallState_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		// promotion_acquisition (add (HIVE 1.9.0))
		case TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_INFO : {			// (1)
			btn = MenuItemImage::create(
					"btn_MListScene_PromoAcquisition_UserAcquisition_normal.png", "btn_MListScene_PromoAcquisition_UserAcquisition_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_QRCODESAVE : {	// (2)
			btn = MenuItemImage::create(
					"btn_MListScene_PromoAcquisition_QrcodeSave_normal.png", "btn_MListScene_PromoAcquisition_QrcodeSave_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_QRCODESHARE : {	// (3)
			btn = MenuItemImage::create(
					"btn_MListScene_PromoAcquisition_QrcodeShare_normal.png", "btn_MListScene_PromoAcquisition_QrcodeShare_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_LINKSHARE : {		// (4)
			btn = MenuItemImage::create(
					"btn_MListScene_PromoAcquisition_LinkShare_normal.png", "btn_MListScene_PromoAcquisition_LinkShare_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_HIVELINKSHARE : {		// (5)
			btn = MenuItemImage::create(
                    "btn_MListScene_Promoacquisition_HiveLinkShareMessage_normal.png", "btn_MListScene_Promoacquisition_HiveLinkShareMessage_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_HIVELINKSHAREPLUSMESSAGE : {
			btn = MenuItemImage::create(
                    "btn_MListScene_Promoacquisition_HiveLinkShare_normal.png", "btn_MListScene_Promoacquisition_HiveLinkShare_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_SENDERINFO : {
			btn = MenuItemImage::create(
					"btn_MListScene_Promoacquisition_SenderInfo_normal.png", "btn_MListScene_Promoacquisition_SenderInfo_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		// invite_friends
		case TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_GAME : {
			btn = MenuItemImage::create(
					"btn_MListScene_InviteHiveGame_normal.png", "btn_MListScene_InviteHiveGame_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_OTHER : {
			btn = MenuItemImage::create(
					"btn_MListScene_InviteHiveOther_normal.png", "btn_MListScene_InviteHiveOther_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_ALL : {
			btn = MenuItemImage::create(
					"btn_MListScene_InviteHiveAll_normal.png", "btn_MListScene_InviteHiveAll_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_GETINVITATIONS : {			// getInvitations
			btn = MenuItemImage::create(
					"btn_MListScene_InviteHiveAll_normal.png", "btn_MListScene_InviteHiveAll_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_INVITEFRIENDS_FACEBOOK_RECEIVEDINVITE : {
			btn = MenuItemImage::create(
					"btn_MListScene_InviteFBReceived_normal.png", "btn_MListScene_InviteFBReceived_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_INVITEFRIENDS_FACEBOOK_MESSAGE : {
			btn = MenuItemImage::create(
					"btn_MListScene_InviteFBMessage_normal.png", "btn_MListScene_InviteFBMessage_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_MESSAGE : {
			btn = MenuItemImage::create(
					"btn_MListScene_InviteHiveMessage_normal.png", "btn_MListScene_InviteHiveMessage_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_SESSION : {
			btn = MenuItemImage::create(
					"btn_MListScene_InviteHiveSession_normal.png", "btn_MListScene_InviteHiveSession_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_INVITEFRIENDS_FACEBOOK_SESSION : {
			btn = MenuItemImage::create(
					"btn_MListScene_InviteFBSession_normal.png", "btn_MListScene_InviteFBSession_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}case TAG_BTN_MODULE_LIST_INVITEFRIENDS_FACEBOOK_FRIENDSLIST : {
			btn = MenuItemImage::create(
					"btn_MListScene_SnsFriendsFBFriends_normal.png", "btn_MListScene_SnsFriendsFBFriends_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		// inapp
		case TAG_BTN_MODULE_LIST_INAPP_LEBI_PREPARE : {
			if(DataManager::isWeChatTestSampleMode)
				btn = MenuItemImage::create(
						"btn_MListScene_InappLebiPrepare_normal_en.png", "btn_MListScene_InappLebiPrepare_clicked_en.png",
						CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			else
				btn = MenuItemImage::create(
						"btn_MListScene_InappLebiPrepare_normal.png", "btn_MListScene_InappLebiPrepare_clicked.png",
						CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_INAPP_LEBI_REQUESTBALANCE : {
			if(DataManager::isWeChatTestSampleMode)
				btn = MenuItemImage::create(
						"btn_MListScene_InappLebiRequest_normal_en.png", "btn_MListScene_InappLebiRequest_clicked_en.png",
						CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			else
				btn = MenuItemImage::create(
						"btn_MListScene_InappLebiRequest_normal.png", "btn_MListScene_InappLebiRequest_clicked.png",
						CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_INAPP_PLATFORM_PURCHASE : {
			if(DataManager::isWeChatTestSampleMode)
				btn = MenuItemImage::create(
						"btn_MListScene_InappPfPurchase_normal_en.png", "btn_MListScene_InappPfPurchase_clicked_en.png",
						CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			else
				btn = MenuItemImage::create(
						"btn_MListScene_InappPfPurchase_normal.png", "btn_MListScene_InappPfPurchase_clicked.png",
						CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_INAPP_PLATFORM_RESTORE : {
			if(DataManager::isWeChatTestSampleMode)
				btn = MenuItemImage::create(
						"btn_MListScene_InappPfRestore_normal_en.png", "btn_MListScene_InappPfRestore_clicked_en.png",
						CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			else
				btn = MenuItemImage::create(
						"btn_MListScene_InappPfRestore_normal.png", "btn_MListScene_InappPfRestore_clicked.png",
						CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_INAPP_PLATFORM_FINISH : {
			if(DataManager::isWeChatTestSampleMode)
				btn = MenuItemImage::create(
						"btn_MListScene_InappPfFinish_normal_en.png", "btn_MListScene_InappPfFinish_clicked_en.png",
						CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			else
				btn = MenuItemImage::create(
						"btn_MListScene_InappPfFinish_normal.png", "btn_MListScene_InappPfFinish_clicked.png",
						CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		// HIVE WebView
		case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_HIVEHOME : {
			btn = MenuItemImage::create(
					"btn_MListScene_HiveWebViewHome_normal.png", "btn_MListScene_HiveWebViewHome_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_HIVEUSER : {
			btn = MenuItemImage::create(
					"btn_MListScene_HiveWebViewUser_normal.png", "btn_MListScene_HiveWebViewUser_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_HIVECAFE : {
			btn = MenuItemImage::create(
					"btn_MListScene_HiveWebViewCafe_normal.png", "btn_MListScene_HiveWebViewCafe_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_EDITPROFILE : {
			btn = MenuItemImage::create(
					"btn_MListScene_HiveWebViewEditProfile_normal.png", "btn_MListScene_HiveWebViewEditProfile_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_FINDFRIENDS : {
			btn = MenuItemImage::create(
					"btn_MListScene_HiveWebViewFindFriends_normal.png", "btn_MListScene_HiveWebViewFindFriends_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_INQUIRY : {
			btn = MenuItemImage::create(
					"btn_MListScene_HiveWebViewInquiry_normal.png", "btn_MListScene_HiveWebViewInquiry_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_CHATBOT : {			// HIVE 1.18.1
            btn = MenuItemImage::create(
                    "btn_MListScene_HiveWebViewChatbot_normal.png", "btn_MListScene_HiveWebViewChatbot_clicked.png",
                    CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
            btn->setScale(0.7f);
            break;
        }
		case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_TERMS : {
			btn = MenuItemImage::create(
					"btn_MListScene_HiveWebViewTerms_normal.png", "btn_MListScene_HiveWebViewTerms_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_MESSAGE : {
			btn = MenuItemImage::create(
					"btn_MListScene_HiveWebViewMessage_normal.png", "btn_MListScene_HiveWebViewMessage_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_ADULTAUTH : {			// HIVE 1.10.0
			btn = MenuItemImage::create(
					"btn_MListScene_HiveWebview_SocialShow_adultAuth_normal.png", "btn_MListScene_HiveWebview_SocialShow_adultAuth_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_GETNOTICE : {			// HIVE 1.10.0
			btn = MenuItemImage::create(
					"btn_MListScene_HiveWebview_SocialShow_notice_normal.png", "btn_MListScene_HiveWebview_SocialShow_notice_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_GETNOTICEINFO : {		// HIVE 1.10.0
			btn = MenuItemImage::create(
					"btn_MListScene_HiveWebview_SocialInfo_notice_normal.png", "btn_MListScene_HiveWebview_SocialInfo_notice_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_GETPERMISSIONINFO : {	// HIVE 1.17.0
			btn = MenuItemImage::create(
					"btn_MListScene_SocialInfo_PermissionData_normal2.png", "btn_MListScene_SocialInfo_PermissionData_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;

		}
		case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_REQUESTPERMISSIONLIST : {	// HIVE 1.17.2
			btn = MenuItemImage::create(
					"btn_MListScene_HiveWebview_SocialShow_requestPermissionList_normal.png", "btn_MListScene_HiveWebview_SocialShow_requestPermissionList_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;

		}
		// tracker
		case TAG_BTN_MODULE_LIST_TRACKER_EVENT : {
			btn = MenuItemImage::create(
					"btn_MListScene_TrackerEvent_normal.png", "btn_MListScene_TrackerEvent_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_TRACKER_SETENABLE : {
			btn = MenuItemImage::create(
					"btn_MListScene_TrackerSetEnable_normal.png", "btn_MListScene_TrackerSetEnable_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		// HIVE IAP (HIVE 1.10.0)
		case TAG_BTN_MODULE_LIST_HIVEIAP_INFO_ITEM : {
			btn = MenuItemImage::create(
					"btn_MListScene_HiveIAP_Info_Item_normal.png", "btn_MListScene_HiveIAP_Info_Item_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEIAP_INFO_BADGE : {
			btn = MenuItemImage::create(
					"btn_MListScene_HiveIAP_Info_Badge_normal.png", "btn_MListScene_HiveIAP_Info_Badge_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEIAP_INFO_LEBI : {
			btn = MenuItemImage::create(
					"btn_MListScene_HiveIAP_Info_Lebi_normal.png", "btn_MListScene_HiveIAP_Info_Lebi_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEIAP_SHOW_GOOGLELEBISELECT : {
			btn = MenuItemImage::create(
					"btn_MListScene_HiveIAP_Show_GoogleLebi_normal.png", "btn_MListScene_HiveIAP_Show_GoogleLebi_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEIAP_SHOW_LEBICHARGE : {
			btn = MenuItemImage::create(
					"btn_MListScene_HiveIAP_Show_LebiCharge_normal.png", "btn_MListScene_HiveIAP_Show_LebiCharge_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEIAP_PURCHASE : {
			btn = MenuItemImage::create(
					"btn_MListScene_HiveIAP_Purchase_normal.png", "btn_MListScene_HiveIAP_Purchase_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEIAP_RESTORE : {
			btn = MenuItemImage::create(
					"btn_MListScene_HiveIAP_Restore_normal.png", "btn_MListScene_HiveIAP_Restore_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEIAP_POSTBOX : {
			btn = MenuItemImage::create(
					"btn_MListScene_HiveIAP_Postbox_normal.png", "btn_MListScene_HiveIAP_Postbox_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		// ENGAGEMENT (HIVE 1.10.0)
		case TAG_BTN_MODULE_LIST_ENGAGEMENT_GETEVENTS : {
			btn = MenuItemImage::create(
					"btn_MListScene_Engagement_GetEvents_normal.png", "btn_MListScene_Engagement_GetEvents_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_ENGAGEMENT_CONSUMECOUPON : {
			btn = MenuItemImage::create(
					"btn_MListScene_Engagement_ConsumeCoupon_normal.png", "btn_MListScene_Engagement_ConsumeCoupon_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_ENGAGEMENT_SETREADY : {	// HIVE 1.11.0
			btn = MenuItemImage::create(
					"btn_MListScene_Engagement_SetReady_normal.png", "btn_MListScene_Engagement_SetReady_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		// HIVE Promotion (HIVE 1.12.0)
		case TAG_BTN_MODULE_LIST_HIVEPROMOTION_NEWS : {
			btn = MenuItemImage::create(
					"btn_MListScene_HivePromotionShow_news_normal.png", "btn_MListScene_HivePromotionShow_news_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEPROMOTION_FULLBANNER : {
			btn = MenuItemImage::create(
					"btn_MListScene_HivePromotionShow_banner_normal.png", "btn_MListScene_HivePromotionShow_banner_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEPROMOTION_NOTICE : {
			btn = MenuItemImage::create(
					"btn_MListScene_HivePromotionShow_notice_normal.png", "btn_MListScene_HivePromotionShow_notice_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEPROMOTION_CUSTOMVIEW : {
			btn = MenuItemImage::create(
					"btn_MListScene_HivePromotionShow_customView_normal.png", "btn_MListScene_HivePromotionShow_customView_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEPROMOTION_CUSTOMBOARD : {
			btn = MenuItemImage::create(
					"btn_MListScene_HivePromotionShow_customBoard_normal.png", "btn_MListScene_HivePromotionShow_customBoard_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEPROMOTION_SPOT : {
			btn = MenuItemImage::create(
					"btn_MListScene_HivePromotionShow_spot_normal.png", "btn_MListScene_HivePromotionShow_spot_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEPROMOTION_DIRECT : {
			btn = MenuItemImage::create(
					"btn_MListScene_HivePromotionShow_direct_normal.png", "btn_MListScene_HivePromotionShow_direct_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEPROMOTION_OFFERWALL : {
			btn = MenuItemImage::create(
					"btn_MListScene_HivePromotionShow_offerwall_normal.png", "btn_MListScene_HivePromotionShow_offerwall_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEPROMOTION_REVIEW : {
			btn = MenuItemImage::create(
					"btn_MListScene_HivePromotionShow_review_normal.png", "btn_MListScene_HivePromotionShow_review_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEPROMOTION_EXIT : {
			btn = MenuItemImage::create(
					"btn_MListScene_HivePromotionShow_exit_normal.png", "btn_MListScene_HivePromotionShow_exit_clicked.png.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEPROMOTION_REVIEW_EX : {		// HIVE 1.16.0
			btn = MenuItemImage::create(
					"btn_MListScene_HivePromotionShow_reviewEx_normal.png", "btn_MListScene_HivePromotionShow_reviewEx_clicked.png.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		// HIVE PROMOTION INFO (HIVE 1.12.0)
		case TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_BADGEINFO : {
			btn = MenuItemImage::create(
					"btn_MListScene_HivePromotionInfo_badgeInfo_normal.png", "btn_MListScene_HivePromotionInfo_badgeInfo_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_VIEWINFO : {
			btn = MenuItemImage::create(
					"btn_MListScene_HivePromotionInfo_customView_normal.png", "btn_MListScene_HivePromotionInfo_customView_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_BOARDINFO : {
			btn = MenuItemImage::create(
					"btn_MListScene_HivePromotionInfo_customBoard_normal.png", "btn_MListScene_HivePromotionInfo_customBoard_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}case TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_SPOTINFO : {
			btn = MenuItemImage::create(
					"btn_MListScene_HivePromotionInfo_spot_normal.png", "btn_MListScene_HivePromotionInfo_spot_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_DIRECTINFO : {
			btn = MenuItemImage::create(
					"btn_MListScene_HivePromotionInfo_direct_normal.png", "btn_MListScene_HivePromotionInfo_direct_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_SETADDITIONALINFO : {
			btn = MenuItemImage::create(
					"btn_MListScene_HivePromotionInfo_additionalInfo_normal.png", "btn_MListScene_HivePromotionInfo_additionalInfo_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_OFFERWALLSTATE : {
			btn = MenuItemImage::create(
					"btn_MListScene_HivePromotionInfo_offerwallState_normal.png", "btn_MListScene_HivePromotionInfo_offerwallState_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_UPDATEPROMOTIONDATA : {
			btn = MenuItemImage::create(
					"btn_MListScene_HivePromotionInfo_UpdatePromotionData_normal.png", "btn_MListScene_HivePromotionInfo_UpdatePromotionData_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_SETGAMEINFO_SETGAMEINFO : {		// HIVE 1.12.0 (setGameInfo)
			btn = MenuItemImage::create(
					"btn_MListScene_SetGameInfo_setGameInfo_normal.png", "btn_MListScene_SetGameInfo_setGameInfo_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_SETGAMEINFO_GETHIVECOUNTRYCODE : {
			btn = MenuItemImage::create(
					"btn_MListScene_SetGameInfo_getHiveCountryCode_normal.png", "btn_MListScene_SetGameInfo_getHiveCountryCode_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_SETGAMEINFO_GETCHANNEL : {
			btn = MenuItemImage::create(
					"btn_MListScene_SetGameInfo_getChannel_normal.png", "btn_MListScene_SetGameInfo_getChannel_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_SETGAMEINFO_SETCHANNEL : {
			btn = MenuItemImage::create(
					"btn_MListScene_SetGameInfo_setChannel_normal.png", "btn_MListScene_SetGameInfo_setChannel_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_SETGAMEINFO_UPDATE_SERVERID: {
			btn = MenuItemImage::create(
					"btn_MListScene_SetGameInfo_updateServerId_normal.png", "btn_MListScene_SetGameInfo_updateServerId_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_BTN_MODULE_LIST_SETGAMEINFO_UPDATE_GAMELANGUAGE : {
			btn = MenuItemImage::create(
					"btn_MListScene_SetGameInfo_updateGameLanguage_normal.png", "btn_MListScene_SetGameInfo_updateGameLanguage_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
        case TAG_BTN_MODULE_LIST_SETGAMEINFO_SEND_DATA : {
            btn = MenuItemImage::create(
                    "btn_MListScene_SetGameInfo_sendData_normal.png", "btn_MListScene_SetGameInfo_sendData_clicked.png",
                    CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
            btn->setScale(0.7f);
            break;
        }
		// Inapp V4
		case TAG_TYPE_MODULE_LIST_INAPPV4_ITEMINFO : {		// HIVE 1.18.4
			btn = MenuItemImage::create(
					"btn_MListScene_InAppV4_iteminfo_normal.png", "btn_MListScene_InAppV4_iteminfo_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_TYPE_MODULE_LIST_INAPPV4_PURCHASE : {		// HIVE 1.18.4
			btn = MenuItemImage::create(
					"btn_MListScene_InAppV4_purchase_normal.png", "btn_MListScene_InAppV4_purchase_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_TYPE_MODULE_LIST_INAPPV4_RESTORE : {		// HIVE 1.18.4
			btn = MenuItemImage::create(
					"btn_MListScene_InAppV4_restore_normal.png", "btn_MListScene_InAppV4_restore_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_TYPE_MODULE_LIST_INAPPV4_TRANSACTION_FINISH : {		// HIVE 1.18.4
			btn = MenuItemImage::create(
					"btn_MListScene_InAppV4_transcationFinish_normal.png", "btn_MListScene_InAppV4_transcationFinish_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_TYPE_MODULE_LIST_INAPPV4_GET_SELECTED_MARKET : {		// HIVE 1.18.4
			btn = MenuItemImage::create(
					"btn_MListScene_Inappv4_getselectedmarket_normal.png", "btn_MListScene_Inappv4_getselectedmarket_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_TYPE_MODULE_LIST_INAPPV4_GET_ACCOUNT_UUID : {		// HIVE 1.20.3
			btn = MenuItemImage::create(
					"btn_MListScene_Inappv4_getaccountuuid_normal.png", "btn_MListScene_Inappv4_getaccountuuid_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_TYPE_MODULE_LIST_INAPPV4_SELECTMARKET : {		// HIVE 1.18.4
			btn = MenuItemImage::create(
					"btn_MListScene_InAppV4_selectMarket_normal.png", "btn_MListScene_InAppV4_selectMarket_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_TYPE_MODULE_LIST_INAPPV4_REQUEST_BALANCE : {		// HIVE 1.18.4
			btn = MenuItemImage::create(
					"btn_MListScene_InAppV4_requestBalance_normal.png", "btn_MListScene_InAppV4_requestBalance_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_TYPE_MODULE_LIST_INAPPV4_CHARGE_BALANCE : {		// HIVE 1.18.4
			btn = MenuItemImage::create(
					"btn_MListScene_InAppV4_chargeBalance_normal.png", "btn_MListScene_InAppV4_chargeBalance_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_TYPE_MODULE_LIST_INAPPV4_POSTBOX_LOGIN : {		// HIVE 1.18.4
			btn = MenuItemImage::create(
					"btn_MListScene_Inappv4_postbox_login_normal.png", "btn_MListScene_Inappv4_postbox_login_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_TYPE_MODULE_LIST_INAPPV4_POSTBOX_SHOW : {		// HIVE 1.18.4
			btn = MenuItemImage::create(
					"btn_MListScene_Inappv4_postbox_show_normal.png", "btn_MListScene_Inappv4_postbox_show_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
        case TAG_TYPE_MODULE_LIST_INAPPV4_CHECK_PROMOTE_PURCHASE : {    // Hive 1.19.1
            btn = MenuItemImage::create(
                    "btn_MListScene_InAppV4_check_promote_purchase_normal.png", "btn_MListScene_InAppV4_check_promote_purchase_clicked.png",
                    CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
            btn->setScale(0.7f);
            break;
        }
		case TAG_TYPE_MODULE_LIST_INAPPV4_MARKETCONNECT : {    			// Hive 1.20.0
			btn = MenuItemImage::create(
					"btn_MListScene_InAppV4_marketconnect_normal.png", "btn_MListScene_InAppV4_marketconnect_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		case TAG_TYPE_MODULE_LIST_INAPPV4_SHOW_INAPP_MESSAGES : {
			btn = MenuItemImage::create(
					"btn_MListScene_InAppV4_showinappmessages_normal.png", "btn_MListScene_InAppV4_showinappmessages_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		// show log
		case TAG_BTN_SHOW_LOG : {
			btn = MenuItemImage::create(
					"btn_displayLog_normal_en.png", "btn_displayLog_clicked_en.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.8f);
			break;
		}
		// log out
		case TAG_BTN_MODULE_LIST_LOGOUT : {
			btn = MenuItemImage::create(
					"btn_logout_normal.png", "btn_logout_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		// log out with user exit
		case TAG_BTN_MODULE_LIST_LOGOUT_USEREXIT : {
			btn = MenuItemImage::create(
					"btn_logout_userexit_normal.png", "btn_logout_userexit_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		// show info
		case TAG_BTN_MODULE_LIST_SHOWINFO : {
			btn = MenuItemImage::create(
					"btn_showInfo_normal.png", "btn_showInfo_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.8f);
			break;
		}
		// HIVE 1.18.3
		case TAG_BTN_MODULE_LIST_PROMOTIONINFO_GETBANNERINFO : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoBannerInfo_normal.png", "btn_MListScene_PromoBannerInfo_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		// HIVE 1.18.3
		case TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_GETBANNERINFO : {
			btn = MenuItemImage::create(
					"btn_MListScene_PromoBannerInfo_normal.png", "btn_MListScene_PromoBannerInfo_clicked.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			btn->setScale(0.7f);
			break;
		}
		default : {
			btn = MenuItemImage::create(
					"btn_error.png", "btn_error.png",
					CC_CALLBACK_1(ModuleListScene::buttonEvent, this));
			break;
		}
	}
	//btn->setScale(0.8f);
	btn->setTag(tag);
	return btn;
}

MenuItemImage* ModuleListScene::createSettingButtons(int tag)
{
	ModuleListScene::ui_qaLogMessage = ""; // QALog 메시지 초기화.

	auto btn = MenuItemImage::create(
			"btn_setting_normal.png", "btn_setting_clicked.png",
			CC_CALLBACK_1(ModuleListScene::settingButtonEvent, this));
	//btn->setScale(0.3f);
	btn->setTag(tag);
	btn->setScale(0.7f);
	return btn;
}

void ModuleListScene::update_mainButtons(int currentModuleList)
{
	Size visibleSize = Director::getInstance()->getVisibleSize();
	ScrollView *scrollView = (ScrollView *)this->getChildByTag(TAG_SCLVIEW_MODULE_LIST);
	auto btns_main = Menu::create();

	switch(currentModuleList) {
		case TAG_BTN_MODULE_GROUP_LIST_PUSHMESSAGE : {
			// push
			log("### ModuleListScene : create main buttons(push) ###");
			btns_main = ModuleMenu::create(
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PUSH_REGISTER),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PUSH_UNREGISTER),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PUSH_DESCRIPTION),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PUSH_REGISTER_LIST),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PUSH_UNREGISTER_LIST),
                    ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PUSH_REQUEST_PERMISSION),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_USERINFO : {
			// sns_user
			log("### ModuleListScene : create main buttons(sns_user) ###");
			btns_main = ModuleMenu::create(
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_SNS_USER_HIVE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_SNS_USER_FACEBOOK),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_SNS_USER_MESSAGE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_SNS_USER_DELETE),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_SNSPOSTING : {
			// sns_user
			log("### ModuleListScene : create main buttons(sns_post) ###");
			btns_main = ModuleMenu::create(
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_SNS_POST_FACEBOOK),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_SNS_POST_GOOGLEPLUS),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_PROMOTION : {
			// promotion
			log("### ModuleListScene : create main buttons(promotion) ###");
			btns_main = ModuleMenu::create(
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTION_NOTICE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTION_FOCRED_NOTICE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTION_EVENT),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTION_FORCED_EVENT),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTION_FULLBANNER),
                    ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTION_FULLBANNER_IMAGE_BANNER),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTION_OFFERWALL),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTION_NOTICEONLY),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTION_CUSTOMWEBVIEW),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTION_REVIEW),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTION_MOREGAMES),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTION_REVIEW_EX),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_PROMOTIONINFO : {
			// promotion_info
			log("### ModuleListScene : create main buttons(promotion_info) ###");
			btns_main = ModuleMenu::create(
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_NOTICE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_EVENT),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_NOTICEONLY),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_CUSTOMWEBVIEW),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_FULLBANNER),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_NOTICE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_EVENT),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_NOTICEONLY),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_CUSTOMWEBVIEW),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_GETOFFERWALLSTATE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_GETBANNERINFO),	// HIVE 1.18.3
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_PROMOTIONACQUISITION : {
			// promotion_acquisition
			log("### ModuleListScene : create main buttons(promotion_info) ###");
			btns_main = ModuleMenu::create(
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_INFO),
//					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_QRCODESAVE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_QRCODESHARE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_LINKSHARE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_HIVELINKSHARE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_HIVELINKSHAREPLUSMESSAGE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_SENDERINFO),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_INVITEFRIENDS : {
			// invite_friends
			log("### ModuleListScene : create main buttons(invite_friends) ###");
			btns_main = ModuleMenu::create(
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_GAME),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_OTHER),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_ALL),
					//ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_GETINVITATIONS),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_FACEBOOK_RECEIVEDINVITE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_FACEBOOK_MESSAGE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_MESSAGE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_SESSION),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_FACEBOOK_SESSION),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_FACEBOOK_FRIENDSLIST),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_INAPP : {
			// inapp
			log("### ModuleListScene : create main buttons(inapp) ###");
			btns_main = ModuleMenu::create(
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_INAPP_LEBI_PREPARE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_INAPP_LEBI_REQUESTBALANCE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_INAPP_PLATFORM_PURCHASE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_INAPP_PLATFORM_RESTORE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_INAPP_PLATFORM_FINISH),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_HIVEWEBVIEW : {
			// HIVE_WebView
			log("### ModuleListScene : create main buttons(HIVE_WebView) ###");
			btns_main = ModuleMenu::create(
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_HIVEHOME),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_HIVEUSER),
//					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_HIVECAFE),				// GCPHQ-3577 - remove 1.20.2.2
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_EDITPROFILE),
//					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_FINDFRIENDS),			// GCPHQ-3577 - remove 1.20.2.2
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_INQUIRY),
                    ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_CHATBOT),            // HIVE 1.18.1
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_TERMS),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_MESSAGE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_ADULTAUTH),			// HIVE 1.10.0
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_GETNOTICE),			// HIVE 1.10.0
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_GETNOTICEINFO),		// HIVE 1.10.0
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_GETPERMISSIONINFO),		// HIVE 1.17.0
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_REQUESTPERMISSIONLIST),		// HIVE 1.17.2
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_TRACKER : {
			// tracker
			log("### ModuleListScene : create main buttons(tracker) ###");
			btns_main = ModuleMenu::create(
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_TRACKER_EVENT),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_TRACKER_SETENABLE),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_HIVEIAP : {			// HIVE 1.10.0
			// HIVE IAP
			log("### ModuleListScene : create main buttons(HIVEIAP) ###");
			btns_main = ModuleMenu::create(
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEIAP_INFO_ITEM),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEIAP_INFO_BADGE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEIAP_INFO_LEBI),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEIAP_SHOW_GOOGLELEBISELECT),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEIAP_SHOW_LEBICHARGE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEIAP_PURCHASE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEIAP_RESTORE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEIAP_POSTBOX),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_ENGAGEMENT : {		// HIVE 1.10.0
			// ENGAGEMENT
			log("### ModuleListScene : create main buttons(ENGAGEMENT) ###");
			btns_main = ModuleMenu::create(
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_ENGAGEMENT_GETEVENTS),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_ENGAGEMENT_CONSUMECOUPON),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_ENGAGEMENT_SETREADY),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_HIVEPROMOTION : {	// HIVE 1.12.0
			// HIVE Promotion
			log("### ModuleListScene : create main buttons(HIVE Promotion) ###");
			btns_main = ModuleMenu::create(
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_NEWS),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_FULLBANNER),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_NOTICE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_CUSTOMVIEW),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_CUSTOMBOARD),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_SPOT),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_DIRECT),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_OFFERWALL),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_REVIEW),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_EXIT),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_REVIEW_EX),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_HIVEPROMOTIONINFO : {	// HIVE 1.12.0
			// HIVE Promotion
			log("### ModuleListScene : create main buttons(HIVE Promotion Info) ###");
			btns_main = ModuleMenu::create(
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_BADGEINFO),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_VIEWINFO),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_BOARDINFO),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_SPOTINFO),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_DIRECTINFO),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_SETADDITIONALINFO),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_OFFERWALLSTATE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_GETBANNERINFO),	// HIVE 1.18.3
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_UPDATEPROMOTIONDATA),	// HIVE 1.19.0
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_SETGAMEINFO : {	// HIVE 1.12.0
			// SetGameInfo_SetGameInfo
			log("### ModuleListScene : create main buttons(SetGameInfo) ###");
			btns_main = ModuleMenu::create(
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_SETGAMEINFO_SETGAMEINFO),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_SETGAMEINFO_GETHIVECOUNTRYCODE),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_SETGAMEINFO_GETCHANNEL),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_SETGAMEINFO_SETCHANNEL),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_SETGAMEINFO_UPDATE_SERVERID),
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_SETGAMEINFO_UPDATE_GAMELANGUAGE),
                    ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_SETGAMEINFO_SEND_DATA),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_INAPPV4 : {	// HIVE 1.18.4
			// INAPP V4
			log("### ModuleListScene : create main buttons(INAPP V4) ###");

			btns_main = ModuleMenu::create(
					ModuleListScene::createButtons(TAG_TYPE_MODULE_LIST_INAPPV4_ITEMINFO),
					ModuleListScene::createButtons(TAG_TYPE_MODULE_LIST_INAPPV4_PURCHASE),
					ModuleListScene::createButtons(TAG_TYPE_MODULE_LIST_INAPPV4_RESTORE),
					ModuleListScene::createButtons(TAG_TYPE_MODULE_LIST_INAPPV4_TRANSACTION_FINISH),
					ModuleListScene::createButtons(TAG_TYPE_MODULE_LIST_INAPPV4_GET_SELECTED_MARKET),
					ModuleListScene::createButtons(TAG_TYPE_MODULE_LIST_INAPPV4_GET_ACCOUNT_UUID),
					ModuleListScene::createButtons(TAG_TYPE_MODULE_LIST_INAPPV4_SELECTMARKET),
					ModuleListScene::createButtons(TAG_TYPE_MODULE_LIST_INAPPV4_REQUEST_BALANCE),
					ModuleListScene::createButtons(TAG_TYPE_MODULE_LIST_INAPPV4_CHARGE_BALANCE),
					ModuleListScene::createButtons(TAG_TYPE_MODULE_LIST_INAPPV4_POSTBOX_LOGIN),
					ModuleListScene::createButtons(TAG_TYPE_MODULE_LIST_INAPPV4_POSTBOX_SHOW),
                    ModuleListScene::createButtons(TAG_TYPE_MODULE_LIST_INAPPV4_CHECK_PROMOTE_PURCHASE),
					ModuleListScene::createButtons(TAG_TYPE_MODULE_LIST_INAPPV4_MARKETCONNECT),
					ModuleListScene::createButtons(TAG_TYPE_MODULE_LIST_INAPPV4_SHOW_INAPP_MESSAGES),
					NULL);
			break;
		}
		default : {
			// error
			log("### ModuleListScene : create main buttons(error) ###");
			break;
		}
	}
	btns_main->alignItemsVerticallyWithPadding(visibleSize.height * ModuleListScene::ui_paddingSize);
	btns_main->setPosition(
			visibleSize.width * ModuleListScene::ui_mainBtns_x,
			ModuleListScene::ui_scrollViewConSize * 0.5);
	//btns_main->setPosition(
	//		visibleSize.width * 0.5,
	//		visibleSize.height * 0.5);
	scrollView->addChild(btns_main);
}

void ModuleListScene::update_settingButtons(int currentModuleList)
{
	Size visibleSize = Director::getInstance()->getVisibleSize();
	ScrollView *scrollView = (ScrollView *)this->getChildByTag(TAG_SCLVIEW_MODULE_LIST);
	auto btns_set = Menu::create();

	switch(currentModuleList) {
		case TAG_BTN_MODULE_GROUP_LIST_PUSHMESSAGE : {
			// push
			log("### ModuleListScene : create setting buttons(push) ###");
			btns_set = ModuleMenu::create(
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PUSH_REGISTER),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PUSH_UNREGISTER),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PUSH_DESCRIPTION),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PUSH_REGISTER_LIST),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PUSH_UNREGISTER_LIST),
                    ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PUSH_REQUEST_PERMISSION),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_USERINFO : {
			// sns_user
			log("### ModuleListScene : create setting buttons(sns_user) ###");
			btns_set = ModuleMenu::create(
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_SNS_USER_HIVE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_SNS_USER_FACEBOOK),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_SNS_USER_MESSAGE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_SNS_USER_DELETE),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_SNSPOSTING : {
			// sns_user
			log("### ModuleListScene : create setting buttons(sns_post) ###");
			btns_set = ModuleMenu::create(
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_SNS_POST_FACEBOOK),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_SNS_POST_GOOGLEPLUS),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_PROMOTION : {
			// promotion
			log("### ModuleListScene : create setting buttons(promotion) ###");
			btns_set = ModuleMenu::create(
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTION_NOTICE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTION_FOCRED_NOTICE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTION_EVENT),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTION_FORCED_EVENT),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTION_FULLBANNER),
                    ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTION_FULLBANNER_IMAGE_BANNER),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTION_OFFERWALL),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTION_NOTICEONLY),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTION_CUSTOMWEBVIEW),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTION_REVIEW),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTION_MOREGAMES),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTION_REVIEW_EX),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_PROMOTIONINFO : {
			// promotion_info
			log("### ModuleListScene : create setting buttons(promotion_info) ###");
			btns_set = ModuleMenu::create(
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_NOTICE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_EVENT),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_NOTICEONLY),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_CUSTOMWEBVIEW),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_FULLBANNER),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_NOTICE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_EVENT),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_NOTICEONLY),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_CUSTOMWEBVIEW),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_GETOFFERWALLSTATE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTIONINFO_GETBANNERINFO),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_PROMOTIONACQUISITION : {
			// promotion_acquisition (add (HIVE 1.9.0))
			log("### ModuleListScene : create setting buttons(promotion_acquisition) ###");
			btns_set = ModuleMenu::create(
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_INFO),
//					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_QRCODESAVE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_QRCODESHARE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_LINKSHARE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_HIVELINKSHARE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_HIVELINKSHAREPLUSMESSAGE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_SENDERINFO),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_INVITEFRIENDS : {
			// invite_friends
			log("### ModuleListScene : create setting buttons(invite_friends) ###");
			btns_set = ModuleMenu::create(
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_GAME),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_OTHER),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_ALL),
					//ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_GETINVITATIONS),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_FACEBOOK_RECEIVEDINVITE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_FACEBOOK_MESSAGE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_MESSAGE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_SESSION),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_FACEBOOK_SESSION),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_INVITEFRIENDS_FACEBOOK_FRIENDSLIST),		// HIVE 1.10.2b
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_INAPP : {
			// inapp
			log("### ModuleListScene : create setting buttons(inapp) ###");
			btns_set = ModuleMenu::create(
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_INAPP_LEBI_PREPARE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_INAPP_LEBI_REQUESTBALANCE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_INAPP_PLATFORM_PURCHASE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_INAPP_PLATFORM_RESTORE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_INAPP_PLATFORM_FINISH),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_HIVEWEBVIEW : {
			// HIVE_WebView
			log("### ModuleListScene : create setting buttons(HIVE_WebView) ###");
			btns_set = ModuleMenu::create(
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_HIVEHOME),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_HIVEUSER),
//					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_HIVECAFE),				// GCPHQ-3577 - remove 1.20.2.2
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_EDITPROFILE),
//					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_FINDFRIENDS),				// GCPHQ-3577 - remove 1.20.2.2
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_INQUIRY),
                    ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_CHATBOT),             // HIVE 1.18.1
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_TERMS),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_MESSAGE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_ADULTAUTH),			// HIVE 1.10.0
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_GETNOTICE),			// HIVE 1.10.0
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_GETNOTICEINFO),		// HIVE 1.10.0
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_GETPERMISSIONINFO),   // HIVE 1.17.0
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEWEBVIEW_REQUESTPERMISSIONLIST),   // HIVE 1.17.2
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_TRACKER : {
			// tracker
			log("### ModuleListScene : create setting buttons(tracker) ###");
			btns_set = ModuleMenu::create(
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_TRACKER_EVENT),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_TRACKER_SETENABLE),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_HIVEIAP : {			// HIVE 1.10.0
			// HIVE IAP
			log("### ModuleListScene : create main buttons(HIVEIAP) ###");
			btns_set = ModuleMenu::create(
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEIAP_INFO_ITEM),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEIAP_INFO_BADGE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEIAP_INFO_LEBI),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEIAP_SHOW_GOOGLELEBISELECT),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEIAP_SHOW_LEBICHARGE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEIAP_PURCHASE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEIAP_RESTORE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEIAP_POSTBOX),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_ENGAGEMENT : {		// HIVE 1.10.0
			// ENGAGEMENT
			log("### ModuleListScene : create main buttons(ENGAGEMENT) ###");
			btns_set = ModuleMenu::create(
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_ENGAGEMENT_GETEVENTS),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_ENGAGEMENT_CONSUMECOUPON),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_ENGAGEMENT_SETREADY),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_HIVEPROMOTION : {		// HIVE 1.12.0
			// HIVE Promotion show
			log("### ModuleListScene : create main buttons(HIVE Promotion) ###");
			btns_set = ModuleMenu::create(
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_NEWS),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_FULLBANNER),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_NOTICE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_CUSTOMVIEW),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_CUSTOMBOARD),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_SPOT),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_DIRECT),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_OFFERWALL),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_REVIEW),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_EXIT),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTION_REVIEW_EX),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_HIVEPROMOTIONINFO : {	// HIVE 1.12.0
			// HIVE Promotion Info
			log("### ModuleListScene : create main buttons(HIVE Promotion) ###");
			btns_set = ModuleMenu::create(
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_BADGEINFO),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_VIEWINFO),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_BOARDINFO),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_SPOTINFO),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_DIRECTINFO),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_SETADDITIONALINFO),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_OFFERWALLSTATE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_GETBANNERINFO),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_UPDATEPROMOTIONDATA),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_SETGAMEINFO : {			// HIVE 1.12.0
			// SetGameInfo
			log("### ModuleListScene : create main buttons(SetGameInfo) ###");
			btns_set = ModuleMenu::create(
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_SETGAMEINFO_SETGAMEINFO),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_SETGAMEINFO_GETHIVECOUNTRYCODE),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_SETGAMEINFO_GETCHANNEL),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_SETGAMEINFO_SETCHANNEL),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_SETGAMEINFO_UPDATE_SERVERID),
					ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_SETGAMEINFO_UPDATE_GAMELANGUAGE),
                    ModuleListScene::createSettingButtons(TAG_BTN_MODULE_LIST_SETGAMEINFO_SEND_DATA),
					NULL);
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_INAPPV4 : {			// HIVE 1.18.4
			// InAppV4
			log("### ModuleListScene : create main buttons(InAppV4) ###");
			btns_set = ModuleMenu::create(
					ModuleListScene::createSettingButtons(TAG_TYPE_MODULE_LIST_INAPPV4_ITEMINFO),
					ModuleListScene::createSettingButtons(TAG_TYPE_MODULE_LIST_INAPPV4_PURCHASE),
					ModuleListScene::createSettingButtons(TAG_TYPE_MODULE_LIST_INAPPV4_RESTORE),
					ModuleListScene::createSettingButtons(TAG_TYPE_MODULE_LIST_INAPPV4_TRANSACTION_FINISH),
					ModuleListScene::createSettingButtons(TAG_TYPE_MODULE_LIST_INAPPV4_GET_SELECTED_MARKET),
					ModuleListScene::createSettingButtons(TAG_TYPE_MODULE_LIST_INAPPV4_GET_ACCOUNT_UUID),
					ModuleListScene::createSettingButtons(TAG_TYPE_MODULE_LIST_INAPPV4_SELECTMARKET),
					ModuleListScene::createSettingButtons(TAG_TYPE_MODULE_LIST_INAPPV4_REQUEST_BALANCE),
					ModuleListScene::createSettingButtons(TAG_TYPE_MODULE_LIST_INAPPV4_CHARGE_BALANCE),
					ModuleListScene::createSettingButtons(TAG_TYPE_MODULE_LIST_INAPPV4_POSTBOX_LOGIN),
					ModuleListScene::createSettingButtons(TAG_TYPE_MODULE_LIST_INAPPV4_POSTBOX_SHOW),
                    ModuleListScene::createSettingButtons(TAG_TYPE_MODULE_LIST_INAPPV4_CHECK_PROMOTE_PURCHASE),
					ModuleListScene::createSettingButtons(TAG_TYPE_MODULE_LIST_INAPPV4_MARKETCONNECT),
					ModuleListScene::createSettingButtons(TAG_TYPE_MODULE_LIST_INAPPV4_SHOW_INAPP_MESSAGES),
					NULL);
			break;
		}
		default : {
			// error
			log("### ModuleListScene : create setting buttons(error) ###");
			break;
		}
	}
	btns_set->alignItemsVerticallyWithPadding(visibleSize.height * ModuleListScene::ui_paddingSize);
	btns_set->setPosition(
			visibleSize.width * ModuleListScene::ui_settingBtns_x,
			ModuleListScene::ui_scrollViewConSize * 0.5);
	//btns_set->setPosition(
	//		visibleSize.width * 0.5,
	//		visibleSize.height * 0.5);
	scrollView->addChild(btns_set);
}

void ModuleListScene::update_labels(int currentModuleList)
{
	Size visibleSize = Director::getInstance()->getVisibleSize();
	Layer *layer = (Layer *)this->getChildByTag(TAG_LAYER_MODULE_LIST);

	switch(currentModuleList) {
	case TAG_BTN_MODULE_GROUP_LIST_INFO : {
		auto label = Label::createWithSystemFont("vid#sessionToken : ", "arial.ttf", Parameter::paramSampleSetting.normalFontSize);

		label->setColor(Color3B(0, 0, 0));
		auto item = MenuItemLabel::create(label, CC_CALLBACK_1(ModuleListScene::labelEvent, this));
		item->setTag(-999);

		auto labels = Menu::create();

		labels = Menu::create(item, NULL);
		labels->setPosition(visibleSize.width * ModuleListScene::ui_paramLabel_x,
				visibleSize.height * (ModuleListScene::ui_paramLabel_y - (ModuleListScene::ui_paddingSize_y * 0)));
		layer->addChild(labels);
		}
	}
}

void ModuleListScene::labelEvent(Ref *sender)
{
	auto label = (MenuItem*) sender;

	switch(label->getTag()) {
		default : {
			// no event
			break;
		}
	}
}

void ModuleListScene::update_textFields(int currentModuleList)
{
	Size visibleSize = Director::getInstance()->getVisibleSize();
	auto btns_set = Menu::create();

	switch(currentModuleList) {
		case TAG_BTN_MODULE_GROUP_LIST_INFO : {
			// Info Scene
			log("### ModuleListScene::update_textFields(TAG_BTN_MODULE_GROUP_LIST_INFO) ###");
			ModuleListScene::createTextFields(TAG_TF_MLIST_INFO_VID_SESSIONKEY,
					ModuleListScene::ui_paramLabel_y - (ModuleListScene::ui_paddingSize_y * 0));
			break;
		}
		default : {
			log("### ModuleListScene::update_textFields(int currentModuleList) error ###");
		}
	}
}

void ModuleListScene::createTextFields(int tag, float textFieldPos_y)
{
	Size visibleSize = Director::getInstance()->getVisibleSize();

	TextFieldTTF* textField = cocos2d::TextFieldTTF::textFieldWithPlaceHolder("input here!",
				cocos2d::Size(130, 17 * 5), TextHAlignment::LEFT, "arial.ttf", Parameter::paramSampleSetting.normalFontSize);
	textField->setPosition(
			Point(visibleSize.width * ParameterListScene::ui_textField_x,
					visibleSize.height * textFieldPos_y));
	textField->setColor(ccc3(0, 0, 255));
	textField->setColorSpaceHolder(Color3B::BLUE);
	textField->setTag(tag);
	textField->setDelegate(this);

	this->addChild(textField);

	auto touchListener = EventListenerTouchOneByOne::create();

	switch(tag) {
	case TAG_TF_MLIST_INFO_VID_SESSIONKEY : {	// (1) vid 및 sesseionKey(sessionToken) 보이기.

		std::string vid_sessionKey = "";
		// vid와 sessionkey(sessionToken) 가져오기.
		vid_sessionKey = DataManager::getParamValue(TAG_APIACTION_LOGIN, "vid");
		vid_sessionKey += "#";
		vid_sessionKey += DataManager::getParamValue(TAG_APIACTION_LOGIN, "sessionToken");

		textField->setString(vid_sessionKey);	// set default or saved text
		ModuleListScene::tfArea_info_vid_sesseionKey = textField->getBoundingBox();	// change this

		touchListener->onTouchBegan = [](cocos2d::Touch* touch, cocos2d::Event * event) -> bool {
			try {
				CCPoint point = touch->getLocationInView();
				point = Director::sharedDirector()->convertToGL(point);

				if(ModuleListScene::tfArea_info_vid_sesseionKey.containsPoint(point)) {	// change this
					// Show the on screen keyboard
					ModuleListScene::focusedTextFieldState = TAG_TF_MLIST_INFO_VID_SESSIONKEY;	// change this
					auto textField_ = dynamic_cast<TextFieldTTF *>(event->getCurrentTarget());
					textField_->attachWithIME();

				}
				return true;
			}
			catch(std::bad_cast * err) {
				return true;
			}
		};
		break;
	}
	default : {
		log("### ModuleListScene::createTextField(...) error ###");
		break;
	}
	}
	this->_eventDispatcher->addEventListenerWithSceneGraphPriority(touchListener, textField);
}

void ModuleListScene::update_otherButtons(int btnType)
{
	Size visibleSize = Director::getInstance()->getVisibleSize();
	Layer *layer = (Layer *)this->getChildByTag(TAG_LAYER_MODULE_LIST);
	switch(btnType) {
		case TAG_BTN_PREVIOUS_SCENE : {
			// previous
			log("### ModuleListScene : create other buttons(previous) ###");
			auto btn = Menu::create(
					ModuleListScene::createButtons(TAG_BTN_PREVIOUS_SCENE),
					NULL);
			btn->setPosition(
					visibleSize.width * ModuleListScene::ui_previousBtn_x,
					visibleSize.height * ModuleListScene::ui_previousBtn_y);
			layer->addChild(btn);
			return;
			break;
		}
		case TAG_BTN_SHOW_LOG : {
			// show_log
			if(DataManager::getCurrentModule() == TAG_BTN_MODULE_GROUP_LIST_INFO)
				return;		// info 화면이면 생성하지 않음

			log("### ModuleListScene : create other buttons(show_log) ###");
			auto btn_log = Menu::create(
					ModuleListScene::createButtons(TAG_BTN_SHOW_LOG), NULL);
			btn_log->setPosition(
					visibleSize.width * ModuleListScene::ui_showLogBtn_x,
					visibleSize.height * ModuleListScene::ui_showLogBtn_y);
			layer->addChild(btn_log);
			break;
		}
		case TAG_BTN_MODULE_LIST_LOGOUT : {
			// logout
			if(DataManager::getCurrentModule() != TAG_BTN_MODULE_GROUP_LIST_INFO)
				return;		// info 화면이 아니면 생성하지 않음

			log("### ModuleListScene : create other buttons(logout) ###");
			auto btn = Menu::create(
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_LOGOUT), NULL);
			btn->setPosition(
					visibleSize.width * ModuleListScene::ui_logoutBtn_x,
					visibleSize.height * ModuleListScene::ui_logoutBtn_y);
			layer->addChild(btn);
			break;
		}
		case TAG_BTN_MODULE_LIST_LOGOUT_USEREXIT : {		// Hive 1.19.5.2, 1.20.0+
			// logout (userExit)
			if(DataManager::getCurrentModule() != TAG_BTN_MODULE_GROUP_LIST_INFO)
				return;		// info 화면이 아니면 생성하지 않음

			log("### ModuleListScene : create other buttons(logout_userexit) ###");
			auto btn = Menu::create(
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_LOGOUT_USEREXIT), NULL);
			btn->setPosition(
					visibleSize.width * 0.75,
					visibleSize.height * ModuleListScene::ui_logoutBtn_y);
			layer->addChild(btn);
			break;
		}
		case TAG_BTN_MODULE_LIST_SHOWINFO : {
			//showInfo
			if(DataManager::getCurrentModule() != TAG_BTN_MODULE_GROUP_LIST_INFO)
				return;		// info 화면이 아니면 생성하지 않음

			log("### ModuleListScene : create other buttons(showInfo) ###");
			auto btn = Menu::create(
					ModuleListScene::createButtons(TAG_BTN_MODULE_LIST_SHOWINFO), NULL);
			btn->setPosition(
					visibleSize.width * ModuleListScene::ui_showInfoBtn_x,
					visibleSize.height * ModuleListScene::ui_showInfoBtn_y);
			layer->addChild(btn);
			break;
		}
		default : {
			// error
			log("### ModuleListScene : create other buttons(error) ###");
			break;
		}
	}
}

LabelTTF* ModuleListScene::createLabels(int tag)
{
	Size visibleSize = Director::getInstance()->getVisibleSize();
	LabelTTF* label = LabelTTF::create();
	std::string temp = "";

	switch(tag) {
		case TAG_BTN_MODULE_GROUP_LIST_PUSHMESSAGE : {
			temp = "푸시 메시지";
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_USERINFO : {
			temp = "유저 정보";
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_SNSPOSTING : {
			temp = "SNS 포스팅";
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_PROMOTION : {
			temp = "프로모션";
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_PROMOTIONINFO : {
			temp = "프로모션 정보";
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_PROMOTIONACQUISITION : {	// add(HIVE 1.9.0)
			temp = "프로모션 집객";
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_INVITEFRIENDS : {
			temp = "친구초대";
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_INAPP : {
			if(DataManager::isWeChatTestSampleMode)
				temp = "Payment";		// 구매하기
			else
				temp = "구매하기";
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_HIVEWEBVIEW : {
			temp = "HIVE 웹뷰";
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_TRACKER : {
			temp = "마케팅 트래킹툴";
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_HIVEIAP : {
			temp = "HIVE 인앱 구매";
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_ENGAGEMENT : {
			temp = "Engagement";
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_HIVEPROMOTION : {		// HIVE 1.12.0
			temp = "HIVE Promotion";
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_HIVEPROMOTIONINFO : {	// HIVE 1.12.0
			temp = "HIVE Promo Info";
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_SETGAMEINFO : {			// HIVE 1.12.0
			temp = "SetGameInfo";
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_INAPPV4 : {			// HIVE 1.18.4
			temp = "InAppV4";
			break;
		}
		case TAG_BTN_MODULE_GROUP_LIST_INFO : {
			temp = "Info";
			break;
		}
		case TAG_LABEL_MODULE_GROUP_LIST_LOGINSERVERTYPE : {
			std::string temp_loginType = DataManager::getParamValue(TAG_APIACTION_LOGIN, "getLoginType");
			std::string temp_serverType = DataManager::getParamValue(TAG_APIACTION_INITIALIZE, "useSandbox");
			std::string temp = "Login Type : " + temp_loginType;

			// (1) Live Server 혹은 Sandbox로 설정
			std::string temp_serverType_select = "";
			if(temp_serverType == "0") {
				temp_serverType_select = "\nServer Type : Live Server";
			} else if (temp_serverType == "1") {
				temp_serverType_select = "\nServer Type : Sandbox";
			}
            // (2) Staging일 경우 설정 (staging이 true일 경우 우선 변경)
            if(DataManager::getParamValue_bool(TAG_APIACTION_INITIALIZE, "useStaging") == true) {
                temp_serverType_select = "\nServer Type : Staging";
            }
			temp = temp + temp_serverType_select;	// label 종합하기

			label = LabelTTF::create(temp, "arial.ttf", Parameter::paramSampleSetting.normalFontSize);
			label->setPosition(
					visibleSize.width * ModuleGroupListScene::ui_loginServerTypeLabel_x,
					visibleSize.height * ModuleGroupListScene::ui_loginServerTypeLabel_y);
			label->setColor(Color3B(0, 0, 0));
			label->setTag(TAG_LABEL_MODULE_GROUP_LIST_LOGINSERVERTYPE);
			return label;
			break;
		}
		default :
			break;
	}
	label = LabelTTF::create(temp, "arial.ttf", Parameter::paramSampleSetting.normalFontSize);
	label->setPosition(
			visibleSize.width * ModuleListScene::ui_moduleListNameLabel_x,
			visibleSize.height * ModuleListScene::ui_moduleListNameLabel_y);
	label->setColor(Color3B(0, 0, 0));
	return label;
}

void ModuleListScene::buttonEvent(Ref *sender)
{
	/*
	 * (1) 로컬 로그 초기화 --> (2) 각 API에 맞는 버튼 액션(혹은 일반 버튼 액션)
	 */
	if(ModuleListScene::ui_isOpenedWindow == true)	// 중복 클릭 방지
		return;

	//LogController::clearLocalLog();		// 로컬 로그 초기화
	auto btn = (MenuItem*) sender;

	switch(btn->getTag()) {				// 외부 버튼
		case TAG_BTN_PREVIOUS_SCENE : {
            Director::getInstance()->getScheduler()->performFunctionInCocosThread([=]()->void{
                auto scene = TransitionFadeTR::create(1.0, ModuleGroupListScene::createScene());
                Director::getInstance()->replaceScene(scene);
            });
			break;
		}case TAG_BTN_SHOW_LOG : {		// 로그 보기
			// log scene
            Director::getInstance()->getScheduler()->performFunctionInCocosThread([=]()->void{
                Director::getInstance()->pushScene(LogScene::createScene());
            });
			break;
		}case TAG_BTN_MODULE_LIST_LOGOUT : {    // 로그 아웃
			DataManager::setParamValue_bool(TAG_APIACTION_LOGIN, "callState", false);
			DataManager::getApiAction(TAG_APIACTION_LOGOUT);
            Director::getInstance()->getScheduler()->performFunctionInCocosThread([=]()->void{
                auto scene = TransitionFadeTR::create(1.0, LoginListScene::createScene());
                Director::getInstance()->replaceScene(scene);
            });
			break;
		}case TAG_BTN_MODULE_LIST_LOGOUT_USEREXIT : {    // 회원 탈퇴 로그 아웃 (v1.19.5.2, v1.20.0+)
			DataManager::setParamValue_bool(TAG_APIACTION_LOGIN, "callState", false);
			DataManager::getApiAction(TAG_APIACTION_LOGOUT_USEREXIT);
            Director::getInstance()->getScheduler()->performFunctionInCocosThread([=]()->void{
                auto scene = TransitionFadeTR::create(1.0, LoginListScene::createScene());
                Director::getInstance()->replaceScene(scene);
            });
			break;
		}
		case TAG_BTN_MODULE_LIST_SHOWINFO : {	// 인포 보기
			ModuleListScene::displayInfo(DataManager::getCurrentModule());
			break;
		}
	}

	if(ModuleListScene::allowClickAction() == true)	// 스크롤뷰 안의 영역이 클릭되었을 때
	{
		LogController::setApiRequestState(btn->getTag());		// QA Log 출력을 위한 버튼 클릭 상태값 저장.
		switch(btn->getTag()) {				// 메인 버튼 액션
				// PUSH
				case TAG_BTN_MODULE_LIST_PUSH_REGISTER : {
					DataManager::getApiAction(TAG_APIACTION_PUSHREGISTER);			// 푸시 등록
					break;
				}case TAG_BTN_MODULE_LIST_PUSH_UNREGISTER : {
					DataManager::getApiAction(TAG_APIACTION_PUSHUNREGISTER);		// 푸시 해제
					break;
				}case TAG_BTN_MODULE_LIST_PUSH_DESCRIPTION : {
					DataManager::getApiAction(TAG_APIACTION_PUSHDESCRIPTION);		// 푸시 해제
					break;
				}case TAG_BTN_MODULE_LIST_PUSH_REGISTER_LIST : {
					DataManager::getApiAction(TAG_APIACTION_PUSHREGISTER_LIST);            // 푸시 등록
					break;
				}case TAG_BTN_MODULE_LIST_PUSH_UNREGISTER_LIST : {
					DataManager::getApiAction(TAG_APIACTION_PUSHUNREGISTER_LIST);        // 푸시 해제
					break;
                }case TAG_BTN_MODULE_LIST_PUSH_REQUEST_PERMISSION : {
                    DataManager::getApiAction(TAG_APIACTION_PUSH_REQUEST_PERMISSION);   // iOS 명시적 푸시 알림 요청
                    break;
                }
				// SNS USER
				case TAG_BTN_MODULE_LIST_SNS_USER_HIVE : {							// 하이브 회원 정보 보기
					// --> ParameterListScene 에서 vid 정보 보냄
					DataManager::setParamValue_string(TAG_APIACTION_SNSUSER, "callState", "Hive");
					DataManager::setParamValue_string(TAG_APIACTION_SNSUSER, "type", "");
					DataManager::setParamValue_string(TAG_APIACTION_SNSUSER, "facebookId", "");
					DataManager::getApiAction(TAG_APIACTION_SNSUSER);
					break;
				}case TAG_BTN_MODULE_LIST_SNS_USER_FACEBOOK : {						// 페이스북 회원 정보 보기
					log("### error test: sns user face book ###");
					DataManager::setParamValue_string(TAG_APIACTION_SNSUSER, "callState", "facebook");
					DataManager::setParamValue_string(TAG_APIACTION_SNSUSER, "vid", "");
					DataManager::setParamValue_string(TAG_APIACTION_SNSUSER, "type", "facebook");
					DataManager::getApiAction(TAG_APIACTION_SNSUSER);
					// --> ParameterListScene 에서 facebookId 정보 보냄
					break;
				}case TAG_BTN_MODULE_LIST_SNS_USER_MESSAGE : {						// 페이스북 메시지 보기
					DataManager::setParamValue_string(TAG_APIACTION_SNSUSER, "callState", "message");
					DataManager::setParamValue_string(TAG_APIACTION_SNSUSER, "type", "message");
					DataManager::getApiAction(TAG_APIACTION_SNSUSER);
					break;
				}case TAG_BTN_MODULE_LIST_SNS_USER_DELETE : {						// 계정 삭제
                    DataManager::getApiAction(TAG_APIACTION_SNSUSER_DELETE);
					break;
				}
				// SNS POSTING
				case TAG_BTN_MODULE_LIST_SNS_POST_HIVE : {							// 하이브에 포스팅 하기
					// 보류된 기능
					break;
				}case TAG_BTN_MODULE_LIST_SNS_POST_FACEBOOK : {						// 페이스북에 포스팅 하기
					DataManager::setParamValue_string(TAG_APIACTION_SNSPOSTING, "service", "facebook");
					DataManager::getApiAction(TAG_APIACTION_SNSPOSTING);
					break;
				}case TAG_BTN_MODULE_LIST_SNS_POST_GOOGLEPLUS : {					// 구글플러스에 포스팅 하기
					DataManager::setParamValue_string(TAG_APIACTION_SNSPOSTING, "service", "googleplus");
					DataManager::getApiAction(TAG_APIACTION_SNSPOSTING);
					break;
				// PROMOTION
				}case TAG_BTN_MODULE_LIST_PROMOTION_NOTICE : {				// 프로모션 : 공지 보기
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONSHOW, "type", "notice");
					DataManager::setParamValue_bool(TAG_APIACTION_PROMOTIONSHOW, "forced", false);
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTION_FOCRED_NOTICE : {		// 프로모션 : 강제 공지 보기
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONSHOW, "type", "notice");
					DataManager::setParamValue_bool(TAG_APIACTION_PROMOTIONSHOW, "forced", true);
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTION_EVENT : {				// 프로모션 : 이벤트 보기
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONSHOW, "type", "event");
					DataManager::setParamValue_bool(TAG_APIACTION_PROMOTIONSHOW, "forced", false);
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTION_FORCED_EVENT : {		// 프로모션 : 강제 이벤트 보기
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONSHOW, "type", "event");
					DataManager::setParamValue_bool(TAG_APIACTION_PROMOTIONSHOW, "forced", true);
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTION_FULLBANNER : {			// 프로모션 : 전체 배너
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONSHOW, "type", "fullBanner");
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTION_FULLBANNER_IMAGE_BANNER : {			// 프로모션 : 전체 배너
                    DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONSHOW, "type", "fullBannerLegacy");
                    DataManager::getApiAction(TAG_APIACTION_PROMOTIONSHOW);
                    break;
                }case TAG_BTN_MODULE_LIST_PROMOTION_OFFERWALL : {			// 프로모션 : 오퍼월
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONSHOW, "type", "offerwall");
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTION_NOTICEONLY : {			// 프로모션 : 공지만 보기
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONSHOW, "type", "noticeOnly");
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTION_CUSTOMWEBVIEW : {		// 프로모션 : 커스텀 웹뷰
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONSHOW, "type", "customWebview");
					// --> 커스텀 웹뷰 viewId 셋팅 완료 (ParameterListScene)
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTION_REVIEW : {				// 프로모션 : 리뷰
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONSHOW, "type", "review");
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTION_MOREGAMES : {			// 프로모션 : 게임 더보기
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONSHOW, "type", "moreGames");
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTION_REVIEW_EX : {			// 프로모션 : 네이티브 리뷰 (HIVE 1.16.0)
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONSHOW, "type", "reviewEx");
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONSHOW);
					break;
				}
				// PROMOTION INFO
				case TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_NOTICE : {			// 프로모션 정보 : 뱃지(공지)
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "callState", "PromotionInfo");
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "type", "badge");
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "viewId", "notice");
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONINFO);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_EVENT : {				// 프로모션 정보 : 뱃지(이벤트)
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "callState", "PromotionInfo");
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "type", "badge");
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "viewId", "event");
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONINFO);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_NOTICEONLY : {		// 프로모션 정보 : 뱃지(공지만)
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "callState", "PromotionInfo");
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "type", "badge");
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "viewId", "noticeOnly");
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONINFO);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTIONINFO_BADGE_CUSTOMWEBVIEW : {		// 프로모션 정보 : 뱃지(커스텀 웹뷰)
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "callState", "customWebview");
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "type", "badge");
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONINFO);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_FULLBANNER : {			// 프로모션 정보 : 뷰(전체 배너)
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "callState", "PromotionInfo");
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "type", "view");
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "viewId", "fullBanner");
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONINFO);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_NOTICE : {				// 프로모션 정보 : 뷰(공지)
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "callState", "PromotionInfo");
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "type", "view");
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "viewId", "notice");
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONINFO);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_EVENT : {				// 프로모션 정보 : 뷰(이벤트)
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "callState", "PromotionInfo");
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "type", "view");
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "viewId", "event");
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONINFO);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_NOTICEONLY : {			// 프로모션 정보 : 뷰(공지만)
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "callState", "PromotionInfo");
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "type", "view");
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "viewId", "noticeOnly");
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONINFO);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTIONINFO_VIEW_CUSTOMWEBVIEW : {		// 프로모션 정보 : 뷰(커스텀 웹뷰)
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "callState", "customWebview");
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "type", "view");
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONINFO);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTIONINFO_GETOFFERWALLSTATE : {		// 프로모션 정보 : 오퍼월 얻기 상태
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "callState", "GetOfferwallState");
					DataManager::setParamValue_string(TAG_APIACTION_PROMOTIONINFO, "type", "offerwall");
					DataManager::getApiAction(TAG_APIACTION_PROMOTIONINFO);
					break;
				}
				// PROMOTION ACQUISITION  (add (HIVE 1.9.0))
				case TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_INFO : {				// 프로모션 집객 정보 받기
					DataManager::setParamValue_int(TAG_APIACTION_USERACQUISITION, "callState", 1);
					DataManager::getApiAction(TAG_APIACTION_USERACQUISITION);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_QRCODESAVE : {		// Qrcode 저장
					DataManager::setParamValue_int(TAG_APIACTION_USERACQUISITION, "callState", 2);
					DataManager::getApiAction(TAG_APIACTION_USERACQUISITION);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_QRCODESHARE : {		// Qrcode 공유
					DataManager::setParamValue_int(TAG_APIACTION_USERACQUISITION, "callState", 3);
					DataManager::getApiAction(TAG_APIACTION_USERACQUISITION);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_LINKSHARE : {		// 초대 링크 공유
					DataManager::setParamValue_int(TAG_APIACTION_USERACQUISITION, "callState", 4);
					DataManager::getApiAction(TAG_APIACTION_USERACQUISITION);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_HIVELINKSHARE : {	// HIVE 초대 링크 공유
					DataManager::setParamValue_int(TAG_APIACTION_USERACQUISITION, "callState", 5);
					DataManager::getApiAction(TAG_APIACTION_USERACQUISITION);
					break;
				}case TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_HIVELINKSHAREPLUSMESSAGE : {	// HIVE 초대 링크 공유
					DataManager::setParamValue_int(TAG_APIACTION_USERACQUISITION, "callState", 6);
					DataManager::getApiAction(TAG_APIACTION_USERACQUISITION);
					break;
				}
				case TAG_BTN_MODULE_LIST_PROMOTIONACQUISITION_SENDERINFO : {	// UA 초대자 정보
					DataManager::setParamValue_int(TAG_APIACTION_USERACQUISITION, "callState", 7);
					DataManager::getApiAction(TAG_APIACTION_USERACQUISITION);
					break;
				}
				// SNS FRIENDS
				case TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_GAME : {				// 친구초대 : 하이브 게임 친구
					DataManager::setParamValue_string(TAG_APIACTION_SNSFRIENDS, "service", "");
					DataManager::setParamValue_string(TAG_APIACTION_SNSFRIENDS, "type", "game");
					DataManager::getApiAction(TAG_APIACTION_SNSFRIENDS);
					break;
				}case TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_OTHER : {				// 친구초대 : 하이브 게임하지 않는 친구
					DataManager::setParamValue_string(TAG_APIACTION_SNSFRIENDS, "service", "");
					DataManager::setParamValue_string(TAG_APIACTION_SNSFRIENDS, "type", "other");
					DataManager::getApiAction(TAG_APIACTION_SNSFRIENDS);
					break;
				}case TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_ALL : {				// 친구초대 : 하이브 전체 친구
					DataManager::setParamValue_string(TAG_APIACTION_SNSFRIENDS, "service", "");
					DataManager::setParamValue_string(TAG_APIACTION_SNSFRIENDS, "type", "all");
					DataManager::getApiAction(TAG_APIACTION_SNSFRIENDS);
					break;
				}case TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_GETINVITATIONS : {		// 친구초대 : 하이브에서 나를 초대한 친구 목록.
					DataManager::setParamValue_string(TAG_APIACTION_SNSFRIENDS, "type", "getInvitations");
					DataManager::getApiAction(TAG_APIACTION_SNSFRIENDS);
					break;
				}case TAG_BTN_MODULE_LIST_INVITEFRIENDS_FACEBOOK_RECEIVEDINVITE : {	// 친구초대 : 페이스북 초대한 유저 정보
					DataManager::setParamValue_string(TAG_APIACTION_SNSFRIENDS, "type", "receivedInvite");
					DataManager::setParamValue_string(TAG_APIACTION_SNSFRIENDS, "service", "facebook");
					DataManager::getApiAction(TAG_APIACTION_SNSFRIENDS);
					break;
				}case TAG_BTN_MODULE_LIST_INVITEFRIENDS_FACEBOOK_MESSAGE : {		// 친구초대 : 페이스북 메시지 보내기
					DataManager::setParamValue_string(TAG_APIACTION_SNSMESSAGE, "callState", "facebook");
					DataManager::setParamValue_string(TAG_APIACTION_SNSMESSAGE, "service", "facebook");
					DataManager::setParamValue_string(TAG_APIACTION_SNSMESSAGE, "type", "inviteDialog");
					DataManager::getApiAction(TAG_APIACTION_SNSMESSAGE);
					break;
				}case TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_MESSAGE : {			// 친구초대 : 하이브 메시지 보내기
					DataManager::setParamValue_string(TAG_APIACTION_SNSMESSAGE, "callState", "Hive");
					DataManager::getApiAction(TAG_APIACTION_SNSMESSAGE);
					break;
				}case TAG_BTN_MODULE_LIST_INVITEFRIENDS_HIVE_SESSION : {			// 친구초대 : 하이브 세션 확인
					DataManager::setParamValue_string(TAG_APIACTION_SNSSESSION, "service", "");
					DataManager::getApiAction(TAG_APIACTION_SNSSESSION);
					break;
				}case TAG_BTN_MODULE_LIST_INVITEFRIENDS_FACEBOOK_SESSION : {		// 친구초대 : 페이스북 세션 확인
					DataManager::setParamValue_string(TAG_APIACTION_SNSSESSION, "service", "facebook");
					DataManager::getApiAction(TAG_APIACTION_SNSSESSION);
					break;
				}case TAG_BTN_MODULE_LIST_INVITEFRIENDS_FACEBOOK_FRIENDSLIST : {	// 친구초대 : 페이스북 친구이면서 하이브 친구 목록 확인.
					DataManager::setParamValue_string(TAG_APIACTION_SNSFRIENDS, "service", "facebook");
					DataManager::setParamValue_string(TAG_APIACTION_SNSFRIENDS, "type", "friendsList");
					DataManager::setParamValue_string(TAG_APIACTION_SNSFRIENDS, "callState", "friendsList");
					DataManager::getApiAction(TAG_APIACTION_SNSFRIENDS);
					break;
				}

				// INAPP
				case TAG_BTN_MODULE_LIST_INAPP_LEBI_PREPARE : {			// 인앱 : 러비로 설정
					DataManager::getApiAction(TAG_APIACTION_PREPARE);
					break;
				}case TAG_BTN_MODULE_LIST_INAPP_LEBI_REQUESTBALANCE : {		// 인앱 : 잔액 조회
					DataManager::getApiAction(TAG_APIACTION_REQUESTBALANCE);
					break;
				}case TAG_BTN_MODULE_LIST_INAPP_PLATFORM_PURCHASE : {		// 인앱 : 구매 하기
					DataManager::getApiAction(TAG_APIACTION_INAPPPURCHASE);
					break;
				}case TAG_BTN_MODULE_LIST_INAPP_PLATFORM_RESTORE : {		// 인앱 : 구매 복구
					DataManager::getApiAction(TAG_APIACTION_INAPPRESTORE);
					break;
				}case TAG_BTN_MODULE_LIST_INAPP_PLATFORM_FINISH : {			// 인앱 : 구매 종료
					DataManager::getApiAction(TAG_APIACTION_INAPPFINISH);
					break;
				}
				// SNS SHOW
				case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_HIVEHOME : {		// 하이브웹뷰 : 하이브 홈
					DataManager::setParamValue_string(TAG_APIACTION_SNSSHOW, "type", "home");
					DataManager::getApiAction(TAG_APIACTION_SNSSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_HIVEUSER : {		// 하이브웹뷰 : 프로필 페이지
					DataManager::setParamValue_string(TAG_APIACTION_SNSSHOW, "type", "user");
					DataManager::getApiAction(TAG_APIACTION_SNSSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_HIVECAFE : {		// 하이브웹뷰 : 하이브 카페
					DataManager::setParamValue_string(TAG_APIACTION_SNSSHOW, "type", "cafe");
					DataManager::getApiAction(TAG_APIACTION_SNSSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_EDITPROFILE : {	// 하이브웹뷰 : 프로필 변경
					DataManager::setParamValue_string(TAG_APIACTION_SNSSHOW, "type", "edit/profile/picture");
					DataManager::getApiAction(TAG_APIACTION_SNSSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_FINDFRIENDS : {	// 하이브웹뷰 : 친구 추천
					DataManager::setParamValue_string(TAG_APIACTION_SNSSHOW, "type", "find_friends");
					DataManager::getApiAction(TAG_APIACTION_SNSSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_INQUIRY : {		// 하이브웹뷰 : 1:1 문의
					DataManager::setParamValue_string(TAG_APIACTION_SNSSHOW, "type", "inquiry");
					DataManager::getApiAction(TAG_APIACTION_SNSSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_CHATBOT : {		// 하이브웹뷰 :챗봇
                    DataManager::setParamValue_string(TAG_APIACTION_SNSSHOW, "type", "chatbot");        // HIVE 1.18.1
                    DataManager::getApiAction(TAG_APIACTION_SNSSHOW);
                    break;
                }case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_TERMS : {			// 하이브웹뷰 : 약관 다시보기
					DataManager::setParamValue_string(TAG_APIACTION_SOCIALSHOW, "type", "terms");
					DataManager::getApiAction(TAG_APIACTION_SOCIALSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_MESSAGE : {		// 하이브웹뷰 : 쪽지보기
					DataManager::setParamValue_string(TAG_APIACTION_SNSSHOW, "type", "message");
					DataManager::getApiAction(TAG_APIACTION_SNSSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_ADULTAUTH : {		// 하이브웹뷰 : 성인 약관 보기.
					DataManager::setParamValue_string(TAG_APIACTION_SOCIALSHOW, "type", "adultAuth");
					DataManager::getApiAction(TAG_APIACTION_SOCIALSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_GETNOTICE : {		// 하이브웹뷰 : 점검 팝업 보기, 강제 업데이트 팝업 보기.	(add HIVE 1.10.0)
					DataManager::setParamValue_string(TAG_APIACTION_SOCIALSHOW, "type", "notice");
					DataManager::getApiAction(TAG_APIACTION_SOCIALSHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_GETNOTICEINFO : {
					DataManager::setParamValue_string(TAG_APIACTION_SOCIALINFO, "type", "notice");
					DataManager::getApiAction(TAG_APIACTION_SOCIALINFO);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_GETPERMISSIONINFO : {
					DataManager::setParamValue_string(TAG_APIACTION_SOCIALINFO, "type", "permission");		// HIVE 1.17.0
					DataManager::getApiAction(TAG_APIACTION_SOCIALINFO);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEWEBVIEW_REQUESTPERMISSIONLIST : {
					DataManager::setParamValue_string(TAG_APIACTION_SOCIALSHOW, "type", "requestPermissionList");		// HIVE 1.17.2
					DataManager::getApiAction(TAG_APIACTION_SOCIALSHOW);
					break;
				}
				// TRACKER
				case TAG_BTN_MODULE_LIST_TRACKER_EVENT : {			// 트래커 : 이벤트 전송
					DataManager::getApiAction(TAG_APIACTION_SENDEVENT);
					break;
				}case TAG_BTN_MODULE_LIST_TRACKER_SETENABLE : {		// 트래커 : 활성화 설정
					DataManager::getApiAction(TAG_APIACTION_SETENABLE);
					break;
				}
				// HIVE IAP (HIVE 1.10.0)
				case TAG_BTN_MODULE_LIST_HIVEIAP_INFO_ITEM : {		// HIVE IAP : 아이템 정보.
					DataManager::setParamValue_string(TAG_APIACTION_HIVEIAP_INFO, "type", "shopInfo");
					DataManager::getApiAction(TAG_APIACTION_HIVEIAP_INFO);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEIAP_INFO_BADGE : {	// HIVE IAP : 뱃지 정보.
					DataManager::setParamValue_string(TAG_APIACTION_HIVEIAP_INFO, "type", "badge");
					DataManager::getApiAction(TAG_APIACTION_HIVEIAP_INFO);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEIAP_INFO_LEBI : {		// HIVE IAP : 러비 잔액 정보.
					DataManager::setParamValue_string(TAG_APIACTION_HIVEIAP_INFO, "type", "balance");
					DataManager::getApiAction(TAG_APIACTION_HIVEIAP_INFO);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEIAP_SHOW_GOOGLELEBISELECT : {		// HIVE IAP : 러비/구글 플레이 선택.
					DataManager::setParamValue_string(TAG_APIACTION_HIVEIAP_SHOW, "type", "payment");
					DataManager::getApiAction(TAG_APIACTION_HIVEIAP_SHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEIAP_SHOW_LEBICHARGE : {			// HIVE IAP : 러비 충전.
					DataManager::setParamValue_string(TAG_APIACTION_HIVEIAP_SHOW, "type", "charge");
					DataManager::getApiAction(TAG_APIACTION_HIVEIAP_SHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEIAP_PURCHASE : {					// HIVE IAP : 구매하기.
					DataManager::getApiAction(TAG_APIACTION_HIVEIAP_PURCHASE);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEIAP_RESTORE : {					// HIVE IAP : 구매복구.
					DataManager::getApiAction(TAG_APIACTION_HIVEIAP_RESTORE);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEIAP_POSTBOX : {					// HIVE IAP : 우편함.
					DataManager::getApiAction(TAG_APIACTION_HIVEIAP_POSTBOX);
					break;
				}
				// ENGAGEMENT (HIVE 1.10.0)
				case TAG_BTN_MODULE_LIST_ENGAGEMENT_GETEVENTS : {				// GetEvents
					DataManager::getApiAction(TAG_APIACTION_ENGAGEMENT_GETEVENTS);
					break;
				}case TAG_BTN_MODULE_LIST_ENGAGEMENT_CONSUMECOUPON : {			// ConsumeCoupon
					DataManager::getApiAction(TAG_APIACTION_ENGAGEMENT_CONSUMECOUPON);
					break;
				}case TAG_BTN_MODULE_LIST_ENGAGEMENT_SETREADY : {				// SetReady (HIVE 1.11.0)
					DataManager::getApiAction(TAG_APIACTION_ENGAGEMENT_SETREADY);
					break;
				}
				// HIVE Promotion (HIVE 1.12.0)
				case TAG_BTN_MODULE_LIST_HIVEPROMOTION_NEWS : {					// FullBanner
					DataManager::setParamValue_string(TAG_APIACTION_HIVEPROMOTION_SHOW, "type", "news");
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_SHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEPROMOTION_FULLBANNER : {			// News
					DataManager::setParamValue_string(TAG_APIACTION_HIVEPROMOTION_SHOW, "type", "banner");
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_SHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEPROMOTION_NOTICE : {				// Notice
					DataManager::setParamValue_string(TAG_APIACTION_HIVEPROMOTION_SHOW, "type", "notice");
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_SHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEPROMOTION_CUSTOMVIEW : {			// Custom View
					DataManager::setParamValue_string(TAG_APIACTION_HIVEPROMOTION_SHOW, "type", "customView");
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_SHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEPROMOTION_CUSTOMBOARD : {			// Custom Board
					DataManager::setParamValue_string(TAG_APIACTION_HIVEPROMOTION_SHOW, "type", "customBoard");
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_SHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEPROMOTION_SPOT : {				// Spot Banner
					DataManager::setParamValue_string(TAG_APIACTION_HIVEPROMOTION_SHOW, "type", "spot");
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_SHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEPROMOTION_DIRECT : {				// Direct Banner
					DataManager::setParamValue_string(TAG_APIACTION_HIVEPROMOTION_SHOW, "type", "direct");
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_SHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEPROMOTION_OFFERWALL : {			// Offerwall
					DataManager::setParamValue_string(TAG_APIACTION_HIVEPROMOTION_SHOW, "type", "offerwall");
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_SHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEPROMOTION_REVIEW : {				// OfferwallState
					DataManager::setParamValue_string(TAG_APIACTION_HIVEPROMOTION_SHOW, "type", "review");
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_SHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEPROMOTION_EXIT : {				// Review Popup
					DataManager::setParamValue_string(TAG_APIACTION_HIVEPROMOTION_SHOW, "type", "moreGames");
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_SHOW);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEPROMOTION_REVIEW_EX : {			// ReviewEx popup
					DataManager::setParamValue_string(TAG_APIACTION_HIVEPROMOTION_SHOW, "type", "reviewEx");
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_SHOW);
					break;
				}
				// HIVE Promotion Info (HIVE 1.12.0)
				case TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_BADGEINFO : {				// Badge info
					DataManager::setParamValue_string(TAG_APIACTION_HIVEPROMOTION_INFO, "type", "badge");
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_INFO);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_VIEWINFO : {				// View Info
					DataManager::setParamValue_string(TAG_APIACTION_HIVEPROMOTION_INFO, "type", "customView");
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_INFO);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_BOARDINFO : {				// Board Info
					DataManager::setParamValue_string(TAG_APIACTION_HIVEPROMOTION_INFO, "type", "customBoard");
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_INFO);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_SPOTINFO : {				// spot Info
					DataManager::setParamValue_string(TAG_APIACTION_HIVEPROMOTION_INFO, "type", "spot");
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_INFO);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_DIRECTINFO : {				// direct Info
					DataManager::setParamValue_string(TAG_APIACTION_HIVEPROMOTION_INFO, "type", "direct");
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_INFO);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_SETADDITIONALINFO : {		// set additionalInfo
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_SETADDITIONALINFO);
					break;
				}case TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_OFFERWALLSTATE : {			// offerwallState
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_GETOFFERWALLSTATE);
					break;
				}
				// SetGameInfo (HIVE 1.12.0)
				case TAG_BTN_MODULE_LIST_SETGAMEINFO_SETGAMEINFO : {					// setGameInfo
					DataManager::getApiAction(TAG_APIACTION_SETGAMEINFO_SETGAMEINFO);
					break;
				}
				case TAG_BTN_MODULE_LIST_SETGAMEINFO_GETHIVECOUNTRYCODE : {
					DataManager::getApiAction(TAG_APIACTION_SETGAMEINFO_GETHIVECOUNTRYCODE);
					break;
				}
				case TAG_BTN_MODULE_LIST_SETGAMEINFO_GETCHANNEL : {
					DataManager::getApiAction(TAG_APIACTION_SETGAMEINFO_GETCHANNEL);
					break;
				}
				case TAG_BTN_MODULE_LIST_SETGAMEINFO_SETCHANNEL : {
					DataManager::getApiAction(TAG_APIACTION_SETGAMEINFO_SETCHANNEL);
					break;
				}
				case TAG_BTN_MODULE_LIST_SETGAMEINFO_UPDATE_SERVERID : {
					DataManager::getApiAction(TAG_APIACTION_SETGAMEINFO_UPDATE_SERVERID);
					break;
				}
				case TAG_BTN_MODULE_LIST_SETGAMEINFO_UPDATE_GAMELANGUAGE : {
					DataManager::getApiAction(TAG_APIACTION_SETGAMEINFO_UPDATE_GAMELANGUAGE);
					break;
				}
                case TAG_BTN_MODULE_LIST_SETGAMEINFO_SEND_DATA : {
                    DataManager::getApiAction(TAG_APIACTION_SETGAMEINFO_SEND_DATA);
                    break;
                }
				// Get Banner Info (HIVE 1.18.3)
				case TAG_BTN_MODULE_LIST_PROMOTIONINFO_GETBANNERINFO: {
					DataManager::getApiAction(TAG_APIACTION_PROMOTION_GETBANNERINFO);
					break;
				}
				case TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_GETBANNERINFO : {
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_GETBANNERINFO);
					break;
				}
				case TAG_BTN_MODULE_LIST_HIVEPROMOTIONINFO_UPDATEPROMOTIONDATA : {			// updatePromotionData
					DataManager::getApiAction(TAG_APIACTION_HIVEPROMOTION_UPDATEPROMOTIONDATA);
					break;
				}
				/**
				 * INAPP V4 (HIVE 1.18.4)
				 */
				case TAG_TYPE_MODULE_LIST_INAPPV4_ITEMINFO : {
                    DataManager::getApiAction(TAG_APIACTION_INAPPV4_ITEMINFO);
					break;
				}
				case TAG_TYPE_MODULE_LIST_INAPPV4_PURCHASE : {
                    DataManager::getApiAction(TAG_APIACTION_INAPPV4_PURCHASE);
					break;
				}
				case TAG_TYPE_MODULE_LIST_INAPPV4_RESTORE : {
                    DataManager::getApiAction(TAG_APIACTION_INAPPV4_RESTORE);
					break;
				}
				case TAG_TYPE_MODULE_LIST_INAPPV4_TRANSACTION_FINISH : {
                    DataManager::getApiAction(TAG_APIACTION_INAPPV4_TRANSACTION_FINISH);
					break;
				}
				case TAG_TYPE_MODULE_LIST_INAPPV4_GET_SELECTED_MARKET : {
					DataManager::getApiAction(TAG_APIACTION_INAPPV4_GET_SELECTED_MARKET);
					break;
				}
				case TAG_TYPE_MODULE_LIST_INAPPV4_GET_ACCOUNT_UUID : {
					DataManager::getApiAction(TAG_APIACTION_INAPPV4_GET_ACCOUNT_UUID);
					break;
				}
				case TAG_TYPE_MODULE_LIST_INAPPV4_SELECTMARKET : {
                    DataManager::getApiAction(TAG_APIACTION_INAPPV4_MARKET_SELECT);
					break;
				}
				case TAG_TYPE_MODULE_LIST_INAPPV4_REQUEST_BALANCE : {
                    DataManager::getApiAction(TAG_APIACTION_INAPPV4_REQUEST_BALANCE);
					break;
				}
				case TAG_TYPE_MODULE_LIST_INAPPV4_CHARGE_BALANCE : {
                    DataManager::getApiAction(TAG_APIACTION_INAPPV4_CHARGE_BALANCE);
					break;
				}
				case TAG_TYPE_MODULE_LIST_INAPPV4_POSTBOX_LOGIN : {
					DataManager::getApiAction(TAG_APIACTION_INAPPV4_POSTBOX_LOGIN);
					break;
				}
				case TAG_TYPE_MODULE_LIST_INAPPV4_POSTBOX_SHOW : {
#ifdef __ANDROID__
					if(Parameter::paramInitialize.usePostboxV4forAndroid == false) {
						DataManager::getApiAction(TAG_APIACTION_INAPPV4_POSTBOX_SHOW);
					} else {
						// KS Cocos Postbox 개선 (Postbox v4 사용)
                        DataManager::setCurrentParameter(TAG_TYPE_MODULE_LIST_INAPPV4_POSTBOX_SHOW);
                        Director::getInstance()->getScheduler()->performFunctionInCocosThread([=]()->void{
                            auto scene = TransitionFadeTR::create(1.0, ParameterListScene::createScene());
                            Director::getInstance()->replaceScene(scene);
                        });
					}
#else
					// KS Cocos Postbox 개선 (Postbox v4 사용)
					DataManager::setCurrentParameter(TAG_TYPE_MODULE_LIST_INAPPV4_POSTBOX_SHOW);
                    Director::getInstance()->getScheduler()->performFunctionInCocosThread([=]()->void{
                        auto scene = TransitionFadeTR::create(1.0, ParameterListScene::createScene());
                        Director::getInstance()->replaceScene(scene);
                    });

#endif
					break;
				}
                case TAG_TYPE_MODULE_LIST_INAPPV4_CHECK_PROMOTE_PURCHASE : {
                    DataManager::getApiAction(TAG_APIACTION_INAPPV4_CHECK_PROMOTE_PURCHASE);
                    break;
                }
				case TAG_TYPE_MODULE_LIST_INAPPV4_MARKETCONNECT : {
					DataManager::getApiAction(TAG_APIACTION_INAPPV4_MARKETCONNECT);
					break;
				}
				case TAG_TYPE_MODULE_LIST_INAPPV4_SHOW_INAPP_MESSAGES : {
					DataManager::getApiAction(TAG_APIACTION_INAPPV4_SHOW_INAPP_MESSAGES);
					break;
				}
				default : {
					log("### ModuleListScene::buttonEvent ### error.. has not case of call");
					break;
				}
			} // switch end
	}
}

void ModuleListScene::settingButtonEvent(Ref *sender)
{
	auto btn = (MenuItem*) sender;
	DataManager::setCurrentParameter(btn->getTag());
    Director::getInstance()->getScheduler()->performFunctionInCocosThread([=]()->void{
        auto scene = TransitionFadeTR::create(1.0, ParameterListScene::createScene());
        Director::getInstance()->replaceScene(scene);
    });
}

void ModuleListScene::displayInfo(int tag)
{
	if(tag != TAG_BTN_MODULE_GROUP_LIST_INFO)
		return;

	MessageBox(ModuleListScene::ui_infoText.c_str(), "");
}

void ModuleListScene::update_info(float dt)
{
	// 초기화
	ModuleListScene::ui_infoText = "";
	// uid
	ModuleListScene::ui_infoText += "- UID : ";
	ModuleListScene::ui_infoText += (DataManager::getParamValue(TAG_APIACTION_LOGIN, "uid") + "\n");
	// vid
	ModuleListScene::ui_infoText += "- VID : ";
	ModuleListScene::ui_infoText += (DataManager::getParamValue(TAG_APIACTION_LOGIN, "vid") + "\n");
	// DID
	ModuleListScene::ui_infoText += "- DID : ";
	ModuleListScene::ui_infoText += (DataManager::getParamValue(TAG_APIACTION_INITIALIZE, "did") + "\n");
	// Server
	ModuleListScene::ui_infoText += "- Server : ";
    if(DataManager::getParamValue_bool(TAG_APIACTION_INITIALIZE, "useStaging") == true) {		// for staging
        ModuleListScene::ui_infoText += "Staging\n";
    } else if(DataManager::getParamValue_bool(TAG_APIACTION_INITIALIZE, "useSandbox") == true) {
		ModuleListScene::ui_infoText += "Sandbox\n";
	} else if(DataManager::getParamValue_bool(TAG_APIACTION_INITIALIZE, "useSandbox") == false) {
		ModuleListScene::ui_infoText += "Live Server\n";
	}
	// AppId
	ModuleListScene::ui_infoText += "- AppId : ";
	ModuleListScene::ui_infoText += (DataManager::getParamValue(TAG_APIACTION_INITIALIZE, "appId") + "\n");
	// Login State
	ModuleListScene::ui_infoText += "- Login State(getLoginType) : ";
	ModuleListScene::ui_infoText += ::GetLoginType();
	ModuleListScene::ui_infoText += ("\n");
	// sessionKey
	ModuleListScene::ui_infoText += "- SessionKey : ";
	ModuleListScene::ui_infoText += (DataManager::getParamValue(TAG_APIACTION_LOGIN, "sessionToken")  + "\n");
	// 콜백 방법.
	ModuleListScene::ui_infoText += "- Delegate : ";
	if(DataManager::getParamValue_bool(TAG_APIACTION_INITIALIZE, "delegate") == true) {
		ModuleListScene::ui_infoText += "전역 Delegate 사용중\n";
	} else {
		ModuleListScene::ui_infoText += "CompletionHandler 사용중\n";
	}

	// facebook, singular, adjust 버전 표기
	// TODO : getVersion 해야 하나 임시로 수동 표기
#ifdef __ANDROID__
	ModuleListScene::ui_infoText += "- 적용 3rd party lib(Android)\n";
	ModuleListScene::ui_infoText += "-> Facebook sdk : ";
	ModuleListScene::ui_infoText += getAndroidFacebookVersion();
	ModuleListScene::ui_infoText += "\n";
	ModuleListScene::ui_infoText += "-> Adjust sdk : ";
	ModuleListScene::ui_infoText += getAndroidAdjustVersion();
	ModuleListScene::ui_infoText += "\n";
	ModuleListScene::ui_infoText += "-> Singular sdk : ";
	ModuleListScene::ui_infoText += getAndroidSingularVersion();
	ModuleListScene::ui_infoText += "\n";
	ModuleListScene::ui_infoText += "-> AppsFlyer sdk : ";
	ModuleListScene::ui_infoText += getAndroidAppsFlyerVersion();
	ModuleListScene::ui_infoText += "\n";
#else
	ModuleListScene::ui_infoText += "- 적용 3rd party lib(iOS)\n";
	ModuleListScene::ui_infoText += "-> Facebook sdk : v12.3.1\n";
	ModuleListScene::ui_infoText += "-> Adjust sdk : v4.33.3\n";
    ModuleListScene::ui_infoText += "-> Singular sdk : v12.0.1\n";
    ModuleListScene::ui_infoText += "-> AppsFlyer sdk : v6.9.1\n";
#endif
}

bool ModuleListScene::allowClickAction()
{
	Size visibleSize = Director::getInstance()->getVisibleSize();

	if(ModuleGroupListScene::ui_touchStart_y > (visibleSize.height * 0.1)
			&& ModuleGroupListScene::ui_touchStart_y < (visibleSize.height * 0.8))
		return true;	// 스크롤뷰 안에는 클릭했을 때
	else
		return false;	// 스크롤뷰 외부를 클릭했을 떼
}

void ModuleListScene::update_QALOG(float dt)
{
	// !! 만약 ModuleListScene과 LogController와의 종속성을 최소한으로 한다면
	// schedule 함수를 활용하여 LogController의 createQALog를 주기적으로 호출(삭제 -- 갱신)

	// 기존 로그가 있다면 삭제.
	Layer *layer = (Layer *)this->getChildByTag(TAG_LAYER_MODULE_LIST);
	layer->removeChildByTag(TAG_LABEL_QALOG, true);

	// 새로 로그 생성
	Size visibleSize = Director::getInstance()->getVisibleSize();
	LabelTTF* label = LabelTTF::create(ModuleListScene::ui_qaLogMessage, "arial.ttf", Parameter::paramSampleSetting.normalFontSize, Size(visibleSize.width * 0.7, visibleSize.height * 0.1),
			TextHAlignment::CENTER, TextVAlignment::CENTER );
	label->setPosition(visibleSize.width * 0.3, visibleSize.height * 0.05);
	label->setColor(Color3B(0, 0, 0));
	label->setTag(TAG_LABEL_QALOG);

	layer->addChild(label);
}

void ModuleListScene::update_server_loginState(float dt)
{
	Layer *layer = (Layer *)this->getChildByTag(TAG_LAYER_MODULE_LIST);
	// remove
	layer->removeChildByTag(TAG_LABEL_MODULE_GROUP_LIST_LOGINSERVERTYPE, true);
	// re-draw
	layer->addChild(ModuleListScene::createLabels(TAG_LABEL_MODULE_GROUP_LIST_LOGINSERVERTYPE));
}

// display acquisition qrcode image (HIVE 1.9.0)
void ModuleListScene::display_acquisitionQrcode()
{
	Size visibleSize = Director::getInstance()->getVisibleSize();

	typedef unsigned char byte;
	Image *image = new Image();
	image->initWithImageData(
			DataManager::convertHexStringToBytes(
					DataManager::getParamValue(TAG_APIACTION_USERACQUISITION, "qrcode")).data(),
			DataManager::convertHexStringToBytes(
					DataManager::getParamValue(TAG_APIACTION_USERACQUISITION, "qrcode")).size());

	Texture2D* texture = new Texture2D();
	texture->initWithImage(image);

	Sprite* sp = Sprite::createWithTexture(texture);
	sp->setPosition(visibleSize.width * 0.8, visibleSize.height * 0.85);
	sp->setScale(2.0f);
	sp->setTag(TAG_IMAGE_QRCODE);
	this->addChild(sp);
}

void ModuleListScene::displayQRcodePopup(float dt)
{
	if(ModuleListScene::isDisplayedQRcode == true) {
		// Display Pop-up
		log("### ModuleListScene : displayQRcodePopup 001 ###");
		ModuleListScene::ui_isOpenedWindow = true;

		log("### ModuleListScene : displayQRcodePopup 002 ###");
		Size visibleSize = Director::getInstance()->getVisibleSize();		// 사용 x, 320 x 480 고정 중.
		auto popup_layer = LayerColor::create(Color4B(0, 125, 125, 255));
		popup_layer->setContentSize(Size(visibleSize.width * 0.90, visibleSize.height * 0.5));
		popup_layer->setPosition(Point(visibleSize.width * 0.05, visibleSize.height * 0.05));
		popup_layer->setTag(-10000);
		this->addChild(popup_layer);

		log("### ModuleListScene : displayQRcodePopup 003 ###");
		MenuItemFont::setFontSize(15);
		auto button_1 = MenuItemFont::create("[ close ]",
				CC_CALLBACK_1(ModuleListScene::closeQRcodePopup, this));
		button_1->setColor(ccc3(255, 255, 255));
		button_1->setTag(TAG_BTN_POPUP_CLOSE);
		auto Popup_1 = Menu::create(button_1, NULL);
		Popup_1->setPosition(Point(visibleSize.width * 0.45, visibleSize.height * 0.1));
		popup_layer->addChild(Popup_1);

		// draw QRcode
		log("### ModuleListScene : displayQRcodePopup 004 ###");
		typedef unsigned char byte;
		Image *image = new Image();
		image->initWithImageData(
				DataManager::convertHexStringToBytes(
						DataManager::getParamValue(TAG_APIACTION_USERACQUISITION, "qrcode")).data(),
				DataManager::convertHexStringToBytes(
						DataManager::getParamValue(TAG_APIACTION_USERACQUISITION, "qrcode")).size());

		log("### ModuleListScene : displayQRcodePopup 005 ###");
		Texture2D* texture = new Texture2D();
		log("### ModuleListScene : displayQRcodePopup 0051 ###");
		texture->initWithImage(image);
		log("### ModuleListScene : displayQRcodePopup 0052 ###");
		Sprite* sp = Sprite::createWithTexture(texture);
		log("### ModuleListScene : displayQRcodePopup 0053 ###");
		sp->setPosition(visibleSize.width * 0.45, visibleSize.height * 0.25);
		log("### ModuleListScene : displayQRcodePopup 0054 ###");
		sp->setScale(0.8f);
		log("### ModuleListScene : displayQRcodePopup 0055 ###");
		sp->setTag(TAG_IMAGE_QRCODE);
		log("### ModuleListScene : displayQRcodePopup 0056 ###");
		popup_layer->addChild(sp);

		log("### ModuleListScene : displayQRcodePopup 006 ###");
		ModuleListScene::isDisplayedQRcode = false;
	}
}

void ModuleListScene::displayHiveIAPInfoItemPopup(float dt)
{
	if(ModuleListScene::isDisplayHiveIAPInfoItem == true) {
		// Display Pop-up
		ModuleListScene::ui_isOpenedWindow = true;

		Size visibleSize = Director::getInstance()->getVisibleSize();		// 사용 x, 320 x 480 고정 중.
		auto popup_layer = LayerColor::create(Color4B(0, 125, 125, 255));
		// 배경 레이어 생성.
		popup_layer->setContentSize(Size(visibleSize.width * 0.90, visibleSize.height * 0.9));
		popup_layer->setPosition(Point(visibleSize.width * 0.05, visibleSize.height * 0.05));
		popup_layer->setTag(-20000);
		this->addChild(popup_layer);

		// 텍스트 버튼 생성 : close btn
		MenuItemFont::setFontSize(15);
		auto button_1 = MenuItemFont::create("[ close ]",
				CC_CALLBACK_1(ModuleListScene::closeHiveIAPInfoItemPopup, this));
		button_1->setColor(ccc3(255, 255, 255));
		button_1->setTag(TAG_BTN_POPUP_CLOSE);
		auto Popup_1 = Menu::create(button_1, NULL);
		Popup_1->setPosition(Point(visibleSize.width * 0.8, visibleSize.height * 0.03));
		popup_layer->addChild(Popup_1);

		// draw item image
		log("##### start draw image #####");
		for(int i = 0; i < Parameter::paramHiveIAPInfoItemList.size(); i++) {
			log("##### draw image loop start #####");
			HiveIAPInfoItem infoItem;
			infoItem = Parameter::paramHiveIAPInfoItemList[i];
			ModuleListScene::loadImageEx(infoItem.productImageUrl);
		}

		// draw text label
		int buyListCnt = -20001;
		for(int i = 0; i < Parameter::paramHiveIAPInfoItemList.size(); i++)	{
			HiveIAPInfoItem infoItem;
			infoItem = Parameter::paramHiveIAPInfoItemList[i];
			std::string textLabel = "";

			textLabel += ("title : " + infoItem.title + "\n");
			textLabel += ("description : " + infoItem.description + "\n");
			textLabel += ("Price : " + infoItem.displayPrice + "\n");
			textLabel += ("marketPid : " + infoItem.marketPid + "\n");
			textLabel += ("pid : " + infoItem.pid);

			auto label = LabelTTF::create(textLabel, "arial.ttf", Parameter::paramSampleSetting.normalFontSize);
			label->setPosition(
					Vec2(visibleSize.width * 0.45,
					visibleSize.height * (1 - (ModuleListScene::ui_HiveIAPInfoItemImage_y + 0.13 + (ModuleListScene::ui_HiveIAPInfoItemImage_margin_y * i)))));
			label->setColor(Color3B(0, 0, 0));

			Layer *popupLayer = (Layer *)this->getChildByTag(-20000);
			popupLayer->addChild(label);

			// 텍스트 버튼 생성 : buy btn
			MenuItemFont::setFontSize(15);
			auto button_1 = MenuItemFont::create("[ buy ]",
					CC_CALLBACK_1(ModuleListScene::closeHiveIAPInfoItemPopup, this));
			button_1->setColor(ccc3(255, 255, 255));
			button_1->setTag(buyListCnt);
			buyListCnt--;
			auto Popup_1 = Menu::create(button_1, NULL);
			Popup_1->setPosition(Point(visibleSize.width * 0.45, visibleSize.height * (1 - (ModuleListScene::ui_HiveIAPInfoItemImage_y + 0.22 + (ModuleListScene::ui_HiveIAPInfoItemImage_margin_y * i)))));
			popup_layer->addChild(Popup_1);
		}

		ModuleListScene::isDisplayHiveIAPInfoItem = false;
		ModuleListScene::imageCntHiveIAPInfoItem = 0;
	}
}

void ModuleListScene::displayHiveIAPPurchaseItemPopup(float dt)
{
	if(ModuleListScene::isDisplayHiveIAPPurchaseItem == true) {
		// Display Pop-up
		ModuleListScene::ui_isOpenedWindow = true;

		Size visibleSize = Director::getInstance()->getVisibleSize();		// 사용 x, 320 x 480 고정 중.
		auto popup_layer = LayerColor::create(Color4B(30, 144, 255, 250));
		// 배경 레이어 생성.
		popup_layer->setContentSize(Size(visibleSize.width * 0.90, visibleSize.height * 0.9));
		popup_layer->setPosition(Point(visibleSize.width * 0.05, visibleSize.height * 0.05));
		popup_layer->setTag(-20000);
		this->addChild(popup_layer);

		// 텍스트 버튼 생성 : close btn
		MenuItemFont::setFontSize(15);
		auto button_1 = MenuItemFont::create("[ close ]",
				CC_CALLBACK_1(ModuleListScene::closeHiveIAPInfoItemPopup, this));
		button_1->setColor(ccc3(255, 255, 255));
		button_1->setTag(TAG_BTN_POPUP_CLOSE);
		auto Popup_1 = Menu::create(button_1, NULL);
		Popup_1->setPosition(Point(visibleSize.width * 0.8, visibleSize.height * 0.03));
		popup_layer->addChild(Popup_1);

		// draw item image
		log("##### start draw image #####");
		for(int i = 0; i < Parameter::paramHiveIAPInfoItemPurchaseList.size(); i++) {
			log("##### draw image loop start #####");
			HiveIAPInfoItem infoItem;
			infoItem = Parameter::paramHiveIAPInfoItemPurchaseList[i];
			ModuleListScene::loadImageEx(infoItem.productImageUrl);
		}

		// draw text label
		int buyListCnt = -20001;
		for(int i = 0; i < Parameter::paramHiveIAPInfoItemPurchaseList.size(); i++)	{
			HiveIAPInfoItem infoItem;
			infoItem = Parameter::paramHiveIAPInfoItemPurchaseList[i];
			std::string textLabel = "";

			textLabel += ("title : " + infoItem.title + "\n");
			textLabel += ("description : " + infoItem.description + "\n");
			textLabel += ("Price : " + infoItem.displayPrice + "\n");
			textLabel += ("marketPid : " + infoItem.marketPid + "\n");
			textLabel += ("pid : " + infoItem.pid);

			auto label = LabelTTF::create(textLabel, "arial.ttf", Parameter::paramSampleSetting.normalFontSize);
			label->setPosition(
					Vec2(visibleSize.width * 0.45,
					visibleSize.height * (1 - (ModuleListScene::ui_HiveIAPInfoItemImage_y + 0.13 + (ModuleListScene::ui_HiveIAPInfoItemImage_margin_y * i)))));
			label->setColor(Color3B(0, 0, 0));

			Layer *popupLayer = (Layer *)this->getChildByTag(-20000);
			popupLayer->addChild(label);

//			// 텍스트 버튼 생성 : buy btn
//			MenuItemFont::setFontSize(15);
//			auto button_1 = MenuItemFont::create("[ buy ]",
//					CC_CALLBACK_1(ModuleListScene::closeHiveIAPInfoItemPopup, this));
//			button_1->setColor(ccc3(255, 255, 255));
//			button_1->setTag(buyListCnt);
//			buyListCnt--;
//			auto Popup_1 = Menu::create(button_1, NULL);
//			Popup_1->setPosition(Point(visibleSize.width * 0.45, visibleSize.height * (1 - (ModuleListScene::ui_HiveIAPInfoItemImage_y + 0.22 + (ModuleListScene::ui_HiveIAPInfoItemImage_margin_y * i)))));
//			popup_layer->addChild(Popup_1);
		}

		ModuleListScene::isDisplayHiveIAPPurchaseItem = false;
		ModuleListScene::imageCntHiveIAPInfoItem = 0;
	}
}

void ModuleListScene::closeQRcodePopup(Ref *sender)
{
	auto mItem = (MenuItem*) sender;
	// remove
	switch(mItem->getTag()) {
		case TAG_BTN_POPUP_CLOSE : {
			this->removeChildByTag(-10000, true);
			ModuleListScene::ui_isOpenedWindow = false;
			break;
		}
		default :
			break;
	}
}

void ModuleListScene::closeHiveIAPInfoItemPopup(Ref *sender)
{
	auto mItem = (MenuItem*) sender;
	// remove
	switch(mItem->getTag()) {
		case TAG_BTN_POPUP_CLOSE : {
			this->removeChildByTag(-20000, true);
			ModuleListScene::ui_isOpenedWindow = false;
			break;
		}
		case -20001 : {
			HiveIAPInfoItem temp_itemInfo;
			temp_itemInfo = Parameter::paramHiveIAPInfoItemList[0];
			DataManager::setParamValue_string(TAG_APIACTION_HIVEIAP_PURCHASE, "pid", temp_itemInfo.pid);
			DataManager::getApiAction(TAG_APIACTION_HIVEIAP_PURCHASE);
			break;
		}
		case -20002 : {
			HiveIAPInfoItem temp_itemInfo;
			temp_itemInfo = Parameter::paramHiveIAPInfoItemList[1];
			DataManager::setParamValue_string(TAG_APIACTION_HIVEIAP_PURCHASE, "pid", temp_itemInfo.pid);
			DataManager::getApiAction(TAG_APIACTION_HIVEIAP_PURCHASE);
			break;
		}
		case -20003 : {
			HiveIAPInfoItem temp_itemInfo;
			temp_itemInfo = Parameter::paramHiveIAPInfoItemList[2];
			DataManager::setParamValue_string(TAG_APIACTION_HIVEIAP_PURCHASE, "pid", temp_itemInfo.pid);
			DataManager::getApiAction(TAG_APIACTION_HIVEIAP_PURCHASE);
			break;
		}
		default :
			break;
	}
}

//void ModuleListScene::downLoadImage()
//{
//	std::string strImage = "img.png";
//
//	CCHttpRequest* request = new CCHttpRequest();
//	request->setUrl("http://http://112.175.60.185/hiveiapimage/438/201607/1438677191_Sneakers(2).png");
//	request->setRequestType(CCHttpRequest::kHttpGet);
//	request->setResponseCallback(this, httpresponse_selector(ClassName::onImageDownLoaded));
//	request->setTag(strImage.c_str());
//	CCHttpClient::getInstance()->send(request);
//	request->release();
//}
//
//void ModuleListScene::onImageDownLoaded(HttpClient* pSender, HttpResponse* pResponse)
//{
//	HttpClient* response = pResponse;
//
//	if (!response)
//	{
//		log("No Response");
//		return;
//	}
//	int statusCode = response->getResponseCode();
//
//	char statusString[64] = {};
//	sprintf(statusString, "HTTP Status Code: %d, tag = %s", statusCode, response->getHttpRequest()->getTag());
//	log("response code: %d", statusCode);
//
//	if (!response->isSucceed())
//	{
//		log("response failed");
//		log("error buffer: %s", response->getErrorBuffer());
//		return;
//	}
//	std::vector<char> *buffer = response->getResponseData();
//
//	CCImage * img=new CCImage();
//	img->initWithImageData(&(buffer->front()), buffer->size());
//
//	// Save image file to device.
//	std::string writablePath = CCFileUtils::sharedFileUtils()->getWritablePath();
//	writablePath.append(response->getHttpRequest()->getTag());
//
//
//	//Now create Sprite from downloaded image
//	CCSprite* pSprite = CCSprite::create(writablePath.c_str());
//	addchild(pSprite);
//}

void ModuleListScene::loadImage()
{
    log("onHttpRequestCompleted In the request");
    HttpRequest* request = new (std::nothrow) HttpRequest();
    request->setUrl("http://112.175.60.185/hiveiapimage/438/201607/1438677191_Sneakers(2).png");
    request->setRequestType(cocos2d::network::HttpRequest::Type::GET);
    request->setResponseCallback(CC_CALLBACK_2(ModuleListScene::onRequestImgCompleted, this));
    request->setTag("Post test2");
    HttpClient::getInstance()->send(request);
    request->release();
}

void ModuleListScene::loadImageEx(std::string urlPath)
{
    log("onHttpRequestCompleted In the request");
    HttpRequest* request = new (std::nothrow) HttpRequest();

    if(urlPath.find(".png") != std::string::npos || urlPath.find(".jpg") != std::string::npos)
    	request->setUrl(urlPath.c_str());			// loadImageEx : support png and jpg
    else {
    	// not support type
    	Sprite * sprite = Sprite :: create("Logo_Hive.png");

    	// position setting
    	ModuleListScene::imageCntHiveIAPInfoItem++;
    	Size visibleSize = Director::getInstance()->getVisibleSize();		// 사용 x, 320 x 480 고정 중.
    	sprite->setPosition(Vec2(visibleSize.width * ModuleListScene::ui_HiveIAPInfoItemImage_x,
    			visibleSize.height * (1 - (ModuleListScene::ui_HiveIAPInfoItemImage_y
    					+ (ModuleListScene::ui_HiveIAPInfoItemImage_margin_y * (ModuleListScene::imageCntHiveIAPInfoItem - 1))))));
    	sprite->setScale(0.8);
    	Layer *popupLayer = (Layer *)this->getChildByTag(-20000);
    	popupLayer->addChild(sprite);
    	return;
    }
    request->setRequestType(cocos2d::network::HttpRequest::Type::GET);
    request->setResponseCallback(CC_CALLBACK_2(ModuleListScene::onRequestImgCompletedEx, this));
    request->setTag("InfoItem");
    HttpClient::getInstance()->send(request);
    request->release();
}

void ModuleListScene::onRequestImgCompleted(HttpClient *sender, HttpResponse *response)
{
	Size visibleSize = Director::getInstance()->getVisibleSize();		// 사용 x, 320 x 480 고정 중.

    log("In HelloWorld");
    log("AppDelegate::onHttpRequestCompleted - onHttpRequestCompleted BEGIN");
    if (!response)
    {
    	log("onHttpRequestCompleted - No Response");
        return;
    }

    log("onHttpRequestCompleted - Response code: %lu", response->getResponseCode());

    if (!response->isSucceed())
    {
        log("onHttpRequestCompleted - Response failed");
        log("onHttpRequestCompleted - Error buffer: %s", response->getErrorBuffer());
        return;
    }
//    log("onHttpRequestCompleted - Response code: %s", response->getResponseDataString());

    std::vector<char> *buffer = response->getResponseData();
    const char* file_char = buffer->data();
    log("onHttpRequestCompleted - Response code: %s", file_char);
    Image * image = new  Image ();
    image-> initWithImageData ( reinterpret_cast<const unsigned char*>(&(buffer->front())), buffer->size());
    Texture2D * texture = new  Texture2D ();
    texture-> initWithImage (image);
    Sprite * sprite = Sprite :: createWithTexture (texture);
    sprite->setPosition(Vec2(visibleSize.width/2, visibleSize.height / 2));
//    addChild(sprite);
    this->addChild(sprite);

    log("onHttpRequestCompleted height %f", sprite->getContentSize().height);

}

void ModuleListScene::onRequestImgCompletedEx(HttpClient *sender, HttpResponse *response)
{
	Size visibleSize = Director::getInstance()->getVisibleSize();		// 사용 x, 320 x 480 고정 중.

    log("AppDelegate::onHttpRequestCompleted - onHttpRequestCompleted BEGIN");
    if (!response)
    {
    	log("onHttpRequestCompleted - No Response");
        return;
    }

    log("onHttpRequestCompleted - Response code: %lu", response->getResponseCode());

    if (!response->isSucceed())
    {
        log("onHttpRequestCompleted - Response failed");
        log("onHttpRequestCompleted - Error buffer: %s", response->getErrorBuffer());
        return;
    }
//    log("onHttpRequestCompleted - Response code: %s", response->getResponseDataString());

    std::vector<char> *buffer = response->getResponseData();
    const char* file_char = buffer->data();
    log("onHttpRequestCompleted - Response code: %s", file_char);

    Image * image = new  Image ();
    image-> initWithImageData ( reinterpret_cast<const unsigned char*>(&(buffer->front())), buffer->size());
    Texture2D * texture = new  Texture2D ();
    texture-> initWithImage (image);
    Sprite * sprite = Sprite :: createWithTexture (texture);

    // position setting
    ModuleListScene::imageCntHiveIAPInfoItem++;
    sprite->setPosition(Vec2(visibleSize.width * ModuleListScene::ui_HiveIAPInfoItemImage_x,
    		visibleSize.height * (1 - (ModuleListScene::ui_HiveIAPInfoItemImage_y
    				+ (ModuleListScene::ui_HiveIAPInfoItemImage_margin_y * (ModuleListScene::imageCntHiveIAPInfoItem - 1))))));
    sprite->setScale(0.8);
    Layer *popupLayer = (Layer *)this->getChildByTag(-20000);
    popupLayer->addChild(sprite);

//    log("onHttpRequestCompleted height %f", sprite->getContentSize().height);
}

// test
size_t write_data(void *ptr, size_t size, size_t nmemb, FILE *stream) {
	size_t written = fwrite(ptr, size, nmemb, stream);
	return written;
}


void ModuleListScene::download() //c2ssinki 임시 삭제
{
//	CURL *curl;
//	FILE *fp;
//	CURLcode res;
//	char *url = "http://112.175.60.185/hiveiapimage/438/201607/1438677191_Sneakers(2).png";
//	char outfilename[FILENAME_MAX] = "1438677191_Sneakers(2).png";
//	curl = curl_easy_init();
//	if(curl) {
//		fp = fopen(outfilename, "wb");
//		curl_easy_setopt(curl, CURLOPT_URL, url);
//		curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION,  write_data);
//		curl_easy_setopt(curl, CURLOPT_WRITEDATA, fp);
//		res = curl_easy_perform(curl);
//
//		curl_easy_cleanup(curl);
//		fclose(fp);
//	}
}

// ********** Touch Event (TextField) ********** ********** ********** **********
bool ModuleListScene::onTextFieldAttachWithIME(TextFieldTTF *sender) {
	//log("### onTextFieldAttachWithIME  ###");
	//ParameterListScene::ui_isOpenedWindow = true;
	//ParameterListScene::isFocusedTextField = true;
	return TextFieldDelegate::onTextFieldAttachWithIME(sender);
}

bool ModuleListScene::onTextFieldDetachWithIME(TextFieldTTF *sender) {
	//log("### onTextFieldDetachWithIME  ###");
	//ParameterListScene::ui_isOpenedWindow = false;
	return TextFieldDelegate::onTextFieldDetachWithIME(sender);
}

bool ModuleListScene::onTextFieldInsertText(TextFieldTTF *sender,
		const char *text, size_t nLen) {
	return TextFieldDelegate::onTextFieldInsertText(sender, text, nLen);
}

bool ModuleListScene::onTextFieldDeleteBackward(
		TextFieldTTF *sender, const char *delText, size_t nLen) {
	return TextFieldDelegate::onTextFieldDeleteBackward(sender, delText, nLen);
}
// ********** ********** ********** ********** ********** ********** ********** ********** **********

