//
//  DefaultSDKv1Parameters.hpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/10.
//

#ifndef DefaultSDKv1Parameters_hpp
#define DefaultSDKv1Parameters_hpp

#include "../../src/C2SModuleApi.h"
#include "../../src/C2SModuleErrorCode.h"
#include <stdio.h>
#include <string>
#include <set>
#include <vector>
#include <algorithm>
#include <map>


using namespace std;

// 응답, 결과 등으로 획득된 현재 식별자 정보
class CurrentValue {

private:
    //
    string vid;
    string uid;
    string did;
    string sessionKey;
public:
    /**
     * set
     */
    //
    void setVid(string value) { vid = value; }
    void setUid(string value) { uid = value; }
    void setDid(string value) { did = value; }
    void setSessionKey(string value) { sessionKey = value; }

    /**
     * other
     */
    void writeCommonParametersToFile();
    void readCommonParametersFromFile();

    /**
     * get
     */
    //
    string getVid() { return vid; }
    string getUid() { return uid; }
    string getDid() { return did; }
    string getSessionKey() { return sessionKey; }
};

// MARK : InitializeParameters
class InitializeParameters {

private:
    /* Default Parameter
     개발자 사이트 설명 기준
     */
    string server = "live";
    bool showLog = true;
    string appIdForComboBox = "com.com2us.ninepb3d.normal.freefull.google.global.android.common";
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    string appId = "com.com2us.hivesdk.normal.freefull.apple.global.ios.universal";
#else
    string appId = "com.com2us.hivesdk.normal.freefull.google.global.android.common";
#endif
    bool permissionWriteExternalStorage = false;
    string facebookPermissions = "";
    string serverId = "KR";
    bool showNotice = true;
    bool useHivePromotion = true;
    string gameLanguage = "ko";
    int timeoutSeconds = 5;
    bool showPermission = true;
    bool useImageBanner = true;
    bool useHercules = true;
    string hiveOrientation = "all";
    string channel = "C2S";
    //
    string guestVid = "123456789";
    string guestSessionToken = "abcdef1234567";
    //
    string did = "";
    bool isAuthorized = false;
    string autoLoginType = "";

public:
    void setServer(string value) { server = value; }
    void setShowLog(bool value) { showLog = value; }
    void setAppIdForComboBox(string value) { appIdForComboBox = value; }
    void setAppId(string value) { appId = value; }
    void setPermissionWriteExternalStorage(bool value) { permissionWriteExternalStorage = value; }
    void setFacebookPermissions(string value) { facebookPermissions = value; }
    void setServerId(string value) { serverId = value; }
    void setShowNotice(bool value) { showNotice = value; }
    void setUseHivePromotion(bool value) { useHivePromotion = value; }
    void setGameLanguage(string value) { gameLanguage = value; }
    void setTimeoutSeconds(int value) { timeoutSeconds = value; }
    void setShowPermission(bool value) { showPermission = value; }
    void setUseImageBanner(bool value) { useImageBanner = value; }
    void setUseHercules(bool value) { useHercules = value; }
    void setHiveOrientation(string value) { hiveOrientation = value; }
    void setChannel(string value) { channel = value; }
    //
    void setGuestVid(string value) { guestVid = value; }
    void setGuestSessionToken(string value) { guestSessionToken = value; }
    //
    void setDid(string value) { did = value; }
    void setIsAuthorized(bool value) { isAuthorized = value; }
    void setAutoLoginType(string value) { autoLoginType = value; }


    string getServer() { return server; }
    bool getShowLog() { return showLog; }
    string getAppIdForComboBox() { return appIdForComboBox; }
    string getAppId() { return appId; }
    bool getPermissionWriteExternalStorage() { return permissionWriteExternalStorage; }
    string getFacebookPermissions() { return facebookPermissions; }
    string getServerId() { return serverId; }
    bool getShowNotice() { return showNotice; }
    bool getUseHivePromotion() { return useHivePromotion; }
    string getGameLanguage() { return gameLanguage; }
    int getTimeoutSeconds() { return timeoutSeconds; }
    bool getShowPermission() { return showPermission; }
    bool getUseImageBanner() { return useImageBanner; }
    bool getUseHercules() { return useHercules; }
    string getHiveOrientation() { return hiveOrientation; }
    string getChannel() { return channel; }
    //
    string getGuestVid() { return guestVid; }
    string getGuestSessionToken() { return guestSessionToken; }
    //
    string getDid() { return did; }
    bool getIsAuthorized() { return isAuthorized; }
    string getAutoLoginType() { return autoLoginType; }
};

// MARK : LoginParameters
class LoginParameters {

private:
    /* Default Parameter
     개발자 사이트 설명 기준
     */
    string guestVid = "";
    string hiveVid = "";
    string hiveUid = "";
    string vidSessionToken = "";
    string currentLoginType = "";
    string customSelectTarget = "use_hive_user_info"; // use_hive_user_info, use_guest_user_info

public:
    void setGuestVid(string value) { guestVid = value; }
    void setHiveVid(string value) { hiveVid = value; }
    void setHiveUid(string value) { hiveUid = value; }
    void setVidSessionToken(string value) { vidSessionToken = value; }
    void setCurrentLoginType(string value) { currentLoginType = value; }
    void setCustomSelectTarget(string value) { customSelectTarget = value; }


    string getGuestVid() { return guestVid; }
    string getHiveVid() { return hiveVid; }
    string getHiveUid() { return hiveUid; }
    string getVidSessionToken() { return vidSessionToken; }
    string getCurrentLoginType() { return currentLoginType; }
    string getCustomSelectTarget() { return customSelectTarget; }
};

// MARK : PushMessageParameters
class PushMessageParameters {

private:
    /* Default Parameter
     개발자 사이트 설명 기준
     */
    int pushId = 12345;
    string title = "CocosSDKv1_PushTitle";
    string message = "CocosSDKv1_PushMessage";
    int after = 5;
    string type = "bar";       // "bar", "popup", "bar,popup"
    string groupId = "hivev1";

    //
    // true, false, null
    string usePush = "true";
    string sound = "true";
    string vibrate = "true";
    string operationGCM = "true";           // android
    string operationRemote = "true";        // ios
    string operationLocal = "true";
    string useNotice = "true";
    string useNight = "true";

public:
    void setPushId(int value) { pushId = value; }
    void setTitle(string value) { title = value; }
    void setMessage(string value) { message = value; }
    void setAfter(int value) { after = value; }
    void setType(string value) { type = value; }
    void setGroupId(string value) { groupId = value; }
    //
    void setUsePush(string value) { usePush = value; }
    void setSound(string value) { sound = value; }
    void setVibrate(string value) { vibrate = value; }
    void setOperationGCM(string value) { operationGCM = value; }
    void setOperationRemote(string value) { operationRemote = value; }
    void setOperationLocal(string value) { operationLocal = value; }
    void setUseNotice(string value) { useNotice = value; }
    void setUseNight(string value) { useNight = value; }


    int getPushId() { return pushId; }
    string getTitle() { return title; }
    string getMessage() { return message; }
    int getAfter() { return after; }
    string getType() { return type; }
    string getGroupId() { return groupId; }
    //
    string getUsePush() { return usePush; }
    string getSound() { return sound; }
    string getVibrate() { return vibrate; }
    string getOperationGCM() { return operationGCM; }
    string getOperationRemote() { return operationRemote; }
    string getOperationLocal() { return operationLocal; }
    string getUseNotice() { return useNotice; }
    string getUseNight() { return useNight; }
};

// MARK : UserInfoParameters
class UserInfoParameters {

private:
    /* Default Parameter
     개발자 사이트 설명 기준
     */
    string useMyVid = "myVid";  // "myVid", "otherVid"
    string userVid = "123456789";
    string facebookId = "facebook1234abcd";

public:
    void setUseMyVid(string value) { useMyVid = value; }
    void setUserVid(string value) { userVid = value; }
    void setFacebookId(string value) { facebookId = value; }


    string getUseMyVid() { return useMyVid; }
    string getUserVid() { return userVid; }
    string getFacebookId() { return facebookId; }
};

// MARK : SnsPostParameters
class SnsPostParameters {

private:
    /* Default Parameter
     개발자 사이트 설명 기준
     */
    string facebookAppLinkUrl = "https://www.withhive.com";
    string googlePlusMessage = "play withhive";

public:
    void setFacebookAppLinkUrl(string value) { facebookAppLinkUrl = value; }
    void setGooglePlusMessage(string value) { googlePlusMessage = value; }


    string getFacebookAppLinkUrl() { return facebookAppLinkUrl; }
    string getGooglePlusMessage() { return googlePlusMessage; }
};

// MARK : PromotionV1Parameters
class PromotionV1Parameters {

private:
    /* Default Parameter
     개발자 사이트 설명 기준
     */
    string viewId = "310000";

public:
    void setViewId(string value) { viewId = value; }


    string getViewId() { return viewId; }
};

// MARK : PromotionV1InfoParameters
class PromotionV1InfoParameters {

private:
    /* Default Parameter
     개발자 사이트 설명 기준
     */
    string viewId = "310000";
    string campaignType = "all";  // "all", "event", "notice", "cross", "custom input"
    string bannerType = "great"; // "great", "small", "rolling", "custom input"
    string campaignTypeCustom = "";
    string bannerTypeCustom = "";

public:
    void setViewId(string value) { viewId = value; }
    void setCampaignType(string value) { campaignType = value; }
    void setBannerType(string value) { bannerType = value; }
    void setCampaignTypeCustom(string value) { campaignTypeCustom = value; }
    void setBannerTypeCustom(string value) { bannerTypeCustom = value; }


    string getViewId() { return viewId; }
    string getCampaignType() { return campaignType; }
    string getBannerType() { return bannerType; }
    string getCampaignTypeCustom() { return campaignTypeCustom; }
    string getBannerTypeCustom() { return bannerTypeCustom; }
};

// MARK : UserAcquisitionParameters
class UserAcquisitionParameters {

private:
    /* Default Parameter
     개발자 사이트 설명 기준
     */
    string additionalInfo = "{\"server\":\"0\",\"character\":\"0\"}";
    string qrCodeData = "";
    bool needShowQRCodePopup = false;
    string message = "Hello, invite you with hive ua!";
    string linkShareUrl = "";
    // url 사용 문제가 있을 경우 다른 프로필 이미지 url을 example 데이터로 활용
    string imageURL = "http://image-glb.qpyou.cn/hubweb/hive_img/U/P/122349090/20151028/82f610b6f4590863934cefb2b875c87a.jpg";
    string thumbnailURL = "http://image-glb.qpyou.cn/hubweb/hive_img/U/P/122349090/20151028/82f610b6f4590863934cefb2b875c87a.jpg";
    bool useVid = true; // false : use uid
    string vidOrUid = "123456789";
    bool usePush = true;
    bool invite = false;
    bool linkShare = true;

public:
    void setAdditionalInfo(string value) { additionalInfo = value; }
    void setQrCodeData(string value) { qrCodeData = value; }
    void setNeedShowQRCodePopup(bool value) { needShowQRCodePopup = value; }
    void setMessage(string value) { message = value; }
    void setLinkShareUrl(string value) { linkShareUrl = value; }
    void setImageURL(string value) { imageURL = value; }
    void setThumbnailURL(string value) { thumbnailURL = value; }
    void setUseVid(bool value) { useVid = value; }
    void setVidOrUid(string value) { vidOrUid = value; }
    void setUsePush(bool value) { usePush = value; }
    void setInvite(bool value) { invite = value; }
    void setLinkShare(bool value) { linkShare = value; }


    string getAdditionalInfo() { return additionalInfo; }
    string getQrCodeData() { return qrCodeData; }
    bool getNeedShowQRCodePopup() { return needShowQRCodePopup; }
    string getMessage() { return message; }
    string getLinkShareUrl() { return linkShareUrl; }
    string getImageURL() { return imageURL; }
    string getThumbnailURL() { return thumbnailURL; }
    bool getUseVid() { return useVid; }
    string getVidOrUid() { return vidOrUid; }
    bool getUsePush() { return usePush; }
    bool getInvite() { return invite; }
    bool getLinkShare() { return linkShare; }
};

// MARK : FriendsParameters
class FriendsParameters {

private:
    /* Default Parameter
     개발자 사이트 설명 기준
     */
    int limit = 0;
    string message = "Hello, invite you with Hive Message!";
    // url 사용 문제가 있을 경우 다른 프로필 이미지 url을 example 데이터로 활용
    string imageURL = "http://image-glb.qpyou.cn/hubweb/hive_img/U/P/122349090/20151028/82f610b6f4590863934cefb2b875c87a.jpg";
    string thumbnailURL = "http://image-glb.qpyou.cn/hubweb/hive_img/U/P/122349090/20151028/82f610b6f4590863934cefb2b875c87a.jpg";
    bool useVid = true; // false : use uid
    string vidOrUid = "123456789";
    bool usePush = true;
    bool invite = false;

public:
    void setLimit(int value) { limit = value; }
    void setMessage(string value) { message = value; }
    void setImageURL(string value) { imageURL = value; }
    void setThumbnailURL(string value) { thumbnailURL = value; }
    void setUseVid(bool value) { useVid = value; }
    void setVidOrUid(string value) { vidOrUid = value; }
    void setUsePush(bool value) { usePush = value; }
    void setInvite(bool value) { invite = value; }


    int getLimit() { return limit; }
    string getMessage() { return message; }
    string getImageURL() { return imageURL; }
    string getThumbnailURL() { return thumbnailURL; }
    bool getUseVid() { return useVid; }
    string getVidOrUid() { return vidOrUid; }
    bool getUsePush() { return usePush; }
    bool getInvite() { return invite; }
};

// MARK : HiveWebviewParameters
class HiveWebviewParameters {

private:
    /* Default Parameter
     개발자 사이트 설명 기준
     */
    string additionalInfo = "{\"server\":\"0\",\"character\":\"0\"}";
    string uid = "123456789";
//    string androidPermission = "{\"requests\":[\"android.permission.READ_PHONE_STATE\",\"android.permission.WRITE_EXTERNAL_STORAGE\"]}";

public:
    void setAdditionalInfo(string value) { additionalInfo = value; }
    void setUid(string value) { uid = value; }
//    void setAndroidPermission(string value) { androidPermission = value; }


    string getAdditionalInfo() { return additionalInfo; }
    string getUid() { return uid; }
//    string getAndroidPermission() { return androidPermission; }
};

// MARK : TrackingParameters
class TrackingParameters {

private:
    /* Default Parameter
     개발자 사이트 설명 기준
     */
    string event = "TutorialComplete";
    bool enableAdjust = true;
    bool enableSingular = true;
    bool enableAppsFlyer = true;

public:
    void setEvent(string value) { event = value; }
    void setEnableAdjust(bool value) { enableAdjust = value; }
    void setEnableSingular(bool value) { enableSingular = value; }
    void setEnableAppsFlyer(bool value) { enableAppsFlyer = value; }


    string getEvent() { return event; }
    bool getEnableAdjust() { return enableAdjust; }
    bool getEnableSingular() { return enableSingular; }
    bool getEnableAppsFlyer() { return enableAppsFlyer; }
};

// MARK : UserEngagementParameters
class UserEngagementParameters {

private:
    /* Default Parameter
     개발자 사이트 설명 기준
     */
    string couponId = "COUPONTEST";
    string additionalInfo = "{\"server\":\"0\",\"character\":\"0\"}";

public:
    void setCouponId(string value) { couponId = value; }
    void setAdditionalInfo(string value) { additionalInfo = value; }


    string getCouponId() { return couponId; }
    string getAdditionalInfo() { return additionalInfo; }
};

// MARK : PromotionV2Parameters
class PromotionV2Parameters {

private:
    /* Default Parameter
     개발자 사이트 설명 기준
     */
    bool forcedNews = false;
    bool forcedBanner = false;
    bool forcedNotice = false;
    string contentKeyCustomView = "310000";
    string contentKeyCustomBoard = "200007";
    string contentKeySpot = "1004";
    string contentKeyDirect = "209019";

public:
    void setForcedNews(bool value) { forcedNews = value; }
    void setForcedBanner(bool value) { forcedBanner = value; }
    void setForcedNotice(bool value) { forcedNotice = value; }
    void setContentKeyCustomView(string value) { contentKeyCustomView = value; }
    void setContentKeyCustomBoard(string value) { contentKeyCustomBoard = value; }
    void setContentKeySpot(string value) { contentKeySpot = value; }
    void setContentKeyDirect(string value) { contentKeyDirect = value; }


    bool getForcedNews() { return forcedNews; }
    bool getForcedBanner() { return forcedBanner; }
    bool getForcedNotice() { return forcedNotice; }
    string getContentKeyCustomView() { return contentKeyCustomView; }
    string getContentKeyCustomBoard() { return contentKeyCustomBoard; }
    string getContentKeySpot() { return contentKeySpot; }
    string getContentKeyDirect() { return contentKeyDirect; }
};

// MARK : PromotionV2InfoParameters
class PromotionV2InfoParameters {

private:
    /* Default Parameter
     개발자 사이트 설명 기준
     */
    string customViewContentKey = "310000";
    string customBoardContentKey = "200007";
    string spotContentKey = "1004";
    string directContentKey = "209019";
    string additionalInfo = "{\"server\":\"0\",\"character\":\"0\"}";
    string campaignType = "all";  // "all", "event", "notice", "cross", "custom input"
    string bannerType = "great"; // "great", "small", "rolling", "custom input"
    string campaignTypeCustom = "";
    string bannerTypeCustom = "";

public:
    void setCustomViewContentKey(string value) { customViewContentKey = value; }
    void setCustomBoardContentKey(string value) { customBoardContentKey = value; }
    void setSpotContentKey(string value) { spotContentKey = value; }
    void setDirectContentKey(string value) { directContentKey = value; }
    void setAdditionalInfo(string value) { additionalInfo = value; }
    void setCampaignType(string value) { campaignType = value; }
    void setBannerType(string value) { bannerType = value; }
    void setCampaignTypeCustom(string value) { campaignTypeCustom = value; }
    void setBannerTypeCustom(string value) { bannerTypeCustom = value; }


    string getCustomViewContentKey() { return customViewContentKey; }
    string getCustomBoardContentKey() { return customBoardContentKey; }
    string getSpotContentKey() { return spotContentKey; }
    string getDirectContentKey() { return directContentKey; }
    string getAdditionalInfo() { return additionalInfo; }
    string getCampaignType() { return campaignType; }
    string getBannerType() { return bannerType; }
    string getCampaignTypeCustom() { return campaignTypeCustom; }
    string getBannerTypeCustom() { return bannerTypeCustom; }
};

// MARK : SetGameInfoParameters
class SetGameInfoParameters {

private:
    /* Default Parameter
     개발자 사이트 설명 기준
     */
    string serverId = "KR";
    string gameLanguage = "ko";
    bool emergencyMode = false;
    string channel = "C2S";
    string funnelsId = "700";
    string optionTag = "";

public:
    void setServerId(string value) { serverId = value; }
    void setGameLanguage(string value) { gameLanguage = value; }
    void setEmergencyMode(bool value) { emergencyMode = value; }
    void setChannel(string value) { channel = value; }
    void setFunnelsId(string value) { funnelsId = value; }
    void setOptionTag(string value) { optionTag = value; }


    string getServerId() { return serverId; }
    string getGameLanguage() { return gameLanguage; }
    bool getEmergencyMode() { return emergencyMode; }
    string getChannel() { return channel; }
    string getFunnelsId() { return funnelsId; }
    string getOptionTag() { return optionTag; }
};

// MARK : IAPV4Parameters
class IAPV4Parameters {

private:
    /* Default Parameter
     개발자 사이트 설명 기준
     */
    string type = "consumable"; // consumable, subscription, market(iOS Only)
    string marketPidList = "";  // (iOS Only)
    string pid = "custom_pid";
    string customPid = "";
    string postId = "";
    string iapPayload = "";
    int categoryId = 3;    // (Android Only)

public:
    vector<string> pidList = { "custom_pid" };

    void setType(string value) { type = value; }
    void setMarketPidList(string value) { marketPidList = value; }
    void setPid(string value) { pid = value; }
    void setCustomPid(string value) { customPid = value; }
    void setPostId(string value) { postId = value; }
    void setIapPayload(string value) { iapPayload = value; }
    void setCategoryId(int value) { categoryId = value; }

    string getType() { return type; }
    string getMarketPidList() { return marketPidList; }
    string getPid() { return pid; }
    string getCustomPid() { return customPid; }
    string getPostId() { return postId; }
    string getIapPayload() { return iapPayload; }
    int getCategoryId() { return categoryId; }
};

#endif /* DefaultSDKv1Parameters_hpp */
