//
//  FriendsParamScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/19.
//

#include "FriendsParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include <map>

using namespace::std;
using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;


bool FriendsParamScene::init()
{
    if(!BaseView::init())
        return false;

    addButton("OK", BUTTON_OK);

    scheduleUpdate();

    return true;
}

void FriendsParamScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void FriendsParamScene::onButtonEvent(int buttonTag)
{
    if (buttonTag != BUTTON_OK) {
        return;
    }

    setCompletionHandler();
    this->popView();
}

void FriendsParamScene::showParameters(int tag)
{
    api = static_cast<SDKv1FuncList::FriendsSceneFunc>(tag);
    switch (api) {
        case SDKv1FuncList::FRIENDS_SCENE_FACEBOOK_INVITATION_USER: {
            showFacebookInvitationUserParam([=]() {
                // limit
                int limit = stoi(getEditboxValue(EDITBOX_FACEBOOK_INVITATION_USER_LIMIT));
                ParameterSDKv1DataManager::getInstance()->friendsParameters->setLimit(limit);
            });
            break;
        }
        case SDKv1FuncList::FRIENDS_SCENE_FACEBOOK_SEND_MESSAGE: {
            showFacebookMessageParam([=]() {
                // message
                string message = getEditboxValue(EDITBOX_FACEBOOK_FRIENDS_MESSAGE);
                ParameterSDKv1DataManager::getInstance()->friendsParameters->setMessage(message);
            });
            break;
        }
        case SDKv1FuncList::FRIENDS_SCENE_HIVE_SEND_MESSAGE: {
            showHiveMessageParam([=]() {
                // useVid
                bool useVid = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_USE_VID_OR_UID));
                ParameterSDKv1DataManager::getInstance()->friendsParameters->setUseVid(useVid);

                // vid (or uid)
                string vidOruid = getEditboxValue(EDITBOX_FRIENDS_VID_OR_UID);
                ParameterSDKv1DataManager::getInstance()->friendsParameters->setVidOrUid(vidOruid);

                // message
                string message = getEditboxValue(EDITBOX_FACEBOOK_FRIENDS_MESSAGE);
                ParameterSDKv1DataManager::getInstance()->friendsParameters->setMessage(message);

                // imageURL
                string imageURL = getEditboxValue(EDITBOX_FRIENDS_IMAGE_URL);
                ParameterSDKv1DataManager::getInstance()->friendsParameters->setImageURL(imageURL);

                // thumbnailURL
                string thumbnailURL = getEditboxValue(EDITBOX_FRIENDS_THUMBNAIL_URL);
                ParameterSDKv1DataManager::getInstance()->friendsParameters->setThumbnailURL(thumbnailURL);

                // usePush
                bool usePush = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_FRIENDS_USEPUSH));
                ParameterSDKv1DataManager::getInstance()->friendsParameters->setUsePush(usePush);

                // invite
                bool invite = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_FRIENDS_INVITE));
                ParameterSDKv1DataManager::getInstance()->friendsParameters->setInvite(invite);
            });
            break;
        }
        default:
            break;
    }
}

void FriendsParamScene::showFacebookInvitationUserParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- Facebook 초대 유저 -");
    setCompletionHandler = completionHandler;

    // limit
    string limit = to_string(ParameterSDKv1DataManager::getInstance()->friendsParameters->getLimit());
    addEditBox("limit : ", limit.c_str(), "", EDITBOX_FACEBOOK_INVITATION_USER_LIMIT);
}

void FriendsParamScene::showFacebookMessageParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- Facebook 메시지 전송 -");
    setCompletionHandler = completionHandler;

    // message
    string message = ParameterSDKv1DataManager::getInstance()->friendsParameters->getMessage();
    addEditBox("message : ", message.c_str(), "", EDITBOX_FACEBOOK_FRIENDS_MESSAGE);
}

void FriendsParamScene::showHiveMessageParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- Hive 메시지 전송 -");
    setCompletionHandler = completionHandler;

    // useVid( or Uid )
    int useVid = int(ParameterSDKv1DataManager::getInstance()->friendsParameters->getUseVid());
    addTrueFalseToggleButton("use vid(or uid) : ", useVid, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_USE_VID_OR_UID);

    // vid ( or uid)
    string vidOrUid = ParameterSDKv1DataManager::getInstance()->friendsParameters->getVidOrUid();
    addEditBox("vid(or uid) : ", vidOrUid.c_str(), "", EDITBOX_FRIENDS_VID_OR_UID);

    // message
    string message = ParameterSDKv1DataManager::getInstance()->friendsParameters->getMessage();
    addEditBox("message : ", message.c_str(), "", EDITBOX_FACEBOOK_FRIENDS_MESSAGE);

    // imageURL
    string imageURL = ParameterSDKv1DataManager::getInstance()->friendsParameters->getImageURL();
    addEditBox("imageURL : ", imageURL.c_str(), "", EDITBOX_FRIENDS_IMAGE_URL);

    // thumbnailURL
    string thumbnailURL = ParameterSDKv1DataManager::getInstance()->friendsParameters->getThumbnailURL();
    addEditBox("thumbnailURL : ", thumbnailURL.c_str(), "", EDITBOX_FRIENDS_THUMBNAIL_URL);

    // usePush
    int usePush = int(ParameterSDKv1DataManager::getInstance()->friendsParameters->getUsePush());
    addTrueFalseToggleButton("usePush : ", usePush, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_FRIENDS_USEPUSH);

    // invite
    int invite = int(ParameterSDKv1DataManager::getInstance()->friendsParameters->getInvite());
    addTrueFalseToggleButton("invite : ", invite, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_FRIENDS_INVITE);
}
