//
//  IAPV4ParamScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/28.
//

#include "IAPV4ParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include <map>

using namespace::std;
using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;


bool IAPV4ParamScene::init()
{
    if(!BaseView::init())
        return false;

    addButton("OK", BUTTON_OK);

    scheduleUpdate();

    return true;
}

void IAPV4ParamScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void IAPV4ParamScene::onButtonEvent(int buttonTag)
{
    if (buttonTag != BUTTON_OK) {
        return;
    }

    setCompletionHandler();
    this->popView();
}

void IAPV4ParamScene::showParameters(int tag)
{
    api = static_cast<SDKv1FuncList::InAppV4SceneFunc>(tag);
    switch (api) {
        case SDKv1FuncList::INAPPV4_SCENE_ITEM_INFO: {
            showItemInfoParam([=]() {
                // type
                string type = getComboboxValue(COMBOBOX_TYPE);
                ParameterSDKv1DataManager::getInstance()->iapv4Parameters->setType(type);

                // marketPidList
                string marketPidList = getEditboxValue(EDITBOX_MARKET_PID_LIST);
                ParameterSDKv1DataManager::getInstance()->iapv4Parameters->setMarketPidList(marketPidList);
            });
            break;
        }
        case SDKv1FuncList::INAPPV4_SCENE_PURCHASE: {
            showPurchaseParam([=]() {
                // type
                string type = getComboboxValue(COMBOBOX_TYPE);
                ParameterSDKv1DataManager::getInstance()->iapv4Parameters->setType(type);

                // pid
                string pid = getComboboxValue(COMBOBOX_PID);
                ParameterSDKv1DataManager::getInstance()->iapv4Parameters->setPid(pid);

                // customPid
                string customPid = getEditboxValue(EDITBOX_PID);
                ParameterSDKv1DataManager::getInstance()->iapv4Parameters->setCustomPid(customPid);

                // iapPayload
                string iapPayload = getEditboxValue(EDITBOX_IAPPAYLOAD);
                ParameterSDKv1DataManager::getInstance()->iapv4Parameters->setIapPayload(iapPayload);
            });
            break;
        }
        case SDKv1FuncList::INAPPV4_SCENE_RESTORE: {
            showRestoreParam([=]() {
                // type
                string type = getComboboxValue(COMBOBOX_TYPE);
                ParameterSDKv1DataManager::getInstance()->iapv4Parameters->setType(type);
            });
            break;
        }
        case SDKv1FuncList::INAPPV4_SCENE_TRANSACTION_FINISH: {
            showTransactionFinishParam([=]() {
                // pid
                string pid = getComboboxValue(COMBOBOX_PID);
                ParameterSDKv1DataManager::getInstance()->iapv4Parameters->setPid(pid);

                // customPid
                string customPid = getEditboxValue(EDITBOX_PID);
                ParameterSDKv1DataManager::getInstance()->iapv4Parameters->setCustomPid(customPid);
            });
            break;
        }
        case SDKv1FuncList::INAPPV4_SCENE_POSTBOX_RECEIVE: {
            showPostBoxReceiveParam([=]() {
                // postId
                string postId = getEditboxValue(EDITBOX_POST_ID);
                ParameterSDKv1DataManager::getInstance()->iapv4Parameters->setPostId(postId);
            });
            break;
        }
        case SDKv1FuncList::INAPPV4_SCENE_ANDROID_SHOW_INAPP_MESSAGES: {
            showShowInAppMessagesParam([=]() {
                // categoryId
                int categoryId = stoi(getEditboxValue(EDITBOX_CATEGORY_ID));
                ParameterSDKv1DataManager::getInstance()->iapv4Parameters->setCategoryId(categoryId);
            });
            break;
        }
        default:
            break;
    }
}

void IAPV4ParamScene::showItemInfoParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- 아이템 정보 -");
    setCompletionHandler = completionHandler;

    // type
    vector<const char*> typeList = { "consumable",
                                       "subscription",
                                       "market"};
    string type = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getType();
    int findIndex = static_cast<int>(find(typeList.begin(), typeList.end(),
                                          type) - typeList.begin());

    addCombobox("type : ", typeList, findIndex, COMBOBOX_TYPE);

    // marketPidList
    string marketPidList = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getMarketPidList();
    addEditBox("marketPidList : ", marketPidList.c_str(), "", EDITBOX_MARKET_PID_LIST);
}

void IAPV4ParamScene::showPurchaseParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- 구매하기 -");
    setCompletionHandler = completionHandler;

    // type
    vector<const char*> typeList = { "consumable",
                                     "subscription"};
    string type = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getType();
    int findIndex = static_cast<int>(find(typeList.begin(), typeList.end(),
                                          type) - typeList.begin());

    addCombobox("type : ", typeList, findIndex, COMBOBOX_TYPE);

    // pid (list)
    vector<const char*> tempPidList;
    for(int i = 0; i < ParameterSDKv1DataManager::getInstance()->iapv4Parameters->pidList.size(); i++) {
        tempPidList.push_back(ParameterSDKv1DataManager::getInstance()->iapv4Parameters->pidList[i].c_str());
    }
    string pid = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getPid();
    int findIndex2 = static_cast<int>(find(tempPidList.begin(), tempPidList.end(),
                                           pid) - tempPidList.begin());

    addCombobox("pid(major) : ", tempPidList, findIndex2, COMBOBOX_PID);

    // customPid
    string customPid = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getCustomPid();
    addEditBox("pid(custom)(minor) : ", customPid.c_str(), "", EDITBOX_PID);

    // iapPayload
    string iapPayload = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getIapPayload();
    addEditBox("iapPayload : ", iapPayload.c_str(), "", EDITBOX_IAPPAYLOAD);
}

void IAPV4ParamScene::showRestoreParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- 구매복구 -");
    setCompletionHandler = completionHandler;

    // type
    vector<const char*> typeList = { "consumable",
                                     "subscription"};
    string type = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getType();
    int findIndex = static_cast<int>(find(typeList.begin(), typeList.end(),
                                          type) - typeList.begin());

    addCombobox("type : ", typeList, findIndex, COMBOBOX_TYPE);
}


void IAPV4ParamScene::showTransactionFinishParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- 구매종료 -");
    setCompletionHandler = completionHandler;

    // pid (list)
    vector<const char*> tempPidList;
    for(int i = 0; i < ParameterSDKv1DataManager::getInstance()->iapv4Parameters->pidList.size(); i++) {
        tempPidList.push_back(ParameterSDKv1DataManager::getInstance()->iapv4Parameters->pidList[i].c_str());
    }
    string pid = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getPid();
    int findIndex2 = static_cast<int>(find(tempPidList.begin(), tempPidList.end(),
                                           pid) - tempPidList.begin());

    addCombobox("pid(major) : ", tempPidList, findIndex2, COMBOBOX_PID);

    // customPid
    string customPid = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getCustomPid();
    addEditBox("pid(custom)(minor) : ", customPid.c_str(), "", EDITBOX_PID);
}

void IAPV4ParamScene::showPostBoxReceiveParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- PostBox 아이템수령 -");
    setCompletionHandler = completionHandler;

    // postId
    string postId = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getPostId();
    addEditBox("post_id : ", postId.c_str(), "", EDITBOX_POST_ID);
}

void IAPV4ParamScene::showShowInAppMessagesParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- ShowInAppMessages -");
    setCompletionHandler = completionHandler;

    // categoryId
    int categoryId = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getCategoryId();
    addEditBox("categoryId : ", to_string(categoryId).c_str(), "", EDITBOX_CATEGORY_ID);
}
