//
//  IAPV4Scene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/24.
//

#include "SDKv1FuncList.hpp"
#include "IAPV4Scene.hpp"
#include "IAPV4ParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include "../../src/cocos2d-x/Module_Cocos2d-x.h"
#include "../../src/C2SModule.h"
#include "../../../Classes/Postbox.hpp"
#include <map>
#include <sstream>

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;
using namespace::std;


bool IAPV4Scene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- [SDK v1 InAppV4] -");

    addButton("아이템 정보\n(C2SModuleInappv4_ItemInfo)", ButtonMode::NEED_PARAM, SDKv1FuncList::INAPPV4_SCENE_ITEM_INFO);
    addButton("구매하기\n(C2SModuleInappv4_Purchase)", ButtonMode::NEED_PARAM, SDKv1FuncList::INAPPV4_SCENE_PURCHASE);
    addButton("복구하기\n(C2SModuleInappv4_Restore)", ButtonMode::NEED_PARAM, SDKv1FuncList::INAPPV4_SCENE_RESTORE);
    addButton("구매종료\n(C2SModuleInappv4_TransactionFinish)", ButtonMode::NEED_PARAM, SDKv1FuncList::INAPPV4_SCENE_TRANSACTION_FINISH);
    addButton("설정된 마켓 보기\n(C2SModuleInappv4_GetSelectedMarket)", SDKv1FuncList::INAPPV4_SCENE_GET_SELECTED_MARKET);
    addButton("Hive Account UUID\n(C2SModuleInappV4_GetAccountUuid)", SDKv1FuncList::INAPPV4_SCENE_HIVE_ACCOUNT_UUID);
    addButton("마켓선택\n(C2SModuleInappV4_SelectMarket)", SDKv1FuncList::INAPPV4_SCENE_SELECT_MARKET);
    addButton("러비잔액\n(C2SModuleInappV4_RequestBalance)", SDKv1FuncList::INAPPV4_SCENE_LEBI_BALANCE);
    addButton("러비충전\n(C2SModuleInappV4_ChargeBalance)", SDKv1FuncList::INAPPV4_SCENE_LEBI_CHARGE);
    addButton("PostBox 상태보기", SDKv1FuncList::INAPPV4_SCENE_POSTBOX_STATE);
    addButton("PostBox 아이템수령", ButtonMode::NEED_PARAM, SDKv1FuncList::INAPPV4_SCENE_POSTBOX_RECEIVE);
    addButton("구매 요청 확인\n(C2SModuleInappV4_CheckPromotePurchase)", SDKv1FuncList::INAPPV4_SCENE_IOS_CHECKREMOTE);
    addButton("마켓연결\n(C2SModuleInappV4_MarketConnect)", SDKv1FuncList::INAPPV4_SCENE_MARKET_CONNECT);
    addButton("ShowInAppMessages\n(C2SModuleInappV4_ShowInAppMessages)", ButtonMode::NEED_PARAM, SDKv1FuncList::INAPPV4_SCENE_ANDROID_SHOW_INAPP_MESSAGES);

    scheduleUpdate();

    return true;
}

void IAPV4Scene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void IAPV4Scene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        IAPV4ParamScene* paramScene = IAPV4ParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    SDKv1FuncList::InAppV4SceneFunc api = static_cast<SDKv1FuncList::InAppV4SceneFunc>(buttonTag);
    switch (api) {
        case SDKv1FuncList::INAPPV4_SCENE_ITEM_INFO: {
            string type = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getType();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value(type);

            // (iOS Only) market 인 경우 marketPidList 확인
            // ex : "marketPidList":"12345,6789,34567"
            if(type != "market") {
                string marketPidList = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getMarketPidList();
                picojson::value::array vectorMarketPidList;

                if(marketPidList != "") {
                    string delimiter = ",";

                    size_t pos = 0;
                    picojson::value token;
                    bool existDelimiter = false;
                    while ((pos = marketPidList.find(delimiter)) != string::npos) {
                        token = picojson::value(marketPidList.substr(0, pos));
                        vectorMarketPidList.push_back(token);

                        marketPidList.erase(0, pos + delimiter.length());
                        existDelimiter = true;
                    }

                    if(existDelimiter) {
                        // delimeter 구분된 데이터가 2개 이상인 경우
                        paramsDefault["marketPidList"] = picojson::value(vectorMarketPidList);
                    }
                    else {
                        // delimeter 구분이 없는 경우
                        vectorMarketPidList.push_back(picojson::value(marketPidList));
                        paramsDefault["marketPidList"] = picojson::value(vectorMarketPidList);
                    }
                }
            }

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleInappV4_ItemInfo(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleInappV4_ItemInfo", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleInappV4_ItemInfo", paramString);
            break;
        }
        case SDKv1FuncList::INAPPV4_SCENE_PURCHASE: {
            string type = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getType();
            string pid = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getPid();
            string customPid = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getCustomPid();
            string iapPayload = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getIapPayload();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value(type);
            if(pid != "custom_pid") {
                paramsDefault["pid"] = picojson::value(pid);
            }
            else {
                paramsDefault["pid"] = picojson::value(customPid);
            }
            paramsDefault["iapPayload"] = picojson::value(iapPayload);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleInappV4_Purchase(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleInappV4_Purchase", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleInappV4_Purchase", paramString);
            break;
        }
        case SDKv1FuncList::INAPPV4_SCENE_RESTORE: {
            string type = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getType();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value(type);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleInappV4_Restore(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleInappV4_Restore", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleInappV4_Restore", paramString);
            break;
        }
        case SDKv1FuncList::INAPPV4_SCENE_TRANSACTION_FINISH: {
            string pid = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getPid();
            string customPid = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getCustomPid();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            if(pid != "custom_pid") {
                paramsDefault["pid"] = picojson::value(pid);
            }
            else {
                paramsDefault["pid"] = picojson::value(customPid);
            }

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleInappV4_TransactionFinish(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleInappV4_TransactionFinish", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleInappV4_TransactionFinish", paramString);
            break;
        }
        case SDKv1FuncList::INAPPV4_SCENE_GET_SELECTED_MARKET: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleInappV4_GetSelectedMarket(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleInappV4_GetSelectedMarket", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleInappV4_GetSelectedMarket", paramString);
            break;
        }
        case SDKv1FuncList::INAPPV4_SCENE_HIVE_ACCOUNT_UUID: {
            /**
            * Set Argument
            */
            picojson::object paramsDefault;

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleInappV4_GetAccountUuid(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleInappV4_GetAccountUuid", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleInappV4_GetAccountUuid", paramString);
            break;
        }
        case SDKv1FuncList::INAPPV4_SCENE_SELECT_MARKET: {
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
            MessageBox("C2SModuleInappV4_SelectMarket", "Android만 제공되는 기능입니다!");
#else
            /**
            * Set Argument
            */
            picojson::object paramsDefault;

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleInappV4_SelectMarket(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleInappV4_SelectMarket", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleInappV4_SelectMarket", paramString);
#endif
            break;
        }
        case SDKv1FuncList::INAPPV4_SCENE_LEBI_BALANCE: {
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
            MessageBox("C2SModuleInappV4_RequestBalance", "Android만 제공되는 기능입니다!");
#else
            /**
            * Set Argument
            */
            picojson::object paramsDefault;

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleInappV4_RequestBalance(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleInappV4_RequestBalance", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleInappV4_RequestBalance", paramString);
#endif
            break;
        }
        case SDKv1FuncList::INAPPV4_SCENE_LEBI_CHARGE: {
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
            MessageBox("C2SModuleInappV4_ChargeBalance", "Android만 제공되는 기능입니다!");
#else
            /**
            * Set Argument
            */
            picojson::object paramsDefault;

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleInappV4_ChargeBalance(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleInappV4_ChargeBalance", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleInappV4_ChargeBalance", paramString);
#endif
            break;
        }
        case SDKv1FuncList::INAPPV4_SCENE_POSTBOX_STATE: {
            // postbox login
            ParameterSDKv1DataManager::getInstance()->resultDelegate->postBoxV1Login();
            MessageBox("PostBox 상태보기 호출", "ShowLog에서 PostBox 상태를 확인해주세요!");
            break;
        }
        case SDKv1FuncList::INAPPV4_SCENE_POSTBOX_RECEIVE: {
            // postbox receive
            string postId = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getPostId();
            ParameterSDKv1DataManager::getInstance()->resultDelegate->postBoxV1Receive(postId);
            MessageBox("PostBox 아이템수령 호출", "ShowLog에서 PostBox 상태를 확인해주세요!");
            break;
        }
        case SDKv1FuncList::INAPPV4_SCENE_IOS_CHECKREMOTE: {
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
            /**
            * Set Argument
            */
            picojson::object paramsDefault;

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleInappV4_CheckPromotePurchase(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleInappV4_CheckPromotePurchase", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleInappV4_CheckPromotePurchase", paramString);
#else
            MessageBox("C2SModuleInappV4_CheckPromotePurchase", "iOS만 제공되는 기능입니다!");
#endif
            break;
        }
        case SDKv1FuncList::INAPPV4_SCENE_MARKET_CONNECT: {
            /**
            * Set Argument
            */
            picojson::object paramsDefault;

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleInappV4_MarketConnect(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleInappV4_MarketConnect", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleInappV4_MarketConnect", paramString);
            break;
        }
        case SDKv1FuncList::INAPPV4_SCENE_ANDROID_SHOW_INAPP_MESSAGES: {
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
            MessageBox("C2SModuleInappV4_ShowInAppMessages", "Android만 제공되는 기능입니다!");
#else
            /**
            * Set Argument
            */
            picojson::object paramsDefault;
            int categoryId = ParameterSDKv1DataManager::getInstance()->iapv4Parameters->getCategoryId();

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleInappV4_ShowInAppMessages(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleInappV4_ShowInAppMessages", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleInappV4_ShowInAppMessages", paramString);
#endif
        }
        default:
            break;
    }
}
