//
//  InfoScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/12.
//

#include "SDKv1FuncList.hpp"
#include "InfoScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include "../../src/cocos2d-x/Module_Cocos2d-x.h"
#include "../../src/C2SModule.h"
#include <map>
#include <sstream>

#ifdef __ANDROID__
#include "JNISample.h"
#endif

USING_NS_CC;

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;
using namespace::std;


bool InfoScene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- [SDK v1 정보 보기] -");

    addButton("샘플 현재 정보 보기", SDKv1FuncList::INFO_SCENE_SAMPLE_INFO);

    scheduleUpdate();

    return true;
}

void InfoScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void InfoScene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
//        LoginParamScene* paramScene = LoginParamScene::create();
//        BaseView::pushView(paramScene);
//        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    SDKv1FuncList::InfoSceneFunc api = static_cast<SDKv1FuncList::InfoSceneFunc>(buttonTag);
    switch (api) {
        case SDKv1FuncList::INFO_SCENE_SAMPLE_INFO: {
            string guestVid = ParameterSDKv1DataManager::getInstance()->loginParameters->getGuestVid();
            string hiveVid = ParameterSDKv1DataManager::getInstance()->loginParameters->getHiveVid();
            string hiveUid = ParameterSDKv1DataManager::getInstance()->loginParameters->getHiveUid();
            string vidSessionToken = ParameterSDKv1DataManager::getInstance()->loginParameters->getVidSessionToken();
            string getLoginType = ::GetLoginType();
            string server = ParameterSDKv1DataManager::getInstance()->initializeParameters->getServer();
            string appId = ParameterSDKv1DataManager::getInstance()->initializeParameters->getAppId();
#ifdef __ANDROID__
            string facebookVer = getAndroidFacebookVersion();
            string adjustVer = getAndroidAdjustVersion();
            string singularVer = getAndroidSingularVersion();
            string appsFlyerVer = getAndroidAppsFlyerVersion();
#else
            // TODO : iOS에서 라이브러리를 통한 버전 참조 필요
            string facebookVer = "v12.3.1";
            string adjustVer = "v4.33.3";
            string singularVer = "v12.0.1";
            string appsFlyerVer = "v6.9.1";
#endif

            string info = "";
            info += ("-Guest VID : " + guestVid + "\n");
            info += ("-Hive VID : " + hiveVid + "\n");
            info += ("-Hive UID : " + hiveUid + "\n");
            info += ("-VID Session : " + vidSessionToken + "\n");
            info += ("-GetLoginType : " + getLoginType + "\n");
            info += ("-Server : " + server + "\n");
            info += ("-AppId : " + appId + "\n\n");

            info += ("3rd Party Ver.\n");
            info += ("- Facebook sdk: " + facebookVer + "\n");
            info += ("- Adjust sdk : " + adjustVer + "\n");
            info += ("- Singular sdk : " + singularVer + "\n");
            info += ("- AppsFlyer sdk : " + appsFlyerVer + "\n");

            MessageBox(info.c_str(), "현재 정보");
            break;
        }
        default:
            break;
    }
}
