//
//  InitializeParamScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/10.
//

#include "InitializeParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include <map>

using namespace::std;
using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;


bool InitializeParamScene::init()
{
    if(!BaseView::init())
        return false;

    addButton("OK", BUTTON_OK);

    scheduleUpdate();

    return true;
}

void InitializeParamScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void InitializeParamScene::onButtonEvent(int buttonTag)
{
    if (buttonTag != BUTTON_OK) {
        return;
    }

    setCompletionHandler();
    this->popView();
}

void InitializeParamScene::showParameters(int tag)
{
    api = static_cast<SDKv1FuncList::InitializeSceneFunc>(tag);
    switch (api) {
        case SDKv1FuncList::INITIALIZE_SCENE_INITIALIZE: {
            showInitializeParam([=]() {
                // server
                string server = getComboboxValue(COMBOBOX_SERVER);
                ParameterSDKv1DataManager::getInstance()->initializeParameters->setServer(server);

                // showLog
                bool showLog = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_SHOW_LOG));
                ParameterSDKv1DataManager::getInstance()->initializeParameters->setShowLog(showLog);

                // appId
                string appIdComboBox = getComboboxValue(COMBOBOX_APPID);
                ParameterSDKv1DataManager::getInstance()->initializeParameters->setAppIdForComboBox(appIdComboBox);

                // appId(custom)
                string appId = getEditboxValue(EDITBOX_APPID);
                if(appId == "") {
                    ParameterSDKv1DataManager::getInstance()->initializeParameters->setAppId(appIdComboBox);
                } else {
                    ParameterSDKv1DataManager::getInstance()->initializeParameters->setAppId(appId);
                }

                // PermissionWriteExternalStorage
                bool permissionWriteExternalStorage = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_PERMISSION_WRITE_EXTERNAL_STORAGE));
                ParameterSDKv1DataManager::getInstance()->initializeParameters->setPermissionWriteExternalStorage(permissionWriteExternalStorage);

                // facebook permission
                string facebookPermissions = getEditboxValue(EDITBOX_FACEBOOK_PERMISSIONS);
                ParameterSDKv1DataManager::getInstance()->initializeParameters->setFacebookPermissions(facebookPermissions);

                // serverId
                string serverId = getEditboxValue(EDITBOX_SERVER_ID);
                ParameterSDKv1DataManager::getInstance()->initializeParameters->setServerId(serverId);

                // showNotice
                bool showNotice = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_SHOW_NOTICE));
                ParameterSDKv1DataManager::getInstance()->initializeParameters->setShowNotice(showNotice);

                // use HivePromotion
                bool useHivePromotion = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_USE_HIVEPROMOTION));
                ParameterSDKv1DataManager::getInstance()->initializeParameters->setUseHivePromotion(useHivePromotion);

                // gameLanguage
                string gameLanguage = getEditboxValue(EDITBOX_GAME_LANGUAGE);
                ParameterSDKv1DataManager::getInstance()->initializeParameters->setGameLanguage(gameLanguage);

                // timeoutSeconds
                int timeoutSeconds = stoi(getEditboxValue(EDITBOX_TIMEOUT_SECONDS));    // string to int
                ParameterSDKv1DataManager::getInstance()->initializeParameters->setTimeoutSeconds(timeoutSeconds);

                // showPermission
                bool showPermission = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_SHOW_PERMISSION));
                ParameterSDKv1DataManager::getInstance()->initializeParameters->setShowPermission(showPermission);

                // useImageBanner
                bool useImageBanner = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_USE_IMAGE_BANNER));
                ParameterSDKv1DataManager::getInstance()->initializeParameters->setUseImageBanner(useImageBanner);

                // useHercules
                bool useHercules = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_USE_HERCULES));
                ParameterSDKv1DataManager::getInstance()->initializeParameters->setUseHercules(useHercules);

                // hiveOrientation
                string hiveOrientation = getComboboxValue(COMBOBOX_HIVE_ORIENTATION);
                ParameterSDKv1DataManager::getInstance()->initializeParameters->setHiveOrientation(hiveOrientation);

                // channel
                string channel = getEditboxValue(EDITBOX_CHANNEL);
                ParameterSDKv1DataManager::getInstance()->initializeParameters->setChannel(channel);
            });
            break;
        }
        case SDKv1FuncList::INITIALIZE_SCENE_SET_GUESTUSER_INFO: {
            showSetGuestUserInfoParam([=]() {
                // guestVid
                string guestVid = getEditboxValue(EDITBOX_GUEST_VID);
                ParameterSDKv1DataManager::getInstance()->initializeParameters->setGuestVid(guestVid);

                // guestSessionToken
                string guestSessionToken = getEditboxValue(EDITBOX_GUEST_SESSION_TOKEN);
                ParameterSDKv1DataManager::getInstance()->initializeParameters->setGuestSessionToken(guestSessionToken);
            });
            break;
        }
        default:
            break;
    }
}

void InitializeParamScene::showInitializeParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- 초기화 진행 -");
    setCompletionHandler = completionHandler;

    // server
    vector<const char*> serverList = { "live",
                                      "sandbox",
                                      "test" };
    string server = ParameterSDKv1DataManager::getInstance()->initializeParameters->getServer();
    int findIndexServer = static_cast<int>(find(serverList.begin(), serverList.end(), server) - serverList.begin());

    addCombobox("server : ", serverList, findIndexServer, COMBOBOX_SERVER);

    // showLog
    int showLog = int(ParameterSDKv1DataManager::getInstance()->initializeParameters->getShowLog());      // bool to int
    addTrueFalseToggleButton("showLog : ", showLog, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_SHOW_LOG);

    // appId (pre-set) (MLB9이닝스, 크리티카)
    vector<const char*> appIdList = { "com.com2us.ninepb3d.normal.freefull.google.global.android.common",
                                      "com.com2us.ninepb3d.normal.freefull.apple.global.ios.universal",
                                      "com.gamevil.kritikamobile.android.google.global.normal",
                                      "com.gamevil.kritikam.ios.apple.global.normal",
                                      "com.com2us.misample.normal.freefull.google.global.android.common",
                                      "com.com2us.hivesdk.normal.freefull.onestore.kr.android.common",
                                      "com.com2us.hivesdk.android.huawei.global.normal.huawei",
                                      "com.com2us.hivesdk.android.amazon.global.normal",
                                      "com.com2us.hivesdk.normal.freefull.google.global.android.common"};
    string appId = ParameterSDKv1DataManager::getInstance()->initializeParameters->getAppIdForComboBox();
    int findIndexAppId  = static_cast<int>(find(appIdList.begin(), appIdList.end(), appId) - appIdList.begin());

    addCombobox("appId(preset)\n(minor) : ", appIdList, findIndexAppId, COMBOBOX_APPID);

    // appId (custom)
    string appIdCustom = ParameterSDKv1DataManager::getInstance()->initializeParameters->getAppId();
    addEditBox("appId(custom)\n(major) : ", appIdCustom.c_str(), "", EDITBOX_APPID);

    // permissionWriteExternal
    int permissionWriteExternal = int(ParameterSDKv1DataManager::getInstance()->initializeParameters->getPermissionWriteExternalStorage());      // bool to int
    addTrueFalseToggleButton("permissionWriteExternal : ", permissionWriteExternal, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_PERMISSION_WRITE_EXTERNAL_STORAGE);

    // facebookPermissions
    string facebookPermissions = ParameterSDKv1DataManager::getInstance()->initializeParameters->getFacebookPermissions();
    addEditBox("facebookPermissions : ", facebookPermissions.c_str(), "", EDITBOX_FACEBOOK_PERMISSIONS);

    // serverId
    string serverId = ParameterSDKv1DataManager::getInstance()->initializeParameters->getServerId();
    addEditBox("serverId : ", serverId.c_str(), "", EDITBOX_SERVER_ID);

    // showNotice
    int showNotice = int(ParameterSDKv1DataManager::getInstance()->initializeParameters->getShowNotice());      // bool to int
    addTrueFalseToggleButton("showNotice : ", showNotice, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_SHOW_NOTICE);

    // useHivePromotion
    int useHivePromotion = int(ParameterSDKv1DataManager::getInstance()->initializeParameters->getUseHivePromotion());      // bool to int
    addTrueFalseToggleButton("useHivePromotion : ", useHivePromotion, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_USE_HIVEPROMOTION);

    // gameLanguage
    string gameLanguage = ParameterSDKv1DataManager::getInstance()->initializeParameters->getGameLanguage();
    addEditBox("gameLanguage : ", gameLanguage.c_str(), "", EDITBOX_GAME_LANGUAGE);

    // timeoutSeconds
    string timeoutSeconds = to_string(ParameterSDKv1DataManager::getInstance()->initializeParameters->getTimeoutSeconds());     // int to string
    addEditBox("timeoutSeconds : ", timeoutSeconds.c_str(), "", EDITBOX_TIMEOUT_SECONDS);

    // showPermission
    int showPermission = int(ParameterSDKv1DataManager::getInstance()->initializeParameters->getShowPermission());      // bool to int
    addTrueFalseToggleButton("showPermission : ", showPermission, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_SHOW_PERMISSION);

    // useImageBanner
    int useImageBanner = int(ParameterSDKv1DataManager::getInstance()->initializeParameters->getUseImageBanner());      // bool to int
    addTrueFalseToggleButton("useImageBanner : ", useImageBanner, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_USE_IMAGE_BANNER);

    // useHercules
    int useHercules = int(ParameterSDKv1DataManager::getInstance()->initializeParameters->getUseHercules());      // bool to int
    addTrueFalseToggleButton("useHercules : ", useHercules, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_USE_HERCULES);

    // HiveOrientation
    vector<const char*> orientationList = { "all",
                                      "portrait",
                                      "landscape" };
    string orientation = ParameterSDKv1DataManager::getInstance()->initializeParameters->getHiveOrientation();
    int findIndexOrientation = static_cast<int>(find(orientationList.begin(), orientationList.end(), orientation) - orientationList.begin());

    addCombobox("hiveOrientation : ", orientationList, findIndexOrientation, COMBOBOX_HIVE_ORIENTATION);

    // channel
    string channel = ParameterSDKv1DataManager::getInstance()->initializeParameters->getChannel();
    addEditBox("channel : ", channel.c_str(), "", EDITBOX_CHANNEL);
}

void InitializeParamScene::showSetGuestUserInfoParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SetGuestUserInfo (Hoolai 기능) -");
    setCompletionHandler = completionHandler;

    // guestVid
    string guestVid = ParameterSDKv1DataManager::getInstance()->initializeParameters->getGuestVid();
    addEditBox("guestVid : ", guestVid.c_str(), "", EDITBOX_GUEST_VID);

    // guestSessionToken
    string guestSessionToken = ParameterSDKv1DataManager::getInstance()->initializeParameters->getGuestSessionToken();
    addEditBox("guestSessionToken : ", guestSessionToken.c_str(), "", EDITBOX_GUEST_SESSION_TOKEN);
}
