//
//  InitializeScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/06.
//

#include "SDKv1FuncList.hpp"
#include "InitializeScene.hpp"
#include "InitializeParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include "../../../Classes/Postbox.hpp"
#include "../../src/cocos2d-x/Module_Cocos2d-x.h"
#include "../../src/C2SModule.h"
#include <map>
#include <sstream>

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;
using namespace::std;


bool InitializeScene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- [SDK v1 초기화] -");

    addButton("초기화 진행\n(C2SModuleSocial_Initialize)", ButtonMode::NEED_PARAM, SDKv1FuncList::INITIALIZE_SCENE_INITIALIZE);
    addButton("SetGuestUserInfo\n(C2SModuleSns_SetGuestUserInfo)", ButtonMode::NEED_PARAM, SDKv1FuncList::INITIALIZE_SCENE_SET_GUESTUSER_INFO);
    addButton("requestPermission\n(C2SModuleSocial_Show)", SDKv1FuncList::INITIALIZE_SCENE_REQUEST_PERMISSION);
    addButton("권한 고지 정보보기\n(C2SModuleSocial_Info)", SDKv1FuncList::INITIALIZE_SCENE_PERMISSION_DATA);

    scheduleUpdate();

    return true;
}

void InitializeScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void InitializeScene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        InitializeParamScene* paramScene = InitializeParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    SDKv1FuncList::InitializeSceneFunc api = static_cast<SDKv1FuncList::InitializeSceneFunc>(buttonTag);
    switch (api) {
        case SDKv1FuncList::INITIALIZE_SCENE_INITIALIZE: {
            string server = ParameterSDKv1DataManager::getInstance()->initializeParameters->getServer();
            bool showLog = ParameterSDKv1DataManager::getInstance()->initializeParameters->getShowLog();
            string appId = ParameterSDKv1DataManager::getInstance()->initializeParameters->getAppId();
            string appIdComboBox = ParameterSDKv1DataManager::getInstance()->initializeParameters->getAppIdForComboBox();
            bool permissionWriteExternal = ParameterSDKv1DataManager::getInstance()->initializeParameters->getPermissionWriteExternalStorage();
            string facebookPermissions = ParameterSDKv1DataManager::getInstance()->initializeParameters->getFacebookPermissions();
            string serverId = ParameterSDKv1DataManager::getInstance()->initializeParameters->getServerId();
            bool showNotice = ParameterSDKv1DataManager::getInstance()->initializeParameters->getShowNotice();
            bool useHivePromotion = ParameterSDKv1DataManager::getInstance()->initializeParameters->getUseHivePromotion();
            string gameLanguage = ParameterSDKv1DataManager::getInstance()->initializeParameters->getGameLanguage();
            int timeoutSeconds = ParameterSDKv1DataManager::getInstance()->initializeParameters->getTimeoutSeconds();
            bool showPermission = ParameterSDKv1DataManager::getInstance()->initializeParameters->getShowPermission();
            bool useImageBanner = ParameterSDKv1DataManager::getInstance()->initializeParameters->getUseImageBanner();
            bool useHercules = ParameterSDKv1DataManager::getInstance()->initializeParameters->getUseHercules();
            string hiveOrientation = ParameterSDKv1DataManager::getInstance()->initializeParameters->getHiveOrientation();
            string channel = ParameterSDKv1DataManager::getInstance()->initializeParameters->getChannel();


            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            picojson::object paramsSns;
            picojson::object paramsPromotion;
            picojson::object paramsHivePromotion;
            picojson::object paramsPush;
            picojson::object paramsInAppV4;

            // server
            if (server == "live") {
                paramsDefault["useDevelop"] = picojson::value(false);
                paramsDefault["useSandbox"] = picojson::value(false);
            } else if (server == "sandbox") {
                paramsDefault["useDevelop"] = picojson::value(false);
                paramsDefault["useSandbox"] = picojson::value(true);
            } else {    // test
                paramsDefault["useDevelop"] = picojson::value(true);
                paramsDefault["useSandbox"] = picojson::value(false);
            }
            // showLog
            paramsDefault["showLog"] = picojson::value(showLog);
            paramsSns["showLog"] = picojson::value(showLog);
            paramsHivePromotion["showLog"] = picojson::value(showLog);
            paramsPromotion["showLog"] = picojson::value(showLog);
            paramsPush["showLog"] = picojson::value(showLog);
            paramsInAppV4["showLog"] = picojson::value(showLog);

            // appId
            if(appId != "") {
                paramsDefault["appId"] = picojson::value(appId);
            } else {
                paramsDefault["appId"] = picojson::value(appIdComboBox);
            }
            // permissionWriteExternalStorage
            paramsDefault["permissionWriteExternalStorage"] = picojson::value(
                    permissionWriteExternal);

            // facebookPermission
            // (array 형태로 추가)
            picojson::value::array vectorFacebookPermissionList;
            if(facebookPermissions != "") {
                string delimiter = ",";

                size_t pos = 0;
                picojson::value token;
                bool existDelimiter = false;
                while ((pos = facebookPermissions.find(delimiter)) != string::npos) {
                    token = picojson::value(facebookPermissions.substr(0, pos));
                    vectorFacebookPermissionList.push_back(token);

                    facebookPermissions.erase(0, pos + delimiter.length());
                    existDelimiter = true;
                }

                if(existDelimiter) {
                    // delimeter 구분된 데이터가 2개 이상인 경우
                    paramsDefault["facebookPermissions"] = picojson::value(vectorFacebookPermissionList);
                }
                else {
                    // delimeter 구분이 없는 경우
                    vectorFacebookPermissionList.push_back(picojson::value(facebookPermissions));
                    paramsSns["facebookPermissions"] = picojson::value(vectorFacebookPermissionList);
                }
            }

            // serverId
            paramsDefault["serverId"] = picojson::value(serverId);
            // showNotice
            paramsDefault["showNotice"] = picojson::value(showNotice);
            // useHivePromotion & useImageBanner
            if (useHivePromotion == true) {
                paramsDefault["HivePromotion"] = picojson::value(
                        paramsHivePromotion);  // add module HivePromotion
            } else { // Mercury
                paramsPromotion["useImageBanner"] = picojson::value(useImageBanner);
                paramsDefault["Promotion"] = picojson::value(
                        paramsPromotion);      // add module Promotion(Mercury)
            }

            // timeoutSeconds
            paramsDefault["timeoutSeconds"] = picojson::value(static_cast<int64_t>(timeoutSeconds));
            // showPermission
            paramsDefault["showHivePermissionView"] = picojson::value(showPermission);

            // useHercules
            paramsDefault["useHercules"] = picojson::value(useHercules);
            // hiveOrientation
            paramsDefault["hiveOrientation"] = picojson::value(hiveOrientation);
            // channel
            paramsDefault["channel"] = picojson::value(channel);

            // add module argument
            paramsDefault["Sns"] = picojson::value(paramsSns);
            paramsDefault["Push"] = picojson::value(paramsPush);
            paramsDefault["InappV4"] = picojson::value(paramsInAppV4);


            // Tracking
            picojson::object paramsTracking;
            picojson::object paramsTrackingAdjust;
//            picojson::object paramsTrackingSingular;          // 계약 만료
            picojson::object paramsTrackingAppsFlyer;
            /**
             * Adjust (android)
             *      Matching
             *          Purchase : 8uwijl
             *          TutorialComplete : wwzrv2
             *          Update : aby3jx
             *      appToken : f02nseiur2m8
             *      AppSecretInfo
             *          secretId : 1
             *          info1 : 1263786494
             *          info2 : 303424327
             *          info3 : 1829017077
             *          info4 : 1052886214
             *  Adjust (iOS)
             *      Matching
             *          Purchase : py318i
             *          TutorialComplete : gvfh2l
             *          Update : sosx8y
             *      appToken : 8szd9kddtt6o
             *      AppSecretInfo
             *          secretId : 1
             *          info1 : 2071917595
             *          info2 : 254356864
             *          info3 : 427896519
             *          info4 : 820983601
             */
            picojson::object paramsTrackingAdjustMatching;
#ifdef __ANDROID__
            paramsTrackingAdjustMatching["Purchase"] = picojson::value("8uwijl");
            paramsTrackingAdjustMatching["TutorialComplete"] = picojson::value("wwzrv2");
            paramsTrackingAdjustMatching["Update"] = picojson::value("aby3jx");
#else
            paramsTrackingAdjustMatching["Purchase"] = picojson::value("py318i");
            paramsTrackingAdjustMatching["TutorialComplete"] = picojson::value("gvfh2l");
            paramsTrackingAdjustMatching["Update"] = picojson::value("sosx8y");
#endif
            paramsTrackingAdjust["Matching"] = picojson::value(paramsTrackingAdjustMatching);

#ifdef __ANDROID__
            paramsTrackingAdjust["appToken"] = picojson::value("f02nseiur2m8");
#else
            paramsTrackingAdjust["appToken"] = picojson::value("8szd9kddtt6o");
#endif
            picojson::object paramsTrackingAdjustAppSecretInfo;
            paramsTrackingAdjustAppSecretInfo["secretId"] = picojson::value("1"); // string

#ifdef __ANDROID__
            paramsTrackingAdjustAppSecretInfo["info1"] = picojson::value("1263786494");
            paramsTrackingAdjustAppSecretInfo["info2"] = picojson::value("303424327");
            paramsTrackingAdjustAppSecretInfo["info3"] = picojson::value("1829017077");
            paramsTrackingAdjustAppSecretInfo["info4"] = picojson::value("1052886214");
#else
            paramsTrackingAdjustAppSecretInfo["info1"] = picojson::value("2071917595");
            paramsTrackingAdjustAppSecretInfo["info2"] = picojson::value("254356864");
            paramsTrackingAdjustAppSecretInfo["info3"] = picojson::value("427896519");
            paramsTrackingAdjustAppSecretInfo["info4"] = picojson::value("820983601");
#endif
            paramsTrackingAdjust["AppSecretInfo"] = picojson::value(paramsTrackingAdjustAppSecretInfo);
            paramsTracking["Adjust"] = picojson::value(paramsTrackingAdjust);

             /**
              * AppsFlyer (android)
              *     Matching
              *         Update : Update
              *         Install : Install
              *         TutorialComplete : Tutorial Complete
              *     key : G6yEQjHw4CrEN8sEzF9onU
              * AppsFlyer (iOS)
              *     Matching
              *         Update : Update
              *         Install : Install
              *         TutorialComplete : Tutorial Complete
              *     key : G6yEQjHw4CrEN8sEzF9onU
              *     itunesConnectAppId : 1202135972
              */
            picojson::object paramsTrackingAppsFlyerMatching;
            paramsTrackingAppsFlyerMatching["Update"] = picojson::value("Update");
            paramsTrackingAppsFlyerMatching["Install"] = picojson::value("Install");
            paramsTrackingAppsFlyerMatching["TutorialComplete"] = picojson::value("Tutorial Complete");
            paramsTrackingAppsFlyer["Matching"] = picojson::value(paramsTrackingAppsFlyerMatching);

#ifdef __ANDROID__
            paramsTrackingAppsFlyer["key"] = picojson::value("G6yEQjHw4CrEN8sEzF9onU");
#else
            paramsTrackingAppsFlyer["key"] = picojson::value("G6yEQjHw4CrEN8sEzF9onU");
            paramsTrackingAppsFlyer["itunesConnectAppId"] = picojson::value("1202135972");
#endif
            paramsTracking["AppsFlyer"] = picojson::value(paramsTrackingAppsFlyer);

            /**
             * Total Argument (SocialInitialize)
             */
            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSocial_Initialize(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSocial_Initialize", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSocial_Initialize", paramString);
            break;
        }
        case SDKv1FuncList::INITIALIZE_SCENE_SET_GUESTUSER_INFO: {
            string guestVid = ParameterSDKv1DataManager::getInstance()->initializeParameters->getGuestVid();
            string guestSessionToken = ParameterSDKv1DataManager::getInstance()->initializeParameters->getGuestSessionToken();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["guestVid"] = picojson::value(guestVid);
            paramsDefault["guestSessionToken"] = picojson::value(guestSessionToken);
            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_SetGuestUserInfo(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_SetGuestUserInfo", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_SetGuestUserInfo", paramString);
            break;
        }
        case SDKv1FuncList::INITIALIZE_SCENE_REQUEST_PERMISSION: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("requestPermission");
            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSocial_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSocial_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSocial_Show", paramString);
            break;
        }
        case SDKv1FuncList::INITIALIZE_SCENE_PERMISSION_DATA: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("permission");
            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSocial_Info(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSocial_Info", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSocial_Info", paramString);

            break;
        }
        default:
            break;
    }
}
