//
//  PromotionV1InfoParamScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/17.
//

#include "PromotionV1InfoParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include <map>

using namespace::std;
using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;


bool PromotionV1InfoParamScene::init()
{
    if(!BaseView::init())
        return false;

    addButton("OK", BUTTON_OK);

    scheduleUpdate();

    return true;
}

void PromotionV1InfoParamScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void PromotionV1InfoParamScene::onButtonEvent(int buttonTag)
{
    if (buttonTag != BUTTON_OK) {
        return;
    }

    setCompletionHandler();
    this->popView();
}

void PromotionV1InfoParamScene::showParameters(int tag)
{
    api = static_cast<SDKv1FuncList::PromotionInfoSceneFunc>(tag);
    switch (api) {
        case SDKv1FuncList::PROMOTION_INFO_SCENE_BADGE_CUSTOM_WEBVIEW: {
            showBadgeCustomViewParam([=]() {
                // viewId
                string viewId = getEditboxValue(EDITBOX_VIEWINFO_VIEW_ID);
                ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->setViewId(viewId);
            });
            break;
        }
        case SDKv1FuncList::PROMOTION_INFO_SCENE_VIEW_CUSTOM_WEBVIEW: {
            showViewCustomViewParam([=]() {
                // viewId
                string viewId = getEditboxValue(EDITBOX_VIEWINFO_VIEW_ID);
                ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->setViewId(viewId);
            });
            break;
        }
        case SDKv1FuncList::PROMOTION_INFO_SCENE_BANNERINFO: {
            showBannerInfoParam([=]() {
                // campaignType
                string campaignType = getComboboxValue(COMBOBOX_BANNERINFO_CAMPAIGN_TYPE);
                ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->setCampaignType(campaignType);

                // bannerType
                string bannerType = getComboboxValue(COMBOBOX_BANNERINFO_BANNER_TYPE);
                ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->setBannerType(bannerType);

                // campaignTypeCustom
                string campaignTypeCustom = getEditboxValue(EDITBOX_BANNERINFO_CAMPAIGN_TYPE);
                ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->setCampaignTypeCustom(campaignTypeCustom);

                // bannerTypeCustom
                string bannerTypeCustom = getEditboxValue(EDITBOX_BANNERINFO_BANNER_TYPE);
                ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->setBannerTypeCustom(bannerTypeCustom);
            });
            break;
        }
        default:
            break;
    }
}

void PromotionV1InfoParamScene::showBadgeCustomViewParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- Badge(customView) -");
    setCompletionHandler = completionHandler;

    // viewId
    string viewId = ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->getViewId();
    addEditBox("viewId : ", viewId.c_str(), "", EDITBOX_VIEWINFO_VIEW_ID);
}

void PromotionV1InfoParamScene::showViewCustomViewParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- View(customView) -");
    setCompletionHandler = completionHandler;

    // viewId
    string viewId = ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->getViewId();
    addEditBox("viewId : ", viewId.c_str(), "", EDITBOX_VIEWINFO_VIEW_ID);
}

void PromotionV1InfoParamScene::showBannerInfoParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- BannerInfo -");
    setCompletionHandler = completionHandler;

    // campaignType
    vector<const char*> customSelectList = { "all",
                                                   "event",
                                                   "notice",
                                                   "cross",
                                                   "custom input" };
    string campaignType = ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->getCampaignType();
    int findIndex1 = static_cast<int>(find(customSelectList.begin(), customSelectList.end(),
                                           campaignType) - customSelectList.begin());

    addCombobox("campaignType(major) : ", customSelectList, findIndex1, COMBOBOX_BANNERINFO_CAMPAIGN_TYPE);

    // bannerType
    vector<const char*> bannerTypeList = { "great",
                                                   "small",
                                                   "rolling",
                                                   "custom input" };
    string bannerType = ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->getBannerType();
    int findIndex = static_cast<int>(find(bannerTypeList.begin(), bannerTypeList.end(),
                                          bannerType) - bannerTypeList.begin());

    addCombobox("bannerType(major) : ", bannerTypeList, findIndex, COMBOBOX_BANNERINFO_BANNER_TYPE);

    // viewId
    string campaignTypeCustom = ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->getCampaignTypeCustom();
    addEditBox("campaignType(minor) : ", campaignTypeCustom.c_str(), "", EDITBOX_BANNERINFO_CAMPAIGN_TYPE);

    // viewId
    string bannerTypeCustom = ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->getBannerTypeCustom();
    addEditBox("bannerType(minor) : ", bannerTypeCustom.c_str(), "", EDITBOX_BANNERINFO_BANNER_TYPE);
}
