//
//  PromotionV2InfoParamScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/20.
//

#include "PromotionV2InfoParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include <map>

using namespace::std;
using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;


bool PromotionV2InfoParamScene::init()
{
    if(!BaseView::init())
        return false;

    addButton("OK", BUTTON_OK);

    scheduleUpdate();

    return true;
}

void PromotionV2InfoParamScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void PromotionV2InfoParamScene::onButtonEvent(int buttonTag)
{
    if (buttonTag != BUTTON_OK) {
        return;
    }

    setCompletionHandler();
    this->popView();
}

void PromotionV2InfoParamScene::showParameters(int tag)
{
    api = static_cast<SDKv1FuncList::HivePromotionInfoSceneFunc>(tag);
    switch (api) {
        case SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_CUSTOM_VIEW: {
            showCustomViewParam([=]() {
                // contentKey
                string contentKey = getEditboxValue(EDITBOX_CUSTOM_VIEW_CONTENT_KEY);
                ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->setCustomViewContentKey(contentKey);
            });
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_CUSTOM_BOARD: {
            showCustomBoardParam([=]() {
                // contentKey
                string contentKey = getEditboxValue(EDITBOX_CUSTOM_BOARD_CONTENT_KEY);
                ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->setCustomBoardContentKey(contentKey);
            });
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_SPOT: {
            showSpotParam([=]() {
                // contentKey
                string contentKey = getEditboxValue(EDITBOX_SPOT_CONTENT_KEY);
                ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->setSpotContentKey(contentKey);
            });
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_DIRECT: {
            showDirectParam([=]() {
                // contentKey
                string contentKey = getEditboxValue(EDITBOX_DIRECT_CONTENT_KEY);
                ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->setDirectContentKey(contentKey);
            });
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_SET_ADDITIONALINFO: {
            showAdditionalInfoParam([=]() {
                // additionalInfo
                string additionalInfo = getEditboxValue(EDITBOX_ADDITIONAL_INFO);
                ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->setAdditionalInfo(additionalInfo);
            });
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_BANNERINFO: {
            showBannerInfoParam([=]() {
                // campaignType
                string campaignType = getComboboxValue(COMBOBOX_BANNERINFO_CAMPAIGN_TYPE);
                ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->setCampaignType(campaignType);

                // bannerType
                string bannerType = getComboboxValue(COMBOBOX_BANNERINFO_BANNER_TYPE);
                ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->setBannerType(bannerType);

                // campaignTypeCustom
                string campaignTypeCustom = getEditboxValue(EDITBOX_BANNERINFO_CAMPAIGN_TYPE);
                ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->setCampaignTypeCustom(campaignTypeCustom);

                // bannerTypeCustom
                string bannerTypeCustom = getEditboxValue(EDITBOX_BANNERINFO_BANNER_TYPE);
                ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->setBannerTypeCustom(bannerTypeCustom);
            });
            break;
        }
        default:
            break;
    }
}

void PromotionV2InfoParamScene::showCustomViewParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- 커스텀뷰 정보) -");
    setCompletionHandler = completionHandler;

    // contentKey
    string contentKey = ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->getCustomViewContentKey();
    addEditBox("contentKey : ", contentKey.c_str(), "", EDITBOX_CUSTOM_VIEW_CONTENT_KEY);
}

void PromotionV2InfoParamScene::showCustomBoardParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- 커스텀보드 정보) -");
    setCompletionHandler = completionHandler;

    // contentKey
    string contentKey = ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->getCustomBoardContentKey();
    addEditBox("contentKey : ", contentKey.c_str(), "", EDITBOX_CUSTOM_BOARD_CONTENT_KEY);
}

void PromotionV2InfoParamScene::showSpotParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- 스팟배너 정보) -");
    setCompletionHandler = completionHandler;

    // contentKey
    string contentKey = ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->getSpotContentKey();
    addEditBox("contentKey : ", contentKey.c_str(), "", EDITBOX_SPOT_CONTENT_KEY);
}

void PromotionV2InfoParamScene::showDirectParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- 다이렉트배너 정보) -");
    setCompletionHandler = completionHandler;

    // contentKey
    string contentKey = ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->getDirectContentKey();
    addEditBox("contentKey : ", contentKey.c_str(), "", EDITBOX_DIRECT_CONTENT_KEY);
}

void PromotionV2InfoParamScene::showAdditionalInfoParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SetAdditionalInfo -");
    setCompletionHandler = completionHandler;

    // additionalInfo
    string additionalInfo = ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->getAdditionalInfo();
    addEditBox("additionalInfo : ", additionalInfo.c_str(), "", EDITBOX_ADDITIONAL_INFO);
}

void PromotionV2InfoParamScene::showBannerInfoParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- BannerInfo -");
    setCompletionHandler = completionHandler;

    // campaignType
    vector<const char*> customSelectList = { "all",
                                                   "event",
                                                   "notice",
                                                   "cross",
                                                   "custom input" };
    string campaignType = ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->getCampaignType();
    int findIndex1 = static_cast<int>(find(customSelectList.begin(), customSelectList.end(),
                                           campaignType) - customSelectList.begin());

    addCombobox("campaignType(major) : ", customSelectList, findIndex1, COMBOBOX_BANNERINFO_CAMPAIGN_TYPE);

    // bannerType
    vector<const char*> bannerTypeList = { "great",
                                                   "small",
                                                   "rolling",
                                                   "custom input" };
    string bannerType = ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->getBannerType();
    int findIndex = static_cast<int>(find(bannerTypeList.begin(), bannerTypeList.end(),
                                          bannerType) - bannerTypeList.begin());

    addCombobox("bannerType(major) : ", bannerTypeList, findIndex, COMBOBOX_BANNERINFO_BANNER_TYPE);

    // viewId
    string campaignTypeCustom = ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->getCampaignTypeCustom();
    addEditBox("campaignType(minor) : ", campaignTypeCustom.c_str(), "", EDITBOX_BANNERINFO_CAMPAIGN_TYPE);

    // viewId
    string bannerTypeCustom = ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->getBannerTypeCustom();
    addEditBox("bannerType(minor) : ", bannerTypeCustom.c_str(), "", EDITBOX_BANNERINFO_BANNER_TYPE);
}
