//
//  PromotionV2ParamScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/20.
//

#include "PromotionV2ParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include <map>

using namespace::std;
using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;


bool PromotionV2ParamScene::init()
{
    if(!BaseView::init())
        return false;

    addButton("OK", BUTTON_OK);

    scheduleUpdate();

    return true;
}

void PromotionV2ParamScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void PromotionV2ParamScene::onButtonEvent(int buttonTag)
{
    if (buttonTag != BUTTON_OK) {
        return;
    }

    setCompletionHandler();
    this->popView();
}

void PromotionV2ParamScene::showParameters(int tag)
{
    api = static_cast<SDKv1FuncList::HivePromotionSceneFunc>(tag);
    switch (api) {
        case SDKv1FuncList::HIVE_PROMOTION_SCENE_NEWS: {
            showNewsParam([=]() {
                // forced
                bool forced = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_NEWS_FORCED));
                ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->setForcedNews(forced);
            });
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_SCENE_BANNER: {
            showBannerParam([=]() {
                // forced
                bool forced = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_BANNER_FORCED));
                ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->setForcedNews(forced);
            });
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_SCENE_NOTICE: {
            showNoticeParam([=]() {
                // forced
                bool forced = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_NOTICE_FORCED));
                ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->setForcedNews(forced);
            });
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_SCENE_CUSTOM_VIEW: {
            showCustomViewParam([=]() {
                // contentKey
                string contentKey = getEditboxValue(EDITBOX_CUSTOM_VIEW_CONTENT_KEY);
                ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->setContentKeyCustomView(contentKey);
            });
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_SCENE_CUSTOM_BOARD: {
            showCustomBoardParam([=]() {
                // contentKey
                string contentKey = getEditboxValue(EDITBOX_CUSTOM_BOARD_CONTENT_KEY);
                ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->setContentKeyCustomBoard(contentKey);
            });
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_SCENE_SPOT_BANNER: {
            showSpotParam([=]() {
                // contentKey
                string contentKey = getEditboxValue(EDITBOX_SPOT_CONTENT_KEY);
                ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->setContentKeySpot(contentKey);
            });
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_SCENE_DIRECT_BANNER: {
            showDirectParam([=]() {
                // contentKey
                string contentKey = getEditboxValue(EDITBOX_DIRECT_CONTENT_KEY);
                ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->setContentKeyDirect(contentKey);
            });
            break;
        }
        default:
            break;
    }
}

void PromotionV2ParamScene::showNewsParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- News -");
    setCompletionHandler = completionHandler;

    // forced
    int forced = ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->getForcedNews();
    addTrueFalseToggleButton("forced : ", forced, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_NEWS_FORCED);
}

void PromotionV2ParamScene::showBannerParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- Banner -");
    setCompletionHandler = completionHandler;

    // forced
    int forced = ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->getForcedBanner();
    addTrueFalseToggleButton("forced : ", forced, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_BANNER_FORCED);
}

void PromotionV2ParamScene::showNoticeParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- Notice -");
    setCompletionHandler = completionHandler;

    // forced
    int forced = ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->getForcedNotice();
    addTrueFalseToggleButton("forced : ", forced, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_NOTICE_FORCED);
}

void PromotionV2ParamScene::showCustomViewParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- 커스텀뷰 -");
    setCompletionHandler = completionHandler;

    // contentKey
    string contentKey = ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->getContentKeyCustomView();
    addEditBox("contentKey : ", contentKey.c_str(), "", EDITBOX_CUSTOM_VIEW_CONTENT_KEY);
}

void PromotionV2ParamScene::showCustomBoardParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- 커스텀보드 -");
    setCompletionHandler = completionHandler;

    // contentKey
    string contentKey = ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->getContentKeyCustomBoard();
    addEditBox("contentKey : ", contentKey.c_str(), "", EDITBOX_CUSTOM_BOARD_CONTENT_KEY);
}

void PromotionV2ParamScene::showSpotParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- 스팟배너 -");
    setCompletionHandler = completionHandler;

    // contentKey
    string contentKey = ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->getContentKeySpot();
    addEditBox("contentKey : ", contentKey.c_str(), "", EDITBOX_SPOT_CONTENT_KEY);
}

void PromotionV2ParamScene::showDirectParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- 다이렉트배너 -");
    setCompletionHandler = completionHandler;

    // contentKey
    string contentKey = ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->getContentKeyDirect();
    addEditBox("contentKey : ", contentKey.c_str(), "", EDITBOX_DIRECT_CONTENT_KEY);
}
