//
//  PushMessageParamScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/12.
//

#include "PushMessageParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include <map>

using namespace::std;
using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;


bool PushMessageParamScene::init()
{
    if(!BaseView::init())
        return false;

    addButton("OK", BUTTON_OK);

    scheduleUpdate();

    return true;
}

void PushMessageParamScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void PushMessageParamScene::onButtonEvent(int buttonTag)
{
    if (buttonTag != BUTTON_OK) {
        return;
    }

    setCompletionHandler();
    this->popView();
}

void PushMessageParamScene::showParameters(int tag)
{
    api = static_cast<SDKv1FuncList::PushSceneFunc>(tag);
    switch (api) {
        case SDKv1FuncList::PUSH_SCENE_REGISTER: {
            showPushRegisterParam([=]() {
                // pushId
                int pushId = stoi(getEditboxValue(EDITBOX_PUSH_ID));
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setPushId(pushId);

                // title
                string title = getEditboxValue(EDITBOX_PUSH_TITLE);
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setTitle(title);

                // title
                string message = getEditboxValue(EDITBOX_PUSH_MESSAGE);
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setMessage(message);

                // after
                int after = stoi(getEditboxValue(EDITBOX_PUSH_AFTER));
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setAfter(after);

                // type
                string type = getComboboxValue(COMBOBOX_PUSH_TYPE);
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setType(type);

                // groupId
                string groupId = getEditboxValue(EDITBOX_PUSH_GROUPID);
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setGroupId(groupId);
            });
            break;
        }
        case SDKv1FuncList::PUSH_SCENE_UNREGISTER: {
            showPushUnRegisterParam([=]() {
                // pushId
                int pushId = stoi(getEditboxValue(EDITBOX_PUSH_ID));
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setPushId(pushId);
            });
            break;
        }
        case SDKv1FuncList::PUSH_SCENE_DESCRIPTION: {
            showPushDescriptionParam([=]() {
                // usePush
                string usePush = getComboboxValue(COMBOBOX_DESC_USE_PUSH);
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setUsePush(usePush);

                // sound
                string sound = getComboboxValue(COMBOBOX_DESC_SOUND);
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setSound(usePush);

                // vibrate
                string vibrate = getComboboxValue(COMBOBOX_DESC_VIBRATE);
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setVibrate(vibrate);

                // operationGCM
                string operationGCM = getComboboxValue(COMBOBOX_DESC_OPERATION_GCM);
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setOperationGCM(operationGCM);
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setOperationRemote(operationGCM);

                // operationLocal
                string operationLocal = getComboboxValue(COMBOBOX_DESC_OPERATION_LOCAL);
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setOperationLocal(operationLocal);

                // useNotice
                string useNotice = getComboboxValue(COMBOBOX_DESC_USE_NOTICE);
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setUseNotice(useNotice);

                // useNight
                string useNight = getComboboxValue(COMBOBOX_DESC_USE_NIGHT);
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setUseNight(useNight);
            });
            break;
        }
        default:
            break;
    }
}

void PushMessageParamScene::showPushRegisterParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- Push 등록 -");
    setCompletionHandler = completionHandler;

    // pushId
    string pushId = to_string(ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getPushId());
    addEditBox("pushId : ", pushId.c_str(), "", EDITBOX_PUSH_ID);

    // title
    string title = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getTitle();
    addEditBox("title : ", title.c_str(), "", EDITBOX_PUSH_TITLE);

    // message
    string message = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getMessage();
    addEditBox("message : ", message.c_str(), "", EDITBOX_PUSH_MESSAGE);

    // after
    string after = to_string(ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getAfter());
    addEditBox("after : ", after.c_str(), "", EDITBOX_PUSH_AFTER);

    // type
    vector<const char*> typeList = { "bar",
                                     "popup",
                                     "bar,popup"};
    string type = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getType();
    int findIndex = static_cast<int>(find(typeList.begin(), typeList.end(), type) - typeList.begin());

    addCombobox("type : ", typeList, findIndex, COMBOBOX_PUSH_TYPE);

    // groupId
    string groupId = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getGroupId();
    addEditBox("groupId : ", groupId.c_str(), "", EDITBOX_PUSH_GROUPID);
}

void PushMessageParamScene::showPushUnRegisterParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- Push 해제 -");
    setCompletionHandler = completionHandler;

    // pushId
    string pushId = to_string(ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getPushId());
    addEditBox("pushId : ", pushId.c_str(), "", EDITBOX_PUSH_ID);
}

void PushMessageParamScene::showPushDescriptionParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- Push 설정 -");
    setCompletionHandler = completionHandler;

    // usePush
    vector<const char*> usePushList = { "true", "false", "null" };
    string usePush = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getUsePush();
    int findIndex = static_cast<int>(find(usePushList.begin(), usePushList.end(), usePush) - usePushList.begin());

    addCombobox("usePush : ", usePushList, findIndex, COMBOBOX_DESC_USE_PUSH);

    // sound
    vector<const char*> soundList = { "true", "false", "null" };
    string sound = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getSound();
    int findIndex2 = static_cast<int>(find(soundList.begin(), soundList.end(), sound) - soundList.begin());

    addCombobox("sound : ", soundList, findIndex2, COMBOBOX_DESC_SOUND);

    // vibrate
    vector<const char*> vibrateList = { "true", "false", "null" };
    string vibrate = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getVibrate();
    int findIndex3 = static_cast<int>(find(vibrateList.begin(), vibrateList.end(), vibrate) - vibrateList.begin());

    addCombobox("vibrate : ", vibrateList, findIndex3, COMBOBOX_DESC_VIBRATE);

    // operationGCM
    vector<const char*> operationGCMList = { "true", "false", "null" };
    string operationGCM = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getOperationGCM();
    int findIndex4 = static_cast<int>(find(operationGCMList.begin(), operationGCMList.end(), operationGCM) - operationGCMList.begin());

    addCombobox("operationGCM : ", operationGCMList, findIndex4, COMBOBOX_DESC_OPERATION_GCM);

    // operationLocal
    vector<const char*> operationLocalList = { "true", "false", "null" };
    string operationLocal = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getOperationLocal();
    int findIndex5 = static_cast<int>(find(operationLocalList.begin(), operationLocalList.end(), operationLocal) - operationLocalList.begin());

    addCombobox("operationLocal : ", operationLocalList, findIndex5, COMBOBOX_DESC_OPERATION_LOCAL);

    // useNotice
    vector<const char*> useNoticeList = { "true", "false", "null" };
    string useNotice = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getUseNotice();
    int findIndex6 = static_cast<int>(find(useNoticeList.begin(), useNoticeList.end(), useNotice) - useNoticeList.begin());

    addCombobox("useNotice : ", useNoticeList, findIndex6, COMBOBOX_DESC_USE_NOTICE);

    // useNight
    vector<const char*> useNightList = { "true", "false", "null" };
    string useNight = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getUseNotice();
    int findIndex7 = static_cast<int>(find(useNightList.begin(), useNightList.end(), useNight) - useNightList.begin());

    addCombobox("useNight : ", useNightList, findIndex7, COMBOBOX_DESC_USE_NIGHT);
}
