//
//  ResultDelegate.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/10.
//

#include "ResultDelegate.hpp"
#include "ParameterSDKv1DataManager.hpp"

#include "cocos2d.h"
#include "../../src/C2SModuleApi.h"
#include "../../src/C2SModuleErrorCode.h"
#include <sstream>

USING_NS_CC;

using namespace std;


void ResultDelegate::SDKv1ResultDelegate(C2SModuleApi api, const char* json, C2SModuleErrorCode code, const char* message) {
    string apiName = ParameterSDKv1DataManager::getInstance()->resultDelegate->getApiName(api);
    ParameterSDKv1DataManager::getInstance()->logManager->addCallbackLog(apiName, json, code, message);

    log("====>>>> ResultDelegate api : %d", api);
    log("====>>>> ResultDelegate apiName : %s", apiName.c_str());
    log("====>>>> ResultDelegate errorCode : %d", code);
    log("====>>>> ResultDelegate errorMsg : %s", message);
    if(json != NULL) {
        log("====>>>> ResultDelegate argument : %s", json);
    } else {
        log("====>>>> ResultDelegate argument : arg(json) is NULL");
    }

    switch(api) {
        case C2SModuleApi_SocialInitialize: {
            log("====>>>> C2SModuleApi_SocialInitialize Result : ");
            if(json != NULL) {
                picojson::value resultJson = ParameterSDKv1DataManager::getInstance()->parseStringToJsonData(json);
                string did = resultJson.get("did").to_str();
                bool isAuthorized = resultJson.get("isAuthorized").get<bool>();
                string autoLoginType = resultJson.get("autoLoginType").to_str();

                ParameterSDKv1DataManager::getInstance()->initializeParameters->setDid(did);
                ParameterSDKv1DataManager::getInstance()->initializeParameters->setIsAuthorized(
                        isAuthorized);
                ParameterSDKv1DataManager::getInstance()->initializeParameters->setAutoLoginType(
                        autoLoginType);

                if (code == 0 && did.empty() == false) {
                    MessageBox("", "초기화 성공");
                }
            } else {
                log("====>>>> C2SModuleApi_SocialInitialize, arg(json) is NULL");
            }
            break;
        }
        case C2SModuleApi_SnsLogin: {
            if(json != NULL) {
                picojson::value resultJson = ParameterSDKv1DataManager::getInstance()->parseStringToJsonData(json);
                string type = resultJson.get("type").to_str();

                if (type == "guest") {
                    log("====>>>> C2SModuleApi_SnsLogin Result : guest");
                    string vid = resultJson.get("vid").to_str();
                    string sessionToken = resultJson.get("sessionToken").to_str();

                    ParameterSDKv1DataManager::getInstance()->loginParameters->setCurrentLoginType(
                            "guest");
                    ParameterSDKv1DataManager::getInstance()->loginParameters->setGuestVid(vid);
                    ParameterSDKv1DataManager::getInstance()->loginParameters->setVidSessionToken(
                            sessionToken);

                    if (code == 0 && vid.empty() == false) {
                        MessageBox("게스트 로그인 성공", "");
                        ParameterSDKv1DataManager::getInstance()->loginParameters->setHiveVid("");
                        ParameterSDKv1DataManager::getInstance()->loginParameters->setHiveUid("");

                        ParameterSDKv1DataManager::getInstance()->resultDelegate->postBoxV1Login();
                    }
                } else if (type == "select") {
                    log("====>>>> C2SModuleApi_SnsLogin Result : select");
                    string hiveVid = resultJson.get("usedVid").to_str();
                    string guestVid = resultJson.get("currentVid").to_str();

                    ParameterSDKv1DataManager::getInstance()->loginParameters->setCurrentLoginType(
                            "select");
                    ParameterSDKv1DataManager::getInstance()->loginParameters->setHiveVid(hiveVid);
                    ParameterSDKv1DataManager::getInstance()->loginParameters->setGuestVid(
                            guestVid);

                    if (hiveVid.empty() == false && guestVid.empty() == false) {
                        MessageBox(
                                "[로그인] 메뉴에서\n- 유저 정보 선택(Select)\n- 유저 정보 선택(Custom Select)\n호출로 계정 전환 바랍니다.",
                                "계정 전환 필요!");
                    }
                } else {
                    string uid = resultJson.get("uid").to_str();
                    if (uid.empty() == false && uid != "") {
                        log("====>>>> C2SModuleApi_SnsLogin Result : hive");
                        string vid = resultJson.get("vid").to_str();
                        string sessionToken = resultJson.get("sessionToken").to_str();

                        ParameterSDKv1DataManager::getInstance()->loginParameters->setCurrentLoginType(
                                "hive");
                        ParameterSDKv1DataManager::getInstance()->loginParameters->setHiveUid(uid);
                        ParameterSDKv1DataManager::getInstance()->loginParameters->setHiveVid(vid);
                        ParameterSDKv1DataManager::getInstance()->loginParameters->setVidSessionToken(
                                sessionToken);

                        if (code == 0 && uid.empty() == false) {
                            MessageBox("", "Hive 로그인 성공");
                            ParameterSDKv1DataManager::getInstance()->loginParameters->setGuestVid(
                                    "");

                            ParameterSDKv1DataManager::getInstance()->resultDelegate->postBoxV1Login();
                        }
                    } else {
                        log("====>>>> C2SModuleApi_SnsLogin Result : no type (not guest/hive/select)");
                    }
                }
            } else {
                log("====>>>> C2SModuleApi_SnsLogin, arg(json) is NULL");
            }
            break;
        }
        case C2SModuleApi_PushDescription: {
            log("====>>>> C2SModuleApi_PushDescription Result : ");
            if(json != NULL) {
                picojson::value resultJson = ParameterSDKv1DataManager::getInstance()->parseStringToJsonData(json);

                bool usePush = resultJson.get("usePush").get<bool>();
                bool sound = resultJson.get("sound").get<bool>();
                bool vibrate = resultJson.get("vibrate").get<bool>();
                bool operationGCM = resultJson.get("operationGCM").get<bool>();
                bool operationLocal = resultJson.get("operationLocal").get<bool>();
                bool useNotice = resultJson.get("useNotice").get<bool>();
                bool useNight = resultJson.get("useNight").get<bool>();
                log("====>>>> C2SModuleApi_PushDescription Result :");
                log("====>>>> usePush : %d", int(usePush));
                log("====>>>> sound : %d", int(sound));
                log("====>>>> vibrate : %d", int(vibrate));
                log("====>>>> operationGCM : %d", int(operationGCM));
                log("====>>>> operationLocal : %d", int(operationLocal));
                log("====>>>> useNotice : %d", int(useNotice));
                log("====>>>> useNight : %d", int(useNight));
                string usePushStr = ParameterSDKv1DataManager::getInstance()->resultDelegate->getBoolToString(
                        usePush);
                string soundStr = ParameterSDKv1DataManager::getInstance()->resultDelegate->getBoolToString(
                        sound);
                string vibrateStr = ParameterSDKv1DataManager::getInstance()->resultDelegate->getBoolToString(
                        vibrate);
                string operationGCMStr = ParameterSDKv1DataManager::getInstance()->resultDelegate->getBoolToString(
                        operationGCM);
                string operationLocalStr = ParameterSDKv1DataManager::getInstance()->resultDelegate->getBoolToString(
                        operationLocal);
                string useNoticeStr = ParameterSDKv1DataManager::getInstance()->resultDelegate->getBoolToString(
                        useNotice);
                string useNightStr = ParameterSDKv1DataManager::getInstance()->resultDelegate->getBoolToString(
                        useNight);

                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setUsePush(
                        usePushStr);
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setSound(soundStr);
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setVibrate(
                        vibrateStr);
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setOperationGCM(
                        operationGCMStr);
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setOperationRemote(
                        operationGCMStr);
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setOperationLocal(
                        operationLocalStr);
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setUseNotice(
                        useNoticeStr);
                ParameterSDKv1DataManager::getInstance()->pushMessageParameters->setUseNight(
                        useNightStr);
            } else {
                log("====>>>> C2SModuleApi_PushDescription, arg(json) is NULL");
            }
            break;
        }
        case C2SModuleApi_PromotionAcquisition: {
            log("====>>>> C2SModuleApi_PromotionAcquisition Result : ");
            if(json != NULL) {
                picojson::value resultJson = ParameterSDKv1DataManager::getInstance()->parseStringToJsonData(
                        json);

                string inviteCommonLink = resultJson.get("inviteCommonLink").to_str();
                string inviteFacebookLink = resultJson.get("inviteFacebookLink").to_str();
                string inviteHivemsgLink = resultJson.get("inviteHivemsgLink").to_str();
                string qrCodeData = resultJson.get("qrcode").to_str();

                if (inviteHivemsgLink.empty() == false) {
                    ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->setLinkShareUrl(
                            inviteHivemsgLink);
                    log("====>>>> inviteHivemsgLink : %s", inviteHivemsgLink.c_str());
                }
                if (qrCodeData.empty() == false) {
                    ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->setQrCodeData(
                            qrCodeData);
                    log("====>>>> qrcode : %s", qrCodeData.c_str());

                    // UserAcquisitionScene 에서 QRCode 이미지를
                    // 팝업으로 띄우도록하는 변수 값 변경
                    ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->setNeedShowQRCodePopup(
                            true);
                }
            } else {
                log("====>>>> C2SModuleApi_PromotionAcquisition, arg(json) is NULL");
            }

            break;
        }
        case C2SModuleApi_InappV4ItemInfo: {
            log("====>>>> C2SModuleApi_InappV4ItemInfo Result : ");
            if(code == 0 && json != NULL) {
                picojson::value resultJson = ParameterSDKv1DataManager::getInstance()->parseStringToJsonData(json);
                picojson::array productListArr = resultJson.get("productList").get<picojson::array>();

                // pid 리스트 파싱 활용
                int iterCnt = 0;
                string forLogPidList = "";
                ParameterSDKv1DataManager::getInstance()->iapv4Parameters->pidList.clear();     // pidList 초기화
                for(picojson::array::iterator iter = productListArr.begin(); iter != productListArr.end(); iter++) {
                    string marketPid = (*iter).get("marketPid").to_str();
                    log("====>>>> %s", marketPid.c_str());

                    forLogPidList += ((*iter).get("marketPid").to_str() + "\n");
                    ParameterSDKv1DataManager::getInstance()->iapv4Parameters->pidList.push_back(marketPid);

                    if(iterCnt == 0) {
                        ParameterSDKv1DataManager::getInstance()->iapv4Parameters->setPid(marketPid);
                    }
                    iterCnt++;
                }
                ParameterSDKv1DataManager::getInstance()->iapv4Parameters->pidList.push_back("custom_pid"); // 마지막 인덱스 값

                MessageBox(forLogPidList.c_str(), "아이팀 정보 획득");
            }
            break;
        }
        case C2SModuleApi_InappV4Purchase: {
            log("====>>>> C2SModuleApi_InappV4Purchase Result : ");
            if(code == 0 && json != NULL) {
                picojson::value resultJson = ParameterSDKv1DataManager::getInstance()->parseStringToJsonData(json);
                picojson::value receiptJson = resultJson.get("receipt");

                string itemType = resultJson.get("type").to_str();
                string bypassInfo = receiptJson.get("bypassInfo").to_str();

                if (bypassInfo.empty() == false && itemType.empty() == false) {
                    // request verify
                    log("====>>>> get bypassinfo, request verify(sample logic)");
                    ParameterSDKv1DataManager::getInstance()->resultDelegate->postBoxV1VerifyReceipt(bypassInfo, itemType);

                } else {
                    log("====>>>> receipt or bypassInfo is empty");
                }
            }

            break;
        }
        case C2SModuleApi_InappV4Restore: {
            log("====>>>> C2SModuleApi_InappV4Restore Result : ");
            if(code == 0 && json != NULL) {
                picojson::value resultJson = ParameterSDKv1DataManager::getInstance()->parseStringToJsonData(json);

                string itemType = resultJson.get("type").to_str();
                if(resultJson.contains("receiptList")) {
                    // restore 시 전달되는 영수증 정보는 receiptList(JsonArray) 형태로 담겨짐
                    picojson::value::array receiptListJsonArr = resultJson.get("receiptList").get<picojson::value::array>();

                    // 영수증 수 만큼 verify 요청
                    for (int i = 0; i < receiptListJsonArr.size(); i++) {
                        string bypassInfo = receiptListJsonArr[i].get("bypassInfo").to_str();

                        if (bypassInfo.empty() == false && itemType.empty() == false) {
                            // request verify
                            log("====>>>> get bypassinfo (idx : %d), request verify(sample logic)", i);
                            ParameterSDKv1DataManager::getInstance()->resultDelegate->postBoxV1VerifyReceipt(bypassInfo, itemType);

                        } else {
                            log("====>>>> receipt or bypassInfo (idx : %d) is empty", i);
                        }
                    }
                }
                else {
                    log("====>>>> not exist receiptList");
                }
            }
            break;
        }
        default:
            break;
    }
}

string ResultDelegate::getApiName(C2SModuleApi api) {
    switch(api) {
        case C2SModuleApi_SocialInitialize: {
            return "C2SModuleApi_SocialInitialize";
        }
        case C2SModuleApi_SocialShow: {
            return "C2SModuleApi_SocialShow";
        }
        case C2SModuleApi_SnsSession: {
            return "C2SModuleApi_SnsSession";
        }
        case C2SModuleApi_SnsLogin: {
            return "C2SModuleApi_SnsLogin";
        }
        case C2SModuleApi_SnsLogout: {
            return "C2SModuleApi_SnsLogout";
        }
        case C2SModuleApi_SnsUnregister: {
            return "C2SModuleApi_SnsUnregister";
        }
        case C2SModuleApi_SnsUser: {
            return "C2SModuleApi_SnsUser";
        }
        case C2SModuleApi_SnsFriends: {
            return "C2SModuleApi_SnsFriends";
        }
        case C2SModuleApi_SnsMessage: {
            return "C2SModuleApi_SnsMessage";
        }
        case C2SModuleApi_SnsPost: {
            return "C2SModuleApi_SnsPost";
        }
        case C2SModuleApi_SnsShow: {
            return "C2SModuleApi_SnsShow";
        }
        case C2SModuleApi_InappPrepare: {
            return "C2SModuleApi_InappPrepare";
        }
        case C2SModuleApi_InappRestore: {
            return "C2SModuleApi_InappRestore";
        }
        case C2SModuleApi_InappPurchase: {
            return "C2SModuleApi_InappPurchase";
        }
        case C2SModuleApi_InappFinish: {
            return "C2SModuleApi_InappFinish";
        }
        case C2SModuleApi_InappDescription: {
            return "C2SModuleApi_InappDescription";
        }
        case C2SModuleApi_PushRegister: {
            return "C2SModuleApi_PushRegister";
        }
        case C2SModuleApi_PushUnregister: {
            return "C2SModuleApi_PushUnregister";
        }
        case C2SModuleApi_PushDescription: {
            return "C2SModuleApi_PushDescription";
        }
        case C2SModuleApi_PromotionShow: {
            return "C2SModuleApi_PromotionShow";
        }
        case C2SModuleApi_InappUpdated: {
            return "C2SModuleApi_InappUpdated";
        }
        case C2SModuleApi_SnsPGSLoginProc: {
            return "C2SModuleApi_SnsPGSLoginProc";
        }
        case C2SModuleApi_InappRequestBalance: {
            return "C2SModuleApi_InappRequestBalance";
        }
        case C2SModuleApi_TrackingSendEvent: {
            return "C2SModuleApi_TrackingSendEvent";
        }
        case C2SModuleApi_TrackingSetEnable: {
            return "C2SModuleApi_TrackingSetEnable";
        }
        case C2SModuleApi_PromotionInfo: {
            return "C2SModuleApi_PromotionInfo";
        }
        case C2SModuleApi_SnsConnect: {
            return "C2SModuleApi_SnsConnect";
        }
        case C2SModuleApi_InterworkStart: {
            return "C2SModuleApi_InterworkStart";
        }
        case C2SModuleApi_InterworkDelegate: {
            return "C2SModuleApi_InterworkDelegate";
        }
        case C2SModuleApi_PromotionAcquisition: {
            return "C2SModuleApi_PromotionAcquisition";
        }
        case C2SModuleApi_HiveIAPInitialize: {
            return "C2SModuleApi_HiveIAPInitialize";
        }
        case C2SModuleApi_HiveIAPInfo: {
            return "C2SModuleApi_HiveIAPInfo";
        }
        case C2SModuleApi_HiveIAPShow: {
            return "C2SModuleApi_HiveIAPShow";
        }
        case C2SModuleApi_HiveIAPPurchase: {
            return "C2SModuleApi_HiveIAPPurchase";
        }
        case C2SModuleApi_HiveIAPRestore: {
            return "C2SModuleApi_HiveIAPRestore";
        }
        case C2SModuleApi_EngagementProcessScheme: {
            return "C2SModuleApi_EngagementProcessScheme";
        }
        case C2SModuleApi_EngagementGetEvents: {
            return "C2SModuleApi_EngagementGetEvents";
        }
        case C2SModuleApi_EngagementConsumeCoupon: {
            return "C2SModuleApi_EngagementConsumeCoupon";
        }
        case C2SModuleApi_EngagementStart: {
            return "C2SModuleApi_EngagementStart";
        }
        case C2SModuleApi_EngagementEnd: {
            return "C2SModuleApi_EngagementEnd";
        }
        case C2SModuleApi_SocialInfo: {
            return "C2SModuleApi_SocialInfo";
        }
        case C2SModuleApi_EngagementGameScheme: {
            return "C2SModuleApi_EngagementGameScheme";
        }
        case C2SModuleApi_HivePromotionShow: {
            return "C2SModuleApi_HivePromotionShow";
        }
        case C2SModuleApi_HivePromotionInfo: {
            return "C2SModuleApi_HivePromotionInfo";
        }
        case C2SModuleApi_HivePromotionGetOfferwallState: {
            return "C2SModuleApi_HivePromotionGetOfferwallState";
        }
        case C2SModuleApi_InappCheckPromotePurchase: {
            return "C2SModuleApi_InappCheckPromotePurchase";
        }
        case C2SModuleApi_HiveIAPCheckPromotePurchase: {
            return "C2SModuleApi_HiveIAPCheckPromotePurchase";
        }
        case C2SModuleApi_PromotionCompanion: {
            return "C2SModuleApi_PromotionCompanion";
        }
        case C2SModuleApi_PromotionPlayback: {
            return "C2SModuleApi_PromotionPlayback";
        }
        case C2SModuleApi_HivePromotionPlayback: {
            return "C2SModuleApi_HivePromotionPlayback";
        }
        case C2SModuleApi_PromotionGetBannerInfo: {
            return "C2SModuleApi_PromotionGetBannerInfo";
        }
        case C2SModuleApi_HivePromotionGetBannerInfo: {
            return "C2SModuleApi_HivePromotionGetBannerInfo";
        }
        case C2SModuleApi_InappV4Initialize: {
            return "C2SModuleApi_InappV4Initialize";
        }
        case C2SModuleApi_InappV4ItemInfo: {
            return "C2SModuleApi_InappV4ItemInfo";
        }
        case C2SModuleApi_InappV4Purchase: {
            return "C2SModuleApi_InappV4Purchase";
        }
        case C2SModuleApi_InappV4Restore: {
            return "C2SModuleApi_InappV4Restore";
        }
        case C2SModuleApi_InappV4TransactionFinish: {
            return "C2SModuleApi_InappV4TransactionFinish";
        }
        case C2SModuleApi_InappV4SelectMarket: {
            return "C2SModuleApi_InappV4SelectMarket";
        }
        case C2SModuleApi_InappV4RequestBalance: {
            return "C2SModuleApi_InappV4RequestBalance";
        }
        case C2SModuleApi_InappV4ChargeBalance: {
            return "C2SModuleApi_InappV4ChargeBalance";
        }
        case C2SModuleApi_InappV4CheckPromotePurchase: {
            return "C2SModuleApi_InappV4CheckPromotePurchase";
        }
        case C2SModuleApi_InappV4GetSelectedMarket: {
            return "C2SModuleApi_InappV4GetSelectedMarket";
        }
        case C2SModuleApi_SocialSetGameInfo: {
            return "C2SModuleApi_SocialSetGameInfo";
        }
        case C2SModuleApi_InappV4MarketConnect: {
            return "C2SModuleApi_InappV4MarketConnect";
        }
        case C2SModuleApi_SnsUserDelete: {
            return "C2SModuleApi_SnsUserDelete";
        }
        case C2SModuleApi_InappV4GetAccountUuid: {
            return "C2SModuleApi_InappV4GetAccountUuid";
        }
        default: {
            return "UnknownAPI";
        }
    }
}

string ResultDelegate::getBoolToString(bool value)
{
    if(value == true) {
        return "true";
    } else
        return "false";
}

void ResultDelegate::postBoxV1Login()
{
    // postbox login
    Postbox::setLoginType(Postbox::v1);
    Postbox::login([=](const Postbox::Login *data, Postbox::Error error){
        stringstream postboxLog;

        if (data != NULL) {
            postboxLog << "nickname : " << data->nickname << '\n';
            postboxLog << "gem : " << data->gem << '\n';
            postboxLog << "gold : " << data->gold << '\n';
        }
        postboxLog << "[error]" << '\n';
        postboxLog << "result_code : " << error.result_code << '\n';
        postboxLog << "result_msg : " << error.result_msg << '\n';

        BaseView::addCallbackLogMessage(postboxLog.str(), "Postbox::loginCB(우편함 로그인) : ");

        ParameterSDKv1DataManager::getInstance()->resultDelegate->postBoxV1Refresh();
    });
}

void ResultDelegate::postBoxV1Refresh()
{
    // postbox posts(refresh)
    Postbox::posts([=](const Postbox::Posts *data, Postbox::Error error){
        stringstream postboxLog;

        if(data != NULL) {
            // postlist 데이터 보여주기
            Postbox::PostList samplePostboxLists;
            samplePostboxLists = data->post_list;

            int postCnt = 1;
            if(samplePostboxLists.size() > 0) {
                for (auto post: samplePostboxLists) {
                    postboxLog << "### count : " << to_string(postCnt) << '\n';
                    postboxLog << "post_id : " << post.post_id << '\n';
                    postboxLog << "asset_code : " << post.asset_code << '\n';
                    postboxLog << "asset_amount : " << to_string(post.asset_amount) << '\n';
                    postboxLog << "user_msg : " << post.user_msg << '\n';
                    postboxLog << "character_key : " << post.character_key << '\n';
                    postboxLog << "date_added : " << post.date_added << '\n';
                    postCnt++;
                }
            } else {
                postboxLog << "### count : 0" << '\n';
            }
            postboxLog << "[error]" << '\n';
            postboxLog << "result_code : " << error.result_code << '\n';
            postboxLog << "result_msg : " << error.result_msg << '\n';
        }

        BaseView::addCallbackLogMessage(postboxLog.str(), "Postbox::posts(refresh)CB(우편함 정보) : ");

        ParameterSDKv1DataManager::getInstance()->resultDelegate->postBoxV1UserInfo();
    });
}

void ResultDelegate::postBoxV1UserInfo()
{
    // postbox userInfo
    Postbox::setLoginType(Postbox::v1);
    Postbox::userInfo([=](const Postbox::UserInfo *data, Postbox::Error error){
        stringstream postboxLog;

        if (data != NULL) {
            postboxLog << "nickname : " << data->nickname << '\n';
            postboxLog << "gem : " << data->gem << '\n';
            postboxLog << "gold : " << data->gold << '\n';
        }
        postboxLog << "[error]" << '\n';
        postboxLog << "result_code : " << error.result_code << '\n';
        postboxLog << "result_msg : " << error.result_msg << '\n';

        BaseView::addCallbackLogMessage(postboxLog.str(), "Postbox::UserInfoCB(우편함 유저 정보) : ");
    });
}

void ResultDelegate::postBoxV1VerifyReceipt(string byPassInfo, string productType)
{
    Postbox::ItemType itemType = Postbox::consumable;

    if (productType == "subscription") {
        itemType = Postbox::subscription;
    }

    // 영수증 검증
    Postbox::verifyReceipt(byPassInfo, itemType, [](const Postbox::VerifyReceipt* verifyReceipt, Postbox::Error error){
        stringstream verifyLog;
        if (verifyReceipt != NULL) {
            verifyLog << "product_id : " << verifyReceipt->product_id << '\n';
        }
        else {
            verifyLog << "product_id : " << "NULL" << '\n';
        }
        verifyLog << "result_code : " << error.result_code << '\n';
        verifyLog << "result_msg : " << error.result_msg << '\n';

        BaseView::addCallbackLogMessage(verifyLog.str(), "Postbox::verifyReceiptCB(영수증 검증) : ");

        ParameterSDKv1DataManager::getInstance()->resultDelegate->postBoxV1Refresh();
    });
}

void ResultDelegate::postBoxV1Receive(string postId)
{
    if(postId.empty() == false) {
        vector<const char *> postIdList;
        postIdList.push_back(postId.c_str());

        Postbox::postReceive(postIdList, [](const Postbox::PostsReceive* data, Postbox::Error error){
            stringstream postboxLog;

            postboxLog << "[error]" << '\n';
            postboxLog << "result_code : " << error.result_code << '\n';
            postboxLog << "result_msg : " << error.result_msg << '\n';

            BaseView::addCallbackLogMessage(postboxLog.str(), "Postbox::postReceiveCB(우편함 아이템수령) : ");

            ParameterSDKv1DataManager::getInstance()->resultDelegate->postBoxV1Refresh();
        });
    }
}

bool LogManager::init()
{
    return true;
}

void LogManager::update(float dt)
{
    //
}

void LogManager::onButtonEvent(int buttonTag)
{
    //
}

void LogManager::addAPICallLog(string apiName, string argument) {
    string argumentLog = "";
    if(argument.empty()) {
        argumentLog = "";
    } else {
        argumentLog = argument;
    }

    string log = "[" + apiName + " called]\n"
                      + "argument : " + argumentLog + "\n\n";
    addLogMessage(log);
}

void LogManager::addErrorLog(string apiName, C2SModule_Cocos2dx::C2SModuleError error) {
    string errorMsg = "";
    if(error.get()->GetMessage() == NULL) {
        errorMsg = "null";
    } else {
        errorMsg = error.get()->GetMessage();
    }

    string errorLog = "[" + apiName + " error]\n"
            + "errorCode : " + to_string(error.get()->GetCode())
            + "\nerrorMsg : " + errorMsg + "\n\n";
    addLogMessage(errorLog);
}

void LogManager::addCallbackLog(string apiName, const char* argument, C2SModuleErrorCode code, const char* message) {
    string argumentLog = "";
    string messageLog = "";
    if(argument == NULL) {
        argumentLog = "null";
    } else {
        argumentLog = argument;
    }
    if(message == NULL) {
        messageLog = "null";
    } else {
        messageLog = message;
    }

    string callbackLog = "[" + apiName + " callback]\n"
            + "errorCode : " + to_string(code)
            + "\nerrorMsg : " + messageLog
            + "\narg : " + argumentLog + "\n\n";
    addLogMessage(callbackLog);
}

