//
//  SetGameInfoParamScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/24.
//

#include "SetGameInfoParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include <map>

using namespace::std;
using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;


bool SetGameInfoParamScene::init()
{
    if(!BaseView::init())
        return false;

    addButton("OK", BUTTON_OK);

    scheduleUpdate();

    return true;
}

void SetGameInfoParamScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void SetGameInfoParamScene::onButtonEvent(int buttonTag)
{
    if (buttonTag != BUTTON_OK) {
        return;
    }

    setCompletionHandler();
    this->popView();
}

void SetGameInfoParamScene::showParameters(int tag)
{
    api = static_cast<SDKv1FuncList::SetGameInfoSceneFunc>(tag);
    switch (api) {
        case SDKv1FuncList::SET_GAME_INFO_SCENE_SET_GAME_INFO: {
            showSetGameInfoParam([=]() {
                // serverId
                string serverId = getEditboxValue(EDITBOX_SERVER_ID);
                ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->setServerId(serverId);

                // gameLanguage
                string gameLanguage = getEditboxValue(EDITBOX_GAME_LANGUAGE);
                ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->setGameLanguage(gameLanguage);

                // emergencyMode
                bool emergencyMode = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_EMERGENCY_MODE));
                ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->setEmergencyMode(emergencyMode);

                // channel
                string channel = getEditboxValue(EDITBOX_CHANNEL);
                ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->setChannel(channel);
            });
            break;
        }
        case SDKv1FuncList::SET_GAME_INFO_SCENE_SET_CHANNEL: {
            showSetChannelParam([=]() {
                // channel
                string channel = getEditboxValue(EDITBOX_CHANNEL);
                ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->setChannel(channel);
            });
            break;
        }
        case SDKv1FuncList::SET_GAME_INFO_SCENE_UPDATE_SERVER_ID: {
            showUpdateServerIdParam([=]() {
                // serverId
                string serverId = getEditboxValue(EDITBOX_SERVER_ID);
                ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->setServerId(serverId);
            });
            break;
        }
        case SDKv1FuncList::SET_GAME_INFO_SCENE_UPDATE_GAMELANGUAGE: {
            showUpdateGameLanguageParam([=]() {
                // gameLanguage
                string gameLanguage = getEditboxValue(EDITBOX_GAME_LANGUAGE);
                ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->setGameLanguage(gameLanguage);
            });
            break;
        }
        case SDKv1FuncList::SET_GAME_INFO_SCENE_SEND_DATA_FUNNELS: {
            showSendDataParam([=]() {
                // funnelsId
                string funnelsId = getEditboxValue(EDITBOX_FUNNELS_ID);
                ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->setFunnelsId(funnelsId);

                // optionTag
                string optionTag = getEditboxValue(EDITBOX_OPTION_TAG);
                ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->setOptionTag(optionTag);
            });
            break;
        }
        default:
            break;
    }
}

void SetGameInfoParamScene::showSetGameInfoParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SetGameInfo -");
    setCompletionHandler = completionHandler;

    // serverId
    string serverId = ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->getServerId();
    addEditBox("serverId : ", serverId.c_str(), "", EDITBOX_SERVER_ID);

    // gameLanguage
    string gameLanguage = ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->getGameLanguage();
    addEditBox("gameLanguage : ", gameLanguage.c_str(), "", EDITBOX_GAME_LANGUAGE);

    // emergencyMode
    int emergencyMode = int(ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->getEmergencyMode());
    addTrueFalseToggleButton("emergencyMode : ", emergencyMode, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_EMERGENCY_MODE);

    // channel
    string channel = ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->getChannel();
    addEditBox("channel : ", channel.c_str(), "", EDITBOX_CHANNEL);
}

void SetGameInfoParamScene::showSetChannelParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SetChannel -");
    setCompletionHandler = completionHandler;

    // channel
    string channel = ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->getChannel();
    addEditBox("channel : ", channel.c_str(), "", EDITBOX_CHANNEL);
}

void SetGameInfoParamScene::showUpdateServerIdParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- UpdateServerId -");
    setCompletionHandler = completionHandler;

    // serverId
    string serverId = ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->getServerId();
    addEditBox("serverId : ", serverId.c_str(), "", EDITBOX_SERVER_ID);
}


void SetGameInfoParamScene::showUpdateGameLanguageParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- UpdateGameLanguage -");
    setCompletionHandler = completionHandler;

    // gameLanguage
    string gameLanguage = ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->getGameLanguage();
    addEditBox("gameLanguage : ", gameLanguage.c_str(), "", EDITBOX_GAME_LANGUAGE);
}

void SetGameInfoParamScene::showSendDataParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SendData -");
    setCompletionHandler = completionHandler;

    // funnelsId
    string funnelsId = ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->getFunnelsId();
    addEditBox("funnelsId : ", funnelsId.c_str(), "", EDITBOX_FUNNELS_ID);

    // optionTag
    string optionTag = ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->getOptionTag();
    addEditBox("optionTag : ", optionTag.c_str(), "", EDITBOX_OPTION_TAG);
}
