//
//  TrackingParamScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/19.
//

#include "TrackingParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include <map>

using namespace::std;
using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;


bool TrackingParamScene::init()
{
    if(!BaseView::init())
        return false;

    addButton("OK", BUTTON_OK);

    scheduleUpdate();

    return true;
}

void TrackingParamScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void TrackingParamScene::onButtonEvent(int buttonTag)
{
    if (buttonTag != BUTTON_OK) {
        return;
    }

    setCompletionHandler();
    this->popView();
}

void TrackingParamScene::showParameters(int tag)
{
    api = static_cast<SDKv1FuncList::TrackingSceneFunc>(tag);
    switch (api) {
        case SDKv1FuncList::TRACKER_SCENE_SEND_EVENT: {
            showEventParam([=]() {
                // event
                string event = getEditboxValue(EDITBOX_EVENT);
                ParameterSDKv1DataManager::getInstance()->trackingParameters->setEvent(event);
            });
            break;
        }
        case SDKv1FuncList::TRACKER_SCENE_ENABLE: {
            showEnableParam([=]() {
                // adjust
                bool adjust = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_ENABLE_ADJUST));
                ParameterSDKv1DataManager::getInstance()->trackingParameters->setEnableAdjust(adjust);

                // singular
                bool singular = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_ENABLE_SINGULAR));
                ParameterSDKv1DataManager::getInstance()->trackingParameters->setEnableSingular(singular);

                // appsfler
                bool appsfler = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_ENABLE_APPSFLYER));
                ParameterSDKv1DataManager::getInstance()->trackingParameters->setEnableSingular(appsfler);
            });
            break;
        }
        default:
            break;
    }
}

void TrackingParamScene::showEventParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- 이벤트 전송 -");
    setCompletionHandler = completionHandler;

    // event
    string event = ParameterSDKv1DataManager::getInstance()->trackingParameters->getEvent();
    addEditBox("event : ", event.c_str(), "", EDITBOX_EVENT);
}

void TrackingParamScene::showEnableParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- 활성화 설정 -");
    setCompletionHandler = completionHandler;

    // adjust
    int adjust = int(ParameterSDKv1DataManager::getInstance()->trackingParameters->getEnableAdjust());
    addTrueFalseToggleButton("Adjust : ", adjust, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_ENABLE_ADJUST);

    // singular
    int singular = int(ParameterSDKv1DataManager::getInstance()->trackingParameters->getEnableSingular());
    addTrueFalseToggleButton("Singular : ", adjust, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_ENABLE_SINGULAR);

    // appsflyer
    int appsflyer = int(ParameterSDKv1DataManager::getInstance()->trackingParameters->getEnableAppsFlyer());
    addTrueFalseToggleButton("AppsFlyer : ", appsflyer, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_ENABLE_APPSFLYER);
}
