//
//  UserAcquisitionParamScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/17.
//

#include "UserAcquisitionParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include <map>

using namespace::std;
using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;


bool UserAcquisitionParamScene::init()
{
    if(!BaseView::init())
        return false;

    addButton("OK", BUTTON_OK);

    scheduleUpdate();

    return true;
}

void UserAcquisitionParamScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void UserAcquisitionParamScene::onButtonEvent(int buttonTag)
{
    if (buttonTag != BUTTON_OK) {
        return;
    }

    setCompletionHandler();
    this->popView();
}

void UserAcquisitionParamScene::showParameters(int tag)
{
    api = static_cast<SDKv1FuncList::UserAcquisitionSceneFunc>(tag);
    switch (api) {
        case SDKv1FuncList::UA_SCENE_INFO: {
            showUAInfoParam([=]() {
                // additionalInfo
                string additionalInfo = getEditboxValue(EDITBOX_UA_ADDITIONAL_INFO);
                ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->setAdditionalInfo(additionalInfo);
            });
            break;
        }
        case SDKv1FuncList::UA_SCENE_LINK_AND_MESSAGE_SHARE: {
            showUAHiveLinkMessageShareParam([=]() {
                // message
                string message = getEditboxValue(EDITBOX_UA_MESSAGE);
                ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->setMessage(message);

                // linkShareURL
                string linkShareURL = getEditboxValue(EDITBOX_UA_LINK_SHARE_URL);
                ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->setLinkShareUrl(message);
            });
            break;
        }
        case SDKv1FuncList::UA_SCENE_HIVE_MESSAGE_WITH_UA_INFO: {
            showUAHiveInviteLinkShareParam([=]() {
                // useVid
                bool useVid = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_USE_VID_OR_UID));
                ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->setUseVid(useVid);

                // vid (or uid)
                string vidOruid = getEditboxValue(EDITBOX_UA_VID_OR_UID);
                ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->setVidOrUid(vidOruid);

                // message
                string message = getEditboxValue(EDITBOX_UA_MESSAGE);
                ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->setMessage(message);

                // imageURL
                string imageURL = getEditboxValue(EDITBOX_UA_IMAGE_URL);
                ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->setImageURL(imageURL);

                // thumbnailURL
                string thumbnailURL = getEditboxValue(EDITBOX_UA_THUMBNAIL_URL);
                ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->setThumbnailURL(thumbnailURL);

                // usePush
                bool usePush = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_UA_USEPUSH));
                ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->setUsePush(usePush);

                // invite
                bool invite = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_UA_INVITE));
                ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->setInvite(invite);

                // linkShare
                bool linkShare = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_UA_LINKSHARE));
                ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->setLinkShare(linkShare);

                // linkShareURL
                string linkShareURL = getEditboxValue(EDITBOX_UA_LINK_SHARE_URL);
                ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->setLinkShareUrl(message);
            });
            break;
        }
        default:
            break;
    }
}

void UserAcquisitionParamScene::showUAInfoParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- 집객 정보 보기 -");
    setCompletionHandler = completionHandler;

    // additionalInfo
    string additionalInfo = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getAdditionalInfo();
    addEditBox("additionalInfo : ", additionalInfo.c_str(), "", EDITBOX_UA_ADDITIONAL_INFO);
}

void UserAcquisitionParamScene::showUAHiveLinkMessageShareParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- 링크 및 메시지 공유 -");
    setCompletionHandler = completionHandler;

    // message
    string message = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getMessage();
    addEditBox("message : ", message.c_str(), "", EDITBOX_UA_MESSAGE);

    // linkShareURL
    string linkShareURL = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getLinkShareUrl();
    addEditBox("linkShareURL : ", linkShareURL.c_str(), "", EDITBOX_UA_LINK_SHARE_URL);
}

void UserAcquisitionParamScene::showUAHiveInviteLinkShareParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- Hive 메시지 전달(집객 정보 활용) -");
    setCompletionHandler = completionHandler;

    // useVid( or Uid )
    int useVid = int(ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getUseVid());
    addTrueFalseToggleButton("use vid(or uid) : ", useVid, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_USE_VID_OR_UID);

    // vid ( or uid)
    string vidOrUid = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getVidOrUid();
    addEditBox("vid(or uid) : ", vidOrUid.c_str(), "", EDITBOX_UA_VID_OR_UID);

    // message
    string message = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getMessage();
    addEditBox("message : ", message.c_str(), "", EDITBOX_UA_MESSAGE);

    // imageURL
    string imageURL = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getImageURL();
    addEditBox("imageURL : ", imageURL.c_str(), "", EDITBOX_UA_IMAGE_URL);

    // thumbnailURL
    string thumbnailURL = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getThumbnailURL();
    addEditBox("thumbnailURL : ", thumbnailURL.c_str(), "", EDITBOX_UA_THUMBNAIL_URL);

    // usePush
    int usePush = int(ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getUsePush());
    addTrueFalseToggleButton("usePush : ", usePush, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_UA_USEPUSH);

    // invite
    int invite = int(ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getInvite());
    addTrueFalseToggleButton("invite : ", invite, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_UA_INVITE);

    // linkShare
    int linkShare = int(ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getLinkShare());
    addTrueFalseToggleButton("linkShare : ", linkShare, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_UA_LINKSHARE);

    // linkShareURL
    string linkShareURL = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getLinkShareUrl();
    addEditBox("linkShareURL : ", linkShareURL.c_str(), "", EDITBOX_UA_LINK_SHARE_URL);
}
