//
//  UserInfoParamScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/14.
//

#include "UserInfoParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include <map>

using namespace::std;
using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;


bool UserInfoParamScene::init()
{
    if(!BaseView::init())
        return false;

    addButton("OK", BUTTON_OK);

    scheduleUpdate();

    return true;
}

void UserInfoParamScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void UserInfoParamScene::onButtonEvent(int buttonTag)
{
    if (buttonTag != BUTTON_OK) {
        return;
    }

    setCompletionHandler();
    this->popView();
}

void UserInfoParamScene::showParameters(int tag)
{
    api = static_cast<SDKv1FuncList::UserInfoSceneFunc>(tag);
    switch (api) {
        case SDKv1FuncList::USERINFO_SCENE_HIVE_USER: {
            showHiveUserInfoParam([=]() {
                // target
                string target = getComboboxValue(COMBOBOX_MY_INFO);
                ParameterSDKv1DataManager::getInstance()->userInfoParameters->setUseMyVid(target);

                // userVid
                string userVid = getEditboxValue(EDITBOX_OTHER_VID);
                ParameterSDKv1DataManager::getInstance()->userInfoParameters->setUserVid(userVid);
            });
            break;
        }
        case SDKv1FuncList::USERINFO_SCENE_FACEBOOK_USER: {
            showFacebookUserInfoParam([=]() {
                // facebookId
                string facebookId = getEditboxValue(EDITBOX_FACEBOOK_ID);
                ParameterSDKv1DataManager::getInstance()->userInfoParameters->setFacebookId(facebookId);
            });
            break;
        }
        default:
            break;
    }
}

void UserInfoParamScene::showHiveUserInfoParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- Hive 유저 정보 -");
    setCompletionHandler = completionHandler;

    // type
    vector<const char*> targetList = { "myVid",
                                     "otherVid" };
    string target = ParameterSDKv1DataManager::getInstance()->userInfoParameters->getUseMyVid();
    int findIndex = static_cast<int>(find(targetList.begin(), targetList.end(), target) - targetList.begin());

    addCombobox("대상 : ", targetList, findIndex, COMBOBOX_MY_INFO);

    // userVid
    string userVid = ParameterSDKv1DataManager::getInstance()->userInfoParameters->getUserVid();
    addEditBox("vid : ", userVid.c_str(), "", EDITBOX_OTHER_VID);
}

void UserInfoParamScene::showFacebookUserInfoParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- Facebook 유저 정보 -");
    setCompletionHandler = completionHandler;

    // facebookId
    string facebook = ParameterSDKv1DataManager::getInstance()->userInfoParameters->getFacebookId();
    addEditBox("facebookId : ", facebook.c_str(), "", EDITBOX_FACEBOOK_ID);
}
