/**
 * @file    refobject.h
 
 * @brief   레퍼런스 오브젝트를 구현한 클래스의 헤더
 
 * @author  Byeong Yeol Jeon
 * @ingroup Cocos2d-x
 * @date    2014/04/25
 */
#ifndef _REFOBJECT_H_
#define _REFOBJECT_H_

#include <cassert>
#ifdef __ANDROID__
#include <stdint.h>
#endif

namespace C2SModule_Cocos2dx {
    /**
     * @class refcounter
     
     * @brief 레퍼렌스 카운트를 구현한 클래스
     * @ingroup Cocos2d-x
     * @author Byeong Yeol Jeon
     
     */
    class refcounter {
    public:
        /** 레퍼런스 카운트 변수의 타입으로 32비트 정수형을 사용 */
        typedef int32_t RefCountValueType;
        /**
         * @brief 현재 레퍼런스 카운트를 구한다.
         
         * @return 레퍼런스 카운트
         * @author Byeong Yeol Jeon
         
         */
        inline RefCountValueType GetRefCount() const {
            return refCount;
        }
    protected:
        /**
         * @brief default constructor
         
         * @author Byeong Yeol Jeon
         
         */
        refcounter() :
        refCount(0) {
        }
        /**
         * @brief 레퍼런스 카운트를 하나 증가시킨다.
         * @return 처리후 레퍼런스 카운트 값
         * @author Byeong Yeol Jeon
         
         */
        inline RefCountValueType AddRefCount() {
            return ++refCount;
        }
        /**
         * @brief 레퍼런스 카운트를 하나 감소시킨다.
         * @return 처리 후 레퍼런스 카운트 값
         
         * @author Byeong Yeol Jeon
         
         */
        inline RefCountValueType ReleaseRefCount() {
            assert(0 < refCount);
            return --refCount;
        }
        
    private:
        /** copy prevent */
        refcounter(const refcounter&);
        /** copy prevent */
        refcounter& operator=(const refcounter&);
        
        /** reference count를 담은 변수 */
        volatile RefCountValueType refCount;
    };
    /**
     * @class refobject
     
     * @brief 레퍼런스 카운트로 수명이 제어되는 오브젝트
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x
     */
    class refobject: public refcounter {
    protected:
        /**
         * @brief default constructor
         * @author Byeong Yeol Jeon
         
         */
        refobject();
        /**
         * @brief default destructor
         * @remark 상속되는 객체이므로 가상으로 선언
         
         * @author Byeong Yeol Jeon
         
         */
        virtual ~refobject();
        
    public:
        /**
         * @brief 레퍼런스 카운트를 증가시킨다.
         
         * @author Byeong Yeol Jeon
         
         */
        void AddRef();
        /**
         * @brief 레퍼런스 카운트를 감소하고, 0이되면 인스턴스를 제거한다.
         
         * @author Byeong Yeol Jeon
         
         */
        virtual void Release();
    };
    
}
#endif
