/**
 * @file    Module_iOS.h
 * @brief   모듈의 iOS 실행 헤더
 * @author  Byeong Yeol Jeon
 * @ingroup iOS
 * @date    2014/04/28
 */
#ifndef _MODULE_IOS_H_
#define _MODULE_IOS_H_

#include "ModuleHelper_iOS.h"
#include "../C2SModuleApi.h"

#if __OBJC__
/**
 * @protocol C2SModuleResultDelegate
 
 * @brief 결과를 통지받는 공통 델리게이트 프로토콜
 * @ingroup iOS
 * @author Byeong Yeol Jeon
 
 */
@protocol C2SModuleResultDelegate <NSObject>
/**
 * @brief 델리게이트 함수
 
 * @param[api] api 호출된 결과의 api 종류
 * @param[api] arg 결과 인자
 * @param[api] error 발생한 에러
 * @author Byeong Yeol Jeon
 
 */
-(void)C2SModuleResult:(C2SModuleApi)api parameter:(NSDictionary *)arg error:(NSError*)error;

@end

@interface C2SModuleSocial : NSObject
/**
 * @brief 결과 통지 공통 델리게이트를 설정한다.
 * @param[in] delegate 설정할 델리게이트 인스턴스
 * @see Social
 * @author Byeong Yeol Jeon
 */
+(void)SetCommonResultDelegate:(id<C2SModuleResultDelegate>)delegate;
/**
 * @brief 카카오, 허브의 openUrl을 오버로드 하는 함수
 * @param[in] url
 * @param[in] sourceApplication
 * @param[in] annotation
 * @return
 * @see Social
 * @author Byeong Yeol Jeon
 */
+(BOOL)openURL:(NSURL *)url sourceApplication:(NSString *)sourceApplication annotation:(id)annotation;
/**
 * @brief 통합 모듈을 초기화하고 결과를 공통 델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * - "isAuthorized" : bool. 현재 단말에서 한번이라도 정식유저로 접속한 적이 있을 경우 true.
 * - "did" : NSString*.
 * - "autoLoginType" : NSString*. 키가 없으면 자동로그인 불가능. "guest"일 경우 게스트로 로그인 가능. "platform"일 경우 정식유저로 로그인 가능.
 * @remark
 * 입력 인자
 * - "showLog" : bool. true일 경우 콘솔에 로그를 출력한다.
 * - "useStaging" : bool. true일 경우 staging 서버를 사용. false일 경우 production 서버를 사용.
 * - "launchOptions" : NSDictionary*. 앱 실행시 받은 초기화 값.
 * - Sns : NSDictionary*. Sns 모듈 초기화 인자들.
 *      - "useStaging" : bool. true일 경우 staging 서버를 사용. false일 경우 production 서버를 사용.
 * - Inapp : NSDictionary*. Inapp 모듈 초기화 인자들.
 *      - "useStaging" : bool. true일 경우 staging 서버를 사용. false일 경우 production 서버를 사용.
 *      - "pidList" : NSArray*. 사용할 인앱 상품들의 아이디 리스트.
 * - Push : NSDictionary*. Push 모듈 초기화 인자들.
 *      - "useStaging" : bool. true일 경우 staging 서버를 사용. false일 경우 production 서버를 사용.
 * - Promotion : NSDictionary*. Promotion 모듈 초기화 인자들.
 *      - "showLog" : bool. true일 경우 콘솔에 로그를 출력한다.
 *      - "useStaging" : bool. true일 경우 staging 서버를 사용. false일 경우 production 서버를 사용.
 * @see Social
 * @author Byeong Yeol Jeon
 */
+(BOOL)Initialize:(id)arg error:(NSError**)error;
/**
 * @brief 통합 모듈을 초기화하고 결과를 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * - "isAuthorized" : bool. 현재 단말에서 한번이라도 정식유저로 접속한 적이 있을 경우 true.
 * - "did" : NSString*.
 * - "autoLoginType" : NSString*. 키가 없으면 자동로그인 불가능. "guest"일 경우 게스트로 로그인 가능. "platform"일 경우 정식유저로 로그인 가능.
 * @remark
 * 입력 인자
 * - "showLog" : bool. true일 경우 콘솔에 로그를 출력한다.
 * - "useStaging" : bool. true일 경우 staging 서버를 사용. false일 경우 production 서버를 사용.
 * - "launchOptions" : NSDictionary*. 앱 실행시 받은 초기화 값.
 * - Sns : NSDictionary*. Sns 모듈 초기화 인자들.
 *      - "useStaging" : bool. true일 경우 staging 서버를 사용. false일 경우 production 서버를 사용.
 * - Inapp : NSDictionary*. Inapp 모듈 초기화 인자들.
 *      - "useStaging" : bool. true일 경우 staging 서버를 사용. false일 경우 production 서버를 사용.
 *      - "pidList" : NSArray*. 사용할 인앱 상품들의 아이디 리스트.
 * - Push : NSDictionary*. Push 모듈 초기화 인자들.
 *      - "useStaging" : bool. true일 경우 staging 서버를 사용. false일 경우 production 서버를 사용.
 * - Promotion : NSDictionary*. Promotion 모듈 초기화 인자들.
 *      - "showLog" : bool. true일 경우 콘솔에 로그를 출력한다.
 *      - "useStaging" : bool. true일 경우 staging 서버를 사용. false일 경우 production 서버를 사용.
 * @see Social
 * @author Byeong Yeol Jeon
 */
+(BOOL)Initialize:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief sns 모듈에 종속적인 뷰를 보여주고 결과를 공통 델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * - "type" : NSString*. 입력시 설정한 타입
 * @remark
 * 입력 인자
 * - "type" : NSString*. "terms"일 경우 이용약관을 출력한다.
 * - "redirectUrl" : NSString*. 기본 이용약관을 사용하지 않을 때 설정한다.
 * @see Social
 * @author Byeong Yeol Jeon
 */
+(BOOL)Show:(id)arg error:(NSError**)error;
/**
 * @brief 통합 모듈을 초기화하고 결과를 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * - "type" : NSString*. 입력시 설정한 타입
 * @remark
 * 입력 인자
 * - "type" : NSString*. "terms"일 경우 이용약관을 출력한다.
 * - "redirectUrl" : NSString*. 기본 이용약관을 사용하지 않을 때 설정한다.
 * @see Social
 * @author Byeong Yeol Jeon
 */
+(BOOL)Show:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief 공지 점검 팝업을 띄운다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * -
 * @remark
 * 입력 인자
 * -
 * @see Social
 * @author Daun Joung
 */
+(BOOL)Info:(id)arg error:(NSError**)error;
/**
 * @brief 공지 점검 팝업을 띄운다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * -
 * @remark
 * 입력 인자
 * -
 * @see Social
 * @author Daun Joung
 */
+(BOOL)Info:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief serverId,gameLanguage 설정
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * -
 * @remark
 * 입력 인자
 * -
 * @see Social
 * @author Daun Joung
 */
+(BOOL)SetGameInfo:(id)arg error:(NSError**)error;
/**
 * @brief serverId,gameLanguage 설정
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * -
 * @remark
 * 입력 인자
 * -
 * @see Social
 * @author Daun Joung
 */
+(BOOL)SetGameInfo:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;

/**
 * @brief gameLanguage 변경. HivePromotion 정보 갱신.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * -
 * @remark
 * 입력 인자
 * -
 * @see Social
 * @author Seokjin Yong
 */
+(BOOL)UpdateGameLanguage:(id)arg error:(NSError**)error;

/**
 * @brief gameLanguage 변경. HivePromotion 정보 갱신.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * -
 * @remark
 * 입력 인자
 * -
 * @see Social
 * @author Seokjin Yong
 */
+(BOOL)UpdateGameLanguage:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary*, NSError*))completionHandler;

/**
 * @brief serverId 변경. HivePromotion 정보 갱신.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * -
 * @remark
 * 입력 인자
 * -
 * @see Social
 * @author Seokjin Yong
 */
+(BOOL)UpdateServerId:(id)arg error:(NSError**)error;

/**
 * @brief serverId 변경. HivePromotion 정보 갱신.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * -
 * @remark
 * 입력 인자
 * -
 * @see Social
 * @author Seokjin Yong
 */
+(BOOL)UpdateServerId:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary*, NSError*))completionHandler;

/**
 * @brief analytics log 전송
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * -
 * @remark
 * 입력 인자
 * -
 * @see Social
 * @author Daun Joung
 */
+(BOOL)SendData:(id)arg error:(NSError**)error;
/**
 * @brief analytics log 전송
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * -
 * @remark
 * 입력 인자
 * -
 * @see Social
 * @author Daun Joung
 */
+(BOOL)SendData:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
+(NSString *)GetHiveCountryCode;
@end

@interface C2SModuleSns : NSObject
/**
 * @brief 현재 로그인한 로그인 센터의 세션 토큰을 구해서 공통 델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * - "sessionToken" : NSString*. 현재 접속에 대한 세션키
 * @remark 입력 인자 없음
 * @see Sns
 * @author Byeong Yeol Jeon
 */
+(BOOL)Session:(id)arg error:(NSError**)error;
/**
 * @brief 현재 로그인한 로그인 센터의 세션 토큰을 구해서 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * - "sessionToken" : NSString*. 현재 접속에 대한 세션키
 * @remark 입력 인자 없음
 * @see Sns
 * @author Byeong Yeol Jeon
 */
+(BOOL)Session:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief 로그인 센터에 로그인하고 결과를 공통 델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * - "vid" : 로그인센터 로그인 아이디
 * - "sessionToken" : 로그인 센터 세션키
 * @remark 
 * 입력 인자
 * - "type" : NSString*. 없을 경우 정식 로그인.
 *      - "guest" : 게스트 로그인
 *      - "select" : (로그인 센터 문서 참조)
 *          - "currentVidData" : NSDictionary*. 현재 게스트의 데이터. 키 "currentVid"에 게스트의 vid를 넣어야한다.
 *          - "usedVidData" : NSDictionary*. 정식 유저의 데이터. 키 "usedVidData"에 정식유저의 vid를 넣어야한다.
 * @see Sns
 * @author Byeong Yeol Jeon
 */
+(BOOL)Login:(id)arg error:(NSError**)error;
/**
 * @brief 로그인 센터에 로그인하고 결과를 공통 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * - "vid" : 로그인센터 로그인 아이디
 * - "sessionToken" : 로그인 센터 세션키
 * @remark
 * 입력 인자
 * - "type" : NSString*. 없을 경우 정식 로그인.
 *      - "guest" : 게스트 로그인
 *      - "select" : (로그인 센터 문서 참조)
 *          - "currentVidData" : NSDictionary*. 현재 게스트의 데이터. 키 "currentVid"에 게스트의 vid를 넣어야한다.
 *          - "usedVidData" : NSDictionary*. 정식 유저의 데이터. 키 "usedVidData"에 정식유저의 vid를 넣어야한다.
 * @see Sns
 * @author Byeong Yeol Jeon
 */
+(BOOL)Login:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief 로그인 센터에서 로그아웃하고 결과를 공통 델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark 입력 인자 없음
 * @see Sns
 * @author Byeong Yeol Jeon
 */
+(BOOL)Logout:(id)arg error:(NSError**)error;
/**
 * @brief 로그인 센터에서 로그아웃하고 결과를 공통 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark 입력 인자 없음
 * @see Sns
 * @author Byeong Yeol Jeon
 */
+(BOOL)Logout:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief 게임을 탈퇴하고 결과를 공통 델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark 입력 인자 없음
 * @see Sns
 * @author Byeong Yeol Jeon
 */
+(BOOL)Unregister:(id)arg error:(NSError**)error;
/**
 * @brief 게임을 탈퇴하고 결과를 공통 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark 입력 인자 없음
 * @see Sns
 * @author Byeong Yeol Jeon
 */
+(BOOL)Unregister:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief 플랫폼 사용자의 정보를 요청하고 결과를 공통 델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * - 유저가 한 명일 경우
 *      - "vid" : NSString*. 로그인센터 유니크 아이디
 *      - "nickName" : NSString*. sns 아이디
 *      - "alias" : NSString*. 개별 사용자가 설정한 이름
 *      - "profileImageUrl" : NSString*. 프로필 이미지 다운로드 url
 *      - "country" : NSString*. 국가코드(Hub만 사용, 옵션)
 *      - "comment" : NSString*. 개별 사용자가 설정한 코멘트(Hub만 사용, 옵션)
 *      - "birthDay" : NSString*. 생일(Hub만 사용, 옵션)
 *      - "facebookId" : NSString. 페이스북 아이디(Hub만 사용, 옵션)
 *      - "e-mail" : NSString*. 이메일(Hub만 사용, 옵션)
 *      - "gender" : NSString*. 성별(Hub만 사용, 옵션)
 *      - "newMessageCount" : 32bit signed. 자신의 정보일때만 데이터가 존재함. 받은 메시지의 갯수(Hub만 사용)
 *      - "newFriendCount" : 32bit signed. 자신의 정보일때만 데이터가 존재함. 새로 수락된 친구의 카운트(Hub만 사용)
 *      - "newRequestCount" : 32bit signed. 자신의 정보일때만 데이터가 존재함. 받은 친구 요청 카운트(Hub만 사용)
 *      .
 * - 유저가 여러 명일 경우
 *      - "list" : NSArray*. 유저들의 정보를 저장한 배열
 * @remark
 * 입력 인자
 * - "vid" : 둘중 하나의 타입으로 요청한다.
 *      - NSString*. 요청할 대상의 vid. 없을 경우 자신의 정보를 요청한다. 
 *      - NSArray*. 요청할 대상들의 vid.
 * @see Sns
 * @author Byeong Yeol Jeon
 */
+(BOOL)User:(id)arg error:(NSError**)error;
/**
 * @brief 플랫폼 사용자의 정보를 요청하고 결과를 공통 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * - 유저가 한 명일 경우
 *      - "vid" : NSString*. 로그인센터 유니크 아이디
 *      - "nickName" : NSString*. sns 아이디
 *      - "alias" : NSString*. 개별 사용자가 설정한 이름
 *      - "profileImageUrl" : NSString*. 프로필 이미지 다운로드 url
 *      - "country" : NSString*. 국가코드(Hub만 사용, 옵션)
 *      - "comment" : NSString*. 개별 사용자가 설정한 코멘트(Hub만 사용, 옵션)
 *      - "birthDay" : NSString*. 생일(Hub만 사용, 옵션)
 *      - "facebookId" : NSString. 페이스북 아이디(Hub만 사용, 옵션)
 *      - "e-mail" : NSString*. 이메일(Hub만 사용, 옵션)
 *      - "gender" : NSString*. 성별(Hub만 사용, 옵션)
 *      - "newMessageCount" : 32bit signed. 자신의 정보일때만 데이터가 존재함. 받은 메시지의 갯수(Hub만 사용)
 *      - "newFriendCount" : 32bit signed. 자신의 정보일때만 데이터가 존재함. 새로 수락된 친구의 카운트(Hub만 사용)
 *      - "newRequestCount" : 32bit signed. 자신의 정보일때만 데이터가 존재함. 받은 친구 요청 카운트(Hub만 사용)
 *      .
 * - 유저가 여러 명일 경우
 *      - "list" : NSArray*. 유저들의 정보를 저장한 배열
 * @remark
 * 입력 인자
 * - "vid" : 둘중 하나의 타입으로 요청한다.
 *      - NSString*. 요청할 대상의 vid. 없을 경우 자신의 정보를 요청한다.
 *      - NSArray*. 요청할 대상들의 vid.
 * @see Sns
 * @author Byeong Yeol Jeon
 */
+(BOOL)User:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief 현재 사용자의 친구 리스트를 요청하고 결과를 공통 델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * - "game" : 게임 친구의 정보 리스트
 * - "other" : 게임하지 않는 친구의 정보 리스트
 * - 로컬 플랫폼일 경우 친구별 정보
 *      - "vid" : NSString*. 로그인센터 유니크 아이디
 *      - "nickName" : NSString*. sns 아이디
 *      - "alias" : NSString*. 개별 사용자가 설정한 이름
 *      - "facebookId" : NSString. 페이스북 아이디(Hub만 사용, 옵션)
 *      - "profileImageUrl" : NSString*. 프로필 이미지 다운로드 url
 *      - "country" : NSString*. 국가코드(Hub만 사용, 옵션)
 *      - "messsageBlocked" : bool. 메시지를 차단했는지의 여부
 * - "service"가 "facebook"일 경우 친구별 정보
 *      - "facebookId" : NSString*. 페이스북 유니크 아이디
 *      - "nickName" : NSString*. sns의 프로필 이름
 *      - "profileImageUrl" : NSString*. 프로필 이미지 다운로드 url
 *      - "country" : NSString*. 국가코드(Hub만 사용, 옵션)
 * - "service"가 "googleplus"일 경우 친구별 정보
 *      - "googleplusId" : NSString*. 페이스북 유니크 아이디
 *      - "nickName" : NSString*. sns의 프로필 이름
 *      - "profileImageUrl" : NSString*. 프로필 이미지 다운로드 url
 *      - "country" : NSString*. 국가코드(Hub만 사용, 옵션)
 * - "service"가 "sinaweibo"일 경우 친구별 정보
 *      - "sinaweiboId" : NSString*. 페이스북 유니크 아이디
 *      - "nickName" : NSString*. sns의 프로필 이름
 *      - "profileImageUrl" : NSString*. 프로필 이미지 다운로드 url
 *      - "country" : NSString*. 국가코드(Hub만 사용, 옵션)
 * @remark
 * 입력 인자
 * - "service" : NSString*. 없을 경우 베이스플랫폼(허브나 카카오)의 친구를 요청
 *      - "facebook" : 페이스북의 친구리스트를 요청.
 *      - "googleplus" : 구글플러스의 친구리스트를 요청.
 *      - "sinaweibo" : 시나웨이보의 친구리스트를 요청한다.
 * - "type" : NSString*. 요청할 친구 분류. 없을 경우 모든 친구를 요청.
 *      - "game" : 게임을 같이 하는 친구를 요청. 
 *      - "other" : 게임을 하지 않는 친구를 요청.
 *      - "all" : 모든 친구를 요청
 * @see Sns
 * @author Byeong Yeol Jeon
 */
+(BOOL)Friends:(id)arg error:(NSError**)error;
/**
 * @brief 현재 사용자의 친구 리스트를 요청하고 결과를 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * - "game" : 게임 친구의 정보 리스트
 * - "other" : 게임하지 않는 친구의 정보 리스트
 * - 로컬 플랫폼일 경우 친구별 정보
 *      - "vid" : NSString*. 로그인센터 유니크 아이디
 *      - "nickName" : NSString*. sns 아이디
 *      - "alias" : NSString*. 개별 사용자가 설정한 이름
 *      - "facebookId" : NSString. 페이스북 아이디(Hub만 사용, 옵션)
 *      - "profileImageUrl" : NSString*. 프로필 이미지 다운로드 url
 *      - "country" : NSString*. 국가코드(Hub만 사용, 옵션)
 *      - "messsageBlocked" : bool. 메시지를 차단했는지의 여부
 * - "service"가 "facebook"일 경우 친구별 정보
 *      - "facebookId" : NSString*. 페이스북 유니크 아이디
 *      - "nickName" : NSString*. sns의 프로필 이름
 *      - "profileImageUrl" : NSString*. 프로필 이미지 다운로드 url
 *      - "country" : NSString*. 국가코드(Hub만 사용, 옵션)
 * - "service"가 "googleplus"일 경우 친구별 정보
 *      - "googleplusId" : NSString*. 페이스북 유니크 아이디
 *      - "nickName" : NSString*. sns의 프로필 이름
 *      - "profileImageUrl" : NSString*. 프로필 이미지 다운로드 url
 *      - "country" : NSString*. 국가코드(Hub만 사용, 옵션)
 * - "service"가 "sinaweibo"일 경우 친구별 정보
 *      - "sinaweiboId" : NSString*. 페이스북 유니크 아이디
 *      - "nickName" : NSString*. sns의 프로필 이름
 *      - "profileImageUrl" : NSString*. 프로필 이미지 다운로드 url
 *      - "country" : NSString*. 국가코드(Hub만 사용, 옵션)
 * @remark
 * 입력 인자
 * - "service" : NSString*. 없을 경우 베이스플랫폼(허브나 카카오)의 친구를 요청
 *      - "facebook" : 페이스북의 친구리스트를 요청.
 *      - "googleplus" : 구글플러스의 친구리스트를 요청.
 *      - "sinaweibo" : 시나웨이보의 친구리스트를 요청한다.
 * - "type" : NSString*. 요청할 친구 분류. 없을 경우 모든 친구를 요청.
 *      - "game" : 게임을 같이 하는 친구를 요청.
 *      - "other" : 게임을 하지 않는 친구를 요청.
 *      - "all" : 모든 친구를 요청
 * @see Sns
 * @author Byeong Yeol Jeon
 */
+(BOOL)Friends:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief 대상에게 메시지를 발송하고 결과를 공통 델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * 입력 인자
 * - "vid" : 메시지를 받는 유저의 로그인 센터 유니크 아이디
 *      - NSString*. 단일 대상일 경우
 *      - NSArray*. 대상이 여러명일 경우.
 * - "message" : NSString*. 메시지의 내용
 * - "imageUrl" : NSString*. 전송될 이미지의 url
 * - "thumbnailUrl" : NSString*. 썸네일의 url
 * - "usePush" : bool. true일 경우 푸쉬가 발송된다.
 * - "invite" : bool. true일 경우 초대메시지를 전송한다.
 * .
 * \n 페이스북으로 보낼 경우
 * - "facebookId" : 메시지를 받는 유저의 페이스북 유니크 아이디
 *      - NSString*. 단일 대상일 경우
 *      - NSArray*. 대상이 여러명일 경우.
 * - "message" : NSString*. 메시지의 내용
 * .
 * \n 구글플러스로 보낼 경우
 * - "googleplusId" : 메시지를 받는 유저의 구글플러스 유니크 아이디
 *      - NSString*. 단일 대상일 경우
 *      - NSArray*. 대상이 여러명일 경우.
 * - "message" : NSString*. 메시지의 내용
 * .
 * \n 시나웨이보로 보낼 경우
 * - "sinaweiboId" : 메시지를 받는 유저의 시나 웨이보 유니크 아이디
 *      - NSString*. 단일 대상일 경우
 *      - NSArray*. 대상이 여러명일 경우.
 * - "message" : NSString*. 메시지의 내용
 * @author Byeong Yeol Jeon
 */
+(BOOL)Message:(id)arg error:(NSError**)error;
/**
 * @brief 대상에게 메시지를 발송한고 결과를 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * 입력 인자
 * - "vid" : 메시지를 받는 유저의 로그인 센터 유니크 아이디
 *      - NSString*. 단일 대상일 경우
 *      - NSArray*. 대상이 여러명일 경우.
 * - "message" : NSString*. 메시지의 내용
 * - "imageUrl" : NSString*. 전송될 이미지의 url
 * - "thumbnailUrl" : NSString*. 썸네일의 url
 * - "usePush" : bool. true일 경우 푸쉬가 발송된다.
 * - "invite" : bool. true일 경우 초대메시지를 전송한다.
 * .
 * \n 페이스북으로 보낼 경우
 * - "facebookId" : 메시지를 받는 유저의 페이스북 유니크 아이디
 *      - NSString*. 단일 대상일 경우
 *      - NSArray*. 대상이 여러명일 경우.
 * - "message" : NSString*. 메시지의 내용
 * .
 * \n 구글플러스로 보낼 경우
 * - "googleplusId" : 메시지를 받는 유저의 구글플러스 유니크 아이디
 *      - NSString*. 단일 대상일 경우
 *      - NSArray*. 대상이 여러명일 경우.
 * - "message" : NSString*. 메시지의 내용
 * .
 * \n 시나웨이보로 보낼 경우
 * - "sinaweiboId" : 메시지를 받는 유저의 시나 웨이보 유니크 아이디
 *      - NSString*. 단일 대상일 경우
 *      - NSArray*. 대상이 여러명일 경우.
 * - "message" : NSString*. 메시지의 내용
 * @author Byeong Yeol Jeon
 */
+(BOOL)Message:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief sns에 포스팅하고 결과를 공통 델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * 입력 인자
 * - "service" : NSString*. 없을 경우 베이스플랫폼(허브나 카카오)에 포스팅
 *      - "facebook" : 페이스북에 포스팅
 *      - "googleplus" : 구글플러스에 포스팅
 *      - "sinaweibo" : 시나웨이보에 포스팅
 * - "service"가 베이스 플랫폼일 경우
 *      - "message" : NSString*. 포스팅할 텍스트.
 *      - "image" : 포스팅할 이미지
 *          - NSString*. 이미지를 base64인코딩한 문자열
 *          - NSData*. 이미지 파일 데이터.
 *          - UIImage*. 이미지
 *      - "imageUrl" : NSString*. 포스팅할 이미지의 링크
 *      - "videoUrl" : NSString*. 포스팅할 비디오의 웹 링크
 * - "service"가 구글 플러스일 경우
 *      - "message" : NSString*. 포스팅할 텍스트.
 * - "service"가 페이스북 또는 시나 웨이보일 경우
 *      - "message" : NSString*. 포스팅할 텍스트.
 *      - "title" : NSString*. 포스팅 타이틀
 *      - "image" : 포스팅할 이미지
 *          - NSString*. 이미지를 base64인코딩한 문자열
 *          - NSData*. 이미지 파일 데이터.
 *          - UIImage*. 이미지
 *      - "imageUrl" : NSString*. 포스팅할 이미지의 링크
 *      - "appLinkUrl" : NSString*. 앱 링크 url
 * @author Byeong Yeol Jeon
 
 */
+(BOOL)Post:(id)arg error:(NSError**)error;
/**
 * @brief sns에 포스팅하고 결과를 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * 입력 인자
 * - "service" : NSString*. 없을 경우 베이스플랫폼(허브나 카카오)에 포스팅
 *      - "facebook" : 페이스북에 포스팅
 *      - "googleplus" : 구글플러스에 포스팅
 *      - "sinaweibo" : 시나웨이보에 포스팅
 * - "service"가 베이스 플랫폼일 경우
 *      - "message" : NSString*. 포스팅할 텍스트.
 *      - "image" : 포스팅할 이미지
 *          - NSString*. 이미지를 base64인코딩한 문자열
 *          - NSData*. 이미지 파일 데이터.
 *          - UIImage*. 이미지
 *      - "imageUrl" : NSString*. 포스팅할 이미지의 링크
 *      - "videoUrl" : NSString*. 포스팅할 비디오의 웹 링크
 * - "service"가 구글 플러스일 경우
 *      - "message" : NSString*. 포스팅할 텍스트.
 * - "service"가 페이스북 또는 시나 웨이보일 경우
 *      - "message" : NSString*. 포스팅할 텍스트.
 *      - "title" : NSString*. 포스팅 타이틀
 *      - "image" : 포스팅할 이미지
 *          - NSString*. 이미지를 base64인코딩한 문자열
 *          - NSData*. 이미지 파일 데이터.
 *          - UIImage*. 이미지
 *      - "imageUrl" : NSString*. 포스팅할 이미지의 링크
 *      - "appLinkUrl" : NSString*. 앱 링크 url
 * @author Byeong Yeol Jeon
 */
+(BOOL)Post:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief sns 웹 뷰를 출력하고 결과를 공통 델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * 입력 인자
 * - "type" : NSString*
 *      - "home" : 허브의 기본 첫 화면(피드)
 *      - "user" : 자신의 프로필 페이지
 *      - "game" : 자사 게임 페이지
 *      - "edit/profile/picture" : 프로필 이미지 변경 페이지
 *      - "edit/facebook" : 페이스북 로그인 연동 설정 페이지
 *      - "find_friends" : 친구추천 페이지
 * @author Byeong Yeol Jeon
 */
+(BOOL)Show:(id)arg error:(NSError**)error;
/**
 * @brief sns 웹 뷰를 출력하고 결과를 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * 입력 인자
 * - "type" : NSString*
 *      - "home" : 허브의 기본 첫 화면(피드)
 *      - "user" : 자신의 프로필 페이지
 *      - "game" : 자사 게임 페이지
 *      - "edit/profile/picture" : 프로필 이미지 변경 페이지
 *      - "edit/facebook" : 페이스북 로그인 연동 설정 페이지
 *      - "find_friends" : 친구추천 페이지
 * @author Byeong Yeol Jeon
 */
+(BOOL)Show:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief 허브에 저장된 데이터를 로드하고 결과를 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * - "data" : NSString*. base64로 인코딩된 데이터
 * - "hash" : NSString*. 원본 데이터의 MD5 Hash
 * @remark
 * 입력 인자 없음
 * @author Byeong Yeol Jeon
 */
+(BOOL)LoadData:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief 허브에 데이터를 저장하고 결과를 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * 입력 인자
 * - "data" : NSString*. base64로 인코딩된 데이터
 * - "hash" : NSString*. 원본 데이터의 MD5 Hash
 * @author Byeong Yeol Jeon
 */
+(BOOL)SaveData:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief 마지막에 로그인했던 타입을 가져온다.
 * @return "Platform","Guest",nil.
 * @remark
 * @author Jeen Woo Lee
 */
+(NSString *)GetLoginType;
+(BOOL)SetGuestUserInfo:(id)arg error:(NSError**)error;
+(BOOL)SetGuestUserInfo:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;

/**
 * @brief 로그인 센터에서 탈퇴하고 결과를 공통 델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark 입력 인자 없음
 * @see Sns
 * @author choiBK
 */
+(BOOL)UserDelete:(id)arg error:(NSError**)error;
/**
 * @brief 로그인 센터에서 탈퇴하고 결과를 공통 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark 입력 인자 없음
 * @see Sns
 * @author choiBK
 */
+(BOOL)UserDelete:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
@end

@interface C2SModulePush : NSObject
/**
 * @brief 로컬 푸쉬, 디바이스 토큰, 프로모션 코드를 등록하고 결과를 공통 델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * - "pushId" : 32bit signed. 제거된 푸쉬의 아이디
 * @remark
 * 입력 인자
 * - "deviceToken" : NSData*. 등록할 디바이스 토큰.
 * - "promotionCode" : bool. 등록할 프로모션 코드
 * - "pushId" : 32bit signed. 등록할 푸쉬 아이디
 * - "after" : 64bit signed. 얼마후에 푸쉬가 실행되는지의 값.
 * - "title" : NSString*. 푸쉬 타이틀.
 * - "message" : NSString*. 푸쉬 메시지.
 * - "sound" : NSString*. 푸쉬 사운드.
 * @author Byeong Yeol Jeon
 */
+(BOOL)Register:(id)arg error:(NSError**)error;
/**
 * @brief 로컬 푸쉬, 디바이스 토큰, 프로모션 코드를 등록하고 결과를 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * - "pushId" : 32bit signed. 제거된 푸쉬의 아이디
 * @remark
 * 입력 인자
 * - "deviceToken" : NSData*. 등록할 디바이스 토큰.
 * - "promotionCode" : bool. 등록할 프로모션 코드
 * - "pushId" : 32bit signed. 등록할 푸쉬 아이디
 * - "after" : 64bit signed. 얼마후에 푸쉬가 실행되는지의 값.
 * - "title" : NSString*. 푸쉬 타이틀.
 * - "message" : NSString*. 푸쉬 메시지.
 * - "sound" : NSString*. 푸쉬 사운드.
 * @author Byeong Yeol Jeon
 */
+(BOOL)Register:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief 등록한 로컬 푸쉬를 제거하고 결과를 공통 델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * - "pushId" : 32bit signed. 제거된 푸쉬의 아이디
 * @remark
 - "pushId" : 32bit signed. 제거할 푸쉬의 아이디
 * @see Push
 * @author Byeong Yeol Jeon
 */
+(BOOL)Unregister:(id)arg error:(NSError**)error;
/**
 * @brief 등록한 로컬 푸쉬를 제거하고 결과를 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * \n 결과 인자
 * - "pushId" : 32bit signed. 제거된 푸쉬의 아이디
 * @remark
 - "pushId" : 32bit signed. 제거할 푸쉬의 아이디
 * @see Push
 * @author Byeong Yeol Jeon
 */
+(BOOL)Unregister:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief 푸쉬 옵션을 선택하고 결과를 공통 델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark 입력 인자 없음
 * @see Push
 * @author Byeong Yeol Jeon
 */
+(BOOL)Description:(id)arg error:(NSError**)error;
/**
 * @brief 푸쉬 옵션을 선택하고 결과를 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark 입력 인자 없음
 * @see Push
 * @author Byeong Yeol Jeon
 */
+(BOOL)Description:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief iOS 12 이상에서 푸시 알림 권한을 뚜렷이 요청하고 결과를 공통 델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark 입력 인자 없음
 * @see Push
 * @author sjyong
 */
+(BOOL)RequestPermission:(id)arg error:(NSError**)error;
/**
 * @brief iOS 12 이상에서 푸시 알림 권한을 뚜렷이 요청하고 결과를 공통 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in] error 실행 중 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark 입력 인자 없음
 * @see Push
 * @author sjyong
 */
+(BOOL)RequestPermission:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary*, NSError*))completionHandler;

@end

@interface C2SModulePromotion : NSObject
/**
 * @brief 프로모션 관련 뷰를 출력하고 결과를 공통 델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * - "type" : NSString*.
 *      - "notice" : 머큐리 공지를 출력한다.
 *      - "event" : 머큐리 이벤트를 출력한다.
 * - "forced" : bool. true일 경우 24시간 다시보지 않기를 무시한다.
 * @author Byeong Yeol Jeon
 */
+(BOOL)Show:(id)arg error:(NSError**)error;
/**
 * @brief 프로모션 관련 뷰를 출력하고 결과를 completionHandler로 전달 받는다
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * - "type" : NSString*. 
 *      - "notice" : 머큐리 공지를 출력한다.
 *      - "event" : 머큐리 이벤트를 출력한다.
 * - "forced" : bool. true일 경우 24시간 다시보지 않기를 무시한다.
 * @author Byeong Yeol Jeon
 */
+(BOOL)Show:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;

/**
 * @brief 프로모션 관련 정보를 얻어오고 completionHandler로 전달 받는다
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * - "type" : NSString*.
 *      - "notice" : 머큐리 공지를 출력한다.
 *      - "event" : 머큐리 이벤트를 출력한다.
 * - "forced" : bool. true일 경우 24시간 다시보지 않기를 무시한다.
 * @author Byeong Yeol Jeon
 */
+(BOOL)Info:(id)arg error:(NSError**)error;

/**
 * @brief 프로모션 관련 정보를 얻어오고 completionHandler로 전달 받는다
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * - "type" : NSString*.
 *      - "notice" : 머큐리 공지를 출력한다.
 *      - "event" : 머큐리 이벤트를 출력한다.
 * - "forced" : bool. true일 경우 24시간 다시보지 않기를 무시한다.
 * @author Byeong Yeol Jeon
 */
+(BOOL)Info:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;

/**
* @brief 프로모션 관련 배너 정보를 얻어오고 completionHandler로 전달 받는다
* @param[in] arg remark 참조
* @param[in, out] error 실행시 발생한 에러
* @param[in] completionHandler 결과를 통지받을 블럭 핸들러
* @return YES일 경우 호출 성공. NO일 경우 호출 실패.
* @remark
* - "campaignType" : NSString*.
*      - "event"  : 이벤트 타입
*      - "notice" : 공지 타입
* - "bannerType" : NSString*.
*      - "great"   : 대배너
*      - "small"   : 일반배너 (띠배너)
*      - "rolling" : 롤링 배너
* @author daunjoung
*/
+ (BOOL)GetBannerInfo:(id) arg error:(NSError**)error;

/**
* @brief 프로모션 관련 배너 정보를 얻어오고 completionHandler로 전달 받는다
* @param[in] arg remark 참조
* @param[in, out] error 실행시 발생한 에러
* @param[in] completionHandler 결과를 통지받을 블럭 핸들러
* @return YES일 경우 호출 성공. NO일 경우 호출 실패.
* @remark
* - "campaignType" : NSString*.
*      - "event"  : 이벤트 타입
*      - "notice" : 공지 타입
* - "bannerType" : NSString*.
*      - "great"   : 대배너
*      - "small"   : 일반배너 (띠배너)
*      - "rolling" : 롤링 배너
* @author daunjoung
*/
+ (BOOL)GetBannerInfo:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;

//1.9.0
+(BOOL)Acquisition:(id)arg error:(NSError**)error;

/**
 * @brief 집객 관련 정보를 얻어오고 completionHandler로 전달 받는다
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * @author Choi BK
 */
+(BOOL)Acquisition:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
@end

@interface C2SModuleTracker : NSObject

/**
 * @brief 트래킹 이벤트를 호출하고 결과를 공통델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @return YES일 경우 성공, NO일 경우 실패.
 * @remark
 * - "event" : 이벤트 이름
 * - "revenu" : 설정시 구매이벤트를 보낸다.
 *      - "title"       : 상품 이름
 *      - "description" : 상품 설명
 *      - "currency"    : 상품가격의 국가코드
 *      - "price"       : 상품가격
 *      - "itemCount"   : 아이템 개수
 * @author cocororo
 */
+(BOOL)SendEvent:(id)arg error:(NSError**)error;
/**
 * @brief 트래킹 이벤트를 호출하고 결과를 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * - "event" : 이벤트 이름
 * - "revenu" : 설정시 구매이벤트를 보낸다.
 *      - "title"       : 상품 이름
 *      - "description" : 상품 설명
 *      - "currency"    : 상품가격의 국가코드
 *      - "price"       : 상품가격
 *      - "itemCount"   : 아이템 개수
 * @author cocororo
 */
+(BOOL)SendEvent:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief 트래킹 모듈의 사용여부를 설정하고 결과를 공통델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * - "MAT" : MAT 사용 여부
 * - "Adjust" : Adjust 사용 여부
 * - "Partytrack" : Partytrack 사용 여부
 * @author cocororo
 */
+(BOOL)SetEnable:(id)arg error:(NSError**)error;
/**
 * @brief 트래킹 모듈의 사용여부를 설정하고 결과를 completionHandler로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * - "MAT" : MAT 사용 여부
 * - "Adjust" : Adjust 사용 여부
 * - "Partytrack" : Partytrack 사용 여부
 * @author cocororo
 */
+(BOOL)SetEnable:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;

@end

#define C2SModuleArgFunc(x); \
+(BOOL)x:(id)arg error:(NSError**)error; \
+(BOOL)x:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;

@interface C2SModuleEngagement : NSObject

/**
 * @brief DeepLink, Push를 통해 전달 받은 URL Scheme의 처리를 요청하고, 공통 델리게이트로 결과를 받는다.
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * - "scheme" : NSString*. 처리를 요청할 URL Scheme.
 * - "immediately" : 결과값을 공통 델리게이트로 즉시 받을지의 여부. (Default == false)
 *      TRUE : 공통 델리게이트로 이벤트 정보를 받는다.
 *      FALSE : 공통 델리게이트로 요청한 정보(Arg)를 돌려 받으며, 이벤트 정보 필요시 GetEvents를 호출해야 한다.
 * @author Hysterit
 */
+(BOOL)ProcessScheme:(id)arg error:(NSError**)error;

/**
 * @brief DeepLink, Push를 통해 전달 받은 Scheme의 처리를 요청하고, completionHandler로 결과를 받는다.
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * - "scheme" : NSString*. 처리를 요청할 URL Scheme.
 * - "immediately" : 결과값을 completionHandler로 즉시 받을지의 여부. (Default == false)
 *      TRUE : completionHandler로 이벤트 정보를 받는다.
 *      FALSE : completionHandler로 요청한 정보(Arg)를 돌려 받으며, 이벤트 정보 필요시 GetEvents를 호출해야 한다.
 * @author Hysterit
 */
+(BOOL)ProcessScheme:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;

/**
 * @brief
 ProcessScheme를 통해 처리 된 이벤트 정보를 공통 델리게이트로 받는다.
 처리 된 이벤트 정보를 수동으로 가져오는 API이다. (Pull Request)
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * @author Hysterit
 * Deprecated in HIVE SDK 1.11.0
 */
//+(BOOL)GetEvents:(id)arg error:(NSError**)error;

/**
 * @brief
 ProcessScheme를 통해 처리 된 이벤트 정보를 completionHandler로 받는다.
 처리 된 이벤트 정보를 수동으로 가져오는 API이다. (Pull Request)
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * @author Hysterit
 * Deprecated in HIVE SDK 1.11.0
 */
//+(BOOL)GetEvents:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;

/**
 * @brief 쿠폰 소모 처리를 요청하고, 공통 델리게이트로 결과를 받는다.
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * - "coupon" : NSString*. 사용 처리할 쿠폰의 코드.
 * - "lang" : NSString*. 게임 언어 코드.
 * - "etc" : NSString*. JSON 형태의 추가 정보. ex)@"\"{\"key\":\"value\"}\""
 * @author Hysterit
 */
+(BOOL)ConsumeCoupon:(id)arg error:(NSError**)error;

/**
 * @brief 쿠폰 소모 처리를 요청하고, completionHandler로 결과를 받는다.
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * - "coupon" : NSString*. 사용 처리할 쿠폰의 코드.
 * - "lang" : NSString*. 게임 언어 코드.
 * - "etc" : NSString*. JSON 형태의 추가 정보.
 * @author Hysterit
 */
+(BOOL)ConsumeCoupon:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief Engagement 사용 준비가 되는 경우 호출하는 함수.
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * @author Daun Joung
 */
+(BOOL)SetReady:(id)arg error:(NSError**)error;
/**
 * @brief Engagement 사용 준비가 되는 경우 호출하는 함수.
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * @author Daun Joung
 */
+(BOOL)SetReady:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
@end


@interface C2SModuleHivePromotion : NSObject
/**
 * @brief 프로모션 2.0 관련 뷰를 출력하고 결과를 공통 델리게이트로 전달 받는다.
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * - "type" : NSString*.
 *      - "news" : 머큐리 새소식을 출력한다.
 *      - "notice" : 머큐리 공지를 출력한다.
 *      - "banner" : 머큐리 전면배너를 출력한다.
 *      - "customView" : 커스텀웹븊를 출력한다.
 *      - "customBoard" : 커스텀보드를 출력한다.
 *      - "spot" : 스팟배너를 출력한다.
 *      - "direct" : 웹뷰 리다이렉트를 출력한다.
 *      - "review" : 리뷰팝업을 출력한다.
 *      - "offerwall" : 오퍼월웹뷰를 출력한다.
 * - "forced" : bool. true일 경우 24시간 다시보지 않기를 무시한다.
 * - "contentKey" : NSString*. 커스텀웹뷰,커스텀보드,스팟배너,리다이렉트 웹뷰의 컨텐츠 key.
 * @author pnpsinki
 */
+(BOOL)Show:(id)arg error:(NSError**)error;
/**
 * @brief 프로모션 2.0 관련 뷰를 출력하고 결과를 completionHandler로 전달 받는다
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * - "type" : NSString*.
 *      - "news" : 머큐리 새소식을 출력한다.
 *      - "notice" : 머큐리 공지를 출력한다.
 *      - "banner" : 머큐리 전면배너를 출력한다.
 *      - "customView" : 커스텀웹븊를 출력한다.
 *      - "customBoard" : 커스텀보드를 출력한다.
 *      - "spot" : 스팟배너를 출력한다.
 *      - "direct" : 웹뷰 리다이렉트를 출력한다.
 *      - "review" : 리뷰팝업을 출력한다.
 * - "forced" : bool. true일 경우 24시간 다시보지 않기를 무시한다.
 * - "contentKey" : NSString*. 커스텀웹뷰,커스텀보드,스팟배너,리다이렉트 웹뷰의 컨텐츠 key.
 * @author pnpsinki
 */
+(BOOL)Show:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;

/**
 * @brief 프로모션 2.0 관련 정보를 얻어오고 completionHandler로 전달 받는다
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * - "type" : NSString*.
 *      - "badge" : 뱃지 정보를 호출한다.
 *      - "customView" : 커스텀웹븊 정보를 출력한다.
 *      - "customBoard" : 커스텀보드 정보를 출력한다.
 *      - "spot" : 스팟배너 정보를 출력한다.
 *      - "direct" : 웹뷰 리다이렉트 정보를 출력한다.
 * - "contentKey" : NSString*. 커스텀웹뷰,커스텀보드,스팟배너,리다이렉트 웹뷰의 컨텐츠 key.
 * @author pnpsinki
 */
+(BOOL)Info:(id)arg error:(NSError**)error;
/**
 * @brief 프로모션 2.0 관련 뷰를 출력하고 결과를 completionHandler로 전달 받는다
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @remark
 * - "type" : NSString*.
 *      - "badge" : 뱃지 정보를 호출한다.
 *      - "customView" : 커스텀웹븊 정보를 출력한다.
 *      - "customBoard" : 커스텀보드 정보를 출력한다.
 *      - "spot" : 스팟배너 정보를 출력한다.
 *      - "direct" : 웹뷰 리다이렉트 정보를 출력한다.
 * - "contentKey" : NSString*. 커스텀웹뷰,커스텀보드,스팟배너,리다이렉트 웹뷰의 컨텐츠 key.
 * @author pnpsinki
 */
+(BOOL)Info:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;

/**
* @brief 프로모션 2.0 관련 배너 정보를 얻어오고 completionHandler로 전달 받는다 Added in HIVE SDK v1.18.3
* @param[in] arg remark 참조
* @param[in, out] error 실행시 발생한 에러
* @param[in] completionHandler 결과를 통지받을 블럭 핸들러
* @return YES일 경우 호출 성공. NO일 경우 호출 실패.
* @remark
* - "campaignType" : NSString*.
*      - "event"  : 이벤트 타입
*      - "notice" : 공지 타입
* - "bannerType" : NSString*.
*      - "great"   : 대배너
*      - "small"   : 일반배너 (띠배너)
*      - "rolling" : 롤링 배너
* @author daunjoung
*/
+ (BOOL)GetBannerInfo:(id) arg error:(NSError**)error;

/**
* @brief 프로모션 2.0 관련 배너 정보를 얻어오고 completionHandler로 전달 받는다 Added in HIVE SDK v1.18.3
* @param[in] arg remark 참조
* @param[in, out] error 실행시 발생한 에러
* @param[in] completionHandler 결과를 통지받을 블럭 핸들러
* @return YES일 경우 호출 성공. NO일 경우 호출 실패.
* @remark
* - "campaignType" : NSString*.
*      - "event"  : 이벤트 타입
*      - "notice" : 공지 타입
* - "bannerType" : NSString*.
*      - "great"   : 대배너
*      - "small"   : 일반배너 (띠배너)
*      - "rolling" : 롤링 배너
* @author daunjoung
*/
+ (BOOL)GetBannerInfo:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;


/**
 * @brief 프로모션 2.0 관련 GetffewallState 결과를 completionHandler로 전달 받는다
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @author pnpsinki
 */
+(BOOL)GetOfferwallState:(id)arg error:(NSError**)error;
/**
 * @brief 프로모션 2.0 관련 GetffewallState 결과를 completionHandler로 전달 받는다
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @author pnpsinki
 */
+(BOOL)GetOfferwallState:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief 프로모션 2.0 관련 AdditionalInfo를 세팅한다.
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @author pnpsinki
 */
+(BOOL)SetAdditionalInfo:(id)arg error:(NSError**)error;
/**
 * @brief 프로모션 2.0 관련 AdditionalInfo를 결과를 completionHandler로 전달 받는다
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @author pnpsinki
 */
+(BOOL)SetAdditionalInfo:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
/**
 * @brief 프로모션 2.0 관련 UpdatePromotionData를 호출한다.
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @author jschoi
 */
+(BOOL)UpdatePromotionData:(id)arg error:(NSError**)error;
/**
 * @brief 프로모션 2.0 관련 UpdatePromotionData를 결과를 completionHandler로 전달 받는다
 * @param[in] arg remark 참조
 * @param[in, out] error 실행시 발생한 에러
 * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
 * @return YES일 경우 성공. NO일 경우 실패.
 * @author jschoi
 */
+(BOOL)UpdatePromotionData:(id)arg error:(NSError**)error completionHandler:(void(^)(NSDictionary *, NSError*))completionHandler;
@end

@interface C2SModuleInappV4 : NSObject

C2SModuleArgFunc(ItemInfo);
C2SModuleArgFunc(Purchase);
C2SModuleArgFunc(Restore);
C2SModuleArgFunc(TransactionFinish);
C2SModuleArgFunc(CheckPromotePurchase);
C2SModuleArgFunc(GetSelectedMarket);
C2SModuleArgFunc(GetAccountUuid);
C2SModuleArgFunc(MarketConnect);

@end

#endif // __OBJC__

#endif
