/**
 * @file    Module_Android.cpp
 * @brief   통합 모듈 Android NDK 헤더
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 * @date    2014/05/14
 */
#include "Module_Android.h"

#if __ANDROID__
#include <jni.h>
#include <android/log.h>
#include <cstdlib>
#include "../C2SModule.h"
#include "../protection/C2SModuleSecureVariable.h"

/**
 * @brief   java vm의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
JavaVM* C2SModuleJavaVMInstance = NULL;
/**
 * @brief   com.com2us.module.C2SModuleSocial class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
jclass C2SModuleSocialClass = NULL;
/**
 * @brief   com.com2us.module.C2SModuleSns class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
jclass C2SModuleSnsClass = NULL;
/**
 * @brief   com.com2us.module.C2SModuleInapp class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
//jclass C2SModuleInappClass = NULL; // remove 1.19.1
/**
 * @brief   com.com2us.module.C2SModulePush class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
jclass C2SModulePushClass = NULL;
/**
 * @brief   com.com2us.module.C2SModulePromotion class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
jclass C2SModulePromotionClass = NULL;
/**
 * @brief   com.com2us.module.C2SModulePromotion class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
jclass C2SModuleEngagementClass = NULL;

// add 1.5.0 - tracking
/**
 * @brief   com.com2us.module.C2SModuleTracker class의 인스턴스
 * @author  Hyun Seok Kim
 * @ingroup Android
 */
jclass C2SModuleTrackerClass = NULL;

/**
 * @brief   com.com2us.module.C2SModuleArgument class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
jclass C2SModuleArgumentClass = NULL;
/**
 * @brief   com.com2us.module.C2SModuleResultDelegateJNI class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
jclass C2SModuleResultDelegateJNIClass = NULL;
/**
 * @brief   com.com2us.module.C2SModuleError class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
jclass C2SModuleErrorClass = NULL;
/**
 * @brief   com.com2us.module.C2SModuleCompletionHandlerJNI class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
jclass C2SModuleCompletionHandlerJNIClass = NULL;

// add HiveIAP
/**
 * @brief   com.com2us.module.C2SModuleHiveIAP class의 인스턴스
 * @author  Hyun Seok Kim
 * @ingroup Android
 */
//jclass C2SModuleHiveIAPClass = NULL; // remove 1.19.1
/**
 * @brief   com.com2us.module.C2SModuleHivePromotion class의 인스턴스
 * @author  pnpsinki
 * @ingroup Android
 */
jclass C2SModuleHivePromotionClass = NULL;

/**
 * @brief   com.com2us.module.C2SModuleInappV4 class의 인스턴스 (HIVE SDK 1.18.4에서 추가)
 * @author  pnpsinki
 * @ingroup Android
 */
jclass C2SModuleInappV4Class = NULL;

/**
 * @brief   android.graphics.Bitmap class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
jclass C2SModuleBitmapClass = NULL;
/**
 * @brief   android.graphics.BitmapFactory class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
jclass C2SModuleBitmapFactoryClass = NULL;

//jclass C2SModuleThrowableClass = NULL;
/**
 * @brief   java.lang.Boolean class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
jclass C2SModuleBooleanClass = NULL;
/**
 * @brief   java.lang.Character class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
jclass C2SModuleCharClass = NULL;
/**
 * @brief   java.lang.Byte class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
jclass C2SModuleByteClass = NULL;
/**
 * @brief   java.lang.Short class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
jclass C2SModuleShortClass = NULL;
/**
 * @brief   java.lang.Integer class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
jclass C2SModuleIntClass = NULL;
/**
 * @brief   java.lang.Long class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
jclass C2SModuleLongClass = NULL;
/**
 * @brief   java.util.Vector class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
jclass C2SModuleVectorClass = NULL;
/**
 * @brief   java.lang.OutOfMemoryError class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
static jclass C2SModuleOutOfMemoryErrorClass = NULL;
/**
 * @brief   java.lang.NullPointerException class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
static jclass C2SModuleNullPointerExceptionClass = NULL;
/**
 * @brief   java.lang.VerifyError class의 인스턴스
 * @author  Byeong Yeol Jeon
 * @ingroup Android
 */
static jclass C2SModuleVerifyErrorClass = NULL;

#define JAVA_CLASS_INITIALIZE(a, b) \
{ \
	jobject tmp = env->FindClass(b); \
\
	if (NULL == tmp) { \
		__android_log_print(ANDROID_LOG_ERROR, "VMInitialize", b); \
		exit(-1); \
	} \
\
	a = (jclass) env->NewGlobalRef(tmp); \
	env->DeleteLocalRef(tmp); \
}

#ifdef __cplusplus
extern "C" {
#endif
    /**
     * @brief NDK를 사용하기 위해 java에서 호출해야하는 NDK 초기화 함수.
     * @param[in] env jni env
     * @param[in] kClass jni class
     * @author Byeong Yeol Jeon
     * @ingroup Android
     */
    JNIEXPORT void JNICALL Java_com_com2us_module_C2SModule_VMInitialize(
                                                                         JNIEnv* env, jobject kClass) {
        env->GetJavaVM(&C2SModuleJavaVMInstance);
        
        if (NULL == C2SModuleJavaVMInstance)
            exit(-1);
        
        // java class를 미리 구한다.
        JAVA_CLASS_INITIALIZE(C2SModuleSocialClass, "com/com2us/module/C2SModuleSocial");
        JAVA_CLASS_INITIALIZE(C2SModuleSnsClass, "com/com2us/module/C2SModuleSns");
        //JAVA_CLASS_INITIALIZE(C2SModuleInappClass, "com/com2us/module/C2SModuleInapp"); // remove 1.19.1
        JAVA_CLASS_INITIALIZE(C2SModulePushClass, "com/com2us/module/C2SModulePush");
        JAVA_CLASS_INITIALIZE(C2SModulePromotionClass, "com/com2us/module/C2SModulePromotion");
        JAVA_CLASS_INITIALIZE(C2SModuleEngagementClass, "com/com2us/module/C2SModuleEngagement");
        JAVA_CLASS_INITIALIZE(C2SModuleTrackerClass, "com/com2us/module/C2SModuleTracker"); // add 1.5.0 - tracking
        JAVA_CLASS_INITIALIZE(C2SModuleArgumentClass, "com/com2us/module/C2SModuleArgument");
        JAVA_CLASS_INITIALIZE(C2SModuleResultDelegateJNIClass, "com/com2us/module/C2SModuleResultDelegateJNI");
        JAVA_CLASS_INITIALIZE(C2SModuleErrorClass, "com/com2us/module/C2SModuleError");
        JAVA_CLASS_INITIALIZE(C2SModuleCompletionHandlerJNIClass, "com/com2us/module/C2SModuleCompletionHandlerJNI");
        //JAVA_CLASS_INITIALIZE(C2SModuleHiveIAPClass, "com/com2us/module/C2SModuleHiveIAP"); // add HiveIAP // remove 1.19.1
        JAVA_CLASS_INITIALIZE(C2SModuleHivePromotionClass, "com/com2us/module/C2SModuleHivePromotion");	// add Promotion2.0
        JAVA_CLASS_INITIALIZE(C2SModuleInappV4Class, "com/com2us/module/C2SModuleInappV4");	// add InappV4

        JAVA_CLASS_INITIALIZE(C2SModuleBitmapClass, "android/graphics/Bitmap");
        JAVA_CLASS_INITIALIZE(C2SModuleBitmapFactoryClass, "android/graphics/BitmapFactory");
        
        //	JAVA_CLASS_INITIALIZE(C2SModuleThrowableClass, "java/lang/Throwable");
        JAVA_CLASS_INITIALIZE(C2SModuleBooleanClass, "java/lang/Boolean");
        JAVA_CLASS_INITIALIZE(C2SModuleCharClass, "java/lang/Character");
        JAVA_CLASS_INITIALIZE(C2SModuleByteClass, "java/lang/Byte");
        JAVA_CLASS_INITIALIZE(C2SModuleShortClass, "java/lang/Short");
        JAVA_CLASS_INITIALIZE(C2SModuleIntClass, "java/lang/Integer");
        JAVA_CLASS_INITIALIZE(C2SModuleLongClass, "java/lang/Long");
        JAVA_CLASS_INITIALIZE(C2SModuleVectorClass, "java/util/Vector");
        
        JAVA_CLASS_INITIALIZE(C2SModuleOutOfMemoryErrorClass, "java/lang/OutOfMemoryError");
        JAVA_CLASS_INITIALIZE(C2SModuleNullPointerExceptionClass, "java/lang/NullPointerException");
        JAVA_CLASS_INITIALIZE(C2SModuleVerifyErrorClass, "java/lang/VerifyError");
        
        // 성공했다면 로그를 출력
        __android_log_print(ANDROID_LOG_INFO, "VMInitialize", "OK");
    }
    
    /**
     * @brief NDK를 통해서 java 공통 델리게이트를 등록했을때 호출되는 공통 델리게이트 브릿지함수
     * @param[in] env jni env
     * @param[in] kClass jni class
     * @param[in] func 호출될 함수 포인터
     * @param[in] api 통지된 api의 종류
     * @param[in] jsonArg 결과 json 문자열 인자들
     * @param[in] code 발생한 에러의 코드
     * @param[in] message 발생한 에러의 메시지
     * @author Byeong Yeol Jeon
     * @ingroup Android
     */
//    JNIEXPORT void JNICALL Java_com_com2us_module_C2SModuleJNI_JNIResultDelegate(JNIEnv* env, jobject kClass, jlong func, jint api, jstring jsonArg, jint code, jstring message) {
    JNIEXPORT void JNICALL Java_com_com2us_module_C2SModuleResultDelegateJNI_JNIResultDelegate(JNIEnv* env, jobject kClass, jlong func, jint api, jstring jsonArg, jint code, jstring message) {

        if (0 == func)
            return;
        
        const char* szArg;
        const char* szMessage;
        
        if (NULL != jsonArg) {
            szArg = env->GetStringUTFChars(jsonArg, NULL);
        } else {
            szArg = NULL;
        }
        
        if (NULL != message) {
            szMessage = env->GetStringUTFChars(message, NULL);
        } else {
            szMessage = NULL;
        }
        
        CommonResultDelegate function = (CommonResultDelegate) func;
        function((C2SModuleApi) api, szArg, (C2SModuleErrorCode) code, szMessage);
        
        if (NULL != jsonArg) {
            env->ReleaseStringUTFChars(jsonArg, szArg);
        }
        
        if (NULL != message) {
            env->ReleaseStringUTFChars(message, szMessage);
        }
    }
    
    extern void C2SModuleCompletionHandlerJNI_JNIComplete(JNIEnv* env, jobject object, jlong handler, jstring jsonArg, jint code, jstring message);
    
    /**
     * @brief NDK를 통해서 java 커스텀 핸들러를 등록했을때 호출되는 커스텀 핸들러 브릿지함수
     * @param[in] env jni env
     * @param[in] object jni class
     * @param[in] handler 호출된 커스텀 핸들러 인스턴스
     * @param[in] jsonArg 결과 json 문자열 인자들
     * @param[in] code 발생한 에러의 코드
     * @param[in] message 발생한 에러의 메시지
     * @author Byeong Yeol Jeon
     * @ingroup Android
     */
    JNIEXPORT void JNICALL Java_com_com2us_module_C2SModuleCompletionHandlerJNI_JNIComplete(JNIEnv* env, jobject object, jlong handler, jstring jsonArg, jint code, jstring message) {
        
        C2SModuleCompletionHandlerJNI_JNIComplete(env, object, handler, jsonArg, code, message);
    }
    
    /**
     * @brief 암호화 인스턴스를 생성한다.
     * @param[in] env jni env
     * @param[in] object jni class
     * @return 생성된 인스턴스
     * @author Byeong Yeol Jeon
     * @ingroup Android Protection
     */
    JNIEXPORT jlong JNICALL Java_com_com2us_module_secure_C2SModuleSecureVariable_Create(JNIEnv* env, jclass object)
    {
        C2SModuleSecureId tmp;
        C2SModuleSecureVariableCreate(&tmp);
        return (jlong)tmp;
    }
    
    /**
     * @brief 암호화 인스턴스의 핸들을 제거한다.
     * @param[in] env jni env
     * @param[in] object jni class
     * @param[in] handle 암호화 인스턴스 핸들
     * @author Byeong Yeol Jeon
     * @ingroup Android Protection
     */
    JNIEXPORT void JNICALL Java_com_com2us_module_secure_C2SModuleSecureVariable_Release(JNIEnv* env, jclass object, jlong handle)
    {
        if(0 == handle)
            return;
        
        C2SModuleSecureId tmp = (C2SModuleSecureId)handle;
        C2SModuleSecureVariableRelease(&tmp);
    }
    
    /**
     * @brief 에러코드로 안드로이드 exception을 발생시킨다.
     * @param[in] env jni env
     * @param[in] result 에러 코드
     * @author Byeong Yeol Jeon
     * @ingroup Android
     */
    static void C2SModuleSecureErrorTriats(JNIEnv* env, C2SModuleErrorCode result)
    {
        switch(result)
        {
            case C2SModuleErrorCode_InvalidArg:
                env->ThrowNew(C2SModuleNullPointerExceptionClass, "invalid arg");
                break;
            case C2SModuleErrorCode_Broken:
                env->ThrowNew(C2SModuleVerifyErrorClass, "cheating");
                break;
            case C2SModuleErrorCode_MemoryExceed:
                env->ThrowNew(C2SModuleOutOfMemoryErrorClass, "need memory!");
                break;
            default:
                break;
        }
    }
    
    /**
     * @brief 핸들에 저장된 char 데이터를 구한다.
     * @param[in] env jni env
     * @param[in] object jni class
     * @param[in] handle 암호화 인스턴스 핸들
     * @return 암호화 되어있던 데이터
     * @author Byeong Yeol Jeon
     * @ingroup Android Protection
     */
    JNIEXPORT jchar JNICALL Java_com_com2us_module_secure_Character_Get(JNIEnv* env, jclass object, jlong handle)
    {
        C2SModuleSecureId tmp = (C2SModuleSecureId)handle;
        jchar val;
        C2SModuleErrorCode result = C2SModuleSecureVariableGet(tmp, &val, sizeof(val));
        
        C2SModuleSecureErrorTriats(env, result);
        return val;
    }
    
    /**
     * @brief 핸들에 char 데이터를 암호화하여 저장한다.
     * @param[in] env jni env
     * @param[in] object jni class
     * @param[in] handle 암호화 인스턴스 핸들
     * @param[in] value 저장할 데이터
     * @author Byeong Yeol Jeon
     * @ingroup Android Protection
     */
    JNIEXPORT void JNICALL Java_com_com2us_module_secure_Character_Set(JNIEnv* env, jclass object, jlong handle, jchar value)
    {
        C2SModuleSecureId tmp = (C2SModuleSecureId)handle;
        
        C2SModuleErrorCode result = C2SModuleSecureVariableSet(tmp, &value, sizeof(value));
        
        C2SModuleSecureErrorTriats(env, result);
    }
    
    /**
     * @brief 핸들에 저장된 short 데이터를 구한다.
     * @param[in] env jni env
     * @param[in] object jni class
     * @param[in] handle 암호화 인스턴스 핸들
     * @return 암호화 되어있던 데이터
     * @author Byeong Yeol Jeon
     * @ingroup Android Protection
     */
    JNIEXPORT jshort JNICALL Java_com_com2us_module_secure_Short_Get(JNIEnv* env, jclass object, jlong handle)
    {
        C2SModuleSecureId tmp = (C2SModuleSecureId)handle;
        jshort val;
        C2SModuleErrorCode result = C2SModuleSecureVariableGet(tmp, &val, sizeof(val));
        
        C2SModuleSecureErrorTriats(env, result);
        return val;
    }
    
    /**
     * @brief 핸들에 short 데이터를 암호화하여 저장한다.
     * @param[in] env jni env
     * @param[in] object jni class
     * @param[in] handle 암호화 인스턴스 핸들
     * @param[in] value 저장할 데이터
     * @author Byeong Yeol Jeon
     * @ingroup Android Protection
     */
    JNIEXPORT void JNICALL Java_com_com2us_module_secure_Short_Set(JNIEnv* env, jclass object, jlong handle, jshort value)
    {
        C2SModuleSecureId tmp = (C2SModuleSecureId)handle;
        
        C2SModuleErrorCode result = C2SModuleSecureVariableSet(tmp, &value, sizeof(value));
        
        C2SModuleSecureErrorTriats(env, result);
    }

    /**
     * @brief 핸들에 저장된 integer 데이터를 구한다.
     * @param[in] env jni env
     * @param[in] object jni class
     * @param[in] handle 암호화 인스턴스 핸들
     * @return 암호화 되어있던 데이터
     * @author Byeong Yeol Jeon
     * @ingroup Android Protection
     */
    JNIEXPORT jint JNICALL Java_com_com2us_module_secure_Integer_Get(JNIEnv* env, jclass object, jlong handle)
    {
        C2SModuleSecureId tmp = (C2SModuleSecureId)handle;
        jint val;
        C2SModuleErrorCode result = C2SModuleSecureVariableGet(tmp, &val, sizeof(val));
        
        C2SModuleSecureErrorTriats(env, result);
        return val;
    }
    
    /**
     * @brief 핸들에 integer 데이터를 암호화하여 저장한다.
     * @param[in] env jni env
     * @param[in] object jni class
     * @param[in] handle 암호화 인스턴스 핸들
     * @param[in] value 저장할 데이터
     * @author Byeong Yeol Jeon
     * @ingroup Android Protection
     */
    JNIEXPORT void JNICALL Java_com_com2us_module_secure_Integer_Set(JNIEnv* env, jclass object, jlong handle, jint value)
    {
        C2SModuleSecureId tmp = (C2SModuleSecureId)handle;
        
        C2SModuleErrorCode result = C2SModuleSecureVariableSet(tmp, &value, sizeof(value));
        
        C2SModuleSecureErrorTriats(env, result);
    }

    /**
     * @brief 핸들에 저장된 long 데이터를 구한다.
     * @param[in] env jni env
     * @param[in] object jni class
     * @param[in] handle 암호화 인스턴스 핸들
     * @return 암호화 되어있던 데이터
     * @author Byeong Yeol Jeon
     * @ingroup Android Protection
     */
    JNIEXPORT jlong JNICALL Java_com_com2us_module_secure_Long_Get(JNIEnv* env, jclass object, jlong handle)
    {
        C2SModuleSecureId tmp = (C2SModuleSecureId)handle;
        jlong val;
        C2SModuleErrorCode result = C2SModuleSecureVariableGet(tmp, &val, sizeof(val));
        
        C2SModuleSecureErrorTriats(env, result);
        return val;
    }
    
    /**
     * @brief 핸들에 long 데이터를 암호화하여 저장한다.
     * @param[in] env jni env
     * @param[in] object jni class
     * @param[in] handle 암호화 인스턴스 핸들
     * @param[in] value 저장할 데이터
     * @author Byeong Yeol Jeon
     * @ingroup Android Protection
     */
    JNIEXPORT void JNICALL Java_com_com2us_module_secure_Long_Set(JNIEnv* env, jclass object, jlong handle, jlong value)
    {
        C2SModuleSecureId tmp = (C2SModuleSecureId)handle;
        
        C2SModuleErrorCode result = C2SModuleSecureVariableSet(tmp, &value, sizeof(value));
        
        C2SModuleSecureErrorTriats(env, result);
    }

    /**
     * @brief 핸들에 저장된 float 데이터를 구한다.
     * @param[in] env jni env
     * @param[in] object jni class
     * @param[in] handle 암호화 인스턴스 핸들
     * @return 암호화 되어있던 데이터
     * @author Byeong Yeol Jeon
     * @ingroup Android Protection
     */
    JNIEXPORT jfloat JNICALL Java_com_com2us_module_secure_Float_Get(JNIEnv* env, jclass object, jlong handle)
    {
        C2SModuleSecureId tmp = (C2SModuleSecureId)handle;
        jfloat val;
        C2SModuleErrorCode result = C2SModuleSecureVariableGet(tmp, &val, sizeof(val));
        
        C2SModuleSecureErrorTriats(env, result);
        return val;
    }
    
    /**
     * @brief 핸들에 float 데이터를 암호화하여 저장한다.
     * @param[in] env jni env
     * @param[in] object jni class
     * @param[in] handle 암호화 인스턴스 핸들
     * @param[in] value 저장할 데이터
     * @author Byeong Yeol Jeon
     * @ingroup Android Protection
     */
    JNIEXPORT void JNICALL Java_com_com2us_module_secure_Float_Set(JNIEnv* env, jclass object, jlong handle, jfloat value)
    {
        C2SModuleSecureId tmp = (C2SModuleSecureId)handle;
        
        C2SModuleErrorCode result = C2SModuleSecureVariableSet(tmp, &value, sizeof(value));
        
        C2SModuleSecureErrorTriats(env, result);
    }

    /**
     * @brief 핸들에 저장된 double 데이터를 구한다.
     * @param[in] env jni env
     * @param[in] object jni class
     * @param[in] handle 암호화 인스턴스 핸들
     * @return 암호화 되어있던 데이터
     * @author Byeong Yeol Jeon
     * @ingroup Android Protection
     */
    JNIEXPORT jdouble JNICALL Java_com_com2us_module_secure_Double_Get(JNIEnv* env, jclass object, jlong handle)
    {
        C2SModuleSecureId tmp = (C2SModuleSecureId)handle;
        jdouble val;
        C2SModuleErrorCode result = C2SModuleSecureVariableGet(tmp, &val, sizeof(val));
        
        C2SModuleSecureErrorTriats(env, result);
        return val;
    }
    
    /**
     * @brief 핸들에 double 데이터를 암호화하여 저장한다.
     * @param[in] env jni env
     * @param[in] object jni class
     * @param[in] handle 암호화 인스턴스 핸들
     * @param[in] value 저장할 데이터
     * @author Byeong Yeol Jeon
     * @ingroup Android Protection
     */
    JNIEXPORT void JNICALL Java_com_com2us_module_secure_Double_Set(JNIEnv* env, jclass object, jlong handle, jdouble value)
    {
        C2SModuleSecureId tmp = (C2SModuleSecureId)handle;
        
        C2SModuleErrorCode result = C2SModuleSecureVariableSet(tmp, &value, sizeof(value));
        
        C2SModuleSecureErrorTriats(env, result);
    }
    
#ifdef __cplusplus
}
#endif

#endif
