/**
 * @file    ModuleHelper_Cocos2d-x.h
 
 * @brief   c++ 사용시 제공되는 편의 클래스의 헤더
 
 * @author  Byeong Yeol Jeon
 * @ingroup Cocos2d-x
 * @date    2014/04/25
 */
#ifndef _MODULEHELPER_COCOS2D_X_H_
#define _MODULEHELPER_COCOS2D_X_H_

#include "refobject.h"
#include "smartptr.h"

namespace C2SModule_Cocos2dx {
    /**
     * @class C2SModuleImageDesc
     * @ingroup Cocos2d-x
     * @brief raw 이미지 데이터 클래스
     * @author Byeong Yeol Jeon
     
     */
    class C2SModuleImageDesc: public C2SModule_Cocos2dx::refobject {
    public:
        /**
         * @brief default constructor
         * @author Byeong Yeol Jeon
         
         */
        C2SModuleImageDesc();
        /**
         * @brief default destructor
         * @remark 상속받은 객체는 destructor를 구현해야 한다.
         * @author Byeong Yeol Jeon
         
         */
        virtual ~C2SModuleImageDesc() = 0;
        /**
         * @brief 모듈에서 사용하는 raw 데이터를 구한다.
         
         * @return raw 이미지 데이터
         * @author Byeong Yeol Jeon
         
         */
        virtual void* Get() const = 0;
    };
    
    /** 
     * @brief C2SModuleImageDesc의 스마트 포인터
     * @ingroup Cocos2d-x
     */
    typedef C2SModule_Cocos2dx::TSmartPtr<C2SModuleImageDesc> C2SModuleImage;
    
    /**
     * @namespace C2SModuleImageLoader
     
     * @brief raw 이미지 데이터를 로딩하는 함수들의 네임스페이스
     * @ingroup Cocos2d-x
     * @author Byeong Yeol Jeon
     
     */
    namespace C2SModuleImageLoader {
        /**
         * @brief png, jpg, bmp등의 데이터로 raw 이미지를 생성한다.
         * @param[in] imageData 데이터의 주소
         * @param[in] length 데이터의 길이
         * @return 변환된 raw 이미지
         * @ingroup Cocos2d-x
         * @author Byeong Yeol Jeon
         */
        C2SModuleImage FromEncodedData(const void* imageData, int32_t length);
        
        /**
         * @brief RGBA이미지 데이터로 raw 이미지를 생성한다.
         * @param[in] rawImageData RGBA 이미지 데이터
         * @param[in] width 데이터의 넓이
         * @param[in] height 데이터의 높이
         * @return 변환된 raw 이미지
         * @ingroup Cocos2d-x
         * @author Byeong Yeol Jeon
         */
        C2SModuleImage FromRawData(const void* rawImageData, int32_t width, int32_t height);
        
        /**
         * @brief png, jpg, bmp 파일의 데이터로 raw 이미지를 생성한다.
         * @param[in] filePath 파일의 경로
         * @return 변환된 raw 이미지
         * @ingroup Cocos2d-x
         * @author Byeong Yeol Jeon
         */
        C2SModuleImage FromFile(const char filePath[]);
    }
}

#endif
