/**
 * @file    ModuleHelper_iOS.cpp
 
 * @brief   iOS 통합 모듈 개발 편의를 위한 클래스들의 소스
 
 * @author  Byeong Yeol Jeon
 * @ingroup iOS Helper
 * @date    2014/04/28
 */
#include "ModuleHelper_iOS.h"
#include "../C2SModuleArgKey.h"

#if __OBJC__
#import <objc/runtime.h>

@implementation C2SSerializable
-(NSDictionary*)toDictionary
{
    NSMutableDictionary* props = [NSMutableDictionary dictionary];
    uint32_t outCount;
    // 현재 인스턴스의 prop list를 구한다.
    objc_property_t* properties = class_copyPropertyList([self class], &outCount);
    
    for (uint32_t iter = 0; iter < outCount; ++iter) {
        objc_property_t property = properties[iter];
        NSString* propertyName = [NSString stringWithUTF8String:property_getName(property)];
        
        id propertyValue = [self valueForKey:propertyName];
        
        // prop attribute를 체크
        const char* attr = property_getAttributes(property);
        switch(attr[1])
        {
            case 'c':
                [props setObject:[NSNumber numberWithChar:[propertyValue charValue]] forKey:propertyName];
                break;
            case 'C':
                [props setObject:[NSNumber numberWithUnsignedChar:[propertyValue unsignedCharValue]] forKey:propertyName];
                break;
            case 'i':
                [props setObject:[NSNumber numberWithInt:[propertyValue intValue]] forKey:propertyName];
                break;
            case 'I':
                [props setObject:[NSNumber numberWithUnsignedInt:[propertyValue unsignedIntValue]] forKey:propertyName];
                break;
            case 's':
                [props setObject:[NSNumber numberWithShort:[propertyValue shortValue]] forKey:propertyName];
                break;
            case 'S':
                [props setObject:[NSNumber numberWithUnsignedShort:[propertyValue unsignedShortValue]] forKey:propertyName];
                break;
            case 'l':
                [props setObject:[NSNumber numberWithLong:[propertyValue longValue]] forKey:propertyName];
                break;
            case 'L':
                [props setObject:[NSNumber numberWithUnsignedLong:[propertyValue unsignedLongValue]] forKey:propertyName];
                break;
            case 'q':
                [props setObject:[NSNumber numberWithLongLong:[propertyValue longLongValue]] forKey:propertyName];
                break;
            case 'Q':
                [props setObject:[NSNumber numberWithUnsignedLongLong:[propertyValue unsignedLongLongValue]] forKey:propertyName];
                break;
            case 'f':
                [props setObject:[NSNumber numberWithFloat:[propertyValue floatValue]] forKey:propertyName];
                break;
            case 'd':
                [props setObject:[NSNumber numberWithDouble:[propertyValue doubleValue]] forKey:propertyName];
                break;
            case '@':
            {
                if(nil != propertyValue)
                {
                    // prop이 C2SSerializable 타입이면 재귀적으로 처리한다.
                    if([propertyValue isKindOfClass:[C2SSerializable class]])
                    {
                        [props setObject:[propertyValue toDictionary] forKey:propertyName];
                    }
                    else
                    {
                        [props setObject:propertyValue forKey:propertyName];
                    }
                }
            }
                break;
            default:
                break;
        }
    }
    free(properties);
    return props;
}
@end

@implementation C2SSerializableSocial_Initailize_Push

@end

@implementation C2SSerializableSocial_Initailize_Promotion
@end

@implementation C2SSerializableSocial_Initailize
/**
 * @brief default constructor
 * @author Byeong Yeol Jeon
 
 */
-(id)init
{
    self = [super init];
    
    if(self)
    {
        self.Push = [[C2SSerializableSocial_Initailize_Push alloc] init];
        self.Promotion = [[C2SSerializableSocial_Initailize_Promotion alloc] init];
    }
    
    return self;
}

+(C2SSerializableSocial_Initailize*)instance
{
    return [[[self class] alloc] init];
}
@end

@implementation C2SSerializableSocial_Show
+(C2SSerializableSocial_Show*)instance
{
    return [[[self class] alloc] init];
}
@end

@interface C2SSerializableSocial_Show_Terms ()
/** 보여줄 타입 */
@property (nonatomic, strong) NSString* type;
@end

@implementation C2SSerializableSocial_Show_Terms
/**
 * @brief default constructor
 
 * @author Byeong Yeol Jeon
 
 */
-(id)init
{
    self = [super init];
    
    if(self)
    {
        // 타입을 terms로 설정
        self.type = @C2SModuleArgKey_TERMS;
    }
    
    return self;
}

+(C2SSerializableSocial_Show_Terms*)instance
{
    return [[[self class] alloc] init];
}
@end

@implementation C2SSerializableSns_Login
+(C2SSerializableSns_Login*)instance
{
    return [[[self class] alloc] init];
}
@end

@interface C2SSerializableSns_Login_Guest ()
/** 로그인 타입 */
@property (nonatomic, strong) NSString* type;
@end

@implementation C2SSerializableSns_Login_Guest
/**
 * @brief default constructor
 * @author Byeong Yeol Jeon
 
 */
-(id)init
{
    self = [super init];
    
    if(self)
    {
        // 타입을 게스트로 설정
        self.type = @C2SModuleArgKey_GUEST;
    }
    
    return self;
}

+(C2SSerializableSns_Login_Guest*)instance
{
    return [[[self class] alloc] init];
}
@end

@interface C2SSerializableSns_Login_Select ()
/** 로그인 타입 */
@property (nonatomic, strong) NSString* type;
@end

@implementation C2SSerializableSns_Login_Select
/**
 * @brief default constructor
 * @author Byeong Yeol Jeon
 
 */
-(id)init
{
    self = [super init];
    
    if(self)
    {
        // 타입을 select로 설정
        self.type = @"select";
    }
    
    return self;
}

+(C2SSerializableSns_Login_Select*)instance
{
    return [[[self class] alloc] init];
}
@end

@implementation C2SSerializableSns_User
+(C2SSerializableSns_User*)instance
{
    return [[[self class] alloc] init];
}
@end

@implementation C2SSerializableSns_User_Me
+(C2SSerializableSns_User_Me*)instance
{
    return [[[self class] alloc] init];
}
@end

@implementation C2SSerializableSns_Friends
+(C2SSerializableSns_Friends*)instance
{
    return [[[self class] alloc] init];
}
@end

@implementation C2SSerializableSns_Friends_All
+(C2SSerializableSns_Friends_All*)instance
{
    return [[[self class] alloc] init];
}
@end

@interface C2SSerializableSns_Friends_Game ()
/** 요청할 친구 리스트의 타입 */
@property (nonatomic, strong) NSString* type;
@end

@implementation C2SSerializableSns_Friends_Game
/**
 * @brief default constructor
 * @author Byeong Yeol Jeon
 
 */
-(id)init
{
    self = [super init];
    
    if(self)
    {
        // 타입을 game으로 설정
        self.type = @C2SModuleArgKey_GAME;
    }
    
    return self;
}

+(C2SSerializableSns_Friends_Game*)instance
{
    return [[[self class] alloc] init];
}
@end

@interface C2SSerializableSns_Friends_Other ()
/** 요청할 친구 리스트의 타입 */
@property (nonatomic, strong) NSString* type;
@end

@implementation C2SSerializableSns_Friends_Other
/**
 * @brief default constructor
 * @author Byeong Yeol Jeon
 
 */
-(id)init
{
    self = [super init];
    
    if(self)
    {
        // 타입을 other로 설정
        self.type = @C2SModuleArgKey_OTHER;
    }
    
    return self;
}

+(C2SSerializableSns_Friends_Other*)instance
{
    return [[[self class] alloc] init];
}
@end

@interface C2SSerializableSns_Friends_Other_Facebook ()
/** 서비스의 종류 */
@property (nonatomic, strong) NSString* service;
/** 요청할 친구리스트의 타입 */
@property (nonatomic, strong) NSString* type;
@end

@implementation C2SSerializableSns_Friends_Other_Facebook
-(id)init
{
    self = [super init];
    
    if(self)
    {
        // 서비스를 페이스북으로 설정
        self.service = @C2SModuleArgKey_FACEBOOK;
        // 요청할 친구리스트의 종류를 other로 설정
        self.type = @C2SModuleArgKey_OTHER;
    }
    
    return self;
}

+(C2SSerializableSns_Friends_Other_Facebook*)instance
{
    return [[[self class] alloc] init];
}
@end

@implementation C2SSerializableSns_Message_Kakao
+(C2SSerializableSns_Message_Kakao*)instance
{
    return [[[self class] alloc] init];
}
@end

@class UIImage;

@implementation C2SSerializableSns_Message_Kakao_CustomImage
+(C2SSerializableSns_Message_Kakao_CustomImage*)instance
{
    return [[[self class] alloc] init];
}
@end

@implementation C2SSerializableSns_Message_Hub
+(C2SSerializableSns_Message_Hub*)instance
{
    return [[[self class] alloc] init];
}
@end

@implementation C2SSerializableSns_Message_Facebook
+(C2SSerializableSns_Message_Facebook*)instance
{
    return [[[self class] alloc] init];
}
@end

@implementation C2SSerializableSns_Message_GooglePlus
+(C2SSerializableSns_Message_GooglePlus*)instance
{
    return [[[self class] alloc] init];
}
@end

@implementation C2SSerializableSns_Message_SinaWeibo
+(C2SSerializableSns_Message_SinaWeibo*)instance
{
    return [[[self class] alloc] init];
}
@end

@implementation C2SSerializablePush_Register
+(C2SSerializablePush_Register*)instance
{
    return [[[self class] alloc] init];
}
@end

@implementation C2SSerializablePush_Register_DeviceToken
+(C2SSerializablePush_Register_DeviceToken*)instance
{
    return [[[self class] alloc] init];
}
@end

@implementation C2SSerializablePush_Register_PromotionCode
+(C2SSerializablePush_Register_PromotionCode*)instance
{
    return [[[self class] alloc] init];
}
@end

@implementation C2SSerializablePromotion_Show
+(C2SSerializablePromotion_Show*)instance
{
    return [[[self class] alloc] init];
}
@end

@interface C2SSerializablePromotion_Show_Notice ()
/** 보여줄 뷰의 종류 */
@property (nonatomic, strong) NSString* type;
@end

@implementation C2SSerializablePromotion_Show_Notice
/**
 * @brief default constructor
 * @author Byeong Yeol Jeon
 
 */
-(id)init
{
    self = [super init];
    
    if(self)
    {
        // 보여줄 뷰를 공지로 설정
        self.type = @C2SModuleArgKey_NOTICE;
    }
    
    return self;
}

+(C2SSerializablePromotion_Show_Notice*)instance
{
    return [[[self class] alloc] init];
}
@end

@interface C2SSerializablePromotion_Show_Event ()
/** 보여줄 뷰의 종류 */
@property (nonatomic, strong) NSString* type;
@end

@implementation C2SSerializablePromotion_Show_Event
/**
 * @brief default constructor
 
 * @author Byeong Yeol Jeon
 
 */
-(id)init
{
    self = [super init];
    
    if(self)
    {
        // 보여줄 뷰를 이벤트로 설정
        self.type = @C2SModuleArgKey_EVENT;
    }
    
    return self;
}

+(C2SSerializablePromotion_Show_Event*)instance
{
    return [[[self class] alloc] init];
}
@end
#endif
