﻿/**
 * @file    cscrc32.cpp
 
 * @brief   cscrc32를 계산하는 클래스의 소스
 
 * @author  Byeong Yeol Jeon
 * @ingroup iOS Android Cocos2d-x Protection
 * @date    2014/04/30
 */
#include<cstddef>
#include"cscrc32.h"		// modified

const uint32_t POLYNOMIAL = 0x04c11db7;

CSCRC32::CSCRC32()
{
	gen_table();
}

void CSCRC32::gen_table()
{
    uint32_t crc_accum;

	for(uint32_t i=0; i<sizeof(m_secureCRCtable)/sizeof(m_secureCRCtable[0]); ++i)
	{
		crc_accum = (i << 24);
		for(uint32_t j=0; j<8; ++j)
		{
			if(crc_accum & 0x80000000L)
				crc_accum = (crc_accum << 1) ^ POLYNOMIAL;
			else
				crc_accum = (crc_accum << 1);
		}
		m_secureCRCtable[i] = crc_accum;
	}
	return;
}

bool CSCRC32::update(uint32_t& outCrc, uint32_t crc_accum, uint8_t* data, uint32_t length)
{
	if(NULL == data || 0 == length)
		return false;

	for(uint32_t j=0; j<length; j++)
	{
		uint32_t i = ((uint32_t)(crc_accum >> 24) ^ *data++) & 0xff;
		crc_accum = (crc_accum << 8) ^ m_secureCRCtable[i];
	}
    
    outCrc = crc_accum;
	return true;
}
