//
//  FriendsScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/19.
//

#include "SDKv1FuncList.hpp"
#include "FriendsScene.hpp"
#include "FriendsParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include "../../src/cocos2d-x/Module_Cocos2d-x.h"
#include "../../src/C2SModule.h"
#include <map>
#include <sstream>

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;
using namespace::std;


bool FriendsScene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- [SDK v1 친구 초대] -");

    addButton("Hive 게임 친구\n(C2SModuleSns_Friends)", SDKv1FuncList::FRIENDS_SCENE_GAME_FRIENDS);
    addButton("Hive 게임 외 친구\n(C2SModuleSns_Friends)", SDKv1FuncList::FRIENDS_SCENE_NOT_GAME_FRIENDS);
    addButton("Hive 전체 친구\n(C2SModuleSns_Friends)", SDKv1FuncList::FRIENDS_SCENE_ALL_FRIENDS);
    addButton("Facebook 초대 유저\n(C2SModuleSns_Friends)", ButtonMode::NEED_PARAM, SDKv1FuncList::FRIENDS_SCENE_FACEBOOK_INVITATION_USER);
    addButton("Facebook 메시지 전송\n(C2SModuleSns_Friends)", ButtonMode::NEED_PARAM, SDKv1FuncList::FRIENDS_SCENE_FACEBOOK_SEND_MESSAGE);
    addButton("Hive 메시지 전송\n(C2SModuleSns_Message)", ButtonMode::NEED_PARAM, SDKv1FuncList::FRIENDS_SCENE_HIVE_SEND_MESSAGE);
    addButton("Hive 세션 확인\n(C2SModuleSns_Session)", SDKv1FuncList::FRIENDS_SCENE_HIVE_SESSION);
    addButton("Facebook 세션 확인\n(C2SModuleSns_Session)", SDKv1FuncList::FRIENDS_SCENE_FACEBOOK_SESSION);
    addButton("Facebook 친구 리스트\n(C2SModuleSns_Friends)", SDKv1FuncList::FRIENDS_SCENE_FACEBOOK_FRIENDS_LIST);

    scheduleUpdate();

    return true;
}

void FriendsScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void FriendsScene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        FriendsParamScene* paramScene = FriendsParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    SDKv1FuncList::FriendsSceneFunc api = static_cast<SDKv1FuncList::FriendsSceneFunc>(buttonTag);
    switch (api) {
        case SDKv1FuncList::FRIENDS_SCENE_GAME_FRIENDS: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("game");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Friends(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Friends", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Friends", paramString);
            break;
        }
        case SDKv1FuncList::FRIENDS_SCENE_NOT_GAME_FRIENDS: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("other");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Friends(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Friends", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Friends", paramString);
            break;
        }
        case SDKv1FuncList::FRIENDS_SCENE_ALL_FRIENDS: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("all");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Friends(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Friends", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Friends", paramString);
            break;
        }
        case SDKv1FuncList::FRIENDS_SCENE_FACEBOOK_INVITATION_USER: {
            int limit = ParameterSDKv1DataManager::getInstance()->friendsParameters->getLimit();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("receivedInvite");
            paramsDefault["service"] = picojson::value("facebook");
            paramsDefault["limit"] = picojson::value(static_cast<int64_t>(limit));

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Friends(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Friends", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Friends", paramString);
            break;
        }
        case SDKv1FuncList::FRIENDS_SCENE_FACEBOOK_SEND_MESSAGE: {
            string message = ParameterSDKv1DataManager::getInstance()->friendsParameters->getMessage();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("inviteDialog");
            paramsDefault["service"] = picojson::value("facebook");
            paramsDefault["message"] = picojson::value(message);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Friends(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Friends", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Friends", paramString);
            break;
        }
        case SDKv1FuncList::FRIENDS_SCENE_HIVE_SEND_MESSAGE: {
            bool useVid = ParameterSDKv1DataManager::getInstance()->friendsParameters->getUseVid();
            string vidOrUid = ParameterSDKv1DataManager::getInstance()->friendsParameters->getVidOrUid();
            string message = ParameterSDKv1DataManager::getInstance()->friendsParameters->getMessage();
            string imageURL = ParameterSDKv1DataManager::getInstance()->friendsParameters->getImageURL();
            string thumbnailURL = ParameterSDKv1DataManager::getInstance()->friendsParameters->getThumbnailURL();
            bool usePush = ParameterSDKv1DataManager::getInstance()->friendsParameters->getUsePush();
            bool invite = ParameterSDKv1DataManager::getInstance()->friendsParameters->getInvite();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            if(useVid == true) {
                paramsDefault["vid"] = picojson::value(vidOrUid);
            } else {
                paramsDefault["uid"] = picojson::value(vidOrUid);
            }
            paramsDefault["message"] = picojson::value(message);
            paramsDefault["imageURL"] = picojson::value(imageURL);
            paramsDefault["thumbnailURL"] = picojson::value(thumbnailURL);
            paramsDefault["usePush"] = picojson::value(usePush);
            paramsDefault["invite"] = picojson::value(invite);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Message(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Message", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Message", paramString);
            break;
        }
        case SDKv1FuncList::FRIENDS_SCENE_HIVE_SESSION: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Session(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Session", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Session", paramString);
            break;
        }
        case SDKv1FuncList::FRIENDS_SCENE_FACEBOOK_SESSION: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["service"] = picojson::value("facebook");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Session(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Session", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Session", paramString);
            break;
        }
        case SDKv1FuncList::FRIENDS_SCENE_FACEBOOK_FRIENDS_LIST: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["service"] = picojson::value("facebook");
            paramsDefault["type"] = picojson::value("friendsList");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Friends(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Friends", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Friends", paramString);
            break;
        }
        default:
            break;
    }
}
