//
//  HiveWebviewScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/19.
//

#include "SDKv1FuncList.hpp"
#include "HiveWebviewScene.hpp"
#include "HiveWebviewParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include "../../src/cocos2d-x/Module_Cocos2d-x.h"
#include "../../src/C2SModule.h"
#include <map>
#include <sstream>

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;
using namespace::std;


bool HiveWebviewScene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- [SDK v1 Hive 웹뷰, 팝업] -");

    addButton("HiveWebview Home\n(C2SModuleSns_Show)", SDKv1FuncList::HIVE_WEBVIEW_SCENE_HOME);
    addButton("HiveWebview 프로필 페이지\n(C2SModuleSns_Show)", SDKv1FuncList::HIVE_WEBVIEW_SCENE_PROFILE);
    addButton("HiveWebview 프로필 변경\n(C2SModuleSns_Show)", SDKv1FuncList::HIVE_WEBVIEW_SCENE_EDIT_PROFILE);
    addButton("HiveWebview 1:1 문의\n(C2SModuleSns_Show)", SDKv1FuncList::HIVE_WEBVIEW_SCENE_INQUIRY);
    addButton("HiveWebview 챗봇\n(C2SModuleSns_Show)", ButtonMode::NEED_PARAM, SDKv1FuncList::HIVE_WEBVIEW_SCENE_CHATBOT);
    addButton("약관 다시보기\n(C2SModuleSocial_Show)", SDKv1FuncList::HIVE_WEBVIEW_SCENE_TERMS);
    addButton("HiveWebview 쪽지보기\n(C2SModuleSns_Show)", ButtonMode::NEED_PARAM, SDKv1FuncList::HIVE_WEBVIEW_SCENE_MESSAGE);
    addButton("성인 인증 페이지\n(C2SModuleSocial_Show)", SDKv1FuncList::HIVE_WEBVIEW_SCENE_ADULT_AUTH);
    addButton("점검공지 팝업\n(C2SModuleSocial_Show)", SDKv1FuncList::HIVE_WEBVIEW_SCENE_NOTICE_SHOW);
    addButton("점검공지 팝업 정보\n(C2SModuleSocial_Info)", SDKv1FuncList::HIVE_WEBVIEW_SCENE_NOTICE_INFO);
    addButton("권한고지 팝업 정보\n(C2SModuleSocial_Info)", SDKv1FuncList::HIVE_WEBVIEW_SCENE_PERMISSION_INFO);
    addButton("Android 권한 추가 요청\n(C2SModuleSocial_Show)", SDKv1FuncList::HIVE_WEBVIEW_SCENE_REQUEST_ANDROID_PERMISSION);

    scheduleUpdate();

    return true;
}

void HiveWebviewScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void HiveWebviewScene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        HiveWebviewParamScene* paramScene = HiveWebviewParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    SDKv1FuncList::HiveWebviewSceneFunc api = static_cast<SDKv1FuncList::HiveWebviewSceneFunc>(buttonTag);
    switch (api) {
        case SDKv1FuncList::HIVE_WEBVIEW_SCENE_HOME: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("home");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Show", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_WEBVIEW_SCENE_PROFILE: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("user");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Show", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_WEBVIEW_SCENE_EDIT_PROFILE: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("edit/profile/picture");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Show", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_WEBVIEW_SCENE_INQUIRY: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("inquiry");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Show", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_WEBVIEW_SCENE_CHATBOT: {
            string additionalInfo = ParameterSDKv1DataManager::getInstance()->hiveWebviewParameters->getAdditionalInfo();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("chatbot");
            paramsDefault["additionalInfo"] = picojson::value(additionalInfo);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Show", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_WEBVIEW_SCENE_TERMS: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("terms");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSocial_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSocial_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSocial_Show", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_WEBVIEW_SCENE_MESSAGE: {
            string uid = ParameterSDKv1DataManager::getInstance()->hiveWebviewParameters->getUid();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("message");
            paramsDefault["uid"] = picojson::value(uid);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Show", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_WEBVIEW_SCENE_ADULT_AUTH: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("adultAuth");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSocial_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSocial_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSocial_Show", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_WEBVIEW_SCENE_NOTICE_SHOW: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("notice");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSocial_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSocial_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSocial_Show", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_WEBVIEW_SCENE_NOTICE_INFO: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("notice");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSocial_Info(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSocial_Info", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSocial_Info", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_WEBVIEW_SCENE_PERMISSION_INFO: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("permission");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSocial_Info(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSocial_Info", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSocial_Info", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_WEBVIEW_SCENE_REQUEST_ANDROID_PERMISSION: {
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
            MessageBox("C2SModuleSocial_Show(type=requestPermissionList)", "Android만 제공되는 기능입니다!");
#else
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("requestPermissionList");

            // 아래 2개의 권한을 추가 요청하는 가정으로 테스트
            picojson::array permissionArr;
            permissionArr.push_back(picojson::value("android.permission.READ_PHONE_STATE"));
            permissionArr.push_back(picojson::value("android.permission.WRITE_EXTERNAL_STORAGE"));
            picojson::value paramsRequests = picojson::value(permissionArr);
            paramsDefault["requests"] = picojson::value(paramsRequests);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSocial_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSocial_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSocial_Show", paramString);
#endif
            break;
        }
        default:
            break;
    }
}
