//
//  LoginScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/11.
//

#include "SDKv1FuncList.hpp"
#include "LoginScene.hpp"
#include "LoginParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include "../../../Classes/Postbox.hpp"
#include "../../src/cocos2d-x/Module_Cocos2d-x.h"
#include "../../src/C2SModule.h"
#include <map>
#include <sstream>

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;
using namespace::std;


bool LoginScene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- [SDK v1 로그인] -");

    addButton("하이브 로그인\n(C2SModuleSns_Login)", SDKv1FuncList::LOGIN_SCENE_HIVE);
    addButton("게스트 로그인\n(C2SModuleSns_Login)", SDKv1FuncList::LOGIN_SCENE_GUEST);
    addButton("로그인 타입\n(C2SModuleSns_GetLoginType)", SDKv1FuncList::LOGIN_SCENE_LOGIN_TYPE);
    addButton("유저 정보 선택(Select)\n(C2SModuleSns_Login)", SDKv1FuncList::LOGIN_SCENE_SELECT);
    addButton("유저 정보 선택(Custom Select)\n(C2SModuleSns_Login)", ButtonMode::NEED_PARAM,SDKv1FuncList::LOGIN_SCENE_CUSTOM_SELECT);
    addButton("EmergencyMode\n(C2SModuleSocial_SetGameInfo)", SDKv1FuncList::LOGIN_SCENE_EMERGENCY_MODE);
    addButton("GPG 세션 가져오기\n(C2SModuleSns_Session)", SDKv1FuncList::LOGIN_SCENE_GET_GOOGLE_PLAY_GAMES_SESSION);

    scheduleUpdate();

    return true;
}

void LoginScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void LoginScene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        LoginParamScene* paramScene = LoginParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    SDKv1FuncList::LoginSceneFunc api = static_cast<SDKv1FuncList::LoginSceneFunc>(buttonTag);
    switch (api) {
        case SDKv1FuncList::LOGIN_SCENE_HIVE: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Login(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Login", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Login", paramString);
            break;
        }
        case SDKv1FuncList::LOGIN_SCENE_GUEST: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("guest");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Login(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Login", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Login", paramString);
            break;
        }
        case SDKv1FuncList::LOGIN_SCENE_LOGIN_TYPE: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_GetLoginType", paramString);

            string loginType = "";
            if(::GetLoginType() == NULL) {
                loginType = "null";
            } else {
                loginType = ::GetLoginType();
            }

            addLogMessage("GetLoginType() : " + loginType);
            break;
        }
        case SDKv1FuncList::LOGIN_SCENE_SELECT: {
            string guestVid = ParameterSDKv1DataManager::getInstance()->loginParameters->getGuestVid();
            string hiveVid = ParameterSDKv1DataManager::getInstance()->loginParameters->getHiveVid();
            string hiveUid = ParameterSDKv1DataManager::getInstance()->loginParameters->getHiveUid();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            picojson::object paramCurrentGuest;
            picojson::object paramCurrentGuestUserInfo;
            picojson::object paramTargetHive;
            picojson::object paramTargetHiveUserInfo;

            paramsDefault["type"] = picojson::value("select");                      // set type

            // add Guest Info
            paramCurrentGuest["vid"] = picojson::value(guestVid);
            paramCurrentGuestUserInfo["Name"] = picojson::value("Guest1");
            paramCurrentGuestUserInfo["Level"] = picojson::value("10");
            paramCurrentGuestUserInfo["Gold"] = picojson::value("100");
            paramCurrentGuestUserInfo["Cash"] = picojson::value("1000");
            paramCurrentGuest["data"] = picojson::value(paramCurrentGuestUserInfo);
            paramsDefault["currentVidData"] = picojson::value(paramCurrentGuest);

            // add Hive Info
            paramTargetHive["vid"] = picojson::value(guestVid);
            paramTargetHiveUserInfo["Name"] = picojson::value("Hive2");
            paramTargetHiveUserInfo["Level"] = picojson::value("20");
            paramTargetHiveUserInfo["Gold"] = picojson::value("200");
            paramTargetHiveUserInfo["Cash"] = picojson::value("2000");
            paramTargetHive["data"] = picojson::value(paramTargetHiveUserInfo);
            paramsDefault["usedVidData"] = picojson::value(paramTargetHive);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Login(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Login", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Login", paramString);
            break;
        }
        case SDKv1FuncList::LOGIN_SCENE_CUSTOM_SELECT: {
            string selectedTarget = ParameterSDKv1DataManager::getInstance()->loginParameters->getCustomSelectTarget();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;

            paramsDefault["type"] = picojson::value("customSelect");
            if(selectedTarget == "use_guest_user_info") {
                string guestVid = ParameterSDKv1DataManager::getInstance()->loginParameters->getGuestVid();
                paramsDefault["selectVid"] = picojson::value(guestVid);
            } else {
                string hiveVid = ParameterSDKv1DataManager::getInstance()->loginParameters->getHiveVid();
                paramsDefault["selectVid"] = picojson::value(hiveVid);
            }

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Login(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Login", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Login", paramString);
            break;
        }
        case SDKv1FuncList::LOGIN_SCENE_EMERGENCY_MODE: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;

            paramsDefault["emergencyMode"] = picojson::value(true);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSocial_SetGameInfo(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSocial_SetGameInfo", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSocial_SetGameInfo", paramString);
            break;
        }
        case SDKv1FuncList::LOGIN_SCENE_GET_GOOGLE_PLAY_GAMES_SESSION: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["service"] = picojson::value("googlePlayGames");
            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Session(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Session", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Session", paramString);

            break;
        }
        default:
            break;
    }
}
