//
//  PromotionV1InfoScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/17.
//

#include "SDKv1FuncList.hpp"
#include "PromotionV1InfoScene.hpp"
#include "PromotionV1InfoParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include "../../src/cocos2d-x/Module_Cocos2d-x.h"
#include "../../src/C2SModule.h"
#include <map>
#include <sstream>

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;
using namespace::std;


bool PromotionV1InfoScene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- [SDK v1 프로모션(V1) 정보] -");

    addButton("뱃지(notice)\n(C2SModulePromotion_Info)", SDKv1FuncList::PROMOTION_SCENE_NOTICE);
    addButton("뱃지(event)\n(C2SModulePromotion_Info)", SDKv1FuncList::PROMOTION_SCENE_NOTICE_FORCED);
    addButton("뱃지(noticeOnly)\n(C2SModulePromotion_Info)", SDKv1FuncList::PROMOTION_SCENE_EVENT);
    addButton("뱃지(customWebview)\n(C2SModulePromotion_Info)", ButtonMode::NEED_PARAM, SDKv1FuncList::PROMOTION_SCENE_EVENT_FORCED);
    addButton("뷰(fullBanner)\n(C2SModulePromotion_Info)", SDKv1FuncList::PROMOTION_SCENE_FULLBANNER);
    addButton("뷰(notice)\n(C2SModulePromotion_Info)", SDKv1FuncList::PROMOTION_SCENE_FULLBANNER_LEGACY);
    addButton("뷰(event)\n(C2SModulePromotion_Info)", SDKv1FuncList::PROMOTION_SCENE_OFFERWALL);
    addButton("뷰(noticeOnly)\n(C2SModulePromotion_Info)", SDKv1FuncList::PROMOTION_SCENE_NOTICE_ONLY);
    addButton("뷰(customWebview)\n(C2SModulePromotion_Info)", ButtonMode::NEED_PARAM, SDKv1FuncList::PROMOTION_SCENE_CUSTOM_WEBVIEW);
    addButton("getOfferwallState\n(C2SModulePromotion_GetOfferwallState)", SDKv1FuncList::PROMOTION_SCENE_REVIEW);
    addButton("BannerInfo\n(C2SModulePromotion_Info)", ButtonMode::NEED_PARAM, SDKv1FuncList::PROMOTION_SCENE_MOREGAMES);

    scheduleUpdate();

    return true;
}

void PromotionV1InfoScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void PromotionV1InfoScene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        PromotionV1InfoParamScene* paramScene = PromotionV1InfoParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    SDKv1FuncList::PromotionInfoSceneFunc api = static_cast<SDKv1FuncList::PromotionInfoSceneFunc>(buttonTag);
    switch (api) {
        case SDKv1FuncList::PROMOTION_INFO_SCENE_BADGE_NOTICE: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("badge");
            paramsDefault["viewId"] = picojson::value("notice");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Info(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Info", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Info", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_INFO_SCENE_BADGE_EVENT: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("badge");
            paramsDefault["viewId"] = picojson::value("event");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Info(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Info", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Info", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_INFO_SCENE_BADGE_NOTICE_ONLY: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("badge");
            paramsDefault["viewId"] = picojson::value("noticeOnly");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Info(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Info", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Info", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_INFO_SCENE_BADGE_CUSTOM_WEBVIEW: {
            int viewId = stoi(ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->getViewId());

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("badge");
            paramsDefault["viewId"] = picojson::value(static_cast<int64_t>(viewId));

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Info(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Info", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Info", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_INFO_SCENE_VIEW_FULLBANNER: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("view");
            paramsDefault["viewId"] = picojson::value("fullBanner");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Info(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Info", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Info", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_INFO_SCENE_VIEW_NOTICE: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("view");
            paramsDefault["viewId"] = picojson::value("notice");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Info(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Info", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Info", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_INFO_SCENE_VIEW_EVENT: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("view");
            paramsDefault["viewId"] = picojson::value("event");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Info(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Info", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Info", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_INFO_SCENE_VIEW_NOTICE_ONLY: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("view");
            paramsDefault["viewId"] = picojson::value("noticeOnly");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Info(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Info", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Info", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_INFO_SCENE_VIEW_CUSTOM_WEBVIEW: {
            int viewId = stoi(ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->getViewId());

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("view");
            paramsDefault["viewId"] = picojson::value(static_cast<int64_t>(viewId));

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Info(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Info", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Info", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_INFO_SCENE_GET_OFFERWALL_STATE: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_GetOfferwallState", paramString);

            int offerwallState = ::GetOfferwallState();

            addLogMessage("GetOfferwallState() : " + to_string(offerwallState));
            break;
        }
        case SDKv1FuncList::PROMOTION_INFO_SCENE_BANNERINFO: {
            string campaignType = ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->getCampaignType();
            string bannerType = ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->getBannerType();
            string campaignTypeCustom = ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->getCampaignTypeCustom();
            string bannerTypeCustom = ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->getBannerTypeCustom();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            if(campaignType == "custom input") {
                paramsDefault["campaignType"] = picojson::value(campaignTypeCustom);
            } else {
                paramsDefault["campaignType"] = picojson::value(campaignType);
            }
            if(bannerType == "custom input") {
                paramsDefault["bannerType"] = picojson::value(bannerTypeCustom);
            } else {
                paramsDefault["bannerType"] = picojson::value(bannerType);
            }

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Info(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Info", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Info", paramString);
            break;
        }
        default:
            break;
    }
}
