//
//  PromotionV1Scene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/17.
//

#include "SDKv1FuncList.hpp"
#include "PromotionV1Scene.hpp"
#include "PromotionV1ParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include "../../src/cocos2d-x/Module_Cocos2d-x.h"
#include "../../src/C2SModule.h"
#include <map>
#include <sstream>

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;
using namespace::std;


bool PromotionV1Scene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- [SDK v1 프로모션(V1)] -");

    addButton("Notice\n(C2SModulePromotion_Show)", SDKv1FuncList::PROMOTION_SCENE_NOTICE);
    addButton("Notice(forced)\n(C2SModulePromotion_Show)", SDKv1FuncList::PROMOTION_SCENE_NOTICE_FORCED);
    addButton("Event\n(C2SModulePromotion_Show)", SDKv1FuncList::PROMOTION_SCENE_EVENT);
    addButton("Event(forced)\n(C2SModulePromotion_Show)", SDKv1FuncList::PROMOTION_SCENE_EVENT_FORCED);
    addButton("FullBanner\n(C2SModulePromotion_Show)", SDKv1FuncList::PROMOTION_SCENE_FULLBANNER);
    addButton("FullBanner(legacy)\n(C2SModulePromotion_Show)", SDKv1FuncList::PROMOTION_SCENE_FULLBANNER_LEGACY);
    addButton("Offerwall\n(C2SModulePromotion_Show)", SDKv1FuncList::PROMOTION_SCENE_OFFERWALL);
    addButton("NoticeOnly\n(C2SModulePromotion_Show)", SDKv1FuncList::PROMOTION_SCENE_NOTICE_ONLY);
    addButton("CustomWebview\n(C2SModulePromotion_Show)", ButtonMode::NEED_PARAM, SDKv1FuncList::PROMOTION_SCENE_CUSTOM_WEBVIEW);
    addButton("Review\n(C2SModulePromotion_Show)", SDKv1FuncList::PROMOTION_SCENE_REVIEW);
    addButton("MoreGames\n(C2SModulePromotion_Show)", SDKv1FuncList::PROMOTION_SCENE_MOREGAMES);
    addButton("Review(native)\n(C2SModulePromotion_Show)", SDKv1FuncList::PROMOTION_SCENE_REVIEW_NATIVE);

    scheduleUpdate();

    return true;
}

void PromotionV1Scene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void PromotionV1Scene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        PromotionV1ParamScene* paramScene = PromotionV1ParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    SDKv1FuncList::PromotionSceneFunc api = static_cast<SDKv1FuncList::PromotionSceneFunc>(buttonTag);
    switch (api) {
        case SDKv1FuncList::PROMOTION_SCENE_NOTICE: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("notice");
            paramsDefault["forced"] = picojson::value(false);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_SCENE_NOTICE_FORCED: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("notice");
            paramsDefault["forced"] = picojson::value(true);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_SCENE_EVENT: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("event");
            paramsDefault["forced"] = picojson::value(false);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_SCENE_EVENT_FORCED: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("event");
            paramsDefault["forced"] = picojson::value(true);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_SCENE_FULLBANNER: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("fullBanner");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_SCENE_FULLBANNER_LEGACY: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("fullBannerLegacy");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_SCENE_OFFERWALL: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("offerwall");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_SCENE_NOTICE_ONLY: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("noticeOnly");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_SCENE_CUSTOM_WEBVIEW: {
            string viewId = ParameterSDKv1DataManager::getInstance()->promotionV1Parameters->getViewId();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("customWebview");
            paramsDefault["viewId"] = picojson::value(viewId);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_SCENE_REVIEW: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("review");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_SCENE_MOREGAMES: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("moreGames");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_SCENE_REVIEW_NATIVE: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("reviewEx");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Show", paramString);
            break;
        }
        default:
            break;
    }
}
