//
//  PromotionV2InfoScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/20.
//

#include "SDKv1FuncList.hpp"
#include "PromotionV2InfoScene.hpp"
#include "PromotionV2InfoParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include "../../src/cocos2d-x/Module_Cocos2d-x.h"
#include "../../src/C2SModule.h"
#include <map>
#include <sstream>

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;
using namespace::std;


bool PromotionV2InfoScene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- [SDK v1 Hive 프로모션(V2) 정보] -");

    addButton("뱃지 정보\n(C2SModuleHivePromotion_Info)", SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_BADGE);
    addButton("커스텀뷰 정보\n(C2SModuleHivePromotion_Info)", ButtonMode::NEED_PARAM, SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_CUSTOM_VIEW);
    addButton("커스텀보드 정보\n(C2SModuleHivePromotion_Info)", ButtonMode::NEED_PARAM, SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_CUSTOM_BOARD);
    addButton("스팟배너 정보\n(C2SModuleHivePromotion_Info)", ButtonMode::NEED_PARAM, SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_SPOT);
    addButton("다이렉트배너 정보\n(C2SModuleHivePromotion_Info)", ButtonMode::NEED_PARAM, SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_DIRECT);
    addButton("Set AdditionalInfo\n(C2SModuleHivePromotion_SetAdditionalInfo)", ButtonMode::NEED_PARAM, SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_SET_ADDITIONALINFO);
    addButton("OfferwallState 정보\n(C2SModuleHivePromotion_GetOfferwallState)", SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_GET_OFFERWALL_STATE);
    addButton("BannerInfo\n(C2SModuleHivePromotion_Info)", ButtonMode::NEED_PARAM, SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_BANNERINFO);
    addButton("새소식(News) 업데이트\n(C2SModuleHivePromotion_UpdatePromotionData)", SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_UPDATE_PROMOTION_DATA);

    scheduleUpdate();

    return true;
}

void PromotionV2InfoScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void PromotionV2InfoScene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        PromotionV2InfoParamScene* paramScene = PromotionV2InfoParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    SDKv1FuncList::HivePromotionInfoSceneFunc api = static_cast<SDKv1FuncList::HivePromotionInfoSceneFunc>(buttonTag);
    switch (api) {
        case SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_BADGE: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("badge");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleHivePromotion_Info(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleHivePromotion_Info", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleHivePromotion_Info", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_CUSTOM_VIEW: {
            string contentKey = ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->getCustomViewContentKey();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("customView");
            paramsDefault["contentKey"] = picojson::value(contentKey);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleHivePromotion_Info(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleHivePromotion_Info", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleHivePromotion_Info", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_CUSTOM_BOARD: {
            string contentKey = ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->getCustomBoardContentKey();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("customBoard");
            paramsDefault["contentKey"] = picojson::value(contentKey);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleHivePromotion_Info(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleHivePromotion_Info", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleHivePromotion_Info", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_SPOT: {
            string contentKey = ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->getSpotContentKey();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("spot");
            paramsDefault["contentKey"] = picojson::value(contentKey);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleHivePromotion_Info(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleHivePromotion_Info", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleHivePromotion_Info", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_DIRECT: {
            string contentKey = ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->getDirectContentKey();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("direct");
            paramsDefault["contentKey"] = picojson::value(contentKey);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleHivePromotion_Info(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleHivePromotion_Info", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleHivePromotion_Info", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_SET_ADDITIONALINFO: {
            string additionalInfo = ParameterSDKv1DataManager::getInstance()->promotionV2InfoParameters->getAdditionalInfo();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["additionalInfo"] = picojson::value(additionalInfo);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleHivePromotion_SetAdditionalInfo(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleHivePromotion_SetAdditionalInfo", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleHivePromotion_SetAdditionalInfo", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_GET_OFFERWALL_STATE: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleHivePromotion_GetOfferwallState(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleHivePromotion_GetOfferwallState", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleHivePromotion_GetOfferwallState", paramString);
            break;
        }
        case SDKv1FuncList::PROMOTION_INFO_SCENE_BANNERINFO: {
            string campaignType = ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->getCampaignType();
            string bannerType = ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->getBannerType();
            string campaignTypeCustom = ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->getCampaignTypeCustom();
            string bannerTypeCustom = ParameterSDKv1DataManager::getInstance()->promotionV1InfoParameters->getBannerTypeCustom();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            if(campaignType == "custom input") {
                paramsDefault["campaignType"] = picojson::value(campaignTypeCustom);
            } else {
                paramsDefault["campaignType"] = picojson::value(campaignType);
            }
            if(bannerType == "custom input") {
                paramsDefault["bannerType"] = picojson::value(bannerTypeCustom);
            } else {
                paramsDefault["bannerType"] = picojson::value(bannerType);
            }

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Info(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Info", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Info", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_INFO_SCENE_UPDATE_PROMOTION_DATA: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleHivePromotion_UpdatePromotionData(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleHivePromotion_UpdatePromotionData", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleHivePromotion_UpdatePromotionData", paramString);
            break;
        }
        default:
            break;
    }
}
