//
//  PromotionV2Scene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/20.
//

#include "SDKv1FuncList.hpp"
#include "PromotionV2Scene.hpp"
#include "PromotionV2ParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include "../../src/cocos2d-x/Module_Cocos2d-x.h"
#include "../../src/C2SModule.h"
#include <map>
#include <sstream>

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;
using namespace::std;


bool PromotionV2Scene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- [SDK v1 Hive프로모션(V2)] -");

    addButton("뉴스\n(C2SModuleHivePromotion_Show)", ButtonMode::NEED_PARAM, SDKv1FuncList::HIVE_PROMOTION_SCENE_NEWS);
    addButton("배너\n(C2SModuleHivePromotion_Show)", ButtonMode::NEED_PARAM, SDKv1FuncList::HIVE_PROMOTION_SCENE_BANNER);
    addButton("공지\n(C2SModuleHivePromotion_Show)", ButtonMode::NEED_PARAM, SDKv1FuncList::HIVE_PROMOTION_SCENE_NOTICE);
    addButton("커스텀뷰\n(C2SModuleHivePromotion_Show)", ButtonMode::NEED_PARAM, SDKv1FuncList::HIVE_PROMOTION_SCENE_CUSTOM_VIEW);
    addButton("커스텀보드\n(C2SModuleHivePromotion_Show)", ButtonMode::NEED_PARAM, SDKv1FuncList::HIVE_PROMOTION_SCENE_CUSTOM_BOARD);
    addButton("스팟배너\n(C2SModuleHivePromotion_Show)", ButtonMode::NEED_PARAM, SDKv1FuncList::HIVE_PROMOTION_SCENE_SPOT_BANNER);
    addButton("다이렉트배너\n(C2SModuleHivePromotion_Show)", ButtonMode::NEED_PARAM, SDKv1FuncList::HIVE_PROMOTION_SCENE_DIRECT_BANNER);
    addButton("오퍼월\n(C2SModuleHivePromotion_Show)", SDKv1FuncList::HIVE_PROMOTION_SCENE_OFFERWALL);
    addButton("리뷰팝업\n(C2SModuleHivePromotion_Show)", SDKv1FuncList::HIVE_PROMOTION_SCENE_REVIEW);
    addButton("종료팝업\n(C2SModuleHivePromotion_Show)", SDKv1FuncList::HIVE_PROMOTION_SCENE_MOREGAMES);
    addButton("네이티브 리뷰팝업\n(C2SModuleHivePromotion_Show)", SDKv1FuncList::HIVE_PROMOTION_SCENE_REVIEW_NATIVE);

    scheduleUpdate();

    return true;
}

void PromotionV2Scene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void PromotionV2Scene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        PromotionV2ParamScene* paramScene = PromotionV2ParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    SDKv1FuncList::HivePromotionSceneFunc api = static_cast<SDKv1FuncList::HivePromotionSceneFunc>(buttonTag);
    switch (api) {
        case SDKv1FuncList::HIVE_PROMOTION_SCENE_NEWS: {
            bool forced = ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->getForcedNews();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("news");
            paramsDefault["forced"] = picojson::value(forced);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleHivePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleHivePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleHivePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_SCENE_BANNER: {
            bool forced = ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->getForcedBanner();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("banner");
            paramsDefault["forced"] = picojson::value(true);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleHivePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleHivePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleHivePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_SCENE_NOTICE: {
            bool forced = ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->getForcedNotice();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("notice");
            paramsDefault["forced"] = picojson::value(forced);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleHivePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleHivePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleHivePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_SCENE_CUSTOM_VIEW: {
            string contentKey = ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->getContentKeyCustomView();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("customView");
            paramsDefault["contentKey"] = picojson::value(contentKey);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleHivePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleHivePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleHivePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_SCENE_CUSTOM_BOARD: {
            string contentKey = ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->getContentKeyCustomBoard();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("customBoard");
            paramsDefault["contentKey"] = picojson::value(contentKey);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleHivePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleHivePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleHivePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_SCENE_SPOT_BANNER: {
            string contentKey = ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->getContentKeySpot();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("spot");
            paramsDefault["contentKey"] = picojson::value(contentKey);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleHivePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleHivePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleHivePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_SCENE_DIRECT_BANNER: {
            string contentKey = ParameterSDKv1DataManager::getInstance()->promotionV2Parameters->getContentKeyDirect();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("direct");
            paramsDefault["contentKey"] = picojson::value(contentKey);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleHivePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleHivePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleHivePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_SCENE_OFFERWALL: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("offerwall");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleHivePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleHivePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleHivePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_SCENE_REVIEW: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("review");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleHivePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleHivePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleHivePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_SCENE_MOREGAMES: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("moreGames");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleHivePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleHivePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleHivePromotion_Show", paramString);
            break;
        }
        case SDKv1FuncList::HIVE_PROMOTION_SCENE_REVIEW_NATIVE: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("reviewEx");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleHivePromotion_Show(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleHivePromotion_Show", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleHivePromotion_Show", paramString);
            break;
        }
        default:
            break;
    }
}
