//
//  PushMessageScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/12.
//

#include "SDKv1FuncList.hpp"
#include "PushMessageScene.hpp"
#include "PushMessageParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include "../../src/cocos2d-x/Module_Cocos2d-x.h"
#include "../../src/C2SModule.h"
#include <map>
#include <sstream>

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;
using namespace::std;


bool PushMessageScene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- [SDK v1 Push 메시지] -");

    addButton("Push 등록\n(C2SModulePush_Register)", ButtonMode::NEED_PARAM, SDKv1FuncList::PUSH_SCENE_REGISTER);
    addButton("Push 해제\n(C2SModulePush_Unregister)", ButtonMode::NEED_PARAM, SDKv1FuncList::PUSH_SCENE_UNREGISTER);
    addButton("Push 설정\n(C2SModulePush_Description)", ButtonMode::NEED_PARAM, SDKv1FuncList::PUSH_SCENE_DESCRIPTION);
    addButton("Push 다량 등록\n(C2SModulePush_Register)", SDKv1FuncList::PUSH_SCENE_REGISTER_MULTI);
    addButton("Push 다량 해제\n(C2SModulePush_Unregister)", SDKv1FuncList::PUSH_SCENE_UNREGISTER_MULTI);
    addButton("Push 권한 요청\n(C2SModulePush_RequestPermission)", SDKv1FuncList::PUSH_SCENE_REQUEST_PUSH_PERMISSION);

    scheduleUpdate();

    return true;
}

void PushMessageScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void PushMessageScene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        PushMessageParamScene* paramScene = PushMessageParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    SDKv1FuncList::PushSceneFunc api = static_cast<SDKv1FuncList::PushSceneFunc>(buttonTag);
    switch (api) {
        case SDKv1FuncList::PUSH_SCENE_REGISTER: {
            int pushId = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getPushId();
            string title = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getTitle();
            string message = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getMessage();
            int after = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getAfter();
            string type = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getType();
            string groupId = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getGroupId();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["pushId"] = picojson::value(static_cast<int64_t>(pushId));
            paramsDefault["title"] = picojson::value(title);
            paramsDefault["message"] = picojson::value(message);
            paramsDefault["after"] = picojson::value(static_cast<int64_t>(after));
            paramsDefault["type"] = picojson::value(type);
            paramsDefault["groupId"] = picojson::value(groupId);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePush_Register(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePush_Register", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePush_Register", paramString);
            break;
        }
        case SDKv1FuncList::PUSH_SCENE_UNREGISTER: {
            int pushId = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getPushId();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["pushId"] = picojson::value(static_cast<int64_t>(pushId));

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePush_Unregister(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePush_Unregister", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePush_Unregister", paramString);
            break;
        }
        case SDKv1FuncList::PUSH_SCENE_DESCRIPTION: {
            string usePush = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getUsePush();
            string sound = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getSound();
            string vibrate = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getVibrate();
            string operationGCM = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getOperationGCM();
            string operationLocal = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getOperationLocal();
            string useNotice = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getUseNotice();
            string useNight = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getUseNight();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault = setPushDescriptionParam(paramsDefault, "usePush", usePush);
            paramsDefault = setPushDescriptionParam(paramsDefault, "sound", sound);
            paramsDefault = setPushDescriptionParam(paramsDefault, "vibrate", vibrate);
            paramsDefault = setPushDescriptionParam(paramsDefault, "operationGCM", operationGCM);
            paramsDefault = setPushDescriptionParam(paramsDefault, "operationLocal", operationLocal);
            paramsDefault = setPushDescriptionParam(paramsDefault, "useNotice", useNotice);
            paramsDefault = setPushDescriptionParam(paramsDefault, "useNight", useNight);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePush_Description(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePush_Description", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePush_Description", paramString);
            break;
        }
        case SDKv1FuncList::PUSH_SCENE_REGISTER_MULTI: {
            int after = 0;
            for(int pushCnt = 1000001; pushCnt <= 1000050; pushCnt++) {
                string title = (to_string(pushCnt) + "_title");
                string message = (to_string(pushCnt) + "_messsage");
                after += 5;
                string type = ParameterSDKv1DataManager::getInstance()->pushMessageParameters->getType();
                /**
                 * Set Argument
                 */
                picojson::object paramsDefault;
                paramsDefault["pushId"] = picojson::value(static_cast<int64_t>(pushCnt));
                paramsDefault["title"] = picojson::value(title);
                paramsDefault["message"] = picojson::value(message);
                paramsDefault["after"] = picojson::value(static_cast<int64_t>(after));
                paramsDefault["type"] = picojson::value(type);

                string paramString = picojson::value(paramsDefault).serialize();

                /**
                 * API Call
                 */
                C2SModule_Cocos2dx::C2SModuleError error;
                if (::C2SModulePush_Register(paramString.c_str(), &error) == false) {
                    // error
                    ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                            "C2SModulePush_Register", error);
                }

                ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                        "C2SModulePush_Register", paramString);
            }
            break;
        }
        case SDKv1FuncList::PUSH_SCENE_UNREGISTER_MULTI: {
            for(int pushCnt = 1000001; pushCnt <= 1000050; pushCnt++) {
                /**
                 * Set Argument
                 */
                picojson::object paramsDefault;
                paramsDefault["pushId"] = picojson::value(static_cast<int64_t>(pushCnt));

                string paramString = picojson::value(paramsDefault).serialize();

                /**
                 * API Call
                 */
                C2SModule_Cocos2dx::C2SModuleError error;
                if (::C2SModulePush_Unregister(paramString.c_str(), &error) == false) {
                    // error
                    ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                            "C2SModulePush_Unregister", error);
                }

                ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                        "C2SModulePush_Unregister", paramString);
            }
            break;
        }
        case SDKv1FuncList::PUSH_SCENE_REQUEST_PUSH_PERMISSION: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePush_RequestPermission(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePush_RequestPermission", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePush_RequestPermission", paramString);
            break;
        }
        default:
            break;
    }
}

picojson::object PushMessageScene::setPushDescriptionParam(picojson::object param, string keyName, string value)
{
    if(value == "true") {
        param[keyName] = picojson::value(true);
    }
    else if(value == "false") {
        param[keyName] = picojson::value(false);
    }
    else {
        param[keyName] = picojson::value("");
    }
    return param;
}
