//
//  SetGameInfoScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/24.
//

#include "SDKv1FuncList.hpp"
#include "SetGameInfoScene.hpp"
#include "SetGameInfoParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include "../../src/cocos2d-x/Module_Cocos2d-x.h"
#include "../../src/C2SModule.h"
#include <map>
#include <sstream>

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;
using namespace::std;


bool SetGameInfoScene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- [SDK v1 SetGameInfo] -");

    addButton("SetGameInfo\n(C2SModuleSocial_SetGameInfo)", ButtonMode::NEED_PARAM, SDKv1FuncList::SET_GAME_INFO_SCENE_SET_GAME_INFO);
    addButton("GetHiveCountry\n(C2SModuleSocial_SetGameInfo)", SDKv1FuncList::SET_GAME_INFO_SCENE_GET_HIVE_COUNTRY);
    addButton("GetChannel\n(C2SModuleSocial_GetChannel)", SDKv1FuncList::SET_GAME_INFO_SCENE_GET_CHANNEL);
    addButton("SetChanel\n(C2SModuleSocial_SetChannel)", ButtonMode::NEED_PARAM, SDKv1FuncList::SET_GAME_INFO_SCENE_SET_CHANNEL);
    addButton("UpdateServerId\n(C2SModuleSocial_UpdateServerId)", ButtonMode::NEED_PARAM, SDKv1FuncList::SET_GAME_INFO_SCENE_UPDATE_SERVER_ID);
    addButton("UpdateGameLanguage\n(C2SModuleSocial_UpdateGameLanguage)", ButtonMode::NEED_PARAM, SDKv1FuncList::SET_GAME_INFO_SCENE_UPDATE_GAMELANGUAGE);
    addButton("SendData(퍼널로그전송)\n(C2SModuleSocial_SendData)", ButtonMode::NEED_PARAM, SDKv1FuncList::SET_GAME_INFO_SCENE_SEND_DATA_FUNNELS);

    scheduleUpdate();

    return true;
}

void SetGameInfoScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void SetGameInfoScene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        SetGameInfoParamScene* paramScene = SetGameInfoParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    SDKv1FuncList::SetGameInfoSceneFunc api = static_cast<SDKv1FuncList::SetGameInfoSceneFunc>(buttonTag);
    switch (api) {
        case SDKv1FuncList::SET_GAME_INFO_SCENE_SET_GAME_INFO: {
            string serverId = ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->getServerId();
            string gameLanguage = ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->getGameLanguage();
            bool emergencyMode = ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->getEmergencyMode();
            string channel = ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->getChannel();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["serverId"] = picojson::value(serverId);
            paramsDefault["gameLanguage"] = picojson::value(gameLanguage);
            paramsDefault["emergencyMode"] = picojson::value(emergencyMode);
            paramsDefault["channel"] = picojson::value(channel);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSocial_SetGameInfo(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSocial_SetGameInfo", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSocial_SetGameInfo", paramString);
            break;
        }
        case SDKv1FuncList::SET_GAME_INFO_SCENE_GET_HIVE_COUNTRY: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSocial_GetHiveCountry", paramString);

            string hiveCountry = ::GetHiveCountry();

            addLogMessage("GetHiveCountry() : " + hiveCountry);
            break;
        }
        case SDKv1FuncList::SET_GAME_INFO_SCENE_GET_CHANNEL: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSocial_GetChannel", paramString);

            string channel = ::GetChannel();

            addLogMessage("GetChannel() : " + channel);
            break;
        }
        case SDKv1FuncList::SET_GAME_INFO_SCENE_SET_CHANNEL: {
            string channel = ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->getChannel();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSocial_SetChannel", paramString);

            ::SetChannel(channel.c_str());

            addLogMessage("SetChannel() : " + channel);
            break;
        }
        case SDKv1FuncList::SET_GAME_INFO_SCENE_UPDATE_SERVER_ID: {
            string serverId = ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->getServerId();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["serverId"] = picojson::value(serverId);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSocial_UpdateServerId(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSocial_UpdateServerId", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSocial_UpdateServerId", paramString);
            break;
        }
        case SDKv1FuncList::SET_GAME_INFO_SCENE_UPDATE_GAMELANGUAGE: {
            string gameLanguage = ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->getGameLanguage();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["gameLanguage"] = picojson::value(gameLanguage);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSocial_UpdateGameLanguage(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSocial_UpdateGameLanguage", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSocial_UpdateGameLanguage", paramString);
            break;
        }
        case SDKv1FuncList::SET_GAME_INFO_SCENE_SEND_DATA_FUNNELS: {
            string funnelsId = ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->getFunnelsId();
            string optionTag = ParameterSDKv1DataManager::getInstance()->setGameInfoParameters->getOptionTag();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["funnelsId"] = picojson::value(funnelsId);
            paramsDefault["optionTag"] = picojson::value(optionTag);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSocial_SendData(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSocial_SendData", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSocial_SendData", paramString);
            break;
        }
        default:
            break;
    }
}
