//
//  SnsPostScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/17.
//

#include "SDKv1FuncList.hpp"
#include "SnsPostScene.hpp"
#include "SnsPostParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include "../../src/cocos2d-x/Module_Cocos2d-x.h"
#include "../../src/C2SModule.h"
#include <map>
#include <sstream>

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;
using namespace::std;


bool SnsPostScene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- [SDK v1 SnsPost] -");

    addButton("Facebook 포스팅\n(C2SModuleSns_Post)", ButtonMode::NEED_PARAM, SDKv1FuncList::SNSPOST_SCENE_FACEBOOK);
    addButton("GooglePlus 포스팅\n(C2SModuleSns_Post)", ButtonMode::NEED_PARAM, SDKv1FuncList::SNSPOST_SCENE_GOOGLEPLUS);

    scheduleUpdate();

    return true;
}

void SnsPostScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void SnsPostScene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        SnsPostParamScene* paramScene = SnsPostParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    SDKv1FuncList::SnsPostSceneFunc api = static_cast<SDKv1FuncList::SnsPostSceneFunc>(buttonTag);
    switch (api) {
        case SDKv1FuncList::SNSPOST_SCENE_FACEBOOK: {
            string appLinkURL = ParameterSDKv1DataManager::getInstance()->snsPostParameters->getFacebookAppLinkUrl();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["service"] = picojson::value("facebook");
            paramsDefault["appLinkURL"] = picojson::value(appLinkURL);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Post(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Post", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Post", paramString);
            break;
        }
        case SDKv1FuncList::SNSPOST_SCENE_GOOGLEPLUS: {
            string message = ParameterSDKv1DataManager::getInstance()->snsPostParameters->getGooglePlusMessage();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["service"] = picojson::value("goolgeplus");
            paramsDefault["message"] = picojson::value(message);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Post(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Post", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Post", paramString);
            break;
        }
        default:
            break;
    }
}
