//
//  TrackingScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/19.
//

#include "SDKv1FuncList.hpp"
#include "TrackingScene.hpp"
#include "TrackingParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include "../../src/cocos2d-x/Module_Cocos2d-x.h"
#include "../../src/C2SModule.h"
#include <map>
#include <sstream>

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;
using namespace::std;


bool TrackingScene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- [SDK v1 마케팅 트래킹툴] -");

    addButton("이벤트 전송\n(C2SModuleTracker_SendEvent)", ButtonMode::NEED_PARAM, SDKv1FuncList::TRACKER_SCENE_SEND_EVENT);
    addButton("활성화 설정\n(C2SModuleTracker_SetEnable)", ButtonMode::NEED_PARAM, SDKv1FuncList::TRACKER_SCENE_ENABLE);

    scheduleUpdate();

    return true;
}

void TrackingScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void TrackingScene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        TrackingParamScene* paramScene = TrackingParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    SDKv1FuncList::TrackingSceneFunc api = static_cast<SDKv1FuncList::TrackingSceneFunc>(buttonTag);
    switch (api) {
        case SDKv1FuncList::TRACKER_SCENE_SEND_EVENT: {
            string event = ParameterSDKv1DataManager::getInstance()->trackingParameters->getEvent();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["event"] = picojson::value(event);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleTracker_SendEvent(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleTracker_SendEvent", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleTracker_SendEvent", paramString);
            break;
        }
        case SDKv1FuncList::TRACKER_SCENE_ENABLE: {
            bool adjust = ParameterSDKv1DataManager::getInstance()->trackingParameters->getEnableAdjust();
            bool singular = ParameterSDKv1DataManager::getInstance()->trackingParameters->getEnableSingular();
            bool appsflyer = ParameterSDKv1DataManager::getInstance()->trackingParameters->getEnableAppsFlyer();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["Adjust"] = picojson::value(adjust);
            paramsDefault["Singular"] = picojson::value(singular);
            paramsDefault["AppsFlyer"] = picojson::value(appsflyer);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleTracker_SetEnable(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleTracker_SetEnable", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleTracker_SetEnable", paramString);
            break;
        }
        default:
            break;
    }
}
