//
//  UserAcquisitionScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/18.
//

#include "SDKv1FuncList.hpp"
#include "UserAcquisitionScene.hpp"
#include "UserAcquisitionParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include "../../src/cocos2d-x/Module_Cocos2d-x.h"
#include "../../src/C2SModule.h"
#include <map>
#include <sstream>

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;
using namespace::std;


bool UserAcquisitionScene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- [SDK v1 프로모션 집객] -");

    addButton("집객 정보 보기\n(C2SModulePromotion_Acquisition)", ButtonMode::NEED_PARAM, SDKv1FuncList::UA_SCENE_INFO);
    addButton("QR Code 공유하기\n(C2SModulePromotion_Acquisition)", SDKv1FuncList::UA_SCENE_QRCODE_SHARE);
    addButton("집객 링크 공유\n(C2SModulePromotion_Acquisition)", SDKv1FuncList::UA_SCENE_UA_LINK_SHARE);
    addButton("링크 및 메시지 공유\n(C2SModulePromotion_Acquisition)", ButtonMode::NEED_PARAM, SDKv1FuncList::UA_SCENE_LINK_AND_MESSAGE_SHARE);
    addButton("Hive 메시지 전달(집객 정보 활용)\n(C2SModuleSns_Message)", ButtonMode::NEED_PARAM, SDKv1FuncList::UA_SCENE_HIVE_MESSAGE_WITH_UA_INFO);
    addButton("UA 초대자 정보 보기\n(C2SModulePromotion_Acquisition)", SDKv1FuncList::UA_SCENE_UA_SENDER_INFO);

    scheduleUpdate();

    // ResultDelegate.cpp 에서 Acquisition의 QRCode 정보를 받았을 경우
    // QRcode 팝업을 띄우기 위한 스캐쥴 등록
    schedule(schedule_selector(UserAcquisitionScene::showQRCodePopup));

    return true;
}

void UserAcquisitionScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void UserAcquisitionScene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        UserAcquisitionParamScene* paramScene = UserAcquisitionParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    SDKv1FuncList::UserAcquisitionSceneFunc api = static_cast<SDKv1FuncList::UserAcquisitionSceneFunc>(buttonTag);
    switch (api) {
        case SDKv1FuncList::UA_SCENE_INFO: {
            string additionalInfo = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getAdditionalInfo();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["additionalInfo"] = picojson::value(additionalInfo);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Acquisition(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Acquisition", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Acquisition", paramString);
            break;
        }
        case SDKv1FuncList::UA_SCENE_QRCODE_SHARE: {
            string qrCodeData = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getQrCodeData();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("qrcodeShare");
            paramsDefault["data"] = picojson::value(qrCodeData);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Acquisition(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Acquisition", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Acquisition", paramString);
            break;
        }
        case SDKv1FuncList::UA_SCENE_UA_LINK_SHARE: {
            string url = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getLinkShareUrl();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("linkShare");
            paramsDefault["url"] = picojson::value(url);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Acquisition(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Acquisition", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Acquisition", paramString);
            break;
        }
        case SDKv1FuncList::UA_SCENE_LINK_AND_MESSAGE_SHARE: {
            string url = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getLinkShareUrl();
            string message = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getMessage();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("linkShare");
            paramsDefault["url"] = picojson::value(url);
            paramsDefault["message"] = picojson::value(message);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Acquisition(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Acquisition", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Acquisition", paramString);
            break;
        }
        case SDKv1FuncList::UA_SCENE_HIVE_MESSAGE_WITH_UA_INFO: {
            bool useVid = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getUseVid();
            string vidOrUid = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getVidOrUid();
            string message = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getMessage();
            string imageURL = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getImageURL();
            string thumbnailURL = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getThumbnailURL();
            bool usePush = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getUsePush();
            bool invite = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getInvite();
            bool linkShare = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getLinkShare();
            string url = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getLinkShareUrl();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            if(useVid == true) {
                paramsDefault["vid"] = picojson::value(vidOrUid);
            } else {
                paramsDefault["uid"] = picojson::value(vidOrUid);
            }
            paramsDefault["message"] = picojson::value(message);
            paramsDefault["imageURL"] = picojson::value(imageURL);
            paramsDefault["thumbnailURL"] = picojson::value(thumbnailURL);
            paramsDefault["usePush"] = picojson::value(usePush);
            paramsDefault["invite"] = picojson::value(invite);
            paramsDefault["linkShare"] = picojson::value(linkShare);
            paramsDefault["linkShareURL"] = picojson::value(url);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_Message(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_Message", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_Message", paramString);
            break;
        }
        case SDKv1FuncList::UA_SCENE_UA_SENDER_INFO: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("senderInfo");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModulePromotion_Acquisition(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModulePromotion_Acquisition", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModulePromotion_Acquisition", paramString);
            break;
        }
        default:
            break;
    }
}

void UserAcquisitionScene::showQRCodePopup(float dt)
{
    if(ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getNeedShowQRCodePopup()) {

        // 팝업 배경 레이아웃 설정
        Size visibleSize = Director::getInstance()->getVisibleSize();		// 사용 x, 320 x 480 고정 중.
        auto popup_layer = LayerColor::create(Color4B(0, 125, 125, 255));
        popup_layer->setContentSize(Size(visibleSize.width * 0.90, visibleSize.height * 0.5));
        popup_layer->setPosition(Point(visibleSize.width * 0.05, visibleSize.height * 0.05));
        popup_layer->setTag(TAG_LAYER_QR_POPUP);
        this->addChild(popup_layer);

        // 팝업 내 버튼 추가
        MenuItemFont::setFontSize(15);
        auto button_1 = MenuItemFont::create("[ close ]",
                                             CC_CALLBACK_1(UserAcquisitionScene::doPopupAction, this));
        button_1->setColor(ccc3(255, 255, 255));
        button_1->setTag(TAG_BTN_CLOSE_QR_POPUP);
        auto Popup_1 = Menu::create(button_1, NULL);
        Popup_1->setPosition(Point(visibleSize.width * 0.45, visibleSize.height * 0.1));
        popup_layer->addChild(Popup_1);

        // QRCode string 데이터(HexString)를 Bytes 로 변환
        string strQRData = ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->getQrCodeData();

        std::stringstream ss;
        std::vector<unsigned char> byteQRData;
        unsigned int buffer;
        int offset = 0;
        while (offset < strQRData.length()) {
            ss.clear();
            ss << std::hex << strQRData.substr(offset, 2);
            ss >> buffer;
            byteQRData.push_back(static_cast<unsigned char>(buffer));
            offset += 2;
        }

        // QRCode 이미지 layer 에 추가
        typedef unsigned char byte;
        Image *image = new Image();
        image->initWithImageData(byteQRData.data(), byteQRData.size());

        Texture2D* texture = new Texture2D();
        texture->initWithImage(image);
        Sprite* sp = Sprite::createWithTexture(texture);
        sp->setPosition(visibleSize.width * 0.45, visibleSize.height * 0.25);
        sp->setScale(0.8f);
        popup_layer->addChild(sp);

        // 팝업 노출 제어값 비활성
        ParameterSDKv1DataManager::getInstance()->userAcquisitionParameters->setNeedShowQRCodePopup(false);
    }
}

void UserAcquisitionScene::doPopupAction(Ref *sender)
{
    auto mItem = (MenuItem*)sender;

    switch(mItem->getTag()) {
        case TAG_BTN_CLOSE_QR_POPUP : {
            this->removeChildByTag(TAG_LAYER_QR_POPUP, true);
            break;
        }
        default :
            break;
    }
}