//
//  UserEngagementScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/20.
//

#include "SDKv1FuncList.hpp"
#include "UserEngagementScene.hpp"
#include "UserEngagementParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include "../../src/cocos2d-x/Module_Cocos2d-x.h"
#include "../../src/C2SModule.h"
#include <map>
#include <sstream>

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;
using namespace::std;


bool UserEngagementScene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- [SDK v1 유저 인게이지먼트] -");

    addButton("Get Events(C2SModuleEngagement_GetEvents)", SDKv1FuncList::UE_SCENE_GET_EVENT);
    addButton("Consume Coupon\n(C2SModuleEngagement_ConsumeCoupon)" , ButtonMode::NEED_PARAM, SDKv1FuncList::UE_SCENE_CONSUME_COUPON);
    addButton("SetReady\n(C2SModuleEngagement_SetReady)", SDKv1FuncList::UE_SCENE_SET_READY);

    scheduleUpdate();

    return true;
}

void UserEngagementScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void UserEngagementScene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        UserEngagementParamScene* paramScene = UserEngagementParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    SDKv1FuncList::EngagementSceneFunc api = static_cast<SDKv1FuncList::EngagementSceneFunc>(buttonTag);
    switch (api) {
        case SDKv1FuncList::UE_SCENE_GET_EVENT: {
            MessageBox("", "Hive 1.11.0 에서 GetEvents API는 삭제되었습니다.");
            break;
        }
        case SDKv1FuncList::UE_SCENE_CONSUME_COUPON: {
            string couponId = ParameterSDKv1DataManager::getInstance()->userEngagementParameters->getCouponId();
            string additionalInfo = ParameterSDKv1DataManager::getInstance()->userEngagementParameters->getAdditionalInfo();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["couponId"] = picojson::value(couponId);
            paramsDefault["additionalInfo"] = picojson::value(additionalInfo);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleEngagement_ConsumeCoupon(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleEngagement_ConsumeCoupon", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleEngagement_ConsumeCoupon", paramString);
            break;
        }
        case SDKv1FuncList::UE_SCENE_SET_READY: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleEngagement_SetReady(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleEngagement_SetReady", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleEngagement_SetReady", paramString);
            break;
        }
        default:
            break;
    }
}
