//
//  UserInfoScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by jschoi on 2023/04/14.
//

#include "SDKv1FuncList.hpp"
#include "UserInfoScene.hpp"
#include "UserInfoParamScene.hpp"
#include "ParameterSDKv1DataManager.hpp"
#include "ui/CocosGUI.h"
#include "../../src/cocos2d-x/Module_Cocos2d-x.h"
#include "../../src/C2SModule.h"
#include <map>
#include <sstream>

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;
using namespace::std;


bool UserInfoScene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- [SDK v1 유저 정보] -");

    addButton("Hive 유저 정보\n(C2SModuleSns_User)", ButtonMode::NEED_PARAM, SDKv1FuncList::USERINFO_SCENE_HIVE_USER);
    addButton("Facebook 유저 정보\n(C2SModuleSns_User)", ButtonMode::NEED_PARAM, SDKv1FuncList::USERINFO_SCENE_FACEBOOK_USER);
    addButton("Hive 신규 메시지 확인\n(C2SModuleSns_User)", SDKv1FuncList::USERINFO_SCENE_CHECK_NEW_MESSAGE);
    addButton("계정 삭제\n(C2SModuleSns_UserDelete)", SDKv1FuncList::USERINFO_SCENE_DELETE_ID);

    scheduleUpdate();

    return true;
}

void UserInfoScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void UserInfoScene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        UserInfoParamScene* paramScene = UserInfoParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    SDKv1FuncList::UserInfoSceneFunc api = static_cast<SDKv1FuncList::UserInfoSceneFunc>(buttonTag);
    switch (api) {
        case SDKv1FuncList::USERINFO_SCENE_HIVE_USER: {
            string useMyVid = ParameterSDKv1DataManager::getInstance()->userInfoParameters->getUseMyVid();
            string userVid = ParameterSDKv1DataManager::getInstance()->userInfoParameters->getUserVid();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            if(useMyVid == "myVid") {
                //
            } else {
                // "otherVid"
                // vid 단일 혹은 vid list 인지 확인 (ex : "12345" / "12345,6789,2345")
                picojson::value::array vectorVidList;

                if(userVid != "") {
                    string delimiter = ",";

                    size_t pos = 0;
                    picojson::value token;
                    bool existDelimiter = false;
                    while ((pos = userVid.find(delimiter)) != string::npos) {
                        token = picojson::value(userVid.substr(0, pos));
                        vectorVidList.push_back(token);

                        userVid.erase(0, pos + delimiter.length());
                        existDelimiter = true;
                    }

                    if(existDelimiter) {
                        // delimeter 구분된 데이터가 2개 이상인 경우
                        paramsDefault["vid"] = picojson::value(vectorVidList);
                    }
                    else {
                        // delimeter 구분이 없는 경우
                        paramsDefault["vid"] = picojson::value(userVid);
                    }
                }
            }

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_User(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_User", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_User", paramString);
            break;
        }
        case SDKv1FuncList::USERINFO_SCENE_FACEBOOK_USER: {
            string facebookId = ParameterSDKv1DataManager::getInstance()->userInfoParameters->getFacebookId();

            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("facebook");
            paramsDefault["facebookId"] = picojson::value(facebookId);

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_User(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_User", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_User", paramString);
            break;
        }
        case SDKv1FuncList::USERINFO_SCENE_CHECK_NEW_MESSAGE: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;
            paramsDefault["type"] = picojson::value("message");

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_User(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_User", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_User", paramString);
            break;
        }
        case SDKv1FuncList::USERINFO_SCENE_DELETE_ID: {
            /**
             * Set Argument
             */
            picojson::object paramsDefault;

            string paramString = picojson::value(paramsDefault).serialize();

            /**
             * API Call
             */
            C2SModule_Cocos2dx::C2SModuleError error;
            if (::C2SModuleSns_UserDelete(paramString.c_str(), &error) == false) {
                // error
                ParameterSDKv1DataManager::getInstance()->logManager->addErrorLog(
                        "C2SModuleSns_UserDelete", error);
            }

            ParameterSDKv1DataManager::getInstance()->logManager->addAPICallLog(
                    "C2SModuleSns_UserDelete", paramString);
            break;
        }
        default:
            break;
    }
}
